unit BitmapLib;

interface

uses Forms,Windows,Types,ExtCtrls,Dialogs,Classes,SysUtils,Graphics,Math,ClipBrd,FilmStp,
     Vector,IntVector,DSupport,GraphicsLib,HugeArry,ImageEx, StreamLib,StatsLib,
     GDIPAPI,GDIPObj;

type TBitmapProgress =procedure(Percent: integer);


type TJPEGResUnits = (juNoUnits=0, juDotsPerInch=1, juDotsPerCentimeter);

const MaxSize = 1024 * 1024 * 128;	{Arrays cannot be more than 2gig, so use this for max}

{24-bit color arrays}

type TRGBTripleArray = array [0..MaxSize] of TRGBTriple;
type PRGBTripleArray = ^TRGBTripleArray;
type TRGBTripleRowArray = array of pRGBTripleArray;

{32-bit color arrays}

type TRGBQuadArray = array [0..MaxSize] of TRGBQuad;
type PRGBQuadArray = ^TRGBQuadArray;


{Hue, Staturation and Brightness structure}

type THSBRecord=record
 Hue,Sat,Bright: extended;
 end;


{Hue, Staturation and Luminesance structure}

type THSLRecord=record
 Hue,Sat,Lume: extended;
 end;


type THistoArray = TDoubleDynArray;


{----------- Gaussian Blur variables ------------------------------------------}


type TRow = array[0..MaxSize] of TRGBTriple;
type PRow = ^TRow;

type TPRows = array[0..MaxSize] of PRow;
type PPRows = ^TPRows;

{the idea is that when using a TKernel you ignore the Weights except for Weights in the range -Size..Size.}

const MaxKernelSize = 100;

type TKernelSize = 1..MaxKernelSize;
type TKernel = record
 Size: TKernelSize;
 Weights: array[-MaxKernelSize..MaxKernelSize] of single;
 end;

procedure MakeGaussianKernel(var K: TKernel; radius: double; MaxData, DataGranularity: double);
procedure GaussianBlur(theBitmap: TBitmap; radius: double); overload;
procedure GaussianBlur(theBitmap: TBitmap; radius: double; Rect: TRect); overload;
procedure GaussBlurCircle(Canvas: TCanvas; P: TPoint; Size: integer; Radius: double);


{Framing options for rotation: }
{foSource = clipped to Source retangle size}
{foDestination = clipped to Destination retangle size}
{foDestCenter - Puts rotational center in center of destination}
{foFit = Rectangle expanded to fit rotated image}
{foWindow = Rectangle always expanded to max rotation size}

type TFrameOptions = (foSource,foDestination,foDestCenter,foFit,foWindow);

type PBitmap = ^TBitmap;

{============= Bitmap list object =============================================}


type TBitmapList = class(TList)
 private
  function GetBitmap(Inx: integer): TBitmap;
  procedure PutBitmap(Inx: integer; Item: TBitmap);
  function GetAuxInfo(Index: integer): integer;
  procedure PutAuxInfo(Index: integer; const Value: integer);
  function GetCount: integer;
  procedure SetCount(V: integer);
 protected
  FAuxInfo: THugeInteger;
 public
  constructor Create;
  destructor Destroy; override;
  function AddBitmap(BM: TBitmap): integer; virtual;
  procedure Clear; override;
  procedure Delete(Index: Integer); virtual;
  procedure Load(Filename: string; Progress: TBitmapProgress);
  procedure Save(Filename: string; Progress: TBitmapProgress);
  procedure PasteFromClipboard;
  procedure CopyToClipboard(Inx: integer);
  property Bitmaps[Inx: integer]: TBitmap read GetBitmap write PutBitmap; default;
  property AuxInfo[Inx: integer]: integer read GetAuxInfo write PutAuxInfo;
  property Count: integer read GetCount write SetCount;
 end;


function MakeRBGTriple(R,G,B: byte): TRGBTriple;
function MakeRGBAQuad(R,G,B,A: byte): TRGBQuad;
procedure FillRGBRowArray(Bmp: TBitmap; var RowArray: TRGBTripleRowArray);

function ColorToQuad(C: TColor): TRGBQuad;
function ColorToTriple(C: TColor): TRGBTriple;
function EqualRGBTriple(C1,C2: TRGBTriple): boolean;
function TripleDistance(C1,C2: TRGBTriple): double;
function ColorDistance(C1,C2: TColor): double;
function XORColors(C1,C2: TColor): TColor;
function InvertColor(C: TColor): TColor;

procedure ColorToRGBBytes(Color: TColor; var R,G,B: byte);
function QuadToColor(T: TRGBQuad): TColor;
function TripleToColor(T: TRGBTriple): TColor;
function ColorToVector(Color: TColor): T3DVector;
function VectorToColor(V: T3DVector): TColor;
function RGBTripleToVector(Color: TRGBTriple): T3DVector;
function Color15ToColor(C: integer): TColor;
function Color16ToColor(C: integer): TColor;
function Color24ToColor(C: integer): TColor;
function ContrastColor(C: TColor): TColor;


procedure CopyBitmap(Dest,Source: TBitmap);


procedure FlipBitmapHoriz(Source,Destination: TBitmap); overload;
procedure FlipBitmapHoriz(BM: TBitmap); overload;
procedure FlipBitmapVertical(Source,Destination: TBitmap); overload;
procedure FlipBitmapVertical(BM: TBitmap); overload;

procedure FlipBitmap90(Source,Destination: TBitmap); overload;
procedure FlipBitmap90(BM: TBitmap); overload;

procedure FlipBitmap180(Source,Destination: TBitmap); overload;
procedure FlipBitmap180(BM: TBitmap); overload;

procedure FlipBitmap270(Source,Destination: TBitmap); overload;
procedure FlipBitmap270(BM: TBitmap); overload;


procedure RotateBitmap(Source,Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
procedure RotateBitmap32(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor); overload;
procedure RotateBitmap32(BM: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor); overload;
procedure RotateBitmapGDIP(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);

procedure BorderBitmap(BM: TBitmap; Color: TColor; Width: integer);

function RGBtoHSB(RGB: TRGBTriple): THSBRecord;
function ColorToHSB(Color: TColor): THSBRecord;
function HSBtoRGB(HSB: THSBRecord): TRGBTriple;
function MakeHSB(Hue,Sat,Bright: extended): THSBRecord;

function HSLToRGB(HSL: THSLRecord): TRGBTriple;
function RGBToHSL(RGB: TRGBTriple): THSLRecord;
function MakeHSL(Hue,Sat,Lume: single): THSLRecord;

function GetRGBIntensity(C: TColor): double;

function RGBToGray(V: TRGBTriple): byte;
procedure MakeBlackAndWhite(Source,Destination: TBitmap; Invert: boolean; Threshold: double);
procedure AdjustHue(Source,Destination: TBitmap; Offset: double); overload;
procedure AdjustHue(BM: TBitmap; Hue: double); overload;
procedure InvertImage(BM: TBitmap);
procedure UnsharpMask(BM: TBitmap; Radius, Threshold, Amount: double);

procedure AdjustSaturation(Source,Destination: TBitmap; Offset: double); overload;
procedure AdjustSaturation(BM: TBitmap; Sat: double); overload;

procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double); overload;
procedure AdjustBrightnessContrastGamma(BM: TBitmap; Brightness,Contrast,Gamma: double); overload;
procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double; Rect: TRect); overload;
procedure AdjustBrightnessContrastGamma(BM: TBitmap; Brightness,Contrast,Gamma: double; Rect: TRect); overload;

procedure DoWinResize(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Mode: integer);
procedure ResizeBitmap(Source,Destination: TBitmap; Percent: double; OverSample: boolean); overload;
procedure ResizeBitmap(Source,Destination: TBitmap; Size: TPoint; OverSample: boolean); overload;
procedure ResizeBitmap(BM: TBitmap; Percent: double; OverSample: boolean); overload;
procedure OversampleBitmap(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap);

procedure AlphaBlendBitmaps(SourceCanvas,DestCanvas: TCanvas; SourceRect,DestRect: TRect; Alpha: byte; PerPixel: boolean); overload;
procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte); overload;
procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte; PerPixel: boolean); overload;
procedure AlphaBlendCanvas(DestCanvas: TCanvas; DestRect: TRect; SourceCanvas: TCanvas; SourceRect: TRect; Alpha: byte; PerPixel: boolean);
procedure SetAlphaByLightness(BM: TBitmap; DoLightness: boolean; Lightness: byte);
procedure SetAlphaByColor(BM: TBitmap; C: TColor; Alpha: byte; Reset: boolean); overload;
procedure SetAlphaByColor(BM: TBitmap; C: TColor); overload;
procedure MakeMask(var Mask: TBitmap; Color: TColor);

procedure WrapImageAngle(BM: TBitmap; Angle,Len: double);
procedure WrapImage(BM: TBitmap; X,Y: integer);
procedure WrapImageVertical(BM: TBitmap; Delta: integer);
procedure WrapImageHorizontal(BM: TBitmap; Delta: integer);
function ClipBitmap(BM: TBitmap; R: TRect): boolean;

function CreateDisabledBitmap(Glyph: TBitmap): TBitmap;


procedure SaveRectToClipboard(Bitmap: TBitmap; R: TRect);

procedure ForceBitmapHandle(BM: TBitmap);
function InterpolateColors(C1,C2: TColor; Width, Pos: double): TColor;
procedure DrawGradientLine(Canvas: TCanvas; P1,P2: TPoint; C1,C2: TColor);
procedure VertGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor);
procedure HorizGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor) ;

function IconToBitmap(Icon: TIcon): TBitmap;
function LoadCursorToBitmap(Filename: string): TBitmap;
function LoadCursorToIcon(Filename: string): TIcon;

function ColorToRGBString(C: TColor): string; overload;
function ColorToRGBString(C: TColor; Hex: boolean): string; overload;


function RGBTripleToString(C: TRGBTriple; Hex: boolean): string; overload;
function RGBTripleToString(C: TRGBTriple): string; overload;
function RGBQuadToString(C: TRGBQuad; Hex: boolean): string; overload;
function RGBQuadToString(C: TRGBQuad): string; overload;

procedure NormalizeImageSize(Image: TImage);
procedure ClearImage(Image: TImage; Color: TColor);
procedure ClearCanvas(Canvas: TCanvas; Rect: TRect; Color: TColor);

procedure DrawPixels(Image: TImage; X,Y, Size: integer; C: TColor);
procedure ContinuousDraw(Image: TImage; Start: boolean; X,Y, Size: integer; C: TColor);
procedure ColorFill(BM: TBitmap; X,Y: integer; Threshold: double; NewColor: TColor);


type TFloodProgress = procedure(Percent: integer);

procedure ThresholdFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
procedure SeedFloodFill(BM: TBitmap; X, Y: integer; Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
procedure AntiAliasFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);

procedure ClearBitmap(BM: TBitmap; Block: TRect; Color: TColor);  overload;
procedure ClearBitmap(BM: TBitmap; Color: TColor);  overload;
function IsBitmapBlank(BM: TBitmap): boolean;

procedure SetBMPFilePPM(FileName: String; PPMX,PPMY: integer);
procedure SetBMPPPM(BM: TBitmap; PPMX,PPMY: integer);
procedure GetBmpPPM(Bitmap: TBitmap; var PPMX,PPMY: integer);
procedure GetBMPFilePPM(FileName: string; var PPMX,PPMY: integer);
procedure CopyBMPPPM(Source,Destination: TBitmap);

procedure GetJPEGResolution(Filename: string; var Units: TJPEGResUnits; var XRes,YRes: integer);
procedure SetJPEGResolution(Filename: string; DPIX, DPIY: Integer; Units: TJPEGResUnits=juDotsPerInch);

procedure SetBMPPixelFormat(BM: TBitmap; PF: Graphics.TPixelFormat);
function GetPixelFormatStr(BM: TBitmap): string;

procedure FadeInBitmap(Image: TImage; BM: TBitmap; Millisec: integer);
procedure FadeToColor(Image: TImage; C: TColor; Millisec: integer);

procedure BitmapToGrayscaleBuffer(BM: TBitmap; AllocMem: boolean; var Buffer: PByteArray);
procedure RGBBitmapToGray(const Bmp: TBitmap);
procedure RGBToBlackAndWhite(Source,Destination: TBitmap; Threshold: double);
procedure WhiteThreshold(Source,Destination: TBitmap; Threshold: double);
procedure BinarizeImage(Src,Dest: TBitmap; Radius: integer; KFactor: double);

procedure RGBBitmapToBytes(const Bmp: TBitmap; var BA: TDynByteArray);
procedure GetMBMPHistogram(const Bmp: TBitmap; var Hist: THistoArray; var HighValue: double);
function GetOtsuLevel(Histo: THistoArray): integer;


function EqualBitmaps(BM1,BM2: TBitmap): boolean;
function BitmapDifference(BM1,BM2: TBitmap): double;
function GetBitsPerPixel(BM: TBitmap): integer;
function GetBitmapSize(BM: TBitmap): int64;
function GetBmpDPI(Bitmap: TBitmap): LongInt;
procedure SetBmpDPI(Bitmap: TBitmap; DPI: Integer);

function GetJPGLength(Stream: TStream; Limit: int64): int64;

type TBitmapType = (btType2,btType3,btType4,btUnknown);

function BitmapTypeToStr(BT: TBitmapType): string;
function GetBMPLength(Stream: TStream; var BMPType: TBitmapType): int64;

function FindImageBorder(StartX,StartY: integer; BM: TBitmap; Thresh: double): TRect;

function GetAImageHash(SrcBmp: TBitmap): Int64;
function GetHamDistance(Hash1, Hash2: Int64): Int64;

function BmpHandleTypeToStr(HT: TBitmapHandleType): string;
function BmpPixelFormatToStr(PF: Graphics.TPixelFormat): string;
function BmpTransModeToStr(TM: TTransparentMode): string;

{Routines to get bitmap rotation }

{Retation analysis statistics}

type TImageAngleStats = record
 PixelCount: Integer;
 TestedPixels: Integer;
 AccumulatorSize: Integer;
 AccumulatedCounts: Integer;
 BestCount: Integer;
 end;

var ImageAngleStats: TImageAngleStats;

function GetImageAngle(const MaxAngle: Double; Treshold, Width, Height: Integer; Pixels: PByteArray; DetectionArea: PRect = nil): double; overload;
function GetImageAngle(BM: TBitmap; const MaxAngle: Double; Treshold: Integer; DetectionArea: PRect = nil): Double; overload;

procedure MedianBlur(Src,Dest: TBitmap; WindowX, WindowY: integer); overload;
procedure MedianBlur(Image: TBitmap; Radius: integer); overload;

type TBilateralMode = (bmBrightness,bmHue,bmSaturation);

procedure BilateralFilter(Src,Dest: TBitmap; Radius: integer; IntSigma, DistSIgma: double; Mode: TBilateralMode = bmBrightness; Progress: TBitmapProgress = nil);

implementation



procedure FillRGBRowArray(Bmp: TBitmap; var RowArray: TRGBTripleRowArray);
{Capture all bitmap scan lines and put them in an array}
var Y: integer;
begin
SetLength(RowArray,Bmp.Height);
for Y:=0 to Bmp.Height-1 do RowArray[Y]:=Bmp.ScanLine[Y];
end;


function MakeRBGTriple(R,G,B: byte): TRGBTriple;
begin
Result.rgbtRed:=R;
Result.rgbtGreen:=G;
Result.rgbtBlue:=B;
end;



function MakeRGBAQuad(R,G,B,A: byte): TRGBQuad;
begin
Result.rgbRed:=R;
Result.rgbGreen:=G;
Result.rgbBlue:=B;
Result.rgbReserved:=A;
end;




procedure ForceBitmapHandle(BM: TBitmap);
{Necessary when assigning bitmaps - Forces bitmaps to be independent}
var H: HBITMAP;
{$HINTS OFF}
begin
{Even though the handle is never used, it must be assigned or
else the Handle will not be created making the BM independent}
H:=BM.Handle;
end;
{$HINTS ON}




function ColorToTriple(C: TColor): TRGBTriple;
{Convert Delphi TColor to RGBTriple}
begin
Result.rgbtRed:=C and $FF;
Result.rgbtGreen:=(C shr 8) and $FF;
Result.rgbtBlue:=(C shr 16) and $FF;
end;



function ColorToQuad(C: TColor): TRGBQuad;
{Convert Delphi TColor to RGBQuad}
begin
Result.rgbRed:=C and $FF;
Result.rgbGreen:=(C shr 8) and $FF;
Result.rgbBlue:=(C shr 16) and $FF;
Result.rgbReserved:=$FF;
end;



function EqualRGBTriple(C1,C2: TRGBTriple): boolean;
begin
Result:=(C1.rgbtBlue = C2.rgbtBlue) and
	(C1.rgbtGreen = C2.rgbtGreen) and
	(C1.rgbtRed = C2.rgbtRed);
end;



function TripleDistance(C1,C2: TRGBTriple): double;
{Calculate distance between two colors - normalized to 0 to 1 range}
const MaxDist = 441.67295593;	{sqrt(sqr(256)+sqr(256)+sqr(256))}
begin
Result:=sqrt(sqr(C1.rgbtBlue - C2.rgbtBlue) +
	     sqr(C1.rgbtGreen - C2.rgbtGreen) +
	     sqr(C1.rgbtRed - C2.rgbtRed));
Result:=EnsureRange(Result/MaxDist,0,1);
end;



function ColorDistance(C1,C2: TColor): double;
{Calculate distance between two colors - normalized to 0 to 1 range}
begin
Result:=TripleDistance(ColorToTriple(C1),ColorToTriple(C2));
end;



function XORColors(C1,C2: TColor): TColor;
{XOR together the components of two colors}
var T1,T2: TRGBTriple;
begin
T1:=ColorToTriple(C1); T2:=ColorToTriple(C2);
Result:=RGB(T1.rgbtRed xor T2.rgbtRed,T1.rgbtGreen xor T2.rgbtGreen,T1.rgbtBlue xor T2.rgbtBlue);
end;



function InvertColor(C: TColor): TColor;
{Invert color by XORing each component with $FF}
var T: TRGBTriple;
begin
T:=ColorToTriple(C);
Result:=RGB(T.rgbtRed xor $FF,T.rgbtGreen xor $FF,T.rgbtBlue xor $FF);
end;



function TripleToColor(T: TRGBTriple): TColor;
{Convert RGBTriple to Delphi TColor}
begin
Result:=T.rgbtRed or (T.rgbtGreen shl 8) or (T.rgbtBlue shl 16);
end;



function QuadToColor(T: TRGBQuad): TColor;
{Convert RGBTriple to Delphi TColor}
begin
Result:=T.rgbRed or (T.rgbGreen shl 8) or (T.rgbBlue shl 16);
end;



procedure ColorToRGBBytes(Color: TColor; var R,G,B: byte);
{Extract individual RGB values for TColor variable}
var C: integer;
begin
{Important: this function handles system colors, don't mask}
C:=ColorToRGB(Color);
R:=GetRValue(C);
G:=GetGValue(C);
B:=GetBValue(C);
end;


function ColorToRGBString(C: TColor; Hex: boolean): string;
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
if Hex then Result:=IntToHex(R,2)+','+IntToHex(G,2)+','+IntToHex(B,2)
else Result:=IntToStr(R)+','+IntToStr(G)+','+IntToStr(B);
end;



function ColorToRGBString(C: TColor): string;
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
Result:=IntToStr(R)+','+IntToStr(G)+','+IntToStr(B);
end;


function RGBTripleToString(C: TRGBTriple; Hex: boolean): string;
begin
if Hex then Result:=IntToHex(C.rgbtRed,2)+','+IntToHex(C.rgbtGreen,2)+','+IntToHex(C.rgbtBlue,2)
else Result:=IntToStr(C.rgbtRed)+','+IntToStr(C.rgbtGreen)+','+IntToStr(C.rgbtBlue);
end;


function RGBTripleToString(C: TRGBTriple): string;
begin
Result:=RGBTripleToString(C,False);
end;


function RGBQuadToString(C: TRGBQuad; Hex: boolean): string;
begin
if Hex then Result:=IntToHex(C.rgbRed,2)+','+IntToHex(C.rgbGreen,2)+','+IntToHex(C.rgbBlue,2)+','+IntToHex(C.rgbReserved,2)
else Result:=IntToStr(C.rgbRed)+','+IntToStr(C.rgbGreen)+','+IntToStr(C.rgbBlue)+','+IntToStr(C.rgbReserved);
end;


function RGBQuadToString(C: TRGBQuad): string;
begin
Result:=RGBQuadToString(C,False);
end;


function ColorToVector(Color: TColor): T3DVector;
{Convert Delphi color to color vector}
var R,G,B: byte;
begin
ColorToRGBBytes(Color,R,G,B);
Result:=MakeVector3D(R,G,B);
Result:=ScalarDivide3D(Result,255);
end;



function RGBTripleToVector(Color: TRGBTriple): T3DVector;
{Convert RGB Triple color to color vector}
begin
Result:=MakeVector3D(Color.rgbtRed,Color.rgbtGreen,Color.rgbtBlue);
Result:=ScalarDivide3D(Result,255);
end;


function ByteRange(V: double): byte;
begin
Result:=Trunc(EnsureRange(V,0,255));
end;


function VectorToColor(V: T3DVector): TColor;
{Convert 3D Vector to Delphi Color}
begin
V:=ScalarProduct3D(V,255);
Result:=RGB(ByteRange(V.X),ByteRange(V.Y),ByteRange(V.Z));
end;



function Color15ToColor(C: integer): TColor;
{Convert a 5:5:5 RGB Color to Delphi Color}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $1F;
G:=(C shr 5) and $1F;
R:=(C shr 10) and $1F;
Result:=RGB(MulDiv(R,255,31),MulDiv(G,255,31),MulDiv(B,255,31));
end;



function Color16ToColor(C: integer): TColor;
{Convert a 5:6:5 Color to Delphi Color}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $1F;
G:=(C shr 5) and $3F;
R:=(C shr 11) and $1F;
Result:=RGB(MulDiv(R,255,31),MulDiv(G,255,63),MulDiv(B,255,31));
end;



function Color24ToColor(C: integer): TColor;
{Convert a 8:8:8 Color to Delphi Color}
{Also works with 32-bit color although alphi is not handled}
{Note: Windows/Delphi Color is BGR}
var R,G,B: byte;
begin
B:=C and $FF;
G:=(C shr 8) and $FF;
R:=(C shr 16) and $FF;
Result:=RGB(R,G,B);
end;




function GetRGBIntensity(C: TColor): double;
{Return intensity 0-1 value}
var R,G,B: byte;
begin
ColorToRGBBytes(C,R,G,B);
Result:=(R*299 + G*587 + B*114)/255000;
end;


function ContrastColor(C: TColor): TColor;
{Return most constrating color to C}
var I: double;
begin
I:=GetRGBIntensity(C);
if I>0.5 then Result:=clBlack else Result:=clWhite;
end;




{--------------------- Bitmap List object -------------------------------------}


function TBitmapList.AddBitmap(BM: TBitmap): integer;
var TBM: TBitmap;
var H: HBITMAP;
{$HINTS OFF}
begin
TBM:=TBitmap.Create;
TBM.Assign(BM);
{Forces bitmap to be independent}
{Ignore warnings Assignment to H required}
H:=TBM.Handle;
Result:=Add(TBM);
FAuxInfo.Add(0);
end;
{$HINTS ON}



function TBitmapList.GetBitmap(Inx: integer): TBitmap;
begin
Result:=nil;
if Inx>=Count then exit;
Result:=Items[Inx];
end;



procedure TBitmapList.PutBitmap(Inx: integer; Item: TBitmap);
begin
if Inx>=Count then exit;
Items[Inx]:=Item;
end;



function TBitmapList.GetAuxInfo(Index: integer): integer;
begin
Result:=FAuxInfo[Index];
end;



procedure TBitmapList.PutAuxInfo(Index: integer; const Value: integer);
begin
FAuxInfo[Index]:=Value;
end;



procedure TBitmapList.Clear;
var I: integer;
begin
for I:=0 to Count-1 do TBitmap(Items[I]).Free;
FAuxInfo.Clear;
inherited Clear;
end;


procedure TBitmapList.Delete(Index: Integer);
begin
TBitmap(Items[Index]).Free;
FAuxInfo.Delete(Index);
inherited Delete(Index);
end;



function TBitmapList.GetCount: integer;
begin
Result:=TList(Self).Count;
end;


procedure TBitmapList.SetCount(V: integer);
var I: integer;
begin
for I:=Count-1 downto V do
	begin
	Delete(I);
	FAuxInfo.Delete(I);
	end;
end;




constructor TBitmapList.Create;
begin
inherited Create;
FAuxInfo:=THugeInteger.Create(5);
end;




destructor TBitmapList.Destroy;
begin
inherited Destroy;
FAuxInfo.Free;
end;



procedure TBitmapList.Load(Filename: string; Progress: TBitmapProgress);
var InFilm: TInFilm;
var I: integer;
var BM: TBitmap;
begin
InFilm:=TInFilm.Create;
try
BM:=TBitmap.Create;
try
InFilm.OpenFile(FileName);
InFilm.Rewind;
Self.Clear;
for I:=0 to InFilm.Count-1 do
	begin
	InFilm.ReadBitmap(BM);
	Self.AddBitmap(BM);
	if Assigned(Progress) then Progress(MulDiv(100,I,InFilm.Count));
	Application.ProcessMessages;
	end;
finally BM.Free; end;
finally InFilm.Free; end;
end;





procedure TBitmapList.Save(Filename: string; Progress: TBitmapProgress);
var OutFilm: TOutFilm;
var I: integer;
begin
OutFilm:=TOutFilm.Create;
try
OutFilm.OpenFile(Filename);
try
for I:=0 to Self.Count-1 do
	begin
	OutFilm.WriteBitmap(Self.Bitmaps[I]);
	if Assigned(Progress) then Progress(MulDiv(100,I,Self.Count));
	Application.ProcessMessages;
	end;
except MessageDlg('Out Of Disk Space',mtWarning,[mbOK], 0); end;
 finally
 OutFilm.CloseFile;
 OutFilm.Free;
 end;
end;


procedure TBitmapList.PasteFromClipboard;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.LoadFromClipboardFormat(cf_BitMap,ClipBoard.GetAsHandle(cf_Bitmap),0);
AddBitmap(BM);
finally BM.Free; end;
end;



procedure TBitmapList.CopyToClipboard(Inx: integer);
var MyFormat: Word;
var AData: THandle;
var APalette: HPALETTE;
begin
if Inx>=Count then raise Exception.Create('Index out of range');
Self.Bitmaps[Inx].SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
end;



{------------------ Gaussian Blur routines ------------------------------------}


procedure MakeGaussianKernel(var K: TKernel; radius: double; MaxData, DataGranularity: double);
{makes K into a gaussian kernel with standard deviation = radius. For the 
current application you set MaxData = 255 and DataGranularity = 1. Now 
the procedure sets the value of K.Size so that when we use K we will 
ignore the Weights that are so small they can't possibly matter. (Small Size 
is good because the execution time is going to be propertional to K.Size.)} 
var j: integer; 
var temp, delta: double; 
var KernelSize: TKernelSize; 
begin 
for j:= Low(K.Weights) to High(K.Weights) do
	begin 
	temp := j / radius;
	K.Weights[j] := exp(-temp * temp / 2); 
	end; 

{now divide by constant so sum(Weights) = 1:} 
temp := 0; 
for j := Low(K.Weights) to High(K.Weights) do  temp := temp + K.Weights[j]; 
for j := Low(K.Weights) to High(K.Weights) do  K.Weights[j] := K.Weights[j] / temp; 

{now discard (or rather mark as ignorable by setting Size) the entries that 
are too small to matter - this is important, otherwise a blur with a small radius 
will take as long as with a large radius...} 

KernelSize := MaxKernelSize; 
delta := DataGranularity / (2 * MaxData); 
temp := 0; 
while (temp < delta) and (KernelSize > 1) do 
	begin 
	temp := temp + 2 * K.Weights[KernelSize]; 
	dec(KernelSize); 
	end; 
K.Size := KernelSize; 

{now just to be correct go back and jiggle again so the sum of the entries we'll be using is exactly 1} 
temp := 0; 
for j := -K.Size to K.Size do temp := temp + K.Weights[j]; 
for j := -K.Size to K.Size do K.Weights[j] := K.Weights[j] / temp; 
end;
   


function TrimInt(Lower, Upper, theInteger: integer): integer; 
begin 
if (theInteger <= Upper) and (theInteger >= Lower) then result := theInteger 
else if theInteger > Upper then result := Upper 
else  result := Lower; 
end; 


function TrimReal(Lower, Upper: integer; x: double): integer;
begin
if (x <=upper) and (x >= lower) then result := round(x)
else if x > Upper then result := Upper
else result := Lower;
end;



procedure BlurRow(var theRow: array of TRGBTriple; K: TKernel; P: PRow);
var  j, n: integer;
var tr, tg, tb: double; {tempRed, etc}
var w: double;
begin
for j := 0 to High(theRow) do
	begin
	tb := 0;
	tg := 0;
	tr := 0;
	for n := -K.Size to K.Size do
		begin
		w := K.Weights[n];
		{the TrimInt keeps us from running off the edge of the row...}
		with theRow[TrimInt(0, High(theRow), j - n)] do
		 begin
		 tb := tb + w * rgbtBlue;
		 tg := tg + w * rgbtGreen;
		 tr := tr + w * rgbtRed;
		 end; 
		end;
	with P[j] do
	 begin 
	 rgbtBlue:= TrimReal(0, 255, tb);
	 rgbtGreen := TrimReal(0, 255, tg);
	 rgbtRed := TrimReal(0, 255, tr);
	 end; 
	end;
Move(P[0], theRow[0], (High(theRow) + 1) * Sizeof(TRGBTriple));
end;



procedure GaussianBlur(theBitmap: TBitmap; radius: double);
var  Row, Col: integer;
var theRows: PPRows;
var K: TKernel;
var ACol: PRow;
var P: PRow;
begin
theBitmap.PixelFormat:=pf24Bit;
MakeGaussianKernel(K, radius, 255, 1);
GetMem(theRows, theBitmap.Height * SizeOf(PRow));
GetMem(ACol, theBitmap.Height * SizeOf(TRGBTriple));

{record the location of the bitmap data:}
for Row := 0 to theBitmap.Height - 1 do theRows[Row] := theBitmap.Scanline[Row];

{blur each row:}
P := AllocMem(theBitmap.Width * SizeOf(TRGBTriple));
for Row := 0 to theBitmap.Height - 1 do BlurRow(Slice(theRows[Row]^, theBitmap.Width), K, P);

{now blur each column}
ReAllocMem(P, theBitmap.Height * SizeOf(TRGBTriple));
for Col := 0 to theBitmap.Width - 1 do
	begin
	{first read the column into a TRow:}
	for Row := 0 to theBitmap.Height - 1 do ACol[Row] := theRows[Row][Col];
	BlurRow(Slice(ACol^, theBitmap.Height), K, P);

	{now put that row, um, column back into the data:}
	for Row := 0 to theBitmap.Height - 1 do theRows[Row][Col] := ACol[Row];
	end;
FreeMem(theRows);
FreeMem(ACol);
ReAllocMem(P, 0);
end;






procedure GaussianBlur(theBitmap: TBitmap; radius: double; Rect: TRect);
var BM: TBitmap;
begin
if Rect.Left>Rect.Right then SwapIntegers(Rect.Left,Rect.Right);
if Rect.Top>Rect.Bottom then SwapIntegers(Rect.Top,Rect.Bottom);
Rect.Left:=EnsureRange(Rect.Left,0,theBitmap.Width-1);
Rect.Right:=EnsureRange(Rect.Right,0,theBitmap.Width-1);
Rect.Top:=EnsureRange(Rect.Top,0,theBitmap.Height-1);
Rect.Bottom:=EnsureRange(Rect.Bottom,0,theBitmap.Height-1);
BM:=TBitmap.Create;
try
BM.Width:=Rect.Right-Rect.Left;
BM.Height:=Rect.Bottom-Rect.Top;
BM.Canvas.CopyRect(BM.Canvas.ClipRect,theBitmap.Canvas,Rect);
GaussianBlur(BM, radius);
theBitmap.Canvas.CopyRect(Rect,BM.Canvas,BM.Canvas.ClipRect);

finally BM.Free; end;
end;





procedure GaussBlurCircle(Canvas: TCanvas; P: TPoint; Size: integer; Radius: double);
var RGN: HRGN;
var SourceRect: TRect;
var BM: TBitmap;
var S1,S2: integer;
begin
SourceRect:=Rect(P,P);
{Get rectangle dimensions accounting for interization error}
S1:=Size div 2; S2:=Ceil(Size/2);
{Create source retangle of specified size}
Dec(SourceRect.Left,S1); Dec(SourceRect.Top,S1);
Inc(SourceRect.Right,S2+1); Inc(SourceRect.Bottom,S2+1);
{Make sure rectangle doesn't exceed image boundary}
IntersectRect(SourceRect,SourceRect,Canvas.ClipRect);

{Create bitmap of same size}
BM:=TBitmap.Create;
try
BM.Width:=(SourceRect.Right-SourceRect.Left)+1;
BM.Height:=(SourceRect.Bottom-SourceRect.Top)+1;
{Copy source part of image to bitmap}
BM.Canvas.CopyRect(BM.Canvas.ClipRect,Canvas,SourceRect);

{Blur the bitmap}
GaussianBlur(BM, Radius);

{Create circular clipping region}
RGN:=CreateEllipticRgnIndirect(SourceRect);
try
SelectClipRgn(Canvas.Handle, RGN);
try
{Draw blurred image clipped to circle}
Canvas.Draw(SourceRect.Left,SourceRect.Top,BM);

{Clean up}
finally SelectClipRgn(Canvas.Handle, 0); end;
finally DeleteObject(RGN); end;
finally BM.Free; end;
end;


procedure InvertImage(BM: TBitmap);
{Make create the negative image of the bitmap}
var X1,Y1: integer;
var Scan: pRGBTripleArray;
begin
BM.PixelFormat:=pf24Bit;
for Y1:=0 to BM.Height-1 do
	begin
	Scan:=BM.ScanLine[Y1];
	for X1:=0 to BM.Width-1 do
		begin
                Scan[X1].rgbtRed:=255-Scan[X1].rgbtRed;
                Scan[X1].rgbtGreen:=255-Scan[X1].rgbtRed;
                Scan[X1].rgbtBlue:=255-Scan[X1].rgbtBlue;
		end;
	end;
end;







procedure UnsharpMask(BM: TBitmap; Radius, Threshold, Amount: double);
var Blur: TBitmap;
var ImageScan,BlurScan: pRGBTripleArray;
var X1,Y1: integer;


	function MergeElements(InColor,BlurColor: byte; Amount,Threshold: double): byte;
	begin
	if abs(InColor - BlurColor) >= Threshold then Result:=max(0, min(255, round(Amount * (InColor - BlurColor)) + InColor))
	else Result:=InColor;
	end;


begin
BM.PixelFormat:=pf24Bit;
Blur:=TBitmap.Create;
try
Blur.Assign(BM);
GaussianBlur(Blur,Radius);
for Y1:=0 to BM.Height-1 do
	begin
	ImageScan:=BM.ScanLine[Y1];
	BlurScan:=Blur.ScanLine[Y1];

	for X1:=0 to BM.Width-1 do
		begin
		ImageScan[X1].rgbtRed:=MergeElements(ImageScan[X1].rgbtRed,BlurScan[X1].rgbtRed,Amount,Threshold);
		ImageScan[X1].rgbtGreen:=MergeElements(ImageScan[X1].rgbtGreen,BlurScan[X1].rgbtGreen,Amount,Threshold);
		ImageScan[X1].rgbtBlue:=MergeElements(ImageScan[X1].rgbtBlue,BlurScan[X1].rgbtBlue,Amount,Threshold);
		end;
	end;

finally Blur.Free; end;
end;




{------------- Drawing and Filling routines -----------------------------------}


procedure ColorFill(BM: TBitmap; X,Y: integer; Threshold: double; NewColor: TColor);
{Do fill all pixels that match the pixel found at X,Y}
{Threshold controls how close the match must be}
var X1,Y1: integer;
var TargetColor: TColor;
var Scan: pRGBTripleArray;
var N,T: TRGBTriple;
var D: double;
begin
BM.PixelFormat:=pf24Bit;
N:=ColorToTriple(NewColor);
TargetColor:=BM.Canvas.Pixels[X,Y];
T:=ColorToTriple(TargetColor);
for Y1:=0 to BM.Height-1 do
	begin
	Scan:=BM.ScanLine[Y1];
	for X1:=0 to BM.Width-1 do
		begin
                D:=TripleDistance(Scan[X1],T);
		if D<=Threshold then Scan[X1]:=N;
		end;
	end;
end;

{-------------- Pen Drawing Routines ------------------------------------------}

procedure DrawPixels(Image: TImage; X,Y, Size: integer; C: TColor);
{Draw pixels at position specified at XY with pen of Size Width and Height}
var OX,OY,X1,Y1: integer;
begin
for OY:=0 to Size-1 do
 for OX:=0 to Size-1 do
	begin
	X1:=X+OX; Y1:=Y+OY;
	if (X1>=0) and (X1<Image.Width) or
	   (Y1>=0) and (Y1<Image.Height) then Image.Canvas.Pixels[X1,Y1]:=C;
	end;
end;


var OldX,OldY: integer;


type TDDAData = record
 Size: integer;
 Image: TImage;
 C: TColor;
 end;
type PDDAData = ^TDDAData;



procedure DDAProc(X,Y: integer; LParam : LongInt); stdcall;
{Custon draw callback procedure}
var Data: TDDaData;
begin
Data:=TDDAData(Pointer(LParam)^);
DrawPixels(Data.Image,X,Y,Data.Size,Data.C);
end;



procedure DrawCustomLine(Image: TImage; X1,Y1,X2,Y2, Size: integer; C: TColor);
{Custom drawing procedure to do continuous drawing between points}
{}
var Data: TDDAData;
begin
Data.Size:=Size;
Data.Image:=Image;
Data.C:=C;
LineDDA(X1, Y1, X2, Y2, @DDAProc, LongInt(@Data));
{Draw last pixel because line routine don't do it.}
DrawPixels(Image,X2,Y2,Size,C);
end;



procedure ContinuousDraw(Image: TImage; Start: boolean; X,Y, Size: integer; C: TColor);
{Draw pixels at position specified at XY with pen of Size Width and Height}
{Guaranteed to be connected with previously drawing pixels}
begin
if Start then
	begin
	DrawPixels(Image,X,Y,Size,C);
	end
else
	begin
	DrawCustomLine(Image,OldX,OldY,X,Y,Size,C);
	end;
OldX:=X; OldY:=Y;
end;


{------------------ Threshold Controllable Fill Routines  --------------------------}

function GetAliasColor(OldColor,FillColor: TRGBTriple): TRGBTriple;
var H1,H2: THSBRecord;
begin
H1:=RGBToHSB(OldColor);
H2:=RGBToHSB(FillColor);
H2.Bright:=H1.Bright * H2.Bright;
Result:=HSBToRGB(H2);
end;


procedure AntiAliasFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{ Fill An Area Of Target Color with Fill Color }
{ Threshold controls which colors match the target color}
{ Threshold = 0 = only an exact match. Threshold = 1 = any color matches}
var Raw: pointer;
var Row: pRGBTRipleArray;
var Que: THugePoint;
var E,BytesPerRow: Integer;
var FC,TC: TRGBTriple;
var MaxInt,I: integer;
const TrimCount=1000;
begin
MaxInt:=round(BM.Width*BM.Height*4);
Que:=THugePoint.Create(5);
try
Que.Clear;
BM.PixelFormat:=pf24Bit;
TC:=ColorToTriple(Target);
FC:=ColorToTriple(Fill);
Raw:=BM.Scanline[0];
BytesPerRow:=(integer(BM.Scanline[1]) - integer(BM.Scanline[0]));
E:= 0;
I:=0;
Que.AddPointer(Point(X,Y),Que.Count);
while E < Que.Count do
	begin
	if ((E mod 1000)=0) and Assigned(ProgressProc) then ProgressProc(MulDiv(100,I,MaxInt));
	X:= Que[E].X; Y:= Que[E].Y;
	Inc(E); Inc(I);
	Row:=pRGBTRipleArray(Integer(Raw) + (BytesPerRow * Y));
	if not EqualRGBTriple(Row[X],FC) and
	   (TripleDistance(Row[X],TC)<=Threshold) then
		begin
		if EqualRGBTriple(Row[X],TC) then
			begin
			Row[X]:=FC;
			if X<BM.Width-1 then Que.AddPointer(Point(X+1, Y),Que.Count);
			if X>0 then Que.AddPointer(Point(X-1, Y),Que.Count);
			if Y<BM.Height-1 then Que.AddPointer(Point(X, Y+1),Que.Count);
			if Y>0 then Que.AddPointer(Point(X, Y-1),Que.Count);
			end
		else Row[X]:=GetAliasColor(Row[X],FC);
		end;

	{Trim off part of array so we don't run out of memory}
	if E>TrimCount then
		begin
		Que.DeleteRange(0,TrimCount-1);
		Dec(E,TrimCount);
		end;
	end;
if Assigned(ProgressProc) then ProgressProc(100);
finally Que.Free; end;
end;



procedure ThresholdFill(BM: TBitmap; X, Y: integer; Target, Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{ Fill An Area Of Target Color with Fill Color }
{ Threshold controls which colors match the target color}
{ Threshold = 0 = only an exact match. Threshold = 1 = any color matches}
var Raw: pointer;
var Row: pRGBTRipleArray;
var Que: THugePoint;
var E,BytesPerRow: Integer;
var FC,TC: TRGBTriple;
var MaxInt,I: integer;
const TrimCount=1000;
begin
MaxInt:=round(BM.Width*BM.Height*4);
Que:=THugePoint.Create(5);
try
Que.Clear;
BM.PixelFormat:=pf24Bit;
TC:=ColorToTriple(Target);
FC:=ColorToTriple(Fill);
Raw:=BM.Scanline[0];
BytesPerRow:=(integer(BM.Scanline[1]) - integer(BM.Scanline[0]));
E:= 0;
I:=0;
Que.AddPointer(Point(X,Y),Que.Count);
while E < Que.Count do
	begin
	X:= Que[E].X; Y:= Que[E].Y; Inc(E);
	Inc(I);
	Row:=pRGBTRipleArray(Integer(Raw) + (BytesPerRow * Y));
	if not EqualRGBTriple(Row[X],FC) and
	   (TripleDistance(Row[X],TC)<=Threshold) then
		begin
		Row[X]:=FC;
		if X<BM.Width-1 then Que.AddPointer(Point(X+1, Y),Que.Count);
		if X>0 then Que.AddPointer(Point(X-1, Y),Que.Count);
		if Y<BM.Height-1 then Que.AddPointer(Point(X, Y+1),Que.Count);
		if Y>0 then Que.AddPointer(Point(X, Y-1),Que.Count);
		end;
	if ((E mod 1000)=0) and Assigned(ProgressProc) then
		begin
		ProgressProc(MulDiv(100,I,MaxInt));
		end;
	{Trim off part of array so we don't run out of memory}
	if E>TrimCount then
		begin
		Que.DeleteRange(0,TrimCount-1);
		Dec(E,TrimCount);
		end;
	end;
if Assigned(ProgressProc) then ProgressProc(100);
finally Que.Free; end;
end;



procedure SeedFloodFill(BM: TBitmap; X, Y: integer; Fill: TColor; Threshold: double; ProgressProc: TFloodProgress);
{Flood fills based on seed at X,Y}
var Target: TColor;
begin
Target:=BM.Canvas.Pixels[X,Y];
ThresholdFill(BM,X,Y,Target,Fill,Threshold,ProgressProc);
end;



{------------------Clipboard routines -----------------------------------------}


procedure SaveRectToClipboard(Bitmap: TBitmap; R: TRect);
{Copy specified retangle from bitmap to clipboard}
var MyFormat: Word;
var AData: THandle;
var APalette: HPALETTE;
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.Width:=R.Right - R.Left;
BM.Height:=R.Bottom - R.Top;
BM.Canvas.CopyRect(Rect(0,0,BM.Width,BM.Height),Bitmap.Canvas,R);
BM.SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
finally BM.Free; end;
end;



{-------------- Routines to set Bitmap Pixels Per Meter  ----------------------}


const PelPerMeterPosX = 38;
const PelPerMeterPosY = 42;


procedure GetBmpStreamPPM(Stream: TStream; var PPMX,PPMY: integer);
begin
Stream.Position := PelPerMeterPosX;
if Stream.Read(PPMX,4) <> 4 then raise Exception.Create('Error reading to Bitmap.');
Stream.Position := PelPerMeterPosY;
if Stream.Read(PPMY,4) <> 4 then raise Exception.Create('Error reading to Bitmap.');
end;





procedure GetBmpPPM(Bitmap: TBitmap; var PPMX,PPMY: integer);
var Stream: TMemoryStream;
begin
Stream := TMemoryStream.Create;
try
Bitmap.SaveToStream(Stream);
GetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;





procedure GetBMPFilePPM(FileName: string; var PPMX,PPMY: integer);
var Stream: TFileStream;
begin
Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
try
GetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;

{-----------------------------------------------------------------------------}



procedure SetBMPStreamPPM(Stream: TStream; PPMX,PPMY: integer);
begin
Stream.Position := PelPerMeterPosX;
if Stream.Write(PPMX,4) <> 4 then raise Exception.Create('Error writing to Bitmap.');
Stream.Position := PelPerMeterPosY;
if Stream.Write(PPMY,4) <> 4 then raise Exception.Create('Error writing to Bitmap.');
Stream.Position := 0;
end;




procedure SetBMPPPM(BM: TBitmap; PPMX,PPMY: integer);
var Stream: TMemoryStream;
begin
Stream := TMemoryStream.Create;
try
BM.SaveToStream(Stream);
SetBMPStreamPPM(Stream,PPMX,PPMY);
Stream.Position := 0;
BM.LoadFromStream(Stream);
finally Stream.Free; end;
end;



procedure SetBMPFilePPM(FileName: String; PPMX,PPMY: integer);
var Stream: TFileStream;
begin
Stream := TFileStream.Create(FileName, fmOpenWrite or fmShareExclusive);
try
SetBMPStreamPPM(Stream,PPMX,PPMY);
finally Stream.Free; end;
end;


procedure CopyBMPPPM(Source,Destination: TBitmap);
{Copy the Bits Per Meter from source to destination}
var X,Y: integer;
begin
GetBmpPPM(Source,X,Y);
SetBmpPPM(Destination,X,Y);
end;


procedure SetBMPPixelFormat(BM: TBitmap; PF: Graphics.TPixelFormat);
{Set pixel format while maintaining other attributes}
var X,Y: integer;
begin
if not BM.Empty then GetBmpPPM(BM,X,Y);
BM.PixelFormat:=PF;
if not BM.Empty then SetBmpPPM(BM,X,Y);
end;

{------------------------------------------------------------------------------}



procedure GetJPEGResolution(Filename: string; var Units: TJPEGResUnits; var XRes,YRes: integer);
const BufferSize = 50;
var Buffer: string;
var Index: integer;
var FileStream: TFileStream;
var DP: Byte;
var HorzRes, VertRes: word;
begin
FileStream := TFileStream.Create(Filename, fmOpenReadWrite);
try
SetLength(Buffer, BufferSize);
FileStream.Read(buffer[1], BufferSize);
Index := Pos('JFIF' + #$00, buffer);
if Index > 0 then
	begin
	FileStream.Seek(Index + 6, soFromBeginning);
	FileStream.Read(DP, 1);
	case DP of
	 0: Units:=juNoUnits;
	 1: Units:=juDotsPerInch;
	 2: Units:=juDotsPerCentimeter;
	 end;
	FileStream.Read(HorzRes, 2); // x axis
	XRes := Swap(HorzRes);
	FileStream.Read(VertRes, 2); // y axis
	YRes := Swap(VertRes);
	end
finally FileStream.Free; end;
end;



procedure SetJPEGResolution(Filename: string; DPIX, DPIY: Integer; Units: TJPEGResUnits=juDotsPerInch);
const BufferSize = 50;
var Buffer: string;
var index: integeR;
var FileStream: TFileStream;
var XResolution: word;
var YResolution: word;
var ResUints: byte;
begin
FileStream := TFileStream.Create(Filename,fmOpenReadWrite);
try
SetLength(Buffer, BufferSize);
FileStream.Read(buffer[1], BufferSize);
index := POS('JFIF' + #$00, buffer);
if index > 0 then
	begin
	FileStream.Seek(index + 6, soFromBeginning);
	ResUints := byte(Units);
	FileStream.write(ResUints, 1);
	XResolution := swap(DPIX);
	FileStream.write(XResolution, 2);
	YResolution := swap(DPIY);
	FileStream.write(YResolution, 2);
	end
finally FileStream.Free; end;
end;




procedure CopyBitmap(Dest,Source: TBitmap);
{Copy bitmap - preferred to Assign, since assign created new GDI object every time}
begin
Dest.Width:=Source.Width;
Dest.Height:=Source.Height;
Dest.Canvas.Draw(0,0,Source);
end;




{-------------- Routines to do Flips and rotations mod 90 degrees -------------}


procedure FlipBitmapHoriz(Source,Destination: TBitmap);
var X,Y,W: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;
W:=Source.Width-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2[W-X]:=Scan1[X];
		end;
	end;
end;



procedure FlipBitmapHoriz(BM: TBitmap);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
FlipBitmapHoriz(Tmp,BM);
finally TMP.Free; end;
end;




procedure FlipBitmapVertical(Source,Destination: TBitmap);
var X,Y,H: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;
H:=Source.Height-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[H-Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2[X]:=Scan1[X];
		end;
	end;
end;


procedure FlipBitmapVertical(BM: TBitmap);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
FlipBitmapVertical(Tmp,BM);
finally TMP.Free; end;
end;



procedure FlipBitmap90(Source,Destination: TBitmap);
var X,Y,H: integer;
var Scan1,Scan2: pRGBQuadArray;
begin
Source.PixelFormat:=pf32Bit;
Destination.PixelFormat:=pf32Bit;
Destination.Width:=Source.Height;
Destination.Height:=Source.Width;
H:=Source.Height-1;
for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		Scan2:=Destination.ScanLine[X];
		Scan2[H-Y]:=Scan1[X];
		end;
	end;
end;


procedure FlipBitmap90(BM: TBitmap);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
FlipBitmap90(Tmp,BM);
finally TMP.Free; end;
end;



procedure FlipBitmap270(Source,Destination: TBitmap);
var BM,BM2: TBitmap;
begin
BM:=TBitmap.Create;
BM2:=TBitmap.Create;
try
FlipBitmap90(Source,BM);
FlipBitmapHoriz(BM,BM2);
FlipBitmapVertical(BM2,Destination);
finally
 BM.Free;
 BM2.Free;
 end;
end;



procedure FlipBitmap270(BM: TBitmap);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
FlipBitmap270(Tmp,BM);
finally TMP.Free; end;
end;


procedure FlipBitmap180(Source,Destination: TBitmap);
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
FlipBitmapHoriz(Source,BM);
FlipBitmapVertical(BM,Destination);
finally
 BM.Free;
 end;
end;



procedure FlipBitmap180(BM: TBitmap);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
FlipBitmap180(Tmp,BM);
finally TMP.Free; end;
end;

{------------------------------------------------------------------------------}



procedure DoRotateBitmap(Source, Destination: TBitmap; Center,Offset: TPoint; Angle,Scale: double; BackGround: TColor);
{Do low level bitmap rotation - highly optimized for speed }
{Copies and rotates bitmap from "Source" to "Destination" (both must be 24-bit)}
{Rotation is around "Center" and image is shifted by "Offset" vector }
{"Angle" controls rotation - Positive angles rotate clockwise}
{"Scale" scales the image up or down in size i.e. }
{"Background" is the color of the background when rotated}
var X,Y: integer;
var BytesPerRowSrc,BytesPerRowDest: integer;
var CosTheta,SinTheta: extended;
var ScaledSinTheta,ScaledCosTheta: integer;
var SourceX,SourceY: integer;
var DY: integer;
var RotatedRow: pRGBTRipleArray;
var BackTriple: TRGBTriple;
var RawSource,RawDestination: Pointer;
var SourceWidth,SourceHeight,DestinationWidth,DestinationHeight: integer;
var XOffFact,YOffFact: integer;
var SinFact1,CosFact1: integer;
var SinFact2,CosFact2: array of integer;
const IntegerScaler = 512;
begin
GDIFlush;
{Convert background color to RGB triple for speed}
BackTriple:=ColorToTriple(BackGround);

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale by power of two so we can use integers for speed}
{Also scale to control size}
ScaledSinTheta:=Round(SinTheta * IntegerScaler * (1/Scale));
ScaledCosTheta:=Round(CosTheta * IntegerScaler * (1/Scale));

{Calculate byte per row}
BytesPerRowSrc:=(integer(Source.Scanline[1]) - integer(Source.Scanline[0]));
BytesPerRowDest:=(integer(Destination.Scanline[1]) - integer(Destination.Scanline[0]));

{Get pointer to raw pixels for speed}
RawSource:=Source.Scanline[0];
RawDestination:= Destination.Scanline[0];

{Use predecremented constants for speed}
SourceWidth:=Source.Width-1;
SourceHeight:=Source.Height-1;
DestinationWidth:=Destination.Width-1;
DestinationHeight:=Destination.Height-1;

{Precalculate Offset factors for speed}
XOffFact:=Center.X + Offset.X;
YOffFact:=Center.Y + Offset.Y;

{Precalculate X-based Sin/Cos factors for speed}
{ First, Allocate memory for arrays}
SetLength(SinFact2,Destination.Width);
SetLength(CosFact2,Destination.Width);
{ Then fill arrays with factors}
for X:=0 to Destination.Width -1 do
	begin
	SinFact2[X]:=(X - XOffFact) * ScaledSinTheta;
	CosFact2[X]:=(X - XOffFact) * ScaledCosTheta;
	end;


{Step through all rows (Decrementing a for-loop is faster)}
for Y:= DestinationHeight downto 0 do
	begin
	{Caculate all "Y" based parameters}
	RotatedRow:= pRGBTRipleArray(Integer(RawDestination) + (BytesPerRowDest * Y));
	DY:=Y - YOffFact;
	SinFact1:=DY * ScaledSinTheta;
	CosFact1:=DY * ScaledCosTheta;
	{Step through all pixels in row}
	for X:= DestinationWidth downto 0 do
		begin
		{calculate the rotated X,Y indexes that point into source bitmap}
		SourceX:= Center.X + (CosFact2[X] - SinFact1) div IntegerScaler;
		SourceY:= Center.Y + (SinFact2[X] + CosFact1) div IntegerScaler;
		{Is Rotated X,Y still inside Source? If not write background color}
		if (SourceX >= 0) and (SourceX <= SourceWidth) and
		   (SourceY >= 0) and (SourceY <= SourceHeight) then
			RotatedRow[X]:= pRGBTRipleArray(Integer(RawSource)+(SourceY * BytesPerRowSrc))^[SourceX]
		else RotatedRow[X]:=BackTriple;
		end;
	end;
end;



procedure RotateBitmap(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
{Rotates 'Source' and returns result in 'Destination'. Positive angles rotate clock wise}
{Options explained above. Center specifies center of rotation only for clipped modes}
{Background is the color of the background when rotated}
var Offset: TPoint;
var CosTheta,SinTheta: extended;
var SS: TPoint;
begin
Destination.Canvas.Lock;
Source.Canvas.Lock;
try
{Force pixel format and make DIBs (otherwise there will be memory limitations)}
Source.PixelFormat:=pf24bit;
Destination.PixelFormat:= pf24bit;

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale Source Size for scaling operations}
SS:=IntScalarProduct(Point(Source.Width,Source.Height),Scale);
{Handle framing options}
case Options of
 foSource:
	begin
	{Use un-rotated source for bitmap size}
	Destination.Width:= Source.Width;
	Destination.Height:= Source.Height;
	Offset.X:=0; Offset.Y:=0;
	end;
 foDestination:
	begin
	{Use un-rotated destination for bitmap size}
       	Offset.X:=0; Offset.Y:=0;
	end;
 foDestCenter:
	begin
	{Put rotational center in center of destination}
	Offset.X:=-(Center.X - (Destination.Width div 2));
	Offset.Y:=-(Center.Y - (Destination.Height div 2));
	end;
 foFit:
	begin
	{Use rotated image as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:= abs(round(SS.Y * sinTheta)) + abs(round(SS.X * cosTheta));
	Destination.Height:= abs(round(SS.X * sinTheta)) + abs(round(SS.Y * cosTheta));
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 foWindow:
	begin
	{Use largest possible size as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:=round(sqrt(sqr(SS.Y) + sqr(SS.X)));
	Destination.Height:=Destination.Width;
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 else begin Offset.X:=0; Offset.Y:=0; end;
 end;

DoRotateBitmap(Source, Destination,Center,Offset,Angle,Scale,BackGround);

finally
 Destination.Canvas.Unlock;
 Source.Canvas.Unlock;
 end;
end;



{------------------------------------------------------------------------------}


procedure DoRotateBitmap32(Source, Destination: TBitmap; Center,Offset: TPoint; Angle,Scale: double; BackGround: TColor);
{Do low level bitmap rotation - highly optimized for speed }
{Copies and rotates bitmap from "Source" to "Destination" (both must be 32-bit)}
{Rotation is around "Center" and image is shifted by "Offset" vector }
{"Angle" controls rotation - Positive angles rotate clockwise}
{"Scale" scales the image up or down in size i.e. }
{"Background" is the color of the background when rotated}
var X,Y: integer;
var BytesPerRowSrc,BytesPerRowDest: integer;
var CosTheta,SinTheta: extended;
var ScaledSinTheta,ScaledCosTheta: integer;
var SourceX,SourceY: integer;
var DY: integer;
var RotatedRow: pRGBQuadArray;
var BackQuad: TRGBQuad;
var RawSource,RawDestination: Pointer;
var SourceWidth,SourceHeight,DestinationWidth,DestinationHeight: integer;
var XOffFact,YOffFact: integer;
var SinFact1,CosFact1: integer;
var SinFact2,CosFact2: array of integer;
const IntegerScaler = 512;
begin
GDIFlush;
{Convert background color to RGB Quad for speed}
BackQuad:=ColorToQuad(BackGround);
BackQuad.rgbReserved:=0;
{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale by power of two so we can use integers for speed}
{Also scale to control size}
ScaledSinTheta:=Round(SinTheta * IntegerScaler * (1/Scale));
ScaledCosTheta:=Round(CosTheta * IntegerScaler * (1/Scale));

{Calculate byte per row}
BytesPerRowSrc:=(integer(Source.Scanline[1]) - integer(Source.Scanline[0]));
BytesPerRowDest:=(integer(Destination.Scanline[1]) - integer(Destination.Scanline[0]));

{Get pointer to raw pixels for speed}
RawSource:=Source.Scanline[0];
RawDestination:= Destination.Scanline[0];

{Use predecremented constants for speed}
SourceWidth:=Source.Width-1;
SourceHeight:=Source.Height-1;
DestinationWidth:=Destination.Width-1;
DestinationHeight:=Destination.Height-1;

{Precalculate Offset factors for speed}
XOffFact:=Center.X + Offset.X;
YOffFact:=Center.Y + Offset.Y;

{Precalculate X-based Sin/Cos factors for speed}
{ First, Allocate memory for arrays}
SetLength(SinFact2,Destination.Width);
SetLength(CosFact2,Destination.Width);
{ Then fill arrays with factors}
for X:=0 to Destination.Width -1 do
	begin
	SinFact2[X]:=(X - XOffFact) * ScaledSinTheta;
	CosFact2[X]:=(X - XOffFact) * ScaledCosTheta;
	end;


{Step through all rows (Decrementing a for-loop is faster)}
for Y:= DestinationHeight downto 0 do
	begin
	{Caculate all "Y" based parameters}
	RotatedRow:= pRGBQuadArray(Integer(RawDestination) + (BytesPerRowDest * Y));
	DY:=Y - YOffFact;
	SinFact1:=DY * ScaledSinTheta;
	CosFact1:=DY * ScaledCosTheta;
	{Step through all pixels in row}
	for X:= DestinationWidth downto 0 do
		begin
		{calculate the rotated X,Y indexes that point into source bitmap}
		SourceX:= Center.X + (CosFact2[X] - SinFact1) div IntegerScaler;
		SourceY:= Center.Y + (SinFact2[X] + CosFact1) div IntegerScaler;

		{Is Rotated X,Y still inside Source? If not write background color}
		if (SourceX >= 0) and (SourceX <= SourceWidth) and
		   (SourceY >= 0) and (SourceY <= SourceHeight) then
			RotatedRow[X]:= pRGBQuadArray(Integer(RawSource)+(SourceY * BytesPerRowSrc))^[SourceX]
		else RotatedRow[X]:=BackQuad;
		end;
	end;
end;



procedure RotateBitmap32(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
{Rotates 'Source' and returns result in 'Destination'. Positive angles rotate clock wise}
{Options explained above. Center specifies center of rotation only for clipped modes}
{Background is the color of the background when rotated}
var Offset: TPoint;
var CosTheta,SinTheta: extended;
var SS: TPoint;
begin
Destination.Canvas.Lock;
Source.Canvas.Lock;
try
{Force pixel format and make DIBs (otherwise there will be memory limitations)}
Source.PixelFormat:=pf32bit;
Destination.PixelFormat:=pf32bit;

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale Source Size for scaling operations}
SS:=IntScalarProduct(Point(Source.Width,Source.Height),Scale);
{Handle framing options}
case Options of
 foSource:
	begin
	{Use un-rotated source for bitmap size}
	Destination.Width:= Source.Width;
	Destination.Height:= Source.Height;
	Offset.X:=0; Offset.Y:=0;
	end;
 foDestination:
	begin
	{Use un-rotated destination for bitmap size}
       	Offset.X:=0; Offset.Y:=0;
	end;
 foDestCenter:
	begin
	{Put rotational center in center of destination}
	Offset.X:=-(Center.X - (Destination.Width div 2));
	Offset.Y:=-(Center.Y - (Destination.Height div 2));
	end;
 foFit:
	begin
	{Use rotated image as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:= abs(round(SS.Y * sinTheta)) + abs(round(SS.X * cosTheta));
	Destination.Height:= abs(round(SS.X * sinTheta)) + abs(round(SS.Y * cosTheta));
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 foWindow:
	begin
	{Use largest possible size as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:=round(sqrt(sqr(SS.Y) + sqr(SS.X)));
	Destination.Height:=Destination.Width;
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 else begin Offset.X:=0; Offset.Y:=0; end;
 end;

DoRotateBitmap32(Source, Destination,Center,Offset,Angle,Scale,BackGround);

finally
 Destination.Canvas.Unlock;
 Source.Canvas.Unlock;
 end;
end;




procedure RotateBitmap32(BM: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
RotateBitmap32(Tmp,BM,Options, Center, Angle,Scale,BackGround);
finally TMP.Free; end;
end;



{------------------------------------------------------------------------------}



procedure DoRotateBitmapGDIP(Source, Destination: TBitmap; Center,Offset: TPoint; Angle, Scale: double; BkColor: TColor = clNone);
{Do low level GDIP bitmap rotation - highly optimized for speed }
{Copies and rotates bitmap from "Source" to "Destination" (both must be 32-bit)}
{Rotation is around "Center" and image is shifted by "Offset" vector }
{"Angle" controls rotation - Positive angles rotate clockwise}
{"Scale" scales the image up or down in size i.e. }
{"Background" is the color of the background when rotated}
var Tmp: TGPBitmap;
var Matrix: TGPMatrix;
var Graphs: TGPGraphics;
begin
Source.PixelFormat:=pf24Bit;
Destination.PixelFormat:=pf24Bit;
Tmp := TGPBitmap.Create(Source.Handle, Source.Palette);
Matrix := TGPMatrix.Create;
try
Matrix.Translate(Offset.X,Offset.Y);
Matrix.RotateAt(Angle, MakePoint(Center.X + 0.0, Center.Y));
Matrix.Scale(Scale,Scale);
Graphs := TGPGraphics.Create(Destination.Canvas.Handle);
try
Graphs.Clear(ColorRefToARGB(ColorToRGB(BkColor)));
Graphs.SetTransform(Matrix);
Graphs.DrawImage(Tmp, (Cardinal(Source.Width) - Tmp.GetWidth) div 2, (Cardinal(Source.Height) - Tmp.GetHeight) div 2);
finally Graphs.Free; end;
finally
 Matrix.Free;
 Tmp.Free;
 end;
end;



procedure RotateBitmapGDIP(Source, Destination: TBitmap; Options: TFrameOptions; Center: TPoint; Angle,Scale: double; BackGround: TColor);
{Use GDIP to Rotates 'Source' and returns result in 'Destination'. Positive angles rotate clock wise}
{Options explained above. Center specifies center of rotation only for clipped modes}
{Background is the color of the background when rotated}
var Offset: TPoint;
var CosTheta,SinTheta: extended;
var SS: TPoint;
begin
Destination.Canvas.Lock;
Source.Canvas.Lock;
try
{Force pixel format and make DIBs (otherwise there will be memory limitations)}
Source.PixelFormat:=pf24bit;
Destination.PixelFormat:= pf24bit;

{We rotate by reading rotated pixels and writing unrotated}
{This is why the sign of the angle is negated}
SinCos(DegToRad(-Angle), SinTheta, CosTheta);

{Scale Source Size for scaling operations}
SS:=IntScalarProduct(Point(Source.Width,Source.Height),Scale);
{Handle framing options}
case Options of
 foSource:
	begin
	{Use un-rotated source for bitmap size}
	Destination.Width:= Source.Width;
	Destination.Height:= Source.Height;
	Offset.X:=0; Offset.Y:=0;
	end;
 foDestination:
	begin
	{Use un-rotated destination for bitmap size}
       	Offset.X:=0; Offset.Y:=0;
	end;
 foDestCenter:
	begin
	{Put rotational center in center of destination}
	Offset.X:=-(Center.X - (Destination.Width div 2));
	Offset.Y:=-(Center.Y - (Destination.Height div 2));
	end;
 foFit:
	begin
	{Use rotated image as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:= abs(round(SS.Y * sinTheta)) + abs(round(SS.X * cosTheta));
	Destination.Height:= abs(round(SS.X * sinTheta)) + abs(round(SS.Y * cosTheta));
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 foWindow:
	begin
	{Use largest possible size as bitmap size}
	Center.X:=Source.Width div 2;
	Center.Y:=Source.Height div 2;
	Destination.Width:=round(sqrt(sqr(SS.Y) + sqr(SS.X)));
	Destination.Height:=Destination.Width;
	Offset.X:=(Destination.Width - Source.Width) div 2;
	Offset.Y:=(Destination.Height - Source.Height) div 2;
	end;
 else begin Offset.X:=0; Offset.Y:=0; end;
 end;

DoRotateBitmapGDIP(Source, Destination,Center,Offset,Angle,Scale,BackGround);

finally
 Destination.Canvas.Unlock;
 Source.Canvas.Unlock;
 end;
end;



{------------------------------------------------------------------------------}



procedure BorderBitmap(BM: TBitmap; Color: TColor; Width: integer);
{Add equal border around bitmap of specified width and color}
var BM2: TBitmap;
begin
BM2:=TBitmap.Create;
try
BM2.Assign(BM);
BM.Width:=BM.Width + Width * 2;
BM.Height:=BM.Height + Width * 2;
BM.Canvas.Brush.Color:=Color;
BM.Canvas.Pen.Color:=Color;
BM.Canvas.Rectangle(Rect(0,0,BM.Width,BM.Height));
BM.Canvas.Draw(Width,Width,BM2);
finally BM2.Free; end;
end;

{---------------- Routines to Manipulate RGB and HSV --------------------------}



function RGBToGray(V: TRGBTriple): byte;
begin
Result:=(299 * V.rgbtRed + 587 * V.rgbtGreen + 114 * V.rgbtBlue) div 1000;
end;



procedure MakeBlackAndWhite(Source,Destination: TBitmap; Invert: boolean; Threshold: double);
{Make image black and white based on threshold setting (0-1)}
var X,Y: integer;
var Scan1,Scan2: pRGBTripleArray;
var A: double;
const MaxAbs = 1.73205080757;
var Black,White,T: TRGBTriple;
begin
Threshold:=Threshold * MaxAbs;
Black:=MakeRBGTriple(0,0,0);
White:=MakeRBGTriple(255,255,255);
if Invert then begin T:=Black; Black:=White; White:=T; end;

Source.PixelFormat:=pf24Bit;
Destination.PixelFormat:=pf24Bit;
Destination.Width:=Source.Width;
Destination.Height:=Source.Height;

for Y:=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.ScanLine[Y];
	for X:=0 to Source.Width-1 do
		begin
		A:=VectorABS3D(RGBTripleToVector(Scan1[X]));
		if A>Threshold then Scan2[X]:=White
		else Scan2[X]:=Black;
		end;
	end;
end;



function RGBtoHSB(RGB: TRGBTriple): THSBRecord;
{Convert RGB to  Hue,Saturation and Brightness}
{Hue = 0-360, Sat=0-1, Bright = 0-1}
var MinValue,MaxValue,Range: extended;
var RedPercent,GreenPercent,BluePercent: extended;
begin
Result.Sat:=0;
Result.Hue:=-1;
{Convert RGB to percentage}
RedPercent:=RGB.rgbtRed/255; GreenPercent:=RGB.rgbtGreen/255; BluePercent:=RGB.rgbtBlue/255;
MinValue:=min(RedPercent,min(GreenPercent,BluePercent));
MaxValue:=max(RedPercent,max(GreenPercent,BluePercent));
{Calculate Value (Brightness)}
Result.Bright:=MaxValue;
{Calculate Saturation}
Range:=MaxValue-MinValue;
if MaxValue<>0 then Result.Sat:=Range/MaxValue else  exit;
{Calculate Hue}
if Range=0 then Result.Hue:=0
else if RedPercent=MaxValue then Result.Hue:= (GreenPercent-BluePercent)/Range	// between yellow & magenta
else if GreenPercent=MaxValue then Result.Hue:=2+(BluePercent-RedPercent)/Range	// between cyan & yellow
else Result.Hue:=4+(RedPercent-GreenPercent)/Range;				// between magenta & cyan
{Convert to degrees}
Result.Hue:=Result.Hue * 60;
while Result.Hue<0 do Result.Hue:=Result.Hue + 360;
end;



function ColorToHSB(Color: TColor): THSBRecord;
var RGB: TRGBTriple;
begin
RGB:=ColorToTriple(Color);
Result:=RGBToHSB(RGB);
end;



function MakeHSB(Hue,Sat,Bright: extended): THSBRecord;
begin
Result.Hue:=Hue;
Result.Sat:=Sat;
Result.Bright:=Bright;
end;



function HSBtoRGB(HSB: THSBRecord): TRGBTriple;
{Convert Hue,Saturation and Brightness to RGB}
{Hue = 0-360, Sat=0-1, Bright = 0-1}
var HexIndex: integer;
var Fraction,X,Y,Z: double;
var Hextant: double;
begin
{if saturation = 0 then the color is gray}
if HSB.Sat=0 then
	begin
	{Set gray value according to brightness}
	Result.rgbtRed:=trunc(HSB.Bright*255);
	Result.rgbtGreen:=trunc(HSB.Bright*255);
	Result.rgbtBlue:=trunc(HSB.Bright*255);
	exit;
	end;
{Calculate Hue Hextant}
Hextant:=HSB.Hue/60;
HexIndex:=floor(Hextant);
Fraction:=Hextant-HexIndex;

{Calculate coor vector}
X:=HSB.Bright * (1-HSB.Sat);
Y:=HSB.Bright * (1-HSB.Sat*Fraction);
Z:=HSB.Bright * (1-HSB.Sat*(1-Fraction));
case HexIndex of
 0:
	begin
	{Handle Red - Yellow Hextant}
	Result.rgbtRed := trunc(HSB.Bright*255);
	Result.rgbtGreen := trunc(Z*255);
	Result.rgbtBlue := trunc(X*255);
	end;
 1:
	begin
	{Handle Yellow - Green Hextant}
	Result.rgbtRed := trunc(Y*255);
	Result.rgbtGreen := trunc(HSB.Bright*255);
	Result.rgbtBlue := trunc(X*255);
	end;
 2:
	begin
	{Handle Green - Cyan Hextant}
	Result.rgbtRed:= trunc(X*255);
	Result.rgbtGreen := trunc(HSB.Bright*255);
	Result.rgbtBlue := trunc(Z*255);
	end;
 3:
	begin
	{Handle Cyan - Blue Hextant}
	Result.rgbtRed := trunc(X*255);
	Result.rgbtGreen := trunc(Y*255);
	Result.rgbtBlue := trunc(HSB.Bright*255);
	end;
 4:
	begin
	{Handle Blue - Magenta Hextant}
	Result.rgbtRed := trunc(Z*255);
	Result.rgbtGreen := trunc(X*255);
	Result.rgbtBlue := trunc(HSB.Bright*255);
	end;
 else
	begin
	{Handle Magenta - Red Hextant}
	Result.rgbtRed := trunc(HSB.Bright*255);
	Result.rgbtGreen := trunc(X*255);
	Result.rgbtBlue := trunc(Y*255);
	end;
 end;
end;


{------------------ Routines to manipulate HSL Colors models ------------------}


function HSLToRGB(HSL: THSLRecord): TRGBTriple;
{Convert Hue,Saturation and Luminesance to RGB}
{Hue = 0-360, Sat=0-1, Lume = 0-1}
var M1, M2: Single;

	function HueToColourValue(Hue: Single): Byte;
	var V: Double;
	begin
	Hue := Hue - Floor(Hue);

	if 6 * Hue < 1 then V := M1 + (M2 - M1) * Hue * 6
	else if 2 * Hue < 1 then V := M2
	else if 3 * Hue < 2 then V := M1 + (M2 - M1) * (2 / 3 - Hue) * 6
	else V := M1;
	Result := Round(255 * V);
	end;


begin
{Convert 0-360 to 0-1}
HSL.Hue:=HSL.Hue/360;
if HSL.Sat = 0 then
	begin
	Result.rgbtRed := Round(255 * HSL.Lume);
	Result.rgbtGreen := Result.rgbtRed;
	Result.rgbtBlue := Result.rgbtRed;
	end
else
	begin
	if HSL.Lume <= 0.5 then M2 := HSL.Lume * (1 + HSL.Sat)
	else M2 := HSL.Lume + HSL.Sat - HSL.Lume * HSL.Sat;
	M1 := 2 * HSL.Lume - M2;
	Result.rgbtRed := HueToColourValue(HSL.Hue + 1 / 3);
	Result.rgbtGreen := HueToColourValue(HSL.Hue);
	Result.rgbtBlue := HueToColourValue(HSL.Hue - 1 / 3)
	end;
end;




function RGBToHSL(RGB: TRGBTriple): THSLRecord;
{Convert RGB to Hue,Saturation and Luminesance }
{Hue = 0-360, Sat=0-1, Lume = 0-1}
var R, G, B, D, Cmax, Cmin: Single;
begin
R := RGB.rgbtRed / 255;
G := RGB.rgbtGreen / 255;
B := RGB.rgbtBlue / 255;
Cmax := Max(R, Max(G, B));
Cmin := Min(R, Min(G, B));
Result.Lume := (Cmax + Cmin) / 2;

if Cmax = Cmin then
	begin
	Result.Hue := 0;
	Result.Sat := 0
	end
else
	begin
	D := Cmax - Cmin;
	if Result.Lume < 0.5 then Result.Sat := D / (Cmax + Cmin)
	else Result.Sat := D / (2 - Cmax - Cmin);
	if R = Cmax then Result.Hue := (G - B) / D
	else if G = Cmax then Result.Hue := 2 + (B - R) /D
	else Result.Hue := 4 + (R - G) / D;
	Result.Hue := Result.Hue / 6;
	if Result.Hue < 0 then Result.Hue := Result.Hue + 1;
	Result.Hue := Result.Hue * 360;
	end;
end;



function MakeHSL(Hue,Sat,Lume: single): THSLRecord;
begin
Result.Hue:=Hue;
Result.Sat:=Sat;
Result.Lume:=Lume;
end;


{------------------------------------------------------------------------------}


procedure AdjustHue(Source,Destination: TBitmap; Offset: double);
{Adjusts hue values by Offset }
{Ofset should be in the range of -180 to +180}
var X, Y: Integer;
var Scan1,Scan2: pRGBTripleArray;
var HSB: THSBRecord;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		HSB:=RGBToHSB(Scan1[X]);
		{Offset Hue and normalize to 0-360}
		HSB.Hue:=HSB.Hue + Offset;
		while HSB.Hue>360 do HSB.Hue:=HSB.Hue -360;
		while HSB.Hue<0 do HSB.Hue:=HSB.Hue +360;
		Scan2[X]:=HSBtoRGB(HSB);
		end;
	end;
end;




procedure AdjustHue(BM: TBitmap; Hue: double);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
AdjustHue(Tmp,BM,Hue);
finally TMP.Free; end;
end;



procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double);
{Brightness, Constrast and Gamma should be between 0 and 1 with 0.5 representing no change}
var X, Y: Integer;
var Scale: double;
var Scan1,Scan2: pRGBTripleArray;
var RGB: TRGBTriple;
var AdjustArray: array [0..255] of integer;
var I: integer;
var Tmp: double;
const Gray = 128;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;

{Change Brightness/Contrast ranges from 0 - 1 to 0.1 to 10 }
Brightness:=Power(10,(Brightness - 0.5)*2);
Contrast:=Power(10,(Contrast - 0.5)*2);

{Change Gamma range from 0 - 1 to 0.1 to 10 }
Gamma:=Power(10,(Gamma - 0.50) * 2);
if Gamma<=0 then Gamma:=100 else Gamma:=1/Gamma;

{Used to scale gamma values back to 0 to 255 range}
Scale:=256/power(256,Gamma);

{Precalculate all 256 possible combined Gamma, Constrast and Brightness}
{Gamma function f(x) = x ^ (1 / gamma)}
for I:=0 to 255 do
	begin
	Tmp:=Power(I,Gamma) * Scale;
	Tmp:=((Tmp - Gray) * Contrast) + Gray;
	AdjustArray[I]:=trunc(EnsureRange(Tmp * Brightness,0,255));
	end;

for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		RGB:=Scan1[X];
		{Adjust Gamma, Contrast and Brightness}
		RGB.rgbtRed := AdjustArray[RGB.rgbtRed];
		RGB.rgbtGreen := AdjustArray[RGB.rgbtGreen];
		RGB.rgbtBlue := AdjustArray[RGB.rgbtBlue];
		Scan2[X]:=RGB;
		end;
	end;
end;



procedure AdjustBrightnessContrastGamma(BM: TBitmap; Brightness,Contrast,Gamma: double);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
AdjustBrightnessContrastGamma(Tmp,BM,Brightness,Contrast,Gamma);
finally TMP.Free; end;
end;



procedure AdjustBrightnessContrastGamma(Source,Destination: TBitmap; Brightness,Contrast,Gamma: double; Rect: TRect);
var BM: TBitmap;
begin
if Rect.Left>Rect.Right then SwapIntegers(Rect.Left,Rect.Right);
if Rect.Top>Rect.Bottom then SwapIntegers(Rect.Top,Rect.Bottom);
Rect.Left:=EnsureRange(Rect.Left,0,Source.Width-1);
Rect.Right:=EnsureRange(Rect.Right,0,Source.Width-1);
Rect.Top:=EnsureRange(Rect.Top,0,Source.Height-1);
Rect.Bottom:=EnsureRange(Rect.Bottom,0,Source.Height-1);
BM:=TBitmap.Create;
try
BM.Width:=Rect.Right-Rect.Left;
BM.Height:=Rect.Bottom-Rect.Top;
BM.Canvas.CopyRect(BM.Canvas.ClipRect,Source.Canvas,Rect);
AdjustBrightnessContrastGamma(BM, Brightness,Contrast,Gamma);
Destination.Canvas.CopyRect(Rect,BM.Canvas,BM.Canvas.ClipRect);

finally BM.Free; end;
end;


procedure AdjustBrightnessContrastGamma(BM: TBitmap; Brightness,Contrast,Gamma: double; Rect: TRect);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
AdjustBrightnessContrastGamma(Tmp,BM,Brightness,Contrast,Gamma,Rect);
finally TMP.Free; end;
end;



procedure AdjustSaturation(Source,Destination: TBitmap; Offset: double);
{Adjusts saturation values by Offset }
{Ofset should be in the range of -1 to +1}
var X, Y: Integer;
var Scan1,Scan2: pRGBTripleArray;
var HSB: THSBRecord;
begin
Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		HSB:=RGBToHSB(Scan1[X]);
		{Offset saturation and normalize 0 - 1}
		HSB.Sat:=HSB.Sat + Offset;
		if HSB.Sat>1 then HSB.Sat:=1;
		if HSB.Sat<0 then HSB.Sat:=0;
		Scan2[X]:=HSBtoRGB(HSB);
		end;
	end;
end;



procedure AdjustSaturation(BM: TBitmap; Sat: double);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
Tmp.Assign(BM);
AdjustSaturation(Tmp,BM,Sat);
finally TMP.Free; end;
end;


procedure DoWinResize(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Mode: integer);
{Use windows StretchBlt to resize image}
var Width,Height: integer;
begin
SetStretchBltMode(Canvas.Handle, Mode);
Width:= DestRect.Right  - DestRect.Left;
Height:= DestRect.Bottom - DestRect.Top;
StretchBlt(Canvas.Handle,DestRect.Left,DestRect.Top,Width,Height, Bitmap.Canvas.Handle,0,0, Bitmap.Width, Bitmap.Height, Canvas.CopyMode);
end;


procedure OversampleBitmap(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap);
{Use oversampling to smooth image}
begin
DoWinResize(Canvas,DestRect,Bitmap,HALFTONE);
end;



procedure ResizeBitmap(Source,Destination: TBitmap; Percent: double; OverSample: boolean);
{Resize image using Window API for faster oversampling}
var R: TRect;
begin
R.Left:=0;
R.Top:=0;
R.Right:=Trunc(Source.Width * Percent);
R.Bottom:=Trunc(Source.Height * Percent);
Destination.Width:=R.Right;
Destination.Height:=R.Bottom;
if Oversample then OverSampleBitmap(Destination.Canvas,R,Source)
else Destination.Canvas.StretchDraw(R,Source);
end;



procedure ResizeBitmap(BM: TBitmap; Percent: double; OverSample: boolean);
var BM2: TBitmap;
begin
BM2:=TBitmap.Create;
try
ResizeBitmap(BM,BM2,Percent,OverSample);
BM.Assign(BM2);
finally BM2.Free; end;
end;



procedure ResizeBitmap(Source,Destination: TBitmap; Size: TPoint; OverSample: boolean);
{Resize image using Window API for faster oversampling}
var R: TRect;
begin
R.Left:=0;
R.Top:=0;
R.Right:=Size.X;
R.Bottom:=Size.Y;
Destination.Width:=R.Right;
Destination.Height:=R.Bottom;
if Oversample then OverSampleBitmap(Destination.Canvas,R,Source)
else Destination.Canvas.StretchDraw(R,Source);
end;

{---------------- Alpha Blending Routines -------------------------------------}




procedure SetAlphaByColor(BM: TBitmap; C: TColor; Alpha: byte; Reset: boolean);
{Set each pixel to opaque or transparent based on color}
{Any color that is the same as C will be set tranparent according to the Alpha value}
{If Reset is true, the non matching colors are set to opaque}
{If Reset is false, the non matching colors are ignored }
{This allows different colors to be set to different levels of transparency}
{Used with the AlphaBlendBitmaps routine and per pixel mode}
var X,Y: integer;
var SL: PRGBQuadArray;
var A: integer;
var CT,T: TRGBTriple;
begin
BM.PixelFormat:=pf32Bit;
CT:=ColorToTriple(C);
{make the compiler happy}
A:=0;
for Y:=0 to BM.Height-1 do
	begin
	SL:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
		begin
		T:=PRGBTriple(@SL[X])^;
		if EqualRGBTriple(T,CT) then A:=Alpha
		else if Reset then A:=255 else continue;
		SL[X].rgbReserved:=A;
		SL[X].rgbRed:=MulDiv(SL[X].rgbRed,A,255);
		SL[X].rgbGreen:=MulDiv(SL[X].rgbGreen,A,255);
		SL[X].rgbBlue:=MulDiv(SL[X].rgbBlue,A,255);
		end;
	end;
end;


procedure SetAlphaByColor(BM: TBitmap; C: TColor);
begin
SetAlphaByColor(BM,C,0,True);
end;


function GetPixelBrightness(R,G,B: byte): integer;
{Get perceived brightness of a pixel}
var V: double;
begin
V:=sqrt((0.241*R*R) + (0.691*G*G) + (0.068*B*B));
Result:=trunc(V);
end;



procedure SetAlphaByLightness(BM: TBitmap; DoLightness: boolean; Lightness: byte);
{Set transparency based on Lightness of pixel}
{Pixels that are as light or lighter will be set transparent}
{Used with the AlphaBlendBitmaps routine}
var X,Y: integer;
var SL: PRGBQuadArray;
var Alpha: integer;
var L: byte;
begin
BM.PixelFormat:=pf32Bit;
for Y:=0 to BM.Height-1 do
	begin
	SL:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
		begin
                L:=GetPixelBrightness(SL[X].rgbRed,SL[X].rgbGreen,SL[X].rgbBlue);
		if DoLightness and (L>=Lightness) then Alpha:=0 else Alpha:=255;
		SL[X].rgbReserved:=Alpha;
		SL[X].rgbRed:=MulDiv(SL[X].rgbRed,Alpha,255);
		SL[X].rgbGreen:=MulDiv(SL[X].rgbGreen,Alpha,255);
		SL[X].rgbBlue:=MulDiv(SL[X].rgbBlue,Alpha,255);
		end;
	end;
end;


procedure AlphaBlendBitmaps(SourceCanvas,DestCanvas: TCanvas; SourceRect,DestRect: TRect; Alpha: byte; PerPixel: boolean);
{Do alpha blending of between specified canvas}
var DW,DH,SW,SH: integer;
var Blend: TBlendFunction;
begin
DW:= DestRect.Right  - DestRect.Left;
DH:= DestRect.Bottom - DestRect.Top;

SW:= SourceRect.Right  - SourceRect.Left;
sH:= SourceRect.Bottom - SourceRect.Top;

{Note: AC_SRC_OVER only used with 32bit, per pixel alpha}
if PerPixel then Blend.AlphaFormat:=AC_SRC_ALPHA else Blend.AlphaFormat:=0;
Blend.BlendOp:=AC_SRC_OVER;
Blend.BlendFlags:=0;
Blend.SourceConstantAlpha := Alpha;

AlphaBlend(DestCanvas.Handle,DestRect.Left,DestRect.Top,DW,DH,SourceCanvas.Handle,SourceRect.Left,SourceRect.Top, SW, SH, Blend);
end;




procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte; PerPixel: boolean);
{Do alpha blending of Bitmap with specified canvas}
var Width,Height: integer;
var Blend: TBlendFunction;
begin
Width:= DestRect.Right  - DestRect.Left;
Height:= DestRect.Bottom - DestRect.Top;
{Note: AC_SRC_OVER only used with 32bit, per pixel alpha}
if PerPixel then Blend.AlphaFormat:=AC_SRC_ALPHA
else Blend.AlphaFormat:=0;
Blend.BlendOp:=AC_SRC_OVER;
Blend.BlendFlags:=0;
Blend.SourceConstantAlpha := Alpha;

AlphaBlend(Canvas.Handle,DestRect.Left,DestRect.Top,Width,Height,Bitmap.Canvas.Handle,0,0, Bitmap.Width, Bitmap.Height, Blend);
end;





procedure AlphaBlendBitmaps(Canvas: TCanvas; DestRect: TRect;  Bitmap:  TBitmap; Alpha: byte);
begin
AlphaBlendBitmaps(Canvas, DestRect,  Bitmap, Alpha, False);
end;




procedure AlphaBlendCanvas(DestCanvas: TCanvas; DestRect: TRect; SourceCanvas: TCanvas; SourceRect: TRect; Alpha: byte; PerPixel: boolean);
{Do alpha blending of specified canvases}
var DestSize, SourceSize: TPoint;
var Blend: TBlendFunction;
begin
DestSize:=IntVectorSubtract(DestRect.BottomRight,DestRect.TopLeft);
SourceSize:=IntVectorSubtract(SourceRect.BottomRight,SourceRect.TopLeft);
{Note: AC_SRC_OVER only used with 32bit, per pixel alpha}
if PerPixel then Blend.AlphaFormat:=AC_SRC_ALPHA
else Blend.AlphaFormat:=0;
Blend.BlendOp:=AC_SRC_OVER;
Blend.BlendFlags:=0;
Blend.SourceConstantAlpha := Alpha;

Windows.AlphaBlend(DestCanvas.Handle,DestRect.Left,DestRect.Top,DestSize.X,DestSize.Y,
		   SourceCanvas.Handle,SourceRect.Left,SourceRect.Top,SourceSize.X,SourceSize.Y, Blend);
end;


{------------------ Routines to Handle And Masks ------------------------------}




procedure MakeMask(var Mask: TBitmap; Color: TColor);
{Generate a monochrome mask }
{ All colors match that match "Color" are white eveything else black}
{Solves the problem of the Brush Color interferings with white color}
var BM: TBitmap;
begin
Mask.Mask(Color);
BM:=TBitmap.Create;
BM.Width:=Mask.Width;
BM.Height:=Mask.Height;
BM.Canvas.Brush.Color:=clWhite;
BM.Canvas.Draw(0,0,Mask);
Mask.Assign(BM);
BM.Free;
end;



{-------------------Do Image Clipping -----------------------------------------}



function IsBitmapBlank(BM: TBitmap): boolean;
{Gheck if bitmap is all white or blank}
var X,Y: integer;
var Scan: pRGBTripleArray;
begin
Result:=False;
BM.PixelFormat:=pf24Bit;
for Y:=0 to BM.Height-1 do
	begin
	Scan:=BM.ScanLine[Y];
	for X:=0 to BM.Width-1 do
	 if (Scan[X].rgbtRed<>$FF) or
	    (Scan[X].rgbtGreen<>$FF) or
	    (Scan[X].rgbtBlue<>$FF) then exit;
	end;
Result:=True;
end;

{-------------------Do Image Clipping -----------------------------------------}


function ClipBitmap(BM: TBitmap; R: TRect): boolean;
{Clip bitmap to rectangle}
var R2: TRect;
var T: integer;
begin
BM.Dormant;
BM.FreeImage;
Result:=False;
if R.Left=R.Right then exit;
if R.Top=R.Bottom then exit;

if R.Left>R.Right then begin T:=R.Left; R.Left:=R.Right; R.Right:=T; end;
if R.Top>R.Bottom then begin T:=R.Top; R.Top:=R.Bottom; R.Bottom:=T; end;

if R.Right>BM.Width then R.Right:=BM.Width;
if R.Left<0 then R.Left:=0;
if R.Bottom>BM.Height then R.Bottom:=BM.Height;
if R.Top<0 then R.Top:=0;

R2:=R;
OffsetRect(R2,-R.Left,-R.Top);
try
BM.Canvas.CopyRect(R2,BM.Canvas,R);
BM.Width:=R.Right-R.Left;
BM.Height:=R.Bottom-R.Top;
Result:=True;
except end;
end;



{----------- Routines to Offset bitmap wrapping around the image --------------}



procedure WrapImageHorizontal(BM: TBitmap; Delta: integer);
{Wrap bitmap in the horizontal axis according to the Delta}
var TBM: TBitmap;
var BandFrom,BandTo,BlockFrom,BlockTo: TRect;
begin
TBM:=TBitmap.Create;
try
TBM.Height:=BM.Height;
TBM.Width:=abs(Delta);

BandFrom:=Rect(0,0,abs(Delta),TBM.Height);
BandTo:=BandFrom;
OffsetRect(BandTo,BM.Width-abs(Delta),0);
BlockFrom:=Rect(abs(Delta),0,BM.Width,BM.Height);
BlockTo:=BlockFrom;
OffsetRect(BlockTo,-abs(Delta),0);
if Delta>=0 then
	begin
	SwapRectangles(BandFrom,BandTo);
	SwapRectangles(BlockFrom,BlockTo);
	end;
TBM.Canvas.CopyRect(Rect(0,0,TBM.Width,TBM.Height),BM.Canvas,BandFrom);
BM.Canvas.CopyRect(BlockTo,BM.Canvas,BlockFrom);
BM.Canvas.CopyRect(BandTo,TBM.Canvas,Rect(0,0,TBM.Width,TBM.Height));
finally TBM.Free; end;
end;




procedure WrapImageVertical(BM: TBitmap; Delta: integer);
{Wrap bitmap in the Vertical axis according to the Delta}
var TBM: TBitmap;
var BandFrom,BandTo,BlockFrom,BlockTo: TRect;
begin
TBM:=TBitmap.Create;
try
TBM.Height:=abs(Delta);
TBM.Width:=BM.Width;

BandFrom:=Rect(0,0,TBM.Width,abs(Delta));
BandTo:=BandFrom;
OffsetRect(BandTo,0,BM.Height-abs(Delta));

BlockFrom:=Rect(0,abs(Delta),BM.Width,BM.Height);
BlockTo:=BlockFrom;
OffsetRect(BlockTo,0,-abs(Delta));
if Delta>=0 then
	begin
	SwapRectangles(BandFrom,BandTo);
	SwapRectangles(BlockFrom,BlockTo);
	end;
TBM.Canvas.CopyRect(Rect(0,0,TBM.Width,TBM.Height),BM.Canvas,BandFrom);
BM.Canvas.CopyRect(BlockTo,BM.Canvas,BlockFrom);
BM.Canvas.CopyRect(BandTo,TBM.Canvas,Rect(0,0,TBM.Width,TBM.Height));
finally TBM.Free; end;
end;


procedure WrapImage(BM: TBitmap; X,Y: integer);
{Wrap bitmap in the X,Y axis according to the Delta}
begin
WrapImageHorizontal(BM,X);
WrapImageVertical(BM,Y);
end;


var OldAngle,RemX,RemY: double;	{Remainder of angled wrap}

procedure WrapImageAngle(BM: TBitmap; Angle,Len: double);
{Wrap bitmap along specified angle}
var SinTheta,CosTheta: extended;
var X,Y: double;
begin
Angle:=((360 -Angle)+180) * Degree2Radian;
if Angle<>OldAngle then
	begin
	RemX:=0; RemY:=0;
	OldAngle:=Angle;
	end;
SinCos(Angle,SinTheta,CosTheta);
X:=SinTheta * Len + RemX;
Y:=CosTheta * Len + RemY;
WrapImage(BM, trunc(X),Trunc(Y));
RemX:=Frac(X); RemY:=Frac(Y);
end;



function InterpolateColors(C1,C2: TColor; Width, Pos: double): TColor;
{Find the interpolated color between C1 and C2 at Pos}
var CV1,CV2,Step,R: T3DVector;
begin
Result:=C1;
if (Pos<0) or (Pos>Width) then exit;
CV1:=ColorToVector(C1);
CV2:=ColorToVector(C2);
Step:=ScalarDivide3D(VectorSubtract3D(CV2,CV1),Width);
R:=VectorAdd3D(ScalarProduct3D(Step,Pos),CV1);
Result:=VectorToColor(R);
end;



procedure DrawGradientLine(Canvas: TCanvas; P1,P2: TPoint; C1,C2: TColor);
{Draw gradient line between P1 and P2 with colors between C1 and C2}
var Delta: TPoint;			{ Delta X, Y }
var Increment: TPoint;			{ Plotting Increments }
var DoDiagonal: integer;		{ Decision Variable (move Straight Or Diagonally) }
var StraightInc, DiagonalInc: integer;	{ East, Northeast (straight Or Diagonal) }
var StepAlongY: boolean;		{ Flag: Step Along Y Direction }
var I: Integer;
begin
{Calculate deltas}
Delta:=IntVectorSubtract(P2,P1);

{ If Positive Then Increment Else Absolute Value And Decrement }
if Delta.X >= 0 then Increment.X:=1 else begin Delta.X:= -Delta.X; Increment.X:=-1; end;
if Delta.Y >= 0 then Increment.Y:=1 else begin Delta.Y:= -Delta.Y; Increment.Y:=-1; end;

{Work out step direction}
StepAlongY:=Delta.X < Delta.Y;
if StepAlongY then IntSwapComponents(Delta);

{ Set Up Decision Variable Stuff }
DoDiagonal:=2 * Delta.Y - Delta.X;
StraightInc:=2 * Delta.Y;
DiagonalInc:=2 * (Delta.Y-Delta.X);

{ Plot Initial Point }
Canvas.Pixels[P1.X, P1.Y]:=C1;
for I:= 1 to  Delta.X do
	begin
	if DoDiagonal < 0 then
		begin
		{ Move Straight }
		DoDiagonal:= DoDiagonal + StraightInc;
		if StepAlongY then P1.Y:= P1.Y + Increment.Y else P1.X:= P1.X + Increment.X;
		end
	 else
		begin
		{ Move Diagonally }
		DoDiagonal:= DoDiagonal + DiagonalInc;
		P1:=IntVectorAdd(P1,Increment);
		end;
	Canvas.Pixels[P1.X, P1.Y]:=InterpolateColors(C1,C2,Delta.X,I);
	end;
end;	{ Drawline }



procedure HorizGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor) ;
var X: integer;
var DR,DG,DB,DX:  extended;
var R1,R2,G1,G2,B1,B2: byte;
var R,G,B: Byte;
var Cnt: integer;
begin
ColorToRGBBytes(FromColor,R1,G1,B1);
ColorToRGBBytes(ToColor,R2,G2,B2);
DX:=Rect.Right-Rect.Left;
DR:=(R2-R1) / DX;
DG:=(G2-G1) / DX;
DB:=(B2-B1) / DX;

cnt:=0;
for X:=Rect.Left to Rect.Right-1 do
	begin
	R:=R1+Ceil(DR*Cnt);
	G:=G1+Ceil(DG*Cnt);
	B:=B1+Ceil(DB*cnt);

	Canvas.Pen.Color:=RGB(R,G,B);
	Canvas.MoveTo(X,Rect.Top);
	Canvas.LineTo(X,Rect.Bottom);
	inc(Cnt);
	end;
end;


procedure VertGradientRect(Canvas: TCanvas; Rect: TRect; FromColor, ToColor: TColor);
var Y: integer;
var DR,DG,DB,DY: extended;
var R1,R2,G1,G2,B1,B2: byte;
var R,G,B: Byte;
var Cnt: Integer;
begin
ColorToRGBBytes(FromColor,R1,G1,B1);
ColorToRGBBytes(ToColor,R2,G2,B2);
DY:=Rect.Bottom-Rect.Top;
DR:=(R2-R1) / DY;
DG:=(G2-G1) / DY;
DB:=(B2-B1) / DY;

Cnt:=0;
for Y:=Rect.Top to Rect.Bottom-1 do
	begin
	R:=R1+Ceil(DR*Cnt);
	G:=G1+Ceil(DG*Cnt);
	B:=B1+Ceil(DB*Cnt);

	Canvas.Pen.Color:=RGB(R,G,B);
	Canvas.MoveTo(Rect.Left,Y);
	Canvas.LineTo(Rect.Right,Y);
	Inc(Cnt);
	end;
end;



{-------------------- Routines to handle cursor loading ----------------------}




function IconToBitmap(Icon: TIcon): TBitmap;
begin
Result:=TBitmap.Create;
Result.Width:=Icon.Width;
Result.Height:=ICon.Height;
Result.Canvas.Draw(0,0,Icon);
end;



function LoadCursorToIcon(Filename: string): TIcon;
{Load and convert ".CUR" file to TIcon}
begin
Result:=TIcon.Create;
Result.Handle:=LoadImage(0,PChar(FileName),IMAGE_CURSOR,0,0,LR_DEFAULTSIZE or LR_LOADFROMFILE);
end;




function LoadCursorToBitmap(Filename: string): TBitmap;
{Load and convert ".CUR" file to TBitmap}
{Note: transparent portion of Cursor become opaque}
var Icon: TIcon;
begin
Result:=TBitmap.Create;
Icon:=LoadCursorToIcon(Filename);
try
Icon.Transparent;
Result.Width:=Icon.Width;
Result.Height:=Icon.Height;
Result.Canvas.Draw(0,0,Icon);
finally Icon.Free; end;
end;



procedure ClearBitmap(BM: TBitmap; Block: TRect; Color: TColor);
begin
BM.Canvas.Brush.Color:=Color;
BM.Canvas.Brush.Style:=bsSolid;
BM.Canvas.Pen.Mode:=pmCopy;
BM.Canvas.Pen.Style:=psSolid;
BM.Canvas.Pen.Color:=Color;
BM.Canvas.Rectangle(Block);
end;


procedure ClearBitmap(BM: TBitmap; Color: TColor);
begin
ClearBitmap(BM,Rect(0,0,BM.Width,BM.Height),Color);
end;



{------------------ TImage Support routines -----------------------------------}


procedure NormalizeImageSize(Image: TImage);
{Force Images display size to be the same as the underlying bitmap}
var OldAuto,OldStretch: boolean;
begin
OldAuto:=Image.AutoSize;
OldStretch:=Image.Stretch;
Image.Stretch:=False;
Image.AutoSize:=True;
Image.Width:=Image.Picture.Bitmap.Width;
Image.Height:=Image.Picture.Bitmap.Height;
Image.Invalidate;
Image.AutoSize:=OldAuto;
Image.Stretch:=OldStretch;
end;


procedure ClearCanvas(Canvas: TCanvas; Rect: TRect; Color: TColor);
begin
Canvas.Brush.Color:=Color;
Canvas.Brush.Style:=bsSolid;
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=Color;
Canvas.Rectangle(Rect);
end;



procedure ClearImage(Image: TImage; Color: TColor);
var R: TRect;
begin
R:=Rect(0,0,Image.Picture.Bitmap.Width,Image.Picture.Bitmap.Height);
ClearCanvas(Image.Canvas,R,Color);
Image.Invalidate;
end;



function GetPixelFormatStr(BM: TBitmap): string;
begin
case BM.PixelFormat of
 pfDevice: Result:= 'Device';
 pf1bit: Result:= '1-bit';
 pf4bit: Result:= '4-bit';
 pf8bit: Result:= '8-bit';
 pf15bit: Result:= '15-bit';
 pf16bit: Result:= '16-bit';
 pf24bit: Result:= '24-bit';
 pf32bit: Result:= '32-bit';
 pfCustom: Result:= 'Custom';
 else Result:= 'Unknown';
 end;
end;








procedure FadeInBitmap(Image: TImage; BM: TBitmap; Millisec: integer);
{Fade in specified Bitmap using Alpha blending}
var I: integer;
var Step: integer;
var Alpha: double;
var A: double;

	function GetCurve(X: double): double;
	begin
	Result:=X/(1+A*(1-X));
	end;

begin
Step:=Millisec div 255;
A:=50;
for I:=0 to 255 do
	begin
	Alpha:=GetCurve(I/255);
	AlphaBlendBitmaps(Image.Canvas,Image.ClientRect,BM,round(Alpha*255));
	Image.Invalidate;
	DelayProcessMessages(Step);
	end;
end;



procedure FadeToColor(Image: TImage; C: TColor; Millisec: integer);
{Fade to specified Color using Alpha blending}
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.Width:=Image.Width;
BM.Height:=Image.Height;
ClearBitmap(BM,C);
FadeInBitmap(Image,BM,Millisec);
finally BM.Free; end;
end;



function CreateDisabledBitmap(Glyph: TBitmap): TBitmap;
{Created grayed out disabled bitmap from ordinary image}
const ROP_DSPDxax = $00E20746;
var MonoBmp: TBitmap;
var Col: TColor;
var tr, tg, tb: Byte;
var P, P1, P2, P3: PRGBTripleArray;
var R: PRGBTriple;
var C: array[0..3] of PRGBTriple;
var x, y, i: Integer;
var IsEdge: Boolean;
begin
Result:=TBitmap.Create;
if Assigned(Glyph) then
	begin
	Glyph.PixelFormat := pf24bit;
	Result.Assign(Glyph);
	Result.PixelFormat := pf24bit;
	Result.Canvas.Brush.Color := Glyph.TransparentColor;
	Result.Canvas.FillRect(Rect(0, 0, Result.Width, Result.Height));

	Col := ColorToRGB(Glyph.TransparentColor);
	tr := GetRValue(Col);
	tg := GetGValue(Col);
	tb := GetBValue(Col);
	for y := 0 to Result.Height - 1 do
		begin
		P := Result.ScanLine[y];
		P1 := Glyph.ScanLine[Max(y - 1, 0)];
		P2 := Glyph.ScanLine[y];
		P3 := Glyph.ScanLine[Min(y + 1, Result.Height - 1)];
		for x := 0 to Result.Width - 1 do
			begin
			R := @P2[x];
			if (R.rgbtRed <> tr) or (R.rgbtGreen <> tg) or (R.rgbtBlue <> tb) then
				begin
				IsEdge := (R.rgbtRed + R.rgbtGreen + R.rgbtBlue) < 3 * $80;
				if not IsEdge then
					begin
					C[0] := @P1[x];
					C[1] := @P3[x];
					C[2] := @P2[Max(x - 1, 0)];
					C[3] := @P2[Min(x + 1, Result.Width - 1)];
					for i := 0 to 3 do
					 if (C[i].rgbtRed = tr) and (C[i].rgbtGreen = tg) and (C[i].rgbtBlue = tb) then
					 	begin
					 	IsEdge := True;
					 	Break;
					 	end;
					 end;
				if IsEdge then
					begin
					R := @P[x];
					R.rgbtBlue := 0;
					R.rgbtGreen := 0;
					R.rgbtRed := 0;
					end
				end;
			end;
		end;
	MonoBmp := TBitmap.Create;
	try
	with MonoBmp do
	 begin
	 Assign(Result);
	 HandleType := bmDDB;
	 Canvas.Brush.Color := clBlack;
	 Width := Result.Width;
	 if Monochrome then
	 	begin
	 	Canvas.Font.Color := clWhite;
	 	Monochrome := False;
	 	Canvas.Brush.Color := clWhite;
	 	end;
	 Monochrome := True;
	 end;
with Result.Canvas do
	begin
	Brush.Color := clBtnFace;
	FillRect(Rect(0, 0, Result.Width, Result.Height));
	Brush.Color := clBtnHighlight;
	SetTextColor(Handle, clBlack);
	SetBkColor(Handle, clWhite);
	BitBlt(Handle, 1, 1, Result.Width, Result.Height,
	MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
	Brush.Color := clBtnShadow;
	SetTextColor(Handle, clBlack);
	SetBkColor(Handle, clWhite);
	BitBlt(Handle, 0, 0, Result.Width, Result.Height, MonoBmp.Canvas.Handle, 0, 0, ROP_DSPDxax);
	end;
 finally MonoBmp.Free; end;
 end;
end;




procedure BitmapToGrayscaleBuffer(BM: TBitmap; AllocMem: boolean; var Buffer: PByteArray);
{Convert Bitmap to grayscale and put in byte buffer}
var X: Integer;
var Y: Integer;
var Scan: ^TRGBTriple;
begin
if AllocMem then GetMem(Buffer,BM.Width*BM.Height);
BM.PixelFormat:=pf24Bit;
for Y := 0 to BM.Height - 1 do
	begin
	Scan:= BM.ScanLine[Y];
	for X := 0 to BM.Width - 1 do
		begin
		Buffer^[Y*BM.Width+X]:=Round((0.299 * Scan.rgbtRed) + (0.587 * Scan.rgbtGreen) + (0.114 * Scan.rgbtBlue));
		Inc(Scan);
		end;
	end;
end;




procedure RGBBitmapToGray(const Bmp: TBitmap);
  {From: Pascal Enz, pascal.enz@datacomm.ch }
var X, Y, Gray: Integer;
var Scan: PRGBTripleArray;
begin
Bmp.PixelFormat := pf24Bit;
for y := 0 to Bmp.Height - 1 do
	begin
	Scan:= Bmp.ScanLine[Y];
	for x := 0 to Bmp.Width - 1 do
		begin
		Gray:=Round((0.299 * Scan[X].rgbtRed) + (0.587 * Scan[X].rgbtGreen) + (0.114 * Scan[X].rgbtBlue));
		Scan[X].rgbtRed := Gray;
		Scan[X].rgbtGreen := Gray;
		Scan[X].rgbtBlue := Gray;
		end;
	end;
end;



procedure RGBBitmapToBytes(const Bmp: TBitmap; var BA: TDynByteArray);
{Convert RGB bitmap to array of gray scale bytes}
var X, Y, Inx: Integer;
var Scan: PRGBTripleArray;
begin
Bmp.PixelFormat := pf24Bit;
SetLength(BA, Bmp.Width * Bmp.Height);
for Y := 0 to Bmp.Height - 1 do
	begin
	Scan:= Bmp.ScanLine[Y];
	for X := 0 to Bmp.Width - 1 do
		begin
		Inx:=Y * Bmp.Width + X;
		BA[Inx]:=Round((0.299 * Scan[X].rgbtRed) + (0.587 * Scan[X].rgbtGreen) + (0.114 * Scan[X].rgbtBlue));
		end;
	end;
end;


procedure GetMBMPHistogram(const Bmp: TBitmap; var Hist: THistoArray; var HighValue: double);
{Get histogram of of values scaled between 0 and 1}
var X, Y,I,Inx: integer;
var Scan: PRGBTripleArray;
begin
SetLength(Hist,256);
for I:=0 to High(Hist) do Hist[I]:=0;
Bmp.PixelFormat := pf24Bit;
for y := 0 to Bmp.Height - 1 do
	begin
	Scan:= Bmp.ScanLine[Y];
	for x := 0 to Bmp.Width - 1 do
		begin
		Inx:=Round((0.299 * Scan[X].rgbtRed) + (0.587 * Scan[X].rgbtGreen) + (0.114 * Scan[X].rgbtBlue));
		Hist[Inx]:=Hist[Inx]+1;
		end;
	end;
HighValue:=1;
for I:=0 to High(Hist) do
 if Hist[I]>HighValue then HighValue:=Hist[I];
end;



function GetOtsuLevel(Histo: THistoArray): integer;
{Use Otsu algorith to get block/white threshold}
var Total, T,wB,wF: integer;
var Sum, sumB, varMax, mB,mF,varBetween : double;
begin
{ total number of pixels in the image }
Total:=0;
for T:=0 to High(Histo) do Total:=Total + trunc(Histo[T]);

sum := 0;
for T:=0 to High(Histo) do Sum := Sum + T * Histo[T];

sumB := 0;
wB := 0;

varMax := 0;
Result := 0;

for T:=0 to High(Histo) do
	begin
	wB := wB + trunc(Histo[T]);	{ Weight Background}
	if wB = 0 then continue;

	wF := total - wB;		{ Weight Foreground}
	if wF = 0 then break;

	sumB :=sumB + (t * Histo[t]);

	mB := sumB / wB;		{ Mean Background}
	mF := (sum - sumB) / wF;	{ Mean Foreground}

	{ Calculate Between Class Variance}
	varBetween := wB * wF * (mB - mF) * (mB - mF);

	{ Check if new maximum found}
	if (varBetween > varMax) then
		begin
		varMax := varBetween;
		Result := t;
		end;
	end;
end;




procedure RGBToBlackAndWhite(Source,Destination: TBitmap; Threshold: double);
{Convert RGB to 24-bit black and white  - Threshold = 0 to 1}
var X,Y: integer;
var Scan1,Scan2: pRGBTripleArray;
var Gray: double;
var Black,White: TRGBTriple;
begin
Black:=MakeRBGTriple(0,0,0);
White:=MakeRBGTriple(255,255,255);

Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		Gray:=(0.299 * Scan1[X].rgbtRed/255) + (0.587 * Scan1[X].rgbtGreen/255) + (0.114 * Scan1[X].rgbtBlue/255);
		if Gray>Threshold then Scan2[X]:=White
		else Scan2[X]:=Black;
		end;
	end;
end;




procedure WhiteThreshold(Source,Destination: TBitmap; Threshold: double);
{Convert all light pixels to pure white - Threshold = 0 to 1}
var X,Y: integer;
var Scan1,Scan2: pRGBTripleArray;
var Gray: double;
var White: TRGBTriple;
begin
White:=MakeRBGTriple(255,255,255);

Source.PixelFormat:=pf24bit;
Destination.Width:= Source.Width;
Destination.Height:= Source.Height;
Destination.PixelFormat:= pf24bit;
for Y :=0 to Source.Height-1 do
	begin
	Scan1:=Source.ScanLine[Y];
	Scan2:=Destination.Scanline[Y];
	for X := 0 to Source.Width-1 do
		begin
		Gray:=(0.299 * Scan1[X].rgbtRed/255) + (0.587 * Scan1[X].rgbtGreen/255) + (0.114 * Scan1[X].rgbtBlue/255);
		if Gray>Threshold then Scan2[X]:=White
		else Scan2[X]:=Scan1[X];
		end;
	end;
end;


{----------------------------------------------------------------------------}





procedure BinarizeImage(Src,Dest: TBitmap; Radius: integer; KFactor: double);
{Binarirze Image using NICK algorithm}
var X,Y: integer;
var SrcArray,DestArray: TRGBTripleRowArray;
var Black,White: TRGBTriple;

type TMapWindow = record
 Box: TRect;
 Count: integer;
 end;


	function GetNICKFactor(Win: TMapWindow; K: double): double;
	var Gray: byte;
	var Mean,Sum,Sum2: double;
	var X,Y: integer;
	begin
	Sum:=0;
	Sum2:=0;
	for Y:=Win.Box.Top to Win.Box.Bottom do
	 for X:=Win.Box.Left to Win.Box.Right do
	 	begin
	 	Gray:=RGBToGray(SrcArray[Y,X]);
	 	Sum:=Sum + Gray;
		Sum2:=Sum2 + Gray * Gray;
	 	end;
	Mean:=Sum/Win.COunt;
	Result:=Mean + K * sqrt((Sum2 - Mean*Mean)/Win.COunt);
	end;


	function GetWindow(X,Y, Radius: integer): TMapWindow;
	begin
	Result.Box:=Rect(X - Radius, Y-Radius, X+Radius, Y+Radius);

	if Result.Box.Left<0 then Result.Box.Left:=0;
	if Result.Box.Right>=Src.Width then Result.Box.Right:=Src.Width-1;

	if Result.Box.Top<0 then Result.Box.Top:=0;
	if Result.Box.Bottom>=Src.Height then Result.Box.Bottom:=Src.Height-1;
	Result.Count:=((Result.Box.Right-Result.Box.Left)+1) * ((Result.Box.Bottom-Result.Box.Top)+1);
	end;


	procedure HandlePixel(X,Y: integer);
	var Win: TMapWindow;
	var Gray: byte;
	var NF: double;
	begin
	Win:=GetWindow(X,Y,Radius);
	NF:=GetNICKFactor(Win,KFactor);
	Gray:=RGBToGray(SrcArray[Y,X]);
	if Gray>NF then DestArray[Y,X]:=White else DestArray[Y,X]:=Black;
	end;



begin
Src.PixelFormat:=pf24Bit;
Dest.PixelFormat:=pf24Bit;
Dest.Width:=Src.Width;
Dest.Height:=Src.Height;
Black:=MakeRBGTriple(0,0,0);
White:=MakeRBGTriple(255,255,255);


FillRGBRowArray(Src, SrcArray);
FillRGBRowArray(Dest, DestArray);

for Y:=0 to Src.Height-1 do
 for X:=0 to Src.Width-1 do
	begin
	HandlePixel(X,Y);
	end;
end;



{------------------------------------------------------------------------------}

function WidthToByteCount(Format: Graphics.TPixelFormat; Width: integer): integer;
{Get number of bytes per scanline}
begin
case Format of
 pf1bit: Result:=Width div 8;
 pf4bit: Result:=Width div 2;
 pf8bit: Result:=Width;
 pf15bit, pf16bit: Result:=Width * 2;
 pf24bit: Result:=Width * 3;
 pf32bit: Result:=Width * 4;
 else Result:=0;
 end;
end;



function GetByteCount(BM: TBitmap): integer;
{Get number of bytes per scanline}
begin
case BM.PixelFormat of
 pf1bit: Result:=BM.Width div 8;
 pf4bit: Result:=BM.Width div 2;
 pf8bit: Result:=BM.Width;
 pf15bit, pf16bit: Result:=BM.Width * 2;
 pf24bit: Result:=BM.Width * 3;
 pf32bit: Result:=BM.Width * 4;
 else Result:=0;
 end;
end;



function EqualBitmaps(BM1,BM2: TBitmap): boolean;
{Compare bitmaps and look for any differences}
var L1,L2: PByteArray;
var X,Y: integer;
begin
Result:=False;
if (BM1.Width<>BM2.Width) or (BM1.Height<>BM2.Height) then exit;
if BM1.PixelFormat<>BM2.PixelFormat then exit;
for Y:=0 to BM1.Height-1 do
	begin
	L1:=BM1.ScanLine[Y]; L2:=BM2.ScanLine[Y];
	for X:=0 to GetByteCount(BM1)-1 do
		if L1[X]<>L2[X] then exit;
	end;
Result:=True;
end;



function BitmapDifference(BM1,BM2: TBitmap): double;
{Compare bitmaps return percent different}
var L1,L2: PByteArray;
var X,Y: integer;
var MaxWidth,MaxHeight,MinWidth,MinHeight: integer;
var Count,ByteWidth: integer;
begin
Count:=0;
MaxWidth:=Max(BM1.Width,BM2.Width);
MaxHeight:=Max(BM1.Height,BM2.Height);
MinWidth:=Min(BM1.Width,BM2.Width);
MinHeight:=Min(BM1.Height,BM2.Height);
Count:=Count+(abs(MaxWidth-MinWidth) * MaxHeight);
Count:=Count+(abs(MaxHeight-MinHeight) * MaxWidth);
if BM1.Width<BM2.Width then ByteWidth:=WidthToByteCount(BM1.PixelFormat,BM1.Width)
else ByteWidth:=WidthToByteCount(BM2.PixelFormat,BM2.Width);

for Y:=0 to MinHeight-1 do
	begin
	L1:=BM1.ScanLine[Y]; L2:=BM2.ScanLine[Y];
	for X:=0 to ByteWidth-1 do
	 if L1[X] <> L2[X] then Inc(Count);
	end;
Result:=Count/(ByteWidth * MaxHeight);
end;








function GetBitsPerPixel(BM: TBitmap): integer;
begin
BM.HandleType:=bmDIB;
case BM.PixelFormat of
 pf1bit: Result:=1;
 pf4bit: Result:=4;
 pf8bit: Result:=8;
 pf15bit: Result:=15;
 pf16bit: Result:=16;
 pf24bit: Result:=24;
 pf32bit: Result:=32;
 else Result:=32;
 end;
end;



function GetBitmapSize(BM: TBitmap): int64;
var PS,CT,RS: integer;
begin
{Get header size}
Result:=SizeOf(TBitmapFileHeader);	//14
Inc(Result, SizeOf(TBitmapInfoHeader));		//40
{Get Color Table size}
case BM.PixelFormat of
 pf1bit: CT:=2 * 4;
 pf4bit: CT:=16 * 4;
 pf8bit: CT:=256 * 4;
 pf15bit,pf16bit,pf24bit,pf32bit: CT:=0;
 else CT:=0;
 end;
{Get size of all the pixels}
{Start with Row size}
RS:=((GetBitsPerPixel(BM) * BM.Width + 31) div 32) * 4;
PS:=RS * BM.Height;
{Add in Pixels and Color Table sizes}
Inc(Result,CT);
Inc(Result,PS);
end;


function GetBmpDPI(Bitmap: TBitmap): LongInt;
var Stream: TMemoryStream;
var Data: Word;
var A: Double;
begin
Result := 0;
Stream := TMemoryStream.Create;
try
Bitmap.SaveToStream(Stream);
Stream.Position := 38;
if Stream.Read(Data, 2) = 2 then
	begin
	A := Data;
	Result := Round(A / 39.370079);
	end;
finally Stream.Free; end;
end;


procedure SetBmpDPI(Bitmap: TBitmap; DPI: Integer);
var Stream: TMemoryStream;
var Data: Word;
begin
Stream := TMemoryStream.Create;
try
Bitmap.SaveToStream(Stream);
Data := Round(DPI * 39.370079);
Stream.Position := 38;
if Stream.Write(Data, 2) = 2 then
	begin
	Stream.Position := 42;
	if Stream.Write(Data, 2) = 2 then
		begin
		Stream.Position := 0;
		Bitmap.LoadFromStream(Stream);
		end
	else raise Exception.Create('Error writing to Stream. Data not written.');
	end
else raise Exception.Create('Error writing to Stream. Data not written.');
finally Stream.Free;  end;
end;


{------------------------------------------------------------------------------}

function GetJPGLength(Stream: TStream; Limit: int64): int64;
{Analyze JPG files and get its length}
var StartPos: int64;
var Size: word;


	function OverLimit: boolean;
	begin
	Result:=(Stream.Position-StartPos)>Limit;
	end;


	function GetMarker: word;
	{Markers can begin with multiple $FF's}
	var B: byte;
	begin
	Result:=0;
	B:=0;
	while true do
		begin
		B:=GetStreamByte(Stream);
		if B<>$FF then break;
		if OverLimit then exit;
		end;
	Result:=$FF00 or B;
	end;


	function FindSOS: word;
	{Skip through start of file to Start of Scan (SOS)}
	begin
	Result:=0;
	while true do
		begin
		Result:=GetMarker;
		case Result of
		 $FFD9,$FFDA: break;
		 $FF01,$FFD0..$FFD8: ;
		 else
			begin
			{Size must be read in a separate step}
			Size:=GetStreamWord(Stream);
			Stream.Position:=Stream.Position+Size-2;
			end;
		 end;
		if OverLimit then exit;
		end;
	end;


	function FindEOI: word;
	{Find End of Image}
	var B: Byte;
	begin
	Result:=0;
	while true do
		begin
		B:=GetStreamByte(Stream);
		if B=$FF then
			begin
			B:=GetStreamByte(Stream);
			if B=$D9 then
				begin
				Result:=$FFD9;
				exit;
				end;
			end;
		if OverLimit then exit;
		end;
	end;


begin
Result:=0;
StartPos:=Stream.Position;
StreamEndian:=BigEnd;
if (FindSOS=$FFDA) then
  if (FindEOI=$FFD9) then Result:=Stream.Position-StartPos
else Result:=0;
end;



function BitmapTypeToStr(BT: TBitmapType): string;
begin
case BT of
 btType2: Result:='Type2';
 btType3: Result:='Type3';
 btType4: Result:='Type4';
 btUnknown: Result:='Unknown';
 end;
end;



function GetBMPLength(Stream: TStream; var BMPType: TBitmapType): int64;
var FileHeader: TBitmapFileHeader;
var CoreHeader: TBitmapCoreHeader;
begin
BMPType:=btUnknown;
Result:=0;
Stream.Read(FileHeader,SizeOf(FileHeader));
Stream.Read(CoreHeader,SizeOf(CoreHeader));
if FileHeader.bfType <> $4D42 then exit;
case CoreHeader.bcSize of
 12: BMPType:=btType2;
 40: BMPType:=btType3;
 108: BMPType:=btType4;
 else exit;
 end;
Result:=FileHeader.bfSize;
end;


{------------------------------------------------------------------------------}




function FindImageBorder(StartX,StartY: integer; BM: TBitmap; Thresh: double): TRect;
{Find bounding box for a smaller, rectangular image within the specified bitmap }
{StartX/StartY should be some point within the retangular image. }
{Thresh should a be a value greater than zero. Higher value make it less sensetive false-borders}
{Works on the principle that the border of an image should have very little}
{Variation in brightness compared to the image itself}
var Scan: pRGBTripleArray;
const HalfWidth = 50;
var X1,X2,Y1,Y2: integer;



	function GetRowBrightSTD(X1,X2,Row: integer): extended;
	{Find of the standard deviation of the brightness of the specified row}
	{X1/X2 specify the subsection of the row}
	var X,Cnt: integer;
	var DA: array of double;
	begin
	Cnt:=(X2-X1)+1;
	SetLength(DA,Cnt);
	Scan:=BM.ScanLine[Row];
	for X:=X1 to X2 do DA[X-X1]:=RGBtoHSB(Scan[X]).Bright;
	Result:=GetArraySTD(DA);
	end;


	function GetColBrightSTD(Y1,Y2,Col: integer): extended;
	{Find of the standard deviation of the brightness of the specified Column}
	{X1/X2 specify the subsection of the row}
	var Y,Cnt: integer;
	var DA: array of double;
	begin
	Cnt:=(Y2-Y1)+1;
	SetLength(DA,Cnt);
	for Y:=Y1 to Y2 do
		begin
		Scan:=BM.ScanLine[Y];
		DA[Y-Y1]:=RGBtoHSB(Scan[Col]).Bright;
		end;
	Result:=GetArraySTD(DA);
	end;



	function TestThreashold(OldValue,NewValue: double): boolean;
	{Test for STD that are zero or drop quickly}
	{Thresh specifies the percentage drop to indicate we've reach the border}
	{The sign of Thresh is changed to allow the user to enter postive values }
	{Which are less confusing and more intuitive}
	begin
	Result:=(NewValue=0) or (((NewValue-Oldvalue)/OldValue)<-Thresh);
	end;




	function FindTopBorder: integer;
	{Find the top border}
	var OB,NB: extended;
	begin
	OB:=0;
	for Result:=StartY downto 0 do
		begin
		if Result=StartY then OB:=GetRowBrightSTD(X1,X2,Result);
	        NB:=GetRowBrightSTD(X1,X2,Result);
		if TestThreashold(OB,NB) then exit;
		OB:=NB;
		end;
	Result:=0;
	end;



	function FindBottomBorder: integer;
	{Find the bottom border}
	var OB,NB: extended;
	begin
	OB:=0;
	for Result:=StartY to BM.Height-1 do
		begin
		if Result=StartY then OB:=GetRowBrightSTD(X1,X2,Result);
	        NB:=GetRowBrightSTD(X1,X2,Result);
		if TestThreashold(OB,NB) then exit;
		OB:=NB;
		end;
	Result:=BM.Height-1;
	end;


	function FindLeftBorder: integer;
	{Find the left border}
	var OB,NB: extended;
	begin
	OB:=0;
	for Result:=StartX downto 0 do
		begin
		if Result=StartX then OB:=GetColBrightSTD(Y1,Y2,Result);
	        NB:=GetColBrightSTD(Y1,Y2,Result);
		if TestThreashold(OB,NB) then exit;
		OB:=NB;
		end;
	Result:=0;
	end;



	function FindRightBorder: integer;
	{Find the Right border}
	var OB,NB: extended;
	begin
	OB:=0;
	for Result:=StartX to BM.Width-1 do
		begin
		if Result=StartX then OB:=GetColBrightSTD(Y1,Y2,Result);
	        NB:=GetColBrightSTD(Y1,Y2,Result);
		if TestThreashold(OB,NB) then exit;
		OB:=NB;
		end;
	Result:=BM.Width-1;
	end;



begin
BM.PixelFormat:=pf24Bit;

{Look at a band 100 pixels wide}
X1:=StartX-HalfWidth;
if X1<0 then X1:=0;
X2:=X1 + HalfWidth * 2;
if X2>(BM.Width-1) then X2:=BM.Width-1;

{Find top and bottom borders}
Result.Top:=FindTopBorder;
Result.Bottom:=FindBottomBorder;

{Once we have top and bottom, use them as the column height}
Y1:=Result.Top;
Y2:=Result.Bottom;

Result.Left:=FindLeftBorder;
Result.Right:=FindRightBorder;
end;


{------------------------------------------------------------------------------}


{

Description: http://www.hackerfactor.com/blog/index.php?/archives/432-Looks-Like-It.html

https://blog.csdn.net/cmd9x/article/details/85000408

}


function GetAImageHash(SrcBmp: TBitmap): Int64;
{Calculate a Average Hash for the image}
var I, J: Integer;
var P24: PRGBTripleArray;
var AG, SG: Integer;
var AVG: Single;
var bmp: TBitmap;
var CT: array[0..7, 0..7] of Byte;
begin
bmp := TBitmap.Create;
try
bmp.Width := 8;
bmp.Height := 8;
bmp.PixelFormat := pf24bit;

{Squash Image to 8x8}
SetStretchBltMode(bmp.Canvas.Handle, HALFTONE);
StretchBlt(bmp.Canvas.Handle, 0, 0, bmp.Width, bmp.Height, SrcBmp.Canvas.Handle, 0, 0, SrcBmp.Width, SrcBmp.Height, SRCCOPY); //??????????????
{Convert to gray-scale and sum the pixel values for average }
AG := 0;
for I := 0 to 7 do
	begin
	P24 := bmp.ScanLine[I];
	for J := 0 to 7 do
		begin
		{Convert RGB to gray by weighting RGB}
		SG := Round((0.299 * P24[J].rgbtRed) + (0.587 * P24[J].rgbtGreen) + (0.114 * P24[J].rgbtBlue));;
		CT[I, J] := SG;
		AG := AG + SG;
		end;
	end;
{Calculate the average for all pixels }
AVG := AG / 64;
{Set each bit in the hash acording whether the pixel
is above or below the average}
Result := 0;
for I := 0 to 7 do
 for J := 0 to 7 do Result := Result shl 1 or Ord(CT[I, J] > AVG);
finally Bmp.Free; end;
end;


function GetHamDistance(Hash1, Hash2: Int64): Int64;
{Calculate hamming distance between two hashes}
var A: Int64;
begin
Result := 0;
A := Hash1 xor Hash2;
while A <> 0 do
	begin
	A := A and (A -1);
	Inc(Result);
	end;
end;



function BmpHandleTypeToStr(HT: TBitmapHandleType): string;
begin
case HT of
 bmDIB: Result:='DIB';
 bmDDB: Result:='DDB';
 else Result:='Unknown';
 end;
end;


function BmpPixelFormatToStr(PF: Graphics.TPixelFormat): string;
begin
case PF of
 pfDevice: Result:='Device';
 pf1bit: Result:='1bit';
 pf4bit: Result:='4bit';
 pf8bit: Result:='8bit';
 pf15bit: Result:='15bit';
 pf16bit: Result:='16bit';
 pf24bit: Result:='24bit';
 pf32bit: Result:='32bit';
 pfCustom: Result:='Custom';
 else Result:='Unknown';
 end;
end;


function BmpTransModeToStr(TM: TTransparentMode): string;
begin
case TM of
 tmAuto: Result:='Auto';
 tmFixed: Result:='Fixed';
 else Result:='Unknown';
 end;
end;


{------------------------------------------------------------------------------}



type TLine = record
 Count: Integer;
 Index: Integer;
 Alpha: Double;
 Distance: Double;
 end;

type TLineArray = array of TLine;



function GetImageAngle(const MaxAngle: Double; Treshold, Width, Height: Integer; Pixels: PByteArray; DetectionArea: PRect): Double;
{ Get the rotation angle for 8-bit grayscale byte array of Text or other lined objects}
{ MaxAngle is largest expected angle. (Smaller angles are faster.) }
{ Threshold (0..255) differentiates between black and white, black being text}
{ Based code by Marek Mauder }
var AlphaStart, MinDist, SumAngles: double;
var AlphaSteps, DistCount, AccumulatorSize, I, AccumulatedCounts: integer;
var BestLines: TLineArray;
var HoughAccumulator: array of Integer;
var PageWidth, PageHeight: Integer;
var ContentRect: TRect;
const BestLinesCount = 20;	{ Number of "best" lines we take into account when determining lines with most votes).}
const  AlphaStep = 0.1;		{ Angle step used in alpha parameter quantization}


	function IsPixelBlack(X, Y: Integer): Boolean;
	{ Classifies pixel at [X, Y] as black or white using threshold.}
	begin
	Result := Pixels[Y * Width + X] < Treshold;
	end;


	function GetFinalAngle(StepIndex: Integer): Double;
	{ Calculates final angle for given angle step.}
	begin
	Result := AlphaStart + StepIndex * AlphaStep;
	end;

	procedure CalcLines(X, Y: Integer);
	{ Calculates angle and distance parameters for all lines}
	{ going through point [X, Y].}
	var D, Rads: Double;
	var I, DIndex, Index: Integer;
	var Sin, Cos: Extended;
	begin
	for I := 0 to AlphaSteps - 1 do
		begin
		{ Angle for current step in radians}
		Rads := GetFinalAngle(I) * PI / 180;
		SinCos(Rads, Sin, Cos);
		{ Parameter D(distance from origin) of the line y=tg(alpha)x + d}
		D := Y * Cos - X * Sin;
		{ Calc index into accumulator for current line}
		DIndex := Trunc(D - MinDist);
		Index := DIndex * AlphaSteps + I;
		{ Add one vote for current line}
		HoughAccumulator[Index] := HoughAccumulator[Index] + 1;
		end;
	end;



	procedure CalcHoughTransform;
	{ Uses Hough transform to calculate all lines that intersect}
	{ interesting points (those classified as beign on base line of the text).}
	var Y, X: Integer;
	begin
	for Y := 0 to PageHeight - 1 do
	 for X := 0 to PageWidth - 1 do
		begin
		if IsPixelBlack(ContentRect.Left + X, ContentRect.Top + Y) and
		    not IsPixelBlack(ContentRect.Left + X, ContentRect.Top + Y + 1) then
			begin
			CalcLines(X, Y);
			end;
		end;
	end;



	function GetBestLines(Count: Integer): TLineArray;
	{ Chooses "best" lines (with the most votes) from the accumulator}
	var I, J, DistIndex, AlphaIndex: Integer;
	var Temp: TLine;
	begin
	SetLength(Result, Count);

	for I := 0 to AccumulatorSize - 1 do
		begin
		if HoughAccumulator[I] > Result[Count - 1].Count then
			begin
			{ Current line has more votes than the last selected one,}
			{ let's put it the pot}
			Result[Count - 1].Count := HoughAccumulator[I];
			Result[Count - 1].Index := I;
			J := Count - 1;

			{ Sort the lines based on number of votes}
			while (J > 0) and (Result[J].Count > Result[J - 1].Count) do
				begin
				Temp := Result[J];
				Result[J] := Result[J - 1];
				Result[J - 1] := Temp;
				J := J - 1;
				end;
			end;

		AccumulatedCounts := AccumulatedCounts + HoughAccumulator[I];
		end;

	for I := 0 to Count - 1 do
		begin
		{ Caculate line angle and distance according to index in the accumulator}
		DistIndex := Result[I].Index div AlphaSteps;
		AlphaIndex := Result[I].Index - DistIndex * AlphaSteps;
		Result[I].Alpha := GetFinalAngle(AlphaIndex);
		Result[I].Distance := DistIndex + MinDist;
		end;
	end;



begin
AccumulatedCounts := 0;

{ Use supplied page content rect or just the whole image}
ContentRect := Rect(0, 0, Width, Height);
if DetectionArea <> nil then
	begin
	{ Assert((RectWidth(DetectionArea^) <= Width) and (RectHeight(DetectionArea^) <= Height));}
	ContentRect := DetectionArea^;
	end;

PageWidth := ContentRect.Right - ContentRect.Left;
PageHeight := ContentRect.Bottom - ContentRect.Top;

{ Don't check for black pixels outsize of image in CalcHoughTransform()}
if (ContentRect.Bottom = Height) then Dec(PageHeight);

AlphaStart := -MaxAngle;

{ Number of angle steps = samples from interval <-MaxAngle, MaxAngle>}
AlphaSteps := Ceil(2 * MaxAngle / AlphaStep);
MinDist := -Max(PageWidth, PageHeight);
DistCount := 2 * (PageWidth + PageHeight);

{ Determine the size of line accumulator}
AccumulatorSize := DistCount * AlphaSteps;
SetLength(HoughAccumulator, AccumulatorSize);

{ Calculate Hough transform}
CalcHoughTransform;

{ Get the best lines with most votes}
BestLines := GetBestLines(BestLinesCount);

{ Average angles of the selected lines to get the rotation angle of the image}
SumAngles := 0;
for I := 0 to BestLinesCount - 1 do SumAngles := SumAngles + BestLines[I].Alpha;
Result := SumAngles / BestLinesCount;

{Record stats}
ImageAngleStats.BestCount := BestLines[0].Count;
ImageAngleStats.PixelCount := PageWidth * PageHeight;
ImageAngleStats.AccumulatorSize := AccumulatorSize;
ImageAngleStats.AccumulatedCounts := AccumulatedCounts;
ImageAngleStats.TestedPixels := AccumulatedCounts div AlphaSteps;
end;



function GetImageAngle(BM: TBitmap; const MaxAngle: Double; Treshold: Integer; DetectionArea: PRect): Double;
var BA: TDynByteArray;
begin
RGBBitmapToBytes(BM, BA);
Result:=GetImageAngle(MaxAngle,Treshold,BM.Width,BM.Height,@BA[0]);
end;


{-------------------------------------------------------------------------------}



type THistogram = record
 Bins: array [0..255] of integer;
 Colors: array [0..255] of TRGBTriple;
 end;


procedure MedianBlur(Src,Dest: TBitmap;  WindowX, WindowY: integer);
var x, y, X1, Y1, med, md, dl, delta_l, WX2, WY2: integer;
var I, MedSum, XStart,XEnd, YStart,YEnd, MedInx: integer;
var middle: integer;
var Histogram: THistogram;
var u: byte;
var Color: TRGBTriple;
var SrcRows,DestRows: TRGBTripleRowArray;
begin
WindowX:=WindowX * 2 -1;
WindowY:=WindowY * 2 -1;

Src.PixelFormat:=pf24Bit;
Dest.PixelFormat:=pf24Bit;

Dest.Width:=Src.Width;
Dest.Height:=Src.Height;
SetLength(SrcRows,Src.Height);
SetLength(DestRows,Dest.Height);

{Capture scan lines of both source and destiantion bitmaps}
for Y:=0 to Src.Height-1 do SrcRows[Y]:=Src.ScanLine[Y];
for Y:=0 to Dest.Height-1 do DestRows[Y]:=Dest.ScanLine[Y];


WX2 := WindowX div 2;
WY2 := WindowY div 2;

middle := (WindowX * WindowY-1) div 2;

for y := 0 to SRC.Height-1 do
	begin
	{ Determine the histogram and median for the first element of each row}
	YStart:=Y - WY2;
	YEnd:=Y + WY2;

	{ histogram reset }
	for I := 0 to 255 do Histogram.Bins[I] := 0;

	{recalculation of the histogram for the start element row=y, col=0 }
	for Y1 := YStart to YEnd do
	 for X1 := -WX2 to WX2 do
	 	begin
		{It is the first pixel on the row, so don't worry about right edge}
	 	if (Y1>=0) and (Y1<SRC.Height) and (X1>=0) then Color:=SrcRows[Y1][X1] else Color:=MakeRBGTriple(0,0,0); // Color:=SrcRows[y][0];
		U:=RGBToGray(Color);
	 	inc(Histogram.Bins[U]);
		Histogram.Colors[U]:=Color;
	 	end;

	 { now determine the median }
	 MedSum := 0;
	 for MedInx := 0 to 255 do
	 	begin
	 	inc(MedSum,Histogram.Bins[MedInx]);
	 	if MedSum > middle then break;
	 	end;
	 med := MedInx;

	 delta_l := MedSum - Histogram.Bins[MedInx];
         DestRows[Y][0]:=Histogram.Colors[MedInx];

	 { Loop through each column in this row}
	 for x := 1 to Src.Width-1 do
	 	begin
	 	XStart := x-wx2-1;
	 	XEnd := x+wx2;
	 	{ go to next column }
	 	for Y1 := YStart to YEnd do
	 		begin
	 		if (XStart >= 0) and (Y1 >= 0) and (Y1 < SRC.Height) then Color:=SrcRows[Y1][XStart] else Color:=MakeRBGTriple(0,0,0); //  Color:=SrcRows[Y][X];
	 		U:=RGBToGray(Color);
	 		if Histogram.Bins[u]>0 then dec(Histogram.Bins[u]);
	 		if u < med then dec(delta_l);
	 		if (XEnd < Src.Width) and (Y1 >= 0) and (Y1 < SRC.Height) then Color:=SrcRows[Y1][XEnd] else Color:=MakeRBGTriple(0,0,0); //  Color:=SrcRows[Y][X];
	 		U:=RGBToGray(Color);
	 		inc(Histogram.Bins[u]);
	 		Histogram.Colors[U]:=Color;
	 		if u < med then inc(delta_l);
	 		end;

	 	{ update new median }
	 	dl := delta_l;
	 	md := med;
	 	if dl > middle then
	 		begin
	 		while dl > middle do
	 			begin
	 			dec(md);
	 			if Histogram.Bins[md] > 0 then
	 			dec(dl,Histogram.Bins[md]);
	 			end;
	 		end
	 	else
	 		begin
	 		while dl + Histogram.Bins[md] <= middle do
	 			begin
	 			if Histogram.Bins[md] > 0 then inc(dl,Histogram.Bins[md]);
	 			inc(md);
	 			end;
	 		end;
	 	delta_l := dl;
	 	med := md;
	 	DestRows[Y][X]:= Histogram.Colors[med];
	 	end;   { x loop}
	 end; { y loop}
end;




procedure MedianBlur(Image: TBitmap; Radius: integer);
var Bmp: TBitmap;
begin
Bmp:=TBitmap.Create;
try
MedianBlur(Image,Bmp,Radius,Radius);
Image.Assign(Bmp);

finally Bmp.Free; end;
end;



{===============================================================================}



procedure BilateralFilter(Src,Dest: TBitmap; Radius: integer; IntSigma, DistSIgma: double; Mode: TBilateralMode = bmBrightness; Progress: TBitmapProgress = nil);
{Filter image using bilateral filtering. Works by filtering by two criteria distance and intensity.}
{Pixels far from the target pixels in spacial distance and visual distance are deweighted. }
{IntSigma = Controls Intensity Standard Deviation }
{DistSigma = Controls Distance Standard Deviation }
{Mode allows you to choose which components of the color comprise intensive }
var SrcRows,DestRows: TRGBTripleRowArray;
var Y, Width, Height, I, J: integer;

	function Distance(X, Y, I, J: integer): double;
	{Get Euclidean distance between pixel X,Y and I,J}
	begin
	Result:=sqrt(sqr(x - i) + sqr(y - j));
	end;


	function Gaussian(x, sigma: double): double;
	{Get gaussian as a function of distance from median}
	{X = distance from median, Sigma is standard deviations of the distribueion}
	begin
	Result:=exp(-(sqr(x))/(2 * sqr(sigma)));
	end;


	function GetIntensity(Scan: TRGBTripleRowArray; X,Y: integer): double;
	{Get pixel intensity}
	begin
	case Mode of
         bmBrightness: Result:=RGBToGray(Scan[Y,X])/255;
	 bmHue: Result:=RGBtoHSB(Scan[Y,X]).Hue/360;
         bmSaturation: Result:=RGBtoHSB(Scan[Y,X]).Sat;
         else Result:=RGBToGray(Scan[Y,X])/255;
         end;
	end;


	function GetColorVector(Scan: TRGBTripleRowArray; X,Y: integer): T3DVector;
	{Get color value as a 3D vector}
	begin
	Result.X:=Scan[Y,X].rgbtRed;
	Result.Y:=Scan[Y,X].rgbtGreen;
	Result.Z:=Scan[Y,X].rgbtBlue;
	end;


	procedure FilterPixel(X, Y: integer);
	var iFiltered: T3DVector;
	var wP, gi,gs ,w : double;
	var X1,Y1,NX,NY: integer;
	begin
	iFiltered:=MakeVector3D(0,0,0);
	wP:=0;
	for X1:=-Radius to Radius do
		begin
		for Y1:=-Radius to Radius do
			begin
			NX:=X + X1;
			NY:=Y + Y1;
			{Don't sample pixels outside bitmap}
			if (NX<0) or (NX>=Src.Width) or
			   (NY<0) or (NY>=Src.Height) then continue;
			{Get the gaussian weight of the pixel intensity}
			Gi:=Gaussian( GetIntensity(SrcRows,NX, NY) - GetIntensity(SrcRows,X,Y), IntSigma);
			{Get the gaussian weight of the distance between pixels}
			GS:=Gaussian(distance(X, Y, NX, NY), DistSIgma);
			{Combine the two weights}
			W:=GI * GS;

			{Weight and sum the pixels}
			iFiltered:=VectorAdd3D(iFiltered, ScalarProduct3D(GetColorVector(SrcRows,NX, NY),w));
			{Sum the weights}
			wP:=wP + w;
			end;
		end;
	{Normalize the pixel}
	iFiltered:=ScalarDivide3D(iFiltered,wP);

	{Convert from vector to RGB color}
	DestRows[y,X].rgbtRed:=Trunc(iFiltered.X);
	DestRows[y,X].rgbtGreen:=Trunc(iFiltered.Y);
	DestRows[y,X].rgbtBlue:=Trunc(iFiltered.Z);
	end;



begin
if Assigned(Progress) then Progress(0);
Src.PixelFormat:=pf24Bit;
Dest.PixelFormat:=pf24Bit;

Dest.Width:=Src.Width;
Dest.Height:=Src.Height;
SetLength(SrcRows,Src.Height);
SetLength(DestRows,Dest.Height);

{Capture scan lines of both source and destiantion bitmaps}
for Y:=0 to Src.Height-1 do SrcRows[Y]:=Src.ScanLine[Y];
for Y:=0 to Dest.Height-1 do DestRows[Y]:=Dest.ScanLine[Y];

width:=Src.Width;
height:=Src.Height;

for J:=0 to height - 1 do
	begin
	if Assigned(Progress) then Progress(MulDiv(100,J,Height));
	for I:=0 to width - 1 do
		begin
		FilterPixel(i, j);
		end;
	end;
end;


end.
