unit CertificateLib;

interface

uses Windows,SysUtils,WinCrypt,WinTrust,DSupport;


type TProgramPublisher = record
 ProgramName: string;
 PublisherLink: string;
 MoreInfoLink: string;
 end;


type TCertDetails = record
 SerialNumber: array of byte;
 IssuerName,SubjectName: string;
 end;

type TCertName = record
 Email,Name,FriendlyName: string;
 end;

type TCertNames = record
 Issuer,User: TCertName;
 end;


type TCertificateInfo = record
 IsSigned: boolean;
 Publisher: TProgramPublisher;
 Names: TCertNames;
 SignerDetails,TimeDetails: TCertDetails;
 IsTimeStamped: boolean;
 TimeStamp: TDateTime;
 end;

function GetCertificateInfo(Filename: WideString; var Info: TCertificateInfo): integer;
function GetCertificateInfoStr(Info: TCertificateInfo): string;
function IsFileSigned(Filename: WideString): boolean;


implementation

function IsFileSigned(Filename: WideString): boolean;
var ContentType: DWORD;
var Store: HCERTSTORE;
var Msg: HCRYPTMSG;
begin
ContentType:= 0;
Result:=CryptQueryObject(CERT_QUERY_OBJECT_FILE, @FileName[1], CERT_QUERY_CONTENT_FLAG_PKCS7_SIGNED_EMBED, CERT_QUERY_FORMAT_FLAG_BINARY, 0, nil, @ContentType, nil, @Store, @Msg, nil);
end;


{Encoding that is used for all the certificate data}

const ENCODING = X509_ASN_ENCODING  or  PKCS_7_ASN_ENCODING;


{Structures to recast the pointers into typed arrays}

type TCryptAttribArray = array [0..3200] of CRYPT_ATTRIBUTE;
type PCryptAttribArray = ^TCryptAttribArray;

type TCryptBlobArray = array [0..3200] of _CRYPTOAPI_BLOB;
type PCryptBlobArray = ^TCryptBlobArray;


function GetCertDetailsStr(CD: TCertDetails): string;
var I: integer;
begin
Result:='Serial Number: ';
for I:=0 to High(CD.SerialNumber) do
 Result:=Result+IntToHex(CD.SerialNumber[I],2)+' ';
Result:=Result+CRLF_Char;
Result:=Result+'Issuer Name: '+ CD.IssuerName+CRLF_Char;
Result:=Result+'Subject Name: '+ CD.SubjectName;
Result:=Result+''+CRLF_Char;
end;

function GetCertNamesStr(Names: TCertNames): string;
begin
Result:=Result+'User Name: '+Names.User.Name+CRLF_Char;
Result:=Result+'User Friendly Name: '+Names.User.FriendlyName+CRLF_Char;
Result:=Result+'User Email: '+Names.User.Email+CRLF_Char;
Result:=Result+''+CRLF_Char;

Result:=Result+'Issuer Name: '+Names.Issuer.Name+CRLF_Char;
Result:=Result+'Issuer Friendly Name: '+Names.Issuer.FriendlyName+CRLF_Char;
Result:=Result+'Issuer Email: '+Names.Issuer.Email+CRLF_Char;
Result:=Result+''+CRLF_Char;
end;



function GetCertificateInfoStr(Info: TCertificateInfo): string;
begin
Result:='File Not Signed - No More Info'+CRLF_Char;
if not Info.IsSigned then exit;
Result:='File Signed: Yes'+CRLF_Char;

if Info.Publisher.ProgramName <> '' then Result:=Result+'Program Name : '+ Info.Publisher.ProgramName+CRLF_Char;
if Info.Publisher.PublisherLink <> '' then Result:=Result+'Publisher Link: '+ Info.Publisher.PublisherLink+CRLF_Char;
if Info.Publisher.MoreInfoLink <> '' then Result:=Result+'MoreInfo Link : ' + Info.Publisher.MoreInfoLink+CRLF_Char;

Result:=Result+GetCertNamesStr(Info.Names);
Result:=Result+'Signer Certificate: '+CRLF_Char;
Result:=Result+GetCertDetailsStr(Info.SignerDetails);
Result:=Result+''+CRLF_Char;
Result:=Result+'TimeStamp Certificate:'+CRLF_Char;
Result:=Result+GetCertDetailsStr(Info.TimeDetails);
Result:=Result+'Date of TimeStamp: '+DateTimeToStr(Info.TimeStamp)+CRLF_Char;
Result:=Result+''+CRLF_Char;
end;



function GetCertContextData(CertContext: PCertContext; var Names: TCertNames): string;
var CertName: array[0..255] of WideChar;
begin
Result:='';
if CertGetNameStringW(CertContext, CERT_NAME_EMAIL_TYPE, 0, nil, CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.User.Email:=CertName;
if CertGetNameStringW(CertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, 0, nil,  CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.User.Name:=CertName;
if CertGetNameStringW(CertContext, CERT_NAME_FRIENDLY_DISPLAY_TYPE, 0, nil, CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.User.FriendlyName:=CertName;

if CertGetNameStringW(CertContext, CERT_NAME_EMAIL_TYPE, CERT_NAME_ISSUER_FLAG, nil, CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.Issuer.Email:=CertName;

if CertGetNameStringW(CertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nil, CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.Issuer.Name:=CertName;

if CertGetNameStringW(CertContext, CERT_NAME_FRIENDLY_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nil, CertName, 256) = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
Names.Issuer.FriendlyName:=CertName;
end;



function GetProgAndPublisherInfo(pSignerInfo: PCMSG_SIGNER_INFO; var Publisher: TProgramPublisher): boolean;
var OpusInfo: PSPC_SP_OPUS_INFO ;
var dwData: DWORD ;
var N: integer;
var PCAA: PCryptAttribArray;
var PCBA: PCryptBlobArray;
begin
Result := False;
OpusInfo := nil;
Publisher.ProgramName:='';
Publisher.PublisherLink:='';
Publisher.MoreInfoLink:='';
try
for N:=0 to pSignerInfo.AuthAttrs.cAttr-1 do
	begin
	{ Loop through authenticated attributes and find SPC_SP_OPUS_INFO_OBJID OID.}
	PCAA:=PCryptAttribArray(pSignerInfo.AuthAttrs.rgAttr);
	if lstrcmpA(SPC_SP_OPUS_INFO_OBJID,PCAA^[n].pszObjId) = 0 then
		begin
		{ Get Size of SPC_SP_OPUS_INFO structure.}
		PCBA:=PCryptBlobArray(PCAA^[n].rgValue);
		if not CryptDecodeObject(ENCODING, SPC_SP_OPUS_INFO_OBJID, Pointer(PCBA^[0].pbData), PCBA^[0].cbData, 0, nil, dwData) then RaiseLastOSErrorMsg('CryptDecodeObject failed: ');

		{ Allocate memory for SPC_SP_OPUS_INFO structure.}
		OpusInfo := AllocMem(dwData);
		if OpusInfo=nil then raise Exception.Create('Unable to allocate memory for Publisher Info.');

		{ Decode and get SPC_SP_OPUS_INFO structure.}
		if not CryptDecodeObject(ENCODING, SPC_SP_OPUS_INFO_OBJID, pointer(PCBA^[0].pbData), PCBA^[0].cbData, 0, OpusInfo, dwData) then RaiseLastOSErrorMsg('CryptDecodeObject failed: ');

		{ Fill in Program Name if present.}
		if OpusInfo.pwszProgramName<>nil then Publisher.ProgramName:= OpusInfo.pwszProgramName;

		{ Fill in Publisher Information if present.}
		if OpusInfo.pPublisherInfo<>nil then
			begin
			case OpusInfo.pPublisherInfo.dwLinkChoice of
			 SPC_URL_LINK_CHOICE: Publisher.PublisherLink := OpusInfo.pPublisherInfo.pwszUrl;
			 SPC_FILE_LINK_CHOICE: Publisher.PublisherLink := OpusInfo.pPublisherInfo.pwszFile;
			 end;
			end;

		{ Fill in more info if present}
		if OpusInfo.pMoreInfo<>nil then
			begin
			case OpusInfo.pMoreInfo.dwLinkChoice of
			 SPC_URL_LINK_CHOICE: Publisher.MoreInfoLink := OpusInfo.pMoreInfo.pwszUrl;
			 SPC_FILE_LINK_CHOICE: Publisher.MoreInfoLink := OpusInfo.pMoreInfo.pwszFile;
			 end;
			end;
		Result := TRUE;
		break; { Break from for loop.}
		end; { lstrcmp SPC_SP_OPUS_INFO_OBJID}
	end; { for}
finally if OpusInfo <> nil then FreeMem(OpusInfo); end;
end;




function GetCertDetails(pCertContext: TCertContext; var CD: TCertDetails): BOOL;
var szName: LPTSTR;
var Size: DWORD;
var N: integer;
var S: string;
begin
Result := FALSE;
szName := nil;
try
{ Print Serial Number.}
Size := pCertContext.pCertInfo.SerialNumber.cbData;
SetLength(CD.SerialNumber,Size);
for N:=0 to Size-1 do CD.SerialNumber[N]:=PByteArray(pCertContext.pCertInfo.SerialNumber.pbData)[Size - (n + 1)];

{ Get Issuer name size.}
Size := CertGetNameString(@pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nil, nil, 0);
if Size = 0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
szName := AllocMem(Size);

{ Get Issuer name.}
if CertGetNameString(@pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, CERT_NAME_ISSUER_FLAG, nil, szName, Size)=0 then RaiseLastOSErrorMsg('CertGetNameString failed');
CD.IssuerName:=szName;
FreeMem(szName);
szName := nil;


{ Get Subject name size.}
Size := CertGetNameString(@pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, 0, nil, nil, 0);
if Size=0 then RaiseLastOSErrorMsg('CertGetNameString failed: ');
{ Allocate memory for subject name.}
szName := AllocMem(Size * sizeof(AnsiChar));
{ Get subject name.}
if CertGetNameString(@pCertContext, CERT_NAME_SIMPLE_DISPLAY_TYPE, 0, nil, szName, Size)=0 then RaiseLastOSErrorMsg('CertGetNameString failed.');
{ Print Subject Name.}
CD.SubjectName:=szName;
Result := True;
finally if szName <> nil then FreeMem(szName); end;
end;




function GetDateOfTimeStamp(pSignerInfo: PCMSG_SIGNER_INFO; var ST: SYSTEMTIME ): boolean;
var lft, ft: FILETIME;
var dwData: DWORD;
var N: integer;
var PCAA: PCryptAttribArray;
var PCBA: PCryptBlobArray;
begin
Result:=False;
{ Loop through authenticated attributes and find szOID_RSA_signingTime OID.}
for N:=0 to pSignerInfo.AuthAttrs.cAttr-1 do
	begin
	PCAA:=PCryptAttribArray(pSignerInfo.AuthAttrs.rgAttr);
	if (lstrcmpA(szOID_RSA_signingTime, PCAA^[n].pszObjId) = 0) then
		begin
		{ Decode and get FILETIME structure.}
		dwData:=sizeof(ft);
		PCBA:=PCryptBlobArray(PCAA^[n].rgValue);
		if not CryptDecodeObject(ENCODING, szOID_RSA_signingTime, Pointer(PCBA^[0].pbData), PCBA^[0].cbData, 0, @ft, dwData) then RaiseLastOSErrorMsg('CryptDecodeObject failed: ');
		{ Convert to local time.}
		FileTimeToLocalFileTime(ft, lft);
		FileTimeToSystemTime(lft, st);
		Result := TRUE;
		break; { Break from for loop.}
		end; {lstrcmp szOID_RSA_signingTime}
	end; { for}

end;



function GetTimeStampSignerInfo(pSignerInfo: PCMSG_SIGNER_INFO; var pCounterSignerInfo: PCMSG_SIGNER_INFO ): BOOL;
var pCertContext: PCCERT_CONTEXT;
var dwSize: DWORD;
var N: integer;
var PCAA: PCryptAttribArray;
var PCBA: PCryptBlobArray;
begin
pCertContext := nil;
Result := FALSE;
try
pCounterSignerInfo := nil;
{ Loop through unathenticated attributes for szOID_RSA_counterSign OID.}
for N:=0 to pSignerInfo.UnauthAttrs.cAttr-1 do
	begin
	PCAA:=PCryptAttribArray(pSignerInfo.UnauthAttrs.rgAttr);
	if (lstrcmpA(PCAA^[n].pszObjId,szOID_RSA_counterSign) = 0) then
		begin
		{ Get size of CMSG_SIGNER_INFO structure.}
		PCBA:=PCryptBlobArray(PCAA^[n].rgValue);
		if not CryptDecodeObject(ENCODING, PKCS7_SIGNER_INFO, Pointer(PCBA^[0].pbData), PCBA^[0].cbData, 0, nil, dwSize) then RaiseLastOSErrorMsg('CryptDecodeObject failed with: ');
		{ Allocate memory for CMSG_SIGNER_INFO.}
		pCounterSignerInfo := AllocMem(dwSize);
		{ Decode and get CMSG_SIGNER_INFO structure}
		{ for timestamp certificate.}
		if not CryptDecodeObject(ENCODING, PKCS7_SIGNER_INFO, Pointer(PCBA^[0].pbData), PCBA^[0].cbData, 0, pCounterSignerInfo, dwSize) then RaiseLastOSErrorMsg('CryptDecodeObject failed with: ');
		Result := TRUE;
		break; { Break from for loop.}
		end;
	end;
finally if pCertContext <> nil then CertFreeCertificateContext(pCertContext); end;	{ Clean up.}
end;




function GetCertificateInfo(Filename: WideString; var Info: TCertificateInfo): integer;
var hStore: HCERTSTORE;
var hMsg: HCRYPTMSG;
var pCertContext: PCCERT_CONTEXT;
var fResult: BOOL;
var dwEncoding, dwContentType, dwFormatType: DWORD;
var pSignerInfo: PCMSG_SIGNER_INFO;
var pCounterSignerInfo: PCMSG_SIGNER_INFO;
var dwSignerInfo: DWORD;
var CertInfo: CERT_INFO;
var st: SYSTEMTIME;
var S: string;
begin
try
hStore := nil;
hMsg := nil;
pCertContext := nil;
pSignerInfo := nil;
pCounterSignerInfo := nil;
Info.IsTimeStamped:=False;
{ Get message handle and store handle from the signed file.}
Info.IsSigned:=CryptQueryObject(CERT_QUERY_OBJECT_FILE, @Filename[1], CERT_QUERY_CONTENT_FLAG_PKCS7_SIGNED_EMBED, CERT_QUERY_FORMAT_FLAG_BINARY, 0, @dwEncoding, @dwContentType, @dwFormatType, @hStore, @hMsg, nil);
if not Info.IsSigned then exit;

{ Get signer information size.}
if not CryptMsgGetParam(hMsg, CMSG_SIGNER_INFO_PARAM, 0, nil, dwSignerInfo) then RaiseLastOSErrorMsg('CryptMsgGetParam failed: ');

{ Allocate memory for signer information.}
pSignerInfo := AllocMem(dwSignerInfo);

{ Get Signer Information.}
if not CryptMsgGetParam(hMsg, CMSG_SIGNER_INFO_PARAM, 0,pSignerInfo, dwSignerInfo) then RaiseLastOSErrorMsg('CryptMsgGetParam failed: ');

{ Get program name and publisher information from}
{ signer info structure.}

if GetProgAndPublisherInfo(pSignerInfo, Info.Publisher) then
	begin
	{ Search for the signer certificate in the temporary certificate store.}
	CertInfo.Issuer := pSignerInfo.Issuer;
	CertInfo.SerialNumber := pSignerInfo.SerialNumber;
	pCertContext := CertFindCertificateInStore(hStore, ENCODING, 0, CERT_FIND_SUBJECT_CERT, @CertInfo,nil);
	if pCertContext=nil then raise Exception.Create('CertFindCertificateInStore failed with: '+SysErrorMessage(GetLastError));
	GetCertContextData(pCertContext, Info.Names);

	{ Get Signer certificate information.}
	GetCertDetails(pCertContext^,Info.SignerDetails);

	{ Get the timestamp certificate signerinfo structure.}
	if GetTimeStampSignerInfo(pSignerInfo, pCounterSignerInfo) then
		begin
		{ Search for Timestamp certificate in the temporary}
		{ certificate store.}
		CertInfo.Issuer := pCounterSignerInfo.Issuer;
		CertInfo.SerialNumber := pCounterSignerInfo.SerialNumber;

		pCertContext := CertFindCertificateInStore(hStore, ENCODING, 0, CERT_FIND_SUBJECT_CERT, @CertInfo, nil);
		if pCertContext=nil then RaiseLastOSErrorMsg('CertFindCertificateInStore failed with: ');

		{ Get timestamp certificate information.}
		GetCertDetails(pCertContext^,Info.TimeDetails);

		{ Find Date of timestamp.}
		if GetDateOfTimeStamp(pCounterSignerInfo, st) then
			begin
			Info.TimeStamp:=SystemTimeToDateTime(ST);
			Info.IsTimeStamped:=True;
			end;
		end;
	end;
finally
 { Clean up.}
 if pSignerInfo <> nil then FreeMem(pSignerInfo);
 if pCounterSignerInfo <> nil then FreeMem(pCounterSignerInfo);
 if pCertContext <> nil then CertFreeCertificateContext(pCertContext);
 if hStore <> nil then CertCloseStore(hStore, 0);
 if hMsg <> nil then CryptMsgClose(hMsg);
 end;
Result:=0;
end;




end.
