unit ContourLib;

interface

uses Graphics,Vector,DSupport,HugeArry,Huge2D;

type TContourMaker = class(TObject)
 private
  procedure OrderPoints(var TRI: TTriangle);
  function Interpolate(Line: T3DLine; var IP:T3DVector; Z: double): boolean;
  procedure DoTriSides(var ContourLines: THuge3DLine; L1,L2,L3: T3DLine; Interval: double);
  procedure DoTriangle(var ContourLines: THuge3DLine; Tri:TTriangle;SP:double);
 protected
 public
  MinVert,MaxVert: double;
  procedure GenerateContours(var ContourLines: THuge3DLine; ElevPoints: T2DDoubleArray; LowerLeft: T2DVector; Spacing,Interval: double);
 published
 end;


implementation



procedure TContourMaker.OrderPoints(var TRI: TTriangle);
{ Sort Triangle Points Depths }
var Tmp : T3DVector;
begin
with TRI do
 begin
 if P1.Z>P3.Z then begin Tmp:=P1; P1:=P3; P3:=Tmp; end;
 if P1.Z>P2.Z then begin Tmp:=P1; P1:=P2; P2:=Tmp; end;
 if P2.Z>P3.Z then begin Tmp:=P2; P2:=P3; P3:=Tmp; end;
 end;
end;



function  TContourMaker.Interpolate(Line: T3DLine; var IP:T3DVector; Z: double): boolean;
{ Interpolate between end points of Line Based On Z Values }
var DX,DY,DZ1,DZ2,RATIO : double;
begin
with Line do
 begin
 DX:=P2.X-P1.X; DY:=P2.Y-P1.Y; DZ1:=P2.Z-P1.Z;
 if DZ1=0 then  begin Result:= False; exit; end;
 DZ2:=Z-P1.Z;
 RATIO:=DZ2/DZ1;
 IP.X:=P1.X+(DX*RATIO); IP.Y:=P1.Y+(DY*RATIO); IP.Z:=Z;
 Result:= True;
 end;
end;



procedure  TContourMaker.DoTriSides(var ContourLines: THuge3DLine; L1,L2,L3: T3DLine; Interval: double);
{ Connect Sides Of The Triangle With Contour Lines }
var ELEV : double;


	function FindNearest(N,Interval:double): double;
	{ Find Nearest Multiple Of Sp Above N }
	var T : double;
	begin
	if Frac(N/Interval)=0 then  begin Result:= N; exit; end;
	T:=Int(N/Interval);
	if N>0 then T:=T + 1;
	Result:= Interval * T;
	end;


	procedure  ConnectSideS(L1,L2: T3DLine; Z: double);
	{ Connect Lines L1 To L2  At Places Z Matches }
	var L3: T3DLine;
	var C: TColor;
	begin
	if not Interpolate(L1,L3.P1,Z) then exit;
	if not Interpolate(L2,L3.P2,Z) then exit;
	ContourLines.Add(MakeLine3D(L3.P1,L3.P2));
	end;


begin
ELEV:=FindNearest(L1.P1.Z,Interval);
while ELEV<=L1.P2.Z do
	begin
	if (ELEV>=L2.P1.Z)  and  (ELEV<=L2.P2.Z) then ConnectSides(L1,L2,ELEV);
	if (ELEV>=L3.P1.Z)  and  (ELEV<=L3.P2.Z) then ConnectSides(L1,L3,ELEV);
	ELEV:=ELEV+Interval;
	end;
end;



procedure TContourMaker.DoTriangle(var ContourLines: THuge3DLine; Tri:TTriangle;SP:double);
{ Contour This Triangle }
begin
with Tri do
 begin
 OrderPoints(TRI);
 DoTriSides(ContourLines,MakeLine3D(P1,P2),MakeLine3D(P2,P3),MakeLine3D(P1,P3),SP);
 DoTriSides(ContourLines,MakeLine3D(P2,P3),MakeLine3D(P1,P3),MakeLine3D(P1,P2),SP);
 end;
end;




procedure TContourMaker.GenerateContours(var ContourLines: THuge3DLine; ElevPoints: T2DDoubleArray; LowerLeft: T2DVector; Spacing,Interval: double);
{}
{ Draw Contour Map on canvas }
{PX,PY = Pixel Offset }
{X1,Y1,X2,Y2= Range of point in the array}
{Spacing = Pixel spacing between elevation points }
{Interval = Spacing between contour lines }
var X,Y : Integer;
var P1,P2,P3,P4: T3DVector;
var Tri: TTriangle;


	function GetCornerPoint(X,Y:integer): T3DVector;
	{ Get point from corner of a cell}
	begin
	Result.X:=(X*Spacing)+LowerLeft.X;
	Result.Y:=(Y*Spacing)+LowerLeft.Y;
	Result.Z:=ElevPoints[X,Y];
	end;


	procedure GetMinMax(var Min,Max: double);
	{Find minimum and maximum values in the data}
	var X,Y : Integer;
	begin
	Min:=ElevPoints[0,0]; Max:=Min;
	for Y:=0 to ElevPoints.Rows-1 do
	 for X:=0 to ElevPoints.Cols-1 do
		begin
		if ElevPoints[X,Y]>Max then Max:=ElevPoints[X,Y];
		if ElevPoints[X,Y]<Min then Min:=ElevPoints[X,Y];
		end;
	end;

begin
ContourLines.Clear;
GetMinMax(MinVert,MaxVert);
for Y:=0 to ElevPoints.YIndex-2 do
	begin
	for X:=0 to ElevPoints.XIndex-2 do
		begin
		P1:=GetCornerPoint(X,Y);
		P2:=GetCornerPoint(X+1,Y);
		P3:=GetCornerPoint(X+1,Y+1);
		P4:=GetCornerPoint(X,Y+1);

		Tri.P1:=P1; Tri.P2:=P2; Tri.P3:=P3;
		DoTriangle(ContourLines,Tri,Interval);

		Tri.P1:=P1; Tri.P2:=P3; Tri.P3:=P4;
		DoTriangle(ContourLines,Tri,Interval);
		end;
	end;
end;






end.
