unit DiscreteFourier;

interface

uses Graphics,DSupport,Graphbox;



type TFreqInfo = record
 Frequency: double;
 Magnitude: double;
 Phase: double;
 end;

type TSpectrum = record
 Frequencies: array of TFreqInfo;
 MaxMag: double;
 end;


procedure DoDiscFourier(Start,Stop,Step: double; var Data: array of double; var Spectrum: TSpectrum);
procedure SquareWaveTrans(Start,Stop,Step: double; var Data: array of double; var Spectrum: TSpectrum);
procedure GraphSpectrum(Graph: TGraphBox; Spectrum: TSpectrum; Color: TColor; Bits: integer = 16);

implementation




procedure GraphSpectrum(Graph: TGraphBox; Spectrum: TSpectrum; Color: TColor; Bits: integer = 16);
var I: integer;
var M: double;
var Limit, DBLimit: double;
begin
DBLimit:=-(Bits+1) * 6;
Limit:=AntiLog(DBLimit/20);
Graph.ClearData;
Graph.Decimals1:=0;
Graph.Decimals2:=0;
Graph.SetHorizontalRange(0,Length(Spectrum.Frequencies)+1,8);
Graph.SetModVerticalRange(DBLimit-10,10,20);
for I:=0 to Length(Spectrum.Frequencies)-1 do
	begin
	M:=Spectrum.Frequencies[I].Magnitude;
	M:=20*log(M/Spectrum.MaxMag);
	if M<DBLimit then M:=DBLimit;
	Graph.SetDataPoint(Spectrum.Frequencies[I].Frequency,M,1,0,Color);
	end;
end;




procedure DoDiscFourier(Start,Stop,Step: double; var Data: array of double; var Spectrum: TSpectrum);
var Freq: double;
var CosSum,SinSum,GeoMean,Phase: double;
var Count,I: integer;


	procedure GetFrequencySums(Freq: double; var Data: array of double; var CosSum,SinSum: double);
	{Find the Sin/Cos sum for a specific frequency}
	var CurTime,TimeInc,W: double;
	var I: integer;
	begin
	TimeInc:=1/Length(Data);
	CurTime:=TimeInc;
	CosSum:=0; SinSum:=0;
	for I:=0 to High(Data) do
		begin
	        W:=TwoPI * Freq * CurTime;
		CosSum:=CosSum+Cos(W) * Data[I];
		SinSum:=SinSum+Sin(W) * Data[I];
		CurTime:=CurTime+TimeInc;
		end;
	end;


begin
Freq:=Start;
Count:=Trunc((Stop-Start)/Step) div 2;
SetLength(Spectrum.Frequencies,Count);
Spectrum.MaxMag:=0;
for I:=0 to High(Spectrum.Frequencies) do
	begin
	GetFrequencySums(Freq,Data,CosSum,SinSum);
	Spectrum.Frequencies[I].Magnitude:=Sqrt((CosSum*CosSum)+(SinSum*SinSum));
	Spectrum.Frequencies[I].Phase:=ArcTan2(SinSum,CosSum);
	Spectrum.Frequencies[I].Frequency:=Freq;
	if Spectrum.MaxMag<Spectrum.Frequencies[I].Magnitude then Spectrum.MaxMag:=Spectrum.Frequencies[I].Magnitude;
	Freq:=Freq + Step;
	end;
end;




procedure SquareWaveTrans(Start,Stop,Step: double; var Data: array of double; var Spectrum: TSpectrum);
var Freq: double;
var CosSum,SinSum,GeoMean,Phase: double;
var Count,I: integer;


	function SquareSign(V: double): double;
	{Covert sine was point to square wave point}
	begin
	if V>=0 then Result:=1 else Result:=-1;
	end;


	procedure GetFrequencySums(Freq: double; var Data: array of double; var CosSum,SinSum: double);
	{Find the Sin/Cos sum for a specific frequency}
	var CurTime,TimeInc,W: double;
	var I: integer;
	begin
	TimeInc:=1/Length(Data);
	CurTime:=TimeInc;
	CosSum:=0; SinSum:=0;
	for I:=0 to High(Data) do
		begin
	        W:=TwoPI * Freq * CurTime;
		CosSum:=CosSum + SquareSign(Cos(W)) * Data[I];
		SinSum:=SinSum + SquareSign(Sin(W)) * Data[I];
		CurTime:=CurTime+TimeInc;
		end;
	end;


begin
Freq:=Start;
Count:=Trunc((Stop-Start)/Step) div 2;
SetLength(Spectrum.Frequencies,Count);
Spectrum.MaxMag:=0;
for I:=0 to High(Spectrum.Frequencies) do
	begin
	GetFrequencySums(Freq,Data,CosSum,SinSum);
	Spectrum.Frequencies[I].Magnitude:=Sqrt((CosSum*CosSum)+(SinSum*SinSum));
	Spectrum.Frequencies[I].Phase:=ArcTan2(SinSum,CosSum);
	Spectrum.Frequencies[I].Frequency:=Freq;
	if Spectrum.MaxMag<Spectrum.Frequencies[I].Magnitude then Spectrum.MaxMag:=Spectrum.Frequencies[I].Magnitude;
	Freq:=Freq + Step;
	end;
end;



end.
