unit ExpressionEval;

interface

uses Classes,SysUtils,DSupport,Math;

type TExpressionEvaluator = class(TObject)
 private
  ExpressionStr: string;
  Atom: char;
  EvalPtr: integer;
  VariableList: TStringList;
  procedure SetStatus(S: string);
  function AlgebraicExpression: double;
  function BooleanTerm: double;
  function Expression: double;
  function Factor: double;
  procedure GetChar;
  function GetDecimal: double;
  function GetFunction: double;
  function GetIdentifier: string;
  function HandleMax: double;
  function HandleMin: double;
  function LogicalExpression: double;
  procedure SkipSpaces;
  procedure SkipWhiteSpace;
  function Term: double;
  function GetFunctionArgument: double;
  procedure GetFunctionArguments2(var X, Y: double);
  procedure GetFunctionArguments3(var X, Y, Z: double);
  function HandleIfElse: double;
  function HandlePower: double;
    function HandleMod: double;
 protected
 public
  ErrorMessage: string;
  ErrorFlag: boolean;
  constructor Create;
  destructor Destroy; override;
  function EvaluateExpression(Express: string): double;
  procedure ClearVariableList;
  procedure SetVariable(Name: string; Value: double);
  function GetVariable(Name: string; var Value: double): boolean;
 end;


implementation



procedure TExpressionEvaluator.SetVariable(Name: string; Value: double);
var Inx: integer;
var PD: PDouble;
begin
Inx:=VariableList.IndexOf(Name);
if Inx=-1 then
	begin
	New(PD);
	Inx:=VariableList.AddObject(Name,TObject(PD));
	end;
PD:=PDouble(VariableList.Objects[Inx]);
PD^:=Value;
end;



function TExpressionEvaluator.GetVariable(Name: string; var Value: double): boolean;
var Inx: integer;
begin
Inx:=VariableList.IndexOf(Name);
Result:=Inx<>-1;
if Result then
	begin
	Value:=PDouble(VariableList.Objects[Inx])^;
	end;
end;






procedure TExpressionEvaluator.SetStatus(S: string);
begin
ErrorMessage:=S;
ErrorFlag:=True;
end;




procedure TExpressionEvaluator.GetChar;
begin
if EvalPtr > Length(ExpressionStr) then
	begin
	Atom:= ')';
	end
 else	begin
	Atom:= ExpressionStr[EvalPtr];
	Inc(EvalPtr);
	end;
end;



procedure TExpressionEvaluator.SkipWhiteSpace;
{ Skip Tabs And Spaces In Linebuf }
begin
while Atom < #$21 do GetChar;
end;



procedure TExpressionEvaluator.SkipSpaces;
{ Get Next Character, Ignoring Any Space Characters }
begin
repeat GetChar until Atom > SP_CHAR;
end;



function TExpressionEvaluator.GetDecimal: double;
{ Read In A Decimal String And Return Its Value }
var S: string;
begin
Result:=0;
Dec(EvalPtr);
ExtractNumber(ExpressionStr,EvalPtr,Result);
GetChar;
end;


function TExpressionEvaluator.GetIdentifier: string;
begin
Result:='';
while True do
	begin
	if not IsAlphaNumeric(Atom) then break;
	Result:=Result+Atom;
	GetChar;
	end;
end;

{------------------------------------------------------------------------------}




procedure TExpressionEvaluator.GetFunctionArguments3(var X,Y,Z: double);
begin
if Atom<>'(' then raise Exception.Create('"(" Missing');
GetChar;
X:=Expression;
if Atom<>',' then raise Exception.Create('Comma Missing');
GetChar;
Y:=Expression;
if Atom<>',' then raise Exception.Create('Comma Missing');
GetChar;
Z:=Expression;
if Atom<>')' then raise Exception.Create('")" Missing');
GetChar;
end;



function TExpressionEvaluator.HandleIfElse: double;
var X,Y,Z: double;
begin
GetFunctionArguments3(X,Y,Z);
if X<>0 then Result:=Y else Result:=Z;
end;


procedure TExpressionEvaluator.GetFunctionArguments2(var X,Y: double);
begin
if Atom<>'(' then raise Exception.Create('"(" Missing');
GetChar;
X:=Expression;
if Atom<>',' then raise Exception.Create('Comma Missing');
GetChar;
Y:=Expression;
if Atom<>')' then raise Exception.Create('")" Missing');
GetChar;
end;




function TExpressionEvaluator.HandleMin: double;
var X,Y: double;
begin
GetFunctionArguments2(X,Y);
Result:=Min(X,Y);
end;




function TExpressionEvaluator.HandleMax: double;
var X,Y: double;
begin
GetFunctionArguments2(X,Y);
Result:=Max(X,Y);
end;



function TExpressionEvaluator.HandlePower: double;
var X,Y: double;
begin
GetFunctionArguments2(X,Y);
Result:= Power(X,Y);
end;



function TExpressionEvaluator.GetFunctionArgument: double;
begin
if Atom<>'(' then raise Exception.Create('"(" Missing');
GetChar;
Result:=Expression;
if Atom<>')' then raise Exception.Create('")" Missing');
GetChar;
end;




function TExpressionEvaluator.HandleMod: double;
var X,Y: double;
begin
GetFunctionArguments2(X,Y);
Result:=FloatMod(X,Y);
end;





function TExpressionEvaluator.GetFunction: double;
var ID: string;
var D: double;
begin
ID:=LowerCase(GetIdentifier);
if GetVariable(ID,D) then Result:=D
else if ID='abs' then Result:=abs(GetFunctionArgument)
else if ID='acos' then Result:=ArcCos(GetFunctionArgument)
else if ID='asin' then Result:=ArcSin(GetFunctionArgument)
else if ID='atan' then Result:=ArcTan(GetFunctionArgument)
else if ID='cos' then Result:=cos(GetFunctionArgument)
else if ID='cosh' then Result:=Cosh(GetFunctionArgument)
else if ID='exp' then Result:=Exp(GetFunctionArgument)
else if ID='if' then Result:=HandleIfElse
else if ID='ln' then Result:=Ln(GetFunctionArgument)
else if ID='log' then Result:=Log(GetFunctionArgument)
else if ID='max' then Result:=HandleMax
else if ID='mod' then Result:=HandleMod
else if ID='min' then Result:=HandleMin
else if ID='pi' then Result:=Pi
else if ID='pow' then Result:=HandlePower
else if ID='ran' then Result:=Random * GetFunctionArgument
else if ID='sin' then Result:=sin(GetFunctionArgument)
else if ID='sinh' then Result:=sinh(GetFunctionArgument)
else if ID='sqrt' then Result:=sqrt(GetFunctionArgument)
else if ID='sqr' then Result:=sqr(GetFunctionArgument)
else if ID='tan' then Result:=tan(GetFunctionArgument)
else if ID='tanh' then Result:=Tanh(GetFunctionArgument)
else raise Exception.Create('Invalid Function/indentifier: '+ID);
end;




function TExpressionEvaluator.Factor: double;
{ Returns The Value Of A Factor }
var NEG: boolean;
begin
Result:=0;
while Atom='+' do SkipSpaces;		{ Ignore Unary Plus }
NEG:= False;
while Atom ='-' do			{ Unary Minus }
	begin
	SkipSpaces;
	NEG:= not NEG;
	end;

if IsAlpha(Atom) then Result:=GetFunction			{ Get Functions and Variables }
else if (Atom>='0') and (Atom<='9') then Result:= GetDecimal	{ Unsigned Integer }
 else case Atom of
  '(':	begin							{ Subexpression }
	SkipSpaces;
	Result:= Expression;
	if Atom<>')' then raise Exception.Create('Invalid Expression');
	SkipSpaces;
	end;
  else	raise Exception.Create('Syntax Error: '+Atom);
  end;

{ Identifiers, Numbers, Etc. May Terminate With A Space Or Tab }
SkipWhiteSpace;

{Handle binary operators with higher precedence than Unary Minus}
while Atom='^' do begin SkipSpaces; Result:= Power(Result,Factor); end;

{Apply Unary Minus value}
if NEG then Result:=-Result;
end;




function TExpressionEvaluator.Term: double;
{ Returns Factor * Factor, Etc. }
begin
Result:= Factor;
while True do
	case Atom of
	  '*':	begin SkipSpaces; Result:= Result * Factor; end;
	  '/':	begin SkipSpaces; Result:= Result / Factor; end;
	  else	break;
	end;
end;




function TExpressionEvaluator.AlgebraicExpression: double;
{ Returns Term + Term, Etc. }
begin
Result:= Term;
while True do
	case Atom of
	  '+':	begin SkipSpaces; Result:= Result + Term; end;
	  '-':	begin SkipSpaces; Result:= Result - Term; end
	else	break;
	end;
end; { Algexp }



function TExpressionEvaluator.LogicalExpression: double;
{ Returns A Boolean, E.g: Algexp < Algexp }
begin
if Atom='~' then				{ Unary 'not' Operator }
	begin
	SkipSpaces;
	Result:= not trunc(LogicalExpression);
	end
 else	begin
	Result:= AlgebraicExpression;
	case Atom of
	 '=':	begin SkipSpaces; Result:= longint(Result = AlgebraicExpression); end;
	 '#':	begin SkipSpaces; Result:= longint(Result <> AlgebraicExpression); end;
	 '>':	begin
		SkipSpaces;
		if Atom<>'=' then Result:= longint(Result > AlgebraicExpression)
		else begin SkipSpaces; Result:= longint(Result >= AlgebraicExpression); end;
		end;
	 '<':	begin
		SkipSpaces;
		if Atom<>'=' then Result:= longint(Result < AlgebraicExpression)
		else begin SkipSpaces; Result:= longint(Result <= AlgebraicExpression); end;
		end;
	 end;
	end;
end; { Logexp }



function TExpressionEvaluator.BooleanTerm: double;
{ Returns Logexp & Logexp }
begin
Result:= LogicalExpression;
while True do
	begin
	if Atom='&' then
		begin SkipSpaces; Result:= trunc(Result)  and  trunc(LogicalExpression); end
	 else break;
	end;
end;	{ Boolterm }




function TExpressionEvaluator.Expression: double;
{ Returns The Value Of An Expression }
begin
SkipWhiteSpace;		{ Must Not Have A Tab Or Space }
Result:= BooleanTerm;
while True do
	begin
	case Atom of
	 '!': begin SkipSpaces; Result:= trunc(Result)  or  trunc(BooleanTerm); end;
	 '|': begin SkipSpaces; Result:= trunc(Result)  xor trunc(BooleanTerm); end;
	 else break;
	 end;
	end;
end;	{ Expression }




function TExpressionEvaluator.EvaluateExpression(Express: string): double;
{ Evaluate Expression starting at EvalPtr }
begin
ExpressionStr:=Express;
ErrorFlag:=False;
ErrorMessage:='';
EvalPtr:=1;
try
GetChar;
Result:= Expression;
if Atom <> ')' then raise Exception.Create('Invalid Expression?');
except
 on EZeroDivide do begin SetStatus('Divide by 0'); raise; end;
 on EInvalidArgument do begin SetStatus('Parameter out of range'); raise end;
 on EInvalidOp do begin SetStatus('Processor encountered an undefined instruction'); raise end;
 on EOverflow do begin SetStatus('Floating-point operation overflowed'); raise end;
 on EUnderflow do begin SetStatus('Floating-point operation underflowed'); raise end;
 on EZeroDivide do begin SetStatus('Attempt to divide by zero'); raise end;
 on E: Exception do begin SetStatus(E.Message); raise end;
 end;
end;




constructor TExpressionEvaluator.Create;
begin
VariableList:=TStringList.Create;
VariableList.Sorted:=True;
VariableList.CaseSensitive:=False;
end;


procedure TExpressionEvaluator.ClearVariableList;
var I: integer;
begin
for I:=0 to VariableList.Count-1 do Dispose(Pointer(VariableList.Objects[I]));
VariableList.Clear;
end;



destructor TExpressionEvaluator.Destroy;
begin
ClearVariableList;
VariableList.Free;
inherited;
end;


end.
