unit GDIPLibrary;

interface

uses Windows, Types, Classes, Graphics, SysUtils, Math, GDIPObj, GDIPAPI,Vector, GraphicsLib,
     WinSupport, MatrixLibrary, DSupport;



type THorizAlign = (haLeft,haRight,haCenter);
type TVertAlign = (vaTop,vaBottom,vaBase);

type TTextAlign = record
 Horizontal: THorizAlign;
 Vertical: TVertAlign;
 CursorUpdate: boolean;
 end;

const DefaultTextAlign: TTextAlign =(Horizontal: haLeft; Vertical: vaTop; CursorUpdate: False);


type THybridPen = class;
     THybridBrush = class;
     THybridFont = class;

TGDPCanvas = class(TObject)
 private
  FTextAlign: TTextAlign;
  FGDIPlus: boolean;
  FCanvas: TCanvas;
  FOutlineFont: boolean;
  FLockCount: Integer;
  FTextFlags: Longint;
  FFontOpacity: single;
  FTextHint: TextRenderingHint;
  procedure SetWorldTransform(Matrix: XForm); overload;
  procedure SetWorldTransform(Matrix: TX3Matrix); overload;
  function GetSmoothMode: TSmoothingMode;
  procedure SetSmoothMode(const Value: TSmoothingMode);
  procedure DrawRoundedRectangle(X,Y,Width,Height, CEWidth, CEHeight: integer; Fill: Boolean);
  procedure DoGDIPArc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  procedure DoGDIPPie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  procedure DoGDIPText(X, Y: double; const Text: string);
  function GetFont: TFont;
  procedure SetFont(const Value: TFont);
  procedure ForceTrueType;
  procedure DoNonGDIPText(X, Y: Integer; const Text: string);
  function GetPenPos: TPoint;
  function GetPixel(X, Y: Integer): TColor;
  procedure SetPenPos(const Value: TPoint);
  procedure SetPixel(X, Y: Integer; const Value: TColor);
  function GetCanvasOrientation: TCanvasOrientation;
  function GetClipRect: TRect;
  function GetHandle: HDC;
  procedure SetHandle(const Value: HDC);
  function GetTextBoundginBox(Text: string): TGPRectF;
  procedure BeforeDraw;
  procedure AfterDraw(Close: boolean);
  procedure AdjustTextPos(Text: string; var X, Y: integer);
 protected
 public
  LocalMatrix: TX3Matrix;
  PathActive: boolean;
  GDIPath: TGPGraphicsPath;
  Graphics: TGPGraphics;
  Pen: THybridPen;
  Brush: THybridBrush;
  Font: THybridFont;
  PenPosition: TPoint;
  StatusStr: string;
  constructor Create(Canvas: TCanvas);
  destructor Destroy; override;
  procedure Ellipse(X1, Y1, X2, Y2: Integer); overload;
  procedure Ellipse(const Rect: TRect); overload;
  procedure PolyBezier(const Points: array of TPoint);
  procedure PolyBezierTo(const Points: array of TPoint);
  procedure MoveTo(X, Y: Integer);
  procedure LineTo(X, Y: Integer);
  procedure Rectangle(X1, Y1, X2, Y2: Integer); overload;
  procedure Rectangle(const Rect: TRect); overload;
  procedure RoundRect(X1, Y1, X2, Y2, X3, Y3: Integer);

  procedure Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);

  procedure Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  procedure Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
  procedure Polygon(const Points: array of TPoint);
  procedure Polyline(const Points: array of TPoint);
  procedure TextOut(X, Y: Integer; const Text: string);
  procedure TextRect(Rect: TRect; X, Y: Integer; const Text: string);
  procedure BrushCopy(const Dest: TRect; Bitmap: TBitmap; const Source: TRect; Color: TColor);
  procedure CopyRect(const Dest: TRect; Canvas: TCanvas; const Source: TRect);
  procedure Draw(X, Y: Integer; Graphic: TGraphic);
  procedure DrawFocusRect(const Rect: TRect);

  procedure FillRect(const Rect: TRect);
  procedure FloodFill(X, Y: Integer; Color: TColor; FillStyle: TFillStyle);
  procedure FrameRect(const Rect: TRect);
  procedure SetTextAlign(Flags: integer);
  procedure Lock;

  procedure Refresh;
  procedure StretchDraw(const Rect: TRect; Graphic: TGraphic);
  function TextExtent(const Text: string): TSize;
  function TextHeight(const Text: string): Integer;
  function TextWidth(const Text: string): Integer;
  function TryLock: Boolean;
  procedure Unlock;

  property ClipRect: TRect read GetClipRect;
  property Handle: HDC read GetHandle write SetHandle;
  property LockCount: Integer read FLockCount;
  property CanvasOrientation: TCanvasOrientation read GetCanvasOrientation;
  property TextFlags: Longint read FTextFlags write FTextFlags;

  property PenPos: TPoint read GetPenPos write SetPenPos;
  property Pixels[X, Y: Integer]: TColor read GetPixel write SetPixel;
  procedure BeginPath;
  procedure EndPath;
  procedure StrokeAndFillPath(Close: boolean); overload;
  procedure StrokeAndFillPath; overload;
  procedure StrokePath;

  property SmoothMode: TSmoothingMode read GetSmoothMode write SetSmoothMode;
  property GDIPlus: boolean read FGDIPlus write FGDIPlus default True;
  property Canvas: TCanvas read FCanvas;
//  property Font: TFont read GetFont write SetFont;
  property OutlineFont: boolean read FOutlineFont write FOutlineFont;
  property FontOpacity: single read FFontOpacity write FFontOpacity;
  procedure ResetTransform;
  procedure SetTransform(Matrix: TX3Matrix); overload;
  procedure SetTransform(Matrix: XForm); overload;
  procedure AddRotation(A: double);
  procedure AddScale(A: double);
  procedure SetTranslation(V: T2DVector);
  property TextHint: TextRenderingHint read FTextHint write FTextHint;
end;

 THybridFont = class(TObject)
  private
    FFont: TFont;
    FPixelsPerInch: Integer;
    FRealHeight: double;
    function GetCharset: TFontCharset;
    function GetHandle: HFont;
    function GetHeight: Integer;
    function GetName: TFontName;
    function GetPitch: TFontPitch;
    function GetSize: Integer;
    function GetStyle: TFontStyles;
    procedure SetCharset(const Value: TFontCharset);
    procedure SetColor(const Value: TColor);
    procedure SetHandle(const Value: HFont);
    procedure SetHeight(const Value: Integer);
    procedure SetName(const Value: TFontName);
    procedure SetPitch(const Value: TFontPitch);
    procedure SetSize(const Value: Integer);
    procedure SetStyle(const Value: TFontStyles);
    function GetColor: TColor;
    function GetRealSize: double;
    procedure SetRealSize(const Value: double);
    procedure SetRealHeight(const Value: double);
  protected
  public
    constructor Create(Canvas: TGDPCanvas);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent);
    property Handle: HFont read GetHandle write SetHandle;
    property PixelsPerInch: Integer read FPixelsPerInch write FPixelsPerInch;
    property RealSize: double read GetRealSize write SetRealSize;
    property RealHeight: double read FRealHeight write SetRealHeight;
  published
    property Charset: TFontCharset read GetCharset write SetCharset;
    property Color: TColor read GetColor write SetColor;
    property Height: Integer read GetHeight write SetHeight;
    property Name: TFontName read GetName write SetName;
    property Pitch: TFontPitch read GetPitch write SetPitch default fpDefault;
    property Size: Integer read GetSize write SetSize stored False;
    property Style: TFontStyles read GetStyle write SetStyle;
  end;


THybridPen = class(TGPPen)
 private
  FPen: TPen;
  FMode: TPenMode;
  function GetColor: TColor;
  function GetStyle: TPenStyle;
  function GetWidth: Integer;
  procedure SetColor(const Value: TColor);
  procedure SetMode(const Value: TPenMode);
  procedure SetStyle(const Value: TPenStyle);
  procedure SetWidth(Value: Integer);
  function GetOpacity: single;
  procedure SetOpacity(const Value: single);
 protected
 public
  constructor Create(Canvas: TGDPCanvas);
  destructor Destroy; override;
  property Color: TColor read GetColor write SetColor default clBlack;
  property Mode: TPenMode read FMode write SetMode default pmCopy;
  property Style: TPenStyle read GetStyle write SetStyle default psSolid;
  property Width: Integer read GetWidth write SetWidth default 1;
  property Opacity: single read GetOpacity write SetOpacity;
 end;


THybridBrush = class(TGPSolidBrush)
 private
  FBrush: TBrush;
  function GetStyle: TBrushStyle;
  procedure SetColor(const Value: TColor);
  procedure SetStyle(const Value: TBrushStyle);
  function GetColor: TColor;
  function GetOpacity: single;
  procedure SetOpacity(const Value: single);
 protected
 public
  constructor Create(Canvas: TGDPCanvas);
  destructor Destroy; override;
  property Color: TColor read GetColor write SetColor default clWhite;
  property Style: TBrushStyle read GetStyle write SetStyle default bsSolid;
  property Opacity: single read GetOpacity write SetOpacity;
 end;




function ColorToARGB(Color: TColor): ARGB;
procedure DrawGDPCircle(var Canvas: TGDPCanvas; Pos: TPoint; Radius: integer; Fill: boolean; RadiusColor,FillColor: TColor);

implementation



function ColorToARGB(Color: TColor): ARGB;
begin
Result:=ColorRefToARGB(Color);
end;


{ TGDPCanvas }



constructor TGDPCanvas.Create(Canvas: TCanvas);
begin
FCanvas:=Canvas;
FCanvas.Font.Name:='MS San Serif';
Graphics:=TGPGraphics.Create(Canvas.Handle);
Graphics.SetSmoothingMode(SmoothingModeAntiAlias8x8);
Pen:=THybridPen.Create(Self);
Brush:=THybridBrush.Create(Self);
Font:=THybridFont.Create(Self);
GDIPath:=TGPGraphicsPath.Create;
GDIPath.Reset;
FGDIPlus:=True;
PathActive:=False;
FOutlineFont:=False;
FFontOpacity:=1;
FTextHint:=TextRenderingHintSystemDefault;
LocalMatrix:=X3IdentityMatrix;
Windows.SetWorldTransform(FCanvas.Handle,X3ToXForm(X3IdentityMatrix));
FTextAlign:=DefaultTextAlign;
end;



destructor TGDPCanvas.Destroy;
begin
Font.Free;
GDIPath.Free;
Brush.Free;
Pen.Free;
Graphics.Free;
inherited;
end;




procedure TGDPCanvas.ForceTrueType;
{Force the Canvas to use the closest matching TrueType font}
var LogFont: TLogFont;
var FaceName: array[0..255] of char;
begin
{Get logical font info from the Delphi canvas}
if GetObject(FCanvas.Font.Handle, SizeOf(LogFont), @LogFont) <> 0 then
	begin
	{Force font to be TrueType}
	Logfont.lfOutPrecision:=OUT_TT_ONLY_PRECIS;
	FCanvas.Font.Handle:=CreateFontIndirect(LogFont);
	{Creating the handle doesn't change the FaceName, so get the face name }
	GetTextFace(FCanvas.Handle, 256, FaceName);
	{And put it in the font for use by GDCanvas}
	FCanvas.Font.Name:=FaceName;
	end;
end;



function TGDPCanvas.GetSmoothMode: TSmoothingMode;
begin
Result:=Graphics.GetSmoothingMode;
end;



procedure TGDPCanvas.SetSmoothMode(const Value: TSmoothingMode);
begin
Graphics.SetSmoothingMode(Value);
end;


function DataTypeString(B: PathPointType): string;
begin
case B of
 PathPointTypeStart: Result:='Start:';
 PathPointTypeLine: Result:='Line';
 PathPointTypeBezier: Result:='Bezier';
 PathPointTypePathTypeMask: Result:='TypeMask';
 PathPointTypeDashMode: Result:='DashMode';
 PathPointTypePathMarker: Result:='PathMarker';
 PathPointTypeCloseSubpath: Result:='Subpath';
 end;
end;

{===============================================================================}


procedure TGDPCanvas.BeforeDraw;
begin
if not PathActive then
	begin
	Graphics.ResetClip;
	SetViewportOrgEx(FCanvas.Handle,0,0,nil);
	Self.SetWorldTransform(LocalMatrix);
	end;
end;



procedure TGDPCanvas.AfterDraw(Close: boolean);
begin
if not PathActive then
	begin
	if Close and (Brush.FBrush.Style<>bsClear) then Graphics.FillPath(Brush, GDIPath);
	Graphics.DrawPath(Pen, GDIPath);
	GDIPath.Reset;
	end;
//Form1.Image1.Repaint;
end;


procedure TGDPCanvas.BeginPath;
begin
if GDIPlus then
	begin
	GDIPath.Reset;
	PathActive:=True;
	end
else Windows.BeginPath(FCanvas.Handle);
end;



procedure TGDPCanvas.EndPath;
begin
if GDIPlus then PathActive:=False
else Windows.EndPath(FCanvas.Handle);
end;

var PathVerts: array of TPoint;
var PathTypes: array of byte;

procedure SavePath(DC: HDC);
var Cnt: integer;
var I,J: integer;
var DummyPtr: pointer;
begin
DummyPtr := nil;
Cnt:=GetPath(DC, DummyPtr, DummyPtr, 0);
if Cnt<1 then exit;
SetLength(PathVerts, Cnt);
SetLength(PathTypes, Cnt);
GetPath(DC, PathVerts[0], PathTypes[0], Cnt);
end;

procedure RestorePath(DC: HDC);
begin
BeginPath(DC);
PolyDraw(DC,PathVerts[0], PathTypes[0], Length(PathVerts));
EndPath(DC);
end;


procedure TGDPCanvas.StrokeAndFillPath(Close: boolean);
{This overload allows you to chose closing the path}
begin
if GDIPlus then
	begin
	if Close then GDIPath.CloseFigure;
	Graphics.FillPath(brush, GDIPath);
	Graphics.DrawPath(Pen, GDIPath);
	GDIPath.Reset;
	end
else
	begin
	if Close then Windows.StrokeAndFillPath(FCanvas.Handle)
	else
		begin
		SavePath(FCanvas.Handle);
		Windows.FillPath(FCanvas.Handle);
		RestorePath(FCanvas.Handle);
		Windows.StrokePath(FCanvas.Handle);
		end;
	end;
end;


procedure TGDPCanvas.StrokeAndFillPath;
{This overload follows the original canvas action of closing the path}
begin
StrokeAndFillPath(True);
end;




procedure TGDPCanvas.StrokePath;
begin
if GDIPlus then
	begin
	Graphics.DrawPath(Pen, GDIPath);
	GDIPath.Reset;
	end
else Windows.StrokePath(FCanvas.Handle);
end;

{===============================================================================}


procedure TGDPCanvas.SetWorldTransform(Matrix: XForm);
var GPMatrix: TGPMatrix;
begin
if GDIPlus then
	begin
	GPMatrix:=TGPMatrix.Create;
	Graphics.GetTransform(GPMatrix);
	GPMatrix.SetElements(Matrix.em11, Matrix.em12, Matrix.em21, Matrix.em22, Matrix.edx, Matrix.edy);
	Graphics.SetTransform(GPMatrix);
	GPMatrix.Free;
	end
else
	begin
	SetGraphicsMode(FCanvas.Handle, GM_ADVANCED);
	Windows.SetTextAlign(FCanvas.Handle, TA_BASELINE);
	Windows.SetWorldTransform(FCanvas.Handle,Matrix);
	end;
end;




function ParseTextAlign(Alignment: integer): TTextAlign;
{Convert Text-Align integerin a friendlier format}
begin
{Set text align}
Result.Horizontal:=haLeft;
Result.Vertical:=vaTop;
Result.CursorUpdate:=False;

{Update group}
Result.CursorUpdate:=IsBitSet(Alignment,TA_UPDATECP);
case Alignment and TA_UPDATECP of
 TA_NOUPDATECP: Result.CursorUpdate:=False;	{0}
 TA_UPDATECP: Result.CursorUpdate:=True;	{1}
 end;

{Horizontal group}
case Alignment and TA_CENTER of
 TA_LEFT: Result.Horizontal:=haLeft;			{0}
 TA_RIGHT: Result.Horizontal:=haRight;		{2 - 00010}
 TA_CENTER: Result.Horizontal:=haCenter;	{6 - 00110}
 end;

{Vertical group}
case Alignment and TA_BASELINE of
 TA_TOP: Result.Vertical:=vaTop;			{0}
 TA_BOTTOM: Result.Vertical:=vaBottom;		{8 - 01000}
 TA_BASELINE: Result.Vertical:=vaBase;		{24 -11000 }
 end;
end;




procedure TGDPCanvas.SetWorldTransform(Matrix: TX3Matrix);
begin
SetWorldTransform(X3ToXForm(Matrix));
end;



procedure TGDPCanvas.PolyBezier(const Points: array of TPoint);
begin
if GDIPlus then
	begin
	BeforeDraw;
	GDIPath.AddBeziers(PGPPoint(@Points),High(Points) + 1);
	AfterDraw(False);
	end
else FCanvas.PolyBezier(Points);
end;



procedure TGDPCanvas.PolyBezierTo(const Points: array of TPoint);
var Pnts: array of TPoint;

	procedure AddFirstPoint;
	var I: integer;
	begin
	SetLength(Pnts,Length(Points)+1);
	Pnts[0]:=PenPosition;
	for I:=0 to High(Points) do Pnts[I+1]:=Points[I];
	end;

begin
if GDIPlus then
	begin
	BeforeDraw;
	AddFirstPoint;
	GDIPath.AddBeziers(PGPPoint(Pnts),Length(Pnts));
	PenPosition:=Points[High(Points)];
	AfterDraw(False);
	end
else FCanvas.PolyBezierTo(Points);
end;





procedure TGDPCanvas.Ellipse(X1, Y1, X2, Y2: Integer);
var W,H: integer;
begin
W:=X2 - X1;
H:=Y2 - Y1;
if GDIPlus then
	begin
	BeforeDraw;
	if Brush.FBrush.Style<>bsClear then Graphics.FillEllipse(Brush,  X1, Y1, W, H);
	GDIPath.AddEllipse(X1, Y1, W, H);
	AfterDraw(True);
	end
else FCanvas.Ellipse(X1, Y1, X2, Y2);
end;



procedure TGDPCanvas.Ellipse(const Rect: TRect);
begin
Ellipse(Rect.Left, Rect.Top, Rect.Right, Rect.Bottom);
end;



procedure TGDPCanvas.LineTo(X, Y: Integer);
begin
if GDIPlus then
	begin
	BeforeDraw;
	GDIPath.AddLine(PenPosition.X,PenPosition.Y,X,Y);
	PenPosition:=Point(X,Y);
	AfterDraw(False);
	end
else FCanvas.LineTo(X,Y);
end;


procedure TGDPCanvas.MoveTo(X, Y: Integer);
begin
if GDIPlus then
	begin
	PenPosition:=Point(X,Y);
	end
else FCanvas.MoveTo(X,Y);
end;


function MakeGPRect(X,Y,W,H: single): TGPRectF;
begin
Result.X:=X;
Result.Y:=Y;
Result.Width:=W;
Result.Height:=H;
end;


procedure TGDPCanvas.Rectangle(const Rect: TRect);
var X,Y,W,H: integer;
begin
if GDIPlus then
	begin
	BeforeDraw;
	X:=Rect.Left; Y:=Rect.Top;
	W:=Rect.Right-Rect.Left-1; H:=Rect.Bottom-Rect.Top-1;
	if Brush.FBrush.Style<>bsClear then Graphics.FillRectangle(Brush, X,Y,W,H);
	GDIPath.AddRectangle(MakeGPRect(X,Y,W,H));
	AfterDraw(True);
	end
else FCanvas.Rectangle(Rect);
end;


procedure TGDPCanvas.Rectangle(X1, Y1, X2, Y2: Integer);
begin
Rectangle(Rect(X1,Y1,X2,Y2));
end;



procedure TGDPCanvas.DrawRoundedRectangle(X,Y,Width,Height, CEWidth, CEHeight: integer; Fill: Boolean);
{Duplicate regular GDI Rounded Rectange}
begin
BeforeDraw;
Dec(Width); Dec(Height);
GDIPath.AddLine(X + (CEWidth div 2), Y, X + Width - (CEWidth), Y);
GDIPath.AddArc(X + Width - (CEWidth), Y, CEWidth, CEHeight, 270, 90);
GDIPath.AddLine(X + Width, Y + (CEHeight div 2), X + Width, Y + Height - (CEHeight));
GDIPath.AddArc(X + Width - (CEWidth), Y + Height - (CEHeight), CEWidth, CEHeight, 0, 90);
GDIPath.AddLine(X + Width - (CEWidth), Y + Height, X + (CEWidth div 2), Y + Height);
GDIPath.AddArc(X, Y + Height - (CEHeight), CEWidth, CEHeight, 90, 90);
GDIPath.AddLine(X, Y + Height - (CEHeight), X, Y + (CEHeight div 2));
GDIPath.AddArc(X, Y, CEWidth, CEHeight, 180, 90);
GDIPath.CloseFigure;
AfterDraw(True);
end;



procedure TGDPCanvas.RoundRect(X1, Y1, X2, Y2, X3, Y3: Integer);
var W,H: integer;
begin
if GDIPlus then
	begin
	W:=X2-X1; H:=Y2-Y1;
	if Brush.FBrush.Style<>bsClear then DrawRoundedRectangle(X1,Y1,W,H,X3,Y3,True);
	DrawRoundedRectangle(X1,Y1,W,H,X3,Y3,False);
	end
else FCanvas.RoundRect(X1, Y1, X2, Y2, X3, Y3);
end;


procedure TGDPCanvas.Polygon(const Points: array of TPoint);
begin
if GDIPlus then
	begin
	BeforeDraw;
	if Brush.FBrush.Style<>bsClear then Graphics.FillPolygon(Brush,PGPPoint(@Points),High(Points) + 1);
	GDIPath.AddPolygon(PGPPoint(@Points),High(Points) + 1);
	AfterDraw(True);
	end
else FCanvas.Polygon(Points);
end;



procedure TGDPCanvas.Polyline(const Points: array of TPoint);
begin
if GDIPlus then
	begin
	BeforeDraw;
	GDIPath.AddLines(PGPPoint(@Points),High(Points) + 1);
	AfterDraw(False);
	end
else FCanvas.Polyline(Points);
end;



procedure TGDPCanvas.DoGDIPArc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
var Start,Stop,Sweep: double;
var R1,R2,P1,P2,C: T2DVector;
begin
R1:=MakeVector2D(X1,Y1);
R2:=MakeVector2D(X2,Y2);
C:=ScalarDivide2D(VectorAdd2D(R1,R2),2);
P1:=VectorSubtract2D(MakeVector2D(X3,Y3),C);
P2:=VectorSubtract2D(MakeVector2D(X4,Y4),C);

{GDI Pie is counter-clockwise and GDI+ is clockwise, so start with P2}
Start:=RadToDeg(SignedVectorAngle2D(MakeVector2D(1,0),P2));
{Normalize from -180 to +180 to 0-360}
while Start<0 do Start:=Start+360;

Stop:=RadToDeg(SignedVectorAngle2D(MakeVector2D(1,0),P1));
{The difference }
Sweep:=Stop-Start;
{Normalize from -180 to +180 to 0-360}
while Sweep<0 do Sweep:=Sweep+360;
GDIPath.AddArc(X1, Y1, X2-X1-1, Y2-Y1-1, Start, Sweep);
end;



procedure TGDPCanvas.DoGDIPPie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
var Start,Stop,Sweep: double;
var R1,R2,P1,P2,C: T2DVector;
begin
R1:=MakeVector2D(X1,Y1);
R2:=MakeVector2D(X2,Y2);
C:=ScalarDivide2D(VectorAdd2D(R1,R2),2);
P1:=VectorSubtract2D(MakeVector2D(X3,Y3),C);
P2:=VectorSubtract2D(MakeVector2D(X4,Y4),C);

{GDI Pie is counter-clockwise and GDI+ is clockwise, so start with P2}
Start:=RadToDeg(SignedVectorAngle2D(MakeVector2D(1,0),P2));
{Normalize from -180 to +180 to 0-360}
while Start<0 do Start:=Start+360;

Stop:=RadToDeg(SignedVectorAngle2D(MakeVector2D(1,0),P1));
{The difference }
Sweep:=Stop-Start;
{Normalize from -180 to +180 to 0-360}
while Sweep<0 do Sweep:=Sweep+360;
GDIPath.AddPie(X1, Y1, X2-X1-1, Y2-Y1-1, Start, Sweep);
end;



procedure TGDPCanvas.Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
begin
if GDIPlus then
	begin
	BeforeDraw;
	DoGDIPArc(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
	AfterDraw(False);
	end
else FCanvas.Arc(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
end;



procedure TGDPCanvas.Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
begin
if GDIPlus then
	begin
	BeforeDraw;
       	DoGDIPArc(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
	GDIPath.CloseFigure;
	AfterDraw(True);
	end
else FCanvas.Chord(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
end;



procedure TGDPCanvas.Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4: Integer);
begin
if GDIPlus then
	begin
	BeforeDraw;
	DoGDIPPie(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
	AfterDraw(True);
	end
else FCanvas.Pie(X1, Y1, X2, Y2, X3, Y3, X4, Y4);
end;



type TFontMetrics = record
 PlusEmHeight, PlusAscent, FontSize: double;
 PlusDescent, PlusSpacing: double;
 GDIHeight,GDIAscent: integer;
 GDIDescent,GDIInternalLeading,GDIExternalLeading: integer;
 end;





procedure TGDPCanvas.DoGDIPText(X, Y: double; const Text: string);
{Note: GDI+ doesn't support OpenType fonts under Win7, but does under win 8/10}
{So it is probably not worth figuring out a work around}
var FontFamily: TGPFontFamily;
var Format : TGPStringFormat;
var Origin: TGPPointF;
var Style: TFontStyle;
var FontSize: double;
var P: double;
var FontMetrics: TFontMetrics;





	function GetFontMetrics: TFontMetrics;
	var TM: TTextMetric;
	var B: boolean;
	begin
	Result.PlusEmHeight:= FontFamily.GetEmHeight(Style);
	Result.PlusAscent:=  FontFamily.GetCellAscent(style);
	Result.PlusDescent:=  FontFamily.GetCellDescent(style);
	Result.PlusSpacing:= FontFamily.GetLineSpacing(style);

	B:=GetTextMetrics(Canvas.Handle,TM);
	Result.GDIHeight:=TM.tmHeight;
	Result.GDIAscent:=TM.tmAscent;
	Result.GDIDescent:=TM.tmDescent;
	Result.GDIInternalLeading:=TM.tmInternalLeading;
	Result.GDIExternalLeading :=TM.tmExternalLeading;
	end;



	function GetFontStyle: TFontStyle;
	begin
	Result:=FontStyleRegular;
	if fsBold in FCanvas.Font.Style then Result:=Result or FontStyleBold;
	if fsItalic in FCanvas.Font.Style then Result:=Result or FontStyleItalic;
	if fsUnderline in FCanvas.Font.Style then Result:=Result or FontStyleUnderline;
	if fsStrikeOut in FCanvas.Font.Style then Result:=Result or FontStyleStrikeout;
	end;



	function CalcPadding: Single;
	var BoundBox: TGPRectF;
	var WS: WideString;
	var w1, w2: Single;
	var LayoutRect: TGPRectF;
	var GPFont: TGPFont;
	begin
	GPFont:=TGPFont.Create(FCanvas.Handle);
	try
	LayoutRect:=MakeRect(0.0, 0, 10000, 10000);
	{Get the width of one character}
	WS:='M';
	Graphics.MeasureString(WS, Length(WS), GPFont, LayoutRect, nil, BoundBox, nil, nil);
	w1:= BoundBox.Width;

	{Get the width of two characters}
	WS:= 'MM';
	Graphics.MeasureString(WS, Length(WS), GPFont, LayoutRect, nil, BoundBox, nil, nil);
	w2:= BoundBox.Width;

	{ w2 - w1 = width of one character}
	{ w1 - (w2-w1) = width of padding on both sides}
	Result:= (w1 - (w2 - w1))/2;
	finally GPFont.Free; end;
	end;


	function GetTextBoundary(var Tight,Loose,GDI: TGPRectF): TGPRectF;
	{Get bounding box of text using three different methods}
	var Path: TGPGraphicsPath;
	var Font: TGPFont;
	var O: TGPPointF;
	var Size: TSize;
	begin
	{Get tighly wrapped box}
	Path:=TGPGraphicsPath.Create;
	try
	Path.AddString(Text,Length(Text),FontFamily,Style,FontSize, Origin, Format);
	Path.GetBounds(Tight);
	finally Path.Free; end;

	{Get Box from GDI+ measurements}
	Font:=TGPFont.Create(FCanvas.Handle);
	try
	O:=MakePoint(X,Y);
	Graphics.MeasureString(Text,Length(Text),Font,MakePoint(X,Y),Format,Loose);
	finally Font.Free; end;

	{Get Box from GDI measurements}
	Size:=FCanvas.TextExtent(Text);
	GDI:=MakeRect(X,Y,Size.cx,Size.cy);
	end;



	function GetTextRect: TGPRectF;
	var Tight,Loose,GDI: TGPRectF;
	var W: double;
	begin
	GetTextBoundary(Tight,Loose,GDI);
	Result.X:=Min(Tight.X,X);

	Result.Y:=Min(Loose.Y,Tight.Y);
	Result.Width:=Tight.Width+(Tight.X-Result.X);

	Result.Height:=Max(GDI.Height+(Result.Y-GDI.Y),Tight.Height);
	if OutlineFont then
		begin
		W:=Pen.Width;
		Result.X:=Result.X-(W-1);
		Result.Y:=Result.Y-W;
		Result.Width:=Result.Width+W*2+1;
		Result.Height:=Result.Height+W*2;
		end;
	end;



	procedure HandleTextRectangle(Fill: boolean);
	var R: TGPRectF;
	begin
	R:=GetTextRect;
	if Fill then Graphics.FillRectangle(Brush,R)
	else Graphics.DrawRectangle(Pen,R);
	end;



	procedure DoOutlineText;
	{With outline text, the pen sets outline and brush sets fill color}
	{ Outer Text = Pen.Color }
	{ Inner Text = Font.Color }
	{ Background = Brush.Color }
	var C: TColor;
	var O: single;
	begin
	SetViewportOrgEx(FCanvas.Handle,0,0,nil);
	C:=Brush.Color;
	O:=Brush.Opacity;
	FCanvas.Lock;
	try
	Brush.Color:=Font.Color;
	Brush.Opacity:=FontOpacity;
	GDIPath.AddString(Text,Length(Text),FontFamily,Style,FontSize, Origin, Format);
	Graphics.FillPath(Brush, GDIPath);
	Graphics.DrawPath(Pen, GDIPath);
	Brush.Color:=C;
	Brush.Opacity:=O;
	finally
	 GDIPath.Reset;
	 FCanvas.Unlock;
	 end;
	end;


	procedure DoRegularText;
	var GPFont: TGPFont;
	var C: TColor;
	var O: single;
	begin
	GPFont:=TGPFont.Create(FontFamily,FontSize,Style,UnitPixel);
	try
	C:=Brush.Color;
	O:=Brush.Opacity;
	Brush.Color:=Font.Color;
	Brush.Opacity:=FontOpacity;
	Graphics.DrawString(Text,Length(Text), GPFont, Origin, Brush);
	Brush.Color:=C;
	Brush.Opacity:=O;
	finally GPFont.Free; end;
	end;



begin
Lock;
try
Style:=GetFontStyle;
ForceTrueType;
FontFamily:=TGPFontFamily.Create(Canvas.Font.Name);
try
Format:=TGPStringFormat.Create;
try
Format.GenericTypographic;
Graphics.SetTextRenderingHint(TextHint);

FontMetrics:=GetFontMetrics;
FontSize:=(FontMetrics.GDIAscent/ ((FontMetrics.PlusAscent/ FontMetrics.PlusEmHeight) * Font.RealHeight)) * Font.RealHeight;



P:=CalcPadding;

{Old method, may be slightly better GDI:GDI+ match but only has integer sizes}
//FontSize:=(FontMetrics.GDIAscent/ ((FontMetrics.PlusAscent/ FontMetrics.PlusEmHeight) * Font.RealHeight)) * Font.RealHeight;
{Based on this article: https://docs.microsoft.com/en-us/windows/win32/gdiplus/-gdiplus-drawing-text-use }
FontSize:=Font.RealHeight/((FontMetrics.PlusAscent + FontMetrics.PlusDescent)/FontMetrics.PlusEmHeight);

Origin:=MakePoint(X-P,Y);
if Brush.FBrush.Style<>bsClear then HandleTextRectangle(True);
if OutlineFont then DoOutlineText else DoRegularText;

finally Format.Free; end;
finally FontFamily.Free; end;
finally Unlock; end;
end;



procedure TGDPCanvas.DoNonGDIPText(X, Y: Integer; const Text: string);
{ Outline text color scheme}
{ Outer Text = Pen.Color }
{ Inner Text = Font.Color }
{ Background = Brush.Color }
var TM: TTextMetric;
begin
if OutlineFont then DrawOutlineText(Canvas,X,Y,Text)
else Canvas.TextOut(X, Y, Text);

GetTextMetrics(Canvas.Handle,TM);
end;



procedure TGDPCanvas.AdjustTextPos(Text: string; var X,Y: integer);
{Adjust GDI+ text postion to match GDI alignment}
var TM: TOutlineTextmetric;
var Width: integer;
begin
ForceTrueType;
Width:=FCanvas.TextWidth(Text);
GetOutlineTextMetrics(FCanvas.Handle,SizeOf(TM),@TM);
case FTextAlign.Vertical of
 vaTop: Y:=Y;
 vaBottom: Y:=Y-(TM.otmMacAscent-TM.otmMacDescent);
 vaBase: Y:=Y-TM.otmMacAscent;
 end;
case FTextAlign.Horizontal of
 haLeft: X:=X;
 haRight: X:=Round(X-Width);
 haCenter: X:=X-Round(Width/2);
 end;
end;



procedure TGDPCanvas.TextOut(X, Y: Integer; const Text: string);
begin
StatusStr:='';
if GDIPlus then
	begin
	AdjustTextPos(Text,X,Y);
	DoGDIPText(X,Y,Text);
	end
else DoNonGDIPText(X, Y, Text);
end;




procedure TGDPCanvas.TextRect(Rect: TRect; X, Y: Integer;
  const Text: string);
var Region: TGPRegion;
begin
if GDIPlus then
	begin
	Region:=TGPRegion.Create(MakeRect(Rect.Left,Rect.Top,Rect.Right-Rect.Left,Rect.Bottom-Rect.Top));
	try
	Graphics.SetClip(Region, CombineModeReplace);
	AdjustTextPos(Text,X,Y);
	DoGDIPText(X,Y,Text);
	finally Region.Free; end;
	end
else
	begin
	Canvas.TextRect(Rect, X, Y, Text);
	end;
end;


{------------------------------------------------------------------------------}






function TGDPCanvas.GetFont: TFont;
begin
Result:=FCanvas.Font;
end;



procedure TGDPCanvas.SetFont(const Value: TFont);
begin
FCanvas.Font:=Value;
end;




function TGDPCanvas.GetPenPos: TPoint;
begin
if GDIPlus then Result:=PenPosition
else Result:=FCanvas.PenPos;
end;



function TGDPCanvas.GetPixel(X, Y: Integer): TColor;
begin
Result:=FCanvas.Pixels[X,Y];
end;



procedure TGDPCanvas.SetPenPos(const Value: TPoint);
begin
if GDIPlus then PenPosition:=Value
else FCanvas.PenPos:=Value;
end;



procedure TGDPCanvas.SetPixel(X, Y: Integer; const Value: TColor);
begin
FCanvas.Pixels[X,Y]:=Value;
end;



procedure TGDPCanvas.BrushCopy(const Dest: TRect; Bitmap: TBitmap;
  const Source: TRect; Color: TColor);
begin
FCanvas.BrushCopy(Dest, Bitmap, Source, Color);
end;



procedure TGDPCanvas.CopyRect(const Dest: TRect; Canvas: TCanvas;
  const Source: TRect);
begin
FCanvas.CopyRect(Dest, Canvas, Source);
end;



procedure TGDPCanvas.Draw(X, Y: Integer; Graphic: TGraphic);
begin
FCanvas.Draw(X, Y, Graphic);
end;



procedure TGDPCanvas.DrawFocusRect(const Rect: TRect);
begin
FCanvas.DrawFocusRect(Rect);
end;



procedure TGDPCanvas.FillRect(const Rect: TRect);
begin
FCanvas.FillRect(Rect);
end;



procedure TGDPCanvas.FloodFill(X, Y: Integer; Color: TColor;
  FillStyle: TFillStyle);
begin
FCanvas.FloodFill(X, Y, Color, FillStyle);
end;



procedure TGDPCanvas.FrameRect(const Rect: TRect);
begin
FCanvas.FrameRect(Rect);
end;



function TGDPCanvas.GetCanvasOrientation: TCanvasOrientation;
begin
Result:=FCanvas.CanvasOrientation;
end;



function TGDPCanvas.GetClipRect: TRect;
begin
Result:=FCanvas.ClipRect;
end;



function TGDPCanvas.GetHandle: HDC;
begin
Result:=FCanvas.Handle;
end;



procedure TGDPCanvas.Lock;
begin
FCanvas.Lock;
end;



procedure TGDPCanvas.Refresh;
begin
FCanvas.Refresh;
end;



procedure TGDPCanvas.SetHandle(const Value: HDC);
begin
FCanvas.Handle:=Value;
end;



procedure TGDPCanvas.StretchDraw(const Rect: TRect; Graphic: TGraphic);
begin
FCanvas.StretchDraw(Rect, Graphic);
end;



function TGDPCanvas.GetTextBoundginBox(Text: string): TGPRectF;
var LayoutRect: TGPRectF;
var GPFont: TGPFont;
begin
GPFont:=TGPFont.Create(FCanvas.Handle);
try
LayoutRect:=MakeRect(0.0, 0, 10000, 10000);
Graphics.MeasureString(Text, Length(Text), GPFont, LayoutRect, nil, Result, nil, nil);
finally GPFont.Free; end;
end;



function TGDPCanvas.TextExtent(const Text: string): TSize;
var R: TGPRectF;
begin
if GDIPlus then
	begin
	R:=GetTextBoundginBox(Text);
	Result.cx:=Round(R.Width);
	Result.cy:=Round(R.Height);
	end
else Result:=FCanvas.TextExtent(Text);
end;



function TGDPCanvas.TextHeight(const Text: string): Integer;
var R: TGPRectF;
begin
if GDIPlus then
	begin
	R:=GetTextBoundginBox(Text);
	Result:=Round(R.Height);
	end
else Result:=FCanvas.TextHeight(Text);
end;



function TGDPCanvas.TextWidth(const Text: string): Integer;
var R: TGPRectF;
begin
if GDIPlus then
	begin
	R:=GetTextBoundginBox(Text);
	Result:=Round(R.Width);
	end
else Result:=FCanvas.TextWidth(Text);
end;



function TGDPCanvas.TryLock: Boolean;
begin
Result:=FCanvas.TryLock;
end;



procedure TGDPCanvas.Unlock;
begin
FCanvas.Unlock;
end;



procedure TGDPCanvas.SetTransform(Matrix: TX3Matrix);
begin
LocalMatrix:=Matrix;
SetWorldTransform(LocalMatrix);
end;


procedure TGDPCanvas.SetTransform(Matrix: XForm);
begin
LocalMatrix:=XFormToX3(Matrix);
SetWorldTransform(LocalMatrix);
end;


procedure TGDPCanvas.ResetTransform;
begin
LocalMatrix:=X3IdentityMatrix;
end;


procedure TGDPCanvas.AddRotation(A: double);
{Add rotation in degrees}
begin
ResetTransform;
AddX3Rotation(LocalMatrix,A);
SetWorldTransform(LocalMatrix);
end;


procedure TGDPCanvas.SetTranslation(V: T2DVector);
begin
AddX3Translation(LocalMatrix,V);
SetWorldTransform(LocalMatrix);
end;




procedure TGDPCanvas.AddScale(A: double);
{Add scaling}
begin
AddX3Scale(LocalMatrix,MakeVector2D(A,A));
SetWorldTransform(LocalMatrix);
end;


procedure TGDPCanvas.SetTextAlign(Flags: integer);
begin
Windows.SetTextAlign(FCanvas.Handle,Flags);
FTextAlign:=ParseTextAlign(Flags);
end;


{ THybridPen }

constructor THybridPen.Create(Canvas: TGDPCanvas);
begin
inherited Create(ColorToARGB(clBlack),1);
FPen:=Canvas.FCanvas.Pen;
FPen.Color:=clBlack;
end;



destructor THybridPen.Destroy;
begin
inherited;
end;


{-------------- Property handlers ---------------------------------------------}


function THybridPen.GetColor: TColor;
begin
Result:=FPen.Color;
end;



procedure THybridPen.SetColor(const Value: TColor);
var C: cardinal;
var O: single;
begin
FPen.Color:=Value;
O:=Opacity;
C:=ColorToARGB(Value);
inherited SetColor(C);
Opacity:=O;
end;



function THybridPen.GetStyle: TPenStyle;
begin
{GDI+ doesn't a Style property}
Result:=FPen.Style;
end;



procedure THybridPen.SetStyle(const Value: TPenStyle);
begin
FPen.Style:=Value;
{GDI+ doesn't have a Style property so fake it}
Self.Opacity:=1;
case Value of
 psSolid: SetDashStyle(DashStyleSolid);
 psDash: SetDashStyle(DashStyleDash);
 psDot: SetDashStyle(DashStyleDot);
 psDashDot: SetDashStyle(DashStyleDashDot);
 psDashDotDot: SetDashStyle(DashStyleDashDotDot);
 //SetDashStyle(DashStyleCustom);
 psClear: Self.Opacity:=0;
 end;
end;



function THybridPen.GetWidth: Integer;
begin
Result:=Round(inherited GetWidth);
end;



procedure THybridPen.SetWidth(Value: Integer);
begin
//if Value<1 then Value:=1;
inherited SetWidth(Value);
FPen.Width:=Value;
end;



procedure THybridPen.SetMode(const Value: TPenMode);
begin
{GDI+ doesn't a mode property}
FPen.Mode:=Value;
end;


function THybridPen.GetOpacity: single;
var C: cardinal;
begin
inherited GetColor(C);
Result:=GetAlpha(C) / 255;
end;


procedure THybridPen.SetOpacity(const Value: single);
var C: cardinal;
begin
inherited GetColor(C);
C:=(C and $FFFFFF) or (round(Value * 255) shl AlphaShift);
inherited SetColor(C);
end;



{ THybridBrush }


constructor THybridBrush.Create(Canvas: TGDPCanvas);
begin
inherited Create(ColorToARGB(clWhite));
FBrush:=Canvas.FCanvas.Brush;
end;


destructor THybridBrush.Destroy;
begin
inherited;
end;






{-------------- Property handlers ---------------------------------------------}


function THybridBrush.GetColor: TColor;
begin
Result:=FBrush.Color;
end;




procedure THybridBrush.SetColor(const Value: TColor);
var C: cardinal;
var O: single;
begin
FBrush.Color:=Value;
O:=Opacity;
C:=ColorToARGB(Value);
inherited SetColor(C);
Opacity:=O;
end;



function THybridBrush.GetOpacity: single;
var C: cardinal;
begin
inherited GetColor(C);
Result:=GetAlpha(C) / 255;
end;


procedure THybridBrush.SetOpacity(const Value: single);
var C: cardinal;
begin
inherited GetColor(C);
C:=(C and $FFFFFF) or (round(Value * 255) shl AlphaShift);
inherited SetColor(C);
end;



function THybridBrush.GetStyle: TBrushStyle;
begin
{GDI+ doesn't a Style property}
Result:=FBrush.Style;
end;


procedure THybridBrush.SetStyle(const Value: TBrushStyle);
begin
{GDI+ doesn't a Style property}
FBrush.Style:=Value;
end;


{ THybridFont }

procedure THybridFont.Assign(Source: TPersistent);
begin
inherited;
FFont.Assign(Source);
end;


constructor THybridFont.Create(Canvas: TGDPCanvas);
begin
FFont:=Canvas.FCanvas.Font;
end;

destructor THybridFont.Destroy;
begin
inherited;
end;

function THybridFont.GetCharset: TFontCharset;
begin
Result:=FFont.Charset;
end;

function THybridFont.GetColor: TColor;
begin
Result:=FFont.Color;
end;

function THybridFont.GetHandle: HFont;
begin
Result:=FFont.Handle;
end;

function THybridFont.GetHeight: Integer;
begin
Result:=FFont.Height;
end;

function THybridFont.GetName: TFontName;
begin
Result:=FFont.Name;
end;

function THybridFont.GetPitch: TFontPitch;
begin
Result:=FFont.Pitch;
end;


function THybridFont.GetRealSize: double;
begin
Result := -(FRealHeight * 72 / FPixelsPerInch);
end;


procedure THybridFont.SetRealSize(const Value: double);
begin
FRealHeight := -(Value * FPixelsPerInch / 72);
FFont.Height:=Round(FRealHeight);
end;


procedure THybridFont.SetRealHeight(const Value: double);
begin
FRealHeight := Value;
FFont.Height:=Round(FRealHeight);
end;



function THybridFont.GetSize: Integer;
begin
Result:=FFont.Size;
end;

function THybridFont.GetStyle: TFontStyles;
begin
Result:=FFont.Style;
end;

procedure THybridFont.SetCharset(const Value: TFontCharset);
begin
FFont.Charset:=Value;
end;

procedure THybridFont.SetColor(const Value: TColor);
begin
FFont.Color:=Value;
end;

procedure THybridFont.SetHandle(const Value: HFont);
begin
FFont.Handle:=Value;
end;

procedure THybridFont.SetHeight(const Value: Integer);
begin
FFont.Height:=Value;
RealHeight:=Value;
end;

procedure THybridFont.SetName(const Value: TFontName);
begin
FFont.Name:=Value;
end;

procedure THybridFont.SetPitch(const Value: TFontPitch);
begin
FFont.Pitch:=Value;
end;

procedure THybridFont.SetSize(const Value: Integer);
begin
FFont.Size:=Value;
RealSize:=Value;
end;

procedure THybridFont.SetStyle(const Value: TFontStyles);
begin
FFont.Style:=Value;
end;




{==============================================================================}

procedure DrawGDPCircle(var Canvas: TGDPCanvas; Pos: TPoint; Radius: integer; Fill: boolean; RadiusColor,FillColor: TColor);
{Draw circle of specified Pos, with specified Radius, and Color}
var Rect: TGPRectF;
var X,Y,W,H: single;
begin
Canvas.Pen.Color:=ColorToARGB(RadiusColor);
Canvas.Brush.Color:=ColorToARGB(FillColor);
X:=Pos.X-Radius;
Y:=Pos.Y-Radius;
W:=Radius*2; H:=Radius*2;
Rect:=MakeRect(X,Y,W,H);
if Fill then Canvas.Graphics.FillEllipse(Canvas.Brush,  Rect);
Canvas.Graphics.DrawEllipse(Canvas.Pen, Rect);
end;

end.
