unit FireworksLib;

interface

uses Windows,Types,ExtCtrls,SysUtils,Classes,Graphics,Math,
   DSupport,BitmapLib,HugeArry,Vector;


{Structure holding information for one Rocket}

type TRocketFuel = record
 BurnTime: double;
 Color: TColor;
 end;

type TFuelArray = array [0..5] of TRocketFuel;

type TPayload = record
 Color: TColor;
 BombFuel: TFuelArray;
 Blast3D: boolean;
 Stars: integer;
 FuseTime: double;
 Randomness: double;
 ExplosionVelocity: double;
 ExplosionRandomness: double;
 end;

type TLaunchParams = record
 ThrustForce: double;
 Angle: double;
 DoRandom: boolean;
 end;


type TRocket = record
 Name: string[128];
 Enabled: boolean;
 Mass: double;
 Pos,Vel,Acc,Thrust: T3DVector;
 LaunchParams: TLaunchParams;
 Radius: double;
 Color: TColor;
 Bounces: integer;
 Randomness: double;
 DeathTime: double;
 FuelArray: TFuelArray;
 PayLoad: TPayLoad;
 end;

type PRocket = ^TRocket;



{----------------- Standard Rockets ------------------------------------------}


const NoBurn: TFuelArray = ((BurnTime: 3; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed));

const RedYellowFuel: TFuelArray = ((BurnTime: 0.6; Color: $0000FF),
				   (BurnTime: 0.6; Color: $0033FF),
				   (BurnTime: 0.6; Color: $0066FF),
				   (BurnTime: 0.6; Color: $0099FF),
				   (BurnTime: 0.6; Color: $00CCFF),
				   (BurnTime: 0.6; Color: $00FFFF));

const YellowGreenFuel: TFuelArray = ((BurnTime: 0.6; Color: $FFFF),
				   (BurnTime: 0.6; Color: $E5A0),
				   (BurnTime: 0.6; Color: $CC51),
				   (BurnTime: 0.6; Color: $B211),
				   (BurnTime: 0.6; Color: $1E9900),
				   (BurnTime: 0.6; Color: $408000));

const StarRocket: TRocket = (Mass:0.05; Pos:(X:0;Y:0);  Vel:(X:0;Y:0);  Acc:(X:0;Y:0); Thrust:(X:0;Y:0);
			     LaunchParams: (ThrustForce: 0; Angle: 0; DoRandom: False);
			     Radius:0.05; Color: $FF80FF;
			     Bounces: 0; Randomness: 2; DeathTime: 10;
			     FuelArray:((BurnTime: 3; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed))
			    );


const GlitterRocket: TRocket = (Mass:0.0001; Pos:(X:0;Y:0);  Vel:(X:0;Y:0);  Acc:(X:0;Y:0); Thrust:(X:0;Y:0);
			     LaunchParams: (ThrustForce: 0; Angle: 0; DoRandom: False);
			     Radius:0.01; Color: clYellow;
			     Bounces: 0; Randomness: 2; DeathTime: 10;
			     FuelArray:((BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed))
			    );



const BasePayload: TPayload = (
			      Color: $FF00FF;
			      BombFuel: ((BurnTime: 0.6; Color: $FF0000),
				   (BurnTime: 0.6; Color: $FF0033),
				   (BurnTime: 0.6; Color: $FF0066),
				   (BurnTime: 0.6; Color: $FF0099),
				   (BurnTime: 0.6; Color: $FF00CC),
				   (BurnTime: 0.6; Color: $FF00FF));
			      Blast3D: True;
			      Stars: 35;
			      FuseTime: 5.5;
			      Randomness: 1;
			      ExplosionVelocity: 30;
			      ExplosionRandomness: 2;
			      );


const BasicRocket: TRocket = (Name:'Basic'; Enabled: True; Mass:1; Pos:(X:200;Y:2);  Vel:(X:0;Y:0);  Acc:(X:0;Y:0); Thrust:(X:0;Y:88);
			     LaunchParams: (ThrustForce: 88; Angle: 8; DoRandom: True);
			     Radius:0.1; Color: clBlue;
			     Bounces: 0; Randomness: 2; DeathTime: 10;
			     FuelArray:((BurnTime: 1; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed),
				   (BurnTime: 0; Color: clRed));
			     Payload:(
			      Color: $FF00FF;
			      BombFuel: ((BurnTime: 0.6; Color: $FF0000),
				   (BurnTime: 0.6; Color: $FF0033),
				   (BurnTime: 0.6; Color: $FF0066),
				   (BurnTime: 0.6; Color: $FF0099),
				   (BurnTime: 0.6; Color: $FF00CC),
				   (BurnTime: 0.6; Color: $FF00FF));
			      Blast3D: True;
			      Stars: 35;
			      FuseTime: 5.5;
			      Randomness: 1;
			      ExplosionVelocity: 30;
			      ExplosionRandomness: 2;
			      );
			    );



{Define Dynamic array object of Rockets for modeling and tracking all rockets}

type TRocketArray = array of TRocket;


type TRocketList = class(TObject)
 private
  function GetValue(Inx: integer): TRocket;
  procedure SetValue(Inx: integer; const Value: TRocket);
  function GetCount: integer;
 protected
 public
  Rockets: TRocketArray;
  constructor Create;
  destructor Destroy; override;
  procedure Add(Rocket: TRocket);
  procedure Clear;
  procedure Delete(Inx: integer);
  property Value[Inx: integer]: TRocket read GetValue write SetValue; default;
  property Count: integer read GetCount;
 end;

type TDrawEvent = procedure(Sender: TObject; Image: TImage) of object;

type TFireworks = class(TObject)
 private
  TimeStep: double;
  FOnAfterDraw: TDrawEvent;
  FOnBeforeDraw: TDrawEvent;
  function AddRandomBreeze(Vel: T3DVector; Range: double): T3DVector;
  procedure Bounce(Bottom: double);
  procedure DoRocketaryMotion(TimeStep: double);
  procedure DrawCircle(Canvas: TCanvas; Pos: TPoint; Radius: integer; C: TColor);
  procedure DrawRocket(Image: TImage; P: TRocket; DoGlitter: boolean);
  procedure DropGlitter(R: TRocket; C: TColor; Randomness: double);
  procedure ExplodeBomb(var R: TRocket);
  function FindFirstFuel(FuelArray: TFuelArray): integer;
  procedure FireStarRay(R: TRocket; Angle1, Angle2, Vel: double);
  function GetDirectionVector(Angle, Thrust: double): T3DVector;
  function GetDragVector(var P: TRocket; DeltaTime: double): T3DVector;
  procedure GetPosVelAcc(Earth: TRocket; var P: TRocket; DeltaTime: double);
  function GetRandomAngle(Range: double): double;
  procedure HandleDeath(Inx: integer; TimerStep: double);
  procedure HandleTimeEffects(var R: TRocket; TimeStep: double);
  function IsBombTriggered(var R: TRocket; TimeStep: double): boolean;
  procedure LimitPositions(R: TRect);
  procedure LimitRocket(var Rocket: TRocket; R: TRect);
  function MotorRunning(var R: TRocket; TimeStep: double; var C: TColor): boolean;
  function OffsetColor(C: TColor; Percent: double): TColor;
  function RandomRange(Range: double): double;
  function RealWorldToScreen(Image: TImage; P: TPoint): TPoint;
 protected
 public
  RocketList: TRocketList;
  GlitterStart: double;
  GlitterEnabled: boolean;
  GlitterFreq: integer;
  SparkleBrightness: double;
  TrailDensity: integer;
  TrailRandom: double;
  FramesPerSec: double;
  SpeedAdjust: double;
  UseLimit: boolean;
  Color: TColor;
  constructor Create;
  destructor Destroy; override;
  procedure LaunchRocket(Image: TImage; R: TRocket; X,Y: double); overload;
  procedure LaunchRocket(Image: TImage; R: TRocket); overload;
  procedure DoAnimationStep(Image: TImage);
  procedure DrawAllRockets(Image: TImage);
  procedure AddRocket(Rocket: TRocket);
  procedure ClearAllRockets;
  procedure DeleteRocket(Inx: integer);
  property OnBeforeDraw: TDrawEvent read FOnBeforeDraw write FOnBeforeDraw;
  property OnAfterDraw: TDrawEvent read FOnAfterDraw write FOnAfterDraw;
end;


implementation


const Gravity = 6.67E-11;	{Earth's Gravitational constant }

{Parameters for the Earth. Note the radius is very important nothing will work right without it.}

const EarthParams: TRocket = (Mass:5.8E24 ; Pos:(X:0;Y:0);  Vel:(X:0;Y:0);  Acc:(X:0;Y:0); Radius:6378160; Color: clBlue);


{ ----------------- TRocketList -----------------------------------------------}


function TRocketList.GetCount: integer;
begin
Result:=Length(Rockets);
end;


procedure TRocketList.Add(Rocket: TRocket);
{Insert Rocket in dynamic array}
begin
SetLength(Rockets,Length(Rockets)+1);
Rockets[High(Rockets)]:=Rocket;
end;


procedure TRocketList.Clear;
begin
SetLength(Rockets,0);
end;


procedure TRocketList.Delete(Inx: integer);
{Insert Rocket in dynamic array}
var I: integer;
begin
for I:=Inx to High(Rockets)-1 do Rockets[I]:=Rockets[I+1];
SetLength(Rockets,Length(Rockets)-1);
end;



constructor TRocketList.Create;
begin
Clear;
end;


destructor TRocketList.Destroy;
begin

inherited;
end;


function TRocketList.GetValue(Inx: integer): TRocket;
begin
Result:=Rockets[Inx];
end;

procedure TRocketList.SetValue(Inx: integer; const Value: TRocket);
begin
Rockets[Inx]:=Value;
end;


{------------------ TFireworks ------------------------------------------------}

constructor TFireworks.Create;
begin
RocketList:=TRocketList.Create;
Color:=clBlack;
SparkleBrightness:=90;
FramesPerSec:=10;
SpeedAdjust:=1;
UseLimit:=False;
GlitterStart:=8;
GlitterEnabled:=True;
GlitterFreq:=70;
TrailDensity:=1;
TrailRandom:=0;
end;



destructor TFireworks.Destroy;
begin
RocketList.Free;
inherited;
end;


{--------------- Rocket Drawing Routines ------------------------------------}


function TFireWorks.RealWorldToScreen(Image: TImage; P: TPoint): TPoint;
{Make it so positive Y-axis movements go up the screen}
begin
Result:=P;
Result.Y:=Image.Height - Result.Y;
end;



procedure TFireWorks.DrawCircle(Canvas: TCanvas; Pos: TPoint; Radius: integer; C: TColor);
{Draw circle of specified Pos, with specified Radius, and Color}
var R: TRect;
begin
if Radius<1 then Canvas.Pixels[Pos.X,Pos.Y]:=C
else
	begin
	R:=Rect(Pos.X,Pos.Y,Pos.X,Pos.Y);
	InflateRect(R,Radius,Radius);
	Canvas.Brush.Color:=C;
	Canvas.Ellipse(R);
	end;
end;



function TFireWorks.OffsetColor(C: TColor; Percent: double): TColor;
{Offset all color values by the percentage specified}
var T: TRGBTriple;
var Offset: integer;
begin
T:=ColorToTriple(C);
Offset:=round(255 * Percent);
T.rgbtRed:=EnsureRange(T.rgbtRed + Offset,0,255);
T.rgbtGreen:=EnsureRange(T.rgbtGreen + Offset,0,255);
T.rgbtBlue:=EnsureRange(T.rgbtBlue + Offset,0,255);
Result:=TripleToColor(T);
end;




procedure TFireWorks.DrawRocket(Image: TImage; P: TRocket; DoGlitter: boolean);
{Draw one Rocket}
var R: integer;
var PT: TPoint;
var B: double;
begin
R:=Round(Meters2Inches * P.Radius);
if VectorAbs3D(P.Pos)>1000 then exit;
PT:=Vector3DToPoint(P.Pos);
PT:=RealWorldToScreen(Image,PT);
B:=0;
if P.DeathTime<2 then B:=-1-P.DeathTime/2;
DrawCircle(Image.Canvas,PT,R,OffsetColor(P.Color,B));
if DoGlitter then Image.Canvas.Pixels[PT.X,PT.Y]:=OffsetColor(P.Color,SparkleBrightness/100);
end;




function SortCompare(P1,P2: pointer): integer;
{Comparision routine to sort for depth}
var R1,R2: TRocket;
var D: double;
var S: string;
begin
R1:=PRocket(P1)^;
R2:=PRocket(P2)^;
D:=(R2.Pos.Z-R1.Pos.Z)*1000;
if abs(D)>High(integer) then Result:=round(Sign(D)) else Result:=round(D);
end;



procedure TFireWorks.DrawAllRockets(Image: TImage);
{Draw specifiec Rocket on Graphic Window}
var I: integer;
var S: string;
var Sorter: TList;
var DoGlitter: boolean;
var R: TRocket;
begin
Sorter:=TList.Create;
try
Image.Canvas.Brush.Color:=Color;
Image.Canvas.Pen.Color:=Color;
Image.Canvas.Rectangle(Image.ClientRect);
if Assigned(FOnBeforeDraw) then FOnBeforeDraw(Self,Image);

{Sort depths for the painter's hidden object routine}
for I:=0 to High(RocketList.Rockets) do Sorter.Add(@RocketList.Rockets[I]);
Sorter.Sort(SortCompare);
{Display furthest rockets first}
for I:=RocketList.Count-1 downto 0 do
	begin
	R:=PRocket(Sorter[I])^;
	DoGlitter:=GlitterEnabled and (R.DeathTime<GlitterStart) and (Random(GlitterFreq)=0);
	DrawRocket(Image,R,DoGlitter);
	end;
finally Sorter.Free; end;
if Assigned(FOnAfterDraw) then FOnAfterDraw(Self,Image);
end;


{----------------- Physics Handling routines --------------------------------}


function TFireWorks.AddRandomBreeze(Vel: T3DVector; Range: double): T3DVector;
{Add random perturbation (breeze) to to velocity}
begin
Result:=VectorAdd3D(Vel,MakeVector3D(Range*(Random-0.5),Range*(Random-0.5),Range*(Random-0.5)));
end;


function TFireWorks.GetDragVector(var P: TRocket; DeltaTime: double): T3DVector;
{Returns drag acceleration vector}
const Density = 1.293;
const Drag = 0.08;
var Force,Area,Velocity: double;
var ForceVector: T3DVector;
begin
Area:=sqr(P.Radius) * Pi;
P.Vel:=AddRandomBreeze(P.Vel,P.Randomness);
Velocity:=VectorAbs3D(P.Vel);
Force:=0.5 * Density * sqr(Velocity) * Area * Drag;
ForceVector:=ScalarProduct3D(UnitVector3D(P.Vel),Force);
Result:=ScalarDivide3D(ForceVector,P.Mass);
end;



procedure TFireWorks.GetPosVelAcc(Earth: TRocket;var  P: TRocket; DeltaTime: double);
{Calculate the new Position, Velocity acceleration of a Rocket relative to Earth}
var Distance,Force,ScalarAcc: double;
var Delta,Direction,Acc,Drag: T3DVector;
begin
{Calculate drag accelerations}
Drag:=GetDragVector(P,DeltaTime);

{Calculate distance direct between the two object}
Delta:=VectorSubtract3D(P.Pos,Earth.Pos);
{Remove sign and add in the radii}
Distance:=VectorABS3D(Delta)+Earth.Radius+P.Radius;
Direction:=NegateVector3D(UnitVector3D(Delta));
{Calculate force in Newtons}
Force:=(Earth.Mass*P.Mass*Gravity)/sqr(Distance);
{Calculate accelerations for each mass}
ScalarAcc:=Force/P.Mass;
{Calculate acceleration vectors}
P.Acc:=ScalarProduct3D(Direction,ScalarAcc);

{Add in drag and thrust}
P.Acc:=VectorSubtract3D(P.Acc,Drag);
P.Acc:=VectorAdd3D(P.Acc,ScalarDivide3D(P.Thrust,P.Mass));

{Calculate velocity vectors}
P.Vel:=VectorAdd3D(P.Vel,ScalarProduct3D(P.Acc,DeltaTime));
{Calculate position vectors}
P.Pos:=VectorAdd3D(P.Pos,ScalarProduct3D(P.Vel,DeltaTime));
end;


{-------------- Bomb behaviors ------------------------------------------------}

procedure TFireWorks.FireStarRay(R: TRocket; Angle1,Angle2,Vel: double);
{Fire one star of the star burst at the specified pitch and yaw directions}
var Dir: T3DVector;
var R1: TRocket;
begin
Dir:=PolarToVector3D(MakeVector3D(0,0,0),Angle1,Angle2,Vel);
R1:=StarRocket;
R1.PayLoad:=R.PayLoad;
R1.Color:=R.PayLoad.Color;
R1.Randomness:=R.PayLoad.Randomness;
R1.FuelArray:=R.PayLoad.BombFuel;
R1.Pos:=R.Pos;
R1.Vel:=VectorAdd3D(R.Vel,Dir);
R1.Thrust:=MakeVector3D(0,0,0);
R1.DeathTime:=10 + (Random(5) - 2.5);
AddRocket(R1);
end;



function TFireWorks.RandomRange(Range: double): double;
{Return random number in range of +/- Range/2 }
begin
Result:=(Random*Range) - (Range/2);
end;




procedure TFireWorks.DropGlitter(R: TRocket; C: TColor; Randomness: double);
{Drop glittery small rockets that simulate exhaust debris}
var Dir: T3DVector;
var R1: TRocket;
begin
R1:=GlitterRocket;
R1.Pos:=VectorAdd3D(R.Pos,MakeVector3D(RandomRange(Randomness),RandomRange(Randomness),RandomRange(Randomness)));
R1.Vel:=R.Vel;
R1.Color:=C;
R1.DeathTime:=10 + RandomRange(5);
AddRocket(R1);
end;



function TFireWorks.IsBombTriggered(var R: TRocket; TimeStep: double): boolean;
{Test if payload bomb has been triggered}
var I: integer;
begin
Result:=False;
if R.PayLoad.FuseTime>0 then
	begin
	R.PayLoad.FuseTime:=R.PayLoad.FuseTime - TimeStep;
	Result:=R.PayLoad.FuseTime<=0;
	if Result then
		begin
		R.Color:=clGray;
		R.Mass:=R.Mass/2;
		R.Radius:=R.Radius/2;
		end;
	end;
end;


function TFireWorks.FindFirstFuel(FuelArray: TFuelArray): integer;
{Find first non empty fuel tank }
begin
for Result:=0 to High(FuelArray) do
	if FuelArray[Result].BurnTime>0 then exit;
Result:=0;
end;



function TFireWorks.MotorRunning(var R: TRocket; TimeStep: double; var C: TColor): boolean;
{Test if rocket motor is still burning}
var I: integer;
begin
I:=FindFirstFuel(R.FuelArray);
C:=R.FuelArray[I].Color;
if R.FuelArray[I].BurnTime>0 then
	begin
	R.FuelArray[I].BurnTime:=R.FuelArray[I].BurnTime - TimeStep;
	if R.FuelArray[I].BurnTime<=0 then
		begin
		R.FuelArray[I].BurnTime:=0;
		R.Thrust:=MakeVector3D(0,0,0);
		end;
	end;
Result:=R.FuelArray[I].BurnTime>0;
end;



procedure TFireWorks.ExplodeBomb(var R: TRocket);
{Explode bomb spawning specified number of new rockets}
const ThreeD = true;
var Roll,Yaw: integer;
var RollStars,YawStars: integer;
var RollStep,YawStep: double;
begin
if R.PayLoad.Blast3D then
	begin
	RollStars:=round(sqrt(R.PayLoad.Stars)) +1;
	YawStars:=round(sqrt(R.PayLoad.Stars)) +1;
	end
else
	begin
	RollStars:=R.PayLoad.Stars;
	YawStars:=1;
	end;
RollStep:=360/RollStars;
YawStep:=360/YawStars;
for Roll:=0 to RollStars-1 do
 for Yaw:=0 to YawStars-1 do
	FireStarRay(R,Roll * RollStep,Yaw * YawStep,R.PayLoad.ExplosionVelocity+ Random*R.PayLoad.ExplosionRandomness);
end;



procedure TFireWorks.HandleTimeEffects(var R: TRocket; TimeStep: double);
{Dynamic arrays gets reallocated when new items are added,
so, this must be the last thing that is done with R}
var Explode, Running: boolean;
var R1: TRocket;
var C: TColor;
var I: integer;
begin
Running:=MotorRunning(R,TimeStep,C);
Explode:=IsBombTriggered(R,TimeStep);

{Must be last thing done because dynamic reallocation moves variables}
{Use R1 because R may not be pointing at original location}
R1:=R;
if Explode then ExplodeBomb(R1);
if Running then
	begin
	for I:=1 to TrailDensity do
		DropGlitter(R1,C,TrailRandom);
	end;
end;




procedure TFireWorks.HandleDeath(Inx: integer; TimerStep: double);
{Kill rocket after specified time delay}
var I: integer;
begin
if RocketList[Inx].DeathTime>0 then
	begin
	RocketList.Rockets[Inx].DeathTime:=RocketList[Inx].DeathTime - TimeStep;
	if RocketList[Inx].DeathTime<=0 then
		DeleteRocket(Inx);
	end;
end;


procedure TFireWorks.DoRocketaryMotion(TimeStep: double);
{Calculate the motions as a result of the gravity forces on all objects}
var I: integer;
var Earth: TRocket;
begin
Earth:=EarthParams;
for I:=RocketList.Count-1 downto 0 do HandleTimeEffects(RocketList.Rockets[I],TimeStep);
for I:=RocketList.Count-1 downto 0 do HandleDeath(I,TimeStep);

for I:=0 to RocketList.Count-1 do
	begin
	Earth.Pos.X:=RocketList[I].Pos.X;
	GetPosVelAcc(Earth,RocketList.Rockets[I],TimeStep);
	end;
end;



procedure TFireWorks.LimitRocket(var Rocket: TRocket; R: TRect);
{Limit a Rocket's position by bouncing it off the walls (must be a force field out there)}
begin
if Rocket.Pos.X<0 then
	begin
	Rocket.Vel.X:=-Rocket.Vel.X/2;
	Rocket.Pos.X:=1;
	end;

if Rocket.Pos.X>R.Right then
	begin
	Rocket.Vel.X:=-Rocket.Vel.X/2;
	Rocket.Pos.X:=R.Right;
	end;

if Rocket.Pos.Y>R.Bottom then
	begin
	Rocket.Vel.Y:=-Rocket.Vel.Y/2;
	Rocket.Pos.Y:=R.Bottom;
	end;
end;



procedure TFireWorks.LimitPositions(R: TRect);
{Limit the position of all Rockets in the array}
var I: integer;
begin
for I:=0 to RocketList.Count-1 do LimitRocket(RocketList.Rockets[I],R);
end;



procedure TFireWorks.Bounce(Bottom: double);
{Limit the position of all Rockets in the array}
var I: integer;
begin
for I:=RocketList.Count-1 downto 0 do
	begin
	if RocketList[I].Pos.Y<0 then
		begin
		Inc(RocketList.Rockets[I].Bounces);
		RocketList.Rockets[I].Vel.Y:=-RocketList[I].Vel.Y/4;
		RocketList.Rockets[I].Pos.Y:=0;
		if RocketList[I].Bounces>1 then DeleteRocket(I);
		end;
	end;
end;



{------------------Envent handlers --------------------------------------------}


function TFireWorks.GetDirectionVector(Angle, Thrust: double): T3DVector;
{Get direction vector based on an angle and a magnitude }
{at this point confined to 2D}
var X: double;
begin
Result.X:=Sin(Angle * Degree2Radian)* Thrust;
Result.Y:=Cos(Angle * Degree2Radian)* Thrust;
Result.Z:=0;
end;


function TFireWorks.GetRandomAngle(Range: double): double;
begin
Result:=Range - (Random*Range*2);
end;



procedure TFireWorks.LaunchRocket(Image: TImage; R: TRocket; X,Y: double);
{Launch the specified rocket}
begin
R.Pos.X:=X;
R.Pos.Y:=Y;
if R.LaunchParams.DoRandom then R.Thrust:=GetDirectionVector(GetRandomAngle(R.LaunchParams.Angle),R.LaunchParams.ThrustForce)
else R.Thrust:=GetDirectionVector(R.LaunchParams.Angle,R.LaunchParams.ThrustForce);
AddRocket(R);
DrawAllRockets(Image);
end;


procedure TFireWorks.LaunchRocket(Image: TImage; R: TRocket);
{Launch the specified rocket}
begin
LaunchRocket(Image,R,Image.Width div 2,R.Pos.Y);
end;






procedure TFireWorks.DoAnimationStep(Image: TImage);
{Update and display all rockets for one time-step}
begin
TimeStep:=SpeedAdjust * (1/FramesPerSec);
DoRocketaryMotion(TimeStep);
DrawAllRockets(Image);
Bounce(Image.ClientRect.Bottom);
if UseLimit then LimitPositions(Image.ClientRect);
end;


{----------------- Dynamic array handling routines ---------------------------}


procedure TFireWorks.AddRocket(Rocket: TRocket);
{Insert Rocket in dynamic array}
begin
RocketList.Add(Rocket);
end;



procedure TFireWorks.DeleteRocket(Inx: integer);
{Insert Rocket in dynamic array}
var I: integer;
begin
RocketList.Delete(Inx);
end;



procedure TFireWorks.ClearAllRockets;
{Clear dynamic array}
begin
RocketList.Clear;
end;


end.
