unit GoogleEarthLib;

{This library no longer works with 64-bit windows or Win10}

interface

uses Windows,Dialogs,GoogleEarthAPI,COMObjLibrary;

var GEClassID: TGUID =  '{AFD07A5E-3E20-4D77-825C-2F6D1A50BE5B}';



type TGECameraInfo = record
 Long,Lat,Range,Alt,Azm,Tilt: double;
 end;


procedure BringGEToFront;
function IsGEInstalled: boolean;
procedure SetCameraPosition(Info: TGECameraInfo; Relative: boolean);
procedure GetCameraPosition(var Info: TGECameraInfo; Relative: boolean);


implementation


{------------------------------------------------------------------------------}



procedure GetCameraPosition(var Info: TGECameraInfo; Relative: boolean);
{Get Google Earth Camera Position}
var GE:TApplicationGE;
var Cam: ICameraInfoGE;
var Mode: integer;
begin
GE:=TApplicationGE.Create(nil);
GE.Connect;
Cam:=GE.GetCamera(0);
if Relative then Mode:=RelativeToGroundAltitudeGE
else Mode:=AbsoluteAltitudeGE;
Cam.FocusPointAltitudeMode:=Mode;
Info.Long:=Cam.FocusPointLongitude;
Info.Lat:=Cam.FocusPointLatitude;
Info.Tilt:=Cam.Tilt;
Info.Azm:=Cam.Azimuth;
Info.Range:=Cam.Range;
Info.Alt:=Cam.FocusPointAltitude;
GE.Disconnect;
GE.Free;
end;



procedure SetCameraPosition(Info: TGECameraInfo; Relative: boolean);
{Set Google Earth Camera Position}
var GE: TApplicationGE;
var Cam: ICameraInfoGE;
var Mode: integer;
begin
GE:=TApplicationGE.Create(nil);
GE.Connect;
Cam:=GE.GetCamera(0);
if Relative then Mode:=RelativeToGroundAltitudeGE
else Mode:=AbsoluteAltitudeGE;
Cam.FocusPointAltitudeMode:=Mode;
Cam.FocusPointLongitude:=Info.Long;
Cam.FocusPointLatitude:=Info.Lat;
Cam.Tilt:=Info.Tilt;
Cam.Azimuth:=Info.Azm;
Cam.Range:=Info.Range;
Cam.FocusPointAltitude:=Info.Alt;
GE.SetCamera(Cam,0.3);
GE.Disconnect;
GE.Free;
end;





procedure GetGEView(var Info: IKHViewInfo);
{Get view information from Google Earth}
var GE: TKHInterface;
begin
GE:=TKHInterFace.Create(nil);
try
GE.Connect;
Info:=GE.currentView(0);
finally GE.Free; end;
end;


procedure SetGEView(Info: IKHViewInfo);
{Set Google Earth View}
var GE: TKHInterface;
begin
GE:=TKHInterFace.Create(nil);
GE.Connect;
GE.setView(Info,0,0.3);
GE.Disconnect;
GE.Free;
end;



procedure BringGEToFront;
{Restore Google Earth Window and bring to front}
var GE:TApplicationGE;
var H: HWnd;
begin
GE:=TApplicationGE.Create(nil);
try
GE.Connect;
try
H:=GE.GetMainHwnd;
ShowWindow(H,SW_RESTORE);
SetForegroundWindow(H);
finally GE.Disconnect; end;
finally GE.Free; end;
end;



function IsGEInstalled: boolean;
{Check if GE is installed, display message if not }
begin
Result:=IsComObjectRegistered(GEClassID);
if not Result then  MessageDlg('Google Earth Not Installed', mtInformation, [mbOk], 0);
end;



end.
