unit Huge2d;

interface

uses Windows,SysUtils,Types,Classes,HugeArry;


type TExpandMode = (emLeft,emTop,emBottom,emRight);

{2D integer array - can be resized and redimensioned}

type TIntItem = record
 Int: integer;
 Obj: TObject;
 end;

type TIntegerArray2D = array of array of TIntItem;


type
 T2DIntArray=class(TObject)
 private
   FArray: TIntegerArray2D;
   function GetValue(X,Y:integer): integer;
   procedure SetValue(X,Y: integer; const V: integer);
   function GetColumns: integer;
   function GetRows: integer;
   procedure ShiftValuesRight(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesLeft(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesDown(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesUp(Offset: integer; OldDim: TPoint);
   procedure ReDimLeft(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimTop(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimRight(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimBottom(SizeDelta: integer; OldDim: TPoint);
   procedure SetColumns(const Value: integer);
   procedure SetRows(const Value: integer);
   function GetObject(X, Y: integer): TObject;
   procedure SetObject(X, Y: integer; const Value: TObject);
 protected
 public
   XIndex,YIndex: integer;
   constructor Create(X,Y: integer);
   destructor Destroy; override;
   procedure Clear;
   procedure SetDimensions(X,Y: integer);
   procedure SetNewDim(NewSize: integer; ET: TExpandMode);
   procedure ChangeDim(SizeDelta: integer; ET: TExpandMode);
   procedure Assign(IntArray: T2DIntArray);
   procedure OffsetAssign(IntArray: T2DIntArray; Off: TPoint);
   property Value[X,Y: integer]: integer read GetValue write SetValue; default;
   property Objects[X,Y: integer]: TObject read GetObject write SetObject;
   property Cols: integer read GetColumns write SetColumns;
   property Rows: integer read GetRows write SetRows;
 end;

{Non-Orthogonal 2D array}

type TNO2DIntArray = class(TList)
 private
  function GetValue(X, Y: integer): integer;
  procedure SetValue(X, Y: integer; const Value: integer);
  function GetRows: integer;
  function GetRow(Inx: integer): THugeInteger;
  function GetCols(RowInx: integer): integer;
  function GetColMax: integer;
 protected
  procedure Notify(Ptr: Pointer; Action: TListNotification); override;
 public
    procedure DeleteItem(X, Y: integer);
    procedure DeleteRow(Inx: integer);
    procedure InsertItem(Value,X,Y: integer);
    procedure InsertRow(Inx: integer);
  procedure Clear;
  procedure Assign(NOArray: TNO2DIntArray);
    procedure Append(NOArray: TNO2DIntArray);
  constructor Create;
  destructor Destroy; override;
  function Add: integer; overload;
  function Add(Row,Value: integer): integer; overload;
  property Value[X,Y: integer]: integer read GetValue write SetValue; default;
  property Rows: integer read GetRows;
  property Row[Inx: integer]: THugeInteger read GetRow;
 property Cols[RowInx: integer]: integer read GetCols;
  property ColMax: integer read GetColMax;
 end;

{2D double array - can be resized and redimensioned}

type TDblItem = record
 Dbl: double;
 Obj: TObject;
 end;

type TDoubleArray2D = array of array of TDblItem;

type
 T2DDoubleArray=class(TObject)
 private
   FArray: TDoubleArray2D;
   function GetValue(X,Y:integer): double;
   procedure SetValue(X,Y: integer; const V: double);
   function GetColumns: integer;
   function GetRows: integer;
   procedure ShiftValuesRight(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesLeft(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesDown(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesUp(Offset: integer; OldDim: TPoint);
   procedure ReDimLeft(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimTop(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimRight(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimBottom(SizeDelta: integer; OldDim: TPoint);
   procedure SetColumns(const Value: integer);
   procedure SetRows(const Value: integer);
   function GetObject(X, Y: integer): TObject;
   procedure SetObject(X, Y: integer; const Value: TObject);
 protected
 public
   XIndex,YIndex: integer;
   FillExpansion: boolean;
   FillValue: double;
   constructor Create(X,Y: integer);
   destructor Destroy; override;
   procedure Clear(Value: double); overload;
   procedure Clear(Rect: TRect; Value: double); overload;
   procedure SetDimensions(X,Y: integer);
   procedure ChangeDim(SizeDelta: integer; ET: TExpandMode);
   procedure SetNewDim(NewSize: integer; ET: TExpandMode); virtual;
   procedure Assign(DblArray: T2DDoubleArray);
   procedure OffsetAssign(DblArray: T2DDoubleArray; Off: TPoint);
   property Value[X,Y: integer]: double read GetValue write SetValue; default;
   property Objects[X,Y: integer]: TObject read GetObject write SetObject;
   property Cols: integer read GetColumns write SetColumns;
   property Rows: integer read GetRows write SetRows;
 end;

{2D string array - can be resized and redimensioned}

type TStrItem = record
 Str: string;
 Obj: TObject;
 end;

type TStringArray2D = array of array of TStrItem;


type
 T2DStrArray=class(TObject)
 private
   FArray: TStringArray2D;
   function GetValue(X,Y:integer): string;
   procedure SetValue(X,Y: integer; const V: string);
   function GetColumns: integer;
   function GetRows: integer;
   procedure ShiftValuesRight(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesLeft(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesDown(Offset: integer; OldDim: TPoint);
   procedure ShiftValuesUp(Offset: integer; OldDim: TPoint);
   procedure ReDimLeft(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimTop(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimRight(SizeDelta: integer; OldDim: TPoint);
   procedure ReDimBottom(SizeDelta: integer; OldDim: TPoint);
    procedure SetColumns(const Value: integer);
    procedure SetRows(const Value: integer);
    function GetObject(X, Y: integer): TObject;
    procedure SetObject(X, Y: integer; const Value: TObject);
 protected
 public
   XIndex,YIndex: integer;
   constructor Create(X,Y: integer);
   destructor Destroy; override;
   procedure Clear;
   procedure SetDimensions(X,Y: integer);
   procedure SetNewDim(NewSize: integer; ET: TExpandMode);
   procedure ChangeDim(SizeDelta: integer; ET: TExpandMode);
   procedure Assign(IntArray: T2DStrArray);
   procedure OffsetAssign(IntArray: T2DStrArray; Off: TPoint);
   property Value[X,Y: integer]: string read GetValue write SetValue; default;
   property Objects[X,Y: integer]: TObject read GetObject write SetObject;
   property Cols: integer read GetColumns write SetColumns;
   property Rows: integer read GetRows write SetRows;
 end;

{Holds array of polygons. Each polygon is a vector array}


type TPolygonArray = class(THugeArray)
 private
  procedure Put(Index: integer; Item: THuge3DVector);
  function Get(Index: integer): THuge3DVector;
 protected
 public
  constructor Create(Number: integer);
  destructor Destroy; override;
  function Add: THuge3DVector;
  procedure Delete(Index: integer);
  property Value[Index: integer]: THuge3DVector read Get; default;
  procedure Clear; override;
 end;


implementation


{-------------------------------------------------------------------------}
{              Implementation for 2D Integer Array object                 }
{-------------------------------------------------------------------------}



function T2DIntArray.GetColumns: integer;
begin
Result:=High(FArray)+1;
end;



function T2DIntArray.GetRows: integer;
begin
Result:=High(FArray[0])+1;
end;



constructor T2DIntArray.Create(X,Y: integer);
{Create a huge array item}
begin
inherited Create;
SetLength(FArray,X,Y);
XIndex:=0; YIndex:=0;
end;


destructor T2DIntArray.Destroy;
{Destroy Huge array item}
begin
inherited Destroy;
end;


function T2DIntArray.GetValue(X,Y: integer): integer;
begin
Result:=FArray[X,Y].Int;
end;


procedure T2DIntArray.SetValue(X,Y: integer;const V: integer);
begin
FArray[X,Y].Int:=V;
end;


function T2DIntArray.GetObject(X, Y: integer): TObject;
begin
Result:=FArray[X,Y].Obj;
end;


procedure T2DIntArray.SetObject(X, Y: integer; const Value: TObject);
begin
FArray[X,Y].Obj:=Value;

end;

procedure T2DIntArray.Clear;
var X: integer;
begin
for X:=0 to Cols-1 do
	ZeroMemory(FArray[X],Rows * SizeOf(Integer));
end;


procedure T2DIntArray.SetDimensions(X,Y: integer);
{Set dimensions without regard for the contents}
begin
SetLength(FArray,X,Y);
end;



procedure T2DIntArray.SetColumns(const Value: integer);
begin
SetDimensions(Value,Rows);
end;



procedure T2DIntArray.SetRows(const Value: integer);
begin
SetDimensions(Cols,Value);
end;



procedure T2DIntArray.Assign(IntArray: T2DIntArray);
var X,Y: integer;
begin
Self.SetDimensions(intArray.Cols,IntArray.Rows);
for Y:=0 to IntArray.Rows-1 do
 for X:=0 to IntArray.Cols-1 do
 Self[X,Y]:=IntArray[X,Y];
end;





procedure T2DIntArray.OffsetAssign(IntArray: T2DIntArray; Off: TPoint);
{Assign but inserted starting at offset}
var X,Y: integer;
begin
for Y:=0 to IntArray.Rows-1 do
 for X:=0 to IntArray.Cols-1 do
 Self[X+Off.X,Y+Off.Y]:=IntArray[X,Y];
end;



{----------------- Routines to Adjust Dimensions -----------------------------}


procedure T2DIntArray.ShiftValuesRight(Offset: integer; OldDim: TPoint);
{Shift values in array to right}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=OldDim.X-1 downto 0 do
	begin
	FArray[X+Offset,Y]:=FArray[X,Y];
	FArray[X,Y].Int:=0;
	FArray[X,Y].Obj:=nil;
	end;
end;


procedure T2DIntArray.ShiftValuesLeft(Offset: integer; OldDim: TPoint);
{Shift values in array to the left}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=Offset to OldDim.X-1 do
	FArray[X-Offset,Y]:=FArray[X,Y];
end;



procedure T2DIntArray.ShiftValuesDown(Offset: integer; OldDim: TPoint);
{Shift values in array to the down}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=OldDim.Y-1 downto 0 do
	begin
	FArray[X,Y+Offset]:=FArray[X,Y];
	FArray[X,Y].Int:=0;
	FArray[X,Y].Obj:=nil;
	end;
end;



procedure T2DIntArray.ShiftValuesUp(Offset: integer; OldDim: TPoint);
{Shift values in array to the up}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=Offset to OldDim.Y-1 do
	FArray[X,Y-Offset]:=FArray[X,Y];
end;


procedure T2DIntArray.ReDimLeft(SizeDelta: integer; OldDim: TPoint);
{Expand array on the left side by reallocating and shifting old items right/left}
begin
if SizeDelta<0 then
	begin
	if (SizeDelta+OldDim.X)<1 then exit;
 	ShiftValuesLeft(abs(SizeDelta),OldDim);
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	end
else
	begin
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	ShiftValuesRight(SizeDelta,OldDim);
	end;
end;



procedure T2DIntArray.ReDimTop(SizeDelta: integer; OldDim: TPoint);
{Expand array on the top side by reallocating and shifting old items up/down}
begin
if SizeDelta<1 then
	begin
	if (SizeDelta+OldDim.Y)<1 then exit;
	ShiftValuesUp(abs(SizeDelta),OldDim);
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	end
else
	begin
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	ShiftValuesDown(SizeDelta,OldDim);
	end;
end;



procedure T2DIntArray.ReDimRight(SizeDelta: integer; OldDim: TPoint);
{Expand array of right side. SetLength automatically puts result on right}
begin
if (SizeDelta+OldDim.X)<1 then exit;
SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
end;



procedure T2DIntArray.ReDimBottom(SizeDelta: integer; OldDim: TPoint);
{Expand array of bottom side. SetLength automatically puts result on bottom}
begin
if (SizeDelta+OldDim.Y)<1 then exit;
SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
end;


procedure T2DIntArray.ChangeDim(SizeDelta: integer; ET: TExpandMode);
{Change size while preseving the the content }
{by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(SizeDelta,OldDim);
 emRight: ReDimRight(SizeDelta,OldDim);
 emTop: ReDimTop(SizeDelta,OldDim);
 emBottom: ReDimBottom(SizeDelta,OldDim);
 end;
end;




procedure T2DIntArray.SetNewDim(NewSize: integer; ET: TExpandMode);
{Change size while preseving the the content }
{by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(NewSize-OldDim.X,OldDim);
 emRight: ReDimRight(NewSize-OldDim.X,OldDim);
 emTop: ReDimTop(NewSize-OldDim.Y,OldDim);
 emBottom: ReDimBottom(NewSize-OldDim.Y,OldDim);
 end;
end;

{-------------------------------------------------------------------------}
{              Implementation for 2D non-orthogonal integer array         }
{-------------------------------------------------------------------------}

{ TNO2DIntArray }

constructor TNO2DIntArray.Create;
begin
inherited Create;

end;

destructor TNO2DIntArray.Destroy;
begin

inherited Destroy;
end;


function TNO2DIntArray.Add: integer;
var IA: THugeInteger;
begin
IA:=THugeInteger.Create(5);
IA.Name:=IntToStr(Count);
Result:=inherited Add(IA);
end;


function TNO2DIntArray.Add(Row,Value: integer): integer;
begin
Result:=THugeInteger(Items[Row]).Add(Value);
end;


procedure TNO2DIntArray.Notify(Ptr: Pointer; Action: TListNotification);
begin
inherited;
case Action of
 lnAdded: ;
 lnExtracted:;
 lnDeleted: THugeInteger(Ptr).Free;
 end;
end;



function TNO2DIntArray.GetValue(X, Y: integer): integer;
begin
Result:=THugeInteger(Items[Y])[X];
end;



procedure TNO2DIntArray.SetValue(X, Y: integer; const Value: integer);
begin
THugeInteger(Items[Y])[X]:=Value;
end;



function TNO2DIntArray.GetRows: integer;
begin
Result:=Count;
end;



function TNO2DIntArray.GetRow(Inx: integer): THugeInteger;
begin
Result:=THugeInteger(Items[Inx]);
end;



function TNO2DIntArray.GetCols(RowInx: integer): integer;
begin
Result:=THugeInteger(Items[RowInx]).Count;
end;



function TNO2DIntArray.GetColMax: integer;
{Get the maximum number of columns in the array}
var X,Y: integer;
begin
Result:=0;
for Y:=0 to Count-1 do
 if Cols[Y]>Result then Result:=Cols[Y];
end;



procedure TNO2DIntArray.Assign(NOArray: TNO2DIntArray);
var X,Y,Inx: integer;
begin
Clear;
for Y:=0 to NoArray.Rows-1 do
	begin
	Inx:=Add;
	for X:=0 to NOArray.Cols[Inx]-1 do
		THugeInteger(Items[Inx]).Add(NOArray[X,Y]);
	end;
end;




procedure TNO2DIntArray.Append(NOArray: TNO2DIntArray);
var X,Y,Inx: integer;
begin
Self.Count:=NOArray.Count;
for Y:=0 to NoArray.Rows-1 do
	begin
	for X:=0 to NOArray.Cols[Inx]-1 do
		THugeInteger(Items[Inx]).Add(NOArray[X,Y]);
	end;
end;




procedure TNO2DIntArray.Clear;
var R: integer;
begin
for R:=Rows-1 downto 0 do Delete(R);
inherited Clear;
end;



procedure TNO2DIntArray.DeleteRow(Inx: integer);
begin
if (Inx<0) or (Inx>=Count) then exit;
Delete(Inx);
end;


procedure TNO2DIntArray.InsertRow(Inx: integer);
var IA: THugeInteger;
begin
if (Inx<0) or (Inx>=Count) then exit;
IA:=THugeInteger.Create(5);
inherited Insert(Inx,IA);
end;


procedure TNO2DIntArray.DeleteItem(X,Y: integer);
begin
if (Y<0) or (Y>=Count) or (X<0) or (X>=THugeInteger(Items[Y]).Count) then exit;
THugeInteger(Items[Y]).Delete(X);
end;


procedure TNO2DIntArray.InsertItem(Value,X,Y: integer);
begin
if (Y<0) or (Y>=Count) then exit;
THugeInteger(Items[Y]).Insert(Value,X);
end;



{-------------------------------------------------------------------------}
{              Implementation for 2D Double Array object                  }
{-------------------------------------------------------------------------}



function T2DDoubleArray.GetColumns: integer;
begin
Result:=High(FArray)+1;
end;



function T2DDoubleArray.GetRows: integer;
begin
if FArray=nil then Result:=0
else Result:=Length(FArray[0]);
end;



constructor T2DDoubleArray.Create(X,Y: integer);
{Create a huge array item}
begin
inherited Create;
SetLength(FArray,X,Y);
XIndex:=0; YIndex:=0;
FillExpansion:=False;
FillValue:=0;
end;



destructor T2DDoubleArray.Destroy;
{Destroy Huge array item}
begin
inherited Destroy;
end;


function T2DDoubleArray.GetValue(X,Y: integer): double;
begin
Result:=FArray[X,Y].Dbl;
end;


procedure T2DDoubleArray.SetValue(X,Y: integer;const V: double);
begin
FArray[X,Y].Dbl:=V;
end;


function T2DDoubleArray.GetObject(X, Y: integer): TObject;
begin
Result:=FArray[X,Y].Obj;
end;


procedure T2DDoubleArray.SetObject(X, Y: integer; const Value: TObject);
begin
FArray[X,Y].Obj:=Value;
end;


procedure T2DDoubleArray.SetColumns(const Value: integer);
begin
SetDimensions(Value,Rows);
end;



procedure T2DDoubleArray.SetRows(const Value: integer);
begin
SetDimensions(Cols,Value);
end;



{----------------- Routines to Adjust Dimensions -----------------------------}


procedure T2DDoubleArray.ShiftValuesRight(Offset: integer; OldDim: TPoint);
{Shift values in array to rigth}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=OldDim.X-1 downto 0 do
	begin
	FArray[X+Offset,Y]:=FArray[X,Y];
	FArray[X,Y].Dbl:=0;
	FArray[X,Y].Obj:=nil;
	end;
end;


procedure T2DDoubleArray.ShiftValuesLeft(Offset: integer; OldDim: TPoint);
{Shift values in array to the left}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=Offset to OldDim.X-1 do
	FArray[X-Offset,Y]:=FArray[X,Y];
end;



procedure T2DDoubleArray.ShiftValuesDown(Offset: integer; OldDim: TPoint);
{Shift values in array to the down}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=OldDim.Y-1 downto 0 do
	begin
	FArray[X,Y+Offset]:=FArray[X,Y];
	FArray[X,Y].Dbl:=0;
	FArray[X,Y].Obj:=nil;
	end;
end;



procedure T2DDoubleArray.ShiftValuesUp(Offset: integer; OldDim: TPoint);
{Shift values in array to the up}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=Offset to OldDim.Y-1 do
	FArray[X,Y-Offset]:=FArray[X,Y];
end;



procedure T2DDoubleArray.ReDimLeft(SizeDelta: integer; OldDim: TPoint);
{Expand array on the left side by reallocating and shifting old items right/left}
begin
if SizeDelta<0 then
	begin
	if (SizeDelta+OldDim.X)<1 then exit;
 	ShiftValuesLeft(abs(SizeDelta),OldDim);
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	end
else
	begin
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	ShiftValuesRight(SizeDelta,OldDim);
	if FillExpansion then
		begin
		Clear(Rect(0,0,SizeDelta-1,OldDim.Y-1),FillValue);
		end
	end;
end;



procedure T2DDoubleArray.ReDimTop(SizeDelta: integer; OldDim: TPoint);
{Expand array on the top side by reallocating and shifting old items up/down}
begin
if SizeDelta<1 then
	begin
	if (SizeDelta+OldDim.Y)<1 then exit;
	ShiftValuesUp(abs(SizeDelta),OldDim);
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	end
else
	begin
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	ShiftValuesDown(SizeDelta,OldDim);
	if FillExpansion then
		begin
		Clear(Rect(0,0,OldDim.X-1,SizeDelta-1),FillValue);
		end
	end;
end;




procedure T2DDoubleArray.ReDimBottom(SizeDelta: integer; OldDim: TPoint);
{Expand array of bottom side. SetLength automatically puts result on bottom}
begin
if (SizeDelta+OldDim.Y)<1 then exit;
SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
if FillExpansion then
	begin
	Clear(Rect(0,OldDim.Y,OldDim.X-1,OldDim.Y+SizeDelta-1),FillValue);
	end
end;



procedure T2DDoubleArray.ReDimRight(SizeDelta: integer; OldDim: TPoint);
{Expand array of right side. SetLength automatically puts result on right}
begin
if (SizeDelta+OldDim.X)<1 then exit;
SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
if FillExpansion then
	begin
	Clear(Rect(OldDim.X,0,OldDim.X+SizeDelta-1,OldDim.Y-1),FillValue);
	end
end;



procedure T2DDoubleArray.ChangeDim(SizeDelta: integer; ET: TExpandMode);
{Change size by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(SizeDelta,OldDim);
 emRight: ReDimRight(SizeDelta,OldDim);
 emTop: ReDimTop(SizeDelta,OldDim);
 emBottom: ReDimBottom(SizeDelta,OldDim);
 end;
end;




procedure T2DDoubleArray.SetNewDim(NewSize: integer; ET: TExpandMode);
{Change size while preseving the the content }
{by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(NewSize-OldDim.X,OldDim);
 emRight: ReDimRight(NewSize-OldDim.X,OldDim);
 emTop: ReDimTop(NewSize-OldDim.Y,OldDim);
 emBottom: ReDimBottom(NewSize-OldDim.Y,OldDim);
 end;
end;




procedure T2DDoubleArray.SetDimensions(X,Y: integer);
begin
SetLength(FArray,X,Y);
end;



procedure T2DDoubleArray.Assign(DblArray: T2DDoubleArray);
var X,Y: integer;
begin
Self.SetDimensions(DblArray.Cols,DblArray.Rows);
for Y:=0 to DblArray.Rows-1 do
 for X:=0 to DblArray.Cols-1 do
 Self[X,Y]:=DblArray[X,Y];
end;


procedure T2DDoubleArray.OffsetAssign(DblArray: T2DDoubleArray; Off: TPoint);
{Assign but inserted starting at offset}
var X,Y: integer;
begin
for Y:=0 to DblArray.Rows-1 do
 for X:=0 to DblArray.Cols-1 do
 Self[X+Off.X,Y+Off.Y]:=DblArray[X,Y];
end;



procedure T2DDoubleArray.Clear(Rect: TRect; Value: double);
{Clear specified rectangle with specified value}
var X,Y: integer;       
begin
for Y:=Rect.Top to Rect.Bottom do
 for X:=Rect.Left to Rect.Right do
 Self[X,Y]:=Value;
end;



procedure T2DDoubleArray.Clear(Value: double);
{Clear whole array with specified value}
begin
Clear(Rect(0,0,Self.Cols-1,Self.Rows-1),Value);
end;


{-------------------------------------------------------------------------}
{ Implementation Polygon Array object based on huge array object          }
{-------------------------------------------------------------------------}


constructor TPolygonArray.Create(Number: Integer);
{Create a huge Polygon Array}
begin
inherited Create(1,SizeOf(THuge3DVector));
Add;
end;


function TPolygonArray.Add: THuge3DVector;
{Add item to Polygon Array}
var PTR: ^THuge3DVector;
begin
Result:=THuge3DVector.Create(5);
ExpandArray;
PTR:=GetItemPtr(ItemIndex);
PTR^:=Result;
ItemIndex:=ItemIndex+1;
end;


procedure TPolygonArray.Put(Index: integer; Item: THuge3DVector);
{Insert Polygon at index}
var PTR: ^THuge3DVector;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function TPolygonArray.Get(Index: Longint): THuge3DVector;
{Get Polygon at the index}
var PTR: ^THuge3DVector;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;



procedure TPolygonArray.Delete(Index: integer);
begin
Get(Index).Free;
DeleteItem(Index);
end;


procedure TPolygonArray.Clear;
var I: integer;
begin
for I:=0 to ItemIndex-1 do Get(I).Free;
inherited Clear;
end;


destructor TPolygonArray.Destroy;
{Destroy Huge array item}
begin
Clear;
inherited Destroy;
end;



{-------------------------------------------------------------------------}
{              Implementation for 2D string Array object                  }
{-------------------------------------------------------------------------}



function T2DStrArray.GetColumns: integer;
begin
Result:=High(FArray)+1;
end;



function T2DStrArray.GetRows: integer;
begin
Result:=High(FArray[0])+1;
end;



procedure T2DStrArray.SetColumns(const Value: integer);
begin
SetDimensions(Value,Rows);
end;



procedure T2DStrArray.SetRows(const Value: integer);
begin
SetDimensions(Cols,Value);
end;



constructor T2DStrArray.Create(X,Y: integer);
{Create a huge array item}
begin
inherited Create;
SetLength(FArray,X,Y);
XIndex:=0; YIndex:=0;
end;


destructor T2DStrArray.Destroy;
{Destroy Huge array item}
begin
inherited Destroy;
end;


function T2DStrArray.GetValue(X,Y: integer): string;
begin
Result:=FArray[X,Y].Str;
end;


procedure T2DStrArray.SetValue(X,Y: integer;const V: string);
begin
FArray[X,Y].Str:=V;
end;



function T2DStrArray.GetObject(X, Y: integer): TObject;
begin
Result:=FArray[X,Y].Obj;
end;



procedure T2DStrArray.SetObject(X, Y: integer; const Value: TObject);
begin
FArray[X,Y].Obj:=Value;
end;



procedure T2DStrArray.Clear;
var X: integer;
begin
for X:=0 to Cols-1 do
	ZeroMemory(FArray[X],Rows * SizeOf(Integer));
end;


procedure T2DStrArray.SetDimensions(X,Y: integer);
{Set dimensions without regard for the contents}
begin
SetLength(FArray,X,Y);
end;



procedure T2DStrArray.Assign(IntArray: T2DStrArray);
var X,Y: integer;
begin
Self.SetDimensions(intArray.Cols,IntArray.Rows);
for Y:=0 to IntArray.Rows-1 do
 for X:=0 to IntArray.Cols-1 do
 Self[X,Y]:=IntArray[X,Y];
end;


procedure T2DStrArray.OffsetAssign(IntArray: T2DStrArray; Off: TPoint);
{Assign but inserted starting at offset}
var X,Y: integer;
begin
for Y:=0 to IntArray.Rows-1 do
 for X:=0 to IntArray.Cols-1 do
 Self[X+Off.X,Y+Off.Y]:=IntArray[X,Y];
end;



{----------------- Routines to Adjust Dimensions -----------------------------}


procedure T2DStrArray.ShiftValuesRight(Offset: integer; OldDim: TPoint);
{Shift values in array to right}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=OldDim.X-1 downto 0 do
	begin
	FArray[X+Offset,Y]:=FArray[X,Y];
	FArray[X,Y].Str:='';
	FArray[X,Y].Obj:=nil;
	end;
end;


procedure T2DStrArray.ShiftValuesLeft(Offset: integer; OldDim: TPoint);
{Shift values in array to the left}
var X,Y: integer;
begin
for Y:=0 to OldDim.Y-1 do
 for X:=Offset to OldDim.X-1 do
	FArray[X-Offset,Y]:=FArray[X,Y];
end;



procedure T2DStrArray.ShiftValuesDown(Offset: integer; OldDim: TPoint);
{Shift values in array to the down}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=OldDim.Y-1 downto 0 do
	begin
	FArray[X,Y+Offset]:=FArray[X,Y];
	FArray[X,Y].Str:='';
	FArray[X,Y].Obj:=nil;
	end;
end;



procedure T2DStrArray.ShiftValuesUp(Offset: integer; OldDim: TPoint);
{Shift values in array to the up}
var X,Y: integer;
begin
for X:=0 to OldDim.X-1 do
 for Y:=Offset to OldDim.Y-1 do
	FArray[X,Y-Offset]:=FArray[X,Y];
end;


procedure T2DStrArray.ReDimLeft(SizeDelta: integer; OldDim: TPoint);
{Expand array on the left side by reallocating and shifting old items right/left}
begin
if SizeDelta<0 then
	begin
	if (SizeDelta+OldDim.X)<1 then exit;
 	ShiftValuesLeft(abs(SizeDelta),OldDim);
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	end
else
	begin
	SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
	ShiftValuesRight(SizeDelta,OldDim);
	end;
end;



procedure T2DStrArray.ReDimTop(SizeDelta: integer; OldDim: TPoint);
{Expand array on the top side by reallocating and shifting old items up/down}
begin
if SizeDelta<1 then
	begin
	if (SizeDelta+OldDim.Y)<1 then exit;
	ShiftValuesUp(abs(SizeDelta),OldDim);
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	end
else
	begin
	SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
	ShiftValuesDown(SizeDelta,OldDim);
	end;
end;



procedure T2DStrArray.ReDimRight(SizeDelta: integer; OldDim: TPoint);
{Expand array of right side. SetLength automatically puts result on right}
begin
if (SizeDelta+OldDim.X)<1 then exit;
SetLength(FArray,SizeDelta+OldDim.X,OldDim.Y);
end;



procedure T2DStrArray.ReDimBottom(SizeDelta: integer; OldDim: TPoint);
{Expand array of bottom side. SetLength automatically puts result on bottom}
begin
if (SizeDelta+OldDim.Y)<1 then exit;
SetLength(FArray,OldDim.X,SizeDelta+OldDim.Y);
end;


procedure T2DStrArray.ChangeDim(SizeDelta: integer; ET: TExpandMode);
{Change size while preseving the the content }
{by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(SizeDelta,OldDim);
 emRight: ReDimRight(SizeDelta,OldDim);
 emTop: ReDimTop(SizeDelta,OldDim);
 emBottom: ReDimBottom(SizeDelta,OldDim);
 end;
end;




procedure T2DStrArray.SetNewDim(NewSize: integer; ET: TExpandMode);
{Change size while preseving the the content }
{by adding or subtract to left, right, top or bottom}
var OldDim: TPoint;
begin
OldDim.X:=Cols;
OldDim.Y:=Rows;
case ET of
 emLeft: ReDimLeft(NewSize-OldDim.X,OldDim);
 emRight: ReDimRight(NewSize-OldDim.X,OldDim);
 emTop: ReDimTop(NewSize-OldDim.Y,OldDim);
 emBottom: ReDimBottom(NewSize-OldDim.Y,OldDim);
 end;
end;


end.
