unit HugeArry;

interface

uses Windows,Classes,SysUtils,Vector,DSupport;

{------------------------------------------------------------}
{----------- Huge Array based TFileStream -------------------}
{------------------------------------------------------------}

{Note; this array is really a disk file, wo writing to it will
permanently modify disk data }


type TCache = record
 Start: int64;
 Size: int64;
 Buffer: array [0..511] of byte;
 Valid: boolean;
 end;


type TDiskArray = class(TObject)
 private
  Cache: TCache;
  FFileStream: TFileStream;
  function Get(Index: Int64): byte;
  procedure Put(Index: Int64; const Value: byte);
  function GetCount: int64;
  function GetItemIndex: Int64;
  procedure SetItemIndex(const Value: Int64);
  function GetMaxItems: Int64;
  procedure SetMaxItems(const Value: Int64);
 protected
 public
  constructor Create;
  destructor Destroy; override;
  procedure Open(Filename: string; Mode: Word);
  procedure Close;
  procedure Clear;
  function ReadString(Index, Size: int64): string;
  procedure WriteString(Index: int64; S: string);
  property FileStream: TFileStream read FFileStream;
  property Value[Index: Int64]: byte read Get write Put; default;
  property Count: int64 read GetCount;
  property ItemIndex: Int64 read GetItemIndex write SetItemIndex;
  property MaxItems: Int64 read GetMaxItems write SetMaxItems;
 end;



{------------------------------------------------------------}
{----------- Declaration for Huge array object --------------}
{------------------------------------------------------------}



type THugeArray = class(TObject)
 private
  ItemSize,MemSize: integer;
  MemPointer: Pointer;
  ExpandSize: integer;
  StartMax: integer;
  FMaxItems: integer;
  FSortAscending: boolean;
  FOnChange: TNotifyEvent;
 protected
  procedure SetMaxItems(V: integer);
 public
  ItemIndex: integer;
  Pointers: TList;
  Name: TComponentName;
  constructor Create(Number,Size: LongInt);
  destructor Destroy; override;
  procedure Change; virtual;
  function GetItemPtr(Index: LongInt): Pointer;
  procedure ReAllocMem(NewMaxItems: LongInt);
  procedure ExpandArray;
  procedure Assign(Source: THugeArray);
  procedure Append(Source: THugeArray);
  procedure SaveToStream(Stream: TStream);
  procedure LoadFromStream(Stream: TStream);
  procedure LoadFromFile(const FileName: string);
  procedure SaveToFile(const FileName: string);
  function AddItem(var Item): boolean;
  function AddPointer(var Item; Ptr: integer): Integer;
  function Add(var Item): integer; virtual;
  procedure InsertItem(var Item; Index: integer);
  procedure DeleteItem(Index: integer); virtual;
  procedure PutItem(var Item; Index: integer);
  procedure GetItem(var Item; Index: integer);
  procedure BlockMove(Source,Destin,Size: integer);
  procedure DeleteRange(First, Last: integer);
  procedure Exchange(Index1, Index2: Integer); virtual;
  property SortAscending: boolean read FSortAscending write FSortAscending;
  procedure Sort(Compare: TListSortCompare); overload; virtual;
  procedure Sort; overload; virtual;
  procedure Clear; dynamic;
  procedure FillArray(var Value);
  property Count: integer read ItemIndex write ItemIndex;
  property MaxItems: integer read FMaxItems write SetMaxItems;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Integer Array object ---------}
{------------------------------------------------------------}


type THugeInteger = class(THugeArray)
 private
  procedure Put(Index: integer; Item: integer); virtual;
  function Get(Index: integer): integer;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: integer): integer; reintroduce;
  function AddPointer(Item: integer; Ptr: integer): Integer; virtual;
  procedure Insert(Value: integer; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: integer);
  property Value[Index: integer]: integer read Get write Put; default;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Cardinal Array object --------}
{------------------------------------------------------------}


type THugeCardinal = class(THugeArray)
 private
  procedure Put(Index: Cardinal; Item: Cardinal); virtual;
  function Get(Index: Cardinal): Cardinal;
 protected
 public
  constructor Create(Number: Cardinal);
  function Add(Item: Cardinal): Cardinal; reintroduce;
  function AddPointer(Item: Cardinal; Ptr: Cardinal): Cardinal; virtual;
  procedure Insert(Value: Cardinal; Index: Cardinal); virtual;
  procedure Delete(Index: Cardinal);
  procedure Move(CurIndex, NewIndex: Cardinal);
  procedure Sort; override;
  procedure FillArray(Value: cardinal);
  property Value[Index: Cardinal]: Cardinal read Get write Put; default;
 end;


{------------------------------------------------------------}
{---------- Declaration for Huge Byte Array object ----------}
{------------------------------------------------------------}

type THugeByte = class(THugeArray)
 private
  procedure Put(Index: integer; Item: Byte); virtual;
  function Get(Index: integer): Byte;
  protected
 public
  constructor Create(Number: integer);
  function Add(Item: Byte): integer; reintroduce;
  function AddPointer(Item: byte; Ptr: integer): Integer; virtual;
  procedure Insert(Value: byte; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: byte);
  property Value[Index: integer]: byte read Get write Put; default;
 end;

{------------------------------------------------------------}
{---------- Declaration for Huge char Array object ----------}
{------------------------------------------------------------}

type THugeChar = class(THugeArray)
 private
  procedure Put(Index: integer; Item: char); virtual;
  function Get(Index: integer): char;
  protected
 public
  constructor Create(Number: integer);
  function Add(Item: char): integer; reintroduce;
  function AddPointer(Item: char; Ptr: integer): Integer; virtual;
  procedure Insert(Value: char; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: char);
  property Value[Index: integer]: char read Get write Put; default;
 end;



{------------------------------------------------------------}
{-------- Declaration for Huge Boolean Array object ---------}
{------------------------------------------------------------}


type THugeBoolean = class(THugeArray)
 private
  procedure Put(Index: integer; Item: Boolean); virtual;
  function Get(Index: integer): boolean;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: boolean): integer; reintroduce;
  function AddPointer(Item: boolean; Ptr: integer): Integer; virtual;
  procedure Insert(Value: boolean; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure FillArray(Value: boolean);
  property Value[Index: integer]: boolean read Get write Put; default;
 end;



{------------------------------------------------------------}
{-------- Declaration for Huge Longint Array object ---------}
{------------------------------------------------------------}

type THugeLongInt = THugeInteger;


{------------------------------------------------------------}
{-------- Declaration for Huge Double  Array object ---------}
{------------------------------------------------------------}

type THugeDouble = class(THugeArray)
 private
  procedure Put(Index: integer; Item: double); virtual;
  function Get(Index: integer): double;
 protected
 public
  constructor Create(Number: integer);
 function Add(Item: double): integer; reintroduce;
  function AddPointer(Item: double; Ptr: integer): Integer; virtual;
  procedure Insert(Value: double; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: double);
  property Value[Index: integer]: Double read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge Single  Array object ---------}
{------------------------------------------------------------}

type THugeSingle = class(THugeArray)
 private
  procedure Put(Index: integer; Item: single); virtual;
  function Get(Index: integer): single;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: single): integer; reintroduce;
  function AddPointer(Item: single; Ptr: integer): Integer; virtual;
  procedure Insert(Value: single; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: single);
  property Value[Index: integer]: Single read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge String  Array object ---------}
{------------------------------------------------------------}

type THugeString = class;

 THugeStringSortCompare = function(HugeString: THugeString; Index1, Index2: Integer): Integer;

 THugeString = class(THugeArray)
 private
  FCaseSensitive: boolean;
  procedure Put(Index: integer; Item: ShortString); virtual;
  function Get(Index: integer): ShortString;
  procedure QuickSort(L, R: Integer; SCompare: THugeStringSortCompare);
 protected
 public
  constructor Create(Number,Size: integer);
  destructor Destroy; override;
  procedure CustomSort(Compare: THugeStringSortCompare);
  procedure Exchange(Index1, Index2: Integer); override;
  function Add(Item: ShortString): integer; reintroduce;
  function AddPointer(Item: shortstring; Ptr: integer): Integer; virtual;
  function Compare(Target:ShortString; Index: integer): boolean;
  procedure Clear; override;
  procedure Insert(Value: shortstring; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Sort; override;
  procedure FillArray(Value: ShortString);
  property Value[Index: integer]: ShortString read Get write Put; default;
  property CaseSensitive: boolean read FCaseSensitive write FCaseSensitive;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Long String  Array object ----}
{------------------------------------------------------------}


type THugeLongString = class;

 THugeLongStringSortCompare = function(HugeLongString: THugeLongString; Index1, Index2: Integer): Integer;


 THugeLongString = class(THugeArray)
 private
  FCaseSensitive: boolean;
  procedure Put(Index: integer; Item: string); virtual;
  function Get(Index: integer): string;
  procedure QuickSort(L, R: Integer; SCompare: THugeLongStringSortCompare);
 protected
 public
  constructor Create(Number: integer);
  destructor Destroy; override;
  procedure CustomSort(Compare: THugeLongStringSortCompare);
  procedure Exchange(Index1, Index2: Integer); override;
  function Add(Item: string): integer; reintroduce;
  function AddPointer(Item: string; Ptr: integer): Integer; virtual;
  function Compare(Target: string; Index: integer): boolean;
  procedure Clear; override;
  procedure Insert(Value: string; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: string read Get write Put; default;
  procedure Sort; override;
  procedure FillArray(Value: string);
  property CaseSensitive: boolean read FCaseSensitive write FCaseSensitive;
 end;

{------------------------------------------------------------}
{-------- Declaration for 3DVector Array object -------------}
{------------------------------------------------------------}

type THuge3DVector = class(THugeArray)
 private
 protected
  procedure Put(Index: integer; Item: T3DVector); virtual;
  function Get(Index: integer): T3DVector;
  public
  constructor Create(Number: integer);
  function Add(Item: T3DVector): integer; reintroduce; virtual;
  function AddPointer(Item: T3DVector; Ptr: integer): Integer; virtual;
  procedure Insert(Value: T3DVector; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure FillArray(Value: T3DVector);
  property Value[Index: integer]: T3DVector read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for 2DVector Array object -------------}
{------------------------------------------------------------}

type THuge2DVector = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T2DVector); virtual;
  function Get(Index: integer): T2DVector;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T2DVector): integer; reintroduce; virtual;
  function AddPointer(Item: T2DVector; Ptr: integer): Integer; virtual;
  procedure Insert(Value: T2DVector; Index: integer); virtual;
  procedure Delete(Index: integer); virtual;
  procedure Move(CurIndex, NewIndex: Integer); virtual;
  procedure FillArray(Value: T2DVector);
  property Value[Index: integer]: T2DVector read Get write Put; default;
  procedure Rotate(Center: T2DVector; Angle: double);
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge 3DLine Array object ----------}
{------------------------------------------------------------}


type THuge3DLine = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T3DLine); virtual;
  function Get(Index: integer): T3DLine;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T3DLine): integer; reintroduce; virtual;
  function AddPointer(Item: T3DLine; Ptr: integer): Integer; virtual;
  procedure Insert(Value: T3DLine; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure FillArray(Value: T3DLine);
  procedure Move(CurIndex, NewIndex: Integer);
  property Value[Index: integer]: T3DLine read Get write Put; default;
 end;


{------------------------------------------------------------}
{-------- Declaration for Huge 2DLine Array object ----------}
{------------------------------------------------------------}

type THuge2DLine = class(THugeArray)
 private
  procedure Put(Index: integer; Item: T2DLine); virtual;
  function Get(Index: integer): T2DLine;
 protected
  public
  constructor Create(Number: integer);
  function Add(Item: T2DLine): integer; reintroduce; virtual;
  function AddPointer(Item: T2DLine; Ptr: integer): Integer; virtual;
  procedure Insert(Value: T2DLine; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure Rotate(Center: T2DVector; Angle: double);
  procedure FillArray(Value: T2DLine);
  property Value[Index: integer]: T2DLine read Get write Put; default;
 end;

{------------------------------------------------------------}
{-------- Declaration for Huge Point Array object -----------}
{------------------------------------------------------------}

type THugePoint = class(THugeArray)
 private
  procedure Put(Index: integer; Item: TPoint); virtual;
  function Get(Index: integer): TPoint;
 protected
 public
  constructor Create(Number: integer);
  function Add(Item: TPoint): integer; reintroduce; overload; virtual;
  function Add(X,Y: integer): integer; reintroduce; overload; virtual;
  function AddPointer(Item: TPoint; Ptr: integer): Integer; virtual;
  procedure Insert(Value: TPoint; Index: integer); virtual;
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  procedure FillArray(Value: TPoint);
  property Value[Index: integer]: TPoint read Get write Put; default;
 procedure Rotate(Center: T2DVector; Angle: double);
 end;


function IntSortCompareUp(Item1, Item2: Pointer): integer;
function IntSortCompareDown(Item1, Item2: Pointer): integer;



implementation


{Standard Sort Compare routines}


function IntSortCompareUp(Item1, Item2: Pointer): integer;
begin
Result:=PInteger(Item1)^-PInteger(Item2)^;
end;


function IntSortCompareDown(Item1, Item2: Pointer): integer;
begin
Result:=PInteger(Item2)^-PInteger(Item1)^;
end;

function CardSortCompareUp(Item1, Item2: Pointer): integer;
begin
Result:=PCardinal(Item1)^-PCardinal(Item2)^;
end;


function CardSortCompareDown(Item1, Item2: Pointer): integer;
begin
Result:=PCardinal(Item2)^-PCardinal(Item1)^;
end;


function ByteSortCompareUp(Item1, Item2: Pointer): integer;
begin
Result:=PByte(Item1)^-PByte(Item2)^;
end;


function ByteSortCompareDown(Item1, Item2: Pointer): integer;
begin
Result:=PByte(Item2)^-PByte(Item1)^;
end;




function DoubleSortCompareUp(Item1, Item2: Pointer): integer;
var R: double;
begin
R:=PDouble(Item1)^-PDouble(Item2)^;
if R>0 then Result:=1
else if R<0 then Result:=-1
else Result:=0;
end;


function DoubleSortCompareDown(Item1, Item2: Pointer): integer;
var R: double;
begin
R:=PDouble(Item2)^-PDouble(Item1)^;
if R>0 then Result:=1
else if R<0 then Result:=-1
else Result:=0;
end;



function SingleSortCompareUp(Item1, Item2: Pointer): integer;
var R: double;
begin
R:=PSingle(Item1)^-PSingle(Item2)^;
if R>0 then Result:=1
else if R<0 then Result:=-1
else Result:=0;
end;



function SingleSortCompareDown(Item1, Item2: Pointer): integer;
var R: double;
begin
R:=PSingle(Item2)^-PSingle(Item1)^;
if R>0 then Result:=1
else if R<0 then Result:=-1
else Result:=0;
end;


var CaseSense: boolean;


function ShortStrSortCompareUp(Strings: THugeString; Index1, Index2: Integer): Integer;
begin
if CaseSense then Result:=CompareStr(Strings[Index1],Strings[Index2])
else Result:=CompareText(Strings[Index1],Strings[Index2]);
end;



function ShortStrSortCompareDown(Strings: THugeString; Index1, Index2: Integer): Integer;
begin
if CaseSense then Result:=CompareStr(Strings[Index2],Strings[Index1])
else Result:=CompareText(Strings[Index2],Strings[Index1]);
end;


function StringSortCompareUp(Strings: THugeLongString; Index1, Index2: Integer): Integer;
begin
if CaseSense then Result:=CompareStr(Strings[Index1],Strings[Index2])
else Result:=CompareText(Strings[Index1],Strings[Index2]);
end;


function StringSortCompareDown(Strings: THugeLongString; Index1, Index2: Integer): Integer;
begin
if CaseSense then Result:=CompareStr(Strings[Index2],Strings[Index1])
else Result:=CompareText(Strings[Index2],Strings[Index1]);
end;



{-------------------------------------------------------------------------}
{                  Implementation for TDiskArray object                   }
{-------------------------------------------------------------------------}




procedure TDiskArray.Clear;
begin
if FFileStream<>nil then FFileStream.Size:=0;
end;

procedure TDiskArray.Close;
begin
if FFileStream<>nil then FreeAndNil(FFileStream);
end;



constructor TDiskArray.Create;
begin
Cache.Valid:=False;
FFileStream:=nil;
end;



destructor TDiskArray.Destroy;
begin
if FFileStream<>nil then FreeAndNil(FFileStream);
inherited;
end;



{function TDiskArray.Get(Index: Int64): byte;
var B: Byte;
begin
if (FFileStream=nil) or (Index>=FFileStream.Size) or (Index<0) then exit;
FFileStream.Seek(Index,soBeginning);
FFileStream.ReadBuffer(B,1);
Result:=B;
end;
}

function TDiskArray.Get(Index: Int64): byte;
var B: Byte;
const BuffSize = 512;
var Mask: int64;
begin
Mask:=not (BuffSize-1);
if (FFileStream=nil) or (Index>=FFileStream.Size) or (Index<0) then exit;

if not Cache.Valid or
   (Index<Cache.Start) or
   (Index>=Cache.Start+BuffSize) then
	begin
	Cache.Start:=Index and Mask;
	FFileStream.Seek(Cache.Start,soBeginning);
	FFileStream.ReadBuffer(Cache.Buffer,512);
	Cache.Valid:=True;
	end;

Result:=Cache.Buffer[Index-Cache.Start];
end;



function TDiskArray.GetCount: int64;
begin
if FFileStream<>nil then Result:=FFileStream.Size
else Result:=0;
end;



function TDiskArray.GetItemIndex: int64;
begin
if FFileStream<>nil then Result:=FFileStream.Position
else Result:=0;
end;

function TDiskArray.GetMaxItems: Int64;
begin
if FFileStream<>nil then Result:=FFileStream.Size
else Result:=0;
end;

procedure TDiskArray.Open(Filename: string; Mode: Word);
begin
FFileStream:=TFileStream.Create(Filename,Mode);
end;



procedure TDiskArray.Put(Index: Int64; const Value: byte);
begin
if (FFileStream=nil) or (Index>=FFileStream.Size) or (Index<0) then exit;
Cache.Valid:=False;
FFileStream.Seek(Index,soBeginning);
FFileStream.WriteBuffer(Value,1);
end;



function TDiskArray.ReadString(Index, Size: int64): string;
begin
if FFileStream<>nil then
	begin
	SetLength(Result,Size);
	FFileStream.Seek(Index,soBeginning);
	FFileStream.ReadBuffer(Result[1],Size);
	end
else Result:='';
end;



procedure TDiskArray.SetItemIndex(const Value: int64);
begin
if FFileStream<>nil then FFileStream.Position:=Value;
end;



procedure TDiskArray.SetMaxItems(const Value: Int64);
begin
if FFileStream<>nil then FFileStream.Size:=Value;
end;



procedure TDiskArray.WriteString(Index: int64; S: string);
begin
if FFileStream<>nil then
	begin
	FFileStream.Seek(Index,soBeginning);
	FFileStream.WriteBuffer(S[1],Length(S));
	end;
end;

{-------------------------------------------------------------------------}
{                  Implementation for huge array object                   }
{-------------------------------------------------------------------------}




constructor THugeArray.Create(Number,Size: integer);
{Create a huge array item}
begin
inherited Create;
Pointers:=TList.Create;
FMaxItems:=Number;
StartMax:=Number;
ItemSize:=Size;
MemSize:=FMaxItems*ItemSize;
if MemSize=0 then raise Exception.Create('Empty Array Allocation');
{It doesn't work to test memory size with "GlobalMemoryStatus"
here because Delphi's memory manager may not have a contiguous
block big enough to allocate the full amount}
GetMem(MemPointer,MemSize);
if MemPointer=nil then OutOfMemoryError;
ItemIndex:=0;
ExpandSize:=4;
SortAscending:=True;
end;



procedure THugeArray.Change;
{Trigger on change event}
begin
if Assigned(FOnChange) then FOnChange(Self);
end;



destructor THugeArray.Destroy;
{Destroy Huge array item}
begin
FreeMem(MemPointer);
Pointers.Free;
inherited Destroy;
end;


function THugeArray.GetItemPtr(Index: integer): Pointer;
{ Return pointer to item based on index }
begin
Result:=Pointer(Integer(MemPointer) + ItemSize*Index);
end;


procedure THugeArray.ReAllocMem(NewMaxItems: integer);
{ Reallocate memory for new number of items }
begin
if NewMaxItems =0 then raise Exception.Create('Empty Array Allocation');

MemSize:=NewMaxItems*ItemSize;
FMaxItems :=MemSize div ItemSize;
System.ReallocMem(MemPointer,MemSize);
end;



procedure THugeArray.FillArray(var Value);
begin
FillPattern(GetItemPtr(0),@Value,Count*ItemSize,ItemSize);
end;


procedure THugeArray.SetMaxItems(V: integer);
begin
if V<>FMaxItems then
	begin
	if V<1 then V:=1;
	ReAllocMem(V);
	if Count>FMaxItems then Count:=FMaxItems;
	end;
end;



procedure THugeArray.ExpandArray;
{Expand array if necessary}
begin
if ItemIndex>=FMaxItems then
	begin
	FMaxItems:=ItemIndex;
	if FMaxItems > 64 then ExpandSize := FMaxItems div 4
	else if FMaxItems > 8 then ExpandSize := 16
	else ExpandSize := 4;
	ReAllocMem(FMaxItems + ExpandSize);
	end;
end;



procedure THugeArray.SaveToStream(Stream: TStream);
begin
Stream.Write(MemPointer^,ItemSize * Count);
end;



procedure THugeArray.LoadFromStream(Stream: TStream);
var Items: integer;
begin
Items:=Stream.Size div ItemSize;
ReAllocMem(Items);
Stream.Position:=0;
{Set the size this way in case the file}
{is not an even multiple of ItemSize}
Stream.Read(MemPointer^, Items * ItemSize);
Count:=Items;
Change;
end;



procedure THugeArray.LoadFromFile(const FileName: string);
var Stream: TStream;
begin
Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
try
LoadFromStream(Stream);
finally Stream.Free; end;
end;



procedure THugeArray.SaveToFile(const FileName: string);
var Stream: TStream;
begin
Stream := TFileStream.Create(FileName, fmCreate);
try
SaveToStream(Stream);
finally Stream.Free; end;
end;




function THugeArray.AddPointer(var Item; Ptr: integer): Integer;
{Add Item and Pointer to arrays}
begin
Result:=ItemIndex;
ExpandArray;
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*ItemIndex)^,ItemSize);
ItemIndex:=ItemIndex+1;
Pointers.Add(Pointer(Ptr));
Change;
end;



function THugeArray.AddItem(var Item): boolean;
{ Add item to the array, expanding memory if necessary }
begin
AddPointer(Item,0);
Result:=True;
end;


function THugeArray.Add(var Item): integer;
{ Add item to the array, expanding memory if necessary }
begin
Result:=AddPointer(Item,0);
end;


procedure THugeArray.PutItem(var Item; Index: integer);
{overwrite item at index}
begin
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*Index)^,ItemSize);
Change;
end;


procedure THugeArray.InsertItem(var Item; Index: integer);
{Insert item at index}
var Source,Destination: pointer;
var Size: integer;
begin
if (Index<0) or (Index> ItemIndex) then exit;
ExpandArray;
Source:=Pointer(Integer(MemPointer) + ItemSize*Index);
Destination:=Pointer(Integer(Source) + ItemSize);
Size:=(ItemIndex - Index) * ItemSize;
System.Move(Source^,Destination^,Size);
System.Move(Item,Pointer(Integer(MemPointer) + ItemSize*Index)^,ItemSize);
Inc(ItemIndex);
Pointers.Insert(Index,0);
Change;
end;


procedure THugeArray.DeleteItem(Index: integer);
{Delete item at index}
var Source,Destination: pointer;
var Size: integer;
begin
if (Index<0) or (Index>=ItemIndex) then exit;
Source:=Pointer(Integer(MemPointer) + ItemSize*(Index+1));
Destination:=Pointer(Integer(Source) - ItemSize);
Size:=(ItemIndex - (Index+1)) * ItemSize;
System.Move(Source^,Destination^,Size);
Dec(ItemIndex);
Pointers.Delete(Index);
Change;
end;


procedure THugeArray.GetItem(var Item; Index: integer);
{Get item at the index}
begin
System.Move(Pointer(Integer(MemPointer) + ItemSize*Index)^,Item,ItemSize);
end;



procedure THugeArray.BlockMove(Source,Destin,Size: integer);
{Do cross-segment huge block move}
begin
System.Move(GetItemPtr(Source)^,GetItemPtr(Destin)^,Size);
end;


procedure THugeArray.Clear;
{Clear data and reset to original state}
begin
ItemIndex:=0;
ReAllocMem(StartMax);
Pointers.Clear;
Change;
end;


procedure THugeArray.Exchange(Index1, Index2: Integer);
{Exchange the contents of pointed to by Index1 and Index2}
var B: byte;
var P1,P2: PByteArray;
var I: integer;
begin
P1:=GetItemPtr(Index1);
P2:=GetItemPtr(Index2);
for I:=0 to ItemSize-1 do
	begin
	B:=P1[I]; P1[I]:=P2[I]; P2[I]:=B;
	end;
Pointers.Exchange(Index1,Index2);
Change;
end;



procedure THugeArray.Assign(Source: THugeArray);
begin
Self.Clear;
ItemIndex:=Source.ItemIndex;
ExpandArray;
System.Move(Source.MemPointer^,MemPointer^,ItemSize*Source.Count);
Pointers.Assign(Source.Pointers);
Change;
end;



procedure THugeArray.Append(Source: THugeArray);
var I: integer;
begin
for I:=0 to Source.Count-1 do Self.Add(Source.GetItemPtr(I)^);
end;



procedure THugeArray.DeleteRange(First,Last: integer);
var Number,Size: integer;
begin
if First<0 then First:=0;
if Last>=Count then Last:=Count-1;
if First>Last then raise Exception.Create('Range Error in DeleteRange');
Number:=(Last-First)+1;
Size:=(Count-Last)*ItemSize;
BlockMove(Last+1,First,Size);
Dec(ItemIndex,Number);
System.Move(Pointers.List[Last+1], Pointers.List[First], (Pointers.Count-Last) * SizeOf(Pointer));
Pointers.Count:=Count;
Change;
end;




procedure QuickSort(SortList: THugeArray; L, R: Integer; SCompare: TListSortCompare);
{Do quick sort on items held in HugeArray}
var I, J: Integer;
var P: Pointer;
begin
repeat
	begin
	I := L;
	J := R;
	P := SortList.GetItemPtr((L + R) shr 1);
	repeat
		begin
		while SCompare(SortList.GetItemPtr(I), P) < 0 do Inc(I);
		while SCompare(SortList.GetItemPtr(J), P) > 0 do Dec(J);
		if I <= J then
			begin
			SortList.Exchange(I,J);
			{This part added to make it work with pointers (not sure why)}
			if P = SortList.GetItemPtr(I) then P := SortList.GetItemPtr(J)
			else if P = SortList.GetItemPtr(J) then P := SortList.GetItemPtr(I);
			Inc(I);
			Dec(J);
			end;
		end
	until I > J;
	if L < J then QuickSort(SortList, L, J, SCompare);
	L := I;
	end
until I >= R;
end;



procedure THugeArray.Sort(Compare: TListSortCompare);
begin
if (Self<>nil) and (ItemIndex>0) then QuickSort(Self, 0, ItemIndex-1,Compare);
end;


procedure THugeArray.Sort;
begin
{Virtual procedure filled in by descendent classes}
end;


{-------------------------------------------------------------------------}
{ Implementation for huge byte array object based on huge array object    }
{-------------------------------------------------------------------------}


constructor THugeByte.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(Byte));
end;


function THugeByte.Add(Item: byte): integer;
{Add item to byte array}
begin
Result:=inherited Add(Item);
end;


function THugeByte.AddPointer(Item: byte; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeByte.Put(Index: integer; Item: byte);
{Insert byte at index}
begin
PutItem(Item,Index);
end;


function THugeByte.Get(Index: integer): byte;
{Get byte at the index}
var PTR: ^Byte;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeByte.Insert(Value: byte; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeByte.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugeByte.Move(CurIndex, NewIndex: Integer);
var T: Byte;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeByte.Sort;
begin
inherited;
if SortAscending then Sort(ByteSortCompareUp)
else Sort(ByteSortCompareDown);
end;


procedure THugeByte.FillArray(Value: byte);
begin
Inherited FillArray(Value);
end;


{-------------------------------------------------------------------------}
{ Implementation for huge char array object based on huge array object    }
{-------------------------------------------------------------------------}


constructor THugeChar.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(char));
end;


function THugeChar.Add(Item: char): integer;
{Add item to byte array}
begin
Result:=inherited Add(Item);
end;

function THugeChar.AddPointer(Item: char; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;

procedure THugeChar.Put(Index: integer; Item: char);
{Insert byte at index}
begin
PutItem(Item,Index);
end;


function THugeChar.Get(Index: integer): char;
{Get byte at the index}
var PTR: ^char;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;



procedure THugeChar.Insert(Value: char; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeChar.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeChar.Move(CurIndex, NewIndex: Integer);
var T: Char;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeChar.Sort;
begin
inherited;
if SortAscending then Sort(ByteSortCompareUp)
else Sort(ByteSortCompareDown);
end;


procedure THugeChar.FillArray(Value: char);
begin
Inherited FillArray(Value);
end;



{-------------------------------------------------------------------------}
{ Implementation for huge boolean array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeBoolean.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(boolean));
end;


function THugeBoolean.Add(Item: boolean): integer;
{Add item to boolean array}
begin
Result:=inherited Add(Item);
end;


function THugeBoolean.AddPointer(Item: boolean; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeBoolean.Put(Index: integer; Item: boolean);
{Insert boolean at index}
begin
PutItem(Item,Index);
end;

function THugeBoolean.Get(Index: integer): boolean;
{Get boolean at the index}
var PTR: ^Boolean;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeBoolean.Insert(Value: boolean; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeBoolean.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeBoolean.Move(CurIndex, NewIndex: Integer);
var T: Boolean;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeBoolean.FillArray(Value: boolean);
begin
Inherited FillArray(Value);
end;



{-------------------------------------------------------------------------}
{ Implementation for huge integer array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeInteger.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(integer));
end;


function THugeInteger.Add(Item: integer): integer;
{Add item to integer array}
begin
Result:=inherited Add(Item);
end;


function THugeInteger.AddPointer(Item: integer; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeInteger.Put(Index: integer; Item: integer);
{Insert integer at index}
begin
PutItem(Item,Index);
end;

function THugeInteger.Get(Index: integer): integer;
{Get integer at the index}
var PTR: ^Integer;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeInteger.Insert(Value: integer; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeInteger.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeInteger.Move(CurIndex, NewIndex: Integer);
var T: integer;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


procedure THugeInteger.Sort;
begin
inherited;
if SortAscending then Sort(IntSortCompareUp)
else Sort(IntSortCompareDown);
end;


procedure THugeInteger.FillArray(Value: integer);
begin
Inherited FillArray(Value);
end;


{-------------------------------------------------------------------------}
{ Implementation for huge Cardinal array object based on huge array object }
{-------------------------------------------------------------------------}


constructor THugeCardinal.Create(Number: Cardinal);
{Create a huge Cardinal array}
begin
inherited Create(Number,SizeOf(Cardinal));
end;


function THugeCardinal.Add(Item: Cardinal): Cardinal;
{Add item to Cardinal array}
begin
Result:=inherited Add(Item);
end;


function THugeCardinal.AddPointer(Item: Cardinal; Ptr: Cardinal): Cardinal;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeCardinal.Put(Index: Cardinal; Item: Cardinal);
{Insert Cardinal at index}
begin
PutItem(Item,Index);
end;

function THugeCardinal.Get(Index: Cardinal): Cardinal;
{Get Cardinal at the index}
var PTR: ^Cardinal;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeCardinal.Insert(Value: Cardinal; Index: Cardinal);
begin
InsertItem(Value,Index);
end;


procedure THugeCardinal.Delete(Index: Cardinal);
begin
DeleteItem(Index);
end;


procedure THugeCardinal.Move(CurIndex, NewIndex: Cardinal);
var T: Cardinal;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeCardinal.Sort;
begin
inherited;
if SortAscending then Sort(CardSortCompareUp)
else Sort(CardSortCompareDown);
end;



procedure THugeCardinal.FillArray(Value: cardinal);
begin
Inherited FillArray(Value);
end;

{------------------------------------------------------------------------}
{  Implementation for huge double array object based on huge array object}
{------------------------------------------------------------------------}


constructor THugeDouble.Create(Number: integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(double));
end;


function THugeDouble.Add(Item: double): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THugeDouble.AddPointer(Item: double; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeDouble.Put(Index: integer; Item: double);
{Insert Double at index}
begin
PutItem(Item,Index);
end;


function THugeDouble.Get(Index: integer): Double;
{Get Double at the index}
var PTR: ^Double;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeDouble.Insert(Value: double; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeDouble.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeDouble.Move(CurIndex, NewIndex: Integer);
var T: double;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeDouble.Sort;
begin
inherited;
if SortAscending then Sort(DoubleSortCompareUp)
else Sort(DoubleSortCompareDown);
end;



procedure THugeDouble.FillArray(Value: double);
begin
Inherited FillArray(Value);
end;


{-----------------------------------------------------------------------}
{ Implementation for huge single array object based on huge array object}
{-----------------------------------------------------------------------}

constructor THugeSingle.Create(Number: integer);
{Create a huge single array}
begin
inherited Create(Number,SizeOf(single));
end;


function THugeSingle.Add(Item: single): integer;
{Add item to single array}
begin
Result:=inherited Add(Item);
end;


function THugeSingle.AddPointer(Item: single; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugeSingle.Put(Index: integer; Item: single);
{Insert Single at index}
begin
PutItem(Item,Index);
end;

function THugeSingle.Get(Index: integer): Single;
{Get Single at the index}
var PTR: ^Single;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugeSingle.Insert(Value: Single; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugeSingle.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugeSingle.Move(CurIndex, NewIndex: Integer);
var T: single;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeSingle.Sort;
begin
inherited;
if SortAscending then Sort(SingleSortCompareUp)
else Sort(SingleSortCompareDown);
end;



procedure THugeSingle.FillArray(Value: single);
begin
Inherited FillArray(Value);
end;

{-----------------------------------------------------------------------}
{ Implementation for huge string array object based on huge array object}
{-----------------------------------------------------------------------}


constructor THugeString.Create(Number,Size: integer);
{Create a huge string array}
begin
CaseSensitive:=True;
inherited Create(Number,Size+1);
end;



destructor THugeString.Destroy;
{Destroy Huge array item}
begin
inherited Destroy;
end;



procedure THugeString.Clear;
begin
inherited Clear;
end;


procedure THugeString.Exchange(Index1, Index2: Integer);
begin
inherited Exchange(Index1,Index2);
end;



procedure THugeString.QuickSort(L, R: Integer; SCompare: THugeStringSortCompare);
var I, J, P: Integer;
begin
repeat
	begin
	I := L; J := R;
	P := (L + R) shr 1;
	repeat
		begin
		while SCompare(Self, I, P) < 0 do Inc(I);
		while SCompare(Self, J, P) > 0 do Dec(J);
		if I <= J then
			begin
			Exchange(I, J);
			if P = I then P := J
			else if P = J then P := I;
			Inc(I);
			Dec(J);
			end;
		end
		until I > J;
	if L < J then QuickSort(L, J, SCompare);
	L := I;
	end
until I >= R;
end;



procedure THugeString.CustomSort(Compare: THugeStringSortCompare);
begin
QuickSort(0, ItemIndex - 1, Compare);
end;



function THugeString.AddPointer(Item: shortstring; Ptr: integer): Integer;
{Add string and Pointer}
begin
Result:=inherited AddPointer(Item,Ptr);
end;


function THugeString.Add(Item: ShortString): integer;
{Add item to string array}
begin
Result:=inherited Add(Item);
end;



procedure THugeString.Put(Index: integer; Item: ShortString);
{Insert String at index}
begin
PutItem(Item,Index);
end;


function THugeString.Get(Index: integer): ShortString;
{Get String at the index}
begin
System.Move(GetItemPtr(Index)^,Result,ItemSize);
end;


function THugeString.Compare(Target: ShortString; Index: integer): boolean;
{Compare Target to string at the index}
begin
if CaseSensitive then Result:=Target=Value[Index]
else Result:=CompareText(Target,Value[Index]) = 0;
end;


procedure THugeString.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THugeString.Insert(Value: shortstring; Index: integer);
begin
InsertItem(Value,Index);
end;



procedure THugeString.Move(CurIndex, NewIndex: Integer);
var T: shortstring;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;




procedure THugeString.Sort;
begin
inherited;
CaseSense:=Self.CaseSensitive;
if SortAscending then CustomSort(ShortStrSortCompareUp)
else CustomSort(ShortStrSortCompareDown);
end;



procedure THugeString.FillArray(Value: ShortString);
{This isn't as fast as the way the non-string versions work}
var I: integer;
begin
for I:=0 to Count-1 do Self[I]:=Value;
end;


{-----------------------------------------------------------------------------}
{ Implementation for huge long string array object based on huge array object }
{-----------------------------------------------------------------------------}


constructor THugeLongString.Create(Number: integer);
{Create a huge string array}
begin
CaseSensitive:=True;
inherited Create(Number,SizeOf(string));
end;



destructor THugeLongString.Destroy;
{Destroy Huge array item}
var I: integer;
var P: PAnsiString;
begin
for I:=0 to Count-1 do
	begin
	P:=PAnsiString(GetItemPtr(I)^);
	Dispose(P);
	end;
inherited Destroy;
end;



procedure THugeLongString.Clear;
var I: integer;
var P: PAnsiString;
begin
for I:=0 to Count-1 do
	begin
	P:=PAnsiString(GetItemPtr(I)^);
	Dispose(P);
	end;
inherited Clear;
end;


procedure THugeLongString.Exchange(Index1, Index2: Integer);
begin
inherited Exchange(Index1,Index2);
end;



procedure THugeLongString.QuickSort(L, R: Integer; SCompare: THugeLongStringSortCompare);
var I, J, P: Integer;
begin
repeat
	begin
	I := L; J := R;
	P := (L + R) shr 1;
	repeat
		begin
		while SCompare(Self, I, P) < 0 do Inc(I);
		while SCompare(Self, J, P) > 0 do Dec(J);
		if I <= J then
			begin
			Exchange(I, J);
			if P = I then P := J
			else if P = J then P := I;
			Inc(I);
			Dec(J);
			end;
		end
		until I > J;
	if L < J then QuickSort(L, J, SCompare);
	L := I;
	end
until I >= R;
end;



procedure THugeLongString.CustomSort(Compare: THugeLongStringSortCompare);
begin
QuickSort(0, ItemIndex - 1, Compare);
end;



function THugeLongString.AddPointer(Item: string; Ptr: integer): Integer;
{Add string and Pointer}
var P: PAnsiString;
begin
New(P);
P^:=Item;
Result:=inherited AddPointer(P,Ptr);
end;



function THugeLongString.Add(Item: string): integer;
{Add item to string array}
begin
Result:=AddPointer(Item,0);
end;



procedure THugeLongString.Put(Index: integer; Item: string);
{Insert String at index}
var P: PAnsiString;
begin
P:=PAnsiString(GetItemPtr(Index)^);
P^:=Item;
Change;
end;



function THugeLongString.Get(Index: integer): string;
{Get String at the index}
var S: ^string;
begin
System.Move(GetItemPtr(Index)^,S,ItemSize);
Result:=S^;
end;


function THugeLongString.Compare(Target :string; Index: integer): boolean;
{Compare Target to string at the index}
begin
if CaseSensitive then Result:=Target=Value[Index]
else Result:=CompareText(Target,Value[Index]) = 0;
end;


procedure THugeLongString.Delete(Index: integer);
var P: PAnsiString;
begin
P:=PAnsiString(GetItemPtr(Index)^);
Dispose(P);
DeleteItem(Index);
end;


procedure THugeLongString.Insert(Value: string; Index: integer);
var P: PAnsiString;
begin
P:=NewStr(Value);
InsertItem(P,Index);
end;



procedure THugeLongString.Move(CurIndex, NewIndex: Integer);
var T: string;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THugeLongString.Sort;
begin
inherited;
CaseSense:=Self.CaseSensitive;
if SortAscending then CustomSort(StringSortCompareUp)
else CustomSort(StringSortCompareDown);
end;



procedure THugeLongString.FillArray(Value: string);
{This isn't as fast as the way the non-string versions work}
var I: integer;
begin
for I:=0 to Count-1 do Self[I]:=Value;
end;

{------------------------------------------------------------------------}
{  Implementation for huge Vector array object based on huge array object}
{------------------------------------------------------------------------}

constructor THuge3DVector.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T3DVector));
end;


function THuge3DVector.Add(Item: T3DVector): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge3DVector.AddPointer(Item: T3DVector; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THuge3DVector.Put(Index: integer; Item: T3DVector);
{Insert Double at index}
begin
PutItem(Item,Index);
end;


function THuge3DVector.Get(Index: integer): T3DVector;
{Get Double at the index}
var PTR: ^T3DVector;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge3DVector.Insert(Value: T3DVector; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge3DVector.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge3DVector.Move(CurIndex, NewIndex: Integer);
var T: T3DVector;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THuge3DVector.FillArray(Value: T3DVector);
begin
Inherited FillArray(Value);
end;



{------------------------------------------------------------------------}
{  Implementation for 2DVector array object based on huge array object   }
{------------------------------------------------------------------------}

constructor THuge2DVector.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T2DVector));
end;


function THuge2DVector.Add(Item: T2DVector): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge2DVector.AddPointer(Item: T2DVector;  Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THuge2DVector.Put(Index: integer; Item: T2DVector);
{Insert Double at index}
begin
PutItem(Item,Index);
end;

function THuge2DVector.Get(Index: integer): T2DVector;
{Get Double at the index}
var PTR: ^T2DVector;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge2DVector.Insert(Value: T2DVector; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge2DVector.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge2DVector.Move(CurIndex, NewIndex: Integer);
var T: T2DVector;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


procedure THuge2DVector.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var P: T2DVector;
begin
for I:=0 to Count-1 do
	begin
	P:=Self[I];
	RotatePoint2D(P,Center,Angle);
	Self[I]:=P;
	end;
end;



procedure THuge2DVector.FillArray(Value: T2DVector);
begin
Inherited FillArray(Value);
end;



{------------------------------------------------------------------------}
{  Implementation for huge 3DLine array object based on huge array object}
{------------------------------------------------------------------------}

constructor THuge3DLine.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T3DLine));
end;


function THuge3DLine.Add(Item: T3DLine): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;



function THuge3DLine.AddPointer(Item: T3DLine; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THuge3DLine.Put(Index: integer; Item: T3DLine);
{Insert Double at index}
begin
PutItem(Item,Index);
end;

function THuge3DLine.Get(Index: integer): T3DLine;
{Get Double at the index}
var PTR: ^T3DLine;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge3DLine.Insert(Value: T3DLine; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge3DLine.Delete(Index: integer);
begin
DeleteItem(Index);
end;


procedure THuge3DLine.Move(CurIndex, NewIndex: Integer);
var T: T3DLine;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;



procedure THuge3DLine.FillArray(Value: T3DLine);
begin
Inherited FillArray(Value);
end;

{------------------------------------------------------------------------}
{  Implementation for huge Vector array object based on huge array object}
{------------------------------------------------------------------------}

constructor THugePoint.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(TPoint));
end;


function THugePoint.Add(Item: TPoint): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THugePoint.Add(X,Y: integer): integer;
{Add item to double array}
var P: TPoint;
begin
P:=Point(X,Y);
Result:=inherited Add(P);
end;





function THugePoint.AddPointer(Item: TPoint; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THugePoint.Put(Index: integer; Item: TPoint);
{Insert Double at index}
begin
PutItem(Item,Index);
end;

function THugePoint.Get(Index: integer): TPoint;
{Get Double at the index}
var PTR: ^TPoint;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THugePoint.Insert(Value: TPoint; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THugePoint.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THugePoint.Move(CurIndex, NewIndex: Integer);
var T: TPoint;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


procedure THugePoint.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var P: T2DVector;
begin
for I:=0 to Count-1 do
	begin
	P:=PointToVector2D(Self[I]);
	RotatePoint2D(P,Center,Angle);
	Self[I]:=Vector2DToPoint(P);
	end;
end;



procedure THugePoint.FillArray(Value: TPoint);
begin
Inherited FillArray(Value);
end;


{------------------------------------------------------------------------}
{ Implementation for huge 2D Line array object based on huge array object}
{------------------------------------------------------------------------}


constructor THuge2DLine.Create(Number: Integer);
{Create a huge double array}
begin
inherited Create(Number,SizeOf(T2DLine));
end;


function THuge2DLine.Add(Item: T2DLine): integer;
{Add item to double array}
begin
Result:=inherited Add(Item);
end;


function THuge2DLine.AddPointer(Item: T2DLine; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure THuge2DLine.Put(Index: integer; Item: T2DLine);
{Insert Double at index}
begin
PutItem(Item,Index);
end;

function THuge2DLine.Get(Index: integer): T2DLine;
{Get Double at the index}
var PTR: ^T2DLine;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure THuge2DLine.Insert(Value: T2DLine; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure THuge2DLine.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure THuge2DLine.Move(CurIndex, NewIndex: Integer);
var T: T2DLine;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;




procedure THuge2DLine.Rotate(Center: T2DVector; Angle: double);
var I: integer;
var L: T2DLine;
begin
for I:=0 to Count-1 do
	begin
	L:=Self[I];
	RotateLine2D(L,Center,Angle);
	Self[I]:=L;
	end;
end;



procedure THuge2DLine.FillArray(Value: T2DLine);
begin
Inherited FillArray(Value);
end;


{}



end.
