unit IEClipper;

interface

uses Windows,Classes,Controls,ExtCtrls,Types,SysUtils,Graphics,Vector,GraphicsLib,
      GLPolygons,HugeArry,Math, IEPolygonLib, BitmapLib;



type TPolyClipper = class(TObject)
 private
  Region,OldRegion: HRGN;
  BMP: TBitmap;
 protected
 public
  Enabled: boolean;
  SplinePoints: integer;
  constructor Create;
  destructor Destroy; override;
  procedure ClearClipper(Canvas: TCanvas);
  procedure SetClipper(Canvas: TCanvas);
  function InitializeBMPClip(TargetBMP: TBitmap): TBitmap;
  procedure DoBMPClip(TargetBMP: TBitmap);
 end;

var PolyClipper: TPolyClipper;


var PolygonScale: double = 1;

procedure DrawAllPolygons(Canvas: TCanvas; HandleType: THandleType);
procedure UpdatePolygons(Scale: double);

procedure HandlePolygonDoubleClick(Scale: double);
procedure PolygonMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Scale: double);
procedure PolygonMouseMove(Shift: TShiftState; X, Y: Integer; Scale: double);
procedure HandlePolygonMouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

procedure PolyDrawMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Scale: double);
procedure PolyDrawMouseMove(Shift: TShiftState; X, Y: Integer; Scale: double);

function GetHandleType: THandleType;



implementation

uses IEPolygonUndo, ImageEditor;



{------------------------------------------------------------------------------}



function GetHandleType: THandleType;
begin
case ImageEditorDlg.HandleRadio.ItemIndex of
 0: Result:=htPoint;
 1: Result:=htSpline;
 2: Result:=htNone;
 else Result:=htNone;
 end;
end;







procedure DisplayPolygonNodeInfo;
var P: T2DVector;
begin
if SelectPoint.Valid then
	begin
	P:=PolygonArray[SelectPoint.PolyInx][SelectPoint.PointInx];
	ImageEditorDlg.PolyDis.Caption:=IntToStr(SelectPoint.PolyInx)+'/'+IntToStr(SelectPoint.PointInx);
	ImageEditorDlg.PolyPosDis.Caption:=FloatToStrF(P.X,ffFixed,18,2)+' '+FloatToStrF(P.Y,ffFixed,18,2);
	end
else
	begin
	ImageEditorDlg.PolyDis.Caption:='';
	ImageEditorDlg.PolyPosDis.Caption:='';
	end;
end;




procedure ScalePolygons(PA: TRealPolygonArray);
var I,J: integer;
begin
for I:=0 to PA.Count-1 do ScaleRealPolygon(PA[I],PolygonScale);
end;









procedure DrawAllPolygons(Canvas: TCanvas; HandleType: THandleType);
var I,J: integer;
var Poly: TRealPolygon;
var PA: TRealPolygonArray;
var C: TColor;
var S: string;
var Selected: Boolean;
begin
PA:=TRealPolygonArray.Create;
PA.Assign(PolygonArray);
ScalePolygons(PA);
for I:=0 to PA.Count-1 do
	begin
	Selected:=SelectPoint.Valid and (SelectPoint.PolyInx=I);
	DrawPolygonLoop(Canvas,PA[I],HandleType,Selected,ImageEditorDlg.SegmentsSpin.Value);
	end;
if SelectPoint.Valid and (HandleType=htPoint) then
	begin
	HighlightSelectedPoint(Canvas,PA[SelectPoint.PolyInx],SelectPoint.PointInx);
	end;
PA.Free;
DisplayPolygonNodeInfo;
end;



procedure UpdatePolygons(Scale: double);
begin
HandleSize:=Round(5/Scale);
HandleSize:=EnsureRange(HandleSize,2,10);
with ImageEditorDlg do
 begin
 Image.RestoreImage;
 DrawAllPolygons(Image.Canvas,GetHandleType);
 Image.InitiateAction;
 Image.Invalidate;
 end;
end;




{------------------- Handle Point Hit testing ---------------------------------}



function FindNearestPoint(Pos: T2DVector): THitPoint;
var I,J,Inx1,Inx2: integer;
var R: T2DRect;
var Line: T2DLine;
var Pnt1,Pnt2: T2DVector;
var Dist,T,D1,D2: double;
var H1,H2: THitPoint;
begin
Dist:=MaxDouble;
Result.Valid:=False;
for I:=0 to PolygonArray.Count-1 do
 for J:=0 to PolygonArray[I].Count do
	begin
	Inx1:=J;
	if J=PolygonArray[I].Count then Inx2:=0 else Inx2:=Inx1+1;
	Line.P1:=ScalarProduct2D(PolygonArray[I][Inx1],PolygonScale);
	Line.P2:=ScalarProduct2D(PolygonArray[I][Inx2],PolygonScale);
	T:=PointLineSegDist2D(Pos,Line);
	if T<Dist then
		begin
		Dist:=T;
		D1:=VectorAbs2D(VectorSubtract2D(Line.P1,Pos));
		D2:=VectorAbs2D(VectorSubtract2D(Line.P2,Pos));
		if D1<D2 then Result.PointInx:=Inx1 else Result.PointInx:=Inx2;
		Result.PolyInx:=I;
		Result.Valid:=True;
		Result.HitType:=htHandle;
		end;
	end;
end;




function PointHitTest(Pos: T2DVector): THitPoint;
var I,J: integer;
var R: T2DRect;
var Pnt: T2DVector;
begin
Result.Valid:=False;
if not SelectPoint.Valid then exit;
I:=SelectPoint.PolyInx;
if I>=PolygonArray.Count then exit;
for J:=0 to PolygonArray[I].Count-1 do
	begin
	Pnt:=ScalarProduct2D(PolygonArray[I][J],PolygonScale);
	R:=MakeRect2D(Pnt.X-HandleSize,Pnt.Y-HandleSize,Pnt.X+HandleSize,Pnt.Y+HandleSize);
	if PointInRect2D(Pos,R) then
		begin
		Result.PolyInx:=I;
		Result.PointInx:=J;
		Result.Valid:=True;
		Result.HitType:=htHandle;
		exit;
		end;
	end;
end;



function ControlPointHitTest(Pos: T2DVector): THitPoint;
var I,J: integer;
var R: T2DRect;
var P: TPoint;
var Pnt: T2DVector;
var RP: TRealPolygon;
begin
Result.Valid:=False;
if not SelectPoint.Valid then exit;
I:=SelectPoint.PolyInx;
if I>=PolygonArray.Count then exit;
for J:=0 to PolygonArray[I].Count-1 do
	begin
	RP:=PolygonArray[I];
	Pnt:=ScalarProduct2D(VectorAdd2D(RP[J],RP.ControlPoints1[J]),PolygonScale);
	R:=MakeRect2D(Pnt.X-HandleSize,Pnt.Y-HandleSize,Pnt.X+HandleSize,Pnt.Y+HandleSize);
	if PointInRect2D(Pos,R) then
		begin
		Result.PolyInx:=I; Result.PointInx:=J;
		Result.Valid:=True; Result.HitType:=htControl1;
		exit;
		end;
	Pnt:=ScalarProduct2D(VectorAdd2D(RP[J],RP.ControlPoints2[J]),PolygonScale);
	R:=MakeRect2D(Pnt.X-HandleSize,Pnt.Y-HandleSize,Pnt.X+HandleSize,Pnt.Y+HandleSize);
	if PointInRect2D(Pos,R) then
		begin
		Result.PolyInx:=I; Result.PointInx:=J;
		Result.Valid:=True; Result.HitType:=htControl2;
		exit;
		end;
	end;
end;



procedure LineHitTest(Pos: T2DVector; var LineInfo: TLineInfo);
var I,J: integer;
var D: double;
var Intercept: T3DVector;
begin
LineInfo.Distance:=1E9;
Pos:=ScalarDivide2D(Pos,PolygonScale);
for I:=0 to PolygonArray.Count-1 do
	begin
	if PolygonLineHitTest(Pos,PolygonArray[I],LineInfo,ImageEditorDlg.SegmentsSpin.Value) then LineInfo.PolyInx:=I;
	end;
end;





procedure InsertPoint(LineInfo: TLineInfo);
var RP: TRealPolygon;
var Inx: integer;
begin
Inx:=LineInfo.PntInx1+1;
RP:=PolygonArray[LineInfo.PolyInx];
RP.Insert(Vector3Dto2DZ(LineInfo.Intercept,True),Inx);
end;


{-------------------- Routines to handle moving polygons ----------------------}


var PolyIndex: integer = -1;
var StartPos: T2DVector;



procedure PolygonToSplinePolygon(RP: TRealPolygon; var Poly2D: T2DVectorArray);
var I: integer;
begin
SetLength(Poly2D,0);
for I:=0 to RP.Count-1 do
	begin
	GetSplineLine(I,RP,ImageEditorDlg.SegmentsSpin.Value,False,Poly2D);
	end;
end;



function PolygonHitTest(Pos: T2DVector): integer;
var I: integer;
var A,Best: double;
var Poly: T2DVectorArray;
begin
Result:=-1;
Best:=1e99;
Pos:=ScalarDivide2D(Pos,PolygonScale);
for I:=0 to PolygonArray.Count-1 do
	begin
	PolygonToSplinePolygon(PolygonArray[I],Poly);
	if PointInPolygon(Poly,Pos) then
		begin
		A:=Abs(PolygonArea(Poly));
		if A<Best then
			begin
			Result:=I;
			Best:=A;
			end;
		end;
	end;
end;




procedure MovePolygon(var Poly: TRealPolygon; Delta: T2DVector);
var I: integer;
begin
for I:=0 to Poly.Count-1 do
Poly[I]:=VectorAdd2D(Poly[I],Delta);
end;



function FindPolygonCenter(Poly: TRealPolygon): T2DVector;
var I: integer;
begin
Result:=MakeVector2D(0,0);
for I:=0 to Poly.Count-1 do
 Result:=VectorAdd2D(Poly[I],Result);
Result:=ScalarDivide2D(Result,Poly.Count);
end;



procedure MoveSelectedPolygons(Delta: T2DVector);
var I: integer;
var SelectPoly,RP: TRealPolygon;
var C: T2DVector;
var SelectArea: double;
begin
SelectPoly:=PolygonArray[PolyIndex];
SelectArea:=Abs(PolygonArea(SelectPoly));
for I:=0 to PolygonArray.Count-1 do
 if I<>PolyIndex then
	begin
	RP:=PolygonArray[I];
	C:=FindPolygonCenter(RP);
	if PointInPolygon(SelectPoly,Vector2DToPoint(C)) and
	   (Abs(PolygonArea(RP))<SelectArea) then
		begin
		MovePolygon(RP,Delta);
		end;
	end;
MovePolygon(SelectPoly,Delta);
end;


{------------------- Event Handlers -------------------------------------------}




procedure HandlePolygonDoubleClick(Scale: double);
var P: TPoint;
var Pos: T2DVector;
var LineInfo: TLineInfo;
const Limit = 2;
var L: double;
begin
GetCursorPos(P);
P:=ImageEditorDlg.Image.ScreenToClient(P);
P.X:=round(P.X/Scale); P.Y:=round(P.Y/Scale);

Pos:=PointToVector2D(P);
L:=Limit/PolygonScale;
LineHitTest(Pos,LineInfo);
if (GetHandleType=htPoint) and (LineInfo.Distance<=L) then
	begin
	AddUndoItem(PolygonArray);
	InsertPoint(LineInfo);
	UpdatePolygons(Scale);
	end
else if PolygonHitTest(Pos)>-1 then
	begin
	if ImageEditorDlg.HandleRadio.ItemIndex<ImageEditorDlg.HandleRadio.Items.Count-2 then ImageEditorDlg.HandleRadio.ItemIndex:=ImageEditorDlg.HandleRadio.ItemIndex+1
        else ImageEditorDlg.HandleRadio.ItemIndex:=0;
	end;
end;



procedure PolygonMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Scale: double);
var HandleType: THandleType;
var Pos: T2DVector;
var T: integer;
var Grid: integer;
begin
HandleType:=GetHandleType;
Pos:=MakeVector2D(X,Y);
if ssLeft in Shift then
	begin
	if ImageEditorDlg.ModuloSnapChk.Checked then
		begin
		Grid:=trunc(ImageEditorDlg.ModuloSpin.Value * PolygonScale);
		X:=(X div Grid) * Grid;
		Y:=(Y div Grid) * Grid;
		end;
	StartPos:=MakeVector2D(X/PolygonScale,Y/PolygonScale);
	PolyIndex:=-1;
	case HandleType of
	 htPoint: HitPoint:=PointHitTest(Pos);
	 htSpline: HitPoint:=ControlPointHitTest(Pos);
	 htNone: HitPoint.Valid:=False;
	 end;
	if HitPoint.Valid then
		begin
		AddUndoItem(PolygonArray);
		if SelectPoint.PolyInx<>HitPoint.PolyInx then
			begin
			end;
		SelectPoint:=HitPoint;
		end;
	if not HitPoint.Valid then
		begin
		PolyIndex:=PolygonHitTest(Pos);
		if (PolyIndex>=0) and (not SelectPoint.Valid or (SelectPoint.PolyInx<>PolyIndex)) then
			begin
			AddUndoItem(PolygonArray);
			if SelectPoint.PolyInx<>PolyIndex then
				begin
				end;
			SelectPoint.Valid:=True;
			SelectPoint.PolyInx:=PolyIndex;
			SelectPoint.PointInx:=0;
			SelectPoint.HitType:=htPolygon;
			end;
		end;
	UpdatePolygons(Scale);
	end;
end;




procedure PolygonMouseMove(Shift: TShiftState; X, Y: Integer; Scale: double);
var NewPos,Delta,Pos: T2DVector;
var RP: TRealPolygon;
var V: T2DVector;
var Grid: integer;
begin
if ImageEditorDlg.ModuloSnapChk.Checked then
	begin
	Grid:=trunc(ImageEditorDlg.ModuloSpin.Value * PolygonScale);
	X:=(X div Grid) * Grid;
	Y:=(Y div Grid) * Grid;
	end;
Pos:=ScalarDivide2D(MakeVector2D(X,Y),PolygonScale);
ImageEditorDlg.MouseDis.Caption:=FloatToStrF(Pos.X,ffFixed,18,2)+' '+FloatToStrF(Pos.Y,ffFixed,18,2);
if not (ssLeft in Shift) then exit;
NewPos:=Pos;
Delta:=VectorSubtract2D(NewPos,StartPos);
StartPos:=NewPos;
if HitPoint.Valid then
	begin
	RP:=PolygonArray[HitPoint.PolyInx];
	V:=RP[HitPoint.PointInx];
	case HitPoint.HitType of
	 htHandle: RP[HitPoint.PointInx]:=Pos;
	 htControl1: MoveControlPoints(RP.ControlPoints1,RP.ControlPoints2,HitPoint.PointInx,Pos,V,Shift);
	 htControl2: MoveControlPoints(RP.ControlPoints2,RP.ControlPoints1,HitPoint.PointInx,Pos,V,Shift);
	 end;
	UpdatePolygons(Scale);
	end
else if PolyIndex>-1 then
	begin
	MoveSelectedPolygons(Delta);
	UpdatePolygons(Scale);
	end;
end;




procedure HandlePolygonMouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
HitPoint.Valid:=False;
PolyIndex:=-1;
end;


{-----------------------------------------------------------------------------}


procedure InsertNewPoint(X, Y: Integer; Scale: double);
var LastPos,Delta,Pos: T2DVector;
var Inx,Cnt: integer;
begin
Pos:=ScalarDivide2D(MakeVector2D(X,Y),PolygonScale);
if SelectPoint.Valid then
	begin
	LastPos:=ScalarDivide2D(PolygonArray[SelectPoint.PolyInx][SelectPoint.PointInx],PolygonScale);
	if VectorABS2D(VectorSubtract2D(Pos,LastPos))<5 then exit;
	end;
if PolygonArray.Count<1 then
	begin
	SelectPoint.PolyInx:=PolygonArray.Add;
	AddUndoItem(PolygonArray);
	end;
Cnt:=PolygonArray[SelectPoint.PolyInx].Count;
if Cnt<1 then
	begin
	SelectPoint.PointInx:=PolygonArray[SelectPoint.PolyInx].Add(Pos);
	AddUndoItem(PolygonArray);
	end
else
	begin
	Inx:=SelectPoint.PointInx+1;
	if Inx>=Cnt then Inx:=Cnt-1;
	PolygonArray[SelectPoint.PolyInx].Insert(Pos,Inx);
	SelectPoint.PointInx:=Inx;
	AddUndoItem(PolygonArray);
	end;
UpdatePolygons(Scale);
end;



procedure PolyDrawMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; Scale: double);
begin
InsertNewPoint(X, Y,Scale);
end;




procedure PolyDrawMouseMove(Shift: TShiftState; X, Y: Integer; Scale: double);
begin
InsertNewPoint(X, Y,Scale);
end;

{ TPolyClipper }



procedure TPolyClipper.SetClipper(Canvas: TCanvas);
begin
if not Enabled then exit;
Region:=GetSplinePolyPolyRegion(PolygonArray,10);
OldRegion := SelectObject(Canvas.Handle, Region);
end;


procedure TPolyClipper.ClearClipper(Canvas: TCanvas);
begin
if not Enabled then exit;
SelectObject(Canvas.Handle, OldRegion);
DeleteObject(Region)
end;



constructor TPolyClipper.Create;
begin
BMP:=TBitmap.Create;
Enabled:=False;
SplinePoints:=10;
end;


destructor TPolyClipper.Destroy;
begin
BMP.Free;
inherited;
end;


function TPolyClipper.InitializeBMPClip(TargetBMP: TBitmap): TBitmap;
begin
if Enabled then
	begin
	CopyBitmap(BMP,TargetBMP);
	Result:=BMP;
	end
else Result:=TargetBMP;
end;


procedure TPolyClipper.DoBMPClip(TargetBMP: TBitmap);
begin
if Enabled then
	begin
	PolyClipper.SetClipper(TargetBMP.Canvas);
	CopyBitmap(TargetBMP,BMP);
	PolyClipper.ClearClipper(TargetBMP.Canvas);
	end;
end;





{-----------------------------------------------------------------------------}





initialization
PolyClipper:=TPolyClipper.Create;
PolyClipper.Enabled:=True;
finalization
PolyClipper.Free;
end.
