unit IEClone;

interface

uses Windows,Graphics,Types,Classes,Math,DSupport,IntVector,BitmapLib;

type TCloneMode = (cmNormal,cmHorizontal,cmVertical,cmDiagonal);

var CloneBase: TPoint;
var CloneSource: TPoint;
var CloneStart: TPoint;
var CloneDest: TPoint;


procedure CloneMouseMove(X,Y,XM,YM: integer; Mode: TCloneMode; Scale: double; Shift: TShiftState);
procedure CloneMouseDown(X,Y: integer; Mode: TCloneMode; Shift: TShiftState);

implementation

uses ImageEditor, IEPenDrawing, IEClipper, IEPolygonLib;


{------------------------------------------------------------------------------}



procedure ShowClonePos;
begin
with ImageEditorDlg do
 begin
 CloneBaseDis.Text:=PointToString(CloneBase);
 CloneSourceDis.Text:=PointToString(CloneSource);
 CloneStartDis.Text:=PointToString(CloneStart);
 CloneDestDis.Text:=PointToString(CloneDest);
 end;
end;



procedure GetModeSign(Mode: TCloneMode; var XSign,YSign: double);
begin
case Mode of
 cmNormal:
	begin
	XSign:=1;
	YSign:=1;
	end;
 cmHorizontal:
	begin
	XSign:=-1;
	YSign:=1;
	end;
 cmVertical:
	begin
	XSign:=1;
	YSign:=-1;
	end;
 cmDiagonal:
	begin
	XSign:=-1;
	YSign:=-1;
	end;
 end;
end;




procedure GetCloneSource(Canvas: TCanvas; Limit: TPoint; var BM: TBitmap; Mode: TCloneMode; Source: TPoint; Size: integer);
var X,Y,SX,SY: integer;
var XSign,YSign: double;
begin
BM.Width:=Size;
BM.Height:=Size;
GetModeSign(Mode,XSign,YSign);
for Y:=0 to Size-1 do
	begin
	SY:=Round(Source.Y+Y * YSIgn);
	if not InRange(SY,0,Limit.Y-1) then continue;
	for X:=0 to Size-1 do
		begin
		SX:=Round(Source.X+X * XSign);
		if not InRange(SX,0,Limit.X-1) then continue;
		BM.Canvas.Pixels[X,Y]:=Canvas.Pixels[SX,SY];
		end;
	end;
end;


procedure DrawClone(Canvas: TCanvas; BM: TBitmap; Dest: TPoint);
begin
ImageEditorDlg.DottedBox2.HideRectangle;
Canvas.Draw(Dest.X,Dest.Y,BM);
{Remember repaint must be on underlying scroll box}
ImageEditorDlg.ScrollBox.Repaint;
ImageEditorDlg.DottedBox2.ShowRectangle;
end;


procedure ClonePoint(BM: TBitmap; Mode: TCloneMode; Source,Dest: TPoint; Size: integer);
var Tmp: TBitmap;
begin
Tmp:=TBitmap.Create;
try
GetCloneSource(BM.Canvas, Point(BM.Width,BM.Height), Tmp, Mode, Source, Size);

PolyClipper.SetClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
DrawClone(BM.Canvas,Tmp,Dest);
PolyClipper.ClearClipper(ImageEditorDlg.Image.Picture.Bitmap.Canvas);
finally Tmp.Free; end;
end;







var NewSource: boolean;



procedure CloneMouseDown(X,Y: integer; Mode: TCloneMode; Shift: TShiftState);
var C: TColor;
begin
if not (ssLeft in Shift) then exit;
if ssCtrl in Shift then
	begin
	CloneBase:=Point(X,Y);
	CloneSource:=CloneBase;
	NewSource:=True;
	end

else
	begin
	ImageEditorDlg.ImageUndoer.StoreUndo;
	if not ImageEditorDlg.AlignedChk.Checked or NewSource then CloneStart:=Point(X,Y);
	CloneDest:=CloneStart;
	ClonePoint(ImageEditorDlg.Image.Picture.Bitmap,Mode,CloneSource,CloneStart,ImageEditorDlg.PenSizeSpin.Value);
	NewSource:=False;
	end;
ShowClonePos;
end;



procedure CloneMouseMove(X,Y,XM,YM: integer; Mode: TCloneMode;  Scale: double; Shift: TShiftState);
var C: TColor;
var Delta: TPoint;
var SBox: TPoint;
var Size: integer;
var XSign,YSign: double;
begin
GetModeSign(Mode,XSign,YSign);
Size:=ImageEditorDlg.PenSizeSpin.Value;
CloneDest:=Point(XM,YM);

if (ssLeft in Shift) then
	begin
	if ssCtrl in Shift then
		begin
		CloneBase:=CloneDest;
		CloneSource:=CloneDest;
		end
	else
		begin
		Delta:=IntVectorSubtract(CloneDest,CloneStart);
		Delta.X:=Round(Delta.X * XSign);
		Delta.Y:=Round(Delta.Y * YSign);
		CloneSource:=IntVectorAdd(CloneBase,Delta);
		ClonePoint(ImageEditorDlg.Image.Picture.Bitmap,Mode,CloneSource,CloneDest,Size);
		end;
	end
else CloneSource:=CloneBase;
DrawPenBox2(ImageEditorDlg.Image,Trunc(CloneSource.X),Trunc(CloneSource.Y),Size, 1,False);
ShowClonePos;
end;



end.
