unit ImageLib;

interface

uses Windows, Forms, Classes,SysUtils,ExtCtrls,Graphics,DSupport,TifRead,TifWrite,
     BitmapProg,BitmapLib, TifGlobal,TGAImage,PCXLib,LBMLib,
     {$IF CompilerVersion < 17}
     GIFImage,
     pngimage;
     {$ELSE}
     Vcl.Imaging.GIFImg,
     vcl.imaging.pngimage;
     {$IFEND}

const GraphicsFilter='All (tga,jpg,jpeg,gif,tif,tiff,png,bmp,ico,emf,wmf,pcx,lbm)|*.jpg;*.jpeg;*.gif;*.tif;*.tiff;*.png;*.bmp;*.ico;*.emf;*.wmf;*.pcx;*.lbm|'+
'Bitmaps (*.bmp)|*.bmp|'+
'JPEG Image File (*.jpg;*.jpeg)|*.jpg;*.jpeg|'+
'GIF Files|*.gif|'+
'TIFF File|*.tif;*.tiff|'+
'PNG Files|*.png|'+
'TGA Image File (*.tga)|*.tga|'+
'PCX Image File (*.pcx)|*.pcx|'+
'LBM Image File (*.lbm)|*.lbm|'+
'Icons (*.ico)|*.ico|'+
'Enhanced Metafiles (*.emf)|*.emf|'+
'Metafiles (*.wmf)|*.wmf';


function LoadBitmapByExtension(BM: TBitmap; FName: string; Progress: boolean): string;
function SaveBitmapByExtension(const BM: TBitmap; FName: string; Progress,Compress,GeoTiff: boolean): string;



var JPEGQuality: integer=100;

implementation

uses JPEG;




var XRes,YRes: integer;



{---------------- Routines to load and save JPEG files -----------------------}

procedure LoadJPEGBitmap(BM: TBitmap; FName: string);
var JImage: TJPEGImage;
var Units: TJPEGResUnits;
var XRes,YRes: integer;
begin
JImage:=TJPEGImage.Create;
try
JImage.LoadFromFile(FName);
BM.FreeImage;
BM.ReleaseHandle;
BM.Width:=JIMage.Width;
BM.Height:=JImage.Height;
BM.PixelFormat:=pf24Bit;
BM.Canvas.Draw(0,0,JImage);
except JImage.Free; end;
{Get JPEG resolution information}
GetJPEGResolution(FName,Units,XRes,YRes);
case Units of
 juNoUnits: begin XRes:=trunc(72 / Inches2Meters); YRes:=trunc(72 / Inches2Meters); end;
 juDotsPerInch: begin XRes:=trunc(XRes / Inches2Meters); YRes:=trunc(YRes / Inches2Meters); end;
 juDotsPerCentimeter: begin XRes:=XRes * 100; YRes:=YRes * 100; end;
 end;
{Insert it in the bitmap}
SetBMPPPM(BM,XRes,YRes);
end;


procedure SaveJPEGBitmap(BM: TBitmap; FName: string);
var JImage: TJPEGImage;
var FS: TFileStream;
var PPMX,PPMY: integer;
var DPIX,DPIY: integer;
begin
JImage:=TJPEGImage.Create;
try
JImage.Assign(BM);
JImage.CompressionQuality:=JPEGQuality;
try
FS:=TFileStream.Create(FName,fmCreate);
JImage.SaveToStream(FS);
finally FS.Free; end;
finally JImage.Free; end;
{Get BMP resoution}
GetBmpPPM(BM, PPMX,PPMY);
DPIX:=trunc(PPMX * Meters2Inches);
DPIY:=trunc(PPMY * Meters2Inches);

SetJPEGResolution(FName, DPIX, DPIY, juDotsPerInch);
end;


{------------------ Routines to Handle direct loading ------------------------}

procedure LoadTIFProgress(var BM: TBitmap; FName: string; Progress: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnTIFProgress:=HandleProgress;
	end;
try
ReadTIFFile(BM,FName);
finally
 if Progress then HideProgress;
 OnTIFProgress:=nil;
 end;
end;




procedure LoadJPEGProgress(var BM: TBitmap; FName: string; Progress: boolean);
var JImage: TJPEGImage;
begin
JImage:=TJPEGImage.Create;
try
if Progress then
	begin
	DisplayProgress;
	JImage.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
JImage.LoadFromFile(FName);
BM.Width:=JIMage.Width;
BM.Height:=JImage.Height;
BM.PixelFormat:=pf24Bit;
BM.Canvas.Draw(0,0,JImage);

finally
 if Progress then HideProgress;
 JImage.Free;
 end;
end;




procedure LoadPNGProgress(var BM: TBitmap; FName: string; Progress: boolean);
var PNG: TPNGObject;
begin
PNG:=TPNGObject.Create;
try
if Progress then
	begin
	DisplayProgress;
	PNG.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
PNG.LoadFromFile(FName);
BM.Assign(PNG);
finally
 if Progress then HideProgress;
 PNG.Free;
 end;
end;




procedure LoadGIFProgress(var BM: TBitmap; FName: string; Progress: boolean);
var GIF: TGIFImage;
begin
GIF:=TGIFImage.Create;
try
if Progress then
	begin
	DisplayProgress;
	GIF.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
GIF.LoadFromFile(FName);
BM.Assign(GIF);
SetBMPPPM(BM,Round(72 * Meters2Inches), Round(72 * Meters2Inches));
finally
 if Progress then HideProgress;
 GIF.Free;
 end;
end;




procedure LoadTGAProgress(var BM: TBitmap; FName: string; Progress: boolean);
var TGA: TTGAImage;
begin
TGA:=TTGAImage.Create;
try
if Progress then
	begin
	DisplayProgress;
	TGA.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
TGA.LoadFromFile(FName);
BM.Assign(TGA);
finally
 if Progress then HideProgress;
 TGA.Free;
 end;
end;



procedure LoadPCXProgress(var BM: TBitmap; FName: string; Progress: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnPCXProgress:=HandleProgress;
	end;
try
ReadPCXFile(BM,FName);
finally
 if Progress then HideProgress;
 OnPCXProgress:=nil;
 end;
end;




procedure LoadLBMProgress(var BM: TBitmap; FName: string; Progress: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnLBMProgress:=HandleProgress;
	end;
try
ReadLBMFile(BM,FName);
finally
 if Progress then HideProgress;
 OnLBMProgress:=nil;
 end;
end;





{-------------------------------------------------------------------------------}


function LoadBitmapByExtension(BM: TBitmap; FName: string; Progress: boolean): string;
{Load bitmap according to extension. Extension returned as lowercase string.}
begin
Result:=ExtractFileExt(FName);
Result:=LowerCase(Result);
if (Result='.jpg') or (Result='.jpeg') then LoadJPEGProgress(BM,FName,Progress)
else if (Result='.tif') or (Result='.tiff') then LoadTIFProgress(BM,FName,Progress)
else if Result='.gif' then LoadGIFProgress(BM,FName,Progress)
else if Result='.png' then LoadPNGProgress(BM,FName,Progress)
else if Result='.tga' then LoadTGAProgress(BM,FName,Progress)
else if Result='.pcx' then LoadPCXProgress(BM,FName,Progress)
else if Result='.lbm' then LoadLBMProgress(BM,FName,Progress)
else BM.LoadFromFile(FName);
end;




{------------------ Routines to Handle direct Saving ------------------------}



procedure SaveTIFProgress(const BM: TBitmap; FName: string; Progress,Compress,GeoTiff: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnTIFProgress:=HandleProgress;
	end;
try
SaveTIFFToFile(FName,BM,Compress,GeoTiff);
finally
 if Progress then HideProgress;
 OnTIFProgress:=nil;
 end;
end;




procedure SaveJPEGProgress(const BM: TBitmap; FName: string; Progress: boolean);
var JImage: TJPEGImage;
var PPMX,PPMY: integer;
var DPIX,DPIY: integer;
begin
JImage:=TJPEGImage.Create;
try
if Progress then
	begin
	DisplayProgress;
	JImage.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
JImage.Assign(BM);
JImage.SaveToFile(FName);
finally
 if Progress then HideProgress;
 JImage.Free;
 end;
GetBmpPPM(BM, PPMX,PPMY);
DPIX:=round(PPMX / Meters2Inches);
DPIY:=round(PPMY / Meters2Inches);
SetJPEGResolution(FName, DPIX, DPIY, juDotsPerInch);
end;



procedure SavePNGProgress(const BM: TBitmap; FName: string; Progress: boolean);
var PNG: TPNGObject;
var XRes,YRes: integer;
begin
PNG:=TPNGObject.Create;
try
if Progress then
	begin
	DisplayProgress;
	PNG.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
PNG.Assign(BM);

GetBmpPPM(BM,XRes,YRes);
PNG.PixelInformation.PPUnitX:=XRes;
PNG.PixelInformation.PPUnitY:=YRes;
PNG.PixelInformation.UnitType:=utMeter;

PNG.SaveToFile(FName);
finally
 if Progress then HideProgress;
 PNG.Free;
 end;
end;




procedure SaveGIFProgress(const BM: TBitmap; FName: string; Progress: boolean);
var GIF: TGIFImage;
begin
GIF:=TGIFImage.Create;
try
if Progress then
	begin
	DisplayProgress;
	GIF.OnProgress:=BitmapProgress.HandleObjectProgress;
	end;
GIF.Assign(BM);
GIF.SaveToFile(FName);
finally
 if Progress then HideProgress;
 GIF.Free;
 end;
end;



procedure SavePCXProgress(const BM: TBitmap; FName: string; Progress: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnPCXProgress:=HandleProgress;
	end;
try
SavePCXToFile(FName,BM);
finally
 if Progress then HideProgress;
 OnPCXProgress:=nil;
 end;
end;




procedure SaveLBMProgress(const BM: TBitmap; FName: string; Progress,Compress: boolean);
begin
if Progress then
	begin
	DisplayProgress;
	OnLBMProgress:=HandleProgress;
	end;
try
SaveLBMToFile(FName,BM,Compress);
finally
 if Progress then HideProgress;
 OnLBMProgress:=nil;
 end;
end;


function SaveBitmapByExtension(const BM: TBitmap; FName: string; Progress,Compress,GeoTiff: boolean): string;
{Save bitmap according to extension. Extension returned as lowercase string.}
var BMP: TBitmap;
begin
BMP:=TBitmap.Create;
try
Result:=ExtractFileExt(FName);
Result:=LowerCase(Result);
{Use separate bitmap or else the overridden Save to file may revert
to the original class - for example you may try to save as BMP and TIF}
GetBmpPPM(BM,XRes,YRes);

BMP.Assign(BM);
GetBmpPPM(BMP,XRes,YRes);
if (Result='.jpg') or (Result='.jpeg') then
	begin
	{Most other formats cannot handle 32 bit images}
	BMP.PixelFormat:=pf24bit;
	CopyBMPPPM(BM,BMP);
	SaveJPEGProgress(BMP,FName,Progress)
	end
else if (Result='.tif') or (Result='.tiff') then
	begin
	{Most other formats cannot handle 32 bit images}
	BMP.PixelFormat:=pf24bit;
	CopyBMPPPM(BM,BMP);
	SaveTIFProgress(BMP,FName,Progress,Compress,GeoTiff)
	end
else if Result='.gif' then
	begin
	{Gif only supports an 8-bit Paletter and The gif object doesn't}
	{necessarily do a good job reducing the color palette to 8-bits}
	{so well force Delphi/Windows to do it.}
	BMP.PixelFormat:=pf8bit;
	CopyBMPPPM(BM,BMP);
	SaveGIFProgress(BMP,FName,Progress)
	end
else if Result='.png' then
	begin
	{Most other formats cannot handle 32 bit images}
	BMP.PixelFormat:=pf24bit;
	CopyBMPPPM(BM,BMP);
	SavePNGProgress(BMP,FName,Progress)
	end
else if Result='.pcx' then SavePCXProgress(BMP,FName,Progress)
else if Result='.lbm' then SaveLBMProgress(BMP,FName,Progress,Compress)
else BMP.SaveToFile(FName);
finally BMP.Free; end;
end;



{=============== Routines to Register Fileformat ==============================}

{Note: this routines return bitmaps so that the image can be manipulated afterwards}
{Otherwise, if the routines return a PNG or GIF, accessing the bitmap will blank it}

{------------- Objects used to register PNG formats-----------------------}

type TPNGBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TPNGBitmap.LoadFromFile(const Filename: string);
var PNG: TPNGObject;
begin
PNG:=TPNGObject.Create;
try
PNG.LoadFromFile(Filename);
Self.Assign(PNG);
SetBMPPPM(Self, PNG.PixelInformation.PPUnitX,PNG.PixelInformation.PPUnitY);
finally PNG.Free; end;
end;


procedure TPNGBitmap.SaveToFile(const Filename: string);
var PNG: TPNGObject;
begin
PNG:=TPNGObject.Create;
try
PNG.Assign(Self);
PNG.SaveToFile(Filename);
finally PNG.Free; end;
end;


{------------- Objects used to register GIF formats-----------------------}

type TGIFBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TGIFBitmap.LoadFromFile(const Filename: string);
var GIF: TGIFImage;
begin
GIF:=TGIFImage.Create;
try
GIF.LoadFromFile(Filename);
Self.Assign(GIF);
SetBMPPPM(Self,Round(72 * Meters2Inches), Round(72 * Meters2Inches));
finally GIF.Free; end;
end;


procedure TGIFBitmap.SaveToFile(const Filename: string);
var GIF: TGIFImage;
begin
GIF:=TGIFImage.Create;
try
GIF.Assign(Self);
GIF.SaveToFile(Filename);
finally GIF.Free; end;
end;


{------------- Objects used to register TGA formats-----------------------}

type TTGABitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TTGABitmap.LoadFromFile(const Filename: string);
var TGA: TTGAImage;
begin
TGA:=TTGAImage.Create;
try
TGA.LoadFromFile(Filename);
Self.Assign(TGA);
SetBMPPPM(Self,Round(72 * Meters2Inches), Round(72 * Meters2Inches));
finally TGA.Free; end;
end;


procedure TTGABitmap.SaveToFile(const Filename: string);
var TGA: TTGAImage;
begin
TGA:=TTGAImage.Create;
try
TGA.Assign(Self);
TGA.SaveToFile(Filename);
finally TGA.Free; end;
end;

{------------- Objects used to register JPEG formats-----------------------}



type TJPGBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TJPGBitmap.LoadFromFile(const Filename: string);
begin
LoadJPEGBitmap(Self,FileName);
end;


procedure TJPGBitmap.SaveToFile(const Filename: string);
begin
SaveJPEGBitmap(Self,FileName);
end;

{------------- Objects used to register TIFF formats-----------------------}


type TTIFFBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TTIFFBitmap.LoadFromFile(const Filename: string);
begin
ReadTIFFile(Self,FileName);
end;


procedure TTIFFBitmap.SaveToFile(const Filename: string);
begin
SaveTIFFToFile(FileName,Self,False,True);
end;


{------------- Objects used to register PCX formats-----------------------}


type TPCXBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TPCXBitmap.LoadFromFile(const Filename: string);
begin
ReadPCXFile(Self,FileName);
SetBMPPPM(Self,Round(72 * Meters2Inches), Round(72 * Meters2Inches));
end;


procedure TPCXBitmap.SaveToFile(const Filename: string);
begin
SavePCXToFile(FileName,Self);
end;



{------------- Objects used to register LBM formats-----------------------}


type TLBMBitmap = class(TBitmap)
  public
    procedure LoadFromFile(const Filename: string); override;
    procedure SaveToFile(const Filename: string); override;
  end;


procedure TLBMBitmap.LoadFromFile(const Filename: string);
begin
ReadLBMFile(Self,FileName);
SetBMPPPM(Self,Round(72 * Meters2Inches), Round(72 * Meters2Inches));
end;


procedure TLBMBitmap.SaveToFile(const Filename: string);
begin
SaveLBMToFile(FileName,Self,True);
end;

{---------------------------------------------------------------------------}



initialization
{Unregister these versions so we can register our own}
{$IF CompilerVersion < 17}
TPicture.UnregisterGraphicClass(TJPEGImage);
TPicture.UnregisterGraphicClass(TGIFImage);
TPicture.UnregisterGraphicClass(TPNGObject);
{$IFEND}

TPicture.RegisterFileFormat('LBM','LBM Files', TLBMBitmap);
TPicture.RegisterFileFormat('PCX','PCX Files', TPCXBitmap);

TPicture.RegisterFileFormat('TGA','TGA Files', TTGABitmap);
TPicture.RegisterFileFormat('TIF','TIFF-Format', TTIFFBitmap);
TPicture.RegisterFileFormat('TIFF','TIFF-Format', TTIFFBitmap);

{$IF CompilerVersion < 17}
TPicture.RegisterFileFormat('JPG','JPEG Files', TJPGBitmap);
TPicture.RegisterFileFormat('JPEG','JPEG Files', TJPGBitmap);
TPicture.RegisterFileFormat('PNG','PNG Files', TPNGBitmap);
TPicture.RegisterFileFormat('GIF','GIF Files', TGIFBitmap);
{$IFEND}

finalization
TPicture.UnRegisterGraphicClass(TTIFFBitmap);

TPicture.UnregisterGraphicClass(TTGABitmap);
TPicture.UnregisterGraphicClass(TPCXBitmap);
TPicture.UnregisterGraphicClass(TLBMBitmap);

{$IF CompilerVersion < 17}
TPicture.UnregisterGraphicClass(TJPGBitmap);
TPicture.UnregisterGraphicClass(TPNGBitmap);
TPicture.UnregisterGraphicClass(TGIFBitmap);
{$IFEND}

end.
