unit FFContextDisplay;

interface



uses SysUtils,ComCtrls,libavutil,libavformat,libavcodec,DSupport;


function ShowNode(Tree: TTreeView; Parent: TTreeNode; S: string; ImageIndex: integer): TTreeNode;

procedure DisplayContext(Tree: TTreeView; Context: TAVFormatContext);


implementation


{------------ Tree handling routines -------------------------------------------}

procedure SetImageIndexes(Node: TTreeNode; Inx: integer);
begin
Node.ImageIndex:=Inx;
Node.StateIndex:=Inx;
end;


function ShowNode(Tree: TTreeView; Parent: TTreeNode; S: string; ImageIndex: integer): TTreeNode;
begin
Result:=Tree.Items.AddChild(Parent,S);
SetImageIndexes(Result,ImageIndex);
end;



function ShowNodeSubNode(Tree: TTreeView; Parent: TTreeNode; NodeStr,SubStr: string; Inx1,Inx2: integer): TTreeNode;
var SubNode: TTreeNode;
begin
SubNode:=Tree.Items.AddChild(Parent,NodeStr);
Result:=Tree.Items.AddChild(SubNode,SubStr);
SetImageIndexes(SubNode,Inx1);
SetImageIndexes(SubNode,Inx2);
end;

{-------------- Data Type display Routines -------------------------------------}


procedure ShowInt64Array(Tree: TTreeView; Node: TTreeNode; Title: string; Size,Limit: integer; Ints: array of int64);
var I: integer;
var SubNode: TTreeNode;
var S: string;
const LineLen = 16;
begin
Tree.Items.BeginUpdate;
try
SubNode:=ShowNode(Tree,Node,Title,2);
S:='00000000: ';
for I:=0 to Size-1 do
	begin
	if ((I<>0) and ((I mod LineLen)=0)) or
	   (I>=Limit) then
		begin
		S:=S+CRLF_Char;
		ShowNode(Tree, SubNode, S,2);
		if I>=Limit then break;
		S:=IntToHex(I,8)+': ';
		end;
	S:=S+IntToHex(Ints[I],8)+', ';
	end;
finally Tree.Items.EndUpdate; end;
end;


procedure ShowIntegerArray(Tree: TTreeView; Node: TTreeNode; Title: string; Size,Limit: integer; Ints: array of integer);
var I: integer;
var SubNode: TTreeNode;
var S: string;
const LineLen = 16;
begin
Tree.Items.BeginUpdate;
try
SubNode:=ShowNode(Tree,Node,Title,2);
S:='00000000: ';
for I:=0 to Size-1 do
	begin
	if ((I<>0) and ((I mod LineLen)=0)) or
	   (I>=Limit) then
		begin
		S:=S+CRLF_Char;
		ShowNode(Tree, SubNode, S,2);
		if I>=Limit then break;
		S:=IntToHex(I,8)+': ';
		end;
	S:=S+IntToHex(Ints[I],8)+', ';
	end;
finally Tree.Items.EndUpdate; end;
end;


procedure ShowArrayOfPByte(Tree: TTreeView; Node: TTreeNode; Title: string; Size,Limit: int64; Ints: array of PByte);
var I: integer;
var SubNode: TTreeNode;
var S: string;
const LineLen = 16;
begin
Tree.Items.BeginUpdate;
try
SubNode:=ShowNode(Tree,Node,Title,2);
S:='00000000: ';
for I:=0 to Size-1 do
	begin
	if ((I<>0) and ((I mod LineLen)=0)) or
	   (I>=Limit) then
		begin
		S:=S+CRLF_Char;
		ShowNode(Tree, SubNode, S,2);
		if I>=Limit then break;
		S:=IntToHex(I,8)+': ';
		end;
	S:=S+IntToHex(Ints[I]^,8)+', ';
	end;
finally Tree.Items.EndUpdate; end;
end;



procedure ShowPByteArray(Tree: TTreeView; Node: TTreeNode; Title: string; Size,Limit: int64; Buffer: PByte);
var I: integer;
var SubNode: TTreeNode;
var S: string;
const LineLen = 16;
begin
Tree.Items.BeginUpdate;
try
SubNode:=ShowNode(Tree,Node,Title,2);
S:='00000000: ';
if (Buffer=nil) or (Cardinal(Buffer)=$FEEEFEEE) then exit;
for I:=0 to Size-1 do
	begin
	if ((I<>0) and ((I mod LineLen)=0)) or
	   (I>=Limit) then
		begin
		S:=S+CRLF_Char;
		ShowNode(Tree, SubNode, S,2);
		if I>=Limit then break;
		S:=IntToHex(I,8)+': ';
		end;
	S:=S+IntToHex(PByteArray(Buffer)[I],2)+', ';
	end;
finally Tree.Items.EndUpdate; end;
end;


procedure ShowByteArray(Tree: TTreeView; Node: TTreeNode; Title: string; Size,Limit: int64; Ints: array of Byte);
var I: integer;
var SubNode: TTreeNode;
var S: string;
const LineLen = 16;
begin
Tree.Items.BeginUpdate;
try
SubNode:=ShowNode(Tree,Node,Title,2);
S:='00000000: ';
for I:=0 to Size-1 do
	begin
	if ((I<>0) and ((I mod LineLen)=0)) or
	   (I>=Limit) then
		begin
		S:=S+CRLF_Char;
		ShowNode(Tree, SubNode, S,2);
		if I>=Limit then break;
		S:=IntToHex(I,2)+': ';
		end;
	S:=S+IntToHex(Ints[I],2)+', ';
	end;
finally Tree.Items.EndUpdate; end;
end;


function IsInvalidPointer(P: pointer; var Report: string): boolean;
begin
Result:=True;
if P=nil then Report:='nil'
else if Cardinal(P)=$FEEEFEEE then Report:='Deallocated'
else Result:=False;
end;


function GetPIntegerStr(PI: PInteger): string;
begin
if IsInvalidPointer(PI,Result) then exit;
Result:=IntToStr(PI^);
end;



function GetPWordStr(PI: PWord): string;
begin
if IsInvalidPointer(PI,Result) then exit;
Result:=IntToStr(PI^);
end;

{===============================================================================}


function StrPtrToStr(SP: PAnsiChar): string;
begin
if IsInvalidPointer(SP,Result) then exit;
Result:=SP;
end;


function ProcAddressStr(P: pointer): string;
begin
Result:='Proc: $'+IntToHex(integer(P),8);
end;



function PointerStr(P: pointer): string;
begin
Result:=IntToHex(integer(P),8);
end;


function GetRationalStr(R: TAVRational): string;
begin
Result:='Num: '+IntToStr(R.num);
Result:=Result+' Den: '+IntToStr(R.den);
end;


{-------------Enumeration Decoders ---------------------------------------------}


function GetCodecIDStr(ID: TAVCodecID): string;
begin
case ID of
 AV_CODEC_ID_NONE: Result:='NONE';
 AV_CODEC_ID_MPEG1VIDEO: Result:='MPEG1VIDEO';
 AV_CODEC_ID_MPEG2VIDEO: Result:='MPEG2VIDEO';
{$ifdef FF_API_XVMC}
 AV_CODEC_ID_MPEG2VIDEO_XVMC: Result:='MPEG2VIDEO_XVMC';
{$endif}
 AV_CODEC_ID_H261: Result:='H261';
 AV_CODEC_ID_H263: Result:='H263';
 AV_CODEC_ID_RV10: Result:='RV10';
 AV_CODEC_ID_RV20: Result:='RV20';
 AV_CODEC_ID_MJPEG: Result:='MJPEG';
 AV_CODEC_ID_MJPEGB: Result:='MJPEGB';
 AV_CODEC_ID_LJPEG: Result:='LJPEG';
 AV_CODEC_ID_SP5X: Result:='SP5X';
 AV_CODEC_ID_JPEGLS: Result:='JPEGLS';
 AV_CODEC_ID_MPEG4: Result:='MPEG4';
 AV_CODEC_ID_RAWVIDEO: Result:='RAWVIDEO';
 AV_CODEC_ID_MSMPEG4V1: Result:='MSMPEG4V1';
 AV_CODEC_ID_MSMPEG4V2: Result:='MSMPEG4V2';
 AV_CODEC_ID_MSMPEG4V3: Result:='MSMPEG4V3';
 AV_CODEC_ID_WMV1: Result:='WMV1';
 AV_CODEC_ID_WMV2: Result:='WMV2';
 AV_CODEC_ID_H263P: Result:='H263P';
 AV_CODEC_ID_H263I: Result:='H263I';
 AV_CODEC_ID_FLV1: Result:='FLV1';
 AV_CODEC_ID_SVQ1: Result:='SVQ1';
 AV_CODEC_ID_SVQ3: Result:='SVQ3';
 AV_CODEC_ID_DVVIDEO: Result:='DVVIDEO';
 AV_CODEC_ID_HUFFYUV: Result:='HUFFYUV';
 AV_CODEC_ID_CYUV: Result:='CYUV';
 AV_CODEC_ID_H264: Result:='H264';
 AV_CODEC_ID_INDEO3: Result:='INDEO3';
 AV_CODEC_ID_VP3: Result:='VP3';
 AV_CODEC_ID_THEORA: Result:='THEORA';
 AV_CODEC_ID_ASV1: Result:='ASV1';
 AV_CODEC_ID_ASV2: Result:='ASV2';
 AV_CODEC_ID_FFV1: Result:='FFV1';
 AV_CODEC_ID_4XM: Result:='4XM';
 AV_CODEC_ID_VCR1: Result:='VCR1';
 AV_CODEC_ID_CLJR: Result:='CLJR';
 AV_CODEC_ID_MDEC: Result:='MDEC';
 AV_CODEC_ID_ROQ: Result:='ROQ';
 AV_CODEC_ID_INTERPLAY_VIDEO: Result:='INTERPLAY_VIDEO';
 AV_CODEC_ID_XAN_WC3: Result:='XAN_WC3';
 AV_CODEC_ID_XAN_WC4: Result:='XAN_WC4';
 AV_CODEC_ID_RPZA: Result:='RPZA';
 AV_CODEC_ID_CINEPAK: Result:='CINEPAK';
 AV_CODEC_ID_WS_VQA: Result:='WS_VQA';
 AV_CODEC_ID_MSRLE: Result:='MSRLE';
 AV_CODEC_ID_MSVIDEO1: Result:='MSVIDEO1';
 AV_CODEC_ID_IDCIN: Result:='IDCIN';
 AV_CODEC_ID_8BPS: Result:='8BPS';
 AV_CODEC_ID_SMC: Result:='SMC';
 AV_CODEC_ID_FLIC: Result:='FLIC';
 AV_CODEC_ID_TRUEMOTION1: Result:='TRUEMOTION1';
 AV_CODEC_ID_VMDVIDEO: Result:='VMDVIDEO';
 AV_CODEC_ID_MSZH: Result:='MSZH';
 AV_CODEC_ID_ZLIB: Result:='ZLIB';
 AV_CODEC_ID_QTRLE: Result:='QTRLE';
 AV_CODEC_ID_TSCC: Result:='TSCC';
 AV_CODEC_ID_ULTI: Result:='ULTI';
 AV_CODEC_ID_QDRAW: Result:='QDRAW';
 AV_CODEC_ID_VIXL: Result:='VIXL';
 AV_CODEC_ID_QPEG: Result:='QPEG';
 AV_CODEC_ID_PNG: Result:='PNG';
 AV_CODEC_ID_PPM: Result:='PPM';
 AV_CODEC_ID_PBM: Result:='PBM';
 AV_CODEC_ID_PGM: Result:='PGM';
 AV_CODEC_ID_PGMYUV: Result:='PGMYUV';
 AV_CODEC_ID_PAM: Result:='PAM';
 AV_CODEC_ID_FFVHUFF: Result:='FFVHUFF';
 AV_CODEC_ID_RV30: Result:='RV30';
 AV_CODEC_ID_RV40: Result:='RV40';
 AV_CODEC_ID_VC1: Result:='VC1';
 AV_CODEC_ID_WMV3: Result:='WMV3';
 AV_CODEC_ID_LOCO: Result:='LOCO';
 AV_CODEC_ID_WNV1: Result:='WNV1';
 AV_CODEC_ID_AASC: Result:='AASC';
 AV_CODEC_ID_INDEO2: Result:='INDEO2';
 AV_CODEC_ID_FRAPS: Result:='FRAPS';
 AV_CODEC_ID_TRUEMOTION2: Result:='TRUEMOTION2';
 AV_CODEC_ID_BMP: Result:='BMP';
 AV_CODEC_ID_CSCD: Result:='CSCD';
 AV_CODEC_ID_MMVIDEO: Result:='MMVIDEO';
 AV_CODEC_ID_ZMBV: Result:='ZMBV';
 AV_CODEC_ID_AVS: Result:='AVS';
 AV_CODEC_ID_SMACKVIDEO: Result:='SMACKVIDEO';
 AV_CODEC_ID_NUV: Result:='NUV';
 AV_CODEC_ID_KMVC: Result:='KMVC';
 AV_CODEC_ID_FLASHSV: Result:='FLASHSV';
 AV_CODEC_ID_CAVS: Result:='CAVS';
 AV_CODEC_ID_JPEG2000: Result:='JPEG2000';
 AV_CODEC_ID_VMNC: Result:='VMNC';
 AV_CODEC_ID_VP5: Result:='VP5';
 AV_CODEC_ID_VP6: Result:='VP6';
 AV_CODEC_ID_VP6F: Result:='VP6F';
 AV_CODEC_ID_TARGA: Result:='TARGA';
 AV_CODEC_ID_DSICINVIDEO: Result:='DSICINVIDEO';
 AV_CODEC_ID_TIERTEXSEQVIDEO: Result:='TIERTEXSEQVIDEO';
 AV_CODEC_ID_TIFF: Result:='TIFF';
 AV_CODEC_ID_GIF: Result:='GIF';
 AV_CODEC_ID_DXA: Result:='DXA';
 AV_CODEC_ID_DNXHD: Result:='DNXHD';
 AV_CODEC_ID_THP: Result:='THP';
 AV_CODEC_ID_SGI: Result:='SGI';
 AV_CODEC_ID_C93: Result:='C93';
 AV_CODEC_ID_BETHSOFTVID: Result:='BETHSOFTVID';
 AV_CODEC_ID_PTX: Result:='PTX';
 AV_CODEC_ID_TXD: Result:='TXD';
 AV_CODEC_ID_VP6A: Result:='VP6A';
 AV_CODEC_ID_AMV: Result:='AMV';
 AV_CODEC_ID_VB: Result:='VB';
 AV_CODEC_ID_PCX: Result:='PCX';
 AV_CODEC_ID_SUNRAST: Result:='SUNRAST';
 AV_CODEC_ID_INDEO4: Result:='INDEO4';
 AV_CODEC_ID_INDEO5: Result:='INDEO5';
 AV_CODEC_ID_MIMIC: Result:='MIMIC';
 AV_CODEC_ID_RL2: Result:='RL2';
 AV_CODEC_ID_ESCAPE124: Result:='ESCAPE124';
 AV_CODEC_ID_DIRAC: Result:='DIRAC';
 AV_CODEC_ID_BFI: Result:='BFI';
 AV_CODEC_ID_CMV: Result:='CMV';
 AV_CODEC_ID_MOTIONPIXELS: Result:='MOTIONPIXELS';
 AV_CODEC_ID_TGV: Result:='TGV';
 AV_CODEC_ID_TGQ: Result:='TGQ';
 AV_CODEC_ID_TQI: Result:='TQI';
 AV_CODEC_ID_AURA: Result:='AURA';
 AV_CODEC_ID_AURA2: Result:='AURA2';
 AV_CODEC_ID_V210X: Result:='V210X';
 AV_CODEC_ID_TMV: Result:='TMV';
 AV_CODEC_ID_V210: Result:='V210';
 AV_CODEC_ID_DPX: Result:='DPX';
 AV_CODEC_ID_MAD: Result:='MAD';
 AV_CODEC_ID_FRWU: Result:='FRWU';
 AV_CODEC_ID_FLASHSV2: Result:='FLASHSV2';
 AV_CODEC_ID_CDGRAPHICS: Result:='CDGRAPHICS';
 AV_CODEC_ID_R210: Result:='R210';
 AV_CODEC_ID_ANM: Result:='ANM';
 AV_CODEC_ID_BINKVIDEO: Result:='BINKVIDEO';
 AV_CODEC_ID_IFF_ILBM: Result:='IFF_ILBM';
 AV_CODEC_ID_KGV1: Result:='KGV1';
 AV_CODEC_ID_YOP: Result:='YOP';
 AV_CODEC_ID_VP8: Result:='VP8';
 AV_CODEC_ID_PICTOR: Result:='PICTOR';
 AV_CODEC_ID_ANSI: Result:='ANSI';
 AV_CODEC_ID_A64_MULTI: Result:='A64_MULTI';
 AV_CODEC_ID_A64_MULTI5: Result:='A64_MULTI5';
 AV_CODEC_ID_R10K: Result:='R10K';
 AV_CODEC_ID_MXPEG: Result:='MXPEG';
 AV_CODEC_ID_LAGARITH: Result:='LAGARITH';
 AV_CODEC_ID_PRORES: Result:='PRORES';
 AV_CODEC_ID_JV: Result:='JV';
 AV_CODEC_ID_DFA: Result:='DFA';
 AV_CODEC_ID_WMV3IMAGE: Result:='WMV3IMAGE';
 AV_CODEC_ID_VC1IMAGE: Result:='VC1IMAGE';
 AV_CODEC_ID_UTVIDEO: Result:='UTVIDEO';
 AV_CODEC_ID_BMV_VIDEO: Result:='BMV_VIDEO';
 AV_CODEC_ID_VBLE: Result:='VBLE';
 AV_CODEC_ID_DXTORY: Result:='DXTORY';
 AV_CODEC_ID_V410: Result:='V410';
 AV_CODEC_ID_XWD: Result:='XWD';
 AV_CODEC_ID_CDXL: Result:='CDXL';
 AV_CODEC_ID_XBM: Result:='XBM';
 AV_CODEC_ID_ZEROCODEC: Result:='ZEROCODEC';
 AV_CODEC_ID_MSS1: Result:='MSS1';
 AV_CODEC_ID_MSA1: Result:='MSA1';
 AV_CODEC_ID_TSCC2: Result:='TSCC2';
 AV_CODEC_ID_MTS2: Result:='MTS2';
 AV_CODEC_ID_CLLC: Result:='CLLC';
 AV_CODEC_ID_MSS2: Result:='MSS2';
 AV_CODEC_ID_VP9: Result:='VP9';
 AV_CODEC_ID_AIC: Result:='AIC';
 AV_CODEC_ID_ESCAPE130: Result:='ESCAPE130';
 AV_CODEC_ID_G2M: Result:='G2M';
 AV_CODEC_ID_WEBP: Result:='WEBP';
 AV_CODEC_ID_HNM4_VIDEO: Result:='HNM4_VIDEO';
 AV_CODEC_ID_HEVC: Result:='HEVC';
 AV_CODEC_ID_FIC: Result:='FIC';
 AV_CODEC_ID_ALIAS_PIX: Result:='ALIAS_PIX';
 AV_CODEC_ID_BRENDER_PIX: Result:='BRENDER_PIX';
 AV_CODEC_ID_PAF_VIDEO: Result:='PAF_VIDEO';
 AV_CODEC_ID_EXR: Result:='EXR';
 AV_CODEC_ID_VP7: Result:='VP7';
 AV_CODEC_ID_SANM: Result:='SANM';
 AV_CODEC_ID_SGIRLE: Result:='SGIRLE';
 AV_CODEC_ID_MVC1: Result:='MVC1';
 AV_CODEC_ID_MVC2: Result:='MVC2';
 AV_CODEC_ID_HQX: Result:='HQX';
 AV_CODEC_ID_TDSC: Result:='TDSC';
 AV_CODEC_ID_HQ_HQA: Result:='HQ_HQA';
 AV_CODEC_ID_HAP: Result:='HAP';
 AV_CODEC_ID_DDS: Result:='DDS';
 AV_CODEC_ID_DXV: Result:='DXV';
 AV_CODEC_ID_SCREENPRESSO: Result:='SCREENPRESSO';
 AV_CODEC_ID_RSCC: Result:='RSCC';
 AV_CODEC_ID_Y41P: Result:='Y41P';
 AV_CODEC_ID_AVRP: Result:='AVRP';
 AV_CODEC_ID_012V: Result:='012V';
 AV_CODEC_ID_AVUI: Result:='AVUI';
 AV_CODEC_ID_AYUV: Result:='AYUV';
 AV_CODEC_ID_TARGA_Y216: Result:='TARGA_Y216';
 AV_CODEC_ID_V308: Result:='V308';
 AV_CODEC_ID_V408: Result:='V408';
 AV_CODEC_ID_YUV4: Result:='YUV4';
 AV_CODEC_ID_AVRN: Result:='AVRN';
 AV_CODEC_ID_CPIA: Result:='CPIA';
 AV_CODEC_ID_XFACE: Result:='XFACE';
 AV_CODEC_ID_SNOW: Result:='SNOW';
 AV_CODEC_ID_SMVJPEG: Result:='SMVJPEG';
 AV_CODEC_ID_APNG: Result:='APNG';
 AV_CODEC_ID_DAALA: Result:='DAALA';
 AV_CODEC_ID_CFHD: Result:='CFHD';
 AV_CODEC_ID_TRUEMOTION2RT: Result:='TRUEMOTION2RT';
 AV_CODEC_ID_M101: Result:='M101';
 AV_CODEC_ID_MAGICYUV: Result:='MAGICYUV';
 AV_CODEC_ID_SHEERVIDEO: Result:='SHEERVIDEO';
 AV_CODEC_ID_YLC: Result:='YLC';
 AV_CODEC_ID_PSD: Result:='PSD';
 AV_CODEC_ID_PIXLET: Result:='PIXLET';
 AV_CODEC_ID_SPEEDHQ: Result:='SPEEDHQ';
 AV_CODEC_ID_FMVC: Result:='FMVC';
 AV_CODEC_ID_SCPR: Result:='SCPR';
 AV_CODEC_ID_CLEARVIDEO: Result:='CLEARVIDEO';
 AV_CODEC_ID_XPM: Result:='XPM';
 AV_CODEC_ID_AV1: Result:='AV1';
 AV_CODEC_ID_BITPACKED: Result:='BITPACKED';
 AV_CODEC_ID_MSCC: Result:='MSCC';
 AV_CODEC_ID_SRGC: Result:='SRGC';
 AV_CODEC_ID_SVG: Result:='SVG';
 AV_CODEC_ID_GDV: Result:='GDV';
 AV_CODEC_ID_FITS: Result:='FITS';
 AV_CODEC_ID_FIRST_AUDIO: Result:='FIRST_AUDIO';
 AV_CODEC_ID_PCM_S16BE: Result:='PCM_S16BE';
 AV_CODEC_ID_PCM_U16LE: Result:='PCM_U16LE';
 AV_CODEC_ID_PCM_U16BE: Result:='PCM_U16BE';
 AV_CODEC_ID_PCM_S8: Result:='PCM_S8';
 AV_CODEC_ID_PCM_U8: Result:='PCM_U8';
 AV_CODEC_ID_PCM_MULAW: Result:='PCM_MULAW';
 AV_CODEC_ID_PCM_ALAW: Result:='PCM_ALAW';
 AV_CODEC_ID_PCM_S32LE: Result:='PCM_S32LE';
 AV_CODEC_ID_PCM_S32BE: Result:='PCM_S32BE';
 AV_CODEC_ID_PCM_U32LE: Result:='PCM_U32LE';
 AV_CODEC_ID_PCM_U32BE: Result:='PCM_U32BE';
 AV_CODEC_ID_PCM_S24LE: Result:='PCM_S24LE';
 AV_CODEC_ID_PCM_S24BE: Result:='PCM_S24BE';
 AV_CODEC_ID_PCM_U24LE: Result:='PCM_U24LE';
 AV_CODEC_ID_PCM_U24BE: Result:='PCM_U24BE';
 AV_CODEC_ID_PCM_S24DAUD: Result:='PCM_S24DAUD';
 AV_CODEC_ID_PCM_ZORK: Result:='PCM_ZORK';
 AV_CODEC_ID_PCM_S16LE_PLANAR: Result:='PCM_S16LE_PLANAR';
 AV_CODEC_ID_PCM_DVD: Result:='PCM_DVD';
 AV_CODEC_ID_PCM_F32BE: Result:='PCM_F32BE';
 AV_CODEC_ID_PCM_F32LE: Result:='PCM_F32LE';
 AV_CODEC_ID_PCM_F64BE: Result:='PCM_F64BE';
 AV_CODEC_ID_PCM_F64LE: Result:='PCM_F64LE';
 AV_CODEC_ID_PCM_BLURAY: Result:='PCM_BLURAY';
 AV_CODEC_ID_PCM_LXF: Result:='PCM_LXF';
 AV_CODEC_ID_S302M: Result:='S302M';
 AV_CODEC_ID_PCM_S8_PLANAR: Result:='PCM_S8_PLANAR';
 AV_CODEC_ID_PCM_S24LE_PLANAR: Result:='PCM_S24LE_PLANAR';
 AV_CODEC_ID_PCM_S32LE_PLANAR: Result:='PCM_S32LE_PLANAR';
 AV_CODEC_ID_PCM_S16BE_PLANAR: Result:='PCM_S16BE_PLANAR';
 AV_CODEC_ID_PCM_S64LE: Result:='PCM_S64LE';
 AV_CODEC_ID_PCM_S64BE: Result:='PCM_S64BE';
 AV_CODEC_ID_PCM_F16LE: Result:='PCM_F16LE';
 AV_CODEC_ID_PCM_F24LE: Result:='PCM_F24LE';
 AV_CODEC_ID_ADPCM_IMA_QT: Result:='ADPCM_IMA_QT';
 AV_CODEC_ID_ADPCM_IMA_WAV: Result:='ADPCM_IMA_WAV';
 AV_CODEC_ID_ADPCM_IMA_DK3: Result:='ADPCM_IMA_DK3';
 AV_CODEC_ID_ADPCM_IMA_DK4: Result:='ADPCM_IMA_DK4';
 AV_CODEC_ID_ADPCM_IMA_WS: Result:='ADPCM_IMA_WS';
 AV_CODEC_ID_ADPCM_IMA_SMJPEG: Result:='ADPCM_IMA_SMJPEG';
 AV_CODEC_ID_ADPCM_MS: Result:='ADPCM_MS';
 AV_CODEC_ID_ADPCM_4XM: Result:='ADPCM_4XM';
 AV_CODEC_ID_ADPCM_XA: Result:='ADPCM_XA';
 AV_CODEC_ID_ADPCM_ADX: Result:='ADPCM_ADX';
 AV_CODEC_ID_ADPCM_EA: Result:='ADPCM_EA';
 AV_CODEC_ID_ADPCM_G726: Result:='ADPCM_G726';
 AV_CODEC_ID_ADPCM_CT: Result:='ADPCM_CT';
 AV_CODEC_ID_ADPCM_SWF: Result:='ADPCM_SWF';
 AV_CODEC_ID_ADPCM_YAMAHA: Result:='ADPCM_YAMAHA';
 AV_CODEC_ID_ADPCM_SBPRO_4: Result:='ADPCM_SBPRO_4';
 AV_CODEC_ID_ADPCM_SBPRO_3: Result:='ADPCM_SBPRO_3';
 AV_CODEC_ID_ADPCM_SBPRO_2: Result:='ADPCM_SBPRO_2';
 AV_CODEC_ID_ADPCM_THP: Result:='ADPCM_THP';
 AV_CODEC_ID_ADPCM_IMA_AMV: Result:='ADPCM_IMA_AMV';
 AV_CODEC_ID_ADPCM_EA_R1: Result:='ADPCM_EA_R1';
 AV_CODEC_ID_ADPCM_EA_R3: Result:='ADPCM_EA_R3';
 AV_CODEC_ID_ADPCM_EA_R2: Result:='ADPCM_EA_R2';
 AV_CODEC_ID_ADPCM_IMA_EA_SEAD: Result:='ADPCM_IMA_EA_SEAD';
 AV_CODEC_ID_ADPCM_IMA_EA_EACS: Result:='ADPCM_IMA_EA_EACS';
 AV_CODEC_ID_ADPCM_EA_XAS: Result:='ADPCM_EA_XAS';
 AV_CODEC_ID_ADPCM_EA_MAXIS_XA: Result:='ADPCM_EA_MAXIS_XA';
 AV_CODEC_ID_ADPCM_IMA_ISS: Result:='ADPCM_IMA_ISS';
 AV_CODEC_ID_ADPCM_G722: Result:='ADPCM_G722';
 AV_CODEC_ID_ADPCM_IMA_APC: Result:='ADPCM_IMA_APC';
 AV_CODEC_ID_ADPCM_VIMA: Result:='ADPCM_VIMA';
 AV_CODEC_ID_ADPCM_AFC: Result:='ADPCM_AFC';
 AV_CODEC_ID_ADPCM_IMA_OKI: Result:='ADPCM_IMA_OKI';
 AV_CODEC_ID_ADPCM_DTK: Result:='ADPCM_DTK';
 AV_CODEC_ID_ADPCM_IMA_RAD: Result:='ADPCM_IMA_RAD';
 AV_CODEC_ID_ADPCM_G726LE: Result:='ADPCM_G726LE';
 AV_CODEC_ID_ADPCM_THP_LE: Result:='ADPCM_THP_LE';
 AV_CODEC_ID_ADPCM_PSX: Result:='ADPCM_PSX';
 AV_CODEC_ID_ADPCM_AICA: Result:='ADPCM_AICA';
 AV_CODEC_ID_ADPCM_IMA_DAT4: Result:='ADPCM_IMA_DAT4';
 AV_CODEC_ID_ADPCM_MTAF: Result:='ADPCM_MTAF';
 AV_CODEC_ID_AMR_NB: Result:='AMR_NB';
 AV_CODEC_ID_AMR_WB: Result:='AMR_WB';
 AV_CODEC_ID_RA_144: Result:='RA_144';
 AV_CODEC_ID_RA_288: Result:='RA_288';
 AV_CODEC_ID_ROQ_DPCM: Result:='ROQ_DPCM';
 AV_CODEC_ID_INTERPLAY_DPCM: Result:='INTERPLAY_DPCM';
 AV_CODEC_ID_XAN_DPCM: Result:='XAN_DPCM';
 AV_CODEC_ID_SOL_DPCM: Result:='SOL_DPCM';
 AV_CODEC_ID_SDX2_DPCM: Result:='SDX2_DPCM';
 AV_CODEC_ID_GREMLIN_DPCM: Result:='GREMLIN_DPCM';
 AV_CODEC_ID_MP2: Result:='MP2';
 AV_CODEC_ID_MP3: Result:='MP3';
 AV_CODEC_ID_AAC: Result:='AAC';
 AV_CODEC_ID_AC3: Result:='AC3';
 AV_CODEC_ID_DTS: Result:='DTS';
 AV_CODEC_ID_VORBIS: Result:='VORBIS';
 AV_CODEC_ID_DVAUDIO: Result:='DVAUDIO';
 AV_CODEC_ID_WMAV1: Result:='WMAV1';
 AV_CODEC_ID_WMAV2: Result:='WMAV2';
 AV_CODEC_ID_MACE3: Result:='MACE3';
 AV_CODEC_ID_MACE6: Result:='MACE6';
 AV_CODEC_ID_VMDAUDIO: Result:='VMDAUDIO';
 AV_CODEC_ID_FLAC: Result:='FLAC';
 AV_CODEC_ID_MP3ADU: Result:='MP3ADU';
 AV_CODEC_ID_MP3ON4: Result:='MP3ON4';
 AV_CODEC_ID_SHORTEN: Result:='SHORTEN';
 AV_CODEC_ID_ALAC: Result:='ALAC';
 AV_CODEC_ID_WESTWOOD_SND1: Result:='WESTWOOD_SND1';
 AV_CODEC_ID_GSM: Result:='GSM';
 AV_CODEC_ID_QDM2: Result:='QDM2';
 AV_CODEC_ID_COOK: Result:='COOK';
 AV_CODEC_ID_TRUESPEECH: Result:='TRUESPEECH';
 AV_CODEC_ID_TTA: Result:='TTA';
 AV_CODEC_ID_SMACKAUDIO: Result:='SMACKAUDIO';
 AV_CODEC_ID_QCELP: Result:='QCELP';
 AV_CODEC_ID_WAVPACK: Result:='WAVPACK';
 AV_CODEC_ID_DSICINAUDIO: Result:='DSICINAUDIO';
 AV_CODEC_ID_IMC: Result:='IMC';
 AV_CODEC_ID_MUSEPACK7: Result:='MUSEPACK7';
 AV_CODEC_ID_MLP: Result:='MLP';
 AV_CODEC_ID_GSM_MS: Result:='GSM_MS';
 AV_CODEC_ID_ATRAC3: Result:='ATRAC3';
{$ifdef FF_API_VOXWARE}
 AV_CODEC_ID_VOXWARE: Result:='VOXWARE';
{$endif}
 AV_CODEC_ID_APE: Result:='APE';
 AV_CODEC_ID_NELLYMOSER: Result:='NELLYMOSER';
 AV_CODEC_ID_MUSEPACK8: Result:='MUSEPACK8';
 AV_CODEC_ID_SPEEX: Result:='SPEEX';
 AV_CODEC_ID_WMAVOICE: Result:='WMAVOICE';
 AV_CODEC_ID_WMAPRO: Result:='WMAPRO';
 AV_CODEC_ID_WMALOSSLESS: Result:='WMALOSSLESS';
 AV_CODEC_ID_ATRAC3P: Result:='ATRAC3P';
 AV_CODEC_ID_EAC3: Result:='EAC3';
 AV_CODEC_ID_SIPR: Result:='SIPR';
 AV_CODEC_ID_MP1: Result:='MP1';
 AV_CODEC_ID_TWINVQ: Result:='TWINVQ';
 AV_CODEC_ID_TRUEHD: Result:='TRUEHD';
 AV_CODEC_ID_MP4ALS: Result:='MP4ALS';
 AV_CODEC_ID_ATRAC1: Result:='ATRAC1';
 AV_CODEC_ID_BINKAUDIO_RDFT: Result:='BINKAUDIO_RDFT';
 AV_CODEC_ID_BINKAUDIO_DCT: Result:='BINKAUDIO_DCT';
 AV_CODEC_ID_AAC_LATM: Result:='AAC_LATM';
 AV_CODEC_ID_QDMC: Result:='QDMC';
 AV_CODEC_ID_CELT: Result:='CELT';
 AV_CODEC_ID_G723_1: Result:='G723_1';
 AV_CODEC_ID_G729: Result:='G729';
 AV_CODEC_ID_8SVX_EXP: Result:='8SVX_EXP';
 AV_CODEC_ID_8SVX_FIB: Result:='8SVX_FIB';
 AV_CODEC_ID_BMV_AUDIO: Result:='BMV_AUDIO';
 AV_CODEC_ID_RALF: Result:='RALF';
 AV_CODEC_ID_IAC: Result:='IAC';
 AV_CODEC_ID_ILBC: Result:='ILBC';
 AV_CODEC_ID_OPUS: Result:='OPUS';
 AV_CODEC_ID_COMFORT_NOISE: Result:='COMFORT_NOISE';
 AV_CODEC_ID_TAK: Result:='TAK';
 AV_CODEC_ID_METASOUND: Result:='METASOUND';
 AV_CODEC_ID_PAF_AUDIO: Result:='PAF_AUDIO';
 AV_CODEC_ID_ON2AVC: Result:='ON2AVC';
 AV_CODEC_ID_DSS_SP: Result:='DSS_SP';
 AV_CODEC_ID_FFWAVESYNTH: Result:='FFWAVESYNTH';
 AV_CODEC_ID_SONIC: Result:='SONIC';
 AV_CODEC_ID_SONIC_LS: Result:='SONIC_LS';
 AV_CODEC_ID_EVRC: Result:='EVRC';
 AV_CODEC_ID_SMV: Result:='SMV';
 AV_CODEC_ID_DSD_LSBF: Result:='DSD_LSBF';
 AV_CODEC_ID_DSD_MSBF: Result:='DSD_MSBF';
 AV_CODEC_ID_DSD_LSBF_PLANAR: Result:='DSD_LSBF_PLANAR';
 AV_CODEC_ID_DSD_MSBF_PLANAR: Result:='DSD_MSBF_PLANAR';
 AV_CODEC_ID_4GV: Result:='4GV';
 AV_CODEC_ID_INTERPLAY_ACM: Result:='INTERPLAY_ACM';
 AV_CODEC_ID_XMA1: Result:='XMA1';
 AV_CODEC_ID_XMA2: Result:='XMA2';
 AV_CODEC_ID_DST: Result:='DST';
 AV_CODEC_ID_ATRAC3AL: Result:='ATRAC3AL';
 AV_CODEC_ID_ATRAC3PAL: Result:='ATRAC3PAL';
 AV_CODEC_ID_DOLBY_E: Result:='DOLBY_E';
 AV_CODEC_ID_FIRST_SUBTITLE: Result:='FIRST_SUBTITLE';
 AV_CODEC_ID_DVB_SUBTITLE: Result:='DVD_SUBTITLE/DVB_SUBTITLE';
 AV_CODEC_ID_TEXT: Result:='TEXT';
 AV_CODEC_ID_XSUB: Result:='XSUB';
 AV_CODEC_ID_SSA: Result:='SSA';
 AV_CODEC_ID_MOV_TEXT: Result:='MOV_TEXT';
 AV_CODEC_ID_HDMV_PGS_SUBTITLE: Result:='HDMV_PGS_SUBTITLE';
 AV_CODEC_ID_DVB_TELETEXT: Result:='DVB_TELETEXT';
 AV_CODEC_ID_SRT: Result:='SRT';
 AV_CODEC_ID_MICRODVD: Result:='MICRODVD: ';
 AV_CODEC_ID_EIA_608: Result:='EIA_608';
 AV_CODEC_ID_JACOSUB: Result:='JACOSUB';
 AV_CODEC_ID_SAMI: Result:='SAMI';
 AV_CODEC_ID_REALTEXT: Result:='REALTEXT';
 AV_CODEC_ID_STL: Result:='STL';
 AV_CODEC_ID_SUBVIEWER1: Result:='SUBVIEWER1';
 AV_CODEC_ID_SUBVIEWER: Result:='SUBVIEWER';
 AV_CODEC_ID_SUBRIP: Result:='SUBRIP';
 AV_CODEC_ID_WEBVTT: Result:='WEBVTT';
 AV_CODEC_ID_MPL2: Result:='MPL2';
 AV_CODEC_ID_VPLAYER: Result:='VPLAYER';
 AV_CODEC_ID_PJS: Result:='PJS';
 AV_CODEC_ID_ASS: Result:='ASS';
 AV_CODEC_ID_HDMV_TEXT_SUBTITLE: Result:='HDMV_TEXT_SUBTITLE';
 AV_CODEC_ID_TTF: Result:='TTF';
 AV_CODEC_ID_SCTE_35: Result:='SCTE_35: ';
 AV_CODEC_ID_BINTEXT: Result:='BINTEXT: ';
 AV_CODEC_ID_XBIN: Result:='XBIN';
 AV_CODEC_ID_IDF: Result:='IDF';
 AV_CODEC_ID_OTF: Result:='OTF';
 AV_CODEC_ID_SMPTE_KLV: Result:='SMPTE_KLV';
 AV_CODEC_ID_DVD_NAV: Result:='DVD_NAV';
 AV_CODEC_ID_TIMED_ID3: Result:='TIMED_ID3';
 AV_CODEC_ID_BIN_DATA: Result:='BIN_DATA';
 AV_CODEC_ID_PROBE: Result:='PROBE';
 AV_CODEC_ID_MPEG2TS: Result:='MPEG2TS';
 AV_CODEC_ID_MPEG4SYSTEMS: Result:='MPEG4SYSTEMS';
 AV_CODEC_ID_FFMETADATA: Result:='FFMETADATA';
 AV_CODEC_ID_WRAPPED_AVFRAME: Result:='WRAPPED_AVFRAME';
 else Result:='Unknown';
 end;
end;

function GetPixelFormatStr(PF: TAVPixelFormat): string;
begin
case PF of
 AV_PIX_FMT_NONE: Result:='NONE';
 AV_PIX_FMT_YUV420P: Result:='YUV420P';
 AV_PIX_FMT_YUYV422: Result:='YUYV422';
 AV_PIX_FMT_RGB24: Result:='RGB24';
 AV_PIX_FMT_BGR24: Result:='BGR24';
 AV_PIX_FMT_YUV422P: Result:='YUV422P';
 AV_PIX_FMT_YUV444P: Result:='YUV444P';
 AV_PIX_FMT_YUV410P: Result:='YUV410P';
 AV_PIX_FMT_YUV411P: Result:='YUV411P';
 AV_PIX_FMT_GRAY8: Result:='GRAY8';
 AV_PIX_FMT_MONOWHITE: Result:='MONOWHITE';
 AV_PIX_FMT_MONOBLACK: Result:='MONOBLACK';
 AV_PIX_FMT_PAL8: Result:='PAL8';
 AV_PIX_FMT_YUVJ420P: Result:='YUVJ420P';
 AV_PIX_FMT_YUVJ422P: Result:='YUVJ422P';
 AV_PIX_FMT_YUVJ444P: Result:='YUVJ444P';
 AV_PIX_FMT_XVMC_MPEG2_MC: Result:='XVMC_MPEG2_MC';
 AV_PIX_FMT_XVMC_MPEG2_IDCT: Result:='XVMC_MPEG2_IDCT';
 AV_PIX_FMT_UYVY422: Result:='UYVY422';
 AV_PIX_FMT_UYYVYY411: Result:='UYYVYY411';
 AV_PIX_FMT_BGR8: Result:='BGR8';
 AV_PIX_FMT_BGR4: Result:='BGR4';
 AV_PIX_FMT_BGR4_BYTE: Result:='BGR4_BYTE';
 AV_PIX_FMT_RGB8: Result:='RGB8';
 AV_PIX_FMT_RGB4: Result:='RGB4';
 AV_PIX_FMT_RGB4_BYTE: Result:='RGB4_BYTE';
 AV_PIX_FMT_NV12: Result:='NV12';
 AV_PIX_FMT_NV21: Result:='NV21';
 AV_PIX_FMT_ARGB: Result:='ARGB';
 AV_PIX_FMT_RGBA: Result:='RGBA';
 AV_PIX_FMT_ABGR: Result:='ABGR';
 AV_PIX_FMT_BGRA: Result:='BGRA';
 AV_PIX_FMT_GRAY16BE: Result:='GRAY16BE';
 AV_PIX_FMT_GRAY16LE: Result:='GRAY16LE';
 AV_PIX_FMT_YUV440P: Result:='YUV440P';
 AV_PIX_FMT_YUVJ440P: Result:='YUVJ440P';
 AV_PIX_FMT_YUVA420P: Result:='YUVA420P';
 AV_PIX_FMT_VDPAU_H264: Result:='VDPAU_H264';
 AV_PIX_FMT_VDPAU_MPEG1: Result:='VDPAU_MPEG1';
 AV_PIX_FMT_VDPAU_MPEG2: Result:='VDPAU_MPEG2';
 AV_PIX_FMT_VDPAU_WMV3: Result:='VDPAU_WMV3';
 AV_PIX_FMT_VDPAU_VC1: Result:='VDPAU_VC1';
 AV_PIX_FMT_RGB48BE: Result:='RGB48BE';
 AV_PIX_FMT_RGB48LE: Result:='RGB48LE';
 AV_PIX_FMT_RGB565BE: Result:='RGB565BE';
 AV_PIX_FMT_RGB565LE: Result:='RGB565LE';
 AV_PIX_FMT_RGB555BE: Result:='RGB555BE';
 AV_PIX_FMT_RGB555LE: Result:='RGB555LE';
 AV_PIX_FMT_BGR565BE: Result:='BGR565BE';
 AV_PIX_FMT_BGR565LE: Result:='BGR565LE';
 AV_PIX_FMT_BGR555BE: Result:='BGR555BE';
 AV_PIX_FMT_BGR555LE: Result:='BGR555LE';
 AV_PIX_FMT_VAAPI_MOCO: Result:='VAAPI_MOCO';
 AV_PIX_FMT_VAAPI_IDCT: Result:='VAAPI_IDCT';
 AV_PIX_FMT_VAAPI_VLD: Result:='VAAPI_VLD';
// AV_PIX_FMT_VAAPI: Result:='VAAPI';
 AV_PIX_FMT_YUV420P16LE: Result:='YUV420P16LE';
 AV_PIX_FMT_YUV420P16BE: Result:='YUV420P16BE';
 AV_PIX_FMT_YUV422P16LE: Result:='YUV422P16LE';
 AV_PIX_FMT_YUV422P16BE: Result:='YUV422P16BE';
 AV_PIX_FMT_YUV444P16LE: Result:='YUV444P16LE';
 AV_PIX_FMT_YUV444P16BE: Result:='YUV444P16BE';
 AV_PIX_FMT_VDPAU_MPEG4: Result:='VDPAU_MPEG4';
 AV_PIX_FMT_DXVA2_VLD: Result:='DXVA2_VLD';
 AV_PIX_FMT_RGB444LE: Result:='RGB444LE';
 AV_PIX_FMT_RGB444BE: Result:='RGB444BE';
 AV_PIX_FMT_BGR444LE: Result:='BGR444LE';
 AV_PIX_FMT_BGR444BE: Result:='BGR444BE';
 AV_PIX_FMT_YA8: Result:='YA8/Y400A/GRAY8A';
 AV_PIX_FMT_BGR48BE: Result:='BGR48BE';
 AV_PIX_FMT_BGR48LE: Result:='BGR48LE';
 AV_PIX_FMT_YUV420P9BE: Result:='YUV420P9BE';
 AV_PIX_FMT_YUV420P9LE: Result:='YUV420P9LE';
 AV_PIX_FMT_YUV420P10BE: Result:='YUV420P10BE';
 AV_PIX_FMT_YUV420P10LE: Result:='YUV420P10LE';
 AV_PIX_FMT_YUV422P10BE: Result:='YUV422P10BE';
 AV_PIX_FMT_YUV422P10LE: Result:='YUV422P10LE';
 AV_PIX_FMT_YUV444P9BE: Result:='YUV444P9BE';
 AV_PIX_FMT_YUV444P9LE: Result:='YUV444P9LE';
 AV_PIX_FMT_YUV444P10BE: Result:='YUV444P10BE';
 AV_PIX_FMT_YUV444P10LE: Result:='YUV444P10LE';
 AV_PIX_FMT_YUV422P9BE: Result:='YUV422P9BE';
 AV_PIX_FMT_YUV422P9LE: Result:='YUV422P9LE';
 AV_PIX_FMT_VDA_VLD: Result:='VDA_VLD';
 AV_PIX_FMT_GBRP: Result:='GBRP/GBR24P';
 AV_PIX_FMT_GBRP9BE: Result:='GBRP9BE';
 AV_PIX_FMT_GBRP9LE: Result:='GBRP9LE';
 AV_PIX_FMT_GBRP10BE: Result:='GBRP10BE';
 AV_PIX_FMT_GBRP10LE: Result:='GBRP10LE';
 AV_PIX_FMT_GBRP16BE: Result:='GBRP16BE';
 AV_PIX_FMT_GBRP16LE: Result:='GBRP16LE';
 AV_PIX_FMT_YUVA422P: Result:='YUVA422P';
 AV_PIX_FMT_YUVA444P: Result:='YUVA444P';
 AV_PIX_FMT_YUVA420P9BE: Result:='YUVA420P9BE';
 AV_PIX_FMT_YUVA420P9LE: Result:='YUVA420P9LE';
 AV_PIX_FMT_YUVA422P9BE: Result:='YUVA422P9BE';
 AV_PIX_FMT_YUVA422P9LE: Result:='YUVA422P9LE';
 AV_PIX_FMT_YUVA444P9BE: Result:='YUVA444P9BE';
 AV_PIX_FMT_YUVA444P9LE: Result:='YUVA444P9LE';
 AV_PIX_FMT_YUVA420P10BE: Result:='YUVA420P10BE';
 AV_PIX_FMT_YUVA420P10LE: Result:='YUVA420P10LE';
 AV_PIX_FMT_YUVA422P10BE: Result:='YUVA422P10BE';
 AV_PIX_FMT_YUVA422P10LE: Result:='YUVA422P10LE';
 AV_PIX_FMT_YUVA444P10BE: Result:='YUVA444P10BE';
 AV_PIX_FMT_YUVA444P10LE: Result:='YUVA444P10LE';
 AV_PIX_FMT_YUVA420P16BE: Result:='YUVA420P16BE';
 AV_PIX_FMT_YUVA420P16LE: Result:='YUVA420P16LE';
 AV_PIX_FMT_YUVA422P16BE: Result:='YUVA422P16BE';
 AV_PIX_FMT_YUVA422P16LE: Result:='YUVA422P16LE';
 AV_PIX_FMT_YUVA444P16BE: Result:='YUVA444P16BE';
 AV_PIX_FMT_YUVA444P16LE: Result:='YUVA444P16LE';
 AV_PIX_FMT_VDPAU: Result:='VDPAU';
 AV_PIX_FMT_XYZ12LE: Result:='XYZ12LE';
 AV_PIX_FMT_XYZ12BE: Result:='XYZ12BE';
 AV_PIX_FMT_NV16: Result:='NV16';
 AV_PIX_FMT_NV20LE: Result:='NV20LE';
 AV_PIX_FMT_NV20BE: Result:='NV20BE';
 AV_PIX_FMT_RGBA64BE: Result:='RGBA64BE';
 AV_PIX_FMT_RGBA64LE: Result:='RGBA64LE';
 AV_PIX_FMT_BGRA64BE: Result:='BGRA64BE';
 AV_PIX_FMT_BGRA64LE: Result:='BGRA64LE';
 AV_PIX_FMT_YVYU422: Result:='YVYU422';
 AV_PIX_FMT_VDA: Result:='VDA';
 AV_PIX_FMT_YA16BE: Result:='YA16BE';
 AV_PIX_FMT_YA16LE: Result:='YA16LE';
 AV_PIX_FMT_GBRAP: Result:='GBRAP';
 AV_PIX_FMT_GBRAP16BE: Result:='GBRAP16BE';
 AV_PIX_FMT_GBRAP16LE: Result:='GBRAP16LE';
 AV_PIX_FMT_QSV: Result:='QSV';
 AV_PIX_FMT_MMAL: Result:='MMAL';
 AV_PIX_FMT_D3D11VA_VLD: Result:='D3D11VA_VLD';
 AV_PIX_FMT_CUDA: Result:='CUDA';
 AV_PIX_FMT_0RGB: Result:='0RGB';
 AV_PIX_FMT_RGB0: Result:='RGB0';
 AV_PIX_FMT_0BGR: Result:='0BGR';
 AV_PIX_FMT_BGR0: Result:='BGR0';
 AV_PIX_FMT_YUV420P12BE: Result:='YUV420P12BE';
 AV_PIX_FMT_YUV420P12LE: Result:='YUV420P12LE';
 AV_PIX_FMT_YUV420P14BE: Result:='YUV420P14BE';
 AV_PIX_FMT_YUV420P14LE: Result:='YUV420P14LE';
 AV_PIX_FMT_YUV422P12BE: Result:='YUV422P12BE';
 AV_PIX_FMT_YUV422P12LE: Result:='YUV422P12LE';
 AV_PIX_FMT_YUV422P14BE: Result:='YUV422P14BE';
 AV_PIX_FMT_YUV422P14LE: Result:='YUV422P14LE';
 AV_PIX_FMT_YUV444P12BE: Result:='YUV444P12BE';
 AV_PIX_FMT_YUV444P12LE: Result:='YUV444P12LE';
 AV_PIX_FMT_YUV444P14BE: Result:='YUV444P14BE';
 AV_PIX_FMT_YUV444P14LE: Result:='YUV444P14LE';
 AV_PIX_FMT_GBRP12BE: Result:='GBRP12BE';
 AV_PIX_FMT_GBRP12LE: Result:='GBRP12LE';
 AV_PIX_FMT_GBRP14BE: Result:='GBRP14BE';
 AV_PIX_FMT_GBRP14LE: Result:='GBRP14LE';
 AV_PIX_FMT_YUVJ411P: Result:='YUVJ411P';
 AV_PIX_FMT_BAYER_BGGR8: Result:='BAYER_BGGR8';
 AV_PIX_FMT_BAYER_RGGB8: Result:='BAYER_RGGB8';
 AV_PIX_FMT_BAYER_GBRG8: Result:='BAYER_GBRG8';
 AV_PIX_FMT_BAYER_GRBG8: Result:='BAYER_GRBG8';
 AV_PIX_FMT_BAYER_BGGR16LE: Result:='BAYER_BGGR16LE';
 AV_PIX_FMT_BAYER_BGGR16BE: Result:='BAYER_BGGR16BE';
 AV_PIX_FMT_BAYER_RGGB16LE: Result:='BAYER_RGGB16LE';
 AV_PIX_FMT_BAYER_RGGB16BE: Result:='BAYER_RGGB16BE';
 AV_PIX_FMT_BAYER_GBRG16LE: Result:='BAYER_GBRG16LE';
 AV_PIX_FMT_BAYER_GBRG16BE: Result:='BAYER_GBRG16BE';
 AV_PIX_FMT_BAYER_GRBG16LE: Result:='BAYER_GRBG16LE';
 AV_PIX_FMT_BAYER_GRBG16BE: Result:='BAYER_GRBG16BE';
// AV_PIX_FMT_XVMC: Result:='XVMC';
 AV_PIX_FMT_YUV440P10LE: Result:='YUV440P10LE';
 AV_PIX_FMT_YUV440P10BE: Result:='YUV440P10BE';
 AV_PIX_FMT_YUV440P12LE: Result:='YUV440P12LE';
 AV_PIX_FMT_YUV440P12BE: Result:='YUV440P12BE';
 AV_PIX_FMT_AYUV64LE: Result:='AYUV64LE';
 AV_PIX_FMT_AYUV64BE: Result:='AYUV64BE';
 AV_PIX_FMT_VIDEOTOOLBOX: Result:='VIDEOTOOLBOX';
 AV_PIX_FMT_P010LE: Result:='P010LE';
 AV_PIX_FMT_P010BE: Result:='P010BE';
 AV_PIX_FMT_GBRAP12BE: Result:='GBRAP12BE';
 AV_PIX_FMT_GBRAP12LE: Result:='GBRAP12LE';
 AV_PIX_FMT_GBRAP10BE: Result:='GBRAP10BE';
 AV_PIX_FMT_GBRAP10LE: Result:='GBRAP10LE';
 AV_PIX_FMT_MEDIACODEC: Result:='MEDIACODEC';
 AV_PIX_FMT_GRAY12BE: Result:='GRAY12BE';
 AV_PIX_FMT_GRAY12LE: Result:='GRAY12LE';
 AV_PIX_FMT_GRAY10BE: Result:='GRAY10BE';
 AV_PIX_FMT_GRAY10LE: Result:='GRAY10LE';
 AV_PIX_FMT_P016LE: Result:='P016LE';
 AV_PIX_FMT_P016BE: Result:='P016BE';
 AV_PIX_FMT_D3D11: Result:='D3D11';
 AV_PIX_FMT_GRAY9BE: Result:='GRAY9BE';
 AV_PIX_FMT_GRAY9LE: Result:='GRAY9LE';
 AV_PIX_FMT_GBRPF32BE: Result:='GBRPF32BE';
 AV_PIX_FMT_GBRPF32LE: Result:='GBRPF32LE';
 AV_PIX_FMT_GBRAPF32BE: Result:='GBRAPF32BE';
 AV_PIX_FMT_GBRAPF32LE: Result:='GBRAPF32LE';
 AV_PIX_FMT_DRM_PRIME: Result:='DRM_PRIME';
 AV_PIX_FMT_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;



function GetSampleFormat(SF: TAVSampleFormat): string;
begin
case SF of
 AV_SAMPLE_FMT_NONE: Result:='NONE';
 AV_SAMPLE_FMT_U8: Result:='U8';
 AV_SAMPLE_FMT_S16: Result:='S16';
 AV_SAMPLE_FMT_S32: Result:='S32';
 AV_SAMPLE_FMT_FLT: Result:='FLT';
 AV_SAMPLE_FMT_DBL: Result:='DBL';
 AV_SAMPLE_FMT_U8P: Result:='U8P';
 AV_SAMPLE_FMT_S16P: Result:='S16P';
 AV_SAMPLE_FMT_S32P: Result:='S32P';
 AV_SAMPLE_FMT_FLTP: Result:='FLTP';
 AV_SAMPLE_FMT_DBLP: Result:='DBLP';
 AV_SAMPLE_FMT_S64: Result:='S64';
 AV_SAMPLE_FMT_S64P: Result:='S64P';
 AV_SAMPLE_FMT_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;


function GetDurEstMethodStr(DM: TAVDurationEstimationMethod): string;
begin
case DM of
 AVFMT_DURATION_FROM_PTS: Result:='Duration accurately estimated from PTSes';
 AVFMT_DURATION_FROM_STREAM: Result:='Duration estimated from a stream with a known duration';
 AVFMT_DURATION_FROM_BITRATE: Result:='Duration estimated from bitrate (less accurate)';
 else Result:='Unknown';
 end;
end;


function GetMediaTypeStr(MT: TAVMediaType): string;
begin
case MT of
 AVMEDIA_TYPE_UNKNOWN: Result:='UNKNOWN';
 AVMEDIA_TYPE_VIDEO: Result:='VIDEO';
 AVMEDIA_TYPE_AUDIO: Result:='AUDIO';
 AVMEDIA_TYPE_DATA: Result:='DATA';
 AVMEDIA_TYPE_SUBTITLE: Result:='SUBTITLE';
 AVMEDIA_TYPE_ATTACHMENT: Result:='ATTACHMENT';
 AVMEDIA_TYPE_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;



function GetProfileStr(PF: TAVProfile): string;
begin
Result:='Profile: '+IntToStr(PF.profile);
Result:=Result+' Name: '+PF.name;
end;



function GetDataMarkerTypeStr(MT: TAVIODataMarkerType): string;
begin
case MT of
 AVIO_DATA_MARKER_HEADER: Result:='HEADER';
 AVIO_DATA_MARKER_SYNC_POINT: Result:='SYNC_POINT';
 AVIO_DATA_MARKER_BOUNDARY_POINT: Result:='BOUNDARY_POINT';
 AVIO_DATA_MARKER_UNKNOWN: Result:='UNKNOWN';
 AVIO_DATA_MARKER_TRAILER: Result:='TRAILER';
 AVIO_DATA_MARKER_FLUSH_POINT: Result:='FLUSH_POINT';
 else Result:='Unknown';
 end;
end;


function GetFractStr(F: TAVFrac): string;
begin
Result:='Val: '+IntToStr(F.val);
Result:=Result+' Num: '+IntToStr(F.num);
Result:=Result+' Den: '+IntToStr(F.den);
end;


function GetDiscardStr(DC: TAVDiscard): string;
begin
case DC of
 AVDISCARD_NONE: Result:='NONE';
 AVDISCARD_DEFAULT: Result:='DEFAULT';
 AVDISCARD_NONREF: Result:='NONREF';
 AVDISCARD_BIDIR: Result:='BIDIR';
 AVDISCARD_NONINTRA: Result:='NONINTRA';
 AVDISCARD_NONKEY: Result:='NONKEY';
 AVDISCARD_ALL: Result:='ALL';
 else Result:='Unknown';
 end;
end;


function GetParseTypeStr(PT: TAVStreamParseType): string;
begin
case PT of
 AVSTREAM_PARSE_NONE: Result:='NONE';
 AVSTREAM_PARSE_FULL: Result:='FULL';
 AVSTREAM_PARSE_HEADERS: Result:='HEADERS';
 AVSTREAM_PARSE_TIMESTAMPS: Result:='TIMESTAMPS';
 AVSTREAM_PARSE_FULL_ONCE: Result:='FULL_ONCE';
 AVSTREAM_PARSE_FULL_RAW: Result:='FULL_RAW';
 else Result:='Unknown';
 end;
end;

function GetPacketSideDataType(DT: TAVPacketSideDataType): string;
begin
case DT of
 AV_PKT_DATA_PALETTE: Result:='PALETTE';
 AV_PKT_DATA_NEW_EXTRADATA: Result:='NEW_EXTRADATA';
 AV_PKT_DATA_PARAM_CHANGE: Result:='PARAM_CHANGE';
 AV_PKT_DATA_H263_MB_INFO: Result:='H263_MB_INFO';
 AV_PKT_DATA_REPLAYGAIN: Result:='REPLAYGAIN';
 AV_PKT_DATA_DISPLAYMATRIX: Result:='DISPLAYMATRIX';
 AV_PKT_DATA_STEREO3D: Result:='STEREO3D';
 AV_PKT_DATA_AUDIO_SERVICE_TYPE: Result:='AUDIO_SERVICE_TYPE';
 AV_PKT_DATA_QUALITY_STATS: Result:='QUALITY_STATS';
 AV_PKT_DATA_FALLBACK_TRACK: Result:='FALLBACK_TRACK';
 AV_PKT_DATA_CPB_PROPERTIES: Result:='CPB_PROPERTIES';
 AV_PKT_DATA_SKIP_SAMPLES: Result:='SKIP_SAMPLES=70';
 AV_PKT_DATA_JP_DUALMONO: Result:='JP_DUALMONO';
 AV_PKT_DATA_STRINGS_METADATA: Result:='STRINGS_METADATA';
 AV_PKT_DATA_SUBTITLE_POSITION: Result:='SUBTITLE_POSITION';
 AV_PKT_DATA_MATROSKA_BLOCKADDITIONAL: Result:='MATROSKA_BLOCKADDITIONAL';
 AV_PKT_DATA_WEBVTT_IDENTIFIER: Result:='WEBVTT_IDENTIFIER';
 AV_PKT_DATA_WEBVTT_SETTINGS: Result:='WEBVTT_SETTINGS';
 AV_PKT_DATA_METADATA_UPDATE: Result:='METADATA_UPDATE';
 AV_PKT_DATA_MPEGTS_STREAM_ID: Result:='MPEGTS_STREAM_ID';
 AV_PKT_DATA_MASTERING_DISPLAY_METADATA: Result:='MASTERING_DISPLAY_METADATA';
 AV_PKT_DATA_SPHERICAL: Result:='SPHERICAL';
 AV_PKT_DATA_CONTENT_LIGHT_LEVEL: Result:='CONTENT_LIGHT_LEVEL';
 AV_PKT_DATA_A53_CC: Result:='A53_CC';
 AV_PKT_DATA_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;


 
function GetColorPrimaryStr(CP: TAVColorPrimaries): string;
begin
case CP of 
 AVCOL_PRI_RESERVED0: Result:='RESERVED0'; 
 AVCOL_PRI_BT709: Result:='BT709';
 AVCOL_PRI_UNSPECIFIED: Result:='UNSPECIFIED';
 AVCOL_PRI_RESERVED: Result:='RESERVED';
 AVCOL_PRI_BT470M: Result:='BT470M';
 AVCOL_PRI_BT470BG: Result:='BT470BG';
 AVCOL_PRI_SMPTE170M: Result:='SMPTE170M';
 AVCOL_PRI_SMPTE240M: Result:='SMPTE240M';
 AVCOL_PRI_FILM: Result:='FILM';
 AVCOL_PRI_BT2020: Result:='BT2020';
 AVCOL_PRI_SMPTE428: Result:='SMPTE428/SMPTEST428_1';
 AVCOL_PRI_SMPTE431: Result:='SMPTE431';
 AVCOL_PRI_SMPTE432: Result:='SMPTE432';
 AVCOL_PRI_JEDEC_P22: Result:='JEDEC_P22';
 AVCOL_PRI_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;



function GetColorTransferCharStr(CT: TAVColorTransferCharacteristic): string;
begin
case CT of 
 AVCOL_TRC_RESERVED0: Result:='RESERVED0';
 AVCOL_TRC_BT709: Result:='BT709';
 AVCOL_TRC_UNSPECIFIED: Result:='UNSPECIFIED';
 AVCOL_TRC_RESERVED: Result:='RESERVED';
 AVCOL_TRC_GAMMA22: Result:='GAMMA22';
 AVCOL_TRC_GAMMA28: Result:='GAMMA28';
 AVCOL_TRC_SMPTE170M: Result:='SMPTE170M';
 AVCOL_TRC_SMPTE240M: Result:='SMPTE240M';
 AVCOL_TRC_LINEAR: Result:='LINEAR';
 AVCOL_TRC_LOG: Result:='LOG';
 AVCOL_TRC_LOG_SQRT: Result:='LOG_SQRT';
 AVCOL_TRC_IEC61966_2_4: Result:='IEC61966_2_4';
 AVCOL_TRC_BT1361_ECG: Result:='BT1361_ECG';
 AVCOL_TRC_IEC61966_2_1: Result:='IEC61966_2_1';
 AVCOL_TRC_BT2020_10: Result:='BT2020_10';
 AVCOL_TRC_BT2020_12: Result:='BT2020_12';
 AVCOL_TRC_SMPTE2084: Result:='SMPTE2084/SMPTEST2084';
 AVCOL_TRC_SMPTE428: Result:='SMPTE428/SMPTEST428_1';
 AVCOL_TRC_ARIB_STD_B67: Result:='ARIB_STD_B67';
 AVCOL_TRC_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;


function GetColorSpaceStr(CS: TAVColorSpace): string;
begin
case CS of
 AVCOL_SPC_RGB: Result:='RGB';
 AVCOL_SPC_BT709: Result:='BT709';
 AVCOL_SPC_UNSPECIFIED: Result:='UNSPECIFIED';
 AVCOL_SPC_RESERVED: Result:='RESERVED';
 AVCOL_SPC_FCC: Result:='FCC';
 AVCOL_SPC_BT470BG: Result:='BT470BG';
 AVCOL_SPC_SMPTE170M: Result:='SMPTE170M';
 AVCOL_SPC_SMPTE240M: Result:='SMPTE240M';
 AVCOL_SPC_YCGCO: Result:='YCGCO/YCOCG';
 AVCOL_SPC_BT2020_NCL: Result:='BT2020_NCL';
 AVCOL_SPC_BT2020_CL: Result:='BT2020_CL';
 AVCOL_SPC_SMPTE2085: Result:='SMPTE2085';
 AVCOL_SPC_CHROMA_DERIVED_NCL: Result:='CHROMA_DERIVED_NCL';
 AVCOL_SPC_CHROMA_DERIVED_CL: Result:='CHROMA_DERIVED_CL';
 AVCOL_SPC_ICTCP: Result:='ICTCP';
 AVCOL_SPC_NB: Result:='NB';
 end;
end;


function GetColorRangeStr(CR: TAVColorRange): string;
begin
case CR of
 AVCOL_RANGE_UNSPECIFIED: Result:='UNSPECIFIED';
 AVCOL_RANGE_MPEG: Result:='MPEG';
 AVCOL_RANGE_JPEG: Result:='JPEG';
 AVCOL_RANGE_NB: Result:='NB';
 end;
end;


function GetChromaLocationStr(CL: TAVChromaLocation): string;
begin
case CL of 
 AVCHROMA_LOC_UNSPECIFIED: Result:='UNSPECIFIED'; 
 AVCHROMA_LOC_LEFT: Result:='LEFT';
 AVCHROMA_LOC_CENTER: Result:='CENTER';
 AVCHROMA_LOC_TOPLEFT: Result:='TOPLEFT';
 AVCHROMA_LOC_TOP: Result:='TOP';
 AVCHROMA_LOC_BOTTOMLEFT: Result:='BOTTOMLEFT';
 AVCHROMA_LOC_BOTTOM: Result:='BOTTOM';
 AVCHROMA_LOC_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;




function GetFieldOrderStr(FO: TAVFieldOrder): string;
begin
case FO of
 AV_FIELD_UNKNOWN: Result:='UNKNOWN';
 AV_FIELD_PROGRESSIVE: Result:='PROGRESSIVE';
 AV_FIELD_TT: Result:='TT';
 AV_FIELD_BB: Result:='BB';
 AV_FIELD_TB: Result:='TB';
 AV_FIELD_BT: Result:='BT';
 else Result:='Unknown';
 end;
end;


function GetAudioServiceTypeStr(AST: TAVAudioServiceType): string;
begin
case AST of
 AV_AUDIO_SERVICE_TYPE_MAIN: Result:='MAIN';
 AV_AUDIO_SERVICE_TYPE_EFFECTS: Result:='EFFECTS';
 AV_AUDIO_SERVICE_TYPE_VISUALLY_IMPAIRED: Result:='VISUALLY_IMPAIRED';
 AV_AUDIO_SERVICE_TYPE_HEARING_IMPAIRED: Result:='HEARING_IMPAIRED';
 AV_AUDIO_SERVICE_TYPE_DIALOGUE: Result:='DIALOGUE';
 AV_AUDIO_SERVICE_TYPE_COMMENTARY: Result:='COMMENTARY';
 AV_AUDIO_SERVICE_TYPE_EMERGENCY: Result:='EMERGENCY';
 AV_AUDIO_SERVICE_TYPE_VOICE_OVER: Result:='VOICE_OVER';
 AV_AUDIO_SERVICE_TYPE_KARAOKE: Result:='KARAOKE';
 AV_AUDIO_SERVICE_TYPE_NB: Result:='NB';
 else Result:='Unknown';
 end;
end;

function GetPictureTypeStr(PT: TAVPictureType): string;
begin
case PT of
 AV_PICTURE_TYPE_NONE: Result:='NONE';
 AV_PICTURE_TYPE_I: Result:='I';
 AV_PICTURE_TYPE_P: Result:='P';
 AV_PICTURE_TYPE_B: Result:='B';
 AV_PICTURE_TYPE_S: Result:='S';
 AV_PICTURE_TYPE_SI: Result:='SI';
 AV_PICTURE_TYPE_SP: Result:='SP';
 AV_PICTURE_TYPE_BI: Result:='BI';
 else Result:='Unknown';
 end;
end;


function GetFrameSideTypeStr(FT: TAVFrameSideDataType): string;
begin
case FT of
 AV_FRAME_DATA_PANSCAN: Result:='PANSCAN';
 AV_FRAME_DATA_A53_CC: Result:='A53_CC';
 AV_FRAME_DATA_STEREO3D: Result:='STEREO3D';
 AV_FRAME_DATA_MATRIXENCODING: Result:='MATRIXENCODING';
 AV_FRAME_DATA_DOWNMIX_INFO: Result:='DOWNMIX_INFO';
 AV_FRAME_DATA_REPLAYGAIN: Result:='REPLAYGAIN';
 AV_FRAME_DATA_DISPLAYMATRIX: Result:='DISPLAYMATRIX';
 AV_FRAME_DATA_AFD: Result:='AFD';
 AV_FRAME_DATA_MOTION_VECTORS: Result:='MOTION_VECTORS';
 AV_FRAME_DATA_SKIP_SAMPLES: Result:='SKIP_SAMPLES';
 AV_FRAME_DATA_AUDIO_SERVICE_TYPE: Result:='AUDIO_SERVICE_TYPE';
 AV_FRAME_DATA_MASTERING_DISPLAY_METADATA: Result:='MASTERING_DISPLAY_METADATA';
 AV_FRAME_DATA_GOP_TIMECODE: Result:='GOP_TIMECODE';
 AV_FRAME_DATA_SPHERICAL: Result:='SPHERICAL';
 AV_FRAME_DATA_CONTENT_LIGHT_LEVEL: Result:='CONTENT_LIGHT_LEVEL';
 AV_FRAME_DATA_ICC_PROFILE: Result:='ICC_PROFILE';
 else Result:='Unknown';
 end;
end;



function GetRealStr(D: double): string;
begin
Result:=FloatToStrF(D,ffFixed,18,3);
end;



{xxx}
{===============================================================================}



{===============================================================================}
{xxx}


procedure ShowStreamInternal(Tree: TTreeView; Node: TTreeNode; Internal: TAVStreamInternal);
begin
with Internal do
 begin
 ShowNode(Tree,Node,'Not defined in this version',2);
 end;
end;


procedure ShowCodecParameters(Tree: TTreeView; Node: TTreeNode; Params: TAVCodecParameters);
var SubNode: TTreeNode;
begin
with Params do
 begin
 ShowNode(Tree,Node,'codec_type: '+GetMediaTypeStr(codec_type),2);   {TAVMediaType}
 ShowNode(Tree,Node,'codec_id: '+GetCodecIDStr(codec_id),2);   {TAVCodecID}
 ShowNode(Tree,Node,'codec_tag: '+IntToStr(codec_tag),2);   {Cardinal}
 ShowPByteArray(Tree,Node,'extradata: ',extradata_size,1000,extradata);   {PByte}
 ShowNode(Tree,Node,'extradata_size: '+IntToStr(extradata_size),2); {integer}
 ShowNode(Tree,Node,'format_: '+IntToStr(format_),2);   {integer}
 ShowNode(Tree,Node,'bit_rate: '+IntToStr(bit_rate),2);   {int64}
 ShowNode(Tree,Node,'bits_per_coded_sample: '+IntToStr(bits_per_coded_sample),2);   {integer}
 ShowNode(Tree,Node,'bits_per_raw_sample: '+IntToStr(bits_per_raw_sample),2);   {integer}
 ShowNode(Tree,Node,'profile: '+IntToStr(profile),2);   {integer}
 ShowNode(Tree,Node,'level: '+IntToStr(level),2);   {integer}
 ShowNode(Tree,Node,'width: '+IntToStr(width),2);   {integer}
 ShowNode(Tree,Node,'height: '+IntToStr(height),2);   {integer}
 ShowNode(Tree,Node,'sample_aspect_ratio: '+GetRationalStr(sample_aspect_ratio),2);   {TAVRational}
 ShowNode(Tree,Node,'field_order: '+GetFieldOrderStr(field_order),2);   { TAVFieldOrder}
 ShowNode(Tree,Node,'color_range: '+GetColorRangeStr(color_range),2);   { TAVColorRange}
 ShowNode(Tree,Node,'color_primaries: '+GetColorPrimaryStr(color_primaries),2);   { TAVColorPrimaries}
 ShowNode(Tree,Node,'color_trc: '+GetColorTransferCharStr(color_trc),2);   { TAVColorTransferCharacteristic}
 ShowNode(Tree,Node,'color_space: '+GetColorSpaceStr(color_space),2);   { TAVColorSpace}
 ShowNode(Tree,Node,'chroma_location: '+GetChromaLocationStr(chroma_location),2);   { TAVChromaLocation}
 ShowNode(Tree,Node,'video_delay: '+IntToStr(video_delay),2);   {integer}
 ShowNode(Tree,Node,'channel_layout: '+IntToStr(channel_layout),2);   {uint64}
 ShowNode(Tree,Node,'channels: '+IntToStr(channels),2);   {integer}
 ShowNode(Tree,Node,'sample_rate: '+IntToStr(sample_rate),2);   {integer}
 ShowNode(Tree,Node,'block_align: '+IntToStr(block_align),2);   {integer}
 ShowNode(Tree,Node,'frame_size: '+IntToStr(frame_size),2);   {integer}
 ShowNode(Tree,Node,'initial_padding: '+IntToStr(initial_padding),2);   {integer}
 ShowNode(Tree,Node,'trailing_padding: '+IntToStr(trailing_padding),2);   {integer}
 ShowNode(Tree,Node,'seek_preroll: '+IntToStr(seek_preroll),2);   {integer}
 end;
end;


procedure ShowCodecDescriptor(Tree: TTreeView; Node: TTreeNode; Desc: PAVCodecDescriptor);
var SubNode: TTreeNode;
var S: string;
begin
if IsInvalidPointer(Desc,S) then ShowNode(Tree,Node,S,2)
else with Desc^ do
 begin
 ShowNode(Tree,Node,'id: '+GetCodecIDStr(id),2); {TAVCodecID}
 ShowNode(Tree,Node,'ttype: '+GetMediaTypeStr(ttype),2); {TAVMediaType}
 ShowNode(Tree,Node,'name: '+StrPtrToStr(name),2); {PAnsiChar}
 ShowNode(Tree,Node,'long_name: '+StrPtrToStr(long_name),2); {PAnsiChar}
 ShowNode(Tree,Node,'props: '+IntToStr(props),2); {Integer}
 ShowNode(Tree,Node,'mime_types: '+StrPtrToStr(mime_types^),2); {PPAnsiChar}
 ShowNode(Tree,Node,'profiles: '+GetProfileStr(profiles^),2); {PAVProfile}
 end;
end;

procedure ShowBufferRef(Tree: TTreeView; Node: TTreeNode; Ref: PAVBufferRef);
var SubNode: TTreeNode;
begin
if Ref=nil then ShowNode(Tree,Node,'Entry=nil' ,2)
else with Ref^ do
 begin
 ShowNode(Tree,Node,'buffer: '+PointerStr(buffer),2); {PAVBuffer}
 ShowPByteArray(Tree,Node,'data:',Size,1000,data); {PByte}
 ShowNode(Tree,Node,'buffer: '+IntToStr(Size),2); {PAVBuffer}
 end;
end;


procedure ShowFrameSideData(Tree: TTreeView; Node: TTreeNode; Data: TAVFrameSideData);
var SubNode: TTreeNode;
begin
with Data do
 begin
 ShowNode(Tree,Node,'type_: '+GetFrameSideTypeStr(type_), 2);    {TAVFrameSideDataType}
 ShowPByteArray(Tree,Node,'data:',Size,1000,data);     {PByte}
 ShowNode(Tree,Node,'size: '+IntToStr(size), 2);     {Integer}
 ShowNode(Tree,Node,'metadata: '+PointerStr(metadata), 2); {PAVDictionary}
 SubNode:=ShowNode(Tree,Node,'buf',2);      {PAVBufferRef}
  ShowBufferRef(Tree,SubNode,buf);
 end;
end;

procedure ShowRcOverride(Tree: TTreeView; Node: TTreeNode; Entry: PRcOverride);
var SubNode: TTreeNode;
begin
if Entry=nil then ShowNode(Tree,Node,'Entry=nil' ,2)
else with Entry^ do
 begin
 ShowNode(Tree,Node,'start_frame: '+IntToStr(start_frame),2); {Integer}
 ShowNode(Tree,Node,'end_frame: '+IntToStr(end_frame),2); {Integer}
 ShowNode(Tree,Node,'qscale: '+IntToStr(qscale),2); {Integer}
 ShowNode(Tree,Node,'quality_factor: '+GetRealStr(quality_factor),2); {Single}
 end;
end;




type TBuffRefArray = array[0..AV_NUM_DATA_POINTERS - 1] of PAVBufferRef;

procedure ShowBufferRefArray(Tree: TTreeView; Node: TTreeNode; Title: string; BRA: TBuffRefArray);
var SubNode: TTreeNode;
var I: integer;
begin
SubNode:=ShowNode(Tree,Node,'Title:',2);
for I:=0 to High(BRA) do
 ShowBufferRef(Tree,SubNode,BRA[I]);
end;



procedure ShowIndexEntry(Tree: TTreeView; Node: TTreeNode; Entry: PAVIndexEntry);
var SubNode: TTreeNode;
begin
if Entry=nil then ShowNode(Tree,Node,'Entry=nil' ,2)
else with Entry^ do
 begin
 ShowNode(Tree,Node,'pos: '+IntToStr(pos),2); {int64}
 ShowNode(Tree,Node,'timestamp: '+IntToStr(timestamp),2); {int64}
 ShowNode(Tree,Node,'flags_size: '+IntToStr(flags_size),2); {integer}
 ShowNode(Tree,Node,'min_distance: '+IntToStr(min_distance),2); {integer}
 end;
end;


procedure ShowOutputFormat(Tree: TTreeView; Node: TTreeNode; Format: PAVOutputFormat);
var SubNode: TTreeNode;
begin
if Format=nil then ShowNode(Tree,Node,'Format=nil' ,2)
else with Format^ do
 begin
 ShowNode(Tree,Node,'name: '+name ,2);	{PAnsiChar}
 ShowNode(Tree,Node,'long_name: '+long_name ,2);	{PAnsiChar}
 ShowNode(Tree,Node,'mime_type: '+mime_type ,2);	{PAnsiChar}
 ShowNode(Tree,Node,'extensions: '+extensions ,2);	{PAnsiChar}
 ShowNode(Tree,Node,'audio_codec: '+GetCodecIDStr(audio_codec) ,2);	{TAVCodecID}
 ShowNode(Tree,Node,'video_codec: '+GetCodecIDStr(video_codec) ,2);	{TAVCodecID}
 ShowNode(Tree,Node,'subtitle_codec: '+ GetCodecIDStr(subtitle_codec) ,2);	{TAVCodecID}
 ShowNode(Tree,Node,'flags: '+ IntToStr(flags) ,2);	{integer}
 ShowNode(Tree,Node,'codec_tag: '+ PointerStr(codec_tag) ,2);	{Pointer}
 ShowNode(Tree,Node,'priv_class: '+ PointerStr(priv_class) ,2);	{Pointer}
 ShowNode(Tree,Node,'next: '+ PointerStr(next) ,2);	{PAVOutputFormat}
 ShowNode(Tree,Node,'priv_data_size: '+ IntToStr(priv_data_size) ,2);	{integer}
 ShowNode(Tree,Node,'write_header: '+ ProcAddressStr(@write_header) ,2);	{function}
 ShowNode(Tree,Node,'write_packet: '+ ProcAddressStr(@write_packet) ,2);	{function}
 ShowNode(Tree,Node,'write_trailer: '+ ProcAddressStr(@write_trailer) ,2);	{function}
 ShowNode(Tree,Node,'interleave_packet: '+ ProcAddressStr(@interleave_packet),2);	{function}
 ShowNode(Tree,Node,'query_codec: '+ ProcAddressStr(@query_codec),2);	{function}
 ShowNode(Tree,Node,'get_output_timestamp: '+ ProcAddressStr(@get_output_timestamp),2);
 ShowNode(Tree,Node,'control_message: '+ ProcAddressStr(@control_message),2);	{function}
 ShowNode(Tree,Node,'write_uncoded_frame: '+ ProcAddressStr(@write_uncoded_frame),2);
 ShowNode(Tree,Node,'get_device_list: '+ ProcAddressStr(@get_device_list),2);	{function}
 ShowNode(Tree,Node,'create_device_capabilities: '+ ProcAddressStr(@create_device_capabilities),2);	{function}
 ShowNode(Tree,Node,'free_device_capabilities: '+ ProcAddressStr(@free_device_capabilities) ,2);	{function}
 ShowNode(Tree,Node,'data_codec: '+ GetCodecIDStr(data_codec),2); {TAVCodecID}
 ShowNode(Tree,Node,'init: '+ ProcAddressStr(@init),2);	{function};
 ShowNode(Tree,Node,'deinit: '+ ProcAddressStr(@deinit),2);	{procedure}
 ShowNode(Tree,Node,'check_bitstream: '+ ProcAddressStr(@check_bitstream),2);	{function}
 end;
end;


{===============================================================================}


procedure ShowInputFormat(Tree: TTreeView; Node: TTreeNode; Format: PAVInputFormat);
var SubNode: TTreeNode;
begin
if Format=nil then ShowNode(Tree,Node,'Format=nil' ,2)
else with Format^ do
 begin
 ShowNode(Tree,Node,'name: '+StrPtrToStr(name),2); {PAnsiChar};
 ShowNode(Tree,Node,'long_name: '+StrPtrToStr(long_name),2); {PAnsiChar};
 ShowNode(Tree,Node,'flags: '+IntToStr(flags),2); {Integer};
 ShowNode(Tree,Node,'extensions: '+StrPtrToStr(extensions),2); {PAnsiChar};
 ShowNode(Tree,Node,'codec_tag: '+PointerStr(codec_tag),2); {PPAVCodecTag};
 ShowNode(Tree,Node,'priv_class: '+PointerStr(priv_class),2); {Pointer};
 ShowNode(Tree,Node,'mime_type: '+PointerStr(mime_type),2); {PAnsiChar};
 ShowNode(Tree,Node,'raw_codec_id: '+IntToStr(raw_codec_id),2); {integer};
 ShowNode(Tree,Node,'priv_data_size: '+IntToStr(priv_data_size),2); {integer};
 ShowNode(Tree,Node,'read_probe: '+ProcAddressStr(@read_probe),2); {function};
 ShowNode(Tree,Node,'read_header: '+ProcAddressStr(@read_header),2); {function};
 ShowNode(Tree,Node,'read_packet: '+ProcAddressStr(@read_packet),2); {function};
 ShowNode(Tree,Node,'read_close: '+ProcAddressStr(@read_close),2); {function};
 ShowNode(Tree,Node,'read_seek: '+ProcAddressStr(@read_seek),2); {function};
 ShowNode(Tree,Node,'read_timestamp: '+ProcAddressStr(@read_timestamp),2); {function}
 ShowNode(Tree,Node,'read_play: '+ProcAddressStr(@read_play),2); {function};
 ShowNode(Tree,Node,'read_pause: '+ProcAddressStr(@read_pause),2); {function};
 ShowNode(Tree,Node,'read_seek2: '+ProcAddressStr(@read_seek2),2); {function};
 ShowNode(Tree,Node,'flags: '+IntToStr(flags),2); {integer}
 ShowNode(Tree,Node,'get_device_list: '+ProcAddressStr(@get_device_list),2); {function};
 ShowNode(Tree,Node,'create_device_capabilities: '+ProcAddressStr(@create_device_capabilities),2); {function};
 ShowNode(Tree,Node,'free_device_capabilities: '+ProcAddressStr(@free_device_capabilities),2); {function}
 end;
end;


procedure ShowAllInputFormats(Tree: TTreeView; Node: TTreeNode; Format: PAVInputFormat);
var SubNode: TTreeNode;
var I: integer;
begin
I:=1;
repeat
	begin
	SubNode:=ShowNode(Tree,Node,IntToStr(I)+'. Format: '+Format.name,2);
	ShowInputFormat(Tree,SubNode, Format);
	Format:=Format.next;
	Inc(I);
	end
until Format=nil;
end;


procedure ShowIOContext(Tree: TTreeView; Node: TTreeNode; Context: PAVIOContext);
var SubNode: TTreeNode;
begin
if Context=nil then ShowNode(Tree,Node,'Format=nil' ,2)
else with Context^ do
 begin
 ShowNode(Tree,Node,'av_class: '+PointerStr(av_class),2); {Pointer}
 ShowPByteArray(Tree,Node,'buffer:',buffer_size,1000,buffer); {PByte}
 ShowNode(Tree,Node,'buffer_size: '+IntToStr(buffer_size),2); {Integer}
 ShowNode(Tree,Node,'buf_ptr: '+PointerStr(buf_ptr),2); {PByte}
 ShowNode(Tree,Node,'buf_end: '+PointerStr(buf_end),2); {PByte}
 ShowNode(Tree,Node,'opaque: '+PointerStr(opaque),2); {Pointer}
 ShowNode(Tree,Node,'read_packet: '+ProcAddressStr(@read_packet),2); {function}
 ShowNode(Tree,Node,'write_packet: '+ProcAddressStr(@write_packet),2); {function}
 ShowNode(Tree,Node,'seek: '+ProcAddressStr(@seek),2); {function}
 ShowNode(Tree,Node,'pos: '+IntToStr(pos),2); {Int64}
 ShowNode(Tree,Node,'must_flush: '+IntToStr(must_flush),2); {Integer}
 ShowNode(Tree,Node,'eof_reached: '+IntToStr(eof_reached),2); {Integer}
 ShowNode(Tree,Node,'write_flag: '+IntToStr(write_flag),2); {Integer}
 ShowNode(Tree,Node,'max_packet_size: '+IntToStr(max_packet_size),2); {Integer}
 ShowNode(Tree,Node,'checksum: '+IntToStr(checksum),2); {Cardinal}
 ShowNode(Tree,Node,'checksum_ptr: '+PointerStr(checksum_ptr),2); {PByte}
 ShowNode(Tree,Node,'update_checksum: '+ProcAddressStr(@update_checksum),2); {function}
 ShowNode(Tree,Node,'error: '+IntToStr(error),2); {Integer}
 ShowNode(Tree,Node,'read_pause: '+ProcAddressStr(@read_pause),2); {function}
 ShowNode(Tree,Node,'read_seek: '+ProcAddressStr(@read_seek),2); {function}
 ShowNode(Tree,Node,'seekable: '+IntToStr(seekable),2); {integer}
 ShowNode(Tree,Node,'maxsize: '+IntToStr(maxsize),2); {int64}
 ShowNode(Tree,Node,'direct: '+IntToStr(direct),2); {integer}
 ShowNode(Tree,Node,'bytes_read: '+IntToStr(bytes_read),2); {int64}
 ShowNode(Tree,Node,'seek_count: '+IntToStr(seek_count),2); {integer}
 ShowNode(Tree,Node,'writeout_count: '+IntToStr(writeout_count),2); {integer}
 ShowNode(Tree,Node,'orig_buffer_size: '+IntToStr(orig_buffer_size),2); {integer}
 ShowNode(Tree,Node,'short_seek_threshold: '+IntToStr(short_seek_threshold),2); {integer}
 ShowNode(Tree,Node,'protocol_whitelist: '+StrPtrToStr(protocol_whitelist),2); {PAnsiChar}
 ShowNode(Tree,Node,'protocol_blacklist: '+StrPtrToStr(protocol_blacklist),2); {PAnsiChar}
 ShowNode(Tree,Node,'write_data_type: '+ProcAddressStr(@write_data_type),2); {function}
 ShowNode(Tree,Node,'ignore_boundary_point: '+IntToStr(ignore_boundary_point),2); {integer}
 ShowNode(Tree,Node,'current_type: '+GetDataMarkerTypeStr(current_type),2); {TAVIODataMarkerType}
 ShowNode(Tree,Node,'last_time: '+IntToStr(last_time),2); {Int64}
 ShowNode(Tree,Node,'short_seek_get: '+ProcAddressStr(@short_seek_get),2); {function}
 ShowNode(Tree,Node,'written: '+IntToStr(written),2); {int64}
 ShowNode(Tree,Node,'buf_ptr_max: '+PointerStr(buf_ptr_max),2); {PByte}
 ShowNode(Tree,Node,'min_packet_size: '+IntToStr(min_packet_size),2); {integer}
 end;
end;


procedure ShowPacketSideData(Tree: TTreeView; Node: TTreeNode; SideData: PAVPacketSideData);
var SubNode: TTreeNode;
begin
if SideData=nil then ShowNode(Tree,Node,'SideData=nil' ,2)
else with SideData^ do
 begin
 ShowPByteArray(Tree,Node,'data:',Size,1000,Data); {PByte}
 ShowNode(Tree,Node,'size: '+IntToStr(size), 2); {Integer}
 ShowNode(Tree,Node,'type_: '+GetPacketSideDataType(type_), 2);
 end;
end;

procedure ShowProbeData(Tree: TTreeView; Node: TTreeNode; ProbeData: TAVProbeData);
var SubNode: TTreeNode;
begin
with ProbeData do
 begin
 ShowNode(Tree,Node,'filename: '+StrPtrToStr(filename), 2); {PAnsiChar}
 ShowPByteArray(Tree,Node,'buf:',buf_size,1000,buf); {PByte}
 ShowNode(Tree,Node,'buf_size: '+IntToStr(buf_size), 2); {Integer}
 ShowNode(Tree,Node,'mime_type: '+StrPtrToStr(mime_type), 2); {PAnsiChar}
 end;
end;


procedure ShowCodec(Tree: TTreeView; Node: TTreeNode; Codec: PAVCodec);
var SubNode: TTreeNode;
begin
if Codec=nil then ShowNode(Tree,Node,'Codec=nil' ,2)
else with Codec^ do
 begin
 ShowNode(Tree,Node,'name: '+StrPtrToStr(name),2); {PAnsiChar}
 ShowNode(Tree,Node,'long_name: '+StrPtrToStr(long_name),2); {PAnsiChar}
 ShowNode(Tree,Node,'type_: '+GetMediaTypeStr(type_),2); {TAVMediaType}
 ShowNode(Tree,Node,'id: '+GetCodecIDStr(id),2); {TAVCodecID}
 ShowNode(Tree,Node,'capabilities: '+IntToStr(capabilities),2); {integer}
 ShowNode(Tree,Node,'supported_framerates: '+GetRationalStr(supported_framerates^),2); {PAVRational}
 ShowNode(Tree,Node,'pix_fmts: '+GetPixelFormatStr(pix_fmts^),2); {PAVPixelFormat}
 ShowNode(Tree,Node,'supported_samplerates: '+GetPIntegerStr(supported_samplerates),2); {PInteger}
 ShowNode(Tree,Node,'sample_fmts: '+GetSampleFormat(sample_fmts^),2); {PAVSampleFormat}
 ShowNode(Tree,Node,'channel_layouts: '+IntToStr(channel_layouts^),2); {PInt64}
 ShowNode(Tree,Node,'max_lowres: '+IntToStr(max_lowres),2); {Byte}
 ShowNode(Tree,Node,'priv_class: '+PointerStr(priv_class),2); {PAVClass - no predefined structure}
 ShowNode(Tree,Node,'profiles: '+GetProfileStr(profiles^),2); {PAVProfile}
 ShowNode(Tree,Node,'priv_data_size: '+IntToStr(priv_data_size),2); {integer}
 SubNode:=ShowNode(Tree,Node,'next: ',2); {PAVCodec}
   ShowCodec(Tree, SubNode, next);
 ShowNode(Tree,Node,'init_thread_copy: '+ProcAddressStr(@init_thread_copy),2); {function}
 ShowNode(Tree,Node,'update_thread_context: '+ProcAddressStr(@update_thread_context),2); {function}
 ShowNode(Tree,Node,'defaults: '+PointerStr(defaults),2); {PAVCodecDefault - no predefined structure}
 ShowNode(Tree,Node,'init_static_data: '+ProcAddressStr(@init_static_data),2); {procedure}
 ShowNode(Tree,Node,'+init: '+ProcAddressStr(@init),2); {function}
 ShowNode(Tree,Node,'encode_sub: '+ProcAddressStr(@encode_sub),2); {function}
 ShowNode(Tree,Node,'encode2: '+ProcAddressStr(@encode2),2); {function}
 ShowNode(Tree,Node,'decode: '+ProcAddressStr(@decode),2); {function}
 ShowNode(Tree,Node,'close: '+ProcAddressStr(@close),2); {function}
 ShowNode(Tree,Node,'send_frame: '+ProcAddressStr(@send_frame),2); {function}
 ShowNode(Tree,Node,'receive_packet: '+ProcAddressStr(@receive_packet),2); {function}
 ShowNode(Tree,Node,'receive_frame: '+ProcAddressStr(@receive_frame),2); {function}
 ShowNode(Tree,Node,'flush: '+ProcAddressStr(@flush),2); {procedure}
 ShowNode(Tree,Node,'caps_internal: '+IntToStr(caps_internal),2); {Integer}
 ShowNode(Tree,Node,'bsfs: '+StrPtrToStr(bsfs),2); {PAnsiChar}
 end;
end;

procedure ShowFrame(Tree: TTreeView; Node: TTreeNode; Frame: PAVFrame);
var SubNode: TTreeNode;
begin
if Frame=nil then ShowNode(Tree,Node,'Frame=nil' ,2)
else with Frame^ do
 begin
 ShowArrayOfPByte(Tree,Node,'data:',AV_NUM_DATA_POINTERS,128,data); {array[0..AV_NUM_DATA_POINTERS-1] of PByte}
 ShowIntegerArray(Tree,Node,'linesize:',AV_NUM_DATA_POINTERS,28,linesize); {array[0..AV_NUM_DATA_POINTERS-1] of Integer}
 ShowPByteArray(Tree,Node,'extended_data:',1,1,extended_data^); {PPByte}
 ShowNode(Tree,Node,'width, height: '+IntToStr(width)+' '+IntToStr(height), 2); {Integer}
 ShowNode(Tree,Node,'nb_samples: '+IntToStr(nb_samples), 2); {Integer}
 ShowNode(Tree,Node,'format: '+IntToStr(format), 2); {Integer}
 ShowNode(Tree,Node,'key_frame: '+IntToStr(key_frame), 2); {Integer}
 ShowNode(Tree,Node,'pict_type: '+GetPictureTypeStr(pict_type), 2); {TAVPictureType}
 ShowNode(Tree,Node,'sample_aspect_ratio: '+GetRationalStr(sample_aspect_ratio), 2); {TAVRational}
 ShowNode(Tree,Node,'pts: '+IntToStr(pts), 2); {Int64}
 ShowNode(Tree,Node,'pkt_pts: '+IntToStr(pkt_pts), 2); {Int64}
 ShowNode(Tree,Node,'pkt_dts: '+IntToStr(pkt_dts), 2); {Int64}
 ShowNode(Tree,Node,'coded_picture_number: '+IntToStr(coded_picture_number), 2); {Integer}
 ShowNode(Tree,Node,'display_picture_number: '+IntToStr(display_picture_number), 2); {Integer}
 ShowNode(Tree,Node,'quality: '+IntToStr(quality), 2); {Integer}
 ShowNode(Tree,Node,'opaque: '+PointerStr(opaque), 2); {Pointer}
 ShowInt64Array(Tree,Node,'error: ',AV_NUM_DATA_POINTERS,128,error); {array[0..AV_NUM_DATA_POINTERS-1] of UInt64}
 ShowNode(Tree,Node,'repeat_pict: '+IntToStr(repeat_pict), 2); {Integer}
 ShowNode(Tree,Node,'interlaced_frame: '+IntToStr(interlaced_frame), 2); {Integer}
 ShowNode(Tree,Node,'top_field_first: '+IntToStr(top_field_first), 2); {Integer}
 ShowNode(Tree,Node,'palette_has_changed: '+IntToStr(palette_has_changed), 2); {Integer}
 ShowNode(Tree,Node,'reordered_opaque: '+IntToStr(reordered_opaque), 2); {Int64}
 ShowNode(Tree,Node,'sample_rate: '+IntToStr(sample_rate), 2); {Integer}
 ShowNode(Tree,Node,'channel_layout: '+IntToStr(channel_layout), 2); {UInt64}
 ShowBufferRefArray(Tree,Node,'buf:', TBuffRefArray(buf)); {array[0..AV_NUM_DATA_POINTERS - 1] of PAVBufferRef}
 SubNode:=ShowNode(Tree,Node,'extended_buf: ', 2); {PPAVBufferRef}
  ShowBufferRef(Tree,SubNode,extended_buf^);
 ShowNode(Tree,Node,'nb_extended_buf: '+IntToStr(nb_extended_buf), 2); {Integer}
 SubNode:=ShowNode(Tree,Node,'side_data: ', 2); {PPAVFrameSideData}
  ShowFrameSideData(Tree,Node,side_data^^);
 ShowNode(Tree,Node,'nb_side_data: '+IntToStr(nb_side_data), 2); {Integer}
 ShowNode(Tree,Node,'flags: '+IntToStr(flags), 2); {Integer}
 ShowNode(Tree,Node,'color_range: '+GetColorRangeStr(color_range), 2); {TAVColorRange}
 ShowNode(Tree,Node,'color_primaries: '+GetColorPrimaryStr(color_primaries), 2); {TAVColorPrimaries}
 ShowNode(Tree,Node,'color_trc: '+GetColorTransferCharStr(color_trc), 2); {TAVColorTransferCharacteristic}
 ShowNode(Tree,Node,'colorspace: '+GetColorSpaceStr(colorspace), 2); {TAVColorSpace}
 ShowNode(Tree,Node,'chroma_location: '+GetChromaLocationStr(chroma_location), 2); {TAVChromaLocation}
 ShowNode(Tree,Node,'best_effort_timestamp: '+IntToStr(best_effort_timestamp), 2); {Int64}
 ShowNode(Tree,Node,'pkt_pos: '+IntToStr(pkt_pos), 2); {Int64}
 ShowNode(Tree,Node,'pkt_duration: '+IntToStr(pkt_duration), 2); {Int64}
 ShowNode(Tree,Node,'metadata: '+PointerStr(metadata), 2); {PAVDictionary}
 ShowNode(Tree,Node,'decode_error_flags: '+IntToStr(decode_error_flags), 2); {Integer}
 ShowNode(Tree,Node,'channels: '+IntToStr(channels), 2); {Integer}
 ShowNode(Tree,Node,'pkt_size: '+IntToStr(pkt_size), 2); {Integer}
 ShowNode(Tree,Node,'qscale_table: '+PointerStr(qscale_table), 2); {PByte}
 ShowNode(Tree,Node,'qstride: '+IntToStr(qstride), 2); {Integer}
 ShowNode(Tree,Node,'qscale_type: '+IntToStr(qscale_type), 2); {Integer}
 SubNode:=ShowNode(Tree,Node,'qp_table_buf: ',2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,qp_table_buf);
 SubNode:=ShowNode(Tree,Node,'hw_frames_ctx: ', 2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,hw_frames_ctx);
 SubNode:=ShowNode(Tree,Node,'opaque_ref: ', 2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,opaque_ref);
 ShowNode(Tree,Node,'crop_top: '+IntToStr(crop_top), 2); {Cardinal}
 ShowNode(Tree,Node,'crop_bottom: '+IntToStr(crop_bottom), 2); {Cardinal}
 ShowNode(Tree,Node,'crop_left: '+IntToStr(crop_left), 2); {Cardinal}
 ShowNode(Tree,Node,'crop_right: '+IntToStr(crop_right), 2); {Cardinal}
 end;
end;


procedure ShowCodecContex(Tree: TTreeView; Node: TTreeNode; Context: PAVCodecContext);
var SubNode: TTreeNode;
begin
if Context=nil then ShowNode(Tree,Node,'Context=nil' ,2)
else with Context^ do
 begin
 ShowNode(Tree,Node,'av_class: '+ PointerStr(av_class), 2); {PAVClass - No predefined structure}
 ShowNode(Tree,Node,'log_level_offset: '+IntToStr(log_level_offset), 2); {Integer}
 ShowNode(Tree,Node,'codec_type: '+GetMediaTypeStr(codec_type), 2); {TAVMediaType}
 SubNode:=ShowNode(Tree,Node,'codec:', 2); {PAVCodec}
   ShowCodec(Tree,SubNode,codec);
 ShowNode(Tree,Node,'codec_name: '+codec_name, 2); {array[0..31] of AnsiChar}
 ShowNode(Tree,Node,'codec_id: '+GetCodecIDStr(codec_id), 2); {TAVCodecID}
 ShowNode(Tree,Node,'codec_tag: '+IntToStr(codec_tag), 2); {Cardinal}
 ShowNode(Tree,Node,'stream_codec_tag: '+IntToStr(stream_codec_tag), 2); {Cardinal}
 ShowNode(Tree,Node,'priv_data: '+PointerStr(priv_data), 2); {Pointer}
 ShowNode(Tree,Node,'internal: '+PointerStr(internal), 2); {PAVCodecInternal}
 ShowNode(Tree,Node,'opaque: '+PointerStr(opaque), 2); {Pointer}
 ShowNode(Tree,Node,'bit_rate: '+IntToStr(bit_rate), 2); {Int64}
 ShowNode(Tree,Node,'bit_rate_tolerance: '+IntToStr(bit_rate_tolerance), 2); {Integer}
 ShowNode(Tree,Node,'global_quality: '+IntToStr(global_quality), 2); {Integer}
 ShowNode(Tree,Node,'compression_level: '+IntToStr(compression_level), 2); {Integer}
 ShowNode(Tree,Node,'flags: '+IntToStr(flags), 2); {Integer}
 ShowNode(Tree,Node,'flags2: '+IntToStr(flags2), 2); {Integer}
 ShowPByteArray(Tree,Node,'extradata: ',extradata_size,1000,extradata); {PByte}
 ShowNode(Tree,Node,'extradata_size: '+IntToStr(extradata_size), 2); {Integer}
 ShowNode(Tree,Node,'time_base: '+GetRationalStr(time_base), 2); {TAVRational}
 ShowNode(Tree,Node,'ticks_per_frame: '+IntToStr(ticks_per_frame), 2); {Integer}
 ShowNode(Tree,Node,'delay: '+IntToStr(delay), 2); {Integer}
 ShowNode(Tree,Node,'width, height: '+IntToStr(width)+' '+IntToStr(height), 2); {Integer}
 ShowNode(Tree,Node,'coded_width, coded_height: '+IntToStr(coded_width)+' '+IntToStr(coded_height), 2); {Integer}
 ShowNode(Tree,Node,'gop_size: '+IntToStr(gop_size), 2); {Integer}
 ShowNode(Tree,Node,'pix_fmt: '+GetPixelFormatStr(pix_fmt), 2); {TAVPixelFormat}
 ShowNode(Tree,Node,'me_method: '+IntToStr(me_method), 2); {Integer}
 ShowNode(Tree,Node,'draw_horiz_band: '+ProcAddressStr(@draw_horiz_band), 2); {procedure}
 ShowNode(Tree,Node,'get_format: '+ProcAddressStr(@get_format), 2); {function}
 ShowNode(Tree,Node,'max_b_frames: '+IntToStr(max_b_frames), 2); {Integer}
 ShowNode(Tree,Node,'b_quant_factor: '+GetRealStr(b_quant_factor), 2); {Single}
 ShowNode(Tree,Node,'rc_strategy: '+IntToStr(rc_strategy), 2); {Integer}
 ShowNode(Tree,Node,'b_frame_strategy: '+IntToStr(b_frame_strategy), 2); {Integer}
 ShowNode(Tree,Node,'b_quant_offset: '+GetRealStr(b_quant_offset), 2); {Single}
 ShowNode(Tree,Node,'has_b_frames: '+IntToStr(has_b_frames), 2); {Integer}
 ShowNode(Tree,Node,'mpeg_quant: '+IntToStr(mpeg_quant), 2); {Integer}
 ShowNode(Tree,Node,'i_quant_factor: '+GetRealStr(i_quant_factor), 2); {Single}
 ShowNode(Tree,Node,'i_quant_offset: '+GetRealStr(i_quant_offset), 2); {Single}
 ShowNode(Tree,Node,'lumi_masking: '+GetRealStr(lumi_masking), 2); {Single}
 ShowNode(Tree,Node,'temporal_cplx_masking: '+GetRealStr(temporal_cplx_masking), 2); {Single}
 ShowNode(Tree,Node,'spatial_cplx_masking: '+GetRealStr(spatial_cplx_masking), 2); {Single}
 ShowNode(Tree,Node,'p_masking: '+GetRealStr(p_masking), 2); {Single}
 ShowNode(Tree,Node,'dark_masking: '+GetRealStr(dark_masking), 2); {Single}
 ShowNode(Tree,Node,'slice_count: '+IntToStr(slice_count), 2); {Integer}
 ShowNode(Tree,Node,'prediction_method: '+IntToStr(prediction_method), 2); {Integer}
 ShowNode(Tree,Node,'slice_offset: '+GetPIntegerStr(slice_offset), 2); {PInteger}
 ShowNode(Tree,Node,'sample_aspect_ratio: '+GetRationalStr(sample_aspect_ratio), 2); {TAVRational}
 ShowNode(Tree,Node,'me_cmp: '+IntToStr(me_cmp), 2); {Integer}
 ShowNode(Tree,Node,'me_sub_cmp: '+IntToStr(me_sub_cmp), 2); {Integer}
 ShowNode(Tree,Node,'mb_cmp: '+IntToStr(mb_cmp), 2); {Integer}
 ShowNode(Tree,Node,'ildct_cmp: '+IntToStr(ildct_cmp), 2); {Integer}
 ShowNode(Tree,Node,'dia_size: '+IntToStr(dia_size), 2); {Integer}
 ShowNode(Tree,Node,'last_predictor_count: '+IntToStr(last_predictor_count), 2); {Integer}
 ShowNode(Tree,Node,'pre_me: '+IntToStr(pre_me), 2); {Integer}
 ShowNode(Tree,Node,'me_pre_cmp: '+IntToStr(me_pre_cmp), 2); {Integer}
 ShowNode(Tree,Node,'pre_dia_size: '+IntToStr(pre_dia_size), 2); {Integer}
 ShowNode(Tree,Node,'me_subpel_quality: '+IntToStr(me_subpel_quality), 2); {Integer}
 ShowNode(Tree,Node,'dtg_active_format: '+IntToStr(dtg_active_format), 2); {Integer}
 ShowNode(Tree,Node,'me_range: '+IntToStr(me_range), 2); {Integer}
 ShowNode(Tree,Node,'intra_quant_bias: '+IntToStr(intra_quant_bias), 2); {Integer}
 ShowNode(Tree,Node,'inter_quant_bias: '+IntToStr(inter_quant_bias), 2); {Integer}
 ShowNode(Tree,Node,'slice_flags: '+IntToStr(slice_flags), 2); {Integer}
 ShowNode(Tree,Node,'xvmc_acceleration: '+IntToStr(xvmc_acceleration), 2); {Integer}
 ShowNode(Tree,Node,'mb_decision: '+IntToStr(mb_decision), 2); {Integer}
 ShowNode(Tree,Node,'intra_matrix: '+PointerStr(intra_matrix), 2); {PWord}
 ShowNode(Tree,Node,'inter_matrix: '+PointerStr(inter_matrix), 2); {PWord}
 ShowNode(Tree,Node,'scenechange_threshold: '+IntToStr(scenechange_threshold), 2); {Integer}
 ShowNode(Tree,Node,'noise_reduction: '+IntToStr(noise_reduction), 2); {Integer}
 ShowNode(Tree,Node,'me_threshold: '+IntToStr(me_threshold), 2); {Integer}
 ShowNode(Tree,Node,'mb_threshold: '+IntToStr(mb_threshold), 2); {Integer}
 ShowNode(Tree,Node,'intra_dc_precision: '+IntToStr(intra_dc_precision), 2); {Integer}
 ShowNode(Tree,Node,'skip_top: '+IntToStr(skip_top), 2); {Integer}
 ShowNode(Tree,Node,'skip_bottom: '+IntToStr(skip_bottom), 2); {Integer}
 ShowNode(Tree,Node,'border_masking: '+GetRealStr(border_masking), 2); {Single}
 ShowNode(Tree,Node,'mb_lmin: '+IntToStr(mb_lmin), 2); {Integer}
 ShowNode(Tree,Node,'mb_lmax: '+IntToStr(mb_lmax), 2); {Integer}
 ShowNode(Tree,Node,'me_penalty_compensation: '+IntToStr(me_penalty_compensation), 2); {Integer}
 ShowNode(Tree,Node,'bidir_refine: '+IntToStr(bidir_refine), 2); {Integer}
 ShowNode(Tree,Node,'brd_scale: '+IntToStr(brd_scale), 2); {Integer}
 ShowNode(Tree,Node,'keyint_min: '+IntToStr(keyint_min), 2); {Integer}
 ShowNode(Tree,Node,'refs: '+IntToStr(refs), 2); {Integer}
 ShowNode(Tree,Node,'chromaoffset: '+IntToStr(chromaoffset), 2); {Integer}
 ShowNode(Tree,Node,'scenechange_factor: '+IntToStr(scenechange_factor), 2); {Integer}
 ShowNode(Tree,Node,'mv0_threshold: '+IntToStr(mv0_threshold), 2); {Integer}
 ShowNode(Tree,Node,'b_sensitivity: '+IntToStr(b_sensitivity), 2); {Integer}
 ShowNode(Tree,Node,'color_primaries: '+GetColorPrimaryStr(color_primaries), 2); {TAVColorPrimaries}
 ShowNode(Tree,Node,'color_trc: '+GetColorTransferCharStr(color_trc), 2); {TAVColorTransferCharacteristic}
 ShowNode(Tree,Node,'colorspace: '+GetColorSpaceStr(colorspace), 2); {TAVColorSpace}
 ShowNode(Tree,Node,'color_range: '+GetColorRangeStr(color_range), 2); {TAVColorRange}
 ShowNode(Tree,Node,'chroma_sample_location: '+GetChromaLocationStr(chroma_sample_location), 2); {TAVChromaLocation}
 ShowNode(Tree,Node,'slices: '+IntToStr(slices), 2); {Integer}
 ShowNode(Tree,Node,'field_order: '+GetFieldOrderStr(field_order), 2); {TAVFieldOrder}
 ShowNode(Tree,Node,'sample_rate: '+IntToStr(sample_rate), 2); {Integer}
 ShowNode(Tree,Node,'channels: '+IntToStr(channels), 2); {Integer}
 ShowNode(Tree,Node,'sample_fmt: '+GetSampleFormat(sample_fmt), 2); {TAVSampleFormat}
 ShowNode(Tree,Node,'frame_size: '+IntToStr(frame_size), 2); {Integer}
 ShowNode(Tree,Node,'frame_number: '+IntToStr(frame_number), 2); {Integer}
 ShowNode(Tree,Node,'block_align: '+IntToStr(block_align), 2); {Integer}
 ShowNode(Tree,Node,'cutoff: '+IntToStr(cutoff), 2); {Integer}
 ShowNode(Tree,Node,'channel_layout: '+IntToStr(channel_layout), 2); {UInt64}
 ShowNode(Tree,Node,'request_channel_layout: '+IntToStr(request_channel_layout), 2); {UInt64}
 ShowNode(Tree,Node,'audio_service_type: '+GetAudioServiceTypeStr(audio_service_type), 2); {TAVAudioServiceType}
 ShowNode(Tree,Node,'request_sample_fmt: '+GetSampleFormat(request_sample_fmt), 2); {TAVSampleFormat}
 ShowNode(Tree,Node,'get_buffer2: '+ProcAddressStr(@get_buffer2), 2); {function}
 ShowNode(Tree,Node,'refcounted_frames: '+IntToStr(refcounted_frames), 2); {Integer}
 ShowNode(Tree,Node,'qcompress: '+GetRealStr(qcompress), 2); {Single}
 ShowNode(Tree,Node,'qblur: '+GetRealStr(qblur), 2); {Single}
 ShowNode(Tree,Node,'qmin: '+IntToStr(qmin), 2); {Integer}
 ShowNode(Tree,Node,'qmax: '+IntToStr(qmax), 2); {Integer}
 ShowNode(Tree,Node,'max_qdiff: '+IntToStr(max_qdiff), 2); {Integer}
 ShowNode(Tree,Node,'rc_qsquish: '+GetRealStr(rc_qsquish), 2); {Single}
 ShowNode(Tree,Node,'rc_qmod_amp: '+GetRealStr(rc_qmod_amp), 2); {Single}
 ShowNode(Tree,Node,'rc_qmod_freq: '+IntToStr(rc_qmod_freq), 2); {Integer}
 ShowNode(Tree,Node,'rc_buffer_size: '+IntToStr(rc_buffer_size), 2); {Integer}
 ShowNode(Tree,Node,'rc_override_count: '+IntToStr(rc_override_count), 2); {Integer}
 SubNode:=ShowNode(Tree,Node,'rc_override:', 2); {PRcOverride}
  ShowRcOverride(Tree,SubNode,rc_override);
 ShowNode(Tree,Node,'rc_eq: '+StrPtrToStr(rc_eq), 2); {PAnsiChar}
 ShowNode(Tree,Node,'rc_max_rate: '+IntToStr(rc_max_rate), 2); {Int64}
 ShowNode(Tree,Node,'rc_min_rate: '+IntToStr(rc_min_rate), 2); {Int64}
 ShowNode(Tree,Node,'rc_buffer_aggressivity: '+GetRealStr(rc_buffer_aggressivity), 2); {Single}
 ShowNode(Tree,Node,'rc_initial_cplx: '+GetRealStr(rc_initial_cplx), 2); {Single}
 ShowNode(Tree,Node,'rc_max_available_vbv_use: '+GetRealStr(rc_max_available_vbv_use), 2); {Single}
 ShowNode(Tree,Node,'rc_min_vbv_overflow_use: '+GetRealStr(rc_min_vbv_overflow_use), 2); {Single}
 ShowNode(Tree,Node,'rc_initial_buffer_occupancy: '+IntToStr(rc_initial_buffer_occupancy), 2); {Integer}
 ShowNode(Tree,Node,'coder_type: '+IntToStr(coder_type), 2); {Integer}
 ShowNode(Tree,Node,'context_model: '+IntToStr(context_model), 2); {Integer}
 ShowNode(Tree,Node,'lmin: '+IntToStr(lmin), 2); {Integer}
 ShowNode(Tree,Node,'lmax: '+IntToStr(lmax), 2); {Integer}
 ShowNode(Tree,Node,'frame_skip_threshold: '+IntToStr(frame_skip_threshold), 2); {Integer}
 ShowNode(Tree,Node,'frame_skip_factor: '+IntToStr(frame_skip_factor), 2); {Integer}
 ShowNode(Tree,Node,'frame_skip_exp: '+IntToStr(frame_skip_exp), 2); {Integer}
 ShowNode(Tree,Node,'frame_skip_cmp: '+IntToStr(frame_skip_cmp), 2); {Integer}
 ShowNode(Tree,Node,'trellis: '+IntToStr(trellis), 2); {Integer}
 ShowNode(Tree,Node,'min_prediction_order: '+IntToStr(min_prediction_order), 2); {Integer}
 ShowNode(Tree,Node,'max_prediction_order: '+IntToStr(max_prediction_order), 2); {Integer}
 ShowNode(Tree,Node,'timecode_frame_start: '+IntToStr(timecode_frame_start), 2); {Int64}
 ShowNode(Tree,Node,'rtp_callback: '+ProcAddressStr(@rtp_callback), 2); {procedure}
 ShowNode(Tree,Node,'rtp_payload_size: '+IntToStr(rtp_payload_size), 2); {Integer}
 ShowNode(Tree,Node,'mv_bits: '+IntToStr(mv_bits), 2); {Integer}
 ShowNode(Tree,Node,'header_bits: '+IntToStr(header_bits), 2); {Integer}
 ShowNode(Tree,Node,'i_tex_bits: '+IntToStr(i_tex_bits), 2); {Integer}
 ShowNode(Tree,Node,'p_tex_bits: '+IntToStr(p_tex_bits), 2); {Integer}
 ShowNode(Tree,Node,'i_count: '+IntToStr(i_count), 2); {Integer}
 ShowNode(Tree,Node,'p_count: '+IntToStr(p_count), 2); {Integer}
 ShowNode(Tree,Node,'skip_count: '+IntToStr(skip_count), 2); {Integer}
 ShowNode(Tree,Node,'misc_bits, 2); '+IntToStr(skip_count), 2); {Integer}
 ShowNode(Tree,Node,'misc_bits: '+IntToStr(misc_bits), 2); {Integer}
 ShowNode(Tree,Node,'frame_bits: '+IntToStr(frame_bits), 2); {Integer}
 ShowNode(Tree,Node,'stats_out: '+stats_out, 2); {PAnsiChar}
 ShowNode(Tree,Node,'stats_in: '+stats_in, 2); {PAnsiChar}
 ShowNode(Tree,Node,'workaround_bugs: '+IntToStr(workaround_bugs), 2); {Integer}
 ShowNode(Tree,Node,'strict_std_compliance: '+IntToStr(strict_std_compliance), 2); {Integer}
 ShowNode(Tree,Node,'error_concealment: '+IntToStr(error_concealment), 2); {Integer}
 ShowNode(Tree,Node,'debug: '+IntToStr(debug), 2); {Integer}
 ShowNode(Tree,Node,'debug_mv: '+IntToStr(debug_mv), 2); {Integer}
 ShowNode(Tree,Node,'err_recognition: '+IntToStr(err_recognition), 2); {Integer}
 ShowNode(Tree,Node,'reordered_opaque: '+IntToStr(reordered_opaque), 2); {Int64}
 ShowNode(Tree,Node,'hwaccel: '+PointerStr(hwaccel), 2); {PAVHWAccel - no predefined structure}
 ShowNode(Tree,Node,'hwaccel_context: '+PointerStr(hwaccel_context), 2); {Pointer}
 ShowInt64Array(Tree,SubNode,'error: ',AV_NUM_DATA_POINTERS,1000,error);
 ShowNode(Tree,Node,'dct_algo: '+IntToStr(dct_algo), 2); {Integer}
 ShowNode(Tree,Node,'idct_algo: '+IntToStr(idct_algo), 2); {Integer}
 ShowNode(Tree,Node,'bits_per_coded_sample: '+IntToStr(bits_per_coded_sample), 2); {Integer}
 ShowNode(Tree,Node,'bits_per_raw_sample: '+IntToStr(bits_per_raw_sample), 2); {Integer}
 ShowNode(Tree,Node,'lowres: '+IntToStr(lowres), 2); {Integer}
 SubNode:=ShowNode(Tree,Node,'coded_frame: ', 2); {PAVFrame}
   ShowFrame(Tree,SubNode,coded_frame);
 ShowNode(Tree,Node,'thread_count: '+IntToStr(thread_count), 2); {Integer}
 ShowNode(Tree,Node,'thread_type: '+IntToStr(thread_type), 2); {Integer}
 ShowNode(Tree,Node,'active_thread_type: '+IntToStr(active_thread_type), 2); {Integer}
 ShowNode(Tree,Node,'thread_safe_callbacks: '+IntToStr(thread_safe_callbacks), 2); {Integer}
 ShowNode(Tree,Node,'execute: '+ProcAddressStr(@execute), 2); {function}
 ShowNode(Tree,Node,'execute2: '+ProcAddressStr(@execute2), 2); {function}
 ShowNode(Tree,Node,'nsse_weight: '+IntToStr(nsse_weight), 2); {Integer}
 ShowNode(Tree,Node,'profile: '+IntToStr(profile), 2); {Integer}
 ShowNode(Tree,Node,'level: '+IntToStr(level), 2); {Integer}
 ShowNode(Tree,Node,'skip_loop_filter: '+GetDiscardStr(skip_loop_filter), 2); {TAVDiscard}
 ShowNode(Tree,Node,'skip_idct: '+GetDiscardStr(skip_idct), 2); {TAVDiscard}
 ShowNode(Tree,Node,'skip_frame: '+GetDiscardStr(skip_frame), 2); {TAVDiscard}
 ShowPByteArray(Tree,Node,'subtitle_header: ',subtitle_header_size,1000,subtitle_header); {PByte}
 ShowNode(Tree,Node,'subtitle_header_size: '+IntToStr(subtitle_header_size), 2); {Integer}
 ShowNode(Tree,Node,'error_rate: '+IntToStr(error_rate), 2); {Integer}
 ShowNode(Tree,Node,'vbv_delay: '+IntToStr(vbv_delay), 2); {UInt64}
 ShowNode(Tree,Node,'side_data_only_packets: '+IntToStr(side_data_only_packets), 2); {Integer}
 ShowNode(Tree,Node,'initial_padding: '+IntToStr(initial_padding), 2); {Integer}
 ShowNode(Tree,Node,'framerate: '+GetRationalStr(framerate), 2); {TAVRational}
 ShowNode(Tree,Node,'sw_pix_fmt: '+GetPixelFormatStr(sw_pix_fmt), 2); {TAVPixelFormat}
 ShowNode(Tree,Node,'pkt_timebase: '+GetRationalStr(pkt_timebase), 2); {TAVRational}
 SubNode:=ShowNode(Tree,Node,'codec_descriptor: ', 2); {PAVCodecDescriptor}
  ShowCodecDescriptor(Tree,SubNode,codec_descriptor);
 ShowNode(Tree,Node,'pts_correction_num_faulty_pts: '+IntToStr(pts_correction_num_faulty_pts), 2); {Int64}
 ShowNode(Tree,Node,'pts_correction_num_faulty_dts: '+IntToStr(pts_correction_num_faulty_dts), 2); {Int64}
 ShowNode(Tree,Node,'pts_correction_last_pts: '+IntToStr(pts_correction_last_pts), 2); {Int64}
 ShowNode(Tree,Node,'pts_correction_last_dts: '+IntToStr(pts_correction_last_dts), 2); {Int64}
 ShowNode(Tree,Node,'sub_charenc: '+sub_charenc, 2); {PAnsiChar}
 ShowNode(Tree,Node,'sub_charenc_mode: '+IntToStr(sub_charenc_mode), 2); {Integer}
 ShowNode(Tree,Node,'skip_alpha: '+IntToStr(skip_alpha), 2); {Integer}
 ShowNode(Tree,Node,'seek_preroll: '+IntToStr(seek_preroll), 2); {Integer}
 ShowNode(Tree,Node,'chroma_intra_matrix: '+GetPWordStr(chroma_intra_matrix), 2); {PWord}
 ShowNode(Tree,Node,'dump_separator: '+PointerStr(dump_separator), 2); {PByte}
 ShowNode(Tree,Node,'codec_whitelist: '+codec_whitelist, 2); {PAnsiChar}
 ShowNode(Tree,Node,'properties: '+IntToStr(properties), 2); {Cardinal}
 SubNode:=ShowNode(Tree,Node,'coded_side_data: ', 2); {PAVPacketSideData}
   ShowPacketSideData(Tree,SubNode,coded_side_data);
 ShowNode(Tree,Node,'nb_coded_side_data: '+IntToStr(nb_coded_side_data), 2); {Integer}
 SubNode:=ShowNode(Tree,Node,'hw_frames_ctx: ', 2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,hw_frames_ctx);
 ShowNode(Tree,Node,'sub_text_format: '+IntToStr(sub_text_format), 2); {Integer}
 ShowNode(Tree,Node,'trailing_padding: '+IntToStr(trailing_padding), 2); {Integer}
 ShowNode(Tree,Node,'max_pixels: '+IntToStr(max_pixels), 2); {Int64}
 SubNode:=ShowNode(Tree,Node,'hw_device_ctx: ', 2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,hw_device_ctx);
 ShowNode(Tree,Node,'hwaccel_flags: '+IntToStr(hwaccel_flags), 2); {Integer}
 ShowNode(Tree,Node,'apply_cropping: '+IntToStr(apply_cropping), 2); {Integer}
 end;
end;



procedure ShowPacket(Tree: TTreeView; Node: TTreeNode; Packet: TAVPacket);
var SubNode: TTreeNode;
begin
with Packet do
 begin
 SubNode:=ShowNode(Tree,Node,'buf: ', 2); {PAVBufferRef}
   ShowBufferRef(Tree,SubNode,buf);
 ShowNode(Tree,Node,'pts: '+IntToStr(pts), 2); {int64}
 ShowNode(Tree,Node,'dts: '+IntToStr(dts), 2); {int64}
 ShowPByteArray(Tree,Node,'data: ',size,1000,data); {PByte}
 ShowNode(Tree,Node,'size: '+IntToStr(size), 2); {integer}
 ShowNode(Tree,Node,'stream_index: '+IntToStr(stream_index), 2); {integer}
 ShowNode(Tree,Node,'flags: '+IntToStr(flags), 2); {integer}
 SubNode:=ShowNode(Tree,Node,'side_data: ', 2); {PAVPacketSideData}
   ShowPacketSideData(Tree,SubNode,side_data);
 ShowNode(Tree,Node,'side_data_elems: '+IntToStr(side_data_elems), 2); {integer}
 ShowNode(Tree,Node,'duration: '+IntToStr(duration), 2); {int64}
 ShowNode(Tree,Node,'pos: '+IntToStr(pos), 2); {int64}
 ShowNode(Tree,Node,'convergence_duration: '+IntToStr(convergence_duration), 2); {int64}
 end;
end;

procedure ShowStreamData(Tree: TTreeView; Node: TTreeNode; AVStream: PPAVStream);
var SubNode: TTreeNode;
var S: string;
begin
if IsInvalidPointer(AVStream,S) then ShowNode(Tree,Node,'Stream='+S ,2)
else if IsInvalidPointer(AVStream^,S) then ShowNode(Tree,Node,'Stream='+S ,2)
else with AVStream^^ do
 begin
 ShowNode(Tree,Node,'index: '+IntToStr(index),2); {integer}
 ShowNode(Tree,Node,'id: '+IntToStr(id),2); {integer}
 SubNode:=ShowNode(Tree,Node,'codec: ',2); {PAVCodecContext}
  ShowCodecContex(Tree,SubNode,codec);
 ShowNode(Tree,Node,'priv_data: '+PointerStr(priv_data),2); {Pointer}
 ShowNode(Tree,Node,'pts: '+GetFractStr(pts),2); {TAVFrac}
 ShowNode(Tree,Node,'time_base: '+GetRationalStr(time_base),2); {TAVRational}
 ShowNode(Tree,Node,'start_time: '+IntToStr(start_time),2); {Int64}
 ShowNode(Tree,Node,'duration: '+IntToStr(duration),2); {Int64}
 ShowNode(Tree,Node,'nb_frames: '+IntToStr(nb_frames),2); {Int64}
 ShowNode(Tree,Node,'disposition: '+IntToStr(disposition),2); {Integer}
 ShowNode(Tree,Node,'discard: '+GetDiscardStr(discard),2); {TAVDiscard}
 ShowNode(Tree,Node,'sample_aspect_ratio: '+GetRationalStr(sample_aspect_ratio),2); {TAVRational}
 ShowNode(Tree,Node,'metadata: '+PointerStr(metadata),2); {PAVDictionary - no predefined structure}
 ShowNode(Tree,Node,'avg_frame_rate: '+GetRationalStr(avg_frame_rate),2); {TAVRational}
 SubNode:=ShowNode(Tree,Node,'attached_pic: ',2); {TAVPacket}
  ShowPacket(Tree,SubNode,attached_pic);
 SubNode:=ShowNode(Tree,Node,'side_data: ',2); {PAVPacketSideData}
  ShowPacketSideData(Tree,SubNode,side_data);
 ShowNode(Tree,Node,'nb_side_data: '+IntToStr(nb_side_data),2); {Integer}
 ShowNode(Tree,Node,'event_flags: '+IntToStr(event_flags),2); {integer}
 ShowNode(Tree,Node,'info: '+PointerStr(info),2); {Pointer}
 ShowNode(Tree,Node,'pts_wrap_bits: '+IntToStr(pts_wrap_bits),2); {integer}
 ShowNode(Tree,Node,'first_dts: '+IntToStr(first_dts),2); {Int64}
 ShowNode(Tree,Node,'cur_dts: '+IntToStr(cur_dts),2); {Int64}
 ShowNode(Tree,Node,'last_IP_pts: '+IntToStr(last_IP_pts),2); {Int64}
 ShowNode(Tree,Node,'last_IP_duration: '+IntToStr(last_IP_duration),2); {Integer}
 ShowNode(Tree,Node,'probe_packets: '+IntToStr(probe_packets),2); {integer}
 ShowNode(Tree,Node,'codec_info_nb_frames: '+IntToStr(codec_info_nb_frames),2); {Integer}
 ShowNode(Tree,Node,'need_parsing: '+GetParseTypeStr(need_parsing),2); {TAVStreamParseType}
 ShowNode(Tree,Node,'parser: '+PointerStr(parser),2); {PAVCodecParserContext - No Predefined structure}
 ShowNode(Tree,Node,'last_in_packet_buffer: '+PointerStr(last_in_packet_buffer),2); {Pointer}
 SubNode:=ShowNode(Tree,Node,'probe_data: ',2); {TAVProbeData}
  ShowProbeData(Tree,SubNode,probe_data);
 ShowInt64Array(Tree, Node, 'pts_buffer: ', MAX_REORDER_DELAY,1000,pts_buffer);  {array[0..MAX_REORDER_DELAY] of int64}
 SubNode:=ShowNode(Tree,Node,'index_entries: ',2); {PAVIndexEntry}
  ShowIndexEntry(Tree,SubNode,index_entries);
 ShowNode(Tree,Node,'nb_index_entries: '+IntToStr(nb_index_entries),2); {integer}
 ShowNode(Tree,Node,'index_entries_allocated_size: '+IntToStr(index_entries_allocated_size),2); {Cardinal}
 ShowNode(Tree,Node,'r_frame_rate: '+GetRationalStr(r_frame_rate),2); {TAVRational}
 ShowNode(Tree,Node,'stream_identifier: '+IntToStr(stream_identifier),2); {Integer}
 ShowNode(Tree,Node,'interleaver_chunk_size: '+IntToStr(interleaver_chunk_size),2); {Int64}
 ShowNode(Tree,Node,'interleaver_chunk_duration: '+IntToStr(interleaver_chunk_duration),2); {Int64}
 ShowNode(Tree,Node,'request_probe: '+IntToStr(request_probe),2); {Integer}
 ShowNode(Tree,Node,'skip_to_keyframe: '+IntToStr(skip_to_keyframe),2); {Integer}
 ShowNode(Tree,Node,'skip_samples: '+IntToStr(skip_samples),2); {Integer}
 ShowNode(Tree,Node,'start_skip_samples: '+IntToStr(start_skip_samples),2); {Int64}
 ShowNode(Tree,Node,'first_discard_sample: '+IntToStr(first_discard_sample),2); {Int64}
 ShowNode(Tree,Node,'last_discard_sample: '+IntToStr(last_discard_sample),2); {Int64}
 ShowNode(Tree,Node,'nb_decoded_frames: '+IntToStr(nb_decoded_frames),2); {Integer}
 ShowNode(Tree,Node,'mux_ts_offset: '+IntToStr(mux_ts_offset),2); {Int64}
 ShowNode(Tree,Node,'pts_wrap_reference: '+IntToStr(pts_wrap_reference),2); {Int64}
 ShowNode(Tree,Node,'pts_wrap_behavior: '+IntToStr(pts_wrap_behavior),2); {Integer}
 ShowNode(Tree,Node,'update_initial_durations_done: '+IntToStr(update_initial_durations_done),2); {Integer}
 ShowInt64Array(Tree,Node,'pts_reorder_error', MAX_REORDER_DELAY,1000, pts_reorder_error);  {array[0..MAX_REORDER_DELAY] of Int64}
 ShowByteArray(Tree,Node,'pts_reorder_error_count', MAX_REORDER_DELAY,1000, pts_reorder_error_count); {array[0..MAX_REORDER_DELAY] of Byte}
 ShowNode(Tree,Node,'last_dts_for_order_check: '+IntToStr(last_dts_for_order_check),2); {Int64}
 ShowNode(Tree,Node,'dts_ordered: '+IntToStr(dts_ordered),2); {Byte}
 ShowNode(Tree,Node,'dts_misordered: '+IntToStr(dts_misordered),2); {Byte}
 ShowNode(Tree,Node,'inject_global_side_data: '+IntToStr(inject_global_side_data),2); {Integer}
 ShowNode(Tree,Node,'recommended_encoder_configuration: '+StrPtrToStr(recommended_encoder_configuration),2); {PAnsiChar}
 ShowNode(Tree,Node,'display_aspect_ratio: '+GetRationalStr(display_aspect_ratio),2); {TAVRational}
 ShowNode(Tree,Node,'priv_pts: '+PointerStr(priv_pts),2); {Pointer}
 SubNode:=ShowNode(Tree,Node,'internal: ',2); {PAVStreamInternal}
   ShowStreamInternal(Tree,SubNode,internal^);
 SubNode:=ShowNode(Tree,Node,'codecpar: ',2); {PAVCodecParameters}
   ShowCodecParameters(Tree,SubNode,codecpar^);
 end;
end;


procedure ShowAllStreams(Tree: TTreeView; Node: TTreeNode; Count: integer; Streams: PPAVStream);
var SubNode: TTreeNode;
var I: integer;
begin
for I:=1 to Count do
	begin
	SubNode:=ShowNode(Tree,Node,IntToStr(I)+'. Stream: ',2);
	ShowStreamData(Tree,SubNode,Streams);
	Inc(Streams);
	end;
end;


procedure ShowInterruptCB(Tree: TTreeView; Node: TTreeNode; CallBack: TAVIOInterruptCB);
var SubNode: TTreeNode;
begin
ShowNode(Tree,Node,'callback: '+ProcAddressStr(@CallBack.callback),2);
ShowNode(Tree,Node,'Pointer: '+PointerStr(CallBack.opaque),2);
end;






{===============================================================================}

procedure ShowContextItems(Tree: TTreeView; Node: TTreeNode; Context: TAVFormatContext);
var SubNode: TTreeNode;
var I: integer;
begin
with Context do
 begin
 ShowNode(Tree,Node,'av_class: '+PointerStr(av_class),2); {Pointer}
 SubNode:=ShowNode(Tree,Node,'iformat:',0); {PAVInputFormat}
  ShowAllInputFormats(Tree,SubNode,iformat);
 SubNode:=ShowNode(Tree,Node,'oformat: ',0); {PAVOutputFormat}
  ShowOutputFormat(Tree,SubNode,oformat);
 ShowNode(Tree,Node,'priv_data: '+PointerStr(priv_data),2); {Pointer}
 SubNode:=ShowNode(Tree,Node,'pb: ',0); {PAVIOContext}
  ShowIOContext(Tree,SubNode,pb);
 ShowNode(Tree,Node,'ctx_flags: '+IntToStr(ctx_flags),2); {integer}
 ShowNode(Tree,Node,'nb_streams: '+IntToStr(nb_streams),2); {cardinal}
 SubNode:=ShowNode(Tree,Node,'streams: '+PointerStr(streams),0); {PPAVStream}
  ShowAllStreams(Tree,SubNode,nb_streams,Streams);
 ShowNode(Tree,Node,'filename: '+filename,2); {array[0..1024-1] of AnsiChar}
 ShowNode(Tree,Node,'start_time: '+IntToStr(start_time),2); {int64}
 ShowNode(Tree,Node,'duration: '+IntToStr(duration),2); {int64}
 ShowNode(Tree,Node,'bit_rate: '+IntToStr(bit_rate),2); {int64}
 ShowNode(Tree,Node,'packet_size: '+IntToStr(packet_size),2); {cardinal}
 ShowNode(Tree,Node,'max_delay: '+IntToStr(max_delay),2); {Integer}
 ShowNode(Tree,Node,'flags: '+IntToStr(flags),2); {integer}
 ShowNode(Tree,Node,'probesize: '+IntToStr(probesize),2); {int64}
 ShowNode(Tree,Node,'max_analyze_duration: '+IntToStr(max_analyze_duration),2); {Int64}
 ShowPByteArray(Tree,Node,'key: ',keylen,1000,key); {PByte}
 ShowNode(Tree,Node,'keylen: '+IntToStr(keylen),2); {Integer}
 ShowNode(Tree,Node,'nb_programs: '+IntToStr(nb_programs),2); {Cardinal}
 ShowNode(Tree,Node,'programs: '+PointerStr(programs),2); {PPAVProgram - no predefined structure}
 ShowNode(Tree,Node,'video_codec_id: '+GetCodecIDStr(video_codec_id),2); {TAVCodecID}
 ShowNode(Tree,Node,'audio_codec_id: '+GetCodecIDStr(audio_codec_id),2); {TAVCodecID}
 ShowNode(Tree,Node,'subtitle_codec_id: '+GetCodecIDStr(subtitle_codec_id),2); {TAVCodecID}
 ShowNode(Tree,Node,'max_index_size: '+IntToStr(max_index_size),2); {Cardinal}
 ShowNode(Tree,Node,'max_picture_buffer: '+IntToStr(max_picture_buffer),2); {Cardinal}
 ShowNode(Tree,Node,'nb_chapters: '+IntToStr(nb_chapters),2); {Cardinal}
 ShowNode(Tree,Node,'chapters: '+PointerStr(chapters),2); {PPAVChapter - no predefined structure}
 ShowNode(Tree,Node,'metadata: '+PointerStr(metadata),2); {PAVDictionary - no predefined structure}
 ShowNode(Tree,Node,'start_time_realtime: '+IntToStr(start_time_realtime),2); {Int64}
 ShowNode(Tree,Node,'fps_probe_size: '+IntToStr(fps_probe_size),2); {Integer}
 ShowNode(Tree,Node,'error_recognition: '+IntToStr(error_recognition),2); {Integer}
 SubNode:=ShowNode(Tree,Node,'interrupt_callback: ',0); {TAVIOInterruptCB}
  ShowInterruptCB(Tree,SubNode,interrupt_callback);
 ShowNode(Tree,Node,'debug: '+IntToStr(debug),2); {Integer}
 ShowNode(Tree,Node,'max_interleave_delta: '+IntToStr(max_interleave_delta),2); {Int64}
 ShowNode(Tree,Node,'strict_std_compliance: '+IntToStr(strict_std_compliance),2); {Integer}
 ShowNode(Tree,Node,'event_flags: '+IntToStr(event_flags),2); {Integer}
 ShowNode(Tree,Node,'max_ts_probe: '+IntToStr(max_ts_probe),2); {Integer}
 ShowNode(Tree,Node,'avoid_negative_ts: '+IntToStr(avoid_negative_ts),2); {integer}
 ShowNode(Tree,Node,'ts_id: '+IntToStr(ts_id),2); {integer}
 ShowNode(Tree,Node,'audio_preload: '+IntToStr(audio_preload),2); {integer}
 ShowNode(Tree,Node,'max_chunk_duration: '+IntToStr(max_chunk_duration),2); {Integer}
 ShowNode(Tree,Node,'max_chunk_size: '+IntToStr(max_chunk_size),2); {Integer}
 ShowNode(Tree,Node,'use_wallclock_as_timestamps: '+IntToStr(use_wallclock_as_timestamps),2); {Integer}
 ShowNode(Tree,Node,'avio_flags: '+IntToStr(avio_flags),2); {Integer}
 SubNode:=ShowNode(Tree,Node,'duration_estimation_method',0); {TAVDurationEstimationMethod}
  ShowNode(Tree,SubNode,GetDurEstMethodStr(duration_estimation_method),2);
 ShowNode(Tree,Node,'skip_initial_bytes: '+IntToStr(skip_initial_bytes),2); {Int64}
 ShowNode(Tree,Node,'correct_ts_overflow: '+IntToStr(correct_ts_overflow),2); {Cardinal}
 ShowNode(Tree,Node,'seek2any: '+IntToStr(seek2any),2); {Integer}
 ShowNode(Tree,Node,'flush_packets: '+IntToStr(flush_packets),2); {Integer}
 ShowNode(Tree,Node,'probe_score: '+IntToStr(probe_score),2); {Integer}
 ShowNode(Tree,Node,'format_probesize: '+IntToStr(format_probesize),2); {Integer}
 ShowNode(Tree,Node,'codec_whitelist: '+StrPtrToStr(codec_whitelist),2); {PAnsiChar}
 ShowNode(Tree,Node,'format_whitelist: '+StrPtrToStr(format_whitelist),2); {PAnsiChar}
 ShowNode(Tree,Node,'internal: '+PointerStr(internal),2); {PAVFormatInternal - no predefined structure}
 ShowNode(Tree,Node,'io_repositioned: '+IntToStr(io_repositioned),2); {integer}
 SubNode:=ShowNode(Tree,Node,'video_codec:',0); {PAVCodec}
  ShowCodec(Tree,SubNode,video_codec);
 SubNode:=ShowNode(Tree,Node,'audio_codec:',0); {PAVCodec}
  ShowCodec(Tree,SubNode,audio_codec);
 SubNode:=ShowNode(Tree,Node,'subtitle_codec:',0); {PAVCodec}
  ShowCodec(Tree,SubNode,subtitle_codec);
 SubNode:=ShowNode(Tree,Node,'data_codec:',0); {PAVCodec}
  ShowCodec(Tree,SubNode,data_codec);
 ShowNode(Tree,Node,'metadata_header_padding: '+IntToStr(metadata_header_padding),2); {integer}
 ShowNode(Tree,Node,'opaque: '+PointerStr(opaque),2); {Pointer}
 ShowNode(Tree,Node,'control_message_cb: '+ProcAddressStr(@control_message_cb),2); {Tav_format_control_message - function call}
 ShowNode(Tree,Node,'output_ts_offset: '+IntToStr(output_ts_offset),2); {int64}
 ShowNode(Tree,Node,'dump_separator: '+PointerStr(@dump_separator),2); {PByte}
 ShowNode(Tree,Node,'data_codec_id: '+GetCodecIDStr(data_codec_id),2); {TAVCodecID}
 ShowNode(Tree,Node,'open_cb: '+ProcAddressStr(@open_cb),2);	{procedure}
 ShowNode(Tree,Node,'protocol_whitelist: '+protocol_whitelist,2); {PAnsiChar}
 ShowNode(Tree,Node,'io_open: '+ProcAddressStr(@io_open),2); {procedure}
 ShowNode(Tree,Node,'io_close: '+ProcAddressStr(@io_close),2); {procedure}
 ShowNode(Tree,Node,'protocol_blacklist: '+protocol_blacklist,2); {PAnsiChar}
 ShowNode(Tree,Node,'max_streams: '+IntToStr(max_streams),2); {integer}
 end;
end;


procedure DisplayContext(Tree: TTreeView; Context: TAVFormatContext);
var Node: TTreeNode;
begin
Tree.Items.BeginUpdate;
try
Tree.Items.Clear;
Node:=ShowNode(Tree,Nil,'Context',0);
ShowContextItems(Tree, Node, Context);
finally Tree.Items.EndUpdate; end;
end;

end.
