unit FFStreamsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, Dialogs, Autobttn,IniFiles,VCLLibrary;

type
  TStreamsDlg = class(TForm)
    Panel1: TPanel;
    CancelBtn: TButton;
    Panel2: TPanel;
    Panel27: TPanel;
    Label17: TLabel;
    Panel29: TPanel;
    DeleteMarkBtn: TSpeedButton;
    MoveDownBtn: TAutoButton;
    MoveUpBtn: TAutoButton;
    StreamGrid: TDrawGrid;
    InsertStreamBtn: TSpeedButton;
    procedure StreamGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormCreate(Sender: TObject);
    procedure StreamGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure StreamGridSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: String);
    procedure StreamGridGetEditText(Sender: TObject; ACol, ARow: Integer;
      var Value: String);
    procedure StreamGridDblClick(Sender: TObject);
    procedure DeleteMarkBtnClick(Sender: TObject);
    procedure MoveUpBtnClick(Sender: TObject);
    procedure MoveDownBtnClick(Sender: TObject);
    procedure InsertStreamBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  StreamsDlg: TStreamsDlg;

procedure ShowStreamsDlg;

implementation

{$R *.dfm}

uses MPEGPlayer;




type TStreamInfo = record
 Name: string;
 URL: string;
 end;

type TStreamArray = array of TStreamInfo;

var Streams: TStreamArray;

procedure ClearStreams;
begin
SetLength(Streams,0);
end;

procedure AddStream(Name,URL: string);
begin
SetLength(Streams,Length(Streams)+1);
Streams[High(Streams)].Name:=Name;
Streams[High(Streams)].URL:=URL;
end;



function DeleteStream(Inx: integer): boolean;
var I: integer;
begin
Result:=False;
if Length(Streams)<1 then exit;
if (Inx<0) and (Inx>High(Streams)) then exit;
for I:=Inx to High(Streams)-1 do
	begin
	Streams[I]:=Streams[I+1];
	end;
SetLength(Streams, Length(Streams)-1);
Result:=True;
end;



function MoveStreamUp(Inx: integer): boolean;
var Tmp: TStreamInfo;
begin
Result:=False;
if Length(Streams)<1 then exit;
if (Inx<=0) or (Inx>High(Streams)) then exit;
Tmp:=Streams[Inx-1];
Streams[Inx-1]:=Streams[Inx];
Streams[Inx]:=Tmp;
Result:=True;
end;



function MoveStreamDown(Inx: integer): boolean;
var Tmp: TStreamInfo;
begin
Result:=False;
if Length(Streams)<1 then exit;
if (Inx<0) or (Inx>=High(Streams)) then exit;
Tmp:=Streams[Inx+1];
Streams[Inx+1]:=Streams[Inx];
Streams[Inx]:=Tmp;
Result:=True;
end;


procedure SaveStreamsData;
{Save BookMark data }
var IniFile: TIniFile;
var I: integer;
var T: TStreamInfo;
begin
IniFile:=TIniFile.Create(IniFilename);
IniFile.EraseSection('Streams');
IniFile.WriteInteger('Streams','Count',Length(Streams));
for I:=0 to High(Streams) do
	begin
	T:=Streams[I];
	IniFile.WriteString('Streams','Name'+IntToStr(I),T.Name);
	IniFile.WriteString('Streams','Path'+IntToStr(I),T.URL);
	end;
IniFile.Free;
end;


procedure LoadStreamData(Filename: string);
{Load Stream data }
var IniFile: TIniFile;
var I,Cnt: integer;
var T: TStreamInfo;
begin
ClearStreams;
IniFile:=TIniFile.Create(Filename);
try
Cnt:=IniFile.ReadInteger('Streams','Count',0);
for I:=0 to Cnt-1 do
	begin
	T.Name:=IniFile.ReadString('Streams','Name'+IntToStr(I),'Test');
	T.URL:=IniFile.ReadString('Streams','Path'+IntToStr(I),'');
	AddStream(T.Name,T.URL);
	end;
if Cnt<25 then StreamsDlg.StreamGrid.RowCount:=25
else StreamsDlg.StreamGrid.RowCount:=Cnt+1;
StreamsDlg.StreamGrid.Invalidate;
finally IniFile.Free; end;
end;



procedure TStreamsDlg.StreamGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var S: string;
var Inx: integer;
var SI: TStreamInfo;
begin
if ARow=0 then
	begin
	case ACol of                            
	 0: OwnerDrawItem(StreamGrid.Canvas,'#',Rect);
	 1: OwnerDrawItem(StreamGrid.Canvas,'Name',Rect);
	 2: OwnerDrawItem(StreamGrid.Canvas,'URL',Rect);
	 end
	end
else
	begin
	Inx:=ARow-1;
	if (Inx<0) or (Inx>High(Streams)) then exit;
	SI:=Streams[Inx];
	case ACol of
	 0: OwnerDrawInteger(StreamGrid.Canvas,ARow,Rect);
	 1: OwnerDrawItem(StreamGrid.Canvas,SI.Name,Rect);
	 2: OwnerDrawItem(StreamGrid.Canvas,SI.URL,Rect);
	 end;
	end;
end;


procedure TStreamsDlg.FormCreate(Sender: TObject);
begin
LoadStreamData(IniFilename);
end;


procedure ShowStreamsDlg;
begin
StreamsDlg.ShowModal;
end;


var SelectingCell: boolean =False;

procedure TStreamsDlg.StreamGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var Inx: integer;
begin
if SelectingCell then exit;
SelectingCell:=True;
try
Inx:=ARow - 1;
CanSelect:=Inx<=Length(Streams);
if not CanSelect then StreamGrid.Options:=StreamGrid.Options - [goRangeSelect]
else StreamGrid.Options:=StreamGrid.Options + [goRangeSelect];
if (ACol=1) and (Inx<=Length(Streams)) and (StreamGrid.Row=ARow) and (StreamGrid.Col=ACol) then
	begin
	StreamGrid.Options:=StreamGrid.Options + [goEditing];
	StreamGrid.Options:=StreamGrid.Options - [goRowSelect];
	StreamGrid.EditorMode:=True;
	end
else
	begin
	StreamGrid.Options:=StreamGrid.Options - [goEditing];
	StreamGrid.Options:=StreamGrid.Options + [goRowSelect];
	StreamGrid.EditorMode:=False;
	end;
finally SelectingCell:=False; end;
end;



procedure TStreamsDlg.StreamGridGetEditText(Sender: TObject; ACol,
  ARow: Integer; var Value: String);
var Inx: integer;
begin
Inx:=ARow-1;
if (Inx<0) or (Inx>High(Streams)) then exit;
Value:=Streams[Inx].Name;
end;


procedure TStreamsDlg.StreamGridSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: String);
var Inx: integer;
begin
Inx:=ARow-1;
if (Inx<0) or (Inx>High(Streams)) then exit;
Streams[Inx].Name:=Value;
SaveStreamsData;
end;


procedure TStreamsDlg.StreamGridDblClick(Sender: TObject);
var Inx: integer;
begin
Inx:=StreamGrid.Row-1;
if (Inx<0) or (Inx>High(Streams)) then exit;
OpenMedia(Streams[Inx].URL);
Close;
end;

procedure TStreamsDlg.DeleteMarkBtnClick(Sender: TObject);
var Inx: integer;
begin
Inx:=StreamGrid.Row-1;
if (Inx<0) or (Inx>High(Streams)) then exit;
if MessageDlg('About To Delete: '+Streams[Inx].Name, mtInformation, [mbOK,mbCancel], 0)<>mrOk then exit;
DeleteStream(Inx);
SaveStreamsData;
StreamGrid.Invalidate;
end;



procedure TStreamsDlg.MoveUpBtnClick(Sender: TObject);
var Inx: integer;
begin
if Length(Streams)<1 then exit;
Inx:=StreamGrid.Row-1;
if MoveStreamUp(Inx) then StreamGrid.Row:=StreamGrid.Row-1;
SaveStreamsData;
StreamGrid.Invalidate;
end;



procedure TStreamsDlg.MoveDownBtnClick(Sender: TObject);
var Inx: integer;
begin
if Length(Streams)<1 then exit;
Inx:=StreamGrid.Row-1;
if MoveStreamDown(Inx) then StreamGrid.Row:=StreamGrid.Row+1;
SaveStreamsData;
StreamGrid.Invalidate;
end;


procedure TStreamsDlg.InsertStreamBtnClick(Sender: TObject);
var URLStr: string;
begin
if not InputQuery('Streaming Media','Enter URL',URLStr) then exit;
AddStream('New',URLStr);
SaveStreamsData;
StreamGrid.Invalidate;
OpenMedia(URLStr);
Close;
end;


end.
