const
(**
 * @brief Undefined timestamp value
 *
 * Usually reported by demuxer that work on containers that do not provide
 * either pts or dts.
 *)

 AV_NOPTS_VALUE = $8000000000000000;

(**
 * Internal time base represented as integer
 *)

 AV_TIME_BASE = 1000000;


type
 TAVPictureType = (
    AV_PICTURE_TYPE_NONE = 0, ///< Undefined
    AV_PICTURE_TYPE_I,     ///< Intra
    AV_PICTURE_TYPE_P,     ///< Predicted
    AV_PICTURE_TYPE_B,     ///< Bi-dir predicted
    AV_PICTURE_TYPE_S,     ///< S(GMC)-VOP MPEG-4
    AV_PICTURE_TYPE_SI,    ///< Switching Intra
    AV_PICTURE_TYPE_SP,    ///< Switching Predicted
    AV_PICTURE_TYPE_BI    ///< BI type
 );

 TAVMediaType = (
    AVMEDIA_TYPE_UNKNOWN = -1,  ///< Usually treated as AVMEDIA_TYPE_DATA
    AVMEDIA_TYPE_VIDEO,
    AVMEDIA_TYPE_AUDIO,
    AVMEDIA_TYPE_DATA,          ///< Opaque data information usually continuous
    AVMEDIA_TYPE_SUBTITLE,
    AVMEDIA_TYPE_ATTACHMENT,    ///< Opaque data information usually sparse
    AVMEDIA_TYPE_NB
 );

type
  AVClassCategory = (                           //
    AV_CLASS_CATEGORY_NA = 0,                   //
    AV_CLASS_CATEGORY_INPUT,                    //
    AV_CLASS_CATEGORY_OUTPUT,                   //
    AV_CLASS_CATEGORY_MUXER,                    //
    AV_CLASS_CATEGORY_DEMUXER,                  //
    AV_CLASS_CATEGORY_ENCODER,                  //
    AV_CLASS_CATEGORY_DECODER,                  //
    AV_CLASS_CATEGORY_FILTER,                   //
    AV_CLASS_CATEGORY_BITSTREAM_FILTER,         //
    AV_CLASS_CATEGORY_SWSCALER,                 //
    AV_CLASS_CATEGORY_SWRESAMPLER,              //
    AV_CLASS_CATEGORY_DEVICE_VIDEO_OUTPUT = 40, //
    AV_CLASS_CATEGORY_DEVICE_VIDEO_INPUT,       //
    AV_CLASS_CATEGORY_DEVICE_AUDIO_OUTPUT,      //
    AV_CLASS_CATEGORY_DEVICE_AUDIO_INPUT,       //
    AV_CLASS_CATEGORY_DEVICE_OUTPUT,            //
    AV_CLASS_CATEGORY_DEVICE_INPUT,             //
    AV_CLASS_CATEGORY_NB
    // < not part of ABI/API
    );
  pAVClassCategory = ^AVClassCategory;


  PVA_LIST = ^VA_LIST;
  VA_LIST = array [0 .. 0] of Pointer;


(**
 * Return a string describing the media_type enum, NULL if media_type
 * is unknown.
 *)
function av_get_media_type_string(media_type: TAVMediaType): PAnsiChar; cdecl;
     external LIBAVUTIL_LIBNAME name 'av_get_media_type_string';



(* *
  * Send the specified message to the log if the level is less than or equal
  * to the current av_log_level. By default, all logging messages are sent to
  * stderr. This behavior can be altered by setting a different logging callback
  * function.
  * @see av_log_set_callback
  *
  * @param avcl A pointer to an arbitrary struct of which the first field is a
  *        pointer to an AVClass struct or NULL if general log.
  * @param level The importance level of the message expressed using a @ref
  *        lavu_log_constants "Logging Constant".
  * @param fmt The format string (printf-compatible) that specifies how
  *        subsequent arguments are converted to output.
*)
// void av_log(void *avcl, int level, const char *fmt, ...) av_printf_format(3, 4);
procedure av_log(avcl: Pointer; level: integer; const fmt: PAnsiChar);
cdecl varargs;
external LIBAVUTIL_LIBNAME;

(* *
  * Send the specified message to the log if the level is less than or equal
  * to the current av_log_level. By default, all logging messages are sent to
  * stderr. This behavior can be altered by setting a different logging callback
  * function.
  * @see av_log_set_callback
  *
  * @param avcl A pointer to an arbitrary struct of which the first field is a
  *        pointer to an AVClass struct.
  * @param level The importance level of the message expressed using a @ref
  *        lavu_log_constants "Logging Constant".
  * @param fmt The format string (printf-compatible) that specifies how
  *        subsequent arguments are converted to output.
  * @param vl The arguments referenced by the format string.
*)
// void av_vlog(void *avcl, int level, const char *fmt, va_list vl);
procedure av_vlog(avcl: Pointer; level: integer; const fmt: PAnsiChar; vl: PVA_LIST); cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Get the current log level
  *
  * @see lavu_log_constants
  *
  * @return Current log level
*)
// int av_log_get_level(void);
function av_log_get_level(): integer; cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Set the log level
  *
  * @see lavu_log_constants
  *
  * @param level Logging level
*)
// void av_log_set_level(int level);
procedure av_log_set_level(level: integer); cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Set the logging callback
  *
  * @note The callback must be thread safe, even if the application does not use
  *       threads itself as some codecs are multithreaded.
  *
  * @see av_log_default_callback
  *
  * @param callback A logging function with a compatible signature.
*)
// void av_log_set_callback(void (*callback)(void*, int, const char*, va_list));
Type
  Tav_log_callback = procedure(p: Pointer; lvl: Integer; fmt: PAnsiChar; vl: PVA_LIST);
cdecl varargs;

procedure av_log_set_callback(callbackproc: Tav_log_callback); cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Default logging callback
  *
  * It prints the message to stderr, optionally colorizing it.
  *
  * @param avcl A pointer to an arbitrary struct of which the first field is a
  *        pointer to an AVClass struct.
  * @param level The importance level of the message expressed using a @ref
  *        lavu_log_constants "Logging Constant".
  * @param fmt The format string (printf-compatible) that specifies how
  *        subsequent arguments are converted to output.
  * @param vl The arguments referenced by the format string.
*)
// void av_log_default_callback(void *avcl, int level, const char *fmt, va_list vl);
procedure av_log_default_callback(avcl: Pointer; level: integer; const fmt: PAnsiChar; vl: PVA_LIST); cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Return the context name
  *
  * @param  ctx The AVClass context
  *
  * @return The AVClass class_name
*)
// const char* av_default_item_name(void* ctx);
function av_default_item_name(ctx: Pointer): PAnsiChar; cdecl; external LIBAVUTIL_LIBNAME;

// AVClassCategory av_default_get_category(void *ptr);
function av_default_get_category(ptr: Pointer): AVClassCategory; cdecl; external LIBAVUTIL_LIBNAME;

(* *
  * Format a line of log the same way as the default callback.
  * @param line          buffer to receive the formatted line
  * @param line_size     size of the buffer
  * @param print_prefix  used to store whether the prefix must be printed;
  *                      must point to a persistent integer initially set to 1
*)
// void av_log_format_line(void *ptr, int level, const char *fmt, va_list vl,
// char *line, int line_size, int *print_prefix);
procedure av_log_format_line(ptr: Pointer; level: integer; const fmt: PAnsiChar; vl: PVA_LIST; line: PAnsiChar; line_size: integer; Var print_prefix: integer); cdecl;
  external LIBAVUTIL_LIBNAME;

(* *
  * Format a line of log the same way as the default callback.
  * @param line          buffer to receive the formatted line;
  *                      may be NULL if line_size is 0
  * @param line_size     size of the buffer; at most line_size-1 characters will
  *                      be written to the buffer, plus one null terminator
  * @param print_prefix  used to store whether the prefix must be printed;
  *                      must point to a persistent integer initially set to 1
  * @return Returns a negative value if an error occurred, otherwise returns
  *         the number of characters that would have been written for a
  *         sufficiently large buffer, not including the terminating null
  *         character. If the return value is not less than line_size, it means
  *         that the log message was truncated to fit the buffer.
*)
// int av_log_format_line2(void *ptr, int level, const char *fmt, va_list vl,
// char *line, int line_size, int *print_prefix);
function av_log_format_line2(ptr: Pointer; level: integer; const fmt: PAnsiChar; vl: PVA_LIST; line: PAnsiChar; line_size: integer; Var print_prefix: integer): integer; cdecl;
  external LIBAVUTIL_LIBNAME;

const
  (* *
    * Skip repeated messages, this requires the user app to use av_log() instead of
    * (f)printf as the 2 would otherwise interfere and lead to
    * "Last message repeated x times" messages below (f)printf messages with some
    * bad luck.
    * Also to receive the last, "last repeated" line if any, the user app must
    * call av_log(NULL, AV_LOG_QUIET, "%s", ""); at the end
  *)
  AV_LOG_SKIP_REPEATED = 1;

  (* *
    * Include the log severity in messages originating from codecs.
    *
    * Results in messages such as:
    * [rawvideo @ $DEADBEEF] [error] encode did not produce valid pts
  *)
  AV_LOG_PRINT_LEVEL = 2;

  // void av_log_set_flags(int arg);
procedure av_log_set_flags(arg: integer); cdecl; external LIBAVUTIL_LIBNAME;

// int av_log_get_flags(void);
function av_log_get_flags(): integer; cdecl; external LIBAVUTIL_LIBNAME;


