unit NetLib;

interface

uses Windows,Classes, WinSock,Forms, Controls, ComCtrls, ExtCtrls, SysUtils,
     DSupport, IdComponent, IdHTTP,ThreadLib, ActiveX, ComObj,
     Variants, Grids,WinINet, IdSSLOpenSSL, IdIcmpClient;

{Extended version TIdHttp component reconfigured to look like TNMHttp component}
{Useful so that programs can be written can use either one without rewriting}

type TOnStatus = procedure(Sender: TComponent; status: string) of object;



type TIdHTTPEx = class(TIdHttp)
 private
  Timer: TThreadTimer;
  FTimeOut: Integer;
  FBody: string;
  FReportLevel: integer;
  FOnPacketRecvd: TNotifyEvent;
  FOnStatus: TOnStatus;
  procedure HandleTimeout(Sender: TObject);

{$IF CompilerVersion >= 17}
  procedure DoReceive(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: int64);
{$ELSE}
  procedure DoReceive(Sender: TObject; AWorkMode: TWorkMode; const AWorkCount: integer);
{$IFEND}

  procedure InheritedStatus(axSender: TObject; const axStatus: TIdStatus; const asStatusText: string);
  function GetBytesRecvd: integer;
  procedure HandleDisconnect(Sender: TObject);
 protected
 public
  procedure Cancel;
  procedure Get(URL: string); overload;
  property BytesRecvd: integer read GetBytesRecvd;
 published
  constructor Create(AOwner: TComponent);
  destructor Destroy; override;
  property TimeOut: Integer read FTimeOut write FTimeOut default 0;
  property Body: string read FBody write FBody;
  property OnPacketRecvd: TNotifyEvent read FOnPacketRecvd write FOnPacketRecvd;
  property OnStatus: TOnStatus read FOnStatus write FOnStatus;
  property ReportLevel: integer read FReportLevel write FReportLevel;
end;

{------------------------------------------------------------------------------}



{Network resources variables}

const NetInfoMax = 250;

type
 TNetworkInfo = record
 DisType: DWord;
 Depth: integer;
 Name,IPAddress: string;
 end;


type
 TNetworkArray = record
 Index: integer;
 Info: array [0..NetInfoMax-1] of TNetworkInfo;
 end;


{Put network recursion in its own thread because it can take a long time to
finish and while it is finishing, it does not allow refreshing of the screen}

var NetData: TNetworkArray;

type TNetThread = class(TThread)
 private
 protected
 public
  procedure Execute; override;
  function WaitForTimeout(TimeOut: integer): boolean;
 end;


type TNetworkRecursor = class(TObject)
 private
  Recurser: TNetThread;
  procedure HandleProgress(Progress: integer);
 protected
 public
  ProgressBar: TProgressBar;
  procedure GetNetworkInfo(var Grid: TStringGrid; var ProgBar: TProgressBar);
  procedure Abort;
 end;


{------------------------------------------------------------------------------}


{Port/Connection variables}


const MIB_TCP_STATE_CLOSED = 1;
const MIB_TCP_STATE_LISTEN = 2;
const MIB_TCP_STATE_SYN_SENT = 3;
const MIB_TCP_STATE_SYN_RCVD = 4;
const MIB_TCP_STATE_ESTAB = 5;
const MIB_TCP_STATE_FIN_WAIT1 = 6;
const MIB_TCP_STATE_FIN_WAIT2 = 7;
const MIB_TCP_STATE_CLOSE_WAIT = 8;
const MIB_TCP_STATE_CLOSING = 9;
const MIB_TCP_STATE_LAST_ACK = 10;
const MIB_TCP_STATE_TIME_WAIT = 11;
const MIB_TCP_STATE_DELETE_TCB = 12;


type TMibTCPRow = packed record
 dwState: DWORD;
 dwLocalAddr: DWORD;
 dwLocalPort: DWORD;
 dwRemoteAddr: DWORD;
 dwRemotePort: DWORD;
 end;
type PTMibTCPRow = ^TMibTCPRow;


type TMibTCPTable = packed record
 dwNumEntries: DWORD;
 Table: array[0..0] of TMibTCPRow;
 end;
type PTMibTCPTable = ^TMibTCPTable;


{TCP connection states }

const TCPConnState: array[1..12] of string =
    ('closed','listening','syn_sent','syn_rcvd','established','fin_wait1',
     'fin_wait2','close_wait','closing','last_ack','time_wait','delete_tcb');


type TWellKnownPort = record
 Prt: DWORD;
 Srv: string[20];
 end;

const WellKnownPorts: array[0..29] of TWellKnownPort = (
	(Prt: 0; Srv: 'LOOPBACK'),
	(Prt: 7; Srv: 'ECHO'),		{Ping}
	(Prt: 9; Srv: 'DISCRD'),	{Discard}
	(Prt: 13; Srv: 'DAYTIM'),	{DayTime}
	(Prt: 17; Srv: 'QOTD'),		{Quote Of The Day}
	(Prt: 19; Srv: 'CHARGEN'),	{CharGen}
	(Prt: 20; Srv: 'FTP'),		{File Transfer Protocol}
	(Prt: 21; Srv: 'FTPC'),		{File Transfer Control Protocol}
	(Prt: 22; Srv: 'SSU'),		{SSH Remote Login Protocol}
	(Prt: 23; Srv: 'TELNET'),	{TelNet}
	(Prt: 25; Srv: 'SMTP'),		{Simple Mail Transfer Protocol}
	(Prt: 37; Srv: 'TIME'),		{Time Protocol}
	(Prt: 43; Srv: 'WHOIS'),	{WHO IS service}
	(Prt: 53; Srv: 'DNS'),		{Domain Name Service}
	(Prt: 67; Srv: 'BOOTPS'),	{BOOTP Server}
	(Prt: 68; Srv: 'BOOTPC'),	{BOOTP Client}
	(Prt: 69; Srv: 'TFTP'),		{Trivial FTP}
	(Prt: 70; Srv: 'GOPHER'),	{Gopher}
	(Prt: 79; Srv: 'FING'),		{Finger}
	(Prt: 80; Srv: 'HTTP'),		{HTTP}
	(Prt: 88; Srv: 'KERB'),		{Kerberos}
	(Prt: 109; Srv: 'POP2'),	{Post Office Protocol Version 2}
	(Prt: 110; Srv: 'POP3'),	{Post Office Protocol Version 3}
	(Prt: 119; Srv: 'NNTP'),	{Network News Transfer Protocol}
	(Prt: 123; Srv: 'NTP'),		{Network Time protocol}
	(Prt: 135; Srv: 'LOCSVC'),	{Location Service}
	(Prt: 137; Srv: 'NBNAME'),	{NETBIOS Name service}
	(Prt: 138; Srv: 'NBDGRAM'),	{NETBIOS Datagram Service}
	(Prt: 139; Srv: 'NBSESS'),	{NETBIOS Session Service}
	(Prt: 161; Srv: 'SNMP')		{Simple Netw. Management Protocol}
	);

type
  TTableArray = packed record
  dwNumEntries: DWORD;
  Table: array[0..250] of TMibTCPRow;
  end;


var SearchEngine: string ='http://www.google.com/search?q=';	{Default search engine, customize if you want}
var Dictionary: string = 'http://dictionary.reference.com/browse/';

function GetTcpTable(pTCPTable: PTMibTCPTable; pDWSize: PDWORD; bOrder: BOOL): DWORD; stdcall; external 'IPHLPAPI.DLL';
function Port2Service(Port: DWORD): string;
function PortToLittle(NetPort: DWORD ): DWORD;
function BEIPAddressToStr(IPAddr: DWORD): string;


type TNetworkProgress = procedure(Progress: integer);

procedure RecurseNetwork(var NetData: TNetworkArray; ProgressProc: TNetworkProgress); overload;
procedure RecurseNetwork(var NetData: TNetworkArray); overload;
procedure GetPortStrings(PortData: TMibTCPRow; Local: boolean; var S1,S2,S3: string);
function GetIPFromHost(const HostName: string): string;
procedure BrowseURL(URL: string);
procedure DoInternetSearch(SearchStr: String);
procedure DoDictionaryLookup(SearchStr: String);


function StringToIPAddress(S: string): cardinal;
function IPAddressToString(IPAddress: cardinal): string; overload;
function IPAddressToString(A,B,C,D: byte): string; overload;

function MACAddressToString(MAC: Int64): string;

function ExtractURLPath(URL: string): string;
function ExtractURLPathAndFile(S: string): string;
function ExcludeTrailingForwardSlash(S: string): string;
function IncludeTrailingForwardSlash(S: string): string;
function ExcludeLeadingForwardSlash(S: string): string;
function IncludeLeadingForwardSlash(S: string): string;
function RemoveWWWFromURL(S: string): string;
function RemoveTransportProtocol(S: string): string;
function RemoveURLPath(S: string): string;
function ExtractDomainName(S: string): string;
function ExtractURLFilename(URL: string): string;

function NormalizeURL(URL: string): string;
function ExtractURLExtension(URL: string): string;
procedure GetIPAddresses(var SL: TStringList);
function CheckUrl(url:string):boolean;


{There are two different methods used here to do pinging}

type TPingStatus = record
 Status: cardinal;
 RoundTripTime:  cardinal;
 end;


type TPingResult = record
 StatusCode: integer;
 ResponseTime: integer;
 Address,ProtocolAddress: string;
 ReplySize: integer;
 TimeToLive: integer;
 end;


function GetPingStatusStr(StatusCode: integer): string;
function PingServer(const Address: string; BufferSize: Word): TPingResult;

function Ping(InetAddress : string; TimeOut: integer; var Status: TPingStatus) : boolean; overload;
function Ping(B1,B2,B3,B4: byte; TimeOut: integer; var Status: TPingStatus) : boolean; overload;
function ValidInternetConnection(var Status: cardinal): boolean;

function GetWinSockErrorStr(ErrorNum: integer): string;
function IPAddressExists(IPAddr: string; TimeOut: integer): boolean;
function IsTCPPortOpen(Port: Word; IPAddress: string; TimeOut: integer): boolean;
function IsUDPPortInUse(IPAddress: string; Port: integer): boolean;


type TPortInfo = record
 IPAddress: string;
 Port: integer;
 end;

type TPortArray = array of TPortInfo;


type TPortProgress = procedure(Percent: integer);

procedure GetUsedUDPPorts(var PA: TPortArray; Progress: TPortProgress);


implementation

uses ShellAPI;


{---------------- Untility Routines -----------------------------------------}



function MACAddressToString(MAC: Int64): string;
var PB: PByteArray;
var I: integer;
begin
PB:=@MAC;
for I:=5 downto 0 do
	begin
	Result:=Result+IntToHex(PB[I],2);
	if I<>0 then Result:=Result+'-';
	end;
end;




function IPAddressToString(IPAddress: cardinal): string;
{ converts Little Endian IP-address to dotted decimal string}
var I: integer;
const ByteCount = 4;
var Bytes: array [0..ByteCount-1] of byte;
begin
for I:=0 to High(Bytes) do
	begin
	Bytes[I]:=IPAddress and $FF;
	IPAddress:=IPAddress shr 8;
	end;
Result:='';
for I:=High(Bytes) downto 0 do
	begin
	Result:=Result + IntToStr(Bytes[I]);
	if I<>0 then Result:=Result+'.';
	end;
end;



function IPAddressToString(A,B,C,D: byte): string; overload;
{Convert four-byte IP address into dotted-string}
begin
Result:=IntToStr(A)+'.'+IntToStr(B)+'.'+IntToStr(C)+'.'+IntToStr(D);
end;




function StringToIPAddress(S: string): cardinal;
{ converts dotted decimal string to Little endian integer IP-address}
var S1,S2,S3: string;
var B1,B2,B3,B4: byte;
var I: integer;
begin
I:=pos('.',S); S1:=Copy(S,1,I-1); Delete(S,1,I);
I:=pos('.',S); S2:=Copy(S,1,I-1); Delete(S,1,I);
I:=pos('.',S); S3:=Copy(S,1,I-1); Delete(S,1,I);
B1:=StrToInt(S1); B2:=StrToInt(S2);
B3:=StrToInt(S3); B4:=StrToInt(S);
{These are separated out to prevent an erroneous
overflow error if overflow testing is on}
Result:=B1 shl 24 or B2 shl 16 or B3 shl 8 or B4;
end;



{----------------------------------------------------------------------------}




function GetIPFromHost(const HostName: string): string;
{Translate Host name for IP Address}
type TSocketAddArray = array[0..10] of PInAddr;
type PSocketAddArray = ^TSocketAddArray;
var HostEntry: PHostEnt;
var SocketAddArray: PSocketAddArray;
var i: Integer;
var WSAData: TWSAData;
begin
WSAStartup($101, WSAData);
Result := '';
HostEntry := GetHostByName(PAnsiChar(HostName));
if HostEntry = nil then Exit;
SocketAddArray := PSocketAddArray(HostEntry^.h_addr_list);
I := 0;
while SocketAddArray^[I] <> nil do
	begin
	if I<>0 then Result:=Result+',';
	Result:=Result+inet_ntoa(SocketAddArray^[I]^);
	Inc(I);
	end;
WSACleanup;
end;



{----------- Routine To Recurse Network Resource information -----}

function GetLastItem(S: string): string;
{Get item in string after last backslash char}
var P: integer;
begin
P:=Length(S);
Result:='';
while (P>0) and (S[P]<>'\') do
 begin Result:=S[P]+Result; Dec(P); end;
end;



procedure ExtractNameData(var D: TNetworkInfo);
{Extract IP Address or Resource Name }
begin
case D.Distype of
 ResourceDisplayType_Server:
	begin
	{Remove leading slashes}
	Delete(D.Name,1,2);
	D.IPAddress:=GetIPFromHost(D.Name);
	end;
 ResourceDisplayType_Share: D.Name:=GetLastItem(D.Name);
 end;
end;



var ProgressMax,Progress: cardinal;


procedure ExtractData(var A: TNetworkArray);
var P: integer;
var S,S2: string;
var Info: TNetworkInfo;
begin
ExtractNameData(A.Info[A.Index]);
{Separate comma-delimited IP addresses}
if A.Info[A.Index].Distype=ResourceDisplayType_Server then
	begin
	P:=1;
	Info:=A.Info[A.Index];
	S:=Info.IPAddress;
	while true do
		begin
		S2:=ExtractToken(S,',',P);
		if S2='' then break;
		A.Info[A.Index]:=Info;
		A.Info[A.Index].IPAddress:=S2;
		Inc(A.Index);
		end;
	end
else Inc(A.Index);
end;



procedure RecurseNetResource(var NetData: TNetworkArray; Depth: Integer; Resource: TNetResource; ProgressProc: TNetworkProgress);
{Recurse through network resources specified Resource}
{Result returned in NetData}
const MaxEntries = 250;
var BufferLength: DWORD;
var I: DWORD;
var R,Count: DWord;
var Handle: THandle;
var ComputerInfo: array [0..MaxEntries-1] of TNetResource;
begin
Inc(Depth);
R := WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, 0, @Resource, Handle);
if R <> NO_ERROR then exit;

Count := MaxEntries;
BufferLength := SizeOf(ComputerInfo);
R := WNetEnumResource(Handle, Count, @ComputerInfo, BufferLength);
if R <> NO_ERROR then exit;

ProgressMax:=ProgressMax + Count;
for I:=0 to Count-1 do
	begin
	NetData.Info[NetData.Index].Depth:=Depth;
	NetData.Info[NetData.Index].DisType:=ComputerInfo[I].dwDisplayType;
	NetData.Info[NetData.Index].Name:=ComputerInfo[I].lpRemoteName;
	ExtractData(NetData);
	if Assigned(ProgressProc) then ProgressProc(MulDiv(100,Progress,ProgressMax));
	RecurseNetResource(NetData,Depth,ComputerInfo[I],ProgressProc);
	end;
end;


procedure RecurseNetwork(var NetData: TNetworkArray; ProgressProc: TNetworkProgress);
{Recurse through all network resources}
var Network: TNetResource;
begin
ProgressMax:=0;
Progress:=0;
FillChar(Network, SizeOf(Network), 0);
Network.dwScope := RESOURCE_GLOBALNET;
Network.dwType := RESOURCETYPE_ANY;
Network.dwUsage := RESOURCEUSAGE_CONTAINER;
NetData.Index:=0;
RecurseNetResource(NetData,-1,Network,ProgressProc);
end;



procedure RecurseNetwork(var NetData: TNetworkArray);
begin
RecurseNetwork(NetData,Nil);
end;


{----------------------------------------------------------------------------}


function PortToLittle(NetPort: DWORD ): DWORD;
{ Converts port number in network byte order to Little Endian}
begin
Result:=Swap(WORD(NetPort));
end;


function Port2Service(Port: DWORD): string;
{ converts well-known port numbers to service ID }
var I: integer;
begin
Port:=PortToLittle(Port);
{ in case port not found}
Result := Format('%4d', [Port]);
for I:= Low(WellKnownPorts) to High(WellKnownPorts) do
 if Port = WellKnownPorts[i].Prt then
	begin
	Result := WellKnownPorts[I].Srv;
	break;
	end;
end;


{----------------------------------------------------------------------}

function BEIPAddressToStr(IPAddr: DWORD): string;
{ converts IP-address in network byte order DWORD (Big Endian) to dotted decimal string}
var I: integer;
begin
Result := '';
for I := 1 to 4 do
	begin
	Result := Result + IntToStr(IPAddr and $FF);
	if I<>4 then Result:=Result+'.';
	IPAddr := IPAddr shr 8;
	end;
end;


function Port2Str( nwoPort: DWORD ): string;
{ converts port number in network byte order to string }
begin
Result := IntToStr(Swap(WORD(nwoPort)));
end;


procedure GetPortStrings(PortData: TMibTCPRow; Local: boolean; var S1,S2,S3: string);
var Address, Port: integer;
begin
S1:='';S2:='';S3:='';
if Local then begin Address:=PortData.dwLocalAddr; Port:=PortData.dwLocalPort; end
else begin Address:=PortData.dwRemoteAddr; Port:=PortData.dwRemotePort; end;

if Local or (Address<>0) then
	begin
	S1:=BEIPAddressToStr(Address);
	S2:=Port2Str(Port);
	S3:=Port2Service(Port);
	end;
end;

{---------------- Routines for access default browser --------------------------}

function GetErrorString(I: integer): string;
begin
case I of
 0: Result:='The operating system is out of memory or resources.';
 ERROR_FILE_NOT_FOUND: Result:='The specified file was not found.';
 ERROR_PATH_NOT_FOUND: Result:='The specified path was not found.';
 ERROR_BAD_FORMAT: Result:='The .EXE file is invalid (non-Win32 .EXE or error in .EXE image).';
 SE_ERR_ACCESSDENIED: Result:='The operating system denied access to the specified file. ';
 SE_ERR_ASSOCINCOMPLETE: Result:='The filename association is incomplete or invalid.';
 SE_ERR_DDEBUSY: Result:='The DDE transaction could not be completed because other DDE transactions were being processed.';
 SE_ERR_DDEFAIL: Result:='The DDE transaction failed.';
 SE_ERR_DDETIMEOUT: Result:='The DDE transaction could not be completed because the request timed out.';
 SE_ERR_DLLNOTFOUND: Result:='The specified dynamic-link library was not found. ';
 SE_ERR_NOASSOC: Result:='There is no application associated with the given filename extension.';
 SE_ERR_OOM: Result:='There was not enough memory to complete the operation.';
 SE_ERR_SHARE: Result:='A sharing violation occurred.';
 end;
end;



procedure BrowseURL(URL: string);
var I: HINST;
begin
{Note: It is normal ShellExecute to an C00005 exception during debugging}
I:=ShellExecute(Application.Handle, PChar('open'),PChar(URL),PChar(0),nil,SW_NORMAL);
if I<32 then Raise Exception.Create(GetErrorString(I));
end;




procedure DoInternetSearch(SearchStr: String);
{Search for term using Default Browser and specified search engine}
var S: string;
begin
S:=SearchEngine+SearchStr;
ShellExecute(0, 'open',pchar(S), nil, nil, SW_SHOW);
end;



procedure DoDictionaryLookup(SearchStr: String);
{Search for term using Default Browser and specified search engine}
var S: string;
begin
S:=Dictionary+SearchStr;
ShellExecute(0, 'open',pchar(S), nil, nil, SW_SHOW);
end;



{------------------------------------------------------------------------------}



function IncludeTrailingForwardSlash(S: string): string;
var L: integer;
begin
Result:=S;
L:=Length(Result);
if (L>0) and (Result[L]<>'/') then Result:=Result+'/';
end;


function ExcludeTrailingForwardSlash(S: string): string;
var L: integer;
begin
Result:=S;
L:=Length(Result);
if (L>0) and (Result[L]='/') then SetLength(Result,Length(Result)-1);
end;



function IncludeLeadingForwardSlash(S: string): string;
var L: integer;
begin
Result:=S;
L:=Length(Result);
if (L>0) and (Result[1]<>'/') then Result:='/'+Result;
end;



function ExcludeLeadingForwardSlash(S: string): string;
var L: integer;
begin
Result:=S;
L:=Length(Result);
if (L>0) and (Result[1]='/') then Delete(Result,1,1);
end;



function RemoveWWWFromURL(S: string): string;
begin
Result:=S;
Result:=RemoveMatchingStr('www.',Result,False);
end;


function RemoveTransportProtocol(S: string): string;
begin
Result:=S;
Result:=RemoveMatchingStr('ftp://',Result,False);
Result:=RemoveMatchingStr('http://',Result,False);
Result:=RemoveMatchingStr('shttp://',Result,False);
end;


function RemoveURLPath(S: string): string;
var Inx: integer;
begin
Result:=S;
while true do
	begin
	Inx:=FindLastOccurrence(Result,'/');
	if Inx<1 then break;
	{Deal with double slash}
	if (Inx>1) and (Result[Inx-1]='/') then break;
	Delete(Result,Inx,Length(Result)-(Inx-1));
	end;
end;



function ExtractDomainName(S: string): string;
begin
Result:=RemoveTransportProtocol(S);
Result:=RemoveWWWFromURL(Result);
Result:=RemoveURLPath(Result);
end;



function ExtractURLPath(URL: string): string;
begin
Result:=StringReplace(URL,'/','\',[rfreplaceall]);
Result:=ExtractFilePath(Result);
Result:=StringReplace(Result,'\','/',[rfreplaceall]);
end;


function ExtractURLPathAndFile(S: string): string;
var Domain: string;
begin
Domain:=ExtractDomainName(S);
Result:=RemoveTransportProtocol(S);
Result:=RemoveWWWFromURL(Result);
Result:=RemoveMatchingStr(Domain,Result,True);
Result:=ExcludeLeadingForwardSlash(Result);
end;


function ExtractURLFilename(URL: string): string;
begin
Result:=StringReplace(URL,'/','\',[rfreplaceall]);
Result:=ExtractFilename(Result);
end;


function ExtractURLExtension(URL: string): string;
{Get normalized extention of URL filename}
begin
Result:=StringReplace(URL,'/','\',[rfreplaceall]);
Result:=ExtractFileExt(Result);
if (Length(Result)>0) and (Result[1]='.') then Delete(Result,1,1);
Result:=LowerCase(Result);
end;



function NormalizeURL(URL: string): string;
{Normalize URL by replacing spaces with %20}
var I: integer;
begin
Result:='';
for I:=1 to Length(URL) do
	begin
	case URL[I] of
	 ' ': Result:=Result+'%20';
	 else Result:=Result+URL[I];
	 end;
	end;
end;


{ TIdHTTPEx }



procedure TIdHTTPEx.HandleTimeout(Sender: TObject);
begin
Timer.Enabled:=False;

Disconnect;
end;

procedure TIdHTTPEx.HandleDisconnect(Sender: TObject);
begin
Timer.Enabled:=False;
end;



{$IF CompilerVersion >= 17}
procedure TIdHTTPEx.DoReceive(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: int64);
{$ELSE}
procedure TIdHTTPEx.DoReceive(Sender: TObject; AWorkMode: TWorkMode; const AWorkCount: integer);
{$IFEND}
begin
if AWorkMode=wmRead then
	begin
	if Assigned(FOnPacketRecvd) then FOnPacketRecvd(Self);
	end;
end;



procedure TIdHTTPEx.InheritedStatus(axSender: TObject; const axStatus: TIdStatus; const asStatusText: string);
begin
if Assigned(FOnStatus) then FOnStatus(Self,asStatusText);
end;


constructor TIdHTTPEx.Create(AOwner: TComponent);
begin
inherited ;
Timer:=TThreadTimer.Create;
Timer.Enabled:=False;
Timer.OnTimer:=HandleTimeout;
inherited OnStatus:=InheritedStatus;
OnWork:=DoReceive;
OnDisconnected:=HandleDisconnect;
end;


destructor TIdHTTPEx.Destroy;
begin
Timer.Free;
inherited;
end;



procedure TIdHTTPEx.Get(URL: string);
var Stream: TStringStream;
begin
Stream:=TStringStream.Create('');
try
Timer.Interval:=FTimeout;
if FTimeout<>0 then Timer.Enabled:=True;
try
Get(URL, Stream);
Body := Stream.DataString;
finally Timer.Enabled:=False; end;
finally Stream.Free; end;
end;



function TIdHTTPEx.GetBytesRecvd: integer;
begin
Result:=FWorkInfos[wmRead].Current;
end;


procedure TIdHTTPEx.Cancel;
begin
Disconnect;
end;


{------------------------------------------------------------------------------}



function GetPingStatusStr(StatusCode: integer): string;
begin
case statusCode of
 0: Result:='Success';
 11001: Result:='Buffer Too Small';
 11002: Result:='Destination Net Unreachable';
 11003: Result:='Destination Host Unreachable';
 11004: Result:='Destination Protocol Unreachable';
 11005: Result:='Destination Port Unreachable';
 11006: Result:='No Resources';
 11007: Result:='Bad Option';
 11008: Result:='Hardware Error';
 11009: Result:='Packet Too Big';
 11010: Result:='Request Timed Out';
 11011: Result:='Bad Request';
 11012: Result:='Bad Route';
 11013: Result:='TimeToLive Expired Transit';
 11014: Result:='TimeToLive Expired Reassembly';
 11015: Result:='Parameter Problem';
 11016: Result:='Source Quench';
 11017: Result:='Option Too Big';
 11018: Result:='Bad Destination';
 11032: Result:='Negotiating IPSEC';
 11050: Result:='General Failure';
 12001: Result:='No Connection';	{My own message}
 else result:='Unknow';
 end;
end;

{----------------- Ping routines based on WMI ---------------------------------}


function PingServer(const Address: string; BufferSize: Word): TPingResult;
{The form of the Address parameter can be either the computer name (wxyz1234),}
{ IPv4 address (192.168.177.124), or IPv6 address (2010:836B:4179::836B:4179) }
var FSWbemLocator: OLEVariant;
var FWMIService: OLEVariant;
var FWbemObjectSet: OLEVariant;
var FWbemObject: OLEVariant;
var oEnum: IEnumvariant;
var iValue: LongWord;
begin;
FSWbemLocator:= CreateOleObject('WbemScripting.SWbemLocator');
FWMIService:= FSWbemLocator.ConnectServer('localhost', 'root\CIMV2', '', '');
FWbemObjectSet:= FWMIService.ExecQuery(Format('SELECT * FROM Win32_PingStatus where Address=%s AND BufferSize=%d',[QuotedStr(Address),BufferSize]),'WQL',0);
oEnum:= IUnknown(FWbemObjectSet._NewEnum) as IEnumVariant;
ZeroMemory(@Result,SizeOf(Result));
if oEnum.Next(1, FWbemObject, iValue) = 0 then
	begin
	Result.StatusCode:=FWbemObject.StatusCode;
	Result.Address:=Address;
	if Result.StatusCode=0 then
		begin
		Result.ResponseTime:=FWbemObject.ResponseTime;
		Result.ProtocolAddress:=FWbemObject.ProtocolAddress;
		Result.ReplySize:=FWbemObject.ReplySize;
		Result.TimeToLive:=FWbemObject.TimeToLive;
		end;
	end
else Result.StatusCode:=-1;
FWbemObject:=Unassigned;
FWbemObjectSet:=Unassigned;
end;

{----------------- Ping routines based on Win32 and WinInet -------------------}

{May be more robust, more responsive, not generate exceptions and not require higher priviledges }



type TSunB = packed record
s_b1, s_b2, s_b3, s_b4: byte;
  end;

type TSunW = packed record
 s_w1, s_w2: word;
 end;

type TIPAddr = record
  case integer of
   0: (S_un_b: TSunB);
   1: (S_un_w: TSunW);
   2: (S_addr: longword);
  end;

type PIPAddr = ^TIPAddr;
type IPAddr = TIPAddr;


type TICMPEchoReply = record
 Address:  IPAddr;
 Status:  ULONG;
 RoundTripTime:  ULONG;
 DataSize: DWord;
 Reserved: DWord;
 Buffer:   array[0..128-1] of AnsiChar;
 end;
type PICMPEchoReply = ^TICMPEchoReply;


function IcmpCreateFile: THandle; stdcall; external 'icmp.dll';
function IcmpCloseHandle(icmpHandle: THandle): boolean; stdcall; external 'icmp.dll'
function IcmpSendEcho(IcmpHandle: THandle; DestinationAddress: IPAddr; RequestData: Pointer; RequestSize: Smallint;
    RequestOptions, ReplyBuffer: Pointer; ReplySize, Timeout: DWORD): DWORD; stdcall; external 'icmp.dll';


procedure LookupIPAddress(AIP: string; var AInAddr);
var phe: PHostEnt;
var pac: PAnsiChar;
var GInitData: TWSAData;
begin
WSAStartup($101, GInitData);
try
phe := GetHostByName(PAnsiChar(AIP));
if Assigned(phe) then
	begin
	pac := phe^.h_addr_list^;
	if Assigned(pac) then
		begin
		with TIPAddr(AInAddr).S_un_b do
		 begin
		 s_b1 := Byte(pac[0]);
		 s_b2 := Byte(pac[1]);
		 s_b3 := Byte(pac[2]);
		 s_b4 := Byte(pac[3]);
		 end;
		end
	else
		begin
		raise Exception.Create('Error getting IP from HostName');
		end;
	end
else
	begin
	raise Exception.Create('Error getting HostName');
	end;
except FillChar(AInAddr, SizeOf(AInAddr), #0); end;
WSACleanup;
end;



function Ping(B1,B2,B3,B4: byte; Timeout: integer; var Status: TPingStatus) : boolean;
{Ping by IP address}
var Handle : THandle;
var InAddr : IPAddr;
var DW : DWORD;
var Reply: TICMPEchoReply;
begin
result := false;
FillChar(Status, SizeOf(Status), 0);
InAddr.S_un_b.s_b1:=B1; InAddr.S_un_b.s_b2:=B2;
InAddr.S_un_b.s_b3:=B3; InAddr.S_un_b.s_b4:=B4;

Handle := IcmpCreateFile;
if Handle = INVALID_HANDLE_VALUE then Exit;
DW := IcmpSendEcho(Handle, InAddr, nil, 0, nil, @Reply, Sizeof(Reply), Timeout);
Result := (DW <> 0);
if Result then
	begin
	Status.Status:=Reply.Status;
	Status.RoundTripTime:=Reply.RoundTripTime;
	end;
IcmpCloseHandle(Handle);
end;



function Ping(InetAddress: string; Timeout: integer; var Status: TPingStatus) : boolean;
{Ping by URL/Host Name}
var InAddr : IPAddr;
begin
LookupIPAddress(InetAddress, InAddr);
Result:=Ping(InAddr.S_un_b.s_b1,InAddr.S_un_b.s_b2,InAddr.S_un_b.s_b3,InAddr.S_un_b.s_b4, Timeout,Status);
end;



function ValidInternetConnection(var Status: cardinal): boolean;
{Test for valid and accessible internet connection}
var Origin: cardinal;
var PS: TPingStatus;
begin
Result:=False;
{Flag no connection}
Status:=12001;
{Test if an internet connection has been configured}
if not InternetGetConnectedState(@Origin,0) then exit;
{Test if it is accessible by pinging it}
if not Ping(8,8,8,8,1000,PS) then exit;
Result:=PS.Status=0;
Status:=PS.Status;
end;



{==============================================================================}




procedure TNetThread.Execute;
begin
RecurseNetwork(NetData,nil);
end;


function TNetThread.WaitForTimeout(TimeOut: integer): boolean;
var R: cardinal;
begin
R:=WaitForSingleObject(Handle,TimeOut);
Result:=WAIT_TIMEOUT<>R;
end;



procedure TNetworkRecursor.HandleProgress(Progress: integer);
begin
ProgressBar.Position:=Progress;
end;



procedure TNetworkRecursor.Abort;
begin
if Recurser<>nil then TerminateThread(Recurser.Handle,0);
end;





procedure TNetworkRecursor.GetNetworkInfo(var Grid: TStringGrid; var ProgBar: TProgressBar);
var I,Row: integer;
var NetName,WrkName,CompName: string;
begin
Self.ProgressBar:=ProgBar;
for I:=1 to Grid.RowCount-1 do Grid.Rows[I].Clear;
try
Recurser:=TNetThread.Create(False);
while not Recurser.WaitForTimeout(100) do
	begin
	Application.ProcessMessages;
	if Assigned(ProgressBar) then HandleProgress((ProgressBar.Position+1) mod 100);
	end;
HandleProgress(0);
Grid.Cells[0,1]:='No Data Available';
Row:=1;
Screen.Cursor:=crHourGlass;
try
for I:=0 to NetData.Index-1 do
	begin
	case NetData.Info[I].DisType of
	 1: WrkName:=NetData.Info[I].Name;
	 2: CompName:=NetData.Info[I].Name;
	 6: NetName:=NetData.Info[I].Name;
	 end;
	if NetData.Info[I].DisType = 2 then
		begin
		Grid.Cells[0,Row]:=NetName;
		Grid.Cells[1,Row]:=WrkName;
		Grid.Cells[2,Row]:=CompName;
		Grid.Cells[3,Row]:=NetData.Info[I].IPAddress;
		Inc(Row);
		end;
	end;
finally Screen.Cursor:=crDefault; end;
finally FreeAndNil(Recurser); end;
end;



function GetWinSockErrorStr(ErrorNum: integer): string;
begin
case ErrorNum of
 6: Result:='Specified event object handle is invalid.';
 8: Result:='Insufficient memory available.';
 87: Result:='One or more parameters are invalid.';
 995: Result:='Overlapped operation aborted.';
 996: Result:='Overlapped I/O event object not in signaled state.';
 997: Result:='Overlapped operations will complete later.';
 10004: Result:='Interrupted function call.';
 10009: Result:='File handle is not valid.';
 10013: Result:='Permission denied.';
 10014: Result:='Bad address.';
 10022: Result:='Invalid argument.';
 10024: Result:='Too many open files.';
 10035: Result:='Resource temporarily unavailable.';
 10036: Result:='Operation now in progress.';
 10037: Result:='Operation already in progress.';
 10038: Result:='Socket operation on nonsocket.';
 10039: Result:='Destination address required.';
 10040: Result:='Message too long.';
 10041: Result:='Protocol wrong type for socket.';
 10042: Result:='Bad protocol option.';
 10043: Result:='Protocol not supported.';
 10044: Result:='Socket type not supported.';
 10045: Result:='Operation not supported.';
 10046: Result:='Protocol family not supported.';
 10047: Result:='Address family not supported by protocol family.';
 10048: Result:='Address already in use.';
 10049: Result:='Cannot assign requested address.';
 10050: Result:='Network is down.';
 10051: Result:='Network is unreachable.';
 10052: Result:='Network dropped connection on reset.';
 10053: Result:='Software caused connection abort.';
 10054: Result:='Connection reset by peer.';
 10055: Result:='No buffer space available.';
 10056: Result:='Socket is already connected.';
 10057: Result:='Socket is not connected.';
 10058: Result:='Cannot send after socket shutdown.';
 10059: Result:='Too many references.';
 10060: Result:='Connection timed out.';
 10061: Result:='Connection refused.';
 10062: Result:='Cannot translate name.';
 10063: Result:='Name too long.';
 10064: Result:='Host is down.';
 10065: Result:='No route to host.';
 10066: Result:='Directory not empty.';
 10067: Result:='Too many processes.';
 10068: Result:='User quota exceeded.';
 10069: Result:='Disk quota exceeded.';
 10070: Result:='Stale file handle reference.';
 10071: Result:='Item is remote.';
 10091: Result:='Network subsystem is unavailable.';
 10092: Result:='Winsock.dll version out of range.';
 10093: Result:='Successful WSAStartup not yet performed.';
 10101: Result:='Graceful shutdown in progress.';
 10102: Result:='No more results.';
 10103: Result:='Call has been canceled.';
 10104: Result:='Procedure call table is invalid.';
 10105: Result:='Service provider is invalid.';
 10106: Result:='Service provider failed to initialize.';
 10107: Result:='System call failure.';
 10108: Result:='Service not found.';
 10109: Result:='Class type not found.';
 10110: Result:='No more results.';
 10111: Result:='Call was canceled.';
 10112: Result:='Database query was refused.';
 11001: Result:='Host not found.';
 11002: Result:='Nonauthoritative host not found.';
 11003: Result:='This is a nonrecoverable error.';
 11004: Result:='Valid name, no data record of requested type.';
 11005: Result:='QoS receivers.';
 11006: Result:='QoS senders.';
 11007: Result:='No QoS senders.';
 11008: Result:='QoS no receivers.';
 11009: Result:='QoS request confirmed.';
 11010: Result:='QoS admission error.';
 11011: Result:='QoS policy failure.';
 11012: Result:='QoS bad style.';
 11013: Result:='QoS bad object.';
 11014: Result:='QoS traffic control error.';
 11015: Result:='QoS generic error.';
 11016: Result:='QoS service type error.';
 11017: Result:='QoS flowspec error.';
 11018: Result:='Invalid QoS provider buffer.';
 11019: Result:='Invalid QoS filter style.';
 11020: Result:='Invalid QoS filter type.';
 11021: Result:='Incorrect QoS filter count.';
 11022: Result:='Invalid QoS object length.';
 11023: Result:='Incorrect QoS flow count.';
 11024: Result:='Unrecognized QoS object.';
 11025: Result:='Invalid QoS policy object.';
 11026: Result:='Invalid QoS flow descriptor.';
 11027: Result:='Invalid QoS provider-specific flowspec.';
 11028: Result:='Invalid QoS provider-specific filterspec.';
 11029: Result:='Invalid QoS shape discard mode object.';
 11030: Result:='Invalid QoS shaping rate object.';
 11031: Result:='Reserved policy QoS element type.';
 else Result:='Unknown.';
 end;
end;



function IPAddressExists(IPAddr: string; TimeOut: integer): boolean;
{Test to see if an IP address exist by pinging it}
{Timeout controls haow to wait for a response}
var ICmp: TIdIcmpClient;
begin
Result:= False;
ICmp:= TIdIcmpClient.Create(nil);
try
ICmp.ReceiveTimeout:=TimeOut;
ICmp.Host:= IPAddr;
ICmp.Ping;
Sleep(1);
Result:=ICmp.ReplyStatus.BytesReceived> 0;
finally ICmp.Free; end;
end;




function IsTCPPortOpen(Port: Word; IPAddress: string; TimeOut: integer): boolean;
{Test if TCP port is opened and in use - will show port used by 0.0.0.0 as }
{well as the specified IP address. wait timeout in ms and abort if no response}
{Note: the Windows default timeout is 20 milliseconds. This timeout only works to shorten it.}
var Client: sockaddr_in;
var Sock, Blocking, Ret,Len: Integer;
var Err: integer;
var setW, setE: TFDSet;
var time_out: timeval;
begin
Result:=False;
{Create Socket}
Client.sin_family:= AF_INET;
Client.sin_addr.s_addr:= inet_addr(PAnsiChar(AnsiString(IPAddress)));
Client.sin_port:= htons(Port);
Sock :=socket(AF_INET, SOCK_STREAM, 0);
if Sock = INVALID_SOCKET then exit;

{Set socket to non-blocking mode}
Blocking:= 1;
if ioctlsocket(Sock, FIONBIO, Blocking) = SOCKET_ERROR then
	begin
	{Unable to set socket, so close and exit}
	CloseSocket(Sock);
	exit;
	end;

{Attempt a connection to the IP-Address and port}
if connect(Sock,Client,SizeOf(Client))=SOCKET_ERROR then
	begin
	{if the result is "WSAEWOULDBLOCK" that means the socket is still working}
	if WSAGetLastError<>WSAEWOULDBLOCK then
		begin
		{if it not "WSAEWOULDBLOCK,"  the connection failed immediately}
		{So close and exit}
		CloseSocket(Sock);
		exit;
		end;

	{The connection pending }
	{set up "Select" paramaters }
	FD_ZERO(setW); FD_SET(Sock, setW);
	FD_ZERO(setE); FD_SET(Sock, setE);

	{Setup "Select" timeout}
	time_out.tv_sec := TimeOut div 1000;
	time_out.tv_usec := (TimeOut mod 1000) * 1000;

	{Get the socket status, waiting the specified timeout}
	Ret:= select(0, nil, @setW, @setE, @time_out);
	if ret <= 0 then
		begin
		{ select failed or connection timed out }
		{ so Close socket and flag a timeout error}
		CloseSocket(Sock);
		if ret = 0 then WSASetLastError(WSAETIMEDOUT);
		exit;
		end;
        {Test "connect" status}
	if FD_ISSET(sock, setE) then
		begin
		{ connection failed }
		Err:= 0; Len:=sizeof(Err);
		{So get the type of failures}
		getsockopt(Sock, SOL_SOCKET, SO_ERROR, @Err, Len);
		{ Close socket, report the error and exit}
		CloseSocket(Sock);
		{Set error type}
		WSASetLastError(err);
		exit;
		end;
	Result:=True;
	end;
{Close socket}
CloseSocket(Sock);
end;



{------------------------------------------------------------------------------}


function IsUDPPortInUse(IPAddress: string; Port: integer): boolean;
{Check if UDP port is in use at the specified IP address}
{The algorithm only works for UDP ports - it will miss TCP ports}
var Sin: sockaddr_in;
var Sock: integer;
var wsaData: TWSAData;
begin
Result:=True;
WSAStartup($101, wsaData);
try
Sock:= socket(AF_INET, SOCK_DGRAM, 0);
if Sock = -1 then exit;
try
Sin.sin_port := htons(port);
Sin.sin_addr.s_addr := inet_addr(PAnsiChar(AnsiString(IPAddress)));;
Sin.sin_family := AF_INET;

Result:=(bind(Sock, Sin, sizeof(sockaddr_in)) = SOCKET_ERROR) and (WSAGetLastError = WSAEADDRINUSE);
finally closesocket(sock); end;
finally WSACleanup; end;
end;




procedure GetIPAddresses(var SL: TStringList);
{Get a list of all local IP addresses}
type TSocketAddArray = array [0..10] of PInAddr;
type PSocketAddArray = ^TSocketAddArray;
var HostEntry: PHostEnt;
var SocketAddArray: PSocketAddArray;
var HostName: array [0..63] of AnsiChar;
var WSAData: TWSADATA;
var Inx: integer;
begin
SL.Clear;
WSAStartup($101, WSAData);
GetHostName(HostName, SizeOf(HostName));
HostEntry :=GetHostByName(HostName);
if HostEntry = nil then exit;
SocketAddArray := PSocketAddArray(HostEntry^.h_addr_list);
Inx:=0;
while SocketAddArray^[Inx] <> nil do
	begin
	SL.Add(StrPas(inet_ntoa(SocketAddArray^[Inx]^)));
	Inc(Inx);
	end;
SL.Add('127.0.0.1');
SL.Add('0.0.0.0');
WSACleanup;
end;



procedure GetUsedUDPPorts(var PA: TPortArray; Progress: TPortProgress);
var I,J,Inx,Max: integer;
var InUSe: boolean;
var IP: string;
var Adds: TStringList;
begin
Adds:=TStringList.Create;
try
GetIPAddresses(Adds);
Max:=Adds.Count * 65535;
SetLength(PA,0);
for I:=0 to Adds.Count-1 do
	begin
	IP:=Adds[I];
	for J:=0 to 65535 do
		begin
		if IsUDPPortInUse(IP,J) then
			begin
			SetLength(PA,Length(PA)+1);
			Inx:=High(PA);
			PA[Inx].IPAddress:=IP;
			PA[Inx].Port:=J;
			end;
		If Assigned(Progress) then Progress(MulDiv(100,I * 65535 + J,Max));
		end;
	end;
finally Adds.Free; end;
end;

Function CheckUrl(url:string):boolean;
var hSession, hfile, hRequest: hInternet;
dwindex,dwcodelen :dword;
dwcode:array[1..20] of char;
res : pchar;
begin
if (pos('http://',lowercase(url))=0) and
   (pos('https://',lowercase(url))=0) then url := 'http://'+url;
Result := false;
hSession := InternetOpen('InetURL:/1.0', INTERNET_OPEN_TYPE_PRECONFIG,nil, nil, 0);
if assigned(hsession) then
	begin
	hfile := InternetOpenUrl(hsession, pchar(url), nil, 0, INTERNET_FLAG_RELOAD, 0);
	dwIndex  := 0;
	dwCodeLen := 10;
	HttpQueryInfo(hfile, HTTP_QUERY_STATUS_CODE, @dwcode, dwcodeLen, dwIndex);
	res := pchar(@dwcode);
	result:= (res ='200') or (res ='302');
	if assigned(hfile) then InternetCloseHandle(hfile);
	InternetCloseHandle(hsession);
	end;
end;


end.
