unit OpenFolderDlg;

interface

uses Windows,Forms,SysUtils,CommCtrl,ShlObj;


const BFFM_IUNKNOWN = 5;
const BFFM_SETEXPANDED = $46a;
const BFFM_SETOKTEXT = $469;

const BIF_NEWDIALOGSTYLE=$40;
const BIF_BROWSEINCLUDEURLS = $0080;
const BIF_UAHINT = $0100;
const BIF_NONEWFOLDERBUTTON=$200;
const BIF_NOTRANSLATETARGETS = $0400;
const BIF_SHAREABLE = $8000;

const BIF_BROWSEFILEJUNCTIONS = $00010000;
const BIF_USENEWUI = BIF_EDITBOX or BIF_NEWDIALOGSTYLE;


type  TDlgOption = (
doReturnOnlyFSDirs, doDontGoBelowDomain, doStatusText, doReturnFSAncestors, doEditBox,
doValidate, doNewDialogStyle, doBrowseIncludeURLs, doUseNewUI, doUAHint, doNoNewFolderButton,
doNoTranslateTargets, doBrowseForComputer, doBrowseForPrinter, doBrowseIncludeFiles,
doShareable, doBrowseFileJunctions);
TDlgOptions = set of TDlgOption;


const DefaultOptions = [DoReturnOnlyFSDirs, doStatusText, doEditBox, doUseNewUI, doUAHint, doNewDialogStyle];

var FStartPath: string;


type TFolderDlg = class(TObject)
 private
  FShowCreateBtn: boolean;
  FOptions: TDlgOptions;
  function GetStartPath: string;
  procedure SetStartPath(const Value: string);
  function GetFlags: integer;
  procedure SetOptions(const Value: TDlgOptions);
 public
  NewPath: string;
  Caption: string;
  constructor Create;
  function Execute: boolean;
  property ShowCreateBtn: boolean read FShowCreateBtn write FShowCreateBtn;
  property Options: TDlgOptions read FOptions write SetOptions default DefaultOptions;
  property StartPath: string read GetStartPath write SetStartPath;
 end;


var FolderDlg: TFolderDlg;


implementation


{ TFolderDlg }


function EnumCallback(hWndChild: HWND; lParam: LPARAM): LongBool; stdcall;
{Find Tree-view window and scroll to selection}
var szClass: array[0..MAX_PATH] of char;
var hNode: HTREEITEM;
var Item: integer;
begin
if (GetClassName(hWndChild, szClass, sizeof(szClass))<>0) and
   (strcomp(szClass,'SysTreeView32')=0) then
    	begin
    	{Get currently selected node}
    	hNode:=TreeView_GetSelection(hWndChild);
    	{Force it to scroll to where it is visible}
	PostMessage(hWndChild, TVM_ENSUREVISIBLE,0,integer(hNode));
	{Delay needed to give time for scrolling to take place}
	Sleep(250);
	PostMessage(hWndChild, TVM_ENSUREVISIBLE,0,integer(hNode));
	{Stop enumerating}
    	Result:=False;
    	end
else Result:=True;	{Continue enumerating}
end;




function BrowseCallBack(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM): Integer stdcall;
var hNode: HTREEITEM;
var hPos: integer;
begin
case uMsg of
 BFFM_INITIALIZED:
	begin
	SendMessage(Wnd,BFFM_SETSELECTION, 1, Integer(@FStartPath[1]));
	end;
 BFFM_SELCHANGED:
	begin
	EnumChildWindows(Wnd, @EnumCallback,0);
	end;
 end;
Result := 0;
end;




constructor TFolderDlg.Create;
begin
StartPath:='C:\';
Caption:='Choose a Folder';
FOptions := DefaultOptions;
ShowCreateBtn:=True;
end;



function TFolderDlg.GetFlags: integer;
begin
Result:=0;
if doReturnOnlyFSDirs in Options then Result:=Result or BIF_RETURNONLYFSDIRS;
if doDontGoBelowDomain in Options then Result:=Result or BIF_DONTGOBELOWDOMAIN;
if doStatusText in Options then Result:=Result or BIF_STATUSTEXT;
if doReturnFSAncestors in Options then Result:=Result or BIF_RETURNFSANCESTORS;
if doEditBox in Options then Result:=Result or BIF_EDITBOX;
if doValidate in Options then Result:=Result or BIF_VALIDATE;
if doNewDialogStyle in Options then Result:=Result or BIF_NEWDIALOGSTYLE;
if doBrowseIncludeURLs in Options then Result:=Result or BIF_BROWSEINCLUDEURLS;
if doUseNewUI in Options then Result:=Result or BIF_USENEWUI;
if doUAHint in Options then Result:=Result or BIF_UAHINT;
if doNoNewFolderButton in Options then Result:=Result or BIF_NONEWFOLDERBUTTON;
if doNoTranslateTargets in Options then Result:=Result or BIF_NOTRANSLATETARGETS;
if doBrowseForComputer in Options then Result:=Result or BIF_BROWSEFORCOMPUTER;
if doBrowseForPrinter in Options then Result:=Result or BIF_BROWSEFORPRINTER;
if doBrowseIncludeFiles in Options then Result:=Result or BIF_BROWSEINCLUDEFILES;
if doShareable in Options then Result:=Result or BIF_SHAREABLE;
if doBrowseFileJunctions in Options then Result:=Result or BIF_BROWSEFILEJUNCTIONS;
end;



function TFolderDlg.Execute: boolean;
var BrowseInfo: TBrowseInfo;
var UserPath: array[0..MAX_PATH] of char;
var FindContext: PItemIDList;
begin
NewPath:='';
FillChar(BrowseInfo,SizeOf(BrowseInfo),#0);
BrowseInfo.pszDisplayName:=@UserPath[0];
BrowseInfo.lpszTitle:=PChar(Caption);
BrowseInfo.ulFlags:=GetFlags;

BrowseInfo.hwndOwner:=Application.Handle;
if StartPath<>'' then BrowseInfo.lpfn:=BrowseCallBack;
FindContext:=SHBrowseForFolder(BrowseInfo);
Result:=Assigned(FindContext);
if Result then
	begin
	if SHGetPathFromIDList(FindContext,UserPath) then NewPath:=UserPath;
	GlobalFreePtr(FindContext);
	end;
end;
               

function TFolderDlg.GetStartPath: string;
begin
Result:=FStartPath;
end;

procedure TFolderDlg.SetStartPath(const Value: string);
begin
FStartPath:=Value;
end;



procedure TFolderDlg.SetOptions(const Value: TDlgOptions);
begin
if FOptions <> Value then
	begin
	FOptions := Value;
	end;
end;




initialization
FolderDlg:=TFolderDlg.Create;
finalization
FolderDlg.Free;
end.
