unit GLTreeView;

interface

uses Windows,Classes,Controls,ComCtrls,TreeViewEx,GLMesh,GLFrame,GLLines,
     GLFonts,GLRenderer,GLViewer,GLLibrary,TreeLib,HugeArry,Math;

type TProgressProc = procedure(Percent: integer);

procedure ResetTreeIterator(Progress: TProgressProc);
function GetTreeIteration(Viewer: TGLViewer; Tree: TTreeViewEx; var O: TGLObject; var Inx: integer; Progress: TProgressProc): boolean;
procedure SelectTreeItem(Tree: TTreeViewEx; Inx: integer);
function GetSelectedTreeObject(Viewer: TGLViewer; Tree: TTreeViewEx; var O: TGLObject; var Inx: integer): boolean;
procedure DeleteSelectedTree(Viewer: TGLViewer; Tree: TTreeViewEx);
function AddAtTreeSelected(Viewer: TGLViewer; Tree: TTreeViewEx; ObjectType: TRenderType): TGLObject;
procedure MoveViewerItems(Viewer: TGLViewer; FromInx,ToInx: integer);
procedure HandleDragOver(Viewer: TGLViewer; Tree: TTreeView; Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
procedure HandleDrop(Viewer: TGLViewer; Tree: TTreeView; Sender, Source: TObject; X, Y: Integer);
procedure DisplayGLViewer(Viewer: TGLViewer; Tree: TTreeViewEx);
function GetTreeFirstSelected(Tree: TTreeViewEx): integer;
function GetParentFrame(Viewer: TGLViewer; Inx: integer): TGLFrame; overload;
function GetParentFrame(Viewer: TGLViewer; GLO: TGLObject): TGLFrame; overload;
function GetFirstCheckedSelected(Tree: TTreeViewEx): integer;
function GetTreeFirstChecked(Tree: TTreeViewEx): integer;


implementation

uses TBMain;

var InteratorInx: integer;
var WasChecked: boolean;


{----------------- Handle Displaying Heirarchy in Tree ------------------------}


type ImageIndices = (iiFrame,iiMesh,iiString,iiLines);

procedure SetImageIndexes(Node: TTreeNode; Inx: ImageIndices);
begin
Node.ImageIndex:=integer(Inx);
Node.StateIndex:=integer(Inx);
end;





procedure DisplayFrame(Tree: TTreeViewEx; Node: TTreeNode; Frame: TGLFrame);
var FrameNode,TmpNode: TTreeNode;
var I: integer;
begin
FrameNode:=Tree.Items.AddChild(Node,Frame.Name);
SetImageIndexes(FrameNode,iiFrame);
for I:=0 to Frame.MeshList.Count-1 do
	begin
	TmpNode:=Tree.Items.AddChild(FrameNode,Frame.MeshList[I].Name);
	SetImageIndexes(TmpNode,iiMesh);
	end;
for I:=0 to Frame.StringList3D.Count-1 do
	begin
	TmpNode:=Tree.Items.AddChild(FrameNode,Frame.StringList3D[I].Name);
	SetImageIndexes(TmpNode,iiString);
	end;
for I:=0 to Frame.LineList.Count-1 do
	begin
	TmpNode:=Tree.Items.AddChild(FrameNode,Frame.LineList[I].Name);
	SetImageIndexes(TmpNode,iiLines);
	end;
for I:=0 to Frame.FrameList.Count-1 do
	begin
	DisplayFrame(Tree,FrameNode,Frame.FrameList[I]);
	end;
{Do this here because it won't expand if there are no children}
FrameNode.Expanded:=Frame.Expanded;
end;





procedure DisplayBaseFrame(Tree: TTreeViewEx; Node: TTreeNode; Frame: TGLFrame);
var TmpNode: TTreeNode;
var I: integer;
begin
for I:=0 to Frame.MeshList.Count-1 do
	begin
	TmpNode:=Tree.Items.Add(Node,Frame.MeshList[I].Name);
	SetImageIndexes(TmpNode,iiMesh);
	end;
for I:=0 to Frame.StringList3D.Count-1 do
	begin
	TmpNode:=Tree.Items.Add(Node,Frame.StringList3D[I].Name);
	SetImageIndexes(TmpNode,iiString);
	end;
for I:=0 to Frame.LineList.Count-1 do
	begin
	TmpNode:=Tree.Items.Add(TmpNode,Frame.LineList[I].Name);
	SetImageIndexes(TmpNode,iiLines);
	end;
for I:=0 to Frame.FrameList.Count-1 do
	begin
	DisplayFrame(Tree,Node,Frame.FrameList[I]);
	end;
end;


procedure SetCheckBoxes(Tree: TTreeViewEx; Viewer: TGLViewer);
{Note: TTreeViewEx has bug that requires check box to be turned off then on
 or some boxes will be missing. At the same time, you can't set checkbox value
 when checkboxes are off. This requires update the check boxes separately}
var I: integer;
begin
for I:=0 to Viewer.ObjectCount-1 do
	Tree.Checked[I]:=Viewer.GLObjects[I].Selected;
end;


procedure DisplayGLViewer(Viewer: TGLViewer; Tree: TTreeViewEx);
var HB: THugeBoolean;
var I,Cnt: integer;
begin
HB:=THugeBoolean.Create(5);
for I:=0 to Tree.Items.Count-1 do HB.Add(Tree.Items[I].Selected);

Tree.CheckBoxes:=False;
Tree.Items.Clear;
DisplayBaseFrame(Tree,nil,Viewer.BaseFrame);
Tree.CheckBoxes:=True;
SetCheckBoxes(Tree,Viewer);
Cnt:=Min(Tree.Items.Count,HB.Count);
for I:=0 to Cnt-1 do Tree.Items[I].Selected:=HB[I];

HB.Free;
end;


{------------------------------------------------------------------------------}

function IsRenderer(O: TGLObject; var R: TGLRenderer): boolean;
begin
Result:=O is TGLRenderer;
if Result then R:=TGLRenderer(O)
else R:=nil;
end;



function Is3DString(O: TGLObject; var GLS: TGL3DString): boolean;
{Test object to see if it is a 3DString and then convert}
begin
Result:=O is TGL3DString;
if Result then GLS:=TGL3DString(O)
else GLS:=nil;
end;




function GetTreeFirstSelected(Tree: TTreeViewEx): integer;
begin
Result:=-1;
if Tree.SelectionCount<1 then exit;
Result:=Tree.Selections[0].AbsoluteIndex;
end;




function GetTreeFirstChecked(Tree: TTreeViewEx): integer;
var I: integer;
begin
Result:=-1;
for I:=0 to Tree.Items.Count-1 do
 if Tree.Checked[I] then
	begin
	Result:=I;
	exit;
	end;
end;




function GetFirstCheckedSelected(Tree: TTreeViewEx): integer;
{Get first checked, if none are checked, get first selected}
begin
Result:=GetTreeFirstChecked(Tree);
if Result<0 then Result:=GetTreeFirstSelected(Tree);
end;




function GetSelectedTreeObject(Viewer: TGLViewer; Tree: TTreeViewEx; var O: TGLObject; var Inx: integer): boolean;
begin
Result:=False;
Inx:=GetTreeFirstSelected(Tree);
if (Inx<0) or (Inx>=Viewer.ObjectCount) then exit;
O:=Viewer.GLObjects[Inx];
Result:=True;
end;



procedure ResetTreeIterator(Progress: TProgressProc);
begin
WasChecked:=False;
InteratorInx:=-1;
if Assigned(Progress) then Progress(0);
end;



function GetTreeIteration(Viewer: TGLViewer; Tree: TTreeViewEx; var O: TGLObject; var Inx: integer; Progress: TProgressProc): boolean;
begin
Result:=False;
while True do
	begin
	Inc(InteratorInx);
	if Assigned(Progress) then Progress(MulDiv(100,InteratorInx,Tree.Items.Count));
	Inx:=InteratorInx;
	{if no checkbox selected then use highlight}
	if InteratorInx>=Tree.Items.Count then
		begin
		if WasChecked then exit;
        	if not GetSelectedTreeObject(Viewer,Tree,O,Inx) then exit;
		WasChecked:=True;
		break;
		end
	else if Tree.Checked[Inx] then
		begin
		O:=Viewer.GLObjects[Inx];
		WasChecked:=True;
		break;
		end;
	end;
Result:=True;
end;



function Get3DStringIteration(Viewer: TGLViewer; Tree: TTreeViewEx; var GLS: TGL3DString; var Inx: integer; Progress: TProgressProc): boolean;
var O: TGLObject;
begin
Result:=False;
while true do
	begin
	if not GetTreeIteration(Viewer,Tree,O,Inx, Progress) then exit;
	if Is3DString(O, GLS) then break;
	end;
Result:=True;
end;




procedure SelectTreeItem(Tree: TTreeViewEx; Inx: integer);
begin
Tree.ClearSelection(False);
Tree.Items[Inx].Selected:=True;
end;



procedure DeleteSelectedTree(Viewer: TGLViewer; Tree: TTreeViewEx);
var Inx: integer;
var O: TGLObject;
begin
if not GetSelectedTreeObject(Viewer,Tree,O,Inx) then exit;
//if Inx=0 then exit;
Viewer.DeleteObject(Inx);
if Tree.Items.Count>0 then
	begin
	if Inx>0 then SelectTreeItem(Tree,Inx-1)
	else SelectTreeItem(Tree,0);
	end;
Tree.Invalidate;
Viewer.Invalidate;
end;


{-------------- Routines for moving tree items --------------------------------}




function GetParentFrame(Viewer: TGLViewer; Inx: integer): TGLFrame;
var I: integer;
begin
for I:=Inx downto 0 do
 if Viewer.GLObjects[I] is TGLFrame then
	begin
	Result:=TGLFrame(Viewer.GLObjects[I]);
	exit;
	end;
Result:=Viewer.BaseFrame;
end;





function GetParentFrame(Viewer: TGLViewer; GLO: TGLObject): TGLFrame;
var Inx: integer;
begin
Inx:=Viewer.BaseFrame.MasterList.IndexOf(GLO);
Result:=GetParentFrame(Viewer,Inx);
end;





function AddAtTreeSelected(Viewer: TGLViewer; Tree: TTreeViewEx; ObjectType: TRenderType): TGLObject;
var Inx,I2: integer;
var ParentFrame: TGLFrame;
var DestObject: TGLObject;
begin
Result:=nil;
Inx:=GetTreeFirstSelected(Tree);
if (Inx<0) or (Inx>=Viewer.ObjectCount) then exit;
DestObject:=Viewer.GLObjects[Inx];
if DestObject.OwnerList<>nil then I2:=DestObject.OwnerList.IndexOf(DestObject) else I2:=0;
if (ObjectType=rtMesh) and (DestObject is TGLMesh) then
	begin
	Result:=TGLObject(DestObject.OwnerList[TGLMeshList(DestObject.OwnerList).Insert(I2)])
	end
else if (ObjectType=rtString3D) and (DestObject is TGL3DString) then
	begin
	Result:=TGLObject(DestObject.OwnerList[TGLStringList3D(DestObject.OwnerList).Insert(I2)])
	end
else if (ObjectType=rtLines) and (DestObject is TGLLines) then
	begin
	Result:=TGLObject(DestObject.OwnerList[TGLLineList(DestObject.OwnerList).Add])
	end
else if (ObjectType=rtFrame) and (DestObject is TGLFrame) and (Inx<>0) then
	begin
	Result:=TGLObject(TGLFrame(DestObject).FrameList[TGLFrame(DestObject).FrameList.Add])
	end
else
	begin
	ParentFrame:=GetParentFrame(Viewer, Inx);
	case ObjectType of
	 rtMesh: Result:=ParentFrame.MeshList[ParentFrame.MeshList.Add];
	 rtString3D: Result:=ParentFrame.StringList3D[ParentFrame.StringList3D.Add];
	 rtLines: Result:=ParentFrame.LineList[ParentFrame.LineList.Add];
	 rtFrame: Result:=ParentFrame.FrameList[ParentFrame.FrameList.Add];
	 end;
	end;
Inx:=Viewer.BaseFrame.MasterList.IndexOf(Result);
if Inx>0 then SelectTreeItem(Tree,Inx);
Tree.Invalidate;
Viewer.Invalidate;
end;




procedure MoveViewerItems(Viewer: TGLViewer; FromInx,ToInx: integer);
var ParentFrame: TGLFrame;
var FromObject,ToObject: TGLObject;
var NewObject: TObject;
var Inx: integer;
begin
FromObject:=Viewer.GLObjects[FromInx];
if ToInx<0 then ToObject:=Viewer.BaseFrame else ToObject:=Viewer.GLObjects[ToInx];
if FromObject=ToObject then exit;
if (FromObject is TGLMesh) and (ToObject is TGLMesh) then
	begin
	Inx:=ToObject.OwnerList.IndexOf(ToObject);
	NewObject:=ToObject.OwnerList[TGLMeshList(ToObject.OwnerList).Insert(Inx)];
	end
else if (FromObject is TGL3DString) and (ToObject is TGL3DString) then
	begin
	Inx:=ToObject.OwnerList.IndexOf(ToObject);
	NewObject:=ToObject.OwnerList[TGLStringList3D(ToObject.OwnerList).Insert(Inx)];
	end
else if (FromObject is TGLLines) and (ToObject is TGLLines) then
	begin
	NewObject:=ToObject.OwnerList[TGLLineList(ToObject.OwnerList).Add];
	end
else if (FromObject is TGLFrame) and (ToObject is TGLFrame) and (FromInx<>0) then
	begin
	NewObject:=TGLFrame(ToObject).FrameList[TGLFrame(ToObject).FrameList.Add];
	end
else
	begin
	ParentFrame:=GetParentFrame(Viewer, ToInx);
	if (FromObject is TGLMesh) then NewObject:=ParentFrame.MeshList[ParentFrame.MeshList.Add]
	else if (FromObject is TGL3DString) then NewObject:=ParentFrame.StringList3D[ParentFrame.StringList3D.Add]
	else if (FromObject is TGLLines) then NewObject:=ParentFrame.LineList[ParentFrame.LineList.Add]
	else if (FromObject is TGLFrame) then
		begin
		NewObject:=ParentFrame.FrameList[ParentFrame.FrameList.Add];
		end;
	end;
TGLObject(NewObject).Name:=TGLObject(FromObject).Name;
TGLObject(NewObject).Assign(FromObject);
Inx:=Viewer.BaseFrame.MasterList.IndexOf(FromObject);
Viewer.DeleteObject(Inx);
end;



function CanDrop(FromNode,ToNode: TTreeNode): boolean;
begin
Result:=True;
if FromNode.ImageIndex<>integer(iiFrame) then exit;
Result:=False;
while ToNode<>nil do
	begin
	if FromNode=ToNode then exit;
	ToNode:=ToNode.Parent;
	end;
Result:=True;
end;



procedure HandleDragOver(Viewer: TGLViewer; Tree: TTreeView; Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
var FromNode,ToNode: TTreeNode;
begin
Accept:=False;
if Source<>Tree then Exit;
FromNode:=Tree.Selected;
ToNode:=Tree.GetNodeAt(X,Y);
Accept:=CanDrop(FromNode,ToNode);
end;




procedure HandleDrop(Viewer: TGLViewer; Tree: TTreeView; Sender, Source: TObject; X, Y: Integer);
var FromInx,ToInx: integer;
var FromNode,ToNode: TTreeNode;
var I: integer;
var List: TList;
begin
if Source<>Tree then exit;
List:=TList.Create;
try
for I := 0 to Tree.SelectionCount - 1 do List.Add(Tree.Selections[I]);
Tree.Items.BeginUpdate;
	ToNode:=Tree.GetNodeAt(X,Y);
try
for I:=0 to List.Count-1 do
	begin
	FromNode:=TTreeNode(List[I]);
	FromInx:=FromNode.AbsoluteIndex;
	if ToNode=nil then ToInx:=-1 else ToInx:=ToNode.AbsoluteIndex;
	if (FromInx<0) then continue;
	MoveViewerItems(Viewer,FromInx,ToInx);
	end;
finally Tree.Items.EndUpdate; end;
finally List.Free; end;
end;




end.
