unit GL3DFigures;

interface

uses Windows,Graphics,Math,GLCustomMesh,GLLines,Vector;


type TTorusProgress = procedure(Percent: integer);

procedure BuildTetrahedron(Mesh: TGLCustomMesh; Size: double; F1,F2,F3,F4: TColor); overload;
procedure BuildTetrahedron(Mesh: TGLCustomMesh; Size: double; C: TColor); overload;
procedure BuildTetrahedron(Mesh: TGLCustomMesh); overload;

procedure DrawCenteredCube(M: TGLCustomMesh; P: T3DVector; Size: double; C:TColor);
procedure BuildPrism(Mesh: TGLCustomMesh; DoCube: boolean); overload;
procedure BuildPrism(Mesh: TGLCustomMesh; Size1,Size2: double; C: TColor); overload;
procedure BuildPrism(Mesh: TGLCustomMesh; SizeX,SizeY,SizeZ: double; C: TColor); overload;
procedure BuildBeveledPrism(Mesh: TGLCustomMesh; SizeX,SizeY,SizeZ,BevelWdith: double; DoBevel: boolean; C: TColor);

procedure BuildTorus(Mesh: TGLCustomMesh; Diameter, HolePercent: double; Sectors,Sections: integer; Progress: TTorusProgress); overload;
procedure BuildTorus(Mesh: TGLCustomMesh; Sectors,Sections: integer; Progress: TTorusProgress); overload;

procedure DrawCone(Mesh: TGLCustomMesh; Sectors: integer; Radius,Height: double; CapEnd: boolean; C: TColor);

procedure BuildSphere(Mesh: TGLCustomMesh; Radius: double; Sectors,Sections: integer; C: TColor);
procedure BuildCylinder(Mesh: TGLCustomMesh; Radius,Height: double; Sectors,Sections: integer; CapEnds: boolean; C: TColor);
procedure BuildElipsoid(Mesh: TGLCustomMesh; X,Y,Z: double; Sectors,Sections: integer; C: TColor);
procedure DrawTruncateIcosahedron(Mesh: TGLCustomMesh; Size: T3DVector;  PentColor,HexColor: TColor);


procedure DrawPolygon(Lines: TGLLines; Sides: integer; Radius: double);

implementation


procedure BuildTriangle(M: TGLCustomMesh; X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3: double; C: TColor);
{Build single triangle based individual points}
var T: TTriangle;
begin
T.P1:=MakeVector3D(X1,Y1,Z1); T.P2:=MakeVector3D(X2,Y2,Z2); T.P3:=MakeVector3D(X3,Y3,Z3);
M.AddTriangle(T,C);
end;


procedure BuildRectangle(M: TGLCustomMesh; X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double; C: TColor);
{Build rectangle based on individual points}
var P1,P2,P3,P4: T3DVector;
begin
P1:=MakeVector3D(X1,Y1,Z1);
P2:=MakeVector3D(X2,Y2,Z2);
P3:=MakeVector3D(X3,Y3,Z3);
P4:=MakeVector3D(X4,Y4,Z4);
M.AddRectangle(P1,P2,P3,P4,C);
end;

{------------------------------------------------------------------------------}


procedure BuildTetrahedron(Mesh: TGLCustomMesh; Size: double; F1,F2,F3,F4: TColor);
{Build a tetrahedron}
begin
Mesh.Clear;
BuildTriangle(Mesh,-Size,-Size,-Size, -Size,Size,Size, Size,Size,-Size, F1);
BuildTriangle(Mesh, -Size,-Size,-Size,  Size,Size,-Size,  Size,-Size,Size,F2);
BuildTriangle(Mesh, -Size,-Size,-Size,  Size,-Size,Size,  -Size,Size,Size,F3);
BuildTriangle(Mesh, -Size,Size,Size,  Size,-Size,Size,  Size,Size,-Size,F4);
Mesh.SetAllNormals;
end;


procedure BuildTetrahedron(Mesh: TGLCustomMesh);
begin
BuildTetrahedron(Mesh,1,clRed,clGreen,clBlue,clAqua);
end;


procedure BuildTetrahedron(Mesh: TGLCustomMesh; Size: double; C: TColor);
begin
BuildTetrahedron(Mesh,1,C,C,C,C);
end;


{------------------------------------------------------------------------------}



procedure DrawCenteredCube(M: TGLCustomMesh; P: T3DVector; Size: double; C:TColor);
var P1,P2: T3DVector;
begin
P2:=VectorSubtract3D(P,MakeVector3D(-Size,-Size,-Size));
P1:=VectorSubtract3D(P,MakeVector3D(Size,Size,Size));
M.AddPrism(P1,P2,C);
end;



procedure BuildPrism(Mesh: TGLCustomMesh; Size1,Size2: double; C: TColor);
{Build prism according to Size1 and Size2}
begin
Mesh.Clear;
BuildRectangle(Mesh,-Size2,-Size1,-Size1, -Size2, Size1,-Size1,  Size2, Size1,-Size1,  Size2,-Size1,-Size1, C);
BuildRectangle(Mesh, Size2,-Size1, Size1,  Size2, Size1, Size1, -Size2, Size1, Size1, -Size2,-Size1, Size1, C);
BuildRectangle(Mesh, Size2,-Size1,-Size1,  Size2, Size1,-Size1,  Size2, Size1, Size1,  Size2,-Size1, Size1, C);
BuildRectangle(Mesh,-Size2, Size1, Size1, -Size2, Size1,-Size1, -Size2,-Size1,-Size1, -Size2,-Size1, Size1, C);
BuildRectangle(Mesh,-Size2, Size1, Size1,  Size2, Size1, Size1,  Size2, Size1,-Size1, -Size2, Size1,-Size1, C);
BuildRectangle(Mesh,-Size2,-Size1,-Size1,  Size2,-Size1,-Size1,  Size2,-Size1, Size1, -Size2,-Size1, Size1, C);
Mesh.SetAllNormals;
end;



procedure BuildBeveledPrism(Mesh: TGLCustomMesh; SizeX,SizeY,SizeZ,BevelWdith: double; DoBevel: boolean; C: TColor);
{Build prism according to SizeX, SizeY and SizeZ}
var SizeX2,SizeY2,SizeZ2: double;
begin
Mesh.Clear;
SizeX2:=SizeX-BevelWdith; SizeY2:=SizeY-BevelWdith; SizeZ2:=SizeZ-BevelWdith;

{+X - Plane}
BuildRectangle(Mesh, SizeX,-SizeY2,-SizeZ2,  SizeX, SizeY2,-SizeZ2,  SizeX, SizeY2, SizeZ2,  SizeX,-SizeY2, SizeZ2, C);
{-X - Plane}
BuildRectangle(Mesh,-SizeX, SizeY2, SizeZ2, -SizeX, SizeY2,-SizeZ2, -SizeX,-SizeY2,-SizeZ2, -SizeX,-SizeY2, SizeZ2, C);
{+Y - Plane}
BuildRectangle(Mesh,-SizeX2, SizeY, SizeZ2,  SizeX2, SizeY, SizeZ2,  SizeX2, SizeY,-SizeZ2, -SizeX2, SizeY,-SizeZ2, C);
{-Y - Plane}
BuildRectangle(Mesh,-SizeX2,-SizeY,-SizeZ2,  SizeX2,-SizeY,-SizeZ2,  SizeX2,-SizeY, SizeZ2, -SizeX2,-SizeY, SizeZ2, C);
{-Z - Plane}
BuildRectangle(Mesh,-SizeX2,-SizeY2,-SizeZ, -SizeX2, SizeY2,-SizeZ,  SizeX2, SizeY2,-SizeZ,  SizeX2,-SizeY2,-SizeZ, C);
{+Z - Plane}
BuildRectangle(Mesh, SizeX2,-SizeY2, SizeZ,  SizeX2, SizeY2, SizeZ, -SizeX2, SizeY2, SizeZ, -SizeX2,-SizeY2, SizeZ, C);

if not DoBevel then begin Mesh.SetAllNormals; exit; end;

{-X, Z bevel}
BuildRectangle(Mesh, -SizeX,SizeY2,SizeZ2, -SizeX,-SizeY2,SizeZ2, -SizeX2,-SizeY2,SizeZ, -SizeX2,SizeY2,SizeZ, C);
{-X, -Z bevel}
BuildRectangle(Mesh, -SizeX,SizeY2,-SizeZ2, -SizeX2,SizeY2,-SizeZ, -SizeX2,-SizeY2,-SizeZ, -SizeX,-SizeY2,-SizeZ2,  C);
{X, Z bevel}
BuildRectangle(Mesh, SizeX,SizeY2,SizeZ2, SizeX2,SizeY2,SizeZ, SizeX2,-SizeY2,SizeZ, SizeX,-SizeY2,SizeZ2, C);
{X, -Z bevel}
BuildRectangle(Mesh, SizeX,SizeY2,-SizeZ2, SizeX,-SizeY2,-SizeZ2, SizeX2,-SizeY2,-SizeZ, SizeX2,SizeY2,-SizeZ,  C);
{Z, Y bevel}
BuildRectangle(Mesh, -SizeX2,SizeY,SizeZ2, -SizeX2,SizeY2,SizeZ, SizeX2,SizeY2,SizeZ, SizeX2,SizeY,SizeZ2, C);
{Z, -Y bevel}
BuildRectangle(Mesh, -SizeX2,-SizeY,SizeZ2, SizeX2,-SizeY,SizeZ2, SizeX2,-SizeY2,SizeZ, -SizeX2,-SizeY2,SizeZ,  C);
{-Z, Y bevel}
BuildRectangle(Mesh, -SizeX2,SizeY,-SizeZ2, SizeX2,SizeY,-SizeZ2, SizeX2,SizeY2,-SizeZ, -SizeX2,SizeY2,-SizeZ, C);
{-Z, -Y bevel}
BuildRectangle(Mesh, -SizeX2,-SizeY,-SizeZ2, -SizeX2,-SizeY2,-SizeZ, SizeX2,-SizeY2,-SizeZ, SizeX2,-SizeY,-SizeZ2,  C);
{-X, +Y bevel}
BuildRectangle(Mesh, -SizeX,SizeY2,-SizeZ2, -SizeX,SizeY2,SizeZ2, -SizeX2,SizeY,SizeZ2, -SizeX2,SizeY,-SizeZ2, C);
{X, +Y bevel}
BuildRectangle(Mesh, SizeX,SizeY2,-SizeZ2, SizeX2,SizeY,-SizeZ2, SizeX2,SizeY,SizeZ2, SizeX,SizeY2,SizeZ2, C);
{-X, -Y bevel}
BuildRectangle(Mesh, -SizeX,-SizeY2,-SizeZ2, -SizeX2,-SizeY,-SizeZ2, -SizeX2,-SizeY,SizeZ2, -SizeX,-SizeY2,SizeZ2, C);
{X, -Y bevel}
BuildRectangle(Mesh, SizeX,-SizeY2,-SizeZ2, SizeX,-SizeY2,SizeZ2, SizeX2,-SizeY,SizeZ2, SizeX2,-SizeY,-SizeZ2, C);

BuildTriangle(Mesh, -SizeX,SizeY2,SizeZ2,  -SizeX2,SizeY2,SizeZ,  -SizeX2,SizeY,SizeZ2, C);
BuildTriangle(Mesh, SizeX,SizeY2,SizeZ2,  SizeX2,SizeY,SizeZ2,  SizeX2,SizeY2,SizeZ, C);
BuildTriangle(Mesh, -SizeX,-SizeY2,SizeZ2,  -SizeX2,-SizeY,SizeZ2,   -SizeX2,-SizeY2,SizeZ,C);
BuildTriangle(Mesh, SizeX,-SizeY2,SizeZ2,  SizeX2,-SizeY2,SizeZ,   SizeX2,-SizeY,SizeZ2,C);

BuildTriangle(Mesh, -SizeX,SizeY2,-SizeZ2, -SizeX2,SizeY,-SizeZ2,  -SizeX2,SizeY2,-SizeZ, C);
BuildTriangle(Mesh, SizeX,SizeY2,-SizeZ2, SizeX2,SizeY2,-SizeZ, SizeX2,SizeY,-SizeZ2, C);
BuildTriangle(Mesh, -SizeX,-SizeY2,-SizeZ2, -SizeX2,-SizeY2,-SizeZ, -SizeX2,-SizeY,-SizeZ2, C);
BuildTriangle(Mesh,  SizeX,-SizeY2,-SizeZ2,  SizeX2,-SizeY,-SizeZ2,  SizeX2,-SizeY2,-SizeZ, C);

Mesh.SetAllNormals;
end;



procedure BuildPrism(Mesh: TGLCustomMesh; SizeX,SizeY,SizeZ: double; C: TColor);
{Build prism according to SizeX, SizeY and SizeZ}
begin
Mesh.Clear;
BuildRectangle(Mesh,-SizeX,-SizeY,-SizeZ, -SizeX, SizeY,-SizeZ,  SizeX, SizeY,-SizeZ,  SizeX,-SizeY,-SizeZ, C);
BuildRectangle(Mesh, SizeX,-SizeY, SizeZ,  SizeX, SizeY, SizeZ, -SizeX, SizeY, SizeZ, -SizeX,-SizeY, SizeZ, C);
BuildRectangle(Mesh, SizeX,-SizeY,-SizeZ,  SizeX, SizeY,-SizeZ,  SizeX, SizeY, SizeZ,  SizeX,-SizeY, SizeZ, C);
BuildRectangle(Mesh,-SizeX, SizeY, SizeZ, -SizeX, SizeY,-SizeZ, -SizeX,-SizeY,-SizeZ, -SizeX,-SizeY, SizeZ, C);
BuildRectangle(Mesh,-SizeX, SizeY, SizeZ,  SizeX, SizeY, SizeZ,  SizeX, SizeY,-SizeZ, -SizeX, SizeY,-SizeZ, C);
BuildRectangle(Mesh,-SizeX,-SizeY,-SizeZ,  SizeX,-SizeY,-SizeZ,  SizeX,-SizeY, SizeZ, -SizeX,-SizeY, SizeZ, C);
Mesh.SetAllNormals;
end;





procedure BuildPrism(Mesh: TGLCustomMesh; DoCube: boolean);
{Build cube or prism according to the DoCube flage}
var CS,CS2: double;
const Scale = 1.00;
const W1 = 1 * Scale; W2 = 1.7 * Scale;
begin
CS:=W1;
if DoCube then CS2:=W1 else CS2:=W2;
BuildPrism(Mesh,CS,CS2,clBlue);
Mesh.SetAllNormals;
end;




{------------------------------------------------------------------------------}

procedure BuildTorus(Mesh: TGLCustomMesh; Diameter, HolePercent: double; Sectors,Sections: integer; Progress: TTorusProgress);
{Create torus of specified size}
var I, J: Integer;
const Pi2=Pi*2;
var LargeRadius,SmallRadius: double;

	procedure GetVertex(I,J: integer;var X,Y,Z: double);
	var A,B: double;
	var SA, CA, SB, CB : double;
	begin
	A:= I/Sectors * Pi2;
	SA:= Sin(A);   CA:= Cos(A);
	B:=J/Sections * Pi2;
	SB:= Sin(B);   CB:= Cos(B);
	X:= (LargeRadius + SmallRadius*CB) * CA;
	Y:= (LargeRadius + SmallRadius*CB) * SA;
	Z:= SmallRadius * SB;
	end;

	procedure BuildTorusRectangle(I,J: integer);
	var X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double;
	begin
	GetVertex(I  ,J  ,X4,Y4,Z4);
	GetVertex(I  ,J+1,X3,Y3,Z3);
	GetVertex(I+1,J+1,X2,Y2,Z2);
	GetVertex(I+1,J  ,X1,Y1,Z1);
	BuildRectangle(Mesh,X1,Y1,Z1, X2,Y2,Z2, X3,Y3,Z3, X4,Y4,Z4,clBlue);
	end;


begin
Mesh.Clear;
LargeRadius:=Diameter/2;  //Diameter/3;
SmallRadius:=LargeRadius * (1 - HolePercent); //LargeRadius/2;
for I:=0 to Sectors-1 do
	begin
	if Assigned(Progress) then Progress(MulDiv(100,I,Sectors));
	for J:= 0 to  Sections-1 do BuildTorusRectangle(I,J);
	end;
Mesh.SetAllNormals;
end;



procedure BuildTorus(Mesh: TGLCustomMesh; Sectors,Sections: integer; Progress: TTorusProgress);
begin
BuildTorus(Mesh,3,0.5,Sectors,Sections,Progress);
end;


{------------------------------------------------------------------------------}

procedure BuildElipsoid(Mesh: TGLCustomMesh; X,Y,Z: double; Sectors,Sections: integer; C: TColor);
var Lat,Long: integer;
var P1,P2,P3,P4: T3DVector;
var ALong,ALat,LongInc,LatInc: double;


	function SpherePoint(Long,Lat: double): T3DVector;
	begin
	Long:=DegToRad(Long);
	Lat:=DegToRad(Lat);
	Result.X:=Sin(Long) * Sin(Lat) * X;
	Result.Y:=Cos(Long) * Sin(Lat) * Y;
	Result.Z:=Cos(Lat) *  Z;
	end;

begin
Mesh.Clear;
LongInc:=360/Sectors;
LatInc:=180/Sections;
for Lat:=0 to Sections-1 do
 for Long:=0 to Sectors-1 do
	begin
	ALong:=Long * LongInc;
	ALat:=Lat * LatInc;
	P1:=SpherePoint(ALong,ALat);
	P2:=SpherePoint(ALong+LongInc,ALat);
	P3:=SpherePoint(ALong+LongInc,ALat+LatInc);
	P4:=SpherePoint(ALong,ALat+LatInc);
	Mesh.AddRectangle(P1,P2,P3,P4,C);
	end;
Mesh.SetAllNormals;
end;


procedure BuildSphere(Mesh: TGLCustomMesh; Radius: double; Sectors,Sections: integer; C: TColor);
begin
BuildElipsoid(Mesh,Radius,Radius,Radius,Sectors,Sections,C);
end;


{------------------------------------------------------------------------------}

{Golden Mean}

const P=1.6180339889;

{Array of vertices for }

const Vertices: array [0..59] of T3DVector = (
    (X:-P; Y:2; Z:-(1+2*P)),
    (X:-1; Y:-3*P; Z:0),
    (X:(2+P); Y:-2*P; Z:1),
    (X:0; Y:1; Z:3*P),
    (X:-1; Y:-(2+P); Z:2*P),
    (X:-3*P; Y:0; Z:1),
    (X:1; Y:-(2+P); Z:2*P),
    (X:-2*P; Y:1; Z:-(2+P)),
    (X:1; Y:(2+P); Z:-2*P),
    (X:2; Y:(1+2*P); Z:-P),
    (X:-(2+P); Y:-2*P; Z:1),
    (X:-(2+P); Y:2*P; Z:-1),
    (X:-1; Y:-(2+P); Z:-2*P ),
    (X:-P; Y:-2; Z:-(1+2*P)),
    (X:-(1+2*P); Y:-P; Z:2),
    (X:(1+2*P); Y:P; Z:2),
    (X:-1; Y:3*P; Z:0),
    (X:(1+2*P); Y:P; Z:-2),
    (X:-2; Y:(1+2*P); Z:P),
    (X:-(1+2*P); Y:-P; Z:-2),
    (X:3*P; Y:0; Z:1),
    (X:P; Y:-2; Z:(1+2*P)),
    (X:(1+2*P); Y:-P; Z:-2 ),
    (X:0; Y:-1; Z:-3*P),
    (X:0; Y:-1; Z:3*P),
    (X:1; Y:(2+P); Z:2*P),
    (X:-(1+2*P); Y:P; Z:-2),
    (X:-2; Y:(1+2*P); Z:-P),
    (X:1; Y:-3*P; Z:0),
    (X:(1+2*P); Y:-P; Z:2),
    (X:-2*P; Y:-1; Z:-(2+P)),
    (X:2*P; Y:1; Z:(2+P)),
    (X:P; Y:2; Z:(1+2*P)),
    (X:(2+P); Y:2*P; Z:-1),
    (X:-(2+P); Y:-2*P; Z:-1),
    (X:P; Y:-2; Z:-(1+2*P)),
    (X:2; Y:-(1+2*P); Z:P),
    (X:3*P; Y:0; Z:-1),
    (X:-2*P; Y:1; Z:(2+P)),
    (X:-(1+2*P); Y:P; Z:2),
    (X:-P; Y:-2; Z:(1+2*P)),
    (X:(2+P); Y:-2*P; Z:-1),
    (X:-1; Y:(2+P); Z:2*P),
    (X:-P; Y:2; Z:(1+2*P)),
    (X:1; Y:3*P; Z:0),
    (X:-2*P; Y:-1; Z:(2+P)),
    (X:P; Y:2; Z:-(1+2*P)),
    (X:1; Y:-(2+P); Z:-2*P),
    (X:2*P; Y:1; Z:-(2+P)),
    (X:2*P; Y:-1; Z:(2+P)),
    (X:-1; Y:(2+P); Z:-2*P),
    (X:2; Y:(1+2*P); Z:P),
    (X:(2+P); Y:2*P; Z:1),
    (X:-2; Y:-(1+2*P); Z:-P),
    (X:0; Y:1; Z:-3*P),
    (X:-3*P; Y:0; Z:-1),
    (X:2*P; Y:-1; Z:-(2+P)),
    (X:2; Y:-(1+2*P); Z:-P),
    (X:-2; Y:-(1+2*P); Z:P),
    (X:-(2+P); Y:2*P; Z:1 ));

{Indexes to pentagon edges}

const Pentagons: array[ 0..12-1,0..5-1] of integer = (
    (13, 23, 35, 47, 12),
    (16, 27, 11, 59, 18),
    (55, 26, 7, 30, 19),
    (21, 24, 40, 4, 6),
    (43, 3, 32, 25, 42),
    (51, 52, 33, 9, 44),
    (46, 54, 0, 50, 8),
    (48, 17, 37, 22, 56),
    (29, 20, 15, 31, 49),
    (14, 45, 38, 39, 5),
    (53, 1, 58, 10, 34),
    (41, 2, 36, 28, 57));


{Indexes to hexagon edges}

const Hexagons: array [0..20-1,0..6-1] of integer = (
    (58, 4, 40, 45, 14, 10),
    (27, 50, 0, 7, 26, 11),
    (18, 59, 39, 38, 43, 42),
    (29, 49, 21, 6, 36, 2),
    (29, 2, 41, 22, 37, 20),
    (32, 31, 15, 52, 51, 25),
    (55, 5, 39, 59, 11, 26),
    (28, 1, 53, 12, 47, 57),
    (36, 6, 4, 58, 1, 28),
    (31, 32, 3, 24, 21, 49),
    (34, 19, 30, 13, 12, 53),
    (37, 17, 33, 52, 15, 20),
    (57, 47, 35, 56, 22, 41),
    (44, 9, 8, 50, 27, 16),
    (40, 24, 3, 43, 38, 45),
    (56, 35, 23, 54, 46, 48),
    (5, 55, 19, 34, 10, 14),
    (44, 16, 18, 42, 25, 51),
    (7, 0, 54, 23, 13, 30),
    (33, 17, 48, 46, 8, 9)
    );



procedure DrawTruncateIcosahedron(Mesh: TGLCustomMesh; Size: T3DVector; PentColor,HexColor: TColor);
{Draw Truncate Icosahedron X,Y, Z size is specified in SIze, }
{Face colors are specified in PentColor and HexColor}
var I: integer;
var P1,P2,P3,P4,P5,P6: T3DVector;
begin
Size:=ScalarProduct3D(Size,1/(P * 3));
for I:=0 to High(Pentagons) do
	begin
	P1:=VectorMultiply3D(Vertices[Pentagons[I,0]],Size);
	P2:=VectorMultiply3D(Vertices[Pentagons[I,1]],Size);
	P3:=VectorMultiply3D(Vertices[Pentagons[I,2]],Size);
	P4:=VectorMultiply3D(Vertices[Pentagons[I,3]],Size);
	P5:=VectorMultiply3D(Vertices[Pentagons[I,4]],Size);
	Mesh.AddPentagon(P1,P2,P3,P4,P5,PentColor);
	end;
for I:=0 to High(Hexagons) do
	begin
	P1:=VectorMultiply3D(Vertices[Hexagons[I,0]],Size);
	P2:=VectorMultiply3D(Vertices[Hexagons[I,1]],Size);
	P3:=VectorMultiply3D(Vertices[Hexagons[I,2]],Size);
	P4:=VectorMultiply3D(Vertices[Hexagons[I,3]],Size);
	P5:=VectorMultiply3D(Vertices[Hexagons[I,4]],Size);
	P6:=VectorMultiply3D(Vertices[Hexagons[I,5]],Size);
	Mesh.AddHexagon(P1,P2,P3,P4,P5,P6,HexColor);
	end;
Mesh.SetAllNormals;
end;

{------------------------------------------------------------------------------}


function CirclePoint(Long,H,R: double): T3DVector;
begin
Long:=DegToRad(Long);
Result.X:=Sin(Long) * R;
Result.Y:=Cos(Long) * R;
Result.Z:=H;
end;


procedure DrawEnd(Mesh: TGLCustomMesh; Height,Radius: double; Sectors: integer; Top: boolean; C: TColor);
var Long: integer;
var P1,P2,P3: T3DVector;
var ALong,LongInc: double;
begin
LongInc:=360/Sectors;
P1:=MakeVector3D(0,0,Height);
for Long:=0 to Sectors-1 do
        begin
        ALong:=Long * LongInc;
        P2:=CirclePoint(ALong,Height,Radius);
        P3:=CirclePoint(ALong+LongInc,Height,Radius);
        if Top then Mesh.AddTriangle(P1,P3,P2,C)
        else Mesh.AddTriangle(P1,P2,P3,C);
        end;
end;



procedure BuildCylinder(Mesh: TGLCustomMesh; Radius,Height: double; Sectors,Sections: integer; CapEnds: boolean; C: TColor);
var Long,H: integer;
var P1,P2,P3,P4: T3DVector;
var ALong,LongInc,HT,HInc: double;
begin
Mesh.Clear;
LongInc:=360/Sectors;
HInc:=Height/Sections;
for H:=0 to Sections-1 do
 for Long:=0 to Sectors-1 do
	begin
	ALong:=Long * LongInc;
	HT:=(H * HInc)-(Height/2);
	P1:=CirclePoint(ALong,HT,Radius);
	P2:=CirclePoint(ALong+LongInc,HT,Radius);
	P3:=CirclePoint(ALong+LongInc,HT+HInc,Radius);
	P4:=CirclePoint(ALong,HT+HInc,Radius);
	Mesh.AddRectangle(P1,P4,P3,P2,C);
	end;
if CapEnds then
	begin
	HT:=-Height/2;
	DrawEnd(Mesh,HT,Radius,Sectors,False,C);
	HT:=Height/2;
	DrawEnd(Mesh,HT,Radius,Sectors,True,C);
	end;
Mesh.SetAllNormals;
end;


procedure DrawCone(Mesh: TGLCustomMesh; Sectors: integer; Radius,Height: double; CapEnd: boolean; C: TColor);
var Long: integer;
var P1,P2,P3: T3DVector;
var ALong,LongInc,H1,H2: double;
begin
Mesh.Clear;
LongInc:=360/Sectors;
H1:=-Height/2;
H2:=Height/2;
for Long:=0 to Sectors-1 do
	begin
	ALong:=Long * LongInc;
	P1:=CirclePoint(ALong,H1,Radius);
	P2:=CirclePoint(ALong+LongInc,H1,Radius);
	P3:=MakeVector3D(0,0,H2);
	Mesh.AddTriangle(P1,P3,P2,C);
	end;
if CapEnd then DrawEnd(Mesh,H1,Radius,Sectors,False,C);
Mesh.SetAllNormals;
end;




procedure DrawPolygon(Lines: TGLLines; Sides: integer;  Radius: double);
var AI,A: double;
var L: T3DLine;
var P2,C: T2DVector;
begin
Lines.Clear;
C:=MakeVector2D(0,0);
AI:=360/Sides;
A:=0;
L.P1:=MakeVector3D(0,Radius,0);
while A<=360 do
	begin
	P2:=PolarToVector2D(C,A,Radius);
	L.P1:=MakeVector3D(P2,0);
	P2:=PolarToVector2D(C,A+AI,Radius);
	L.P2:=MakeVector3D(P2,0);
	Lines.AddLine(L,clRed);
	A:=A+AI;
	end;
end;



end.
