unit GL3DTexture;

interface

uses Windows,Classes,SysUtils,Graphics,HugeArry,ImageLib,GLLibrary;



type TGL3DBitmap = class(THugeByte)
 private
  FWidth,FHeight,FDepth: integer;
  function Get(X, Y, Z: integer): TGLColor;
  procedure Put(X, Y, Z: integer; const Value: TGLColor);
  function GetDepth: integer;
  function GetHeight: integer;
  function GetWidth: integer;
  procedure SetDepth(const Value: integer);
  procedure SetHeight(const Value: integer);
  procedure SetWidth(const Value: integer);
  function GetColor(X, Y, Z: integer): TColor;
  procedure SetColor(X, Y, Z: integer; const Value: TColor);
    function GetLayerPointer(Z: integer): pointer;

 protected
 public
  constructor Create; overload;
  constructor Create(XSize,YSize,ZSize: integer); overload;
  procedure SetDimensions(Width,Height,Depth: integer);
    procedure LoadBitmap(BM: TBitmap; Z: integer);
    procedure LoadFromFile(Filename: string; Z: integer);
    procedure LoadFromResource(Name: string; Z: integer);
    procedure LoadFromStream(Stream: TStream; Z: integer);
  property Value[X,Y,Z: integer]: TGLColor read Get write Put; default;
  property Color[X,Y,Z: integer]: TColor read GetColor write SetColor;
  property Width: integer read GetWidth write SetWidth;
  property Height: integer read GetHeight write SetHeight;
  property Depth: integer read GetDepth write SetDepth;
 end;

implementation

{ TGL3DBitmap }


function TGL3DBitmap.GetLayerPointer(Z: integer): pointer;
var Inx: integer;
begin
Inx:=Z * FWidth * FHeight * SizeOf(TGLColor);
Result:=GetItemPtr(Inx);
end;



function TGL3DBitmap.Get(X, Y, Z: integer): TGLColor;
var Inx: integer;
begin
Inx:=(X + (Y * FWidth) + (Z * FWidth * FHeight)) * SizeOf(TGLColor);
Result:=PGLColor(GetItemPtr(Inx))^;
end;



procedure TGL3DBitmap.Put(X, Y, Z: integer; const Value: TGLColor);
var Inx: integer;
begin
Inx:=(X + (Y * FWidth) + (Z * FWidth * FHeight)) * SizeOf(TGLColor);
PGLColor(GetItemPtr(Inx))^:=Value;
end;




function TGL3DBitmap.GetColor(X, Y, Z: integer): TColor;
begin
Result:=GLColorToColor(Value[X,Y,Z]);
end;



procedure TGL3DBitmap.SetColor(X, Y, Z: integer; const Value: TColor);
begin
Self[X,Y,Z]:=ColorToGLColor(Value);
end;



function TGL3DBitmap.GetWidth: integer;
begin
Result:=FWidth;
end;


function TGL3DBitmap.GetHeight: integer;
begin
Result:=FHeight;
end;


function TGL3DBitmap.GetDepth: integer;
begin
Result:=FDepth;
end;



procedure TGL3DBitmap.SetWidth(const Value: integer);
begin
SetDimensions(Value,FHeight,FDepth);
end;



procedure TGL3DBitmap.SetHeight(const Value: integer);
begin
end;



procedure TGL3DBitmap.SetDepth(const Value: integer);
begin
SetDimensions(FWidth,FHeight,Value);
end;



procedure TGL3DBitmap.SetDimensions(Width, Height, Depth: integer);
begin
if (Width<=0) or (Height<=0) or (Depth<=0) then raise Exception.Create('All Dimension must be >=1');
FWidth:=Width;
FHeight:=Height;
FDepth:=Depth;
SetMaxItems(Width*Height*Depth*SizeOf(TGLColor))
end;


constructor TGL3DBitmap.Create;
begin
inherited Create(5);
Self.SetDimensions(1,1,1);
end;



constructor TGL3DBitmap.Create(XSize, YSize, ZSize: integer);
begin
inherited Create(5);
Self.SetDimensions(XSize, YSize, ZSize);
end;



procedure TGL3DBitmap.LoadBitmap(BM: TBitmap; Z: integer);
{Load bitmap into specific 3D Layer}
var Scanline: PRGBQuadArray;
var X,Y,Inx: integer;
var PB: PByteArray;
begin
BM.PixelFormat:=pf32Bit;
if (BM.Width<>Self.Width) or
   (BM.Height<>Self.Height) then SetDimensions(BM.Width,BM.Height,Self.Depth);
PB:=GetLayerPointer(Z);
for Y:=0 to BM.Height-1 do
	begin
	Scanline:=BM.Scanline[(BM.Height-1)-Y];
	for X:=0 to BM.Width-1 do
		begin
		Inx:=(X + (Y * BM.Width)) * 4;
		PB[Inx]:=Scanline^[X].rgbRed;
		PB[Inx+1]:=Scanline^[X].rgbGreen;
		PB[Inx+2]:=Scanline^[X].rgbBlue;
		PB[Inx+3]:=255;
		end;
	end;
Change;
end;


procedure TGL3DBitmap.LoadFromStream(Stream: TStream; Z: integer);
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
BM.LoadFromStream(Stream);
LoadBitmap(BM,Z);
finally BM.Free; end;
end;




procedure TGL3DBitmap.LoadFromFile(Filename: string; Z: integer);
var BM: TBitmap;
begin
BM:=TBitmap.Create;
try
LoadBitmapByExtension(BM,Filename,False);
LoadBitmap(BM,Z);
finally BM.Free; end;
end;




procedure TGL3DBitmap.LoadFromResource(Name: string; Z: integer);
var BM: TBitmap;
begin
BM := TBitmap.Create;
try
BM.Handle := Windows.LoadBitmap(hInstance, PChar(Name));
LoadBitmap(BM,Z);
finally BM.Free; end;
end;


end.
