unit GLColorArray;

interface

uses Graphics,SysUtils,HugeArry,BitmapLib,GLLibrary;


{------------------------------------------------------------}
{---------- Declaration for Huge Color Array object ---------}
{------------------------------------------------------------}


type TGLColorArray = class(THugeArray)
 private
  procedure Put(Index: integer; Item: TGLFaceColor);
  function Get(Index: integer): TGLFaceColor;
  function GetFaceColor(Index: integer): TColor;
  procedure SetFaceColor(Index: integer; const Value: TColor);
 protected
 public
  procedure SetTransparency(Transparency: double);
  procedure SetFaceColors(Index: integer; const C1,C2,C3: TColor);
  constructor Create(Number: integer);
  function Add(C: TColor): integer; reintroduce; overload;
  function Add(Item: TGLFaceColor): integer; reintroduce; overload;
  function Add(C1,C2,C3: TColor): integer; reintroduce; overload;
  function AddPointer(Item: TGLFaceColor; Ptr: integer): Integer;
  procedure Insert(Value: TGLFaceColor; Index: integer);
  procedure Delete(Index: integer);
  procedure Move(CurIndex, NewIndex: Integer);
  property Face[Index: integer]: TColor read GetFaceColor write SetFaceColor;
  property Value[Index: integer]: TGLFaceColor read Get write Put; default;
 end;


function GLColorToString(GLC: TGLFaceColor): string;


implementation


{Support routines}

function GLColorToString(GLC: TGLFaceColor): string;
begin
Result:=IntToStr(GLC.C1)+','+IntToStr(GLC.C2)+','+IntToStr(GLC.C3);
end;


{-----------------------------------------------------------------------------}
{  Implementation for huge TGLColor array object based on huge array object   }
{-----------------------------------------------------------------------------}


constructor TGLColorArray.Create(Number: integer);
{Create a huge integer array}
begin
inherited Create(Number,SizeOf(TGLFaceColor));
end;


function TGLColorArray.Add(Item: TGLFaceColor): integer;
{Add item to TGLColor array}
begin
Result:=inherited Add(Item);
end;



function TGLColorArray.Add(C1, C2, C3: TColor): integer;
var GLC: TGLFaceColor;
begin
GLC.C1:=C1;
GLC.C2:=C2;
GLC.C3:=C3;
Result:=Add(GLC);
end;


function TGLColorArray.Add(C: TColor): integer;
var GLC: TGLFaceColor;
begin
GLC.C1:=C;
GLC.C2:=C;
GLC.C3:=C;
Result:=Add(GLC);
end;



function TGLColorArray.AddPointer(Item: TGLFaceColor; Ptr: integer): Integer;
begin
Result:=inherited AddPointer(Item,Ptr);
end;


procedure TGLColorArray.SetTransparency(Transparency: double);
var I: integer;
var Mask: cardinal;
var PTR: ^TGLFaceColor;
var T: byte;
begin
T:=trunc(255 * Transparency);
Mask:=T shl 24;
PTR:=GetItemPtr(0);
for I:=0 to Count-1 do
	begin
	PTR^.C1:=(PTR^.C1 and $00FFFFFF) or Mask;
	PTR^.C2:=(PTR^.C2 and $00FFFFFF) or Mask;
	PTR^.C3:=(PTR^.C3 and $00FFFFFF) or Mask;
	Inc(PTR);
	end;
end;


procedure TGLColorArray.Put(Index: integer; Item: TGLFaceColor);
{Insert TGLColor at index}
var PTR: ^TGLFaceColor;
begin
PTR:=GetItemPtr(Index);
PTR^:=Item;
end;


function TGLColorArray.Get(Index: integer): TGLFaceColor;
{Get TGLColor at the index}
var PTR: ^TGLFaceColor;
begin
PTR:=GetItemPtr(Index);
Result:=PTR^;
end;


procedure TGLColorArray.Insert(Value: TGLFaceColor; Index: integer);
begin
InsertItem(Value,Index);
end;


procedure TGLColorArray.Delete(Index: integer);
begin
DeleteItem(Index);
end;



procedure TGLColorArray.Move(CurIndex, NewIndex: Integer);
var T: TGLFaceColor;
begin
T:=Value[CurIndex];
Delete(CurIndex);
Insert(T,NewIndex);
end;


function TGLColorArray.GetFaceColor(Index: integer): TColor;
begin
Result:=Self[Index].C1;
end;



procedure TGLColorArray.SetFaceColors(Index: integer; const C1,C2,C3: TColor);
var GLC: TGLFaceColor;
begin
GLC.C1:=C1;
GLC.C2:=C2;
GLC.C3:=C3;
Self[Index]:=GLC;
end;


procedure TGLColorArray.SetFaceColor(Index: integer; const Value: TColor);
var GLC: TGLFaceColor;
begin
GLC.C1:=Value;
GLC.C2:=Value;
GLC.C3:=Value;
Self[Index]:=GLC;
end;


end.
 