unit GLCubeMap;

interface

uses Classes,OpenGL15,GLLibrary,Vector;

type TCubeTextureMap = (cmLinear,cmReflection);

type TGLCubeMap = class(TPersistent)
 private
  FSPlane, FTPlane, FRPlane: TGLArrayf4;
  FTextureBuilt: boolean;
  FMapMode: TCubeTextureMap;
  FScale: T3DVector;
  FOffset: T3DVector;
  procedure BuildTexture;
  procedure Change;
  procedure HandleBMChange(Sender: TObject);
  procedure SetRPlane(const Value: TGLArrayf4);
  procedure SetSPlane(const Value: TGLArrayf4);
  procedure SetTPlane(const Value: TGLArrayf4);
  procedure SetMapMode(const Value: TCubeTextureMap);
  procedure SetScale(const Value: T3DVector);
  procedure SetOffset(const Value: T3DVector);
 public
  LeftMap,RightMap,TopMap,BottomMap,FrontMap,BackMap: TGLBaseBitmap;
  constructor Create; virtual;
  destructor Destroy; override;
  procedure FinalizeRender;
  procedure Render;
  procedure Assign(Source: TPersistent);
  property RPlane: TGLArrayf4 read FRPlane write SetRPlane;
  property SPlane: TGLArrayf4 read FSPlane write SetSPlane;
  property TPlane: TGLArrayf4 read FTPlane write SetTPlane;
  property MapMode: TCubeTextureMap read FMapMode write SetMapMode;
  property Scale: T3DVector read FScale write SetScale;
  property Offset: T3DVector read FOffset write SetOffset;
 end;

implementation


{------------------ Cube Map --------------------------------------------------}

procedure TGLCubeMap.HandleBMChange(Sender: TObject);
begin
Change;
end;



constructor TGLCubeMap.Create;
begin
FTextureBuilt:=False;
LeftMap:=TGLBaseBitmap.Create;
RightMap:=TGLBaseBitmap.Create;
TopMap:=TGLBaseBitmap.Create;
BottomMap:=TGLBaseBitmap.Create;
FrontMap:=TGLBaseBitmap.Create;
BackMap:=TGLBaseBitmap.Create;

LeftMap.OnChange:=HandleBMChange;
RightMap.OnChange:=HandleBMChange;
TopMap.OnChange:=HandleBMChange;
BottomMap.OnChange:=HandleBMChange;
FrontMap.OnChange:=HandleBMChange;
BackMap.OnChange:=HandleBMChange;
FSPlane:=MakeGLArrayf4(1,0,0,0);
FTPlane:=MakeGLArrayf4(0,1,0,0);
FRPlane:=MakeGLArrayf4(0,0,1,0);
FScale:=MakeVector3D(1,1,1);
FOffset:=MakeVector3D(0,0,0);
FMapMode:=cmLinear;
end;




destructor TGLCubeMap.Destroy;
begin
LeftMap.Free;
RightMap.Free;
TopMap.Free;
BottomMap.Free;
FrontMap.Free;
BackMap.Free;
inherited;
end;



procedure SetupCubeTextureParameters(var GT: TGLBaseBitmap; Target : TGLUInt);
begin
glBindTexture(Target, GT.TextName);
glTexParameteri(Target,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
glTexParameteri(Target,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
glTexParameteri(GL_TEXTURE_CUBE_MAP_EXT, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
gluBuild2DMipmaps(Target, 3, GT.Width, GT.Height, GL_RGBA, GL_UNSIGNED_BYTE,GT.GLBitmap);
end;



procedure TGLCubeMap.BuildTexture;
begin
SetupCubeTextureParameters(LeftMap, GL_TEXTURE_CUBE_MAP_POSITIVE_X_EXT);
SetupCubeTextureParameters(RightMap, GL_TEXTURE_CUBE_MAP_NEGATIVE_X_EXT);
SetupCubeTextureParameters(TopMap,  GL_TEXTURE_CUBE_MAP_POSITIVE_Y_EXT);
SetupCubeTextureParameters(BottomMap,GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_EXT);
SetupCubeTextureParameters(FrontMap, GL_TEXTURE_CUBE_MAP_POSITIVE_Z_EXT);
SetupCubeTextureParameters(BackMap,  GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_EXT);
FTextureBuilt:=True;
end;





procedure TGLCubeMap.Change;
begin
FTextureBuilt:=False;
end;





procedure TGLCubeMap.Render;
begin
if not FTextureBuilt then BuildTexture;

glMatrixMode(GL_TEXTURE);
glLoadIdentity;
glBindTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_X, RightMap.Textname);
glBindTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_X, LEftMap.Textname);
glBindTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_Y, TopMap.Textname);
glBindTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, BottomMap.Textname);
glBindTexture(GL_TEXTURE_CUBE_MAP_POSITIVE_Z, FrontMap.Textname);
glBindTexture(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, BackMap.Textname);

glEnable(GL_TEXTURE_CUBE_MAP);
glScalef(FScale.X, FScale.y, FScale.Z);
glTranslatef(FOffset.X,FOffset.Y,FOffset.Z);

case FMapMode of
 cmLinear:
	begin
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, @FRPlane);
	glTexGenfv(GL_S, GL_OBJECT_PLANE, @FSPlane);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, @FTPlane);
	end;
 cmReflection:
	begin
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP);
	end;
 end;

glEnable(GL_TEXTURE_GEN_S);
glEnable(GL_TEXTURE_GEN_T);
glEnable(GL_TEXTURE_GEN_R);
//glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
glMatrixMode(GL_MODELVIEW);
end;



procedure TGLCubeMap.FinalizeRender;
begin
glDisable(GL_TEXTURE_GEN_S);
glDisable(GL_TEXTURE_GEN_T);
glDisable(GL_TEXTURE_GEN_R);

glDisable(GL_TEXTURE_CUBE_MAP);
end;





procedure TGLCubeMap.SetRPlane(const Value: TGLArrayf4);
begin
if not AreEqualf4(FRPlane,Value) then
	begin
	FRPlane := Value;
	Change;
	end;
end;



procedure TGLCubeMap.SetSPlane(const Value: TGLArrayf4);
begin
if not AreEqualf4(FSPlane,Value) then
	begin
	FSPlane := Value;
	Change;
	end;
end;



procedure TGLCubeMap.SetTPlane(const Value: TGLArrayf4);
begin
if not AreEqualf4(FTPlane,Value) then
	begin
	FTPlane := Value;
	Change;
	end;
end;



procedure TGLCubeMap.SetMapMode(const Value: TCubeTextureMap);
begin
if Value<>FMapMode then
	begin
	FMapMode := Value;
	Change;
	end;
end;



procedure TGLCubeMap.SetScale(const Value: T3DVector);
begin
if not AreEqual3D(Value,FScale) then
	begin
	FScale:=Value;
	Change;
	end;
end;



procedure TGLCubeMap.SetOffset(const Value: T3DVector);
begin
if not AreEqual3D(Value,FOffset) then
	begin
	FOffset:=Value;
	Change;
	end;
end;



procedure TGLCubeMap.Assign(Source: TPersistent);
var ST: TGLCubeMap;
begin
inherited Assign(Source);
if Source is TGLCubeMap then
	begin
	ST:=TGLCubeMap(Source);
	FTextureBuilt:=False;
	LeftMap.Assign(ST.LeftMap);
	RightMap.Assign(ST.RightMap);
	TopMap.Assign(ST.TopMap);
	BottomMap.Assign(ST.BottomMap);
	FrontMap.Assign(ST.FrontMap);
	BackMap.Assign(ST.BackMap);

	RPlane:=ST.RPlane;
	SPlane:=ST.SPlane;
	TPlane:=ST.TPlane;
	MapMode:=ST.MapMode;
	Scale:=ST.Scale;
	Offset:=ST.Offset;
	end;
end;




end.
