unit GLFrame;

interface

uses Classes,SysUtils,Graphics,OpenGL15,Math,Vector,GLQuatLib,GLLibrary,GLMatrix,
     GLRenderer,GLCustomMesh,GLMesh,GLPoints,GLFastMesh,GLFonts,GLFont2D,GLLines,HugeArry,DSupport;



{Structure containing triangle intersect info}

type TTriangleIntersect = record
 Valid: boolean;
 Mesh: TGLCustomMesh;
 Index: integer;
 Triangle: TTriangle;
 Depth: double;
 Ray: T3DLine;
 Intercept: T3DVector;
 HitCount,NearestVertex: integer;
 end;


type TFrameGrid = record
  XYColor,YZColor,XZColor: TColor;
  ShowYZ,ShowXY,ShowXZ: boolean;
  Show: boolean;
 end;

type TListList = array [0..7] of TGLObjectList;

type
 TGLFrameList = class;

 TGLFrame = class(TGLObject)
 private
  procedure ChangeHandler(Sender: TObject);
  procedure ScreenFrameIntersect(X, Y: integer; Frame: TGLFrame; var Best: TTriangleIntersect);
  procedure ScreenMeshIntersect(X, Y: integer; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
  procedure DrawGrid;
  procedure DrawXYGrid;
  procedure DrawXZGrid;
  procedure DrawYZGrid;
  function GetTriangles: integer;
  procedure FillObjectList(List: TGLObjectList);
  procedure RayMeshIntersect(Ray: T3DLine; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
  procedure RayFrameIntersect(X, Y: Integer; Pitch,Yaw,Roll: double; Frame: TGLFrame; var Best: TTriangleIntersect);
  procedure DoRayMeshIntersect(X, Y: Integer; Pitch,Yaw,Roll: double; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
  function GetExtentMax: T3DVector; override;
  function GetExtentMin: T3DVector; override;
 protected
 public
  MeshList: TGLMeshList;
  FastMeshList: TGLFastMeshList;
  StringList3D: TGLStringList3D;
  StringList2D: TGLStringList2D;
  LineList: TGLLineList;
  PointList: TGLPointList;
  FrameList: TGLFrameList;
  ListList: TListList;
  Grid: TFrameGrid;
  Expanded: boolean;
  constructor Create(ListHandler: TListChangeEvent); reintroduce; virtual;
  destructor Destroy; override;
  procedure Clear;
  procedure Render(HitArray: THugePoint); reintroduce; virtual;
  procedure FindNearestTriangle(X,Y: integer; var  Best: TTriangleIntersect); overload;
  procedure FindNearestTriangle(X, Y: Integer; Pitch,Yaw,Roll: double; var  Best: TTriangleIntersect); overload;
  procedure Assign(GLObject: TGLObject); override;
  property Triangles: integer read GetTriangles;
  property ExtentMin: T3DVector read GetExtentMin;
  property ExtentMax: T3DVector read GetExtentMax;
 end;


{ TGLFameist }

TGLFrameList = class(TGLObjectList)
  private
  function GetTriangles: integer;
  protected
  function GetItem(Index: Integer): TGLFrame;
  procedure SetItem(Index: Integer; AObject: TGLFrame);
 public
  function Add: integer; overload;
  function Add(var Frame: TGLFrame): integer; overload;
  property Items[Index: Integer]: TGLFrame read GetItem write SetItem; default;
  procedure Assign(ObjectList: TGLObjectList);
  property Triangles: integer read GetTriangles;
 end;


type TGLBaseFrame = class(TGLFrame)
 private
 protected
  procedure MasterListhandler(GLObject: TGLObject; Action: TListNotification);
  //procedure Change;
 public
  MasterList: TGLObjectList;
  constructor Create; reintroduce; virtual;
  destructor Destroy; override;
  procedure Assign(GLObject: TGLObject); override;
 end;


implementation



{ TGLFrame }




function TGLFrame.GetExtentMax: T3DVector;
begin
Result:=GetMinimumVector(MeshList.ExtentMin,LineList.ExtentMin);
Result:=GetMinimumVector(Result,PointList.ExtentMin);
Result:=GetMinimumVector(Result,FastMeshList.ExtentMin);
Result:=GetMinimumVector(Result,StringList3D.ExtentMin);
Result:=GetMinimumVector(Result,StringList2D.ExtentMin);
Result:=GetMinimumVector(Result,FrameList.ExtentMin);
end;



function TGLFrame.GetExtentMin: T3DVector;
begin
Result:=GetMaximumVector(MeshList.ExtentMax,LineList.ExtentMax);
Result:=GetMaximumVector(Result,PointList.ExtentMax);
Result:=GetMaximumVector(Result,FastMeshList.ExtentMax);
Result:=GetMaximumVector(Result,StringList3D.ExtentMax);
Result:=GetMaximumVector(Result,StringList2D.ExtentMax);
Result:=GetMaximumVector(Result,FrameList.ExtentMax);
end;


procedure TGLFrame.ChangeHandler(Sender: TObject);
begin
Change;
end;



constructor TGLFrame.Create(ListHandler: TListChangeEvent);
{Must be created in the Create Window event of viewer}
{StringList3D needs the OpenGL window handle to work}
begin
inherited Create;
MeshList:=TGLMeshList.Create;
FastMeshList:=TGLFastMeshList.Create;
LineList:=TGLLineList.Create;
PointList:=TGLPointList.Create;
StringList3D:=TGLStringList3D.Create;
StringList2D:=TGLStringList2D.Create;
FrameList:=TGLFrameList.Create;

MeshList.OnListChange:=ListHandler;
FastMeshList.OnListChange:=ListHandler;
LineList.OnListChange:=ListHandler;
PointList.OnListChange:=ListHandler;
StringList3D.OnListChange:=ListHandler;
StringList2D.OnListChange:=ListHandler;
FrameList.OnListChange:=ListHandler;

MeshList.OnChange:=ChangeHandler;
FastMeshList.OnChange:=ChangeHandler;
LineList.OnChange:=ChangeHandler;
PointList.OnChange:=ChangeHandler;
StringList3D.OnChange:=ChangeHandler;
StringList2D.OnChange:=ChangeHandler;
FrameList.OnChange:=ChangeHandler;

ListList[0]:=MeshList;
ListList[1]:=FastMeshList;
ListList[2]:=StringList3D;
ListList[3]:=LineList;
ListList[4]:=FrameList;
ListList[5]:=StringList2D;
ListList[6]:=PointList;

Grid.XYColor:=clLime;
Grid.YZColor:=clYellow;
Grid.XZColor:=clAqua;
Grid.Show:=False;
Grid.ShowXY:=True;
Grid.ShowYZ:=True;
Grid.ShowXZ:=True;

Expanded:=False;
ResetTransforms;
end;



destructor TGLFrame.Destroy;
begin
{Make sure no events are triggered by freeing lists}
MeshList.OnListChange:=nil;
FastMeshList.OnListChange:=nil;
LineList.OnListChange:=nil;
PointList.OnListChange:=nil;
StringList3D.OnListChange:=nil;
StringList2D.OnListChange:=nil;
FrameList.OnListChange:=nil;

MeshList.Free;
FastMeshList.Free;
LineList.Free;
PointList.Free;
StringList3D.Free;
StringList2D.Free;
FrameList.Free;
inherited;
end;

{-----------------------------------------------------------------------------}


procedure TGLFrame.DrawXYGrid;
var X,Y: integer;
var P1,P2: T3DVector;
var R,G,B: single;
begin
GetRGB(Grid.XYColor,R,G,B);
glColor4f(R,G,B,1);
for X:=-3 to 3 do
	begin
	P1:=MakeVector3D(X,-3,0); P2:=MakeVector3D(X,3,0);
	glVertex3dv(@P1); glVertex3dv(@P2);
for Y:=-3 to 3 do
	begin
	P1:=MakeVector3D(-3,Y,0); P2:=MakeVector3D(3,Y,0);
	glVertex3dv(@P1); glVertex3dv(@P2);
	end;
	end;
end;


procedure TGLFrame.DrawXZGrid;
var X,Z: integer;
var P1,P2: T3DVector;
var R,G,B: single;
begin
GetRGB(Grid.XZColor,R,G,B);
glColor4f(R,G,B,1);
for X:=-3 to 3 do
	begin
	P1:=MakeVector3D(X,0,-3); P2:=MakeVector3D(X,0,3);
	glVertex3dv(@P1); glVertex3dv(@P2);
	end;
for Z:=-3 to 3 do
	begin
	P1:=MakeVector3D(-3,0,Z); P2:=MakeVector3D(3,0,Z);
	glVertex3dv(@P1); glVertex3dv(@P2);
	end;
end;


procedure TGLFrame.DrawYZGrid;
var Y,Z: integer;
var P1,P2: T3DVector;
var R,G,B: single;
begin
GetRGB(Grid.YZColor,R,G,B);
glColor4f(R,G,B,1);
for Y:=-3 to 3 do
	begin
	P1:=MakeVector3D(0,Y,-3); P2:=MakeVector3D(0,Y,3);
	glVertex3dv(@P1); glVertex3dv(@P2);
	end;
for Z:=-3 to 3 do
	begin
	P1:=MakeVector3D(0,-3,Z); P2:=MakeVector3D(0,3,Z);
	glVertex3dv(@P1); glVertex3dv(@P2);
	end;
end;



procedure TGLFrame.DrawGrid;
begin
glPushAttrib(GL_ALL_ATTRIB_BITS);
try
glDisable(GL_LIGHTING);

glBegin(GL_LINES);
if Grid.ShowXY then DrawXYGrid;

if Grid.ShowXZ then DrawXZGrid;

if Grid.ShowYZ then DrawYZGrid;
glEnd;
finally glPopAttrib; end;
end;




procedure TGLFrame.Render(HitArray: THugePoint);
var I,HI: integer;
var O: TObject;
begin
glPushMatrix;
try
inherited Render;
if Grid.Show then DrawGrid;
for I:=0 to MeshList.Count-1 do
 if Enabled and MeshList[I].Enabled then
	begin
	glPushMatrix;
	try
	{Record 'Name' of the object we are rendering for hit testing}
	O:=MeshList[I];
	HI:=HitArray.Add(Point(Integer(rtMesh),Integer(O)));
	glLoadName(HI);
	if MeshList[I].Texture.Enabled then MeshList[I].Texture.Render;
	MeshList[I].Render;
	if MeshList[I].Texture.Enabled then MeshList[I].Texture.FinalizeRender;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	MeshList[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;

for I:=0 to FastMeshList.Count-1 do
 if Enabled and FastMeshList[I].Enabled then
	begin
	glPushMatrix;
	try
	{Record 'Name' of the object we are rendering for hit testing}
	O:=FastMeshList[I];
	HI:=HitArray.Add(Point(Integer(rtMesh),Integer(O)));
	glLoadName(HI);
	if FastMeshList[I].Texture.Enabled then FastMeshList[I].Texture.Render;
	FastMeshList[I].Render;
	if FastMeshList[I].Texture.Enabled then FastMeshList[I].Texture.FinalizeRender;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	FastMeshList[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;
for I:=0 to StringList3D.Count-1 do
 if Enabled and StringList3D[I].Enabled then
	begin
	glPushMatrix;
	try
	{Record 'Name' of the object we are rendering for hit testing}
	O:=StringList3D[I];
	HI:=HitArray.Add(Point(Integer(rtString3D),Integer(O)));
	glLoadName(HI);
	if StringList3D[I].Texture.Enabled then StringList3D[I].Texture.Render;
	StringList3D[I].Render;
	if StringList3D[I].Texture.Enabled then StringList3D[I].Texture.FinalizeRender;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	StringList3D[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;

for I:=0 to StringList2D.Count-1 do
 if Enabled and StringList2D[I].Enabled then
	begin
	glPushMatrix;
	try
	StringList2D[I].Render;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	StringList2D[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;

for I:=0 to LineList.Count-1 do
 if Enabled and LineList[I].Enabled then
	begin
	glPushMatrix;
	try
	{Record 'Name' of the object we are rendering for hit testing}
	O:=LineList[I];
	HI:=HitArray.Add(Point(Integer(rtLines),Integer(O)));
	glLoadName(HI);
	if LineList[I].Texture.Enabled then LineList[I].Texture.Render;
	LineList[I].Render;
	if LineList[I].Texture.Enabled then LineList[I].Texture.FinalizeRender;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	LineList[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;

for I:=0 to PointList.Count-1 do
 if Enabled and PointList[I].Enabled then
	begin
	glPushMatrix;
	try
	{Record 'Name' of the object we are rendering for hit testing}
	O:=PointList[I];
	HI:=HitArray.Add(Point(Integer(rtLines),Integer(O)));
	glLoadName(HI);
	if PointList[I].Texture.Enabled then PointList[I].Texture.Render;
	PointList[I].Render;
	if PointList[I].Texture.Enabled then PointList[I].Texture.FinalizeRender;
	finally glPopMatrix; end;
	end
else
	begin
	glPushMatrix;
	try
	PointList[I].UpdateMatrices;	{Make sure matrices are updated even when disabled}
	finally glPopMatrix; end;
	end;

for I:=0 to FrameList.Count-1 do FrameList[I].Render(HitArray);

finally glPopMatrix; end;
end;



{------------------------------------------------------------------------------}



procedure TGLFrame.RayMeshIntersect(Ray: T3DLine; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
{Find the interect between specified Ray and Mesh-triangle if any}
var I: integer;
var Tri: TTriangle;
var Intercept: T3DVector;
var NewDepth: double;
var D1,D2,D3: double;
begin
for I:=0 to Mesh.FaceCount-1 do
        begin
        Tri:=Mesh.FaceTriangles[I];
	if not RayTriangleIntersect(Ray,Tri,True,Intercept) then continue;
	Inc(Best.HitCount);
	NewDepth:=VectorABS3D(VectorSubtract3D(Ray.P1,Intercept));
	if Best.Valid and (NewDepth>=Best.Depth) then continue;
	Best.Valid:=True;
	Best.Mesh:=Mesh;
	Best.Triangle:=Tri;
	Intercept:=GLMatrix.MatrixVectorMult(Mesh.Matrix,Intercept);
	Best.Intercept:=Intercept;
	Best.Index:=I;
	Best.Depth:=NewDepth;
        end;
D1:=VectorABS3D(VectorSubtract3D(Best.Intercept,Best.Triangle.P1));
D2:=VectorABS3D(VectorSubtract3D(Best.Intercept,Best.Triangle.P2));
D3:=VectorABS3D(VectorSubtract3D(Best.Intercept,Best.Triangle.P3));
if D1<D2 then Best.NearestVertex:= PickInteger(D1<D3,0,2)
else Best.NearestVertex:= PickInteger(D2<D3,1,2);
end;


{------------------------------------------------------------------------------}


procedure TGLFrame.DoRayMeshIntersect(X, Y: Integer; Pitch,Yaw,Roll: double; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
var Ray: T3DLine;
begin
glPushMatrix;
try
Mesh.DoPosition;
Mesh.DoRotations;
glScaleD(Mesh.Scale3D.X, Mesh.Scale3D.Y, Mesh.Scale3D.Z);
Ray:=ProjectGLScreenRay(X,Y,Pitch,Yaw,Roll);
RayMeshIntersect(Ray,Mesh,Best);
finally glPopMatrix; end;
end;



procedure TGLFrame.RayFrameIntersect(X, Y: Integer; Pitch,Yaw,Roll: double; Frame: TGLFrame; var Best: TTriangleIntersect);
{Search specified frame for the best X,Y/Triangle intercept}
var I: integer;
begin
if not Frame.HitTestEnabled then exit;
glPushMatrix;
try
Frame.DoPosition;
Frame.DoRotations;
glScaleD(Frame.Scale3D.X, Frame.Scale3D.Y, Frame.Scale3D.Z);
for I:=0 to Frame.MeshList.Count-1 do
 if Frame.MeshList[I].Enabled and Frame.MeshList[I].HitTestEnabled then
	DoRayMeshIntersect(X,Y,Pitch,Yaw,Roll,Frame.MeshList[I],Best);
for I:=0 to Frame.FrameList.Count-1 do RayFrameIntersect(X,Y,Pitch,Yaw,Roll,Frame.FrameList[I],Best);
finally glPopMatrix; end;
end;



procedure TGLFrame.FindNearestTriangle(X, Y: Integer; Pitch,Yaw,Roll: double; var  Best: TTriangleIntersect);
{Find the nearest trangle to ray project from specified point and specified angle}
{Information about the top most triangle is returned in Best}

	function GetTopLevelRay: T3DLine;
	{Get mouse ray for top level frame}
	begin
	glPushMatrix;
	try
	DoPosition;
	DoRotations;
	glScaleD(Scale3D.X, Scale3D.Y, Scale3D.Z);
	Result:=ProjectGLScreenRay(X,Y,Pitch,Yaw,Roll);
	finally glPopMatrix; end;
	end;

begin
Best.HitCount:=0;
Best.Valid:=False;
RayFrameIntersect(X,Y,Pitch,Yaw,Roll,Self,Best);
Best.Ray:=GetTopLevelRay;
end;

{------------------------------------------------------------------------------}


procedure TGLFrame.ScreenMeshIntersect(X,Y: integer; Mesh: TGLCustomMesh; var Best: TTriangleIntersect);
{Search specified Mesh for best Window-X,Y/Triangle intercept}
var Ray: T3DLine;
begin
glPushMatrix;
try
Mesh.DoPosition;
Mesh.DoRotations;
glScaleD(Mesh.Scale3D.X, Mesh.Scale3D.Y, Mesh.Scale3D.Z);
Ray:=GetGLMouseRay(X,Y);
RayMeshIntersect(Ray,Mesh,Best);
finally glPopMatrix; end;
end;



procedure TGLFrame.ScreenFrameIntersect(X,Y: integer; Frame: TGLFrame; var Best: TTriangleIntersect);
{Search specified frame for the best X,Y/Triangle intercept}
var I: integer;
begin
if not Frame.HitTestEnabled then exit;
glPushMatrix;
try
Frame.DoPosition;
Frame.DoRotations;
glScaled(Frame.Scale3D.X, Frame.Scale3D.Y, Frame.Scale3D.Z);
for I:=0 to Frame.MeshList.Count-1 do
 if Frame.MeshList[I].Enabled and Frame.MeshList[I].HitTestEnabled then
	ScreenMeshIntersect(X,Y,Frame.MeshList[I],Best);

for I:=0 to Frame.FastMeshList.Count-1 do
 if Frame.FastMeshList[I].Enabled and Frame.FastMeshList[I].HitTestEnabled then
	ScreenMeshIntersect(X,Y,Frame.FastMeshList[I],Best);

for I:=0 to Frame.FrameList.Count-1 do ScreenFrameIntersect(X,Y,Frame.FrameList[I],Best);
finally glPopMatrix; end;
end;



procedure TGLFrame.FindNearestTriangle(X,Y: integer; var  Best: TTriangleIntersect);
{Find the nearest trangle to mouse click point}
{Information about the top most triangle is returned in Best}

	function GetTopLevelRay: T3DLine;
	{Get mouse ray for top level frame}
	begin
	glPushMatrix;
	try
	DoPosition;
	DoRotations;
	glScaleD(Scale3D.X, Scale3D.Y, Scale3D.Z);
	Result:=GetGLMouseRay(X,Y);
	finally glPopMatrix; end;
	end;

begin
Best.HitCount:=0;
Best.Valid:=False;
ScreenFrameIntersect(X,Y,Self,Best);
if Best.Valid then Best.Ray:=GetTopLevelRay;
end;



{-----------------------------------------------------------------------------}


{ TGLFameList }

function TGLFrameList.Add(var Frame: TGLFrame): integer;
begin
Frame:=TGLFrame.Create(OnListChange);
Result:=inherited Add(Frame);
Frame.OnChange:=Self.OnChange;
Frame.Name:='Frame-'+IntToStr(Result+1);
Change;
end;


function TGLFrameList.Add: integer;
var Frame: TGLFrame;
begin
Result:=Add(Frame);
end;


procedure TGLFrameList.Assign(ObjectList: TGLObjectList);
var I: integer;
var O: TGLFrame;
begin
if ObjectList is TGLFrameList then
	begin
	Clear;
	for I:=0 to ObjectList.Count-1 do
		begin
		Add(O);
		O.Assign(TGLFrame(ObjectList[I]));
		end;
	end
else inherited Assign(ObjectList);
end;


function TGLFrameList.GetItem(Index: Integer): TGLFrame;
begin
Result := TGLFrame(inherited GetItem(Index));
end;



function TGLFrameList.GetTriangles: integer;
var I: integer;
begin
Result:=0;
for I:=0 to Count-1 do Result:=Result + Self[I].Triangles;
end;


procedure TGLFrameList.SetItem(Index: Integer; AObject: TGLFrame);
begin
inherited SetItem(Index, AObject);
end;


{------------------------------------------------------------------------------}



procedure TGLFrame.Assign(GLObject: TGLObject);
begin
inherited Assign(GLObject);
if GLObject is TGLFrame then
	begin
	MeshList.Assign(TGLFrame(GLObject).MeshList);
	FastMeshList.Assign(TGLFrame(GLObject).FastMeshList);
	StringList3D.Assign(TGLFrame(GLObject).StringList3D);
	StringList2D.Assign(TGLFrame(GLObject).StringList2D);
	LineList.Assign(TGLFrame(GLObject).LineList);
	PointList.Assign(TGLFrame(GLObject).PointList);
	FrameList.Assign(TGLFrame(GLObject).FrameList);
	end;
end;




procedure TGLFrame.Clear;
begin
MeshList.Clear;
FastMeshList.Clear;
StringList3D.Clear;
StringList2D.Clear;
LineList.Clear;
PointList.Clear;
FrameList.Clear;
end;




function TGLFrame.GetTriangles: integer;
begin
Result:=MeshList.Triangles;
Result:=Result+FastMeshList.Triangles;
Result:=Result+FrameList.Triangles;
end;




procedure TGLFrame.FillObjectList(List: TGLObjectList);
var I: integer;
begin
for I:=0 to MeshList.Count-1 do List.Add(MeshList[I],MeshList);
for I:=0 to FastMeshList.Count-1 do List.Add(FastMeshList[I],FastMeshList);
for I:=0 to StringList3D.Count-1 do List.Add(StringList3D[I],StringList3D);
for I:=0 to StringList2D.Count-1 do List.Add(StringList2D[I],StringList2D);
for I:=0 to LineList.Count-1 do List.Add(LineList[I],LineList);
for I:=0 to PointList.Count-1 do List.Add(PointList[I],PointList);
for I:=0 to FrameList.Count-1 do
	begin
	List.Add(FrameList[I],FrameList);
	FrameList[I].FillObjectList(List);
	end;
end;



{ TGLBaseFrame }

procedure TGLBaseFrame.Assign(GLObject: TGLObject);
begin
inherited Assign(GLObject);
{Master list is rebuilt as new object are added}
end;




constructor TGLBaseFrame.Create;
begin
inherited Create(MasterListhandler);
MasterList:=TGLObjectList.Create;
{Must be false or objects will get destroyed twice}
MasterList.OwnsObjects:=False;
end;



destructor TGLBaseFrame.Destroy;
begin
inherited;
{Events in underlying object effect masterlist so free it last}
MasterList.Free;
end;



procedure TGLBaseFrame.MasterListhandler(GLObject: TGLObject;
  Action: TListNotification);
begin
MasterList.Clear;
FillObjectList(MasterList);

Change;
end;







end.
