unit GLMappingLib;

interface

uses Vector;

function MapCubeToSphere(V: T3DVector): T3DVector;  overload;
function MapCubeToSphere(V,Min,Max: T3DVector): T3DVector;  overload;

function MapSquareToCircle(V: T2DVector): T2DVector;  overload;
function MapSquareToCircle(V,Min,Max: T2DVector): T2DVector;  overload;


implementation


{------------------- Sphere mapping routines ---------------------------------}


{Note: untested routines }

function MapCubeToSphere(V: T3DVector): T3DVector;  overload;
{Maps a point on Cube in the range of -1 to +1 to a unit Sphere}
begin
V.X:=V.X * Sqrt(1 - Sqr(V.Y)/2 - Sqr(V.Z)/2 + (Sqr(V.Y) * Sqr(V.Z))/3);
V.Y:=V.Y * Sqrt(1 - Sqr(V.Z)/2 - Sqr(V.X)/2 + (Sqr(V.Z) * Sqr(V.X))/3);
V.Z:=V.Z * Sqrt(1 - Sqr(V.X)/2 - Sqr(V.Y)/2 + (Sqr(V.X) * Sqr(V.Y))/3);
end;



function MapCubeToSphere(V,Min,Max: T3DVector): T3DVector;  overload;
{Scales values to any size prism to cube and maps to equivalent elipsoid}
var Size,Center,T: T3DVector;
begin
Center:=ScalarDivide3D(VectorAdd3D(Max,Min),2);
Size:=ScalarDivide3D(VectorSubtract3D(Max,Min),2);
T:=VectorSubtract3D(V,Center);
T:=VectorDivide3D(T,Size);
Result:=MapCubeToSphere(T);
Result:=VectorMultiply3D(Result,Size);
Result:=VectorAdd3D(Result,Center);
end;



{------------------- Circle mapping routines ---------------------------------}



function MapSquareToCircle(V: T2DVector): T2DVector;  overload;
{Maps a point on square in the range of -1 to +1 to a unit circle}
begin
Result.X:=V.X*Sqrt(1-Sqr(V.Y)/2);
Result.Y:=V.Y*Sqrt(1-Sqr(V.X)/2);
end;



function MapSquareToCircle(V,Min,Max: T2DVector): T2DVector;  overload;
{Scales values to any size rectange to square and maps to equivalent elipse}
var Size,Center,T: T2DVector;
begin
Center:=ScalarDivide2D(VectorAdd2D(Max,Min),2);
Size:=ScalarDivide2D(VectorSubtract2D(Max,Min),2);
T:=VectorSubtract2D(V,Center);
T:=VectorDivide2D(T,Size);
Result:=MapSquareToCircle(T);
Result:=VectorMultiply2D(Result,Size);
Result:=VectorAdd2D(Result,Center);
end;

end.
