unit GLMesh;

{This unit hold the TGLMesh object which handles the creation, memory
allocation, normal generation, and rendering of a mesh of triangles.}

interface

uses Windows, Classes, SysUtils, Graphics, Contnrs, OpenGL15, Vector, Math, HugeArry, GLMatrix,
     GLQuatLib, GLLibrary, GLOctTree, GLRenderer, GLColorArray, GLTexture, Hash,GLCustomMesh;


{Mesh object declaration}

type TGLMesh = class(TGLCustomMesh)
 private
  FNormalLength: double;
  Sorter: TList;
  function SortedVertexInx(Inx: integer): integer;
  function GetSortVertex(Inx: integer): T3DVector;
  procedure GetVertexColors(I: integer; var C1,C2,C3: TRGBA);
  procedure SetFaceColor(I: integer);
  procedure SetFlatTriangle(I: integer);
  procedure SetSmoothTriangle(I: integer);
  procedure SortAllVertices;
  function  FindVertex(const V: T3DVector; var Index: Integer): boolean;
  procedure SetFaceNormals;
  function CalculateVertexNormal(Inx: integer; CreaseAngle: double): T3DVector;
  procedure SetVertexNormals(CreaseAngle: double);
  procedure SetWireTriangle(I: integer);
  procedure DrawWireTriangle(Inx,Cell: integer);
  procedure DrawFlatTriangle(Inx,Cell: integer);
  procedure DrawSmoothTriangle(Inx,Cell: integer);
  procedure DoWireTriangles;
  procedure DoFlatTriangles;
  procedure DoSmoothTriangles;
  procedure SetMinMax(V: T3DVector);
  procedure SetupOctaltree;
  procedure PrepareMesh;
  procedure RenderNormal;
  procedure HandleTextureChange(Sender: TObject);
  procedure HandleVertexChange(V: T3DVector; Index: integer);
  procedure DisplayNormals;
  procedure DrawNormalLine(Inx, Cell: integer);
  procedure ConvertAndAppend(Destination, Source: THuge3DVector; Matrix: TGLMatrix);
  procedure MatrixMultiplyVertices(Vertices: THuge3DVector; Matrix: TGLMatrix);
  function VertexToTriangle(I: integer): TTriangle;
  procedure HandleVertexAdd(Value: T3DVector; Index: integer);
  procedure HandleVertexDelete(Value: T3DVector; Index: integer);
  procedure HandleDeleteTriangle(Value: T3DVector; Index: integer);
  procedure FindAdjacentTriangles(Target: T3DVector; var TA: TIntegerHashTable);
  function IndexToTriangle(Inx: integer): TTriangle;
  procedure DisplayFaceNormals;
  procedure DrawFaceNormalLine(Inx, Cell: integer);
    function GetExtentMax: T3DVector;
    function GetExtentMin: T3DVector;
    procedure SetExtentMax(const Value: T3DVector);
    procedure SetExtentMin(const Value: T3DVector);
 protected
  procedure Change; override;
  function GetFaceCount: integer; override;
  function GetFaceNormal(Index: integer): T3DVector; override;
  procedure SetFaceNormal(Index: integer; const Value: T3DVector); override;
  function GetFaceTriangles(Index: integer): TTriangle; override;
  procedure SetFaceTriangles(Index: integer; const Value: TTriangle); override;
  function GetGLColor(Index: integer): TGLFaceColor; override;
  procedure SetGLColor(Index: integer; const Value: TGLFaceColor); override;
  procedure SetRenderMode(const Value: TRenderMode); override;
  procedure SetTransparency(const Value: double); override;
  function GetVertex(Index: integer): T3DVector; override;
  procedure SetVertex(Index: integer; const Value: T3DVector); override;
  function GetVertexColor(Index: integer): TColor; override;
  procedure SetVertexColor(Index: integer; const Value: TColor); override;
  function GetVertexCount: integer; override;
  procedure SetVertexCount(const Value: integer); override;
  function GetVertexNormal(Index: integer): T3DVector; override;
  procedure SetVertexNormal(Index: integer; const Value: T3DVector); override;
  function GetFaceIndices(Index: integer): TFace; override;
  procedure SetFaceIndices(Index: integer; const Value: TFace); override;
 public
  Vertices: THuge3DVectorEx;
  FaceNormals: THuge3DVector;
  VertexNormals: THuge3DVector;
  FaceColor: TGLColorArray;
  VerticesSorted: boolean;
  constructor Create; override;
  destructor Destroy; override;
  procedure Assign(GLObject: TGLObject); override;
  procedure Append(GLObject: TGLObject); override;
  procedure PrepareTexture; override;
  procedure CalculateMinMax; override;

  procedure SetAllNormals(DoVertex: boolean); overload; override;
  procedure SetAllNormals; overload; override;
  procedure SetAllNormals(DoVertex: boolean; CreaseAngle: double); overload; override;
  procedure SetAllNormals(CreaseAngle: double); overload; override;

  procedure SetAdjacentNormals(Target: T3DVector; CreaseAngle: double); override;
  procedure RemoveDegenerateTriangles; override;
  procedure RemoveAdjacentDegenerates(Target: T3DVector); override;
  function RemoveTriangle(I: integer): T3DVector; override;
  procedure FindAdjacentEdge(L: T3DLine; Inx: integer; var Edge: TEdgeInfo); override;

  function AddTriangle(T: TTriangle; GLC: TGLColor): integer; overload; override;
  function AddTriangle(T: TTriangle; C: TColor): integer; overload; override;
  function AddTriangle(P1,P2,P3: T3DVector; C: TColor): integer; overload; override;
  function AddTriangle(T: TTriangle; C1, C2, C3: TColor): integer; overload;
  procedure DeleteTriangle(Index: integer); override;

  procedure AddRectangle(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double; C:TColor); overload; override;
  procedure AddRectangle(P1,P2,P3,P4: T3DVector; C1, C2, C3, C4: TColor); overload;
  procedure AddRectangle(P1,P2: T3DVector; C: TColor); overload; override;
  procedure AddRectangle(P1,P2,P3,P4: T3DVector; C: TColor); overload; override;
  procedure AddRectangle(R: T3DRectangle; C: TColor); overload; override;
  procedure AddPrism(P1,P2,P3,P4,P5,P6,P7,P8: T3DVector; C: TColor); overload; override;
  procedure AddPrism(Min,Max: T3DVector; C: TColor); overload; override;
  procedure AddPrism(Min,Max: T3DVector; XCount,YCount,ZCount: integer; C: TColor); overload; override;
  procedure AddHexagon(P1, P2, P3, P4, P5, P6: T3DVector; C: TColor); override;
  procedure AddPentagon(P1, P2, P3, P4, P5: T3DVector; C: TColor); override;
  procedure NormalizeScale; override;
  procedure CenterMesh; override;
  procedure MeshClear; override;

  procedure Clear; override;
  procedure ClearData; override;
  procedure Render; override;
  procedure SetColor(const Color: TColor); override;
  function GetColor: TColor; override;
  procedure SetFaceColors(Inx: integer; C1,C2,C3: TColor); override;
  function FaceToTriangle(FaceInx: integer): TTriangle; override;
  procedure FindDuplicateVertices(Target: T3DVector; var VA: TIntegerHashTable); override;
  function FindFirstDuplicate(Target: T3DVector): integer; override;
  procedure ChangeAllDuplicateVertices(Old,New: T3DVector); override;
  procedure FindAdjacentVertices(Old: T3DVector; var VA: TIntegerHashTable); override;

  property RenderMode;
  property OnChange;
  property OnNormalProgress;
  property ShowNormals;
  property NormalColor;
  property ShowFaceNormals;
  property FaceNormalColor;
  property Vertex;
  property VertexColor;
  property VertexCount;
  property VertexNormal;
  property FaceCount;
  property FaceTriangles;
  property FaceColors;
  property FaceNormal;
  property ExtentMin: T3DVector read GetExtentMin write SetExtentMin;
  property ExtentMax: T3DVector read GetExtentMax write SetExtentMax;


 end;


var CurrentInstance: TGLMesh;	{Kluge variable to access object variables}

{ TMeshList }


type  TGLMeshList = class(TGLObjectList)
  private
    function GetTriangles: integer;
  protected
    function GetItem(Index: Integer): TGLMesh;
    procedure SetItem(Index: Integer; AObject: TGLMesh);
  public
    function Add(var Mesh: TGLMesh): integer; overload;
    function Add: integer; overload;
    function Insert(Inx: integer; var Mesh: TGLMesh): integer; overload;
    function Insert(Inx: integer): integer; overload;
    property Items[Index: Integer]: TGLMesh read GetItem write SetItem; default;
    procedure Assign(ObjectList: TObjectList);
    property Triangles: integer read GetTriangles;
  end;

function GetEdgePoints(Edge: integer; Tri: TTriangle): T3DLine;

function VectorCompare(V1,V2: T3DVector): integer;
function VectorsEqual(V1,V2: T3DVector): boolean;


implementation

uses GLFrame;


{------------------ Support Routines --------------------------------------}


function TGLMesh.VertexToTriangle(I: integer): TTriangle;
{Extract triangle from vertex array at I}
begin
Result:=TTriangle(Vertices.GetItemPtr(I)^);
end;



function TGLMesh.IndexToTriangle(Inx: integer): TTriangle;
{Find the triangle that corresponds to a vertex index}
{In other words, find the nearest triangle to a vertex index}
var I2: integer;
begin
I2:=Inx - (Inx mod 3);
Result:=VertexToTriangle(I2);
end;




procedure TGLMesh.Change;
begin
inherited;
MeshBuilt:=False;
TextureBuilt:=False;
VerticesSorted:=False;
if Assigned(FOnChange) then FOnChange(Self);
end;


{------------------ Property Setting Routines ---------------------------------}



procedure TGLMesh.SetRenderMode(const Value: TRenderMode);
{Set current render mode}
begin
if FRenderMode<>Value then MeshBuilt:=False;
Inherited ;
end;


procedure TGLMesh.SetTransparency(const Value: double);
{Set current render mode}
begin
if FaceColor<>nil then FaceColor.SetTransparency((100-Value)/100);
Inherited;
end;



{------------------ Sorting and Searching Routines ------------------------}


function TGLMesh.SortedVertexInx(Inx: integer): integer;
{Get the index of the sorted vertex at Inx position}
begin
Result:=Integer(Sorter.Items[Inx]);
end;


function TGLMesh.GetSortVertex(Inx: integer): T3DVector;
{Get the sorted vertex at the Inx position}
begin
Result:=Vertices.Value[SortedVertexInx(Inx)];
end;


function NotEquals(V1,V2: double): boolean;
begin
Result:=Abs(V1-V2)>0.00000000001;
end;


function VectorCompare(V1,V2: T3DVector): integer;
{Compare 3D vectors. Returns 0 if equal, <0 if }
{Returns negative if V1<V2 and positive if V1>V2}
begin
if NotEquals(V1.X,V2.X) then Result:=Math.Sign(V1.X-V2.X)
else if NotEquals(V1.Y,V2.Y) then Result:=Math.Sign(V1.Y-V2.Y)
else if NotEquals(V1.Z,V2.Z) then Result:=Math.Sign(V1.Z-V2.Z)
else Result:=0;
end;


function VectorsEqual(V1,V2: T3DVector): boolean;
{Returns true if vectors are equal}
begin
Result:=VectorCompare(V1,V2)=0;
end;


function SortCompare(P1,P2: Pointer): integer;
{Function is called by the Sorter to the list of vectors}
{Note the "CurrentInstance" kluge to access }
var V1,V2: T3DVector;
begin
V1:=CurrentInstance.Vertices.Value[Integer(P1)];
V2:=CurrentInstance.Vertices.Value[Integer(P2)];
Result:=VectorCompare(V1,V2);
end;



procedure TGLMesh.SortAllVertices;
{Subroutine sorts all vectors}
var I: integer;
begin
if VerticesSorted then exit;
Sorter.Clear;
for I:=0 to Vertices.ItemIndex-1 do Sorter.Add(Pointer(I));
{Must set current instance so sort can access object variables}
CurrentInstance:=Self;
Sorter.Sort(SortCompare);
VerticesSorted:=True;
end;



function  TGLMesh.FindVertex(const V: T3DVector; var Index: Integer): boolean;
{Subroutine finds the first match vertex in the sorted list}
var  L, H, I, C: Integer;
begin
Result := False;
L := 0;
H := Vertices.ItemIndex - 1;
while L <= H do
	begin
	I := (L + H) shr 1;
	C := VectorCompare(GetSortVertex(I), V);
	if C < 0 then L := I + 1 else
		begin
		H := I - 1;
		if C = 0 then Result := True;
		end;
	end;
Index := L;
end;

{------------------Routines To Set Normals -------------------------------}




procedure TGLMesh.SetFaceNormals;
{Sets all face normals}
var I: integer;
var T: TTriangle;
begin
FaceNormals.Clear;
for I:=0 to (Vertices.ItemIndex div 3)-1 do
	begin
	T:=Vertices.Triangle[I];
	FaceNormals.Add(GetTriangleNormal(T));
	DisplayProgress('Face Normals',I,Vertices.ItemIndex div 3);
	end;
end;




function TGLMesh.CalculateVertexNormal(Inx: integer; CreaseAngle: double): T3DVector;
{Returns the normal for a vertex specified by Inx}
{Calculated by averaging the face normals for all}
{faces adjacent to the specified vertex}
var V,BaseNormal,Normal: T3DVector;
var T: TTriangle;
var I,Start,TInx,Count: integer;
var Angle: double;
begin
Result:=MakeVector3D(0,0,0);
{Get the value of the specified vertex}
V:=Vertices.Value[Inx];
BaseNormal:=GetTriangleNormal(IndexToTriangle(Inx));
{Find the first matching vertex in the sorted list}
FindVertex(V,Start);
Count:=0;
{Find all matching vertices}
for I:=Start to Sorter.Count-1 do
	begin
	{We are done with the list if not matching}
	if (VectorCompare(V,GetSortVertex(I))<>0) then break;
	{Added to average sum}
	TInx:=SortedVertexInx(I);
	T:=IndexToTriangle(TInx);
	Normal:=GetTriangleNormal(T);
	Angle:=RadToDeg(VectorAngle3D(BaseNormal,Normal));
	if Angle<CreaseAngle then Result:=VectorAdd3D(Result,Normal);
	Inc(Count);
	end;
{Calculate average}
if Count<>0 then Result:=ScalarDivide3D(Result,Count);
end;




procedure TGLMesh.SetVertexNormals(CreaseAngle: double);
{Build vertex normal list}
var V: T3DVector;
var I: integer;
begin
VertexNormals.Clear;
for I:=0 to Vertices.ItemIndex-1 do
	begin
	V:=CalculateVertexNormal(I,CreaseAngle);
	VertexNormals.Add(V);
	DisplayProgress('Vertex Normals',I,Vertices.ItemIndex);
	end;
end;





procedure TGLMesh.SetAllNormals(DoVertex: boolean; CreaseAngle: double);
begin
DisplayProgress('Setting Face Normals',0,1);
SetFaceNormals;
if DoVertex then
	begin
	DisplayProgress('Sorting Vertices',5,10);
	SortAllVertices;
	DisplayProgress('Setting Vertices Normals',7,10);
	SetVertexNormals(CreaseAngle);
	end;
DisplayProgress('Done',1,1);
end;



procedure TGLMesh.SetAdjacentNormals(Target: T3DVector; CreaseAngle: double);
{Set normals for triangles that are immediately adjacent to Target}
var I,J,TInx,VInx: integer;
var VA: TIntegerHashTable;
var T: TTriangle;
begin
VA:=TIntegerHashTable.Create(5);
try
FindAdjacentTriangles(Target,VA);
for I:=0 to VA.Count-1 do
	begin
	TInx:=VA[I];
	VInx:=TInx*3;
	T:=Vertices.Triangle[TInx];
	FaceNormals[TInx]:=GetTriangleNormal(T);
	for J:=0 to 2 do VertexNormals[VInx+J]:=CalculateVertexNormal(VInx+J,CreaseAngle);
	end;
Change;
finally VA.Free; end;
end;




procedure TGLMesh.SetAllNormals(DoVertex: boolean);
begin
SetAllNormals(DoVertex,89);
end;



procedure TGLMesh.SetAllNormals;
begin
SetAllNormals(True);
end;



procedure TGLMesh.SetAllNormals(CreaseAngle: double);
begin
SetAllNormals(True,CreaseAngle);
end;



{---------------- Object Event handlers ----------------------------------}


procedure TGLMesh.HandleTextureChange(Sender: TObject);
begin
TextureBuilt:=False;
end;


procedure TGLMesh.HandleVertexChange(V: T3DVector; Index: integer);
begin
SetMinMax(V);
Change;
end;




procedure TGLMesh.HandleVertexDelete(Value: T3DVector; Index: integer);
begin
VertexNormals.Delete(Index);
end;



procedure TGLMesh.HandleVertexAdd(Value: T3DVector; Index: integer);
begin
VertexNormals.Add(MakeVector3D(0,0,0));
end;



procedure TGLMesh.HandleDeleteTriangle(Value: T3DVector; Index: integer);
begin
Self.FaceNormals.Delete(Index);
Self.FaceColor.Delete(Index);
end;



constructor TGLMesh.Create;
begin
inherited;
Vertices:=THuge3DVectorEx.CreateAsTriangles(5);
FaceNormals:=THuge3DVector.Create(5);
VertexNormals:=THuge3DVector.Create(5);
FaceColor:=TGLColorArray.Create(5);
Sorter:=TList.Create;
FRenderMode:=rmFlatFace;
VerticesSorted:=False;
Position:=DefaultPos;
ResetPos:=DefaultPos;
Texture.OnChange:=HandleTextureChange;
Vertices.OnChange:=HandleVertexChange;
Vertices.OnDelete:=HandleVertexDelete;
Vertices.OnAdd:=HandleVertexAdd;
Vertices.OnDeleteTriangle:=HandleDeleteTriangle;
Reset;
end;



destructor TGLMesh.Destroy;
begin
Vertices.Free;
FaceNormals.Free;
VertexNormals.Free;
FaceColor.Free;
Sorter.Free;
inherited Destroy;
end;


{---------------- Routines to add faces to the mesh ---------------------------}


procedure TGLMesh.SetMinMax(V: T3DVector);
{Set min/max from of mesh}
begin
if Vertices.ItemIndex=0 then
	begin
	ExtentMin:=V;
	ExtentMax:=V;
	end
else
	begin
	ExtentMin:=GetMinimumVector(ExtentMin,V);
	ExtentMax:=GetMaximumVector(ExtentMax,V);
	end;
end;


procedure TGLMesh.CalculateMinMax;
var I: integer;
var V: T3DVector;
begin
ExtentMin:=MakeVector3D(MaxDouble,MaxDouble,MaxDouble);
ExtentMax:=MakeVector3D(-MaxDouble,-MaxDouble,-MaxDouble);
for I:=0 to Vertices.Count-1 do
	begin
	V:=Vertices[I];
	ExtentMin:=GetMinimumVector(ExtentMin,V);
	ExtentMax:=GetMaximumVector(ExtentMax,V);
	end;
end;


function TGLMesh.AddTriangle(T: TTriangle; C: TColor): integer;
{Add a triangle to the mesh}
begin
Result:=Vertices.AddTriangle(T);
FaceColor.Add(C);
FaceNormals.Add(GetTriangleNormal(T));
Change;
end;



function TGLMesh.AddTriangle(T: TTriangle; C1, C2, C3: TColor): integer;
{Add a triangle to the mesh}
begin
Result:=Vertices.AddTriangle(T);
FaceColor.Add(C1,C2,C3);
FaceNormals.Add(GetTriangleNormal(T));
Change;
end;




function TGLMesh.AddTriangle(T: TTriangle; GLC: TGLColor): integer;
{Add a triangle to the mesh}
var C: TColor;
begin
C:=GLColorToColor(GLC);
Result:=AddTriangle(T,C);
end;



function TGLMesh.AddTriangle(P1,P2,P3: T3DVector; C: TColor): integer;
{Add a triangle to the mesh}
var T: TTriangle;
begin
T.P1:=P1; T.P2:=P2; T.P3:=P3;
Result:=AddTriangle(T,C);
end;



procedure TGLMesh.AddRectangle(P1,P2,P3,P4: T3DVector; C: TColor);
{Add a rectangle to the mesh}
{assumes 1..4 are counter clockwise facing face}
var T: TTriangle;
begin
T.P1:=P1; T.P2:=P2; T.P3:=P4;
AddTriangle(T,C);
T.P1:=P2; T.P2:=P3; T.P3:=P4;
AddTriangle(T,C);
end;



procedure TGLMesh.AddRectangle(R: T3DRectangle; C: TColor);
{Add a rectangle to the mesh}
{assumes 1..4 are counter clockwise facing face}
begin
AddRectangle(R.P1,R.P2,R.P3,R.P4,C);
end;



procedure TGLMesh.AddRectangle(P1,P2: T3DVector; C: TColor);
{Add a rectangle to the mesh}
var P3,P4: T3DVector;
begin
P3:=P2;
P2.Y:=P1.Y;
P4:=P1;
P4.Y:=P3.Y;
AddRectangle(P1,P2,P3,P4,C);
end;


procedure TGLMesh.AddRectangle(X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3,X4,Y4,Z4: double; C:TColor);
{Build rectangle based on individual points}
{assumes 1..4 are counter clockwise facing face}
var P1,P2,P3,P4: T3DVector;
begin
P1:=MakeVector3D(X1,Y1,Z1);
P2:=MakeVector3D(X2,Y2,Z2);
P3:=MakeVector3D(X3,Y3,Z3);
P4:=MakeVector3D(X4,Y4,Z4);
AddRectangle(P1,P2,P3,P4,C);
end;




procedure TGLMesh.AddRectangle(P1,P2,P3,P4: T3DVector; C1, C2, C3, C4: TColor);
{Add a rectangle to the mesh}
{assumes 1..4 are counter clockwise facing face}
var T: TTriangle;
begin
T.P1:=P1; T.P2:=P2; T.P3:=P4;
AddTriangle(T,C1,C2,C4);
T.P1:=P2; T.P2:=P3; T.P3:=P4;
AddTriangle(T,C2,C3,C4);
end;



procedure TGLMesh.AddHexagon(P1,P2,P3,P4,P5,P6: T3DVector; C: TColor);
var Cnt: T3DVector;
begin
Cnt:=VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(P1,P2),P3),P4),P5),P6);
Cnt:=ScalarDivide3D(Cnt,6);
AddTriangle(Cnt,P1,P2,C);
AddTriangle(Cnt,P2,P3,C);
AddTriangle(Cnt,P3,P4,C);
AddTriangle(Cnt,P4,P5,C);
AddTriangle(Cnt,P5,P6,C);
AddTriangle(Cnt,P6,P1,C);
end;



procedure TGLMesh.AddPentagon(P1,P2,P3,P4,P5: T3DVector; C: TColor);
var Cnt: T3DVector;
begin
Cnt:=VectorAdd3D(VectorAdd3D(VectorAdd3D(VectorAdd3D(P1,P2),P3),P4),P5);
Cnt:=ScalarDivide3D(Cnt,5);
AddTriangle(Cnt,P1,P2,C);
AddTriangle(Cnt,P2,P3,C);
AddTriangle(Cnt,P3,P4,C);
AddTriangle(Cnt,P4,P5,C);
AddTriangle(Cnt,P5,P1,C);
end;



procedure TGLMesh.AddPrism(P1,P2,P3,P4,P5,P6,P7,P8: T3DVector; C: TColor);
{Add prism based on points, assumes 1..4 are counter clockwise facing one face }
{ and 5..8 are counter clockwise facing the oposite face}
begin
AddRectangle(P1,P2,P3,P4,C);
AddRectangle(P5,P6,P7,P8,C);
AddRectangle(P1,P6,P5,P2,C);
AddRectangle(P2,P5,P8,P3,C);
AddRectangle(P4,P7,P6,P1,C);
AddRectangle(P4,P3,P8,P7,C);
end;



procedure TGLMesh.AddPrism(Min,Max: T3DVector; C: TColor);
var P1,P2,P3,P4,P5,P6,P7,P8: T3DVector;
begin
P1:=MakeVector3D(Min.X,Min.Y,Max.Z);
P2:=MakeVector3D(Max.X,Min.Y,Max.Z);
P3:=MakeVector3D(Max.X,Max.Y,Max.Z);
P4:=MakeVector3D(Min.X,Max.Y,Max.Z);

P5:=MakeVector3D(Max.X,Min.Y,Min.Z);
P6:=MakeVector3D(Min.X,Min.Y,Min.Z);
P7:=MakeVector3D(Min.X,Max.Y,Min.Z);
P8:=MakeVector3D(Max.X,Max.Y,Min.Z);

AddPrism(P1,P2,P3,P4,P5,P6,P7,P8,C);
end;




procedure TGLMesh.AddPrism(Min,Max: T3DVector; XCount,YCount,ZCount: integer; C: TColor);
var X,Y,Z: integer;
var P1,P2,Step: T3DVector;
begin
Step:=VectorDivide3D(VectorSubtract3D(Max,Min),MakeVector3D(XCount,YCount,ZCount));
for Z:=0 to ZCount-1 do
 for Y:=0 to YCount-1 do
  for X:=0 to XCount-1 do
	begin
	P1:=VectorAdd3D(Min,VectorMultiply3D(MakeVector3D(X,Y,Z),Step));
	P2:=VectorAdd3D(Min,VectorMultiply3D(MakeVector3D(X+1,Y+1,Z+1),Step));
	AddPrism(P1,P2,C);
	end;
end;


procedure TGLMesh.MeshClear;
{Clear Mesh only leaving position, rotation and scaling}
begin
Vertices.Clear;
FaceNormals.Clear;
VertexNormals.Clear;
FaceColor.Clear;
Sorter.Clear;
Change;
ExtentMin:=MakeVector3D(0,0,0);
ExtentMax:=MakeVector3D(0,0,0);
end;



procedure TGLMesh.Clear;
{Clear the mesh}
begin
inherited Clear;
MeshClear;
Change;
end;


procedure TGLMesh.ClearData;
{Clear data without resetting transforms}
begin
MeshClear;
end;


{-------------- Routines To Render Mesh --------------------------------------}



procedure TGLMesh.SetFaceColor(I: integer);
{Tell OpenGL about the color of specified face}
var R,G,B,A: single;
begin
GetRGB(FaceColor.Value[I].C1,R,G,B);
A:=GetCurrentAlpha;
glColor4f(R,G,B,A);
end;



procedure TGLMesh.GetVertexColors(I: integer; var C1,C2,C3: TRGBA);
{Get color for each of three triangle vertices}
var A: single;
var GLC: TGLFaceColor;
begin
A:=GetCurrentAlpha;
C1.A:=A;C2.A:=A;C3.A:=A;
GLC:=(FaceColor.Value[I]);
GetRGB(GLC.C1,C1.R,C1.G,C1.B);
GetRGB(GLC.C2,C2.R,C2.G,C2.B);
GetRGB(GLC.C3,C3.R,C3.G,C3.B);
end;



procedure TGLMesh.SetFlatTriangle(I: integer);
{Tell OpenGL about a face that will be rendered flat}
var T: TTriangle;
var VI: integer;
begin
VI:=I * 3;
glNormal3dv(FaceNormals.GetItemPtr(I));
T:=Vertices.Triangle[I];
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI].X, Texture.UVCoordinates[VI].Y);
glVertex3dv(@T.P1);
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI+1].X, Texture.UVCoordinates[VI+1].Y);
glVertex3dv(@T.P2);
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI+2].X, Texture.UVCoordinates[VI+2].Y);
glVertex3dv(@T.P3);
end;



procedure TGLMesh.SetWireTriangle(I: integer);
{Tell OpenGL about a face that will be rendered as a wire frame}
var T: TTriangle;
begin
T:=Vertices.Triangle[I];
glBegin(GL_LINE_LOOP);
glVertex3dv(@T.P1); glVertex3dv(@T.P2); glVertex3dv(@T.P3);
glEnd;
end;



procedure TGLMesh.SetSmoothTriangle(I: integer);
{Tell OpenGL about a face that will be rendered smooth}
var T: TTriangle;
var VI: integer;
var C1,C2,C3: TRGBA;
begin
GetVertexColors(I,C1,C2,C3);
VI:=I * 3;
T:=Vertices.Triangle[I];
glNormal3dv(VertexNormals.GetItemPtr(VI));
glColor4f(C1.R,C1.G,C1.B,C1.A);
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI].X, Texture.UVCoordinates[VI].Y);
glVertex3dv(@T.P1);

glNormal3dv(VertexNormals.GetItemPtr(VI+1));
glColor4f(C2.R,C2.G,C2.B,C2.A);
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI+1].X, Texture.UVCoordinates[VI+1].Y);
glVertex3dv(@T.P2);

glNormal3dv(VertexNormals.GetItemPtr(VI+2));
glColor4f(C3.R,C3.G,C3.B,C3.A);
if Texture.TextureMode=tmManual then glTexCoord2f(Texture.UVCoordinates[VI+2].X, Texture.UVCoordinates[VI+2].Y);
glVertex3dv(@T.P3);
end;



{-----------------------------------------------------------------------------}


procedure TGLMesh.DrawSmoothTriangle(Inx,Cell: integer);
begin
SetFaceColor(Inx);
SetSmoothTriangle(Inx);
end;




procedure TGLMesh.DoSmoothTriangles;
{Render whole mesh a smooth triangles}
begin
GLShadeModel(GL_SMOOTH);
glBegin(GL_TRIANGLES);
OctalTree.IterateFaces(DrawSmoothTriangle);
glEnd;
end;

{-----------------------------------------------------------------------------}

procedure TGLMesh.DrawFlatTriangle(Inx,Cell: integer);
begin
SetFaceColor(Inx);
SetFlatTriangle(Inx);
end;



procedure TGLMesh.DoFlatTriangles;
{Render whole mesh as flat triangles}
begin
GLShadeModel(GL_FLAT);
glBegin(GL_TRIANGLES);
OctalTree.IterateFaces(DrawFlatTriangle);
glEnd;
end;

{-----------------------------------------------------------------------------}



procedure TGLMesh.DrawWireTriangle(Inx,Cell: integer);
begin
SetFaceColor(Inx);
SetWireTriangle(Inx);
end;


procedure TGLMesh.DoWireTriangles;
{Render whole mesh as wire frame}
begin
GLShadeModel(GL_FLAT);
OctalTree.IterateFaces(DrawWireTriangle);
{glEnd not needed here}
end;



{-----------------------------------------------------------------------------}


procedure TGLMesh.DrawNormalLine(Inx,Cell: integer);
var T,N: TTriangle;
var VI: integer;
begin
T:=Vertices.Triangle[Inx];
VI:=Inx * 3;
N.P1:=VectorAdd3D(T.P1,ScalarProduct3D(VertexNormals[VI],FNormalLength));
N.P2:=VectorAdd3D(T.P2,ScalarProduct3D(VertexNormals[VI+1],FNormalLength));
N.P3:=VectorAdd3D(T.P3,ScalarProduct3D(VertexNormals[VI+2],FNormalLength));

glBegin(GL_LINES);
glVertex3dv(@T.P1); glVertex3dv(@N.P1);
glVertex3dv(@T.P2); glVertex3dv(@N.P2);
glVertex3dv(@T.P3); glVertex3dv(@N.P3);
glEnd;
end;




procedure TGLMesh.DisplayNormals;
{Handle displaying normals as lines}
var R,G,B: single;
begin
FNormalLength:=VectorABS3D(VectorSubtract3D(ExtentMax,ExtentMin));
FNormalLength:=FNormalLength * 0.1;
GetRGB(NormalColor,R,G,B);
glColor4f(R,G,B,1);
glDisable(GL_LIGHTING);
OctalTree.IterateLines(DrawNormalLine);
{glEnd not needed here}

end;

{-----------------------------------------------------------------------------}



procedure TGLMesh.DrawFaceNormalLine(Inx,Cell: integer);
var PC,PN: T3DVector;
var T: TTriangle;
begin
T:=Vertices.Triangle[Inx];
PC:=ScalarDivide3D(VectorAdd3D(VectorAdd3D(T.P1,T.P2),T.P3),3);

PN:=FaceNormals[Inx];
PN:=VectorAdd3D(PC,ScalarProduct3D(PN,FNormalLength));
glBegin(GL_LINES);
glVertex3dv(@PC); glVertex3dv(@PN);
glEnd;
end;



procedure TGLMesh.DisplayFaceNormals;
{Handle displaying normals as lines}
var R,G,B: single;
var I: integer;
begin
FNormalLength:=VectorABS3D(VectorSubtract3D(ExtentMax,ExtentMin));
FNormalLength:=FNormalLength * 0.1;
GetRGB(FaceNormalColor,R,G,B);
glColor4f(R,G,B,1);
glDisable(GL_LIGHTING);
OctalTree.IterateLines(DrawFaceNormalLine);
{glEnd not needed here}
end;

{-----------------------------------------------------------------------------}



procedure TGLMesh.RenderNormal;
{Handle displaying normal GL mode}
begin
case FRenderMode of
 rmWireFrame: DoWireTriangles;
 rmFlatFace: DoFlatTriangles;
 rmSmoothFace: DoSmoothTriangles;
 end;
end;



procedure TGLMesh.SetupOctalTree;
{Point all triangles in the boxes}
{This is only done when the mesh changed}
var I: integer;
begin
OctalTree.ResetBoxes;
OctalTree.MeshDimensions:=MakeBox(ExtentMin,ExtentMax);
for I:=0 to Vertices.TriCount-1 do
	begin
	{If triangles span boxes, we'll use the box for the first point}
	OctalTree.PutTriangleInBox(Vertices.Triangle[I],I);
	end;
end;



procedure TGLMesh.PrepareMesh;
begin
SetupOctalTree;
MeshBuilt:=True;
end;



procedure TGLMesh.PrepareTexture;
begin
Texture.SetupTextureVertices(Vertices,ExtentMin,ExtentMax);
TextureBuilt:=True;
end;



procedure TGLMesh.Render;
{Render whole mesh}
begin
glPushAttrib(GL_ALL_ATTRIB_BITS);
try
if not MeshBuilt then PrepareMesh;
if not TextureBuilt then PrepareTexture;
inherited Render;
OctalTree.FlagBoxesInFrustum;
RenderNormal;
if ShowNormals then DisplayNormals;
if ShowFaceNormals then DisplayFaceNormals;
finally glPopAttrib; end;
end;



procedure TGLMesh.SetColor(const Color: TColor);
var I: integer;
var Mask: cardinal;
var B: byte;
begin
inherited SetColor(Color);
{Convert percent-transparency to byte transparency mask}
B:=trunc(255 * (100-Transparency)/100);
{Shift into highest byte}
Mask:=B shl 24;
for I:=0 to FaceColor.Count-1 do
	begin
	{Color face Adding transparency}
	FaceColor.Face[I]:=(Color and $00FFFFFF) or Mask;;
	end;
end;



function TGLMesh.GetColor: TColor;
begin
if FaceColor.Count>0 then Result:=FaceColor.Face[0]
else Result:=clWhite;
end;



procedure TGLMesh.Assign(GLObject: TGLObject);
begin
Clear;
inherited Assign(GLObject);
if GLObject is TGLMesh then
	begin
	Vertices.Assign(TGLMesh(GLObject).Vertices);
	FaceNormals.Assign(TGLMesh(GLObject).FaceNormals);
	VertexNormals.Assign(TGLMesh(GLObject).VertexNormals);
	FaceColor.Assign(TGLMesh(GLObject).FaceColor);
	Texture.Assign(TGLMesh(GLObject).Texture);
	end;
Change;
end;


procedure TGLMesh.MatrixMultiplyVertices(Vertices: THuge3DVector; Matrix: TGLMatrix);
var I: integer;
begin
for I:=0 to Vertices.Count-1 do
	begin
	Vertices[I]:=MatrixVectorMult(Matrix,Vertices[I]);
	end;
end;


procedure TGLMesh.ConvertAndAppend(Destination,Source: THuge3DVector; Matrix: TGLMatrix);
var VT: THuge3DVectorEx;
begin
VT:=THuge3DVectorEx.Create(1);
try
VT.Assign(Source);
MatrixMultiplyVertices(VT,Matrix);
Destination.Append(VT);
finally VT.Free; end;
end;




procedure TGLMesh.Append(GLObject: TGLObject);
var VT: THuge3DVectorEx;
var M: TGLMatrix;
begin
VT:=THuge3DVectorEx.Create(1);
try
inherited Append(GLObject);
if GLObject is TGLMesh then
	begin
	{Put both meshes in the same relative position by applying each of their matrices}
	MatrixMultiplyVertices(Vertices,LastMatrix);
	ConvertAndAppend(Vertices,TGLMesh(GLObject).Vertices,TGLMesh(GLObject).LastMatrix);
	{Now rotate back to the neutral position}
	M:=InvertGLMatrix(LastMatrix);
	MatrixMultiplyVertices(Vertices,M);

	{Now rotate the new normals into their starting position and then back to the neutral position}
	M:=MatrixMultiply(TGLMesh(GLObject).LastMatrix,M);
	M:=RemoveGLMatrixScaling(RemoveGLMatrixPosition(M));
	ConvertAndAppend(FaceNormals,TGLMesh(GLObject).FaceNormals,M);
       	ConvertAndAppend(VertexNormals,TGLMesh(GLObject).VertexNormals,M);
	FaceColor.Append(TGLMesh(GLObject).FaceColor);
	CalculateMinMax;
	Change;
	end;
finally VT.Free; end;
end;




{----------------- Utility Routines------------------------------------------}




function TGLMesh.FindFirstDuplicate(Target: T3DVector): integer;
var Start: integer;
begin
Result:=-1;
SortAllVertices;
if not FindVertex(Target,Start) then exit;
Result:=SortedVertexInx(Start);
end;




procedure TGLMesh.FindDuplicateVertices(Target: T3DVector; var VA: TIntegerHashTable);
{Build list of duplicate duplicate vertices}
var I,Start: integer;
begin
SortAllVertices;
if not FindVertex(Target,Start) then exit;
for I:=Start to Sorter.Count-1 do
	begin
	{We are done with the list if not matching}
	if (VectorCompare(Target,GetSortVertex(I))<>0) then break;
	VA.AddUnique(SortedVertexInx(I));
	end;
end;



function CompareIntegers(P1,P2: pointer): integer;
{Used for sorting integers}
type PInteger = ^integer;
begin
Result:=PInteger(P1)^ - PInteger(P2)^;
end;



procedure TGLMesh.FindAdjacentTriangles(Target: T3DVector; var TA: TIntegerHashTable);
{Find all triangle adjacent to Target point}
var I,TInx: integer;
var VA: TIntegerHashTable;
begin
VA:=TIntegerHashTable.Create(5);
try
FindDuplicateVertices(Target,VA);
for I:=0 to VA.Count-1 do
	begin
	TInx:=VA[I] div 3;
	TA.AddUnique(TInx);
	end;
TA.Items.Sort(CompareIntegers);
TA.Rehash;

finally VA.Free; end;
end;




procedure TGLMesh.RemoveDegenerateTriangles;
{Remove all triangles that have two duplicate points (makes a line)}
var I: integer;
var T: TTriangle;
var  Cnt: integer;
begin
Cnt:=0;
for I:=Vertices.TriCount-1 downto 0 do
	begin
	T:=Vertices.Triangle[I];
	if (VectorCompare(T.P1,T.P2)=0) or
	   (VectorCompare(T.P1,T.P3)=0) or
	   (VectorCompare(T.P2,T.P3)=0) then
		begin
		Vertices.DeleteTriangle(I);
		Inc(Cnt);
		end;
	end;
if Cnt<>0 then Change;
end;



procedure TGLMesh.RemoveAdjacentDegenerates(Target: T3DVector);
{Remove degenerate triangles that are immediately adjacent to Target}
var I,Cnt: integer;
var VA: TIntegerHashTable;
var T: TTriangle;
begin
Cnt:=0;
VA:=TIntegerHashTable.Create(5);
try
FindAdjacentTriangles(Target,VA);
for I:=VA.Count-1 downto 0 do
	begin
	T:=Vertices.Triangle[VA[I]];
	if (VectorCompare(T.P1,T.P2)=0) or
	   (VectorCompare(T.P1,T.P3)=0) or
	   (VectorCompare(T.P2,T.P3)=0) then
		begin
		Vertices.DeleteTriangle(VA[I]);
		Inc(Cnt);
		end;
	end;
if Cnt<>0 then Change;
finally VA.Free; end;
end;



function GetEdgePoints(Edge: integer; Tri: TTriangle): T3DLine;
begin
case Edge of
 0: begin Result.P1:=Tri.P1; Result.P2:=Tri.P2; end;
 1: begin Result.P1:=Tri.P2; Result.P2:=Tri.P3; end;
 2: begin Result.P1:=Tri.P3; Result.P2:=Tri.P1; end;
 end;
end;



procedure TGLMesh.FindAdjacentEdge(L: T3DLine; Inx: integer; var Edge: TEdgeInfo);
var I,VInx,TInx: integer;
var Tri: TTriangle;
var VA: TIntegerHashTable;
begin
Edge.Inx:=-1;
VA:=TIntegerHashTable.Create(5);
try
FindDuplicateVertices(L.P1,VA);
for I:=0 to VA.Count-1 do 
	begin
	VInx:=VA[I];
	TInx:=VInx div 3;
	if TInx=Inx then continue;
	Tri:=Vertices.Triangle[TInx];
	{Ignore degenerate triangles}
	if VectorsEqual(Tri.P1,Tri.P2) or
	   VectorsEqual(Tri.P2,Tri.P3) or
	   VectorsEqual(Tri.P3,Tri.P1) then continue;
	if (VectorsEqual(Tri.P1, L.P1) and VectorsEqual(Tri.P2,L.P2)) or
	   (VectorsEqual(Tri.P1, L.P2) and VectorsEqual(Tri.P2, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=0;
		exit;
		end
	else if (VectorsEqual(Tri.P2, L.P1) and VectorsEqual(Tri.P3, L.P2)) or
	   (VectorsEqual(Tri.P2, L.P2) and VectorsEqual(Tri.P3, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=1;
		exit;
		end
	else if (VectorsEqual(Tri.P3, L.P1) and VectorsEqual(Tri.P1, L.P2)) or
	   (VectorsEqual(Tri.P3, L.P2) and VectorsEqual(Tri.P1, L.P1)) then
		begin
		Edge.Inx:=I; Edge.Edge:=2;
		exit;
		end;
	end;
finally VA.Free; end;
end;




procedure TGLMesh.ChangeAllDuplicateVertices(Old,New: T3DVector);
{Change all vertices that are adjacent to Old}
var I: integer;
var VA: TIntegerHashTable;
begin
VA:=TIntegerHashTable.Create(5);
try
FindDuplicateVertices(Old,VA);
for I:=0 to VA.Count-1 do Vertices[VA[I]]:=New;
finally VA.Free; end;
end;



procedure TGLMesh.FindAdjacentVertices(Old: T3DVector; var VA: TIntegerHashTable);
{Find all vertices that are connected by a triangle}
var I,TS: integer;
var Inx: integer;
var VH: TIntegerHashTable;
begin
VH:=TIntegerHashTable.Create(5);
try
FindDuplicateVertices(Old,VH);
for I:=0 to VH.Count-1 do
	begin
	Inx:=VH[I];
	TS:=(Inx div 3) * 3;
	VA.ADDUnique(TS);
	VA.ADDUnique(TS+1);
	VA.ADDUnique(TS+2);
	end;
finally VH.Free; end;
end;



procedure TGLMesh.NormalizeScale;
var I: integer;
begin
for I:=0 to Vertices.Count-1 do
	Vertices[I]:=VectorMultiply3D(Vertices[I],Scale3D);
Scale3D:=MakeVector3D(1,1,1);
Change;
end;



procedure TGLMesh.CenterMesh;
var I: integer;
var C: T3DVector;
begin
C:=ScalarDivide3D(VectorAdd3D(ExtentMin,ExtentMax),2);
for I:=0 to Vertices.Count-1 do
	Vertices[I]:=VectorSubtract3D(Vertices[I],C);
Change;
Self.RotationCenter:=MakeVector3D(0,0,0);
end;



function TGLMesh.RemoveTriangle(I: integer): T3DVector;
{Remove triangle by collapsing to single point and moving adjacent vertices}
var T: TTriangle;
begin
T:=Vertices.Triangle[I];
Result:=ScalarDivide3D(VectorAdd3D(VectorAdd3D(T.P1,T.P2),T.P3),3);
ChangeAllDuplicateVertices(T.P1,Result);
ChangeAllDuplicateVertices(T.P2,Result);
ChangeAllDuplicateVertices(T.P3,Result);
Vertices.DeleteTriangle(I);
Change;
end;




procedure TGLMesh.SetFaceColors(Inx: integer; C1, C2, C3: TColor);
var FC: TGLFaceColor;
begin
inherited;
FC.C1:=C1; FC.C2:=C2; FC.C3:=C3;
FaceColor[Inx]:=FC;
end;


function TGLMesh.FaceToTriangle(FaceInx: integer): TTriangle;
var VI: integer;
begin
Result:=inherited FaceToTriangle(FaceInx);
VI:=FaceInx * 3;
Result:=VertexToTriangle(VI);
end;



function TGLMesh.GetVertex(Index: integer): T3DVector;
begin
Result:=Vertices[Index];
end;



procedure TGLMesh.SetVertex(Index: integer; const Value: T3DVector);
begin
Vertices[Index]:=Value;
end;


function TGLMesh.GetVertexColor(Index: integer): TColor;
var GLC: TGLFaceColor;
var VI: integer;
begin
VI:=Index div 3;
GLC:=FaceColor[VI];
case Index mod 3 of
 0: Result:=GLC.C1;
 1: Result:=GLC.C2;
 2: Result:=GLC.C3;
 end;
end;



procedure TGLMesh.SetVertexColor(Index: integer; const Value: TColor);
var GLC: TGLFaceColor;
var VI: integer;
begin
VI:=Index div 3;
GLC:=FaceColor[VI];
case Index mod 3 of
 0: GLC.C1:=Value;
 1: GLC.C2:=Value;
 2: GLC.C3:=Value;
 end;
FaceColor[VI]:=GLC;
end;


function TGLMesh.GetVertexCount: integer;
begin
Result:=Vertices.Count;
end;


procedure TGLMesh.SetVertexCount(const Value: integer);
begin
Vertices.Count:=Value;
end;



function TGLMesh.GetVertexNormal(Index: integer): T3DVector;
begin
Result:=VertexNormals[Index];
end;


procedure TGLMesh.SetVertexNormal(Index: integer; const Value: T3DVector);
begin
VertexNormals[Index]:=Value;
end;

function TGLMesh.GetFaceCount: integer;
begin
Result:=FaceColor.Count;
end;

function TGLMesh.GetFaceTriangles(Index: integer): TTriangle;
begin
Result:=Vertices.Triangle[Index];
end;

procedure TGLMesh.SetFaceTriangles(Index: integer; const Value: TTriangle);
begin
Vertices.Triangle[Index]:=Value;
end;

function TGLMesh.GetGLColor(Index: integer): TGLFaceColor;
begin
Result:=FaceColor[Index];
end;

procedure TGLMesh.SetGLColor(Index: integer; const Value: TGLFaceColor);
begin
FaceColor[Index]:=Value;
end;


function TGLMesh.GetFaceNormal(Index: integer): T3DVector;
begin
Result:=FaceNormals[Index];
end;


procedure TGLMesh.SetFaceNormal(Index: integer; const Value: T3DVector);
begin
FaceNormals[Index]:=Value;
end;



function TGLMesh.GetFaceIndices(Index: integer): TFace;
begin
Result.P1:=Index * 3;;
Result.P2:=Result.P1+1;
Result.P3:=Result.P2+1;
end;



procedure TGLMesh.SetFaceIndices(Index: integer; const Value: TFace);
{Face indices cannot be set in TGLMesh because there is no Face Array}
begin
end;




procedure TGLMesh.DeleteTriangle(Index: integer);
begin
inherited;
Vertices.DeleteTriangle(Index);
end;



function TGLMesh.GetExtentMax: T3DVector;
begin
Result:=inherited GetExtentMax;
end;



function TGLMesh.GetExtentMin: T3DVector;
begin
Result:=inherited GetExtentMin;
end;



procedure TGLMesh.SetExtentMax(const Value: T3DVector);
begin
inherited;
end;



procedure TGLMesh.SetExtentMin(const Value: T3DVector);
begin
inherited;
end;


{ TMeshList }



function TGLMeshList.Add(var Mesh: TGLMesh): integer;
begin
Mesh:=TGLMesh.Create;
Mesh.Name:='Mesh-'+IntToStr(Count);
Result:=inherited Add(Mesh);
end;



function TGLMeshList.Add: integer;
var Mesh: TGLMesh;
begin
Result:=Add(Mesh);
end;




function TGLMeshList.Insert(Inx: integer; var Mesh: TGLMesh): integer;
begin
Mesh:=TGLMesh.Create;
Mesh.Name:='Mesh-'+IntToStr(Count);
inherited Insert(Inx,Mesh);
Result:=Inx;
end;



function TGLMeshList.Insert(Inx: integer): integer;
var Mesh: TGLMesh;
begin
Result:=Insert(Inx,Mesh);
end;



procedure TGLMeshList.Assign(ObjectList: TObjectList);
var I: integer;
var O: TGLMesh;
begin
if ObjectList is TGLMeshList then
	begin
	Clear;
	for I:=0 to ObjectList.Count-1 do
		begin
		Add(O);
		O.Assign(TGLMesh(ObjectList[I]));
		end;
	end
else inherited Assign(ObjectList);
end;




function TGLMeshList.GetItem(Index: Integer): TGLMesh;
begin
Result := TGLMesh(inherited GetItem(Index));
end;



function TGLMeshList.GetTriangles: integer;
var I: integer;
begin
Result:=0;
for I:=0 to Count-1 do Result:=Result + Self[I].Vertices.TriCount;
end;



procedure TGLMeshList.SetItem(Index: Integer; AObject: TGLMesh);
begin
{This is done so user can replace}
inherited SetItem(Index, AObject);
end;


end.
