unit GLLightingDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, OpenGL15, Realspin, ComCtrls, Math, GLLibrary,GLViewer,GLLights,
  SpinButtons, Graphbox, Vector,GLRenderer,GLMesh, CheckedGroup, CheckLst, Dialogs,
  CheckBoxExt, Compass;

type
  TLightingDlg = class(TForm)
    CancelBtn: TButton;
    PageControl1: TPageControl;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    GroupBox4: TGroupBox;
    AmbTrack: TTrackBarExt;
    AmbEdit: TEdit;
    ColorDialog1: TColorDialog;
    Panel1: TPanel;
    LightList: TCheckListBox;
    ResetAmbientBtn: TButton;
    PageControl2: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet4: TTabSheet;
    Label9: TLabel;
    PositionGraphXY: TGraphBox;
    Label10: TLabel;
    GroupBox2: TGroupBox;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    LightXSpin: TRealSpin;
    LightYSpin: TRealSpin;
    LightZSpin: TRealSpin;
    PositionGraphXZ: TGraphBox;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    DiffRedSpin: TRealSpin;
    DiffGreenSpin: TRealSpin;
    DiffBlueSpin: TRealSpin;
    DiffAlphaSpin: TRealSpin;
    GroupBox3: TGroupBox;
    Label8: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label25: TLabel;
    AmbRedSpin: TRealSpin;
    AmbGreenSpin: TRealSpin;
    AmbBlueSpin: TRealSpin;
    AmbAlphaSpin: TRealSpin;
    GroupBox5: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    SpecRedSpin: TRealSpin;
    SpecGreenSpin: TRealSpin;
    SpecBlueSpin: TRealSpin;
    SpecAlphaSpin: TRealSpin;
    TabSheet5: TTabSheet;
    TypeRadio: TRadioGroup;
    GroupBox6: TGroupBox;
    SpotCutoffSpin: TRealSpin;
    Label17: TLabel;
    GroupBox7: TGroupBox;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    ConstAttenSpin: TRealSpin;
    LinearAttenSpin: TRealSpin;
    QuadAttenSpin: TRealSpin;
    SpotCompass: TCompass;
    Label21: TLabel;
    GroupBox8: TGroupBox;
    DirCompass: TCompass;
    Label23: TLabel;
    ResetPointLightBtn: TButton;
    procedure LightXSpinChange(Sender: TObject);
    procedure AmbTrackChange(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure ResetPointLightBtnClick(Sender: TObject);
    procedure PositionGraphXYPaint(Sender: TObject);
    procedure PositionGraphXYMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure PositionGraphXZMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure PositionGraphXZPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LightListClickCheck(Sender: TObject);
    procedure LightListClick(Sender: TObject);
    procedure DiffRedSpinChange(Sender: TObject);
    procedure AmbRedSpinChange(Sender: TObject);
    procedure SpecRedSpinChange(Sender: TObject);
    procedure ResetAmbientBtnClick(Sender: TObject);
    procedure TypeRadioClick(Sender: TObject);
    procedure SpotCutoffSpinChange(Sender: TObject);
    procedure ConstAttenSpinChange(Sender: TObject);
    procedure SpotCompassChanging(Sender: TObject);
    procedure DirCompassChanging(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  LightingDlg: TLightingDlg;


procedure DoLightingDlg(Viewer: TGLViewer);


implementation


{$R *.dfm}


var CurrentViewer: TGLViewer;

{ When WM_WinIniChange is broadcast all spinners }
{will update their display. This may cause a nil }
{version of CurrentViewer to be accessed causing }
{an error. Loading a dummy viewer will prevent this}

var DummyViewer: TGLViewer;


function SetAllGLArrayf4(V: double): TGLArrayf4;
begin
Result[0]:=V; Result[1]:=V; Result[2]:=V; Result[3]:=V;
end;


procedure GLArrayf4ToSpinner3(A: TGLArrayf4; var X,Y,Z: TRealSpin);
begin
X.Value:=A[0]; Y.Value:=A[1]; Z.Value:=A[2];
end;


procedure GLArrayf4ToSpinner4(A: TGLArrayf4; var Red,Green,Blue,Alpha: TRealSpin);
begin
Red.Value:=A[0]; Green.Value:=A[1]; Blue.Value:=A[2];  Alpha.Value:=A[3];
end;

{---------------- Handle Light properties ------------------------------------}

function GetSelectedLight: integer;
begin
Result:=LightingDlg.LightList.ItemIndex;
end;


procedure GetEnabledLights;
var I: integer;
begin
with LightingDlg,CurrentViewer do
 begin
 for I:=0 to MaxLights-1 do LightList.Checked[I]:=Lights[I].Enabled;
 end;
end;


procedure SetEnabledLights;
var I: integer;
begin
with LightingDlg,CurrentViewer do
 begin
 for I:=0 to MaxLights-1 do Lights[I].Enabled:=LightList.Checked[I];
 end;
end;



function MirrorVector(V: T3DVector): T3DVector;
begin
Result:=V;
Result.X:=-Result.X;
end;



procedure GetLightValues;
var I,Inx: integer;
var Up,Dir: T3DVector;
begin
with LightingDlg,CurrentViewer do
 begin
 Inx:=GetSelectedLight;
 GLArrayf4ToSpinner3(Lights[Inx].Position,LightXSPin,LightYSPin,LightZSPin);
 GLArrayf4ToSpinner4(Lights[Inx].Ambient,AmbRedSpin,AmbGreenSpin,AmbBlueSpin,AmbAlphaSpin);
 GLArrayf4ToSpinner4(Lights[Inx].Diffuse,DiffRedSpin,DiffGreenSpin,DiffBlueSpin,DiffAlphaSpin);
 GLArrayf4ToSpinner4(Lights[Inx].Specular,SpecRedSpin,SpecGreenSpin,SpecBlueSpin,SpecAlphaSpin);

 case CurrentViewer.Lights[Inx].LightType of
  ltPoint: TypeRadio.ItemIndex:=0;
  ltSpot: TypeRadio.ItemIndex:=1;
  ltDirectional: TypeRadio.ItemIndex:=2;
  end;

 SpotCutoffSpin.Value:=CurrentViewer.Lights[Inx].SpotCutoff;
 ConstAttenSpin.Value:=CurrentViewer.Lights[Inx].ConstantAttenuation;
 LinearAttenSpin.Value:=CurrentViewer.Lights[Inx].LinearAttenuation;
 QuadAttenSpin.Value:=CurrentViewer.Lights[Inx].QuadraticAttenuation;

 Dir:=GLLibrary.GLArrayToVector3D(CurrentViewer.Lights[Inx].SpotDirection);
 SpotCompass.SetOrientation(MirrorVector(Dir));
 Dir:=GLLibrary.GLArrayToVector3D(CurrentViewer.Lights[Inx].DirectionalDir);
 DirCompass.SetOrientation(MirrorVector(Dir));

 AmbTrack.Position:=Round((GlobalAmbientLight[0]+2)*25);
 end;
end;





procedure DoLightingDlg(Viewer: TGLViewer);
begin
CurrentViewer:=Viewer;
LightingDlg.LightList.ItemIndex:=0;
GetEnabledLights;
GetLightValues;
LightingDlg.Show;
end;



procedure TLightingDlg.LightXSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].Position:=MakeGLArrayf4(LightXSPin.Value,LightYSPin.Value,LightZSPin.Value,1);
CurrentViewer.Invalidate;
PositionGraphXY.Invalidate;
PositionGraphXZ.Invalidate;
end;


procedure TLightingDlg.DiffRedSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].Diffuse:=MakeGLArrayf4(DiffRedSpin.Value,DiffGreenSpin.Value,DiffBlueSpin.Value,DiffAlphaSpin.Value);
CurrentViewer.Invalidate;
end;


procedure TLightingDlg.AmbRedSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].Ambient:=MakeGLArrayf4(AmbRedSpin.Value,AmbGreenSpin.Value,AmbBlueSpin.Value,AmbAlphaSpin.Value);
CurrentViewer.Invalidate;
end;


procedure TLightingDlg.SpecRedSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].Specular:=MakeGLArrayf4(SpecRedSpin.Value,SpecGreenSpin.Value,SpecBlueSpin.Value,SpecAlphaSpin.Value);
CurrentViewer.Invalidate;
end;





procedure TLightingDlg.AmbTrackChange(Sender: TObject);
begin
CurrentViewer.GlobalAmbientLight:=SetAllGLArrayf4((AmbTrack.Position/25-2));
AmbEdit.Text:=FloatToStrF(CurrentViewer.GlobalAmbientLight[0],ffFixed,18,1);
CurrentViewer.Invalidate;
end;



procedure TLightingDlg.CancelBtnClick(Sender: TObject);
begin
LightingDlg.Close;
end;



procedure TLightingDlg.ResetPointLightBtnClick(Sender: TObject);
var I: integer;
begin
for I:=0 to MaxLights-1 do
 if CurrentViewer.Lights[I].Enabled then CurrentViewer.Lights[I].Reset;
GetLightValues;
CurrentViewer.Invalidate;
PositionGraphXY.Invalidate;
PositionGraphXZ.Invalidate;
end;



procedure TLightingDlg.PositionGraphXYPaint(Sender: TObject);
var V: T3DVector;
var Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
V:=GLArrayToVector3D(CurrentViewer.Lights[Inx].Position);
PositionGraphXY.DrawPoint(V.X,V.Y,4,clBlack,clRed);
end;



procedure TLightingDlg.PositionGraphXYMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var X2,Y2: double;
begin
if ssLeft in Shift then
	begin
	PositionGraphXY.XYToScaled(X,Y,X2,Y2);
        LightingDlg.LightXSpin.Value:=X2;
        LightingDlg.LightYSpin.Value:=Y2;
	end;
end;



procedure TLightingDlg.PositionGraphXZMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var X2,Z: double;
begin
if ssLeft in Shift then
	begin
	PositionGraphXZ.XYToScaled(X,Y,X2,Z);
        LightingDlg.LightXSpin.Value:=X2;
        LightingDlg.LightZSpin.Value:=Z;
	end;
end;



procedure TLightingDlg.PositionGraphXZPaint(Sender: TObject);
var V: T3DVector;
var Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
V:=GLArrayToVector3D(CurrentViewer.Lights[Inx].Position);
PositionGraphXZ.DrawPoint(V.X,V.Z,4,clBlack,clRed);
end;





procedure TLightingDlg.LightListClickCheck(Sender: TObject);
begin
SetEnabledLights;
CurrentViewer.Invalidate;
end;



procedure TLightingDlg.LightListClick(Sender: TObject);
begin
GetLightValues;
end;



procedure TLightingDlg.ResetAmbientBtnClick(Sender: TObject);
begin
CurrentViewer.GlobalAmbientLight:= MakeGLArrayf4(0.1,0.1,0.1,0.1);
GetLightValues;
CurrentViewer.Invalidate;
end;



procedure TLightingDlg.TypeRadioClick(Sender: TObject);
var I,Inx: integer;
var LT: TLightType;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
case TypeRadio.ItemIndex of
 0: LT:=ltPoint;
 1: LT:=ltSpot;
 2: LT:=ltDirectional;
 end;
CurrentViewer.Lights[Inx].LightType:=LT;
CurrentViewer.Invalidate;
end;




procedure TLightingDlg.SpotCutoffSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].SpotCutoff:=SpotCutoffSpin.Value;
CurrentViewer.Invalidate;
end;




procedure TLightingDlg.ConstAttenSpinChange(Sender: TObject);
var I,Inx: integer;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
CurrentViewer.Lights[Inx].ConstantAttenuation:=ConstAttenSpin.Value;
CurrentViewer.Lights[Inx].LinearAttenuation:=LinearAttenSpin.Value;
CurrentViewer.Lights[Inx].QuadraticAttenuation:=QuadAttenSpin.Value;
CurrentViewer.Invalidate;
end;


procedure TLightingDlg.SpotCompassChanging(Sender: TObject);
var I,Inx: integer;
var Up,Dir: T3DVector;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
SpotCompass.GetOrientation(Up,Dir);
Dir:=MirrorVector(Dir);
CurrentViewer.Lights[Inx].SpotDirection:=GLLibrary.Vector3DToGLArray(Dir);
CurrentViewer.Invalidate;
end;


procedure TLightingDlg.DirCompassChanging(Sender: TObject);
var I,Inx: integer;
var Up,Dir: T3DVector;
begin
Inx:=GetSelectedLight;
if Inx<0 then exit;
DirCompass.GetOrientation(Up,Dir);
Dir:=MirrorVector(Dir);
CurrentViewer.Lights[Inx].DirectionalDir:=GLLibrary.MakeGLArrayf4(Dir.X,Dir.Y,Dir.Z,0);
CurrentViewer.Invalidate;
end;



procedure TLightingDlg.FormCreate(Sender: TObject);
begin
PageControl1.ActivePageIndex:=0;
DummyViewer:=TGLViewer.Create(LightingDlg);
CurrentViewer:=DummyViewer;
end;



procedure TLightingDlg.FormDestroy(Sender: TObject);
begin
DummyViewer.Free;
end;

end.
