unit GLMaterialsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, OpenGL15, Realspin, ComCtrls, Math, GLLibrary,GLViewer,GLLights,
  SpinButtons, Graphbox, Vector,GLRenderer,GLMesh, CheckedGroup, CheckLst, Dialogs,
  CheckBoxExt;

type
  TMaterialsDlg = class(TForm)
    CancelBtn: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    EmisXSpin: TRealSpin;
    EmisYSpin: TRealSpin;
    EmisZSpin: TRealSpin;
    GroupBox5: TGroupBox;
    ShineTrack: TTrackBarExt;
    GroupBox6: TGroupBox;
    SpecTrack: TTrackBarExt;
    ShineEdit: TEdit;
    SpecEdit: TEdit;
    ResetBtn: TButton;
    Label11: TLabel;
    ObjectList: TCheckListBox;
    GroupBox3: TGroupBox;
    ColorDis: TPanel;
    ColorBtn: TButton;
    ColorDialog1: TColorDialog;
    TabSheet2: TTabSheet;
    TransparentChk: TCheckedGroup;
    Label12: TLabel;
    Label13: TLabel;
    Label14: TLabel;
    TransparentTrack: TTrackBarExt;
    InternalReflectChk: TCheckBoxExt;
    DepthCheckChk: TCheckBoxExt;
    UserBlendChk: TCheckedGroup;
    SourceRadio: TRadioGroup;
    DestRadio: TRadioGroup;
    OperatorRadio: TRadioGroup;
    TabSheet3: TTabSheet;
    GroupBox2: TGroupBox;
    BackColorDis: TPanel;
    BackColorBtn: TButton;
    procedure ShineTrackChange(Sender: TObject);
    procedure SpecTrackChange(Sender: TObject);
    procedure EmisXSpinChange(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure ResetBtnClick(Sender: TObject);
    procedure TransparentChkCheckClick(Sender: TObject);
    procedure ObjectListClick(Sender: TObject);
    procedure ColorBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BackColorBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MaterialsDlg: TMaterialsDlg;


procedure DoMaterialsDlg(Viewer: TGLViewer);


implementation



{$R *.dfm}

var CurrentViewer: TGLViewer;

{ When WM_WinIniChange is broadcast all spinners }
{will update their display. This may cause a nil }
{version of CurrentViewer to be accessed causing }
{an error. Loading a dummy viewer will prevent this}

var DummyViewer: TGLViewer;

function PowerOf2Factor(V,R: double): double;
var T: double;
begin
T:=Log10(R)/Log10(2);
Result:=Power(2,V*T)-1;
end;



function AntiPowerOf2Factor(V,R: double): double;
var T,VT: double;
begin
T:=Log10(R)/Log10(2);
VT:=Log10(V+1)/Log10(2);
Result:=VT/T;
end;


function SetAllGLArrayf4(V: double): TGLArrayf4;
begin
Result[0]:=V; Result[1]:=V; Result[2]:=V; Result[3]:=V;
end;


procedure GLArrayf4ToSpinner(A: TGLArrayf4; var X,Y,Z: TRealSpin);
begin
X.NoEventValue:=A[0]; Y.NoEventValue:=A[1]; Z.NoEventValue:=A[2];
end;

{---------------- Handle Object properties ------------------------------------}



procedure ShowObjects;
var I,C,Inx: integer;
begin
MaterialsDlg.ObjectList.Clear;
C:=CurrentViewer.ObjectCount;
for I:=0 to C-1 do
	begin
	Inx:=MaterialsDlg.ObjectList.Items.Add(CurrentViewer.GLObjects[I].Name);
	MaterialsDlg.ObjectList.Checked[Inx]:=CurrentViewer.GLObjects[I].Selected;
	end;
if C>0 then MaterialsDlg.ObjectList.ItemIndex:=0
else MaterialsDlg.ObjectList.ItemIndex:=-1;
end;




function GetSelectedObject: TGLRenderer;
var I: integer;
begin
Result:=nil;
I:=MaterialsDlg.ObjectList.ItemIndex;
if (I<0) or not (CurrentViewer.GLObjects[I] is TGLRenderer) then exit;
Result:=TGLRenderer(CurrentViewer.GLObjects[I]);
end;



procedure ResetObjectProperties;
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
for I:=0 to MaterialsDlg.ObjectList.Count-1 do
 if MaterialsDlg.ObjectList.Checked[I] then
	begin
	O:=CurrentViewer.GLObjects[I];
	if not (O is TGLRenderer) then continue;
	R:=TGLRenderer(O);
	R.ResetMaterials;
	end;
CurrentViewer.Invalidate;
end;



procedure GetObjectProperties;
var O: TGLRenderer;
begin
O:=GetSelectedObject;
if O=nil then exit;
GLArrayf4ToSpinner(O.LightEmission,MaterialsDlg.EmisXSPin,MaterialsDlg.EmisYSPin,MaterialsDlg.EmisZSPin);
MaterialsDlg.SpecTrack.NoEventPosition:=Round(AntiPowerOf2Factor(O.LightSpecular[0],128)*100);
MaterialsDlg.ShineTrack.NoEventPosition:=Round(AntiPowerOf2Factor(O.LightShininess,128)*100);
MaterialsDlg.TransparentChk.NoEventChecked:=O.DoAlphaBlend;
MaterialsDlg.InternalReflectChk.NoEventChecked:=O.BlendBackground;
MaterialsDlg.DepthCheckChk.NoEventChecked:=O.TransparentDepth;

MaterialsDlg.TransparentTrack.NoEventPosition:=round(O.Transparency);
MaterialsDlg.ColorDis.Color:=O.Color;
MaterialsDlg.BackColorDis.Color:=CurrentViewer.ScreenColor;
end;




procedure TMaterialsDlg.ShineTrackChange(Sender: TObject);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
for I:=0 to MaterialsDlg.ObjectList.Count-1 do
 if MaterialsDlg.ObjectList.Checked[I] then
	begin
	O:=CurrentViewer.GLObjects[I];
	if not (O is TGLRenderer) then continue;
	R:=TGLRenderer(O);
	R.LightShininess:=PowerOf2Factor(ShineTrack.Position/100,128);
	ShineEdit.Text:=FloatToStrF(R.LightShininess,ffFixed,18,1);
	end;
CurrentViewer.Invalidate;
end;


procedure TMaterialsDlg.SpecTrackChange(Sender: TObject);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
for I:=0 to MaterialsDlg.ObjectList.Count-1 do
 if MaterialsDlg.ObjectList.Checked[I] then
	begin
	O:=CurrentViewer.GLObjects[I];
	if not (O is TGLRenderer) then continue;
	R:=TGLRenderer(O);
	R.LightSpecular:=SetAllGLArrayf4(PowerOf2Factor(SpecTrack.Position/100,256));
	SpecEdit.Text:=FloatToStrF(R.LightSpecular[0],ffFixed,18,1);
	end;
CurrentViewer.Invalidate;
end;


procedure TMaterialsDlg.EmisXSpinChange(Sender: TObject);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
for I:=0 to MaterialsDlg.ObjectList.Count-1 do
 if MaterialsDlg.ObjectList.Checked[I] then
	begin
	O:=CurrentViewer.GLObjects[I];
	if not (O is TGLRenderer) then continue;
	R:=TGLRenderer(O);
	R.LightEmission:=MakeGLArrayf4(EmisXSPin.Value,EmisYSPin.Value,EmisZSPin.Value,1);
	end;
CurrentViewer.Invalidate;
end;



{-------------------------------------------------------------------------------}



procedure DoMaterialsDlg(Viewer: TGLViewer);
begin
CurrentViewer:=Viewer;
ShowObjects;
GetObjectProperties;
MaterialsDlg.Show;
end;



procedure TMaterialsDlg.CancelBtnClick(Sender: TObject);
begin
MaterialsDlg.Close;
end;



procedure TMaterialsDlg.ResetBtnClick(Sender: TObject);
begin
CurrentViewer.ResetLightParams;
ResetObjectProperties;
GetObjectProperties;
CurrentViewer.Invalidate;
end;



procedure SetBlendModes(R: TGLRenderer);
begin
R.UserBlend:=MaterialsDlg.UserBlendChk.Checked;
case MaterialsDlg.OperatorRadio.ItemIndex of
 0: R.BlendEquation:=GL_FUNC_ADD;
 1: R.BlendEquation:=GL_FUNC_SUBTRACT;
 2: R.BlendEquation:=GL_FUNC_REVERSE_SUBTRACT;
 3: R.BlendEquation:=GL_FUNC_REVERSE_SUBTRACT;
 4: R.BlendEquation:=GL_MIN;
 5: R.BlendEquation:=GL_MAX;
 end;

case MaterialsDlg.SourceRadio.ItemIndex of
 0: R.SrcUserBlend:=GL_ZERO;
 1: R.SrcUserBlend:=GL_ONE;
 2: R.SrcUserBlend:=GL_DST_COLOR;
 3: R.SrcUserBlend:=GL_ONE_MINUS_DST_COLOR;
 4: R.SrcUserBlend:=GL_SRC_ALPHA;
 5: R.SrcUserBlend:=GL_ONE_MINUS_SRC_ALPHA;
 6: R.SrcUserBlend:=GL_DST_ALPHA;
 7: R.SrcUserBlend:=GL_ONE_MINUS_DST_ALPHA;
 8: R.SrcUserBlend:=GL_SRC_ALPHA_SATURATE;
 end;

case MaterialsDlg.DestRadio.ItemIndex of
 0: R.DestUserBlend:=GL_ZERO;
 1: R.DestUserBlend:=GL_ONE;
 2: R.DestUserBlend:=GL_SRC_COLOR;
 3: R.DestUserBlend:=GL_ONE_MINUS_SRC_COLOR;
 4: R.DestUserBlend:=GL_SRC_ALPHA;
 5: R.DestUserBlend:=GL_ONE_MINUS_SRC_ALPHA;
 6: R.DestUserBlend:=GL_DST_ALPHA;
 7: R.DestUserBlend:=GL_ONE_MINUS_DST_ALPHA;
 end;
end;



procedure SetTransparency(Value: integer; Enable,BackBlend,Depth: boolean);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
for I:=0 to MaterialsDlg.ObjectList.Count-1 do
 if MaterialsDlg.ObjectList.Checked[I] then
	begin
	O:=CurrentViewer.GLObjects[I];
	if not (O is TGLRenderer) then continue;
	R:=TGLRenderer(O);
	R.Transparency:=Value;
	R.DoAlphaBlend:=Enable;
	R.TransparentDepth:=Depth;
	R.BlendBackground:=BackBlend;
	SetBlendModes(R);
	end;
end;




procedure TMaterialsDlg.TransparentChkCheckClick(Sender: TObject);
begin
SetTransparency(TransparentTrack.Position,TransparentChk.Checked,InternalReflectChk.Checked,DepthCheckChk.Checked);
CurrentViewer.Invalidate;
end;



procedure TMaterialsDlg.ObjectListClick(Sender: TObject);
begin
GetObjectProperties;
end;



procedure TMaterialsDlg.ColorBtnClick(Sender: TObject);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
ColorDialog1.Color:=ColorDis.Color;
if ColorDialog1.Execute then
	begin
	ColorDis.Color:=ColorDialog1.Color;
	for I:=0 to MaterialsDlg.ObjectList.Count-1 do
	 if MaterialsDlg.ObjectList.Checked[I] then
		begin
		O:=CurrentViewer.GLObjects[I];
		if not (O is TGLRenderer) then continue;
		R:=TGLRenderer(O);
		R.Color:=ColorDis.Color;
		end;
	CurrentViewer.Invalidate;
	end;
end;



procedure TMaterialsDlg.FormCreate(Sender: TObject);
begin
PageControl1.ActivePageIndex:=0;
DummyViewer:=TGLViewer.Create(MaterialsDlg);
CurrentViewer:=DummyViewer;
end;




procedure TMaterialsDlg.FormDestroy(Sender: TObject);
begin
DummyViewer.Free;
end;



procedure TMaterialsDlg.BackColorBtnClick(Sender: TObject);
var O: TGLObject;
var R: TGLRenderer;
var I: integer;
begin
ColorDialog1.Color:=BackColorDis.Color;
if ColorDialog1.Execute then
	begin
	BackColorDis.Color:=ColorDialog1.Color;
	CurrentViewer.ScreenColor:=BackColorDis.Color;
	CurrentViewer.Invalidate;
	end;
end;



end.
