unit RegistryLib;

interface

uses Windows,SysUtils,Classes,Controls,Dialogs,ShlObj,Registry,DSupport;

{Registry entry types:

1. rtKey = The specified root-key and sub-key combine to form a single, target key.
For example:

Root-key = HKEY_CURRENT_USER
Sub-Key =  Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt

Forms:

HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt

2. rtSubkeys = The specifed rootkey is scanned for immeditate sub-keys and the
specified Root-key, scanned-sub-keys and spedified sub-keys are combined. For
example:

Root-key = HKEY_USERS
Sub-Key =  Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt

Forms:

HKEY_USERS\<Key1>\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt
HKEY_USERS\<Key2>\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt
HKEY_USERS\<Key3>\Software\Microsoft\Windows\CurrentVersion\Explorer\FileExts\.txt

}

type TRegType = (rtKey,rtSubkeys);

type TRegItem = record
 RegType: TRegType;
 RootKey: DWord;
 SubKey: string;
 Value: string;
 end;

type TPIDLInfo = record
 Filename: string;
 Size: integer;
 end;



type TRegInfo = record
 Found: boolean;
 RootKey: DWord;
 Key,ValueName,Value: string;
 end;

type TRegInfoArray = array of TRegInfo;



{Specifies file names in from MRU items}

type TMRUFileInfo = record
 S1,S2,S3: string;
 end;




function RootKeyToStr(RK: dword): string;
function DisplayKeyList(RegArray: array of TRegItem; List: TStrings): integer;
function DeletKeyList(RegArray: array of TRegItem; Prompt: boolean; List: TStrings): integer;
function DisplayPIDLItems(RootKey: DWord; SubKey,Target: string; List: TStrings): integer;
function RenumberRegValues(RootKey: DWord; SubKey: string; List: TStrings): integer;
function DeletePIDLItems(RootKey: DWord; SubKey,Target: string; Prompt: boolean; List: TStrings): integer;
function ReadBinary(Reg: TRegistry; const AName: String; var VBuf: TDynByteArray): Integer;
function ProcessFileList(RootKey: DWord; Key,Target: string; Delete: boolean; List: TStrings): integer;
procedure RenumberMRUListEx(RootKey: DWord; SubKey: string; List: TStrings);
function ScanRecentFilesDir(Delete: boolean; List: TStrings): integer;


implementation



function RootKeyToStr(RK: dword): string;
begin
case RK of
 HKEY_CLASSES_ROOT: Result:='HKEY_CLASSES_ROOT';
 HKEY_CURRENT_USER: Result:='HKEY_CURRENT_USER';
 HKEY_LOCAL_MACHINE: Result:='HKEY_LOCAL_MACHINE';
 HKEY_USERS: Result:='HKEY_USERS';
 HKEY_PERFORMANCE_DATA: Result:='HKEY_PERFORMANCE_DATA';
 HKEY_CURRENT_CONFIG: Result:='HKEY_CURRENT_CONFIG';
 HKEY_DYN_DATA: Result:='HKEY_DYN_DATA';
 end;
end;



function RegInfoToStr(Info: TRegInfo): string;
begin
Result:=RootKeyToStr(Info.RootKey)+'\'+Info.Key+'\'+Info.ValueName+' '+Info.Value;
end;



function ReadBinary(Reg: TRegistry; const AName: String; var VBuf: TDynByteArray): Integer;
begin
SetLength(VBuf, 0);
Result:= Reg.GetDataSize(AName);
if Result > 0 then
	begin
	SetLength(VBuf, Result);
	Result := Reg.ReadBinaryData(AName, VBuf[0], Result);
	SetLength(VBuf, Result);
	end;
end;




function ReadPIDL(Reg: TRegistry; Name: string; var Info: TPIDLInfo): boolean;
var Buffer: array[0..MAX_PATH+1] of char;
var PidlBuf: TDynByteArray;
var Pidl: PItemIDList;
begin
Info.Size:=ReadBinary(Reg, Name, PidlBuf);
Result:=Info.Size>0;
if Result then
	begin
	Pidl := PItemIDList(@PidlBuf[0]);
	if SHGetPathFromIDList(Pidl, Buffer) then
		begin
		Info.Filename:=Buffer;
		end;
	end;
end;



function RenumberRegValues(RootKey: DWord; SubKey: string; List: TStrings): integer;
var Reg: TRegistry;
var I,Inx: integer;
var SL: TStringList;
begin
Result:=0;
Reg := TRegistry.Create;
try
SL:=TStringList.Create;
try
Reg.RootKey:=RootKey;
Reg.Access:=KEY_ALL_ACCESS;
if not Reg.OpenKey(SubKey, False) then exit;
Reg.GetValueNames(SL);
for I:=0 to SL.Count-1 do
	begin
	if SL[I]<>'MRUListEx' then
		begin
		{If the new name isn't the list, you can't rename the old item}
		Inx:=SL.IndexOf(IntToStr(Result));
		if Inx>=0 then
			begin
			{So change the name to it can be renamed}
			Reg.RenameValue(SL[Inx],'000000'+IntToStr(Inx));
			SL[Inx]:='000000'+IntToStr(Inx);
			end;
		Reg.RenameValue(SL[I],IntToStr(Result));
		Inc(Result);
		end;
	end;
List.Add('Key Values Renumbered: '+IntToStr(Result));
finally SL.Free; end;
finally Reg.Free; end;
end;



{------------------------------------------------------------------------------}


procedure ScanRegistryList(RegArray: array of TRegItem; var RegInfoArray: TRegInfoArray);
var Reg: TRegistry;
var I,J: integer;
var SL: TStringList;
var S: string;

	procedure AddRegItem(Found: boolean; RootKey: DWord; Key,Value: string);
	begin
	SetLength(RegInfoArray,Length(RegInfoArray)+1);
	RegInfoArray[High(RegInfoArray)].Found:=Found;
	RegInfoArray[High(RegInfoArray)].RootKey:=RootKey;
	RegInfoArray[High(RegInfoArray)].Key:=Key;
	RegInfoArray[High(RegInfoArray)].Value:=Value;
	end;


begin
SetLength(RegInfoArray,0);
Reg := TRegistry.Create;
try
SL:=TStringList.Create;
try
for I:=0 to High(RegArray) do
	begin
	Reg.RootKey:=RegArray[I].RootKey;
	Reg.Access:=KEY_ALL_ACCESS;
	case RegArray[I].RegType of
	 rtKey: if Reg.OpenKey(RegArray[I].SubKey, False) then
	 		begin
	 		AddRegItem(True,Reg.RootKey,Reg.CurrentPath,'');
	 		Reg.CloseKey;
	 		end
		else AddRegItem(False,Reg.RootKey,RegArray[I].SubKey,'');
	 rtSubkeys:
		begin
		{process all top level sub-keys}
                if not Reg.OpenKey('', False) then continue;
		Reg.GetKeyNames(SL);
		Reg.CloseKey;

		for J:=0 to SL.Count-1 do
			begin
			S:=SL[J]+'\'+RegArray[I].SubKey;
                        if Reg.OpenKey(S, False) then
				begin
			 	AddRegItem(True,Reg.RootKey,S,'');
			 	Reg.CloseKey;
			 	end
                        else AddRegItem(False,Reg.RootKey,S,'');
			end;

  		end;
	 else AddRegItem(False,Reg.RootKey,RegArray[I].SubKey,'');
	 end;
	end;
finally SL.Free; end;
finally Reg.Free; end;
end;



function DisplayKeyList(RegArray: array of TRegItem; List: TStrings): integer;
var Info: TRegInfoArray;
var I: integer;
var S: string;
begin
ScanRegistryList(RegArray,Info);
Result:=0;
for I:=0 to High(Info) do
	begin
	if Info[I].Found then inc(Result);
	S:=PickString(Info[I].Found,'Found: ','Not Found: ')+RegInfoToStr(Info[I]);
	List.Add(S);
	end;
end;


function DeletKeyList(RegArray: array of TRegItem; Prompt: boolean; List: TStrings): integer;
var Reg: TRegistry;
var I: integer;
var Info: TRegInfoArray;


	procedure AddToList(S: string);
	begin
	if List<>nil then List.Add(S);
	end;


	procedure DeleteItem(Key: string);
	begin
	if not Prompt or
	   (MessageDlg('Deleting Key: '+Key,mtConfirmation,[mbYes, mbNo],0)=mrYes) then
		begin
		Reg.CloseKey;
		if Reg.DeleteKey(Key) then AddToList('Key Deleted: '+Key)
		else AddToList('Unable To Delete: '+Key)
		end
	else AddToList('Delete Cancelled: '+Key);
	end;



begin
ScanRegistryList(RegArray,Info);
Reg := TRegistry.Create;
try
Result:=0;
for I:=0 to High(Info) do
	begin
	if Info[I].Found then
		begin
		Reg.RootKey:=Info[I].RootKey;
		DeleteItem(Info[I].Key);
		Inc(Result);
		end;
	end;
finally Reg.Free; end;
end;



{------------------------------------------------------------------------------}

procedure EnumeratePIDLItems(RootKey: DWord; SubKey,Target: string; var RegInfoArray: TRegInfoArray);
var Info: TPIDLInfo;
var Reg: TRegistry;
var J: integer;
var SL: TStringList;


	procedure AddRegItem(Found: boolean; RootKey: DWord; Key,ValueName,Value: string);
	begin
	SetLength(RegInfoArray,Length(RegInfoArray)+1);
	RegInfoArray[High(RegInfoArray)].Found:=Found;
	RegInfoArray[High(RegInfoArray)].RootKey:=RootKey;
	RegInfoArray[High(RegInfoArray)].Key:=Key;
	RegInfoArray[High(RegInfoArray)].ValueName:=ValueName;
	RegInfoArray[High(RegInfoArray)].Value:=Value;
	end;


begin
Reg := TRegistry.Create;
try
SL:=TStringList.Create;
try
Reg.Access:=KEY_ALL_ACCESS;
Reg.RootKey:=RootKey;

if not Reg.OpenKey(SubKey, False) then exit;
Reg.GetValueNames(SL);
for J:=0 to SL.Count-1 do
 if IsUnsigedIntegerStr(SL[J]) then
	begin
	if ReadPIDL(Reg,SL[J], Info) and (PosNoCase(Target,Info.Filename)>0) then AddRegItem(True,Reg.RootKey,SubKey,SL[J], Info.Filename)
	else AddRegItem(False,Reg.RootKey,SubKey,SL[J], Info.Filename);
	end;
finally SL.Free; end;
finally Reg.Free; end;
end;




function DisplayPIDLItems(RootKey: DWord; SubKey,Target: string; List: TStrings): integer;
var Info: TRegInfoArray;
var I: integer;
var S: string;
begin
Result:=0;
EnumeratePIDLItems(RootKey, SubKey,Target, Info);
for I:=0 to High(Info) do
	begin
	if Info[I].Found then Inc(Result);
	S:=PickString(Info[I].Found,'Found: ','Not Found: ')+RegInfoToStr(Info[I]);
	List.Add(S);
	end;
end;





function DeletePIDLItems(RootKey: DWord; SubKey,Target: string; Prompt: boolean; List: TStrings): integer;
var Reg: TRegistry;
var Info: TRegInfoArray;
var I: integer;


	procedure AddToList(S: string);
	begin
	if List<>nil then List.Add(S);
	end;


	procedure DeleteItem(Info: TRegInfo);
	var KS: string;
	begin
	if Info.ValueName='MRUListEx' then exit;
	KS:=RegInfoToStr(Info);
	if not Prompt or
	   (MessageDlg('Deleting Key: '+KS,mtConfirmation,[mbYes, mbNo],0)=mrYes) then
		begin
		Reg.CloseKey;
		Reg.RootKey:=Info.RootKey;
		if not Reg.OpenKey(Info.Key,False) then AddToList('Unable To Open Key: '+KS)
		else if Reg.DeleteValue(Info.ValueName) then AddToList('Key Deleted: '+KS)
		else AddToList('Unable To Delete: '+KS)
		end
	else AddToList('Delete Cancelled: '+KS);
	end;


begin
Result:=0;
EnumeratePIDLItems(RootKey, SubKey,Target, Info);
Reg := TRegistry.Create;
try
for I:=0 to High(Info) do
	begin
	if Info[I].Found then
		begin
		Reg.RootKey:=Info[I].RootKey;
		Reg.Access:=KEY_ALL_ACCESS;
		DeleteItem(Info[I]);
		Inc(Result);
		end;
	end;
if Result>0 then
	begin
	List.Add('Renumber Key Values and "MRUListEx" items');
	RenumberRegValues(RootKey, SubKey, List);
	RenumberMRUListEx(RootKey, SubKey, List);
	end;

finally Reg.Free; end;
RenumberRegValues(RootKey,SubKey,List);
end;


{------------------------------------------------------------------------------}



function ParseFileInfo(Buffer: TDynByteArray): TMRUFileInfo;
{Parse a key value that contains file information binary in binary format }
{This applies to some Most-Recently-Used (MRU) keys}
var I: integer;
var W: word;
begin
Result.S1:='';
Result.S2:='';
Result.S3:='';
I:=0;
while true do
	begin
	if I>=High(Buffer) then exit;
	W:=Buffer[I] + (Buffer[I+1] shl 8);
	if W=0 then break;
	Result.S1:=Result.S1 + char(W);
	Inc(I,2);
	end;
Inc(I,16);
while true do
	begin
	if I>=High(Buffer) then exit;
	W:=Buffer[I];
	if W=0 then break;
	Result.S2:=Result.S2 + char(W);
	Inc(I);
	end;
if (I and 1)=1 then Inc(I,43) else Inc(I,44);
while true do
	begin
	if I>=High(Buffer) then exit;
	W:=Buffer[I] + (Buffer[I+1] shl 8);
	if W=0 then break;
	Result.S3:=Result.S3 + char(W);
	Inc(I,2);
	end;
end;




procedure ShowMRUListEx(RootKey: DWord; SubKey: string; List: TStrings);
{Show the values in an MRUListEx value. The value normally contains}
{The order in which file were accessed }
var Reg: TRegistry;
var Buffer: TDynByteArray;
var I,Index: integer;
begin
Reg := TRegistry.Create;
try
Reg.RootKey:=RootKey;
Reg.Access:=KEY_ALL_ACCESS;
if not Reg.OpenKey(SubKey, False) then exit;
if ReadBinary(Reg, 'MRUListEx', Buffer)<0 then exit;
for I:=0 to (Length(Buffer) div SizeOf(Cardinal))-1 do
	begin
	Index:=TDynDWordArray(Buffer)[I];
	List.Add(IntToStr(Index));
	end;
finally Reg.Free; end;
end;




procedure RenumberMRUListEx(RootKey: DWord; SubKey: string; List: TStrings);
{Put all entries in "MRUListEx" in numberical order }
var Reg: TRegistry;
var Buffer: TDynDWordArray;
var I,Count: integer;
var SL: TStringList;
begin
Reg := TRegistry.Create;
try
SL:=TStringList.Create;
try
Reg.RootKey:=RootKey;
Reg.Access:=KEY_ALL_ACCESS;
if not Reg.OpenKey(SubKey, False) then exit;
Count:=0;
Reg.GetValueNames(SL);
for I:=0 to SL.Count-1 do
 if IsUnsigedIntegerStr(SL[I]) then Inc(Count);
SetLength(Buffer,Count+1);
for I:=0 to Count-1 do Buffer[I]:=I;
Buffer[Count]:=$FFFFFFFF;
Reg.WriteBinaryData('MRUListEx',Buffer[0],(Count+1) * SizeOf(DWord));
List.Add('MRUListEx Renumbered: 0-'+IntToStr(Count));
finally SL.Free; end;
finally Reg.Free; end;
end;




function ProcessFileList(RootKey: DWord; Key,Target: string; Delete: boolean; List: TStrings): integer;
{Process key that contains a list of files and a MRUListEx value}
var Buffer: TDynByteArray;
var Reg: TRegistry;
var I,DelCount: integer;
var Info: TMRUFileInfo;
var SL: TStringList;


	function GetKeyValue(Name: string; var Info: TMRUFileInfo): boolean;
	begin
	Result:=False;
	if ReadBinary(Reg, Name, Buffer)<0 then exit;
	Info:=ParseFileInfo(Buffer);
	Result:=True;
	end;


	function TestKeyValue(Target: string; Info: TMRUFileInfo): boolean;
	var S: string;
	begin
	S:=Info.S1+Info.S2+Info.S3;
	Result:= PosNoCase(Target,S)>0;
	end;


	procedure DisplayKeyValue(Name, Description: string; Info: TMRUFileInfo);
	begin
	List.Add(Description+' - Key: '+Name);
	List.Add('     S1: '+Info.S1);
	List.Add('     S2: '+Info.S2);
	List.Add('     S3: '+Info.S3);
	end;



begin
Result:=0;
Reg := TRegistry.Create;
try
SL:=TStringList.Create;
try
Reg.RootKey:=RootKey;
if not Reg.OpenKey(Key,False) then exit;
DelCount:=0;
Reg.GetValueNames(SL);
for I:=0 to SL.Count-1 do
 if IsUnsigedIntegerStr(SL[I]) then
 if GetKeyValue(SL[I],Info) then
	begin
	if TestKeyValue(Target,Info) then
		begin
		if Delete then
			begin
			Inc(DelCount);
			if Reg.DeleteValue(SL[I]) then
				begin
				DisplayKeyValue(SL[I],'Deleted: ',Info);
				Inc(Result);
				end
			else DisplayKeyValue(SL[I],'Unable to deleted: ',Info)
			end
		else
			begin
			DisplayKeyValue(SL[I],'Found',Info);
			Inc(Result);
			end;
		end;
	end;
List.Add('Deleted: '+IntToStr(DelCount));
if DelCount>0 then
	begin
	List.Add('Renumber Key Values and "MRUListEx" items');
	RenumberRegValues(RootKey, Key, List);
	RenumberMRUListEx(RootKey, Key, List);
	end;
finally SL.Free; end;
finally Reg.Free; end;
end;


{------------------------------------------------------------------------------}



function ScanRecentFilesDir(Delete: boolean; List: TStrings): integer;
{Scan the "Recent-Files" directory }
var  SR: TSearchRec;
var Path,Filename: string;
begin
Result:=0;
Path:=GetEnvironmentVariable('USERPROFILE')+'\AppData\Roaming\Microsoft\Windows\Recent\';
List.Add('Path: '+Path);
if FindFirst(Path+'*.*', $3F, SR)=0 then
	begin
	repeat
		begin
		if PosNoCase('bint.lnk',SR.Name)>0 then
			begin
			Filename:=Path+SR.Name;
			if Delete then
				begin
				if DeleteFile(Filename) then
					begin
					List.Add('File Deleted: '+Filename);
					Inc(Result);
					end
				else List.Add('Unable to Delete: '+Filename);
				end
			else
				begin
				List.Add('Found: '+Filename);
				Inc(Result);
				end;
			end;
		end
	until FindNext(SR)<>0;
	FindClose(SR)
	end
end;



end.
