unit SDTSLoader;

interface

uses Windows,Classes,SysUtils,DSupport,DEMLib,StreamLib,SDTSLib;


procedure ReadSDTSDEM(Filename: string;  DEMArray: TDEMArray; Progress: TDEMProgress);

implementation




{File opening variables}

var CellID,FullName,BaseName,BasePath: string;


{variable for processing SDTS data}

var VoidValue,FillValue: integer;
var VertScale: integer;
var NumberType: TSDTSNumberType;


{Internal Spacial Reference Data}

var ScaleFactorX,ScaleFactorY,XOrigin,YOrigin: double;
var SpacialAddressX,SpacialAddressY: integer;



{----------------- Routines to read sub-file data -------------------}


procedure OpenSubfile(var Stream: TFileStream; FileType: string);
{Open subfile based on  FileType string and base name}
var FileName: string;
begin
FileName:=BasePath+BaseName+FileType+'.DDF';
Stream:=TFileStream.Create(FileName,fmOpenRead);
end;





function GetLevelCode: integer;
{Get Cell Range data}
var Stream: TFileStream;
var DataDescript: TDataDescription;
const ModuleName = 'DQPA';
var S: string;
begin
Result:=1;
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,4);
S:=GetStringSubField(Stream,ModuleName,'COMT', DataDescript,0);
if Pos('DEM LEVEL  ',S)=1 then Result:=byte(S[12]) - $30;
finally Stream.Free; end;
end;




function GetPatternCode: integer;
{Get Cell Range data}
var Stream: TFileStream;
var DataDescript: TDataDescription;
const ModuleName = 'RSDF';
var S: string;
begin
Result:=1;
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,4);

S:=GetStringSubField(Stream,ModuleName,'OBRP', DataDescript,0);
if S='G2' then Result:=1 else Result:=0;
finally Stream.Free; end;
end;



procedure GetElevUnits(var DEMInfo: TDEMInfo);
{Get units used for elevation data}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var Units,Format: string;
const ModuleName = 'DDSH';
begin
DEMInfo.ElevationUnits:=euMeters;
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);
Units:=GetStringSubField(Stream,ModuleName,'UNIT',DataDescript,0);
if Units='FEET' then DEMInfo.ElevationUnits:=euFeet;
Format:=GetStringSubField(Stream,ModuleName,'FMT',DataDescript,0);
NumberType:=GetNumberType(Format);
if Format='BFP32' then VertScale:=10 else VertScale:=1;
if DEMInfo.ElevationUnits=euFeet then VertScale:=1;
{Vertical resolution is always one because}
{we multiply each elevation the by VertScale value}
DEMInfo.Resolution[2]:=1;
finally Stream.Free; end;
end;



procedure GetHeader(var DEMInfo: TDEMInfo);
{Get header data}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var S: string;
const ModuleName = 'IDEN';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);

DemInfo.MapName:=GetStringSubField(Stream,ModuleName,'TITL', DataDescript,0);
DEMInfo.Scale:=GetIntegerSubField(Stream,ModuleName,'SCAL', DataDescript,0);
DEMInfo.DataID:=GetStringSubField(Stream,ModuleName,'DAID', DataDescript,0);
DEMInfo.Date:=GetStringSubField(Stream,ModuleName,'DCDT', DataDescript,0);

finally Stream.Free; end;
end;



procedure GetCellRange(var DEMInfo: TDEMInfo);
{Get Cell Range data}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var RCID,DVAL: integer;
var I: integer;
const ModuleName = 'DDOM';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,4);
for I:=0 to DataDescript.DataRecords.Count-1 do
	begin
	RCID:=GetIntegerSubField(Stream,ModuleName,'RCID', DataDescript,I);
	DVAL:=GetIntegerSubField(Stream,ModuleName,'DVAL', DataDescript,I);
	case RCID of
	 1: VoidValue:=DVAL;
	 2: FillValue:=DVAL;
	 3: DEMInfo.MinElevation:=DVAL;
	 4: DEMInfo.MaxElevation:=DVAL;
         end;
	end;

finally Stream.Free; end;
end;



procedure GetIREF(var DEMInfo: TDEMInfo);
{Get Internal Spacial Reference}
var Stream: TFileStream;
var DataDescript: TDataDescription;
const ModuleName = 'IREF';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);

ScaleFactorX:=GetDoubleSubField(Stream,ModuleName,'SFAX', DataDescript,0);
ScaleFactorY:=GetDoubleSubField(Stream,ModuleName,'SFAY', DataDescript,0);
XOrigin:=GetDoubleSubField(Stream,ModuleName,'XORG', DataDescript,0);
YOrigin:=GetDoubleSubField(Stream,ModuleName,'YORG', DataDescript,0);

DEMInfo.Resolution[0]:=GetDoubleSubField(Stream,ModuleName,'XHRS', DataDescript,0);
DEMInfo.Resolution[1]:=GetDoubleSubField(Stream,ModuleName,'YHRS', DataDescript,0);

finally Stream.Free; end;
end;



procedure GetXREF(var DEMInfo: TDEMInfo);
{Get External Spacial Reference}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var HDS,RFS: string;
const ModuleName = 'XREF';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);

RFS:=GetStringSubField(Stream,ModuleName,'RSNM', DataDescript,0);

if RFS = 'Geographic' then DEMINfo.GroundRef:=0
else if RFS = 'UTM' then DEMINfo.GroundRef:=1
else if RFS = 'State Plane' then DEMINfo.GroundRef:=2;

if DEMInfo.GroundRef=1 then DEMInfo.MapType:=mt7Minute
else DEMInfo.MapType:=mtOneDegree;

DEMInfo.Zone:=GetIntegerSubField(Stream,ModuleName,'ZONE', DataDescript,0);
HDS:=GetStringSubField(Stream,ModuleName,'HDAT', DataDescript,0);

if HDS='NAS' then DEMInfo.HorzDatum:=hdNAD27
else if HDS='WGC' then DEMInfo.HorzDatum:=hdWGS72
else if HDS='WGE' then DEMInfo.HorzDatum:=hdWGS84
else if HDS='NAX' then DEMInfo.HorzDatum:=hdNAD83;

finally Stream.Free; end;
end;



procedure GetNWCorner(var DEMInfo: TDEMInfo);
{Get External Spacial Reference}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var Binary: boolean;
var Format: string;
var I: integer;
const ModuleName = 'RSDF';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);
Format:=GetFormatByTag('SADR',DataDescript);
Binary:=Pos('B',Format)>0;

if Binary then
	begin
	SpacialAddressX:=GetBinarySubField(Stream,'SADR','X', DataDescript,0,4);
	SpacialAddressY:=GetBinarySubField(Stream,'SADR','Y', DataDescript,0,4);
	end
else
	begin
	SpacialAddressX:=trunc(GetDoubleSubField(Stream,'SADR','X', DataDescript,0));
	SpacialAddressY:=trunc(GetDoubleSubField(Stream,'SADR','Y', DataDescript,0));
	end;
DEMInfo.BoundBoxNW.East:= (SpacialAddressX  * ScaleFactorX) + XOrigin;
DEMInfo.BoundBoxNW.North:= (SpacialAddressY  * ScaleFactorY) + YOrigin;
finally Stream.Free; end;
end;



procedure GetDEMQuadBoundries(var DEMInfo: TDEMInfo);
{Get External Spacial Reference}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var SA: TStringArray;
const ModuleName = 'SPDM';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);
SA:=GetArraySubField(Stream,'DMSA','X', DataDescript,0);
if SA.Count=0 then SA:=GetArraySubField(Stream,'DMSA','*X', DataDescript,0);
DEMInfo.SWCorner.East:=GetNumberValue(SA.Strings[0]);
DEMInfo.SWCorner.North:=GetNumberValue(SA.Strings[1]);
DEMInfo.NWCorner.East:=GetNumberValue(SA.Strings[2]);
DEMInfo.NWCorner.North:=GetNumberValue(SA.Strings[3]);
DEMInfo.NECorner.East:=GetNumberValue(SA.Strings[4]);
DEMInfo.NECorner.North:=GetNumberValue(SA.Strings[5]);
DEMInfo.SECorner.East:=GetNumberValue(SA.Strings[6]);
DEMInfo.SECorner.North:=GetNumberValue(SA.Strings[7]);

finally Stream.Free; end;
end;



procedure GetRowColumn(var DEMInfo: TDEMInfo);
{Get Row and Column count}
var Stream: TFileStream;
var DataDescript: TDataDescription;
const ModuleName = 'LDEF';
begin
OpenSubfile(Stream,ModuleName);
try
ReadDataDescription(Stream, DataDescript,1);
DEMInfo.DEMRows:=GetIntegerSubField(Stream,ModuleName,'NROW', DataDescript,0);
DEMInfo.DEMCols:=GetIntegerSubField(Stream,ModuleName,'NCOL', DataDescript,0);
DEMInfo.ActualRows:=DEMInfo.DEMRows;
DEMInfo.ActualCols:=DEMInfo.DEMCols;
finally Stream.Free; end;
end;



procedure InitializeData(var DEMInfo: TDEMInfo);
var I: integer;
begin
DEMInfo.LevelCode := GetLevelCode;
DEMInfo.Pattern:=GetPatternCode;

for I:=0 to High(DEMInfo.Projection) do DEMInfo.Projection[I]:=0;
{always meter in SDTS}
DEMInfo.GroundUnits:=euMeters;
DEMInfo.PolySides := 4;
DEMInfo.Accuracy := 0;
DEMInfo.DEMRows := 1;
DEMInfo.DEMCols:=1;
DEMInfo.Angle := 0;
DEMInfo.DeRotated:=False;
end;




procedure ReadSDTSParameter(var DEMInfo: TDEMInfo);
{Read all parameters necessary to setup read}
begin
InitializeData(DEMInfo);
GetElevUnits(DEMInfo);
GetHeader(DEMInfo);
GetCellRange(DEMInfo);
GetIREF(DEMInfo);
GetXREF(DEMInfo);
GetNWCorner(DEMInfo);
GetDEMQuadBoundries(DEMInfo);
GetRowColumn(DEMInfo);
end;


function FillElevationArray(DEMArray: TDEMArray; Progress: TDEMProgress): integer;
{Fill array with word values}
var Stream: TFileStream;
var DataDescript: TDataDescription;
var Format: string;
var Binary,Repeated: boolean;
var Dir: TDDRDirectory;
var Leader: TDRLeader;
var Start,FieldLen,FieldPos,TagInx: integer;
var CellInx: integer;
var Y: integer;
var MaxElevation,MinElevation: integer;
var MinIndex,MaxIndex: TPoint;
const BaseElevation=-1000;



	procedure HandleProgress(Percent: integer; State: TProgressState; Status: string);
	begin
	if Assigned(Progress) then Progress(Percent,State,Status);
	end;



	procedure TestMinMax(V: integer; X,Y: integer);
	begin
	if V>MaxElevation then
		begin
		MaxElevation:=V;
		MaxIndex:=Point(X,Y);
		end;
	if V<MinElevation then
		begin
		MinElevation:=V;
		MinIndex:=Point(X,Y);
		end;
	end;


	procedure ReadCells(N: integer);
	var X: integer;
	var CellValue: smallint;
	begin
	for X:=0 to N-1 do
		begin
		Stream.Read(CellValue,2);
		CellValue:=VertScale*Swap(CellValue);
		if CellValue < BaseElevation then DEMArray[X,Y]:=EmptyCell
		else
  			begin
			TestMinMax(CellValue,X,Y);
			DEMArray[X,Y]:=CellValue;
			end;
		Inc(CellInx);
		end;
	end;

	procedure ReadCellsReal(N: integer);
	var X: integer;
	var D: double;
	begin
	StreamEndian:=BigEnd;
	for x:=0 to N-1 do
		begin
		D:=GetStreamSingle(Stream);
		if D<-32765  then DEMArray[X,Y]:=EmptyCell
		else
			begin
			TestMinMax(round(D),X,Y);
			DEMArray[X,Y]:=round(D);
			end;
		Inc(CellInx);
		end;
	end;



begin
HandleProgress(0,psStart,'Start');
ReadSDTSParameter(DEMArray.DEMInfo);
MinElevation:=32766;
MaxElevation:=-32765;
MinIndex:=Point(-1,-1);
MaxIndex:=Point(-1,-1);
OpenSubfile(Stream,'CE'+CellID);
try
ReadDataDescription(Stream, DataDescript,1);
TagInx:=FindTag('CVLS',DataDescript.DataRecords.DRDirectorys[0]);
Format:=GetFormatByTag('CVLS',DataDescript);
Binary:=Pos('B',Format)>0;
Repeated:=DataDescript.DataRecords.DRLeaders[0].LeaderID='R';
FieldPos:=DataDescript.DataRecords.DRDirectorys[0].DirArray[TagInx].FieldPos;
Start:=DataDescript.DataRecords.DRDirectorys[0].DataPos+FieldPos;
FieldLen:=DataDescript.DataRecords.DRDirectorys[0].DirArray[TagInx].FieldLen;
CellInx:=0;
DEMArray.SetDimensions(DEMArray.DEMInfo.DEMCols,DEMArray.DEMInfo.DEMRows);

for Y:=DEMArray.DEMInfo.DEMRows-1 downto 0 do
	begin
	Stream.Position:=Start;
	if NumberType = ntBFP32 then ReadCellsReal(FieldLen div 4)
	else ReadCells(FieldLen div 2);

	if Stream.Position>=Stream.Size then break;
	if Repeated then Start:=Start+FieldLen+FieldPos
	else
		begin
		Stream.Position:=Start + FieldLen;
		if Stream.Position>=Stream.Size then break;
		ReadDRLeader(Stream,Leader);
		ReadDRDirectory(Stream, Dir, Leader);
		Start:=Dir.DataPos+Dir.DirArray[TagInx].FieldPos;
		FieldLen:=Dir.DirArray[TagInx].FieldLen;
		end;
	HandleProgress(MulDiv(100,DEMArray.DEMInfo.DEMRows-Y,DEMArray.DEMInfo.DEMRows),psWorking,'Loading');
	end;
DEMArray.DEMInfo.ActualMinVert:=MinElevation;
DEMArray.DEMInfo.ActualMaxVert:=MaxElevation;
DEMArray.DEMInfo.MinVertIndex:=MinIndex;
DEMArray.DEMInfo.MaxVertIndex:=MaxIndex;
FindDEMAngle(DEMArray);
DEMArray.Loaded:=True;
Result:=CellInx;
finally
 Stream.Free;
 HandleProgress(100,psDone,'Done');
 end;
end;



procedure SetupFilenames(InName: string);
begin
FullName:=ExtractFilename(InName);
BasePath:=ExtractFilePath(InName);
BaseName:=Copy(FullName,1,4);
CellID:=Copy(FullName,7,2);
end;



procedure ReadSDTSDEM(Filename: string;  DEMArray: TDEMArray; Progress: TDEMProgress);
var CellCount: integer;
begin
SetupFilenames(Filename);
CellCount:=FillElevationArray(DEMArray,Progress);
end;


end.