unit StrokeFontLib;

interface

uses Types,Classes,Graphics,SysUtils,DSupport,StreamLib,StrokeFontData,Vector,GLQuatLib;

type TFontStroke = record
 Vect: TPoint;
 Opcode: byte;
 end;

type TStrokeArray = array of TFontStroke;

type TCharArray = array [0..255] of TStrokeArray;


type TChrFontInfo = record
 Loaded: boolean;
 ID1,ID2: DWord;
 Descript: string;
 HeadSize: Word;
 Name: string;
 FileSize: Word;
 Version: string;
 Signature: Char;
 NumChars: Byte;
 Undef1: Byte;
 FirstChar: Word;
 DefOffset: Word;
 ScanFlag: byte;
 TopDist: Shortint;
 BaseDist: Shortint;
 BottomDis: Shortint;
 FontName: DWord;
 Unused1: byte;
 OffsetTable: array [0..255] of word;
 WidthTable: array [0..255] of byte;
 CharArray: TCharArray;
 MaxWidth,MinWidth: integer;
 AveWidth: double;
 end;

type TDrawLineProc = procedure (P1,P2,Pos: T3DVector; Height: integer; Color: TColor) of object;


type TStrokeFont = class(TObject)
 private
  FBaseScale: double;
  StartPos: T3DVector;
  WorkPos: T3DVector;
  FLineDrawProc: TDrawLineProc;
  FontQuat: TQuaternion;
  procedure SetBaseScale;
  function RotateFont(V: T3DVector): T3DVector;
 protected
 public
  FontInfo: TChrFontInfo;
  Scale: double;
  Color: TColor;
  Height: integer;
  constructor Create;
  destructor Destroy;
  procedure SetFont(Inx: integer); overload;
  procedure SetFont(var Info: TChrFontInfo); overload;
  procedure LoadFont(Filename: string);
  procedure DrawChar(C: Char);
  procedure DrawStr(S: string); overload;
  procedure DrawStr(Pos: T3DVector; S: string); overload;
    procedure SetFontDirection(Dir: T3DVector; Angle: double);
    procedure SetFontRotation(Pitch, Yaw, Roll: double);
  property LineDrawProc: TDrawLineProc read FLineDrawProc write FLineDrawProc;
 end;


procedure LoadStrokeFile(Stream: TStream; var FontInfo: TChrFontInfo); overload;
procedure LoadStrokeFile(var SA: array of byte; var FontInfo: TChrFontInfo); overload;
procedure LoadStrokeFile(Filename: string; var FontInfo: TChrFontInfo); overload;

function StrokeFontToStr(Info: TChrFontInfo): string;



implementation




function SignExtend7(I: integer): integer;
begin
Result:=I and $7F;
if (Result and $40)<>0 then Result:=Result or Integer($FFFFFFC0);
end;



function SignExtend8(I: integer): integer;
begin
Result:=I and $FF;
if (Result and $80)<>0 then
	Result:=Result or Integer($FFFFFF80);
end;


function ReadStroke(Stream: TStream): TFontStroke;
begin
Result.Vect.X:=GetStreamByte(Stream);
Result.Vect.Y:=GetStreamByte(Stream);
Result.Opcode:=(Result.Vect.X and $80) shr 6;
Result.Opcode:=Result.Opcode or (Result.Vect.Y and $80) shr 7;
Result.Vect.X:=SignExtend7(Result.Vect.X);
Result.Vect.Y:=SignExtend7(Result.Vect.Y);
end;


procedure ReadCharStrokes(Stream: TStream; var SA: TStrokeArray);
begin
repeat
	begin
	SetLength(SA,Length(SA)+1);
	SA[High(SA)]:=ReadStroke(Stream);
	end;
until SA[High(SA)].Opcode=0;
end;


var Base: integer;

procedure LoadStrokeFile(Stream: TStream; var FontInfo: TChrFontInfo);
var I,Inx: integer;
var W: word;
begin
Stream.Seek(0,soFromBeginning);
FontInfo.ID1:=GetStreamDWord(Stream);
FontInfo.ID2:=GetStreamDWord(Stream);
FontInfo.Descript:=GetStreamString(Stream,#$1A);
FontInfo.HeadSize:=GetStreamWord(Stream);
FontInfo.Name:=GetStreamString(Stream,4);
FontInfo.FileSize:=GetStreamWord(Stream);
FontInfo.Version:=IntToStr(GetStreamByte(Stream))+'.'+IntToStr(GetStreamByte(Stream));

Stream.Seek($80,soFromBeginning);
FontInfo.Signature:=char(GetStreamByte(Stream));
FontInfo.NumChars:=GetStreamWord(Stream);
FontInfo.Undef1:=GetStreamByte(Stream);
FontInfo.FirstChar:=GetStreamByte(Stream);
FontInfo.DefOffset:=GetStreamWord(Stream);
FontInfo.ScanFlag:=GetStreamByte(Stream);
FontInfo.TopDist:=SignExtend8(GetStreamByte(Stream));
FontInfo.BaseDist:=SignExtend8(GetStreamByte(Stream));
FontInfo.BottomDis:=SignExtend8(GetStreamByte(Stream));
FontInfo.FontName:=GetStreamDWord(Stream);
FontInfo.Unused1:=GetStreamByte(Stream);
for I:=0 to High(FontInfo.OffsetTable) do FontInfo.OffsetTable[I]:=0;
for I:=0 to FontInfo.NumChars-1 do
	begin
	Inx:=I+FontInfo.FirstChar;
	FontInfo.OffsetTable[Inx]:=GetStreamWord(Stream);
	end;
Base:=Stream.Position;
for I:=0 to High(FontInfo.WidthTable) do FontInfo.WidthTable[I]:=0;
FontInfo.MaxWidth:=0;
FontInfo.MinWidth:=High(Word);
FontInfo.AveWidth:=0;
for I:=0 to FontInfo.NumChars-1 do
	begin
	Inx:=I+FontInfo.FirstChar;
	W:=GetStreamByte(Stream);
	FontInfo.WidthTable[Inx]:=W;
	if W>FontInfo.MaxWidth then FontInfo.MaxWidth:=W;
	if W<FontInfo.MinWidth then FontInfo.MinWidth:=W;
	FontInfo.AveWidth:=FontInfo.AveWidth+W;
	end;
FontInfo.AveWidth:=FontInfo.AveWidth/FontInfo.NumChars;

for I:=0 to High(FontInfo.CharArray) do SetLength(FontInfo.CharArray[I],0);
for I:=0 to FontInfo.NumChars-1 do
	begin
	Inx:=I+FontInfo.FirstChar;
	if FontInfo.WidthTable[Inx]<>0 then ReadCharStrokes(Stream,FontInfo.CharArray[Inx]);
	end;
FontInfo.Loaded:=True;
end;



procedure LoadStrokeFile(Filename: string; var FontInfo: TChrFontInfo);
var Stream: TMemoryStream;
begin
Stream:=TMemoryStream.Create;
try
Stream.LoadFromFile(Filename);
LoadStrokeFile(Stream,FontInfo);
finally Stream.Free; end;
end;



procedure LoadStrokeFile(var SA: array of byte; var FontInfo: TChrFontInfo);
var Stream: TMemoryStream;
begin
Stream:=TMemoryStream.Create;
try
Stream.WriteBuffer(SA, Length(SA));
LoadStrokeFile(Stream,FontInfo);
finally Stream.Free; end;
end;


{-------------------------------------------------------------------------------}


procedure CopyFontInfo(var Source,Dest: TChrFontInfo);
var I,J: integer;
begin
Dest.ID1:=Source.ID1;
Dest.ID2:=Source.ID2;
Dest.Descript:=Source.Descript;
Dest.HeadSize:=Source.HeadSize;
Dest.Name:=Source.Name;
Dest.FileSize:=Source.FileSize;
Dest.Version:=Source.Version;

Dest.Signature:=Source.Signature;
Dest.NumChars:=Source.NumChars;
Dest.Undef1:=Source.Undef1;
Dest.FirstChar:=Source.FirstChar;
Dest.DefOffset:=Source.DefOffset;
Dest.ScanFlag:=Source.ScanFlag;
Dest.TopDist:=Source.TopDist;
Dest.BaseDist:=Source.BaseDist;
Dest.BottomDis:=Source.BottomDis;
Dest.FontName:=Source.FontName;
Dest.Unused1:=Source.Unused1;
for I:=0 to High(Source.OffsetTable) do
	Dest.OffsetTable[I]:=Source.OffsetTable[I];

Dest.MaxWidth:=Source.MaxWidth;
Dest.MinWidth:=Source.MinWidth;
Dest.AveWidth:=Source.AveWidth;
for I:=0 to High(Dest.WidthTable) do
	Dest.WidthTable[I]:=Source.WidthTable[I];

for I:=0 to High(Source.CharArray) do
	begin
	SetLength(Dest.CharArray[I],Length(Source.CharArray[I]));
	for J:=0 to High(Source.CharArray[I]) do
		Dest.CharArray[I,J]:=Source.CharArray[I,J];
	end;
Dest.Loaded:=True;
end;

{-------------------------------------------------------------------------------}


function GetNumStr(Value,Nibbles: integer): string;
begin
Result:='$'+IntToHex(Value,4)+' '+IntToStr(Value);
end;


function GetOpcodeStr(Op: byte): string;
begin
case Op of
 0: Result:='End (0)';
 1: Result:='Do Scan (1)';
 2: Result:='Move To (2)';
 3: Result:='Line To (3)';
 end;
end;


function StrokeFontToStr(Info: TChrFontInfo): string;
var S: string;
var I,J,Inx: integer;
var O: word;
var Stroke: TFontStroke;
begin
Result:='';
Result:=Result+'ID1: '+IntToHex(Info.ID1,8)+CRLF_Char;
Result:=Result+'ID2: '+IntToHex(Info.ID2,8)+CRLF_Char;
Result:=Result+'Descript: '+Trim(Info.Descript)+CRLF_Char;
Result:=Result+'Head Size: '+GetNumStr(Info.HeadSize,4)+CRLF_Char;
Result:=Result+'Name: '+Info.Name+CRLF_Char;
Result:=Result+'File Size: '+GetNumStr(Info.FileSize,4)+CRLF_Char;
Result:=Result+'Version: '+Info.Version+CRLF_Char;
Result:=Result+''+CRLF_Char;
Result:=Result+'Signaure: '+Info.Signature+CRLF_Char;
Result:=Result+'Num Chars: '+GetNumStr(Info.NumChars,4)+CRLF_Char;
Result:=Result+'First Char: '+GetNumStr(Info.FirstChar,2)+CRLF_Char;
Result:=Result+'Def Offset: '+GetNumStr(Info.DefOffset,4)+CRLF_Char;
Result:=Result+'Scan Flag: '+GetNumStr(Info.ScanFlag,2)+CRLF_Char;
Result:=Result+'Top Dist: '+GetNumStr(Info.TopDist,2)+CRLF_Char;
Result:=Result+'Base Dist: '+GetNumStr(Info.BaseDist,2)+CRLF_Char;
Result:=Result+'Bottom Dis: '+GetNumStr(Info.BottomDis,2)+CRLF_Char;
Result:=Result+'Font Name: '+IntToHex(Info.FontName,8)+CRLF_Char;
Result:=Result+'Unused: '+IntToHex(Info.Unused1,2)+CRLF_Char;
Result:=Result+CRLF_Char;
S:='';
for I:=0 to Info.NumChars-1 do
	begin
	Inx:=I+Info.FirstChar;
	O:=Info.OffsetTable[Inx];
	S:=S+char(Inx)+' ('+IntToHex(byte(Inx),2)+') : '+IntToHex(O,4)+' ';
	if (Inx mod 4)=0 then S:=S+CRLF_Char;
	end;
Result:=Result+S+CRLF_Char;
for I:=0 to Info.NumChars-1 do
	begin
	Inx:=I+Info.FirstChar;
	S:='Char '+IntToHex(Inx,2)+' ('+char(Inx)+')';
	S:=S+'    Offset: '+IntToHex(Info.OffsetTable[Inx],4);
	S:=S+'   Width: '+IntToStr(Info.WidthTable[Inx]);
	S:=S+'      Stroke Count: '+IntToStr(Length(Info.CharArray[Inx]));
	Result:=Result+S+CRLF_Char;
	for J:=0 to High(Info.CharArray[Inx]) do
		begin
		Stroke:=Info.CharArray[Inx,J];
		S:='   '+IntToStr(J)+': OpCode: '+GetOpCodeStr(Stroke.Opcode)+
		   '   X:   '+IntToStr(Stroke.Vect.X)+'   Y:   '+IntToStr(Stroke.Vect.Y);
		Result:=Result+S+CRLF_Char;
		end;
	end;
end;


{-------------------------------------------------------------------------------}


{ TStrokeFont }

constructor TStrokeFont.Create;
begin
SetFont(0);
Scale:=1;
Color:=clLime;
SetFontRotation(0,0,0);
end;

destructor TStrokeFont.Destroy;
begin
inherited;
end;


procedure TStrokeFont.SetBaseScale;
begin
Height:=FontInfo.TopDist-FontInfo.BottomDis;
FBaseScale:=1/Height;
end;


procedure TStrokeFont.SetFont(Inx: integer);
begin
if (Inx<0) or (Inx>High(StrokeFontList)) then raise Exception.Create('Invalid Font Index');
LoadStrokeFile(PByteArray(StrokeFontList[Inx].Font)^,FontInfo);
SetBaseScale;
end;


procedure TStrokeFont.SetFont(var Info: TChrFontInfo);
begin
CopyFontInfo(Info,FontInfo);
SetBaseScale;
end;


procedure TStrokeFont.LoadFont(Filename: string);
begin
LoadStrokeFile(Filename,FontInfo);
SetBaseScale;
end;


{-----------------------------------------------------------------------------}



procedure TStrokeFont.SetFontRotation(Pitch,Yaw,Roll: double);
begin
FontQuat:=EulerToQuat(-Pitch,Yaw,Roll);
end;



procedure TStrokeFont.SetFontDirection(Dir: T3DVector; Angle: double);
{Sets font direction along the specified vector}
{Set the font rotation around specifed vector to Angle}
{Note: this is different from the Axis-Angle}
var Q1,Q2: TQuaternion;
var Base: T3DVector;
begin
Base:=MakeVector3D(1,0,0);
Q1:=QuatBetweenVectors(Base,Dir);
Q2:=AxisAngleToQuat(Dir,Angle);
FontQuat:=GLQuatLib.QuatMultiply(Q2,Q1)
end;



function TStrokeFont.RotateFont(V: T3DVector): T3DVector;
begin
V:=VectorAdd3D(V,WorkPos);
V:=VectorSubtract3D(V,StartPos);
V:=QuaternionTransform(FontQuat,V);
V:=VectorAdd3D(V,StartPos);
Result:=V;
end;



procedure TStrokeFont.DrawChar(C: Char);
var B: byte;
var I,H: integer;
var SA: TStrokeArray;
var P1,P2: T3DVector;
begin
if not FontInfo.Loaded then raise Exception.Create('No font loaded.');
if not Assigned(FLineDrawProc) then raise Exception.Create('LineDrawProc not set');
H:=round(Height*Scale*FBaseScale);
B:=byte(C);
if (B<FontInfo.FirstChar) or (B>=(FontInfo.FirstChar+FontInfo.NumChars)) then exit;
SA:=FontInfo.CharArray[B];
for I:=0 to High(SA) do
	begin
	P2:=ScalarProduct3D(PointToVector3D(SA[I].Vect),Scale*FBaseScale);
	P2:=RotateFont(P2);
	case SA[I].Opcode of
	 0,1: ;
	 2: P1:=P2;
	 3:
		begin
		if Assigned(FLineDrawProc) then FLineDrawProc(P1,P2,WorkPos,H,Color)
		else raise Exception.Create('LineDrawProc not set');
		P1:=P2;
		end;
	 end;
	end;
WorkPos.X:=WorkPos.X+FontInfo.WidthTable[B]*Scale*FBaseScale;
end;



procedure TStrokeFont.DrawStr(S: string);
var I,Col: integer;
var H: double;
var C: char;
begin
H:=Height*Scale*FBaseScale;
Col:=0;
for I:=1 to Length(S) do
	begin
	C:=S[I];
	case C of
	 CR_Char: begin WorkPos.X:=StartPos.X; Col:=0; end;
	 LF_Char: WorkPos.Y:=WorkPos.Y-H;
	 Tab_Char:
		begin
		repeat
			begin
			WorkPos.X:=FontInfo.WidthTable[$20]*Scale*FBaseScale;
			Inc(Col);
			end
		until (Col mod 8)=0;
		end;
	 else DrawChar(C);
	 end;
	end;
end;



procedure TStrokeFont.DrawStr(Pos: T3DVector; S: string);
begin
StartPos:=Pos;
WorkPos:=Pos;
DrawStr(S);
end;


end.
