unit TGAImage;

interface


uses Windows,Classes,Graphics, SysUtils,Math,StreamLib,BitmapLib,PCXLib;

type TTGAHeader = packed record
 IDLength: Byte;
 ColorMapType: Byte;
 ImageType: Byte;
 ColorMapOrigin: Word;
 ColorMapLength: Word;
 ColorMapEntrySize: Byte;
 XOrigin: Word;
 YOrigin: Word;
 Width: Word;
 Height: Word;
 PixelSize: Byte;
 ImageDescriptor: Byte;
 end;


type TTGAImage = class(TBitmap)
 private
  procedure GetColorMap(Stream: TStream);
  procedure ReadMappedLine(Stream: TStream; Y: integer; Width: integer; Compress: boolean);
  procedure ReadGrayLine(Stream: TStream; Y, Width: integer; Compress: boolean);
  function GetDataStart: integer;
 protected
 public
  TGAHeader: TTGAHeader;
  ColorArray: array of TRGBTriple;
  constructor Create; override;
  destructor Destroy; override;
  procedure LoadFromStream(stream : TStream); override;
  procedure SaveToStream(stream : TStream); override;
 end;

type  ETGAException = class (Exception);

implementation


type TRGB16 = packed record
 B1,B2: Byte;
 end;

type PRGB16 = ^TRGB16;


type TRGB24 = packed record
 R, G, B: Byte;
 end;

type PRGB24 = ^TRGB24;


type TRGB32 = packed record
 R, G, B, A: Byte;
 end;

type PRGB32 = ^TRGB32;





function rgbToBgr(color: integer): integer;
begin
Result:= ((color and $FF0000) shr 16) + (color and $00FF00) + ((color and $0000FF) shl 16);
end;


procedure ReadAndUnPackRLETGA8(stream: TStream; destBuf: PChar; totalSize: Integer);
var n: Integer;
var color: byte;
var bufEnd: PChar;
var b: Byte;
begin
bufEnd:=@destBuf[totalSize];
while destBuf<bufEnd do
	begin
	stream.Read(b,1);
	if b>=128 then
		begin
		stream.Read(color,1);
		b:=(b and 127)+1;
		while b>0 do
			begin
			PByte(destBuf)^:=color;
			Inc(destBuf,1);
			Dec(b);
			end;
		end
	else
		begin
		n:=((b and 127)+1);
		stream.Read(destBuf^, n);
		Inc(destBuf, n);
		end;
	end;
end;



procedure ReadAndUnPackRLETGA16(stream: TStream; destBuf: PChar; totalSize: Integer);
type TRGB16 = packed record
 B1,B2: Byte;
 end;
type PRGB16 = ^TRGB16;
var n: Integer;
var color: TRGB16;
var bufEnd: PChar;
var b: Byte;
begin
bufEnd:=@destBuf[totalSize];
while destBuf<bufEnd do
	begin
	stream.Read(b, 1);
	if b>=128 then
		begin
		stream.Read(color, 2);
		b:=(b and 127)+1;
		while b>0 do
			begin
			PRGB16(destBuf)^:=color;
			Inc(destBuf, 2);
			Dec(b);
			end;
		end
	else
		begin
		n:=((b and 127)+1)*2;
		stream.Read(destBuf^, n);
		Inc(destBuf, n);
		end;
	end;
end;



procedure ReadAndUnPackRLETGA24(stream: TStream; destBuf: PChar; totalSize: Integer);
var n: Integer;
var color: TRGB24;
var bufEnd: PChar;
var b: Byte;
begin
bufEnd:=@destBuf[totalSize];
while destBuf<bufEnd do
	begin
	stream.Read(b, 1);
	if b>=128 then
		begin
		stream.Read(color, 3);
		b:=(b and 127)+1;
		while b>0 do
			begin
			PRGB24(destBuf)^:=color;
			Inc(destBuf, 3);
			Dec(b);
			end;
		end
	else
		begin
		n:=((b and 127)+1)*3;
		stream.Read(destBuf^, n);
		Inc(destBuf, n);
		end;
	end;
end;



procedure ReadAndUnPackRLETGA32(stream: TStream; destBuf: PChar; totalSize: Integer);
var n: Integer;
var color: TRGB32;
var bufEnd: PChar;
var b: Byte;
begin
bufEnd:=@destBuf[totalSize];
while destBuf<bufEnd do
	begin
	stream.Read(b, 1);
	if b>=128 then
		begin
		// repetition packet
		stream.Read(color, 4);
		b:=(b and 127)+1;
		while b>0 do
			begin
			PRGB32(destBuf)^:=color;
			Inc(destBuf, 4);
			Dec(b);
			end;
		end
	else
		begin
		n:=((b and 127)+1)*4;
		stream.Read(destBuf^, n);
		Inc(destBuf, n);
		end;
	end;
end;



constructor TTGAImage.Create;
begin
inherited Create;
end;



destructor TTGAImage.Destroy;
begin
inherited Destroy;
end;



function GetColor16(Stream: TStream): TRGBTriple;
var C: Word;
begin
C:=StreamLib.GetStreamWord(Stream);
Result.rgbtBlue:=(C and $1F) shl 3;  C:=C shr 5;
Result.rgbtGreen:=(C and $1F) shl 3; C:=C shr 5;
Result.rgbtRed:=(C and $1F) shl 3; C:=C shr 5;
end;



function GetColor24(Stream: TStream): TRGBTriple;
begin
Result.rgbtBlue:=GetStreamByte(Stream);
Result.rgbtGreen:=GetStreamByte(Stream);
Result.rgbtRed:=GetStreamByte(Stream);
end;



function GetColor32(Stream: TStream): TRGBTriple;
begin
Result.rgbtBlue:=GetStreamByte(Stream);
Result.rgbtGreen:=GetStreamByte(Stream);
Result.rgbtRed:=GetStreamByte(Stream);
GetStreamByte(Stream);
end;




procedure TTGAImage.GetColorMap(Stream: TStream);
var I: integer;
begin
Stream.Position:=18 + TGAHeader.IDLength + TGAHeader.ColorMapOrigin;
SetLength(ColorArray,TGAHeader.ColorMapLength);
for I:=0 to TGAHeader.ColorMapLength-1 do
	begin
	case TGAHeader.ColorMapEntrySize of
	 15,16: ColorArray[I]:=GetColor16(Stream);
	 24: ColorArray[I]:=GetColor24(Stream);
	 32: ColorArray[I]:=GetColor32(Stream);
	 end;
	end;
end;



procedure TTGAImage.ReadMappedLine(Stream: TStream; Y: integer; Width: integer; Compress: boolean);
var X: integer;
var A: TByteArray;
var Scan: PRGBTripleArray;
begin
Scan:=Scanline[Y];
if Compress then ReadAndUnPackRLETGA8(Stream,@A,Width)
else Stream.Read(A,Width);
for X:=0 to Width-1 do
	begin
	Scan[X]:=ColorArray[A[X]];
	end;
end;



procedure TTGAImage.ReadGrayLine(Stream: TStream; Y: integer; Width: integer; Compress: boolean);
var X: integer;
var A: TByteArray;
var Scan: PRGBTripleArray;
begin
Scan:=Scanline[Y];
if Compress then ReadAndUnPackRLETGA8(Stream,@A,Width)
else Stream.Read(A,Width);
for X:=0 to Width-1 do
	begin
	Scan[X]:=MakeRBGTriple(A[X],A[X],A[X]);
	end;
end;


function TTGAImage.GetDataStart: integer;
var BPP: integer;
begin
case TGAHeader.ColorMapEntrySize of
 15: BPP:=2;
 16: BPP:=2;
 24: BPP:=3;
 32: BPP:=4;
 else BPP:=2;
 end;
Result:=18 + TGAHeader.IDLength + TGAHeader.ColorMapLength * BPP;
end;


procedure TTGAImage.LoadFromStream(stream: TStream);
var y, rowSize, bufSize,BPP: Integer;
var verticalFlip: Boolean;
var unpackBuf: PChar;
begin
stream.Read(TGAHeader, Sizeof(TTGAHeader));
if TGAHeader.ColorMapType<>0 then GetColorMap(Stream);


{16 TGA format is a 5,5,5 format so we have to set pf15bit}
case TGAHeader.PixelSize of
  8: begin PixelFormat:=pf24Bit; BPP:=1; end;
 15: begin PixelFormat:=pf15bit; BPP:=2; end;
 16: begin PixelFormat:=pf15bit; BPP:=2; end;
 24: begin PixelFormat:=pf24bit; BPP:=3; end;
 32: begin PixelFormat:=pf32bit; BPP:=4; end;
 else raise ETGAException.Create('Unsupported TGA Pixel Size: '+IntToStr(TGAHeader.PixelSize));
 end;

Width:=TGAHeader.Width;
Height:=TGAHeader.Height;

rowSize:=Width*BPP;
verticalFlip:=((TGAHeader.ImageDescriptor and $20)=0);
Stream.Position:=GetDataStart;
case TGAHeader.ImageType of
 0:
	begin
	{ empty image, support is useless but easy ;)}
	Width:=0;
	Height:=0;
	Exit;
	end;
 1:
	begin
	{ Uncompressed Color Mapped}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do	ReadMappedLine(Stream,Height-Y-1,Width,False);
		end
	else
		begin
		for y:=0 to Height-1 do	ReadMappedLine(Stream,Y,Width,False);
		end;
	end;
 2:
	begin { Uncompressed RGB/RGBA}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do	stream.Read(ScanLine[Height-y-1]^, rowSize);
		end
	else
		begin
		for y:=0 to Height-1 do stream.Read(ScanLine[y]^, rowSize);
		end;
	end;
 3:
	begin
	{ Uncompressed Black and White}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do	ReadGrayLine(Stream,Height-Y-1,RowSize,False);
		end
	else
		begin
		for y:=0 to Height-1 do	ReadGrayLine(Stream,Y,RowSize,False);
		end;
	end;
 9:
	begin
	{ RLE compressed Color Mapped}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do	ReadMappedLine(Stream,Height-Y-1,RowSize,True);
		end
	else
		begin
		for y:=0 to Height-1 do	ReadMappedLine(Stream,Y,RowSize,True);
		end;
	end;
 10:
	begin
	{ RLE compressed RGB/RGBA}
	bufSize:=Height*rowSize;
	unpackBuf:=GetMemory(bufSize);
	try
	{ read & unpack everything}
	case TGAHeader.PixelSize of
	 15,16: ReadAndUnPackRLETGA16(stream, unpackBuf, bufSize);
	 24: ReadAndUnPackRLETGA24(stream, unpackBuf, bufSize);
	 32: ReadAndUnPackRLETGA32(stream, unpackBuf, bufSize);
	 end;
	{ fillup bitmap}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do
		 Move(unPackBuf[y*rowSize], ScanLine[Height-y-1]^, rowSize);
		end
	else
		begin
		for y:=0 to Height-1 do
		 Move(unPackBuf[y*rowSize], ScanLine[y]^, rowSize);
		 end;
	finally FreeMemory(unpackBuf); end;
	end;
 11:
	begin
	{ RLE Compressed Black and White Image}
	if verticalFlip then
		begin
		for y:=0 to Height-1 do	ReadGrayLine(Stream,Height-Y-1,RowSize,True);
		end
	else
		begin
		for y:=0 to Height-1 do	ReadGrayLine(Stream,Y,RowSize,True);
		end;
	end;
 else raise ETGAException.Create('Unsupported TGA ImageType '+IntToStr(TGAHeader.ImageType));
 end;
end;



procedure TTGAImage.SaveToStream(stream: TStream);
var y, rowSize: Integer;
var header: TTGAHeader;
begin
FillChar(header, SizeOf(TTGAHeader), 0);
header.ImageType:=2;
header.Width:=Width;
header.Height:=Height;
case PixelFormat of
 pf24bit: header.PixelSize:=24;
 pf32bit: header.PixelSize:=32;
 else raise ETGAException.Create('Unsupported Bitmap format');
 end;
stream.Write(header, SizeOf(TTGAHeader));
rowSize:=(Width*header.PixelSize) div 8;
for y:=0 to Height-1 do
	begin
	stream.Write(ScanLine[Height-y-1]^, rowSize);
	end;
end;




end.

