unit TifGlobal;

interface

uses Windows, classes, Graphics, StreamLib;

{Constants specifing Big or Little endian numbers }

const TifLittleEnd=$4949;	{'II'}
const TifBigEnd=$4D4D;	{'MM'}


{ TIFF variable types}
const tifByte=1;		{Unsigned byte}
const tifASCII=2;		{ASCII string terminated with zero}
const tifShort=3;		{Unsigned 16 bit integer}
const tifLong=4;		{Unsigned 32-bit integer}
const tifFractional=5;		{Two longs representing fraction (numerator/denominator)}

const tifSignedByte=6;		{An 8-bit signed (twos-complement) integer.}
const tifByteUndefined=7;	{An 8-bit byte that may contain anything, depending on the definition of the field.}
const tifSignedInt16=8;		{A 16-bit (2-byte) signed (twos-complement) integer.}
const tifSignedInt32=9;		{A 32-bit (4-byte) signed (twos-complement) integer.}
const tifSignedFraction=10;	{Two longs representing fraction (numerator/denominator)}
const tifFloat=11;		{Single precision (4-byte) IEEE format.}
const tifDouble=12;		{Double precision (8-byte) IEEE format.}

{RGB constants }

const RGB_SIZE=3;


{-------------------------------------------------------------------------}
{-------------------------------------------------------------------------}

{ TIFF tag names - Refer to TIFF 6.0 specification for complete information }

const GPSVersionID = 0;
const GPSLatitudeRef = 1;
const GPSLatitude = 2;
const GPSLongitudeRef = 3;
const GPSLongitude = 4;
const GPSAltitudeRef = 5;
const GPSAltitude = 6;
const GPSTimeStamp = 7;
const GPSSatellites = 8;
const GPSStatus = 9;
const GPSMeasureMode = 10;
const GPSDOP = 11;
const GPSSpeedRef = 12;
const GPSSpeed = 13;
const GPSTrackRef = 14;
const GPSTrack = 15;
const GPSImgDirectionRef = 16;
const GPSImgDirection = 17;
const GPSMapDatum = 18;
const GPSDestLatitudeRef = 19;
const GPSDestLatitude = 20;
const GPSDestLongitudeRef = 21;
const GPSDestLongitude = 22;
const GPSDestBearingRef = 23;
const GPSDestBearing = 24;
const GPSDestDistanceRef = 25;
const GPSDestDistance = 26;
const GPSProcessingMethod = 27;
const GPSAreaInformation = 28;
const GPSDateStamp = 29;
const GPSDifferential = 30;

const NewSubFile=254;
const SubfileType=255;
const ImageWidth=256;
const ImageLength=257;
const BitsPerSample=258;
const Compression=259;

const PhotometricInterp=262;
const Threshholding=263;
const CellWidth=264;
const CellLength=265;
const FillOrder=266;
const DocumentName=269;

const ImageDescription=270;
const Make=271;
const Model=272;
const StripOffsets=273;
const Orientation=274;
const SamplesPerPixel=277;
const RowsPerStrip=278;
const StripByteCounts=279;

const MinSampleValue=280;
const MaxSampleValue=281;
const XResolution=282;
const YResolution=283;
const PlanarConfiguration=284;
const PageName=285;
const XPosition=286;
const YPosition=287;
const FreeOffsets=288;
const FreeByteCounts=289;

const GrayResponseUnit=290;
const GrayResponseCurve=291;
const Group3Options=292;
const Group4Options=293;
const ResolutionUnit=296;
const PageNumber=297;

const ColorResponseUnit=300;
const ColorResponseCurves=301;
const Software=305;
const DateTime=306;

const Artist=315;
const HostComputer=316;
const Predictor = 317;
const WhitePoint = 318;			{ image white point }
const PrimaryChroma =  319;		{ primary chromaticities }

//============================

const ColorMap=320;
const HalfToneHints =  321;		{ Highlight+shadow info }
const TileWidth =  322;			{ Tile width in pixels }
const TileLength = 323;			{ Tile height in pixels }
const TileOffsets =  324;		{ Offsets to data tiles }
const TileByteCounts = 325;		{ Byte counts for tiles }
const BadFaxLines =  326;		{ lines w/ wrong pixel count }
const CleanFaxData = 327;		{ regenerated line info }
const MaxBadFaxLines = 328;		{ max consecutive bad lines }


const SubIFD = 330;			{ subimage descriptors }
const InkSet = 332;			{ Inks in separated image }
const InkNames = 333;			{ ascii names of inks }
const NumberOfInks = 334;		{ Number of inks }
const DotRange = 336;			{ 0% and 100% dot codes }
const TargetPrinter =  337;		{ Separation target }
const ExtraSamples = 338;
const SAMPLEFORMAT = 339;		{ Data sample format }

const SMinSampleValue =  340;		{ Variable MinSampleValue }
const SMaxSampleValue =  341;		{ Variable MaxSampleValue }
const TransferRange = 342;
const ClipPath = 343;			{ %ClipPath [Adobe TIFF technote 2] }
const XClipPathUnits = 344;		{ %XClipPathUnits [Adobe TIFF technote 2] }
const YClipPathUnits = 345;		{ %YClipPathUnits [Adobe TIFF technote 2] }
const Indexed =  346;			{ %Indexed [Adobe TIFF Technote 3] }
const JPEGTables = 347;			{ %JPEG table stream }

const OPIProxy = 351;			{ %OPI Proxy [Adobe TIFF technote] }

const GlobalParametersIFD = 400;
const ProfileType = 401;
const FaxProfile = 402;
const CodingMethods = 403;
const VersionYear = 404;
const ModeNumber = 405;
const Decode = 433;
const DefaultImageColor = 434;

const JPEGProc = 512;

const JPEGIFOffset = 513;		{ Pointer to SOI marker }
const JPEGIFByteCount =  514;		{ jFIF stream length }
const JPEGREStartInterval =  515;	{ Restart interval length }
const JPEGLossLessPredictors = 517;	{ Lossless proc predictor }
const JPEGPointTransform = 518;		{ Lossless point transform }
const JPEGQTables =  519;		{ Q matrice offsets }
const JPEGDCTables = 520;		{ DCT table offsets }
const JPEGACTables = 521;		{ AC coefficient offsets }
const YCBCRCoefficients =  529;		{ RGB -> YCbCr transform }
const YCBCRSubSampling = 530;		{ YCbCr subsampling factors }
const YCBCRPositioning = 531;		{ Subsample positioning }
const ReferenceBlackWhite =  532;	{ Colorimetry info }
const StripRowCounts = 559;

const XMLPacket =  700;			{ %XML packet [Adobe XMP Specification, January 2004 }

const ProjFalseOriginEasting = 3086;
const ProjFalseOriginNorthing = 3087;
const ProjCenterLong = 3088;
const ProjCenterLat = 3089;
const ProjCenterEasting = 3090;
const ProjCenterNorthing = 3091;
const ProjScaleAtNatOrigin = 3092;
const ProjScaleAtCenter = 3093;
const ProjAzimuthAngle = 3094;
const ProjStraightVertPoleLong = 3095;
const ProjRectifiedGridAngle = 3096;

const VerticalCSType = 4096;
const VerticalCitation = 4097;
const VerticalDatum = 4098;
const VerticalUnits = 4099;

const OPIImageID = 32781;		{ %OPI ImageID [Adobe TIFF technote] }
const WangAnnotation = 32932;
const REFPoints = 32953;		{ image reference points }
const RegionTackPoint =  32954;		{ region-xform tack point }
const RegionWarpCorners =  32955;	{ warp quadrilateral }
const RegionAffine = 32956;		{ affine transformation mat }
const Matteing = 32995;			{ $use ExtraSamples }
const DataType = 32996;			{ $use SampleFormat }
const ImageDepth = 32997;		{ z depth of image }
const TileDepth =  32998;		{ z depth/data tile }

const PixarImageFullWidth = 33300;	{ full image size in x }
const PixarImageFullLength =  33301;	{ full image size in y }
const PixarTextureFormat =  33302;	{ texture map format }
const PixarWrapModes =  33303;		{ s & t wrap modes }
const PixarFOVCOT = 33304;		{ cotan(fov) for env. maps }
const PixarMatrixWorldToScreen =33305;
const PixarMatrixWorldToCamera =33306;

const WriterSerialNumber = 33405;	{ device serial number }
const Copyright =  33432;		{ copyright string }
const ExposureTime = 33434;
const FNumber = 33437;
const MDFileTag = 33445;
const MDScalePixel = 33446;
const MDColorTable = 33447;
const MDLabName = 33448;
const MDSampleInfo = 33449;
const MDPrepDate = 33450;
const MDPrepTime = 33451;
const MDFileUnits = 33452;
const ModelPixelScaleTag = 33550;
const RichTIFFIPTC = 33723;

const INGRPacketDataTag = 33918;
const INGRFlagRegisters = 33919;
const IrasBTransformationMatrix = 33920;
const ModelTiepointTag = 33922;


const IT8SiteName =  34016;		{ site name }
const IT8ColorSequence = 34017;		{ color seq. [RGB,CMYK,etc] }
const IT8Header =  34018;		{ DDES Header }
const IT8RasterPadding = 34019;		{ raster scanline padding }
const IT8BitsPerRunLength =  34020;	{ # of bits in short run }
const IT8BitsPerExtendedRunLength =34021;{ # of bits in long run }
const IT8ColorTable =  34022;		{ LW colortable }
const IT8ImageColorIndicator = 34023;	{ BP/BL image color switch }
const IT8BKGColorIndicator = 34024;	{ BP/BL bg color switch }
const IT8ImageColorValue = 34025;	{ BP/BL image color value }
const IT8BKGColorValue = 34026;		{ BP/BL bg color value }
const IT8PixelIntensityRange = 34027;	{ MP pixel intensity value }
const IT8TransparencyIndicator =34028 ;	{ HC transparency switch }
const IT8ColorCharacterization =34029 ;	{ color character. table }
const IT8HCUsage = 34030;		{ HC usage indicator }
const IT8TrapIndicator = 34031;		{ Trapping indicator untrapped=0, trapped=1) }
const IT8CMYKEquivalent =  34032;	{ CMYK color equivalents }
const FrameCount = 34232;		{ Sequence Frame Count }
const ModelTransformationTag = 34264;
const PhotoShop =  34377;
const EXIFIFD =  34665;			{ Pointer to EXIF private directory }
const ICCProfile = 34675;		{ ICC profile data }

const ImageLayer = 34732;
const GeoKeyDirectoryTag = 34735;
const GeoDoubleParamsTag = 34736;
const GeoAsciiParamsTag = 34737;

const JBIGOptions =  34750;		{ JBIG options }

const ExposureProgram = 34850;
const SpectralSensitivity = 34852;
const GPSIFD = 34853;			{ Pointer to GPS private directory }
const ISOSpeedRatings = 34855;
const OECF = 34856;

const FAXRecVParams =  34908;		{ encoded Class 2 ses. parms }
const FaxSubaddress =  34909;		{ received SubAddr string }
const FAXRecVTime =  34910;		{ receive time (secs) }
const FAXDCS = 34911;			{ encoded fax ses. params, Table 2/T.30 }
const STONits =  37439;			{ Sample value to Nits }
const FEDEXEDR =  34929;		{ unknown use }
const ExifVersion = 36864;
const DateTimeOriginal = 36867;
const DateTimeDigitized = 36868;
const ComponentsConfiguration = 37121;
const CompressedBitsPerPixel = 37122;
const ShutterSpeedValue = 37377;
const ApertureValue = 37378;
const BrightnessValue = 37379;
const ExposureBiasValue = 37380;
const MaxApertureValue = 37381;
const SubjectDistance = 37382;
const MeteringMode = 37383;
const LightSource = 37384;
const Flash = 37385;
const FocalLength = 37386;
const SubjectArea = 37396;
const MakerNote = 37500;
const UserComment = 37510;
const SubsecTime = 37520;
const SubsecTimeOriginal = 37521;
const SubsecTimeDigitized = 37522;
const ImageSourceData = 37724;

const FlashpixVersion = 40960;
const ColorSpace = 40961;
const PixelXDimension = 40962;
const PixelYDimension = 40963;
const RelatedSoundFile = 40964;
const InterOperabilityIFD =  40965;	{ Pointer to Interoperability private directory }

const FlashEnergy = 41483;
const SpatialFrequencyResponse = 41484;
const FocalPlaneXResolution = 41486;
const FocalPlaneYResolution = 41487;
const FocalPlaneResolutionUnit = 41488;
const SubjectLocation = 41492;
const ExposureIndex = 41493;
const SensingMethod = 41495;
const FileSource = 41728;
const SceneType = 41729;
const CFAPattern = 41730;
const CustomRendered = 41985;
const ExposureMode = 41986;
const WhiteBalance = 41987;
const DigitalZoomRatio = 41988;
const FocalLengthIn35mmFilm = 41989;
const SceneCaptureType = 41990;
const GainControl = 41991;
const Contrast = 41992;
const Saturation = 41993;
const Sharpness = 41994;
const DeviceSettingDescription = 41995;
const SubjectDistanceRange = 41996;


const ImageUniqueID = 42016;
const GDAL_METADATA = 42112;
const GDAL_NODATA = 42113;
const ChartFormat = 47001;
const ChartSource = 47002;
const ChartSourceEdition = 47003;
const ChartSourceDate = 47004;
const ChartCorrDate = 47005;
const ChartCountryOrigin = 47006;
const ChartRasterEdition = 47007;
const ChartSoundingDatum = 47008;
const ChartDepthUnits = 47009;
const ChartMagVar = 47010;
const ChartMagVarYear = 47011;
const ChartMagVarAnnChange = 47012;
const ChartWGSNSShift = 47013;
const InsetNWPixelX = 47015;

const OceScanjobDescription = 50215;
const OceApplicationSelector = 50216;
const OceIdentificationNumber = 50217;
const OceImageLogicCharacteristics = 50218;

const DNGVersion = 50706;		{ DNG version number }
const DNGBackwardVersion = 50707;	{ DNG compatibility version }
const UniqueCameraModel =  50708;	{ name for the camera model }
const LocalizedCameraModel = 50709;	{ localized camera model name }
const CFAPlaneColor =  50710;		{ CFAPattern->LinearRaw space mapping }
const CFALayout =  50711;		{ spatial layout of the CFA }
const LinearizationTable = 50712;	{ lookup table description }
const BlackLevelRepeatDIM =  50713;	{ repeat pattern size for the BlackLevel tag }
const BlackLevel = 50714;		{ zero light encoding level }
const BlackLevelDeltaH = 50715;		{ zero light encoding level differences (columns) }
const BlackLevelDeltaV = 50716;		{ zero light encoding level differences (rows) }
const WhiteLevel = 50717;		{ fully saturated encoding level }
const DefaultScale = 50718;		{ default scale factors }
const DefaultCropOrigin =  50719;	{ origin of the final image area }
const DefaultCropSize =  50720;		{ size of the final image area }
const ColorMatrix1 = 50721;		{ XYZ->reference color space transformation matrix 1 }
const ColorMatrix2 = 50722;		{ XYZ->reference color space transformation matrix 2 }
const CameraCalibration1 = 50723;	{ calibration matrix 1 }
const CameraCalibration2 = 50724;	{ calibration matrix 2 }
const ReductionMatrix1 = 50725;		{ dimensionality reduction matrix 1 }
const ReductionMatrix2 = 50726;		{ dimensionality reduction matrix 2 }
const AnalogBalance =  50727;		{ gain applied the stored raw values}
const ASShotNeutral =  50728;		{ selected white balance in linear reference space }
const ASShotWhiteXY =  50729;		{ selected white balance in x-y chromaticity coordinates }
const BaseLineExposure = 50730;		{ how much to move the zero point }
const BaseLineNoise =  50731;		{ relative noise level }
const BaseLineSharpness =  50732;	{ relative amount of sharpening }
const BayerGreenSplit =  50733;		{ how closely the values of the green pixels in the blue/green rows track the values of the green pixels in the red/green rows }
const LinearResponseLimit =  50734;	{ non-linear encoding range }
const CameraSerialNumber = 50735;	{ camera's serial number }
const LensInfo = 50736;			{ info about the lens }
const ChromaBlurRadius = 50737;		{ chroma blur radius }
const AntiAliasStrength =  50738;	{ relative strength of the camera's anti-alias filter }
const ShadowScale =  50739;		{ used by Adobe Camera Raw }
const DNGPrivateData = 50740;		{ manufacturer's private data }
const MakerNoteSafety =  50741;		{ whether the EXIF MakerNote tag is safe to preserve along with the rest of the EXIF data }
const CalibrationIlluminant1 = 50778;	{ illuminant 1 }
const CalibrationIlluminant2 = 50779;	{ illuminant 2 }
const BestQualityScale = 50780;		{ best quality multiplier }
const RawDataUniqueID =  50781;		{ unique identifier for the raw image data }
const AliasLayerMetadata = 50784;
const OriginalRawFilename =  50827;	{ file name of the original raw file }
const OriginalRawFiledata =  50828;	{ contents of the original raw file }
const ActiveArea = 50829;		{ active (non-masked) pixels of the sensor }
const MaskedAreas =  50830;		{ list of coordinates of fully masked pixels }
const ASShotICCProfile = 50831;		{ these two tags used to }
const AsShotPreProfileMatrix = 50832;	{ map cameras's color space into ICC profile space }
const CurrentICCProfile =  50833;	{  }
const CurrentPreProfileMatrix =50834;	{  }
const TIFF_RSID = 50908;
const GEO_METADATA = 50909;
const DCSHueShiftValues =  65535;	{ hue shift correction data }



{ Tag constants }

const
 Photometric_ZeroIsWhite = 0;
 Photometric_ZeroIsBlack = 1;
 Photometric_RGB = 2;
 Photometric_Palette = 3;
 Photometric_Mask = 4;
 Photometric_Separated = 5;
 Photometric_YCBCR = 6;
 Photometric_CIELAB = 8;
 Photometric_ICCLAB = 9;	{ ICC L*a*b* [Adobe TIFF Technote 4] }
 Photometric_ITULAB = 10;	{ ITU L*a*b* }
 Photometric_LOGL = 32844;	{ CIE Log2(L) }
 Photometric_LOGLUV = 32845;	{ CIE Log2(L) (u',v') }

const
 Orientation_TopLeft = 1;
 Orientation_TopRight = 2;
 Orientation_BotRight = 3;
 Orientation_BotLeft = 4;
 Orientation_LeftTop = 5;
 Orientation_RightTop = 6;
 Orientation_RightBot = 7;
 Orientation_LeftBot = 8;

const
 PlanarConfig_Contig = 1;
 PlanarConfig_Separate = 2;

const
 ResUnit_None = 1;
 ResUnit_Inch = 2;
 ResUnit_Centimeter = 3;

const
 Prediction_None = 1;
 Prediction_Horz_Differencing = 2;

const
 ExtraSample_Unspecified = 0;
 ExtraSample_Assocalpha = 1;
 ExtraSample_Unassalpha = 2;


const TIFF_Version = 42;


{ Current compression schemes available in TIFF files }

const Compress_None = 1;		{ No Compression }
const Compress_CCITTRLE = 2;		{ CCITT modified Huffman RLE }
const Compress_CCITTFAX3 = 3;		{ CCITT Group 3 fax encoding }
const Compress_CCITTFAX4 = 4;		{ CCITT Group 4 fax encoding }
const Compress_LZW = 5;			{ Lempel-Ziv & Welch }
const Compress_OJPEG = 6;		{ 6.0 JPEG }
const Compress_JPEG = 7;		{ %JPEG DCT compression }

const Compress_NeXT = 32766;		{ NeXT 2-bit RLE }
const Compress_CCITTRLEW = 32771;	{ #1 w/ word alignment }
const Compress_PackBits = 32773;	{ Macintosh RLE }
const Compress_ThunderScan = 32809;	{ ThunderScan RLE }

{ codes 32895-32898 are reserved for ANSI IT8 TIFF/IT <dkelly@etsinc.com) }

const Compress_IT8CTPAD = 32895;	{ IT8 CT w/padding }
const Compress_IT8LW = 32896;		{ IT8 Linework RLE }
const Compress_IT8MP = 32897;		{ IT8 Monochrome picture }
const Compress_IT8BL = 32898;		{ IT8 Binary line art }

{ compression codes 32908-32911 are reserved for Pixar }

const Compress_PixarFilm = 32908;	{ Pixar companded 10bit LZW }
const Compress_PixarLog = 32909;	{ Pixar companded 11bit ZIP }
const Compress_Deflate = 32946;		{ Deflate compression }
const Compress_Adobe_Deflate = 8;	{ Deflate compression, as recognized by Adobe }

{ compression code 32947 is reserved for Oceana Matrix <dev@oceana.com> }

const Compress_DCS = 32947;		{ Kodak DCS encoding }
const Compress_JBIG = 34661;		{ ISO JBIG }
const Compress_SGILOG = 34676;		{ SGI Log Luminance RLE }
const Compress_SGILOG24 = 34677;	{ SGI Log 24-bit packed }
const Compress_JP2000 = 34712;		{ Leadtools JPEG2000 }


var TifNumberType: Word=TifLittleEnd;		{Current file is big or little "endian"}
var TagCount: integer;			{Number of tags that have been sent. }


var ProgressInc: double;	{Amount to increment progress }
var ProgressValue: double;	{Current progress value }


{Defines a progress event. The user can create a subroutine and
{ assign it to FOnProgress and the subroutine will be called  }
{ periodically during the file loading to indicate progress. }

type TTifProgress= procedure(Sender: TObject; Stage: TProgressStage; PercentDone: Byte; RedrawNow: Boolean; const R: TRect; const Msg: string);

var OnTIFProgress: TTifProgress;


procedure WriteTIFTag(Stream: TStream; Tag,TagType: Word; Length,Offset: DWord);
procedure WriteStreamRational(Stream: TStream; N,D: DWord);


function SignedByteToInteger(B: byte): integer;
function Pixels2Bytes(N: integer): integer;
function BytesPerScanline(PixelsPerScanline, BitsPerPixel, Alignment: Longint): Longint;

function GetTagStr(Tag: integer): string;

procedure AdvanceProgress(Stage: TProgressStage; Steps: integer);

implementation

function GetTagStr(Tag: integer): string;
begin
case Tag of
 GPSVersionID : Result:='GPSVersionID';                                	{0}
 GPSLatitudeRef : Result:='GPSLatitudeRef';                            	{1}
 GPSLatitude : Result:='GPSLatitude';                                  	{2}
 GPSLongitudeRef : Result:='GPSLongitudeRef';                          	{3}
 GPSLongitude : Result:='GPSLongitude';                                	{4}
 GPSAltitudeRef : Result:='GPSAltitudeRef';                            	{5}
 GPSAltitude : Result:='GPSAltitude';                                  	{6}
 GPSTimeStamp : Result:='GPSTimeStamp';                                	{7}
 GPSSatellites : Result:='GPSSatellites';                              	{8}
 GPSStatus : Result:='GPSStatus';                                      	{9}
 GPSMeasureMode : Result:='GPSMeasureMode';                            	{10}
 GPSDOP : Result:='GPSDOP';                                            	{11}
 GPSSpeedRef : Result:='GPSSpeedRef';                                  	{12}
 GPSSpeed : Result:='GPSSpeed';                                        	{13}
 GPSTrackRef : Result:='GPSTrackRef';                                  	{14}
 GPSTrack : Result:='GPSTrack';                                        	{15}
 GPSImgDirectionRef : Result:='GPSImgDirectionRef';                    	{16}
 GPSImgDirection : Result:='GPSImgDirection';                          	{17}
 GPSMapDatum : Result:='GPSMapDatum';                                  	{18}
 GPSDestLatitudeRef : Result:='GPSDestLatitudeRef';                    	{19}
 GPSDestLatitude : Result:='GPSDestLatitude';                          	{20}
 GPSDestLongitudeRef : Result:='GPSDestLongitudeRef';                  	{21}
 GPSDestLongitude : Result:='GPSDestLongitude';                        	{22}
 GPSDestBearingRef : Result:='GPSDestBearingRef';                      	{23}
 GPSDestBearing : Result:='GPSDestBearing';                            	{24}
 GPSDestDistanceRef : Result:='GPSDestDistanceRef';                    	{25}
 GPSDestDistance : Result:='GPSDestDistance';                          	{26}
 GPSProcessingMethod : Result:='GPSProcessingMethod';                  	{27}
 GPSAreaInformation : Result:='GPSAreaInformation';                    	{28}
 GPSDateStamp : Result:='GPSDateStamp';                                	{29}
 GPSDifferential : Result:='GPSDifferential';                          	{30}

 NewSubFile: Result:='New Sub-File';			{254}
 SubfileType: Result:='Sub-file	Type';			{255}
 ImageWidth: Result:='Image Width';			{256}
 ImageLength: Result:='Image Length';			{257}
 BitsPerSample:	Result:='Bits Per Sample';		{258}
 Compression: Result:='Compression';			{259}

 PhotometricInterp: Result:='Photometric Interp';   	{262}
 Threshholding:	Result:='Threshholding';		{263}
 CellWidth: Result:='Cell Width';			{264}
 CellLength: Result:='Cell Length';			{265}
 FillOrder: Result:='Fill Order';			{266}
 DocumentName: Result:='Document Name';			{269}

 ImageDescription: Result:='Image Description';		{270}
 Make: Result:='Make';					{271}
 Model:	Result:='Model';				{272}
 StripOffsets: Result:='Strip Offsets';			{273}
 Orientation: Result:='Orientation';			{274}
 SamplesPerPixel: Result:='Samples Per Pixel';		{277}
 RowsPerStrip: Result:='Rows Per Strip';		{278}
							{279}
 StripByteCounts: Result:='Strip Byte Counts';
 MinSampleValue: Result:='Min Sample Value';		{280}
 MaxSampleValue: Result:='Max Sample Value';		{281}
 XResolution: Result:='X-Resolution';			{282}
 YResolution: Result:='Y-Resolution';			{283}
 PlanarConfiguration: Result:='Planar Configuration';	{284}
 PageName: Result:='Page Name';				{285}
 XPosition: Result:='X-Position';			{286}
 YPosition: Result:='Y-Position';			{287}
 FreeOffsets: Result:='Free Offsets';			{288}
 FreeByteCounts: Result:='Free Byte Counts';		{289}

 GrayResponseUnit: Result:='Gray Response Unit';    	{290}
 GrayResponseCurve: Result:='Gray Response Curve';  	{291}
 Group3Options:	Result:='Group-3 Options';		{292}
 Group4Options:	Result:='Group-4 Options';		{293}
 ResolutionUnit: Result:='Resolution Unit';		{296}
 PageNumber: Result:='Page Number';			{297}

 ColorResponseUnit: Result:='Color Response Unit';  	{300}
 ColorResponseCurves: Result:='Color Response Curves';	{301}
 Software: Result:='Software';				{305}
 DateTime: Result:='Date/Time';				{306}

 Artist: Result:='Artist';				{315}
 HostComputer: Result:='Host Computer';			{316}
 Predictor: Result:='Predictor ';			{317}
 WhitePoint: Result:='White Point';			{318}
 PrimaryChroma:	Result:='Primary Chroma';		{319}

 ColorMap: Result:='Color Map';				{320}
 HalfToneHints:	Result:='Half Tone Hints';		{321}
 TileWidth: Result:='Tile Width';			{322}
 TileLength: Result:='Tile Length';			{323}
 TileOffsets: Result:='Tile Offsets';			{324}
 TileByteCounts: Result:='Tile Byte Counts';		{325}
 BadFaxLines: Result:='Bad Fax Lines';			{326}
 CleanFaxData: Result:='Clean Fax Data';		{327}
 MaxBadFaxLines: Result:='Max Bad Fax Lines';

 SubIFD: Result:='Sub IFD';				{330}
 InkSet: Result:='Ink Set';				{332}
 InkNames: Result:='Ink	Names';				{333}
 NumberOfInks: Result:='Number Of Inks';		{334}
 DotRange: Result:='Dot	Range';				{336}
 TargetPrinter:	Result:='Target	Printer';		{337}
 ExtraSamples: Result:='Extra Samples';			{338}
 SampleFormat: Result:='Sample Format';			{339}

 SMinSampleValue: Result:='SMin	Sample Value';		{340}
 SMaxSampleValue: Result:='SMax	Sample Value';		{341}
 TransferRange : Result:='TransferRange';		{342}
 ClipPath: Result:='Clip Path';				{343}
 XClipPathUnits: Result:='X Clip Path Units'; 		{344}
 YClipPathUnits: Result:='Y Clip Path Units'; 		{345}
 Indexed: Result:='Indexed';				{346}
 JPEGTables: Result:='JPEG Tables';			{347}

 OPIProxy: Result:='OPI	Proxy';				{351}

 GlobalParametersIFD : Result:='GlobalParametersIFD';                  	{400}
 ProfileType : Result:='ProfileType';                                  	{401}
 FaxProfile : Result:='FaxProfile';                                    	{402}
 CodingMethods : Result:='CodingMethods';                              	{403}
 VersionYear : Result:='VersionYear';                                  	{404}
 ModeNumber : Result:='ModeNumber';                                    	{405}
 Decode : Result:='Decode';                                            	{433}
 DefaultImageColor : Result:='DefaultImageColor';                      	{434}
 JPEGProc : Result:='JPEGProc';                                        	{512}
 StripRowCounts : Result:='StripRowCounts';                            	{559}


 JPEGIFOffset: Result:='JPEG IF	Offset';				{513}
 JPEGIFByteCount: Result:='JPEG	IF Byte	Count';				{514}
 JPEGREStartInterval: Result:='JPEG ReStart Interval';			{515}
 JPEGLossLessPredictors: Result:='JPEG Loss Less Predictors';		{517}
 JPEGPointTransform: Result:='JPEG Point Transform';			{518}
 JPEGQTables: Result:='JPEG Q Tables';					{519}
 JPEGDCTables: Result:='JPEG DC	Tables';				{520}
 JPEGACTables: Result:='JPEG AC	Tables';				{521}
 YCBCRCoefficients: Result:='YCBCR Coefficients';			{529}
 YCBCRSubSampling: Result:='YCBCR Sub Sampling';			{530}
 YCBCRPositioning: Result:='YCBCR Positioning';				{531}
 ReferenceBlackWhite: Result:='Reference Black White';			{532}

 XMLPacket: Result:='XML Packet';					{700}

 ProjFalseOriginEasting : Result:='ProjFalseOriginEasting';            	{3086}
 ProjFalseOriginNorthing : Result:='ProjFalseOriginNorthing';          	{3087}
 ProjCenterLong : Result:='ProjCenterLong';                            	{3088}
 ProjCenterLat : Result:='ProjCenterLat';                              	{3089}
 ProjCenterEasting : Result:='ProjCenterEasting';                      	{3090}
 ProjCenterNorthing : Result:='ProjCenterNorthing';                    	{3091}
 ProjScaleAtNatOrigin : Result:='ProjScaleAtNatOrigin';                	{3092}
 ProjScaleAtCenter : Result:='ProjScaleAtCenter';                      	{3093}
 ProjAzimuthAngle : Result:='ProjAzimuthAngle';                        	{3094}
 ProjStraightVertPoleLong : Result:='ProjStraightVertPoleLong';        	{3095}
 ProjRectifiedGridAngle : Result:='ProjRectifiedGridAngle';            	{3096}
 VerticalCSType : Result:='VerticalCSType';                            	{4096}
 VerticalCitation : Result:='VerticalCitation';                        	{4097}
 VerticalDatum : Result:='VerticalDatum';                              	{4098}
 VerticalUnits : Result:='VerticalUnits';                              	{4099}

 OPIImageID: Result:='OPI ImageID';					{32781}
 WangAnnotation : Result:='Wang Annotation';                          	{32932}
 REFPoints: Result:='REF Points';					{32953}
 RegionTackPoint: Result:='Region Tack Point';				{32954}
 RegionWarpCorners: Result:='Region Warp Corners';			{32955}
 RegionAffine: Result:='Region Affine';					{32956}
 Matteing: Result:='Matteing';						{32995}
 DataType: Result:='Data Type';						{32996}
 ImageDepth: Result:='Image Depth';					{32997}
 TileDepth: Result:='Tile Depth';					{32998}

 PixarImageFullWidth: Result:='Pixar Image Full Width';			{33300}
 PixarImageFullLength: Result:='Pixar Image Full Length';		{33301}
 PixarTextureFormat: Result:='Pixar Texture Format';			{33302}
 PixarWrapModes: Result:='PixarWrap Modes';				{33303}
 PixarFOVCOT: Result:='Pixar FOVCOT';					{33304}
 PixarMatrixWorldToScreen: Result:='Pixar Matrix World To Screen';	{33305}
 PixarMatrixWorldToCamera: Result:='Pixar Matrix World To Camera';	{33306}
 WriterSerialNumber: Result:='Writer Serial Number';			{33405}
 Copyright: Result:='Copyright';					{33432}

 ExposureTime : Result:='ExposureTime';                                	{33434}
 FNumber : Result:='FNumber';                                          	{33437}
 MDFileTag : Result:='MD FileTag';                                     	{33445}
 MDScalePixel : Result:='MD ScalePixel';                               	{33446}
 MDColorTable : Result:='MD ColorTable';                               	{33447}
 MDLabName : Result:='MD LabName';                                     	{33448}
 MDSampleInfo : Result:='MD SampleInfo';                               	{33449}
 MDPrepDate : Result:='MD PrepDate';                                   	{33450}
 MDPrepTime : Result:='MD PrepTime';                                   	{33451}
 MDFileUnits : Result:='MD FileUnits';                                 	{33452}
 ModelPixelScaleTag : Result:='ModelPixelScaleTag';                    	{33550}
 RichTIFFIPTC: Result:='Rich TIFF IPTC';				{33723}
 INGRPacketDataTag : Result:='INGR Packet Data Tag';                	{33918}
 INGRFlagRegisters : Result:='INGR Flag Registers';                  	{33919}
 IrasBTransformationMatrix : Result:='IrasB Transformation Matrix';  	{33920}
 ModelTiepointTag : Result:='ModelTiepointTag';                        	{33922}


 IT8SiteName: Result:='IT8 Site Name';					     	{34016}
 IT8ColorSequence: Result:='IT8Color Sequence';					{34017}
 IT8Header: Result:='IT8 Header';						{34018}
 IT8RasterPadding: Result:='IT8Raster Padding';					{34019}
 IT8BitsPerRunLength: Result:='IT8 BitsPer RunLength';				{34020}
 IT8BitsPerExtendedRunLength: Result:='IT8 BitsPer Extended Run Length';	{34021}
 IT8ColorTable:Result:='IT8 Color Table';					{34022}
 IT8ImageColorIndicator: Result:='IT8 Image Color Indicator';			{34023}
 IT8BKGColorIndicator: Result:='IT8 BKGColor Indicator';			{34024}
 IT8ImageColorValue: Result:='IT8 ImageColor Value';				{34025}
 IT8BKGColorValue: Result:='IT8BKG Color Value';				{34026}
 IT8PixelIntensityRange: Result:='IT8 Pixel Intensity Range';			{34027}
 IT8TransparencyIndicator: Result:='IT8Transparency Indicator';			{34028 }
 IT8ColorCharacterization: Result:='IT8Color Characterization';			{34029 }
 IT8HCUsage: Result:='IT8 HC Usage';						{34030}
 IT8TrapIndicator: Result:='IT8Trap Indicator';					{34031}
 IT8CMYKEquivalent: Result:='IT8 CMYK Equivalent';				{34032}
 FrameCount: Result:='Frame Count';						{34232}
 ModelTransformationTag : Result:='ModelTransformationTag';            		{34264}
 PhotoShop: Result:='Photo Shop';						{34377}
 EXIFIFD: Result:='EXIFIFD';							{34665}
 ICCProfile: Result:='ICC Profile';						{34675}
 ImageLayer : Result:='ImageLayer';                                    		{34732}
 GeoKeyDirectoryTag : Result:='GeoKeyDirectoryTag';                    		{34735}
 GeoDoubleParamsTag : Result:='GeoDoubleParamsTag';                    		{34736}
 GeoAsciiParamsTag : Result:='GeoAsciiParamsTag';                      		{34737}
 JBIGOptions: Result:='JBIG Options';						{34750}
 ExposureProgram : Result:='ExposureProgram';                          		{34850}
 SpectralSensitivity : Result:='SpectralSensitivity';                  		{34852}
 GPSIFD: Result:='GPS IFD';							{34853}
 ISOSpeedRatings : Result:='ISOSpeedRatings';                          		{34855}
 OECF : Result:='OECF';                                                		{34856}
 FAXRecVParams:Result:='FAX Rec VParams';					{34908}
 FaxSubaddress:Result:='Fax Subaddress';					{34909}
 FAXRecVTime: Result:='FAX Rec VTime';						{34910}
 FAXDCS: Result:='FAX DCS';							{34911}
 FEDEXEDR: Result:='FEDEXE DR';							{34929}
 ExifVersion : Result:='ExifVersion';                                  		{36864}
 DateTimeOriginal : Result:='DateTimeOriginal';                        		{36867}
 DateTimeDigitized : Result:='DateTimeDigitized';                      		{36868}
 ComponentsConfiguration : Result:='ComponentsConfiguration';          	     	{37121}
 CompressedBitsPerPixel : Result:='CompressedBitsPerPixel';            	     	{37122}
 ShutterSpeedValue : Result:='ShutterSpeedValue';                      	     	{37377}
 ApertureValue : Result:='ApertureValue';                              	     	{37378}
 BrightnessValue : Result:='BrightnessValue';                          	     	{37379}
 ExposureBiasValue : Result:='ExposureBiasValue';                      	     	{37380}
 MaxApertureValue : Result:='MaxApertureValue';                        	     	{37381}
 SubjectDistance : Result:='SubjectDistance';                          	     	{37382}
 MeteringMode : Result:='MeteringMode';                                	     	{37383}
 LightSource : Result:='LightSource';                                  	     	{37384}
 Flash : Result:='Flash';                                              	     	{37385}
 FocalLength : Result:='FocalLength';                                  	     	{37386}
 SubjectArea : Result:='SubjectArea';                                  	     	{37396}
  STONits: Result:='STO Nits';							{37439}
 MakerNote : Result:='MakerNote';                                      		{37500}
 UserComment : Result:='UserComment';                                  		{37510}
 SubsecTime : Result:='SubsecTime';                                    		{37520}
 SubsecTimeOriginal : Result:='SubsecTimeOriginal';                    		{37521}
 SubsecTimeDigitized : Result:='SubsecTimeDigitized';                  		{37522}
 ImageSourceData : Result:='ImageSourceData';                          		{37724}
 FlashpixVersion : Result:='FlashpixVersion';                          	{40960}
 ColorSpace : Result:='ColorSpace';                                    	{40961}
 PixelXDimension : Result:='PixelXDimension';                          	{40962}
 PixelYDimension : Result:='PixelYDimension';                          	{40963}
 RelatedSoundFile : Result:='RelatedSoundFile';                        	{40964}
 InterOperabilityIFD: Result:='Interoperability	IFD';				{40965}

 FlashEnergy : Result:='FlashEnergy';                                  	{41483}
 SpatialFrequencyResponse : Result:='SpatialFrequencyResponse';        	{41484}
 FocalPlaneXResolution : Result:='FocalPlaneXResolution';              	{41486}
 FocalPlaneYResolution : Result:='FocalPlaneYResolution';              	{41487}
 FocalPlaneResolutionUnit : Result:='FocalPlaneResolutionUnit';        	{41488}
 SubjectLocation : Result:='SubjectLocation';                          	{41492}
 ExposureIndex : Result:='ExposureIndex';                              	{41493}
 SensingMethod : Result:='SensingMethod';                              	{41495}
 FileSource : Result:='FileSource';                                    	{41728}
 SceneType : Result:='SceneType';                                      	{41729}
 CFAPattern : Result:='CFAPattern';                                    	{41730}
 CustomRendered : Result:='CustomRendered';                            	{41985}
 ExposureMode : Result:='ExposureMode';                                	{41986}
 WhiteBalance : Result:='WhiteBalance';                                	{41987}
 DigitalZoomRatio : Result:='DigitalZoomRatio';                        	{41988}
 FocalLengthIn35mmFilm : Result:='FocalLengthIn35mmFilm';              	{41989}
 SceneCaptureType : Result:='SceneCaptureType';                        	{41990}
 GainControl : Result:='GainControl';                                  	{41991}
 Contrast : Result:='Contrast';                                        	{41992}
 Saturation : Result:='Saturation';                                    	{41993}
 Sharpness : Result:='Sharpness';                                      	{41994}
 DeviceSettingDescription : Result:='DeviceSettingDescription';        	{41995}
 SubjectDistanceRange : Result:='SubjectDistanceRange';                	{41996}

 ImageUniqueID : Result:='ImageUniqueID';                              	{42016}
 GDAL_METADATA : Result:='GDAL_METADATA';                              	{42112}
 GDAL_NODATA : Result:='GDAL_NODATA';                                  	{42113}
 ChartFormat : Result:='ChartFormat';                                  	{47001}
 ChartSource : Result:='ChartSource';                                  	{47002}
 ChartSourceEdition : Result:='ChartSourceEdition';                    	{47003}
 ChartSourceDate : Result:='ChartSourceDate';                          	{47004}
 ChartCorrDate : Result:='ChartCorrDate';                              	{47005}
 ChartCountryOrigin : Result:='ChartCountryOrigin';                    	{47006}
 ChartRasterEdition : Result:='ChartRasterEdition';                    	{47007}
 ChartSoundingDatum : Result:='ChartSoundingDatum';                    	{47008}
 ChartDepthUnits : Result:='ChartDepthUnits';                          	{47009}
 ChartMagVar : Result:='ChartMagVar';                                  	{47010}
 ChartMagVarYear : Result:='ChartMagVarYear';                          	{47011}
 ChartMagVarAnnChange : Result:='ChartMagVarAnnChange';                	{47012}
 ChartWGSNSShift : Result:='ChartWGSNSShift';                          	{47013}
 InsetNWPixelX : Result:='InsetNWPixelX';                              	{47015}
 OceScanjobDescription : Result:='Oce Scanjob Description';           	{50215}
 OceApplicationSelector : Result:='Oce Application Selector';         	{50216}
 OceIdentificationNumber : Result:='Oce Identification Number';       	{50217}
 OceImageLogicCharacteristics : Result:='Oce ImageLogic Characteristics';{50218}
 AliasLayerMetadata : Result:='Alias Layer Metadata';                	{50784}
 TIFF_RSID : Result:='TIFF_RSID';                                      	{50908}
 GEO_METADATA : Result:='GEO_METADATA';                                	{50909}

 DNGVersion: Result:='DNG Version';					{50706}
 DNGBackwardVersion: Result:='DNG Backward Version';			{50707}
 UniqueCameraModel: Result:='Unique Camera Model';			{50708}
 LocalizedCameraModel: Result:='Localized Camera Model';		{50709}
 CFAPlaneColor:	Result:='CFA Plane Color';				{50710}
 CFALayout: Result:='CFA Layout';					{50711}
 LinearizationTable: Result:='Linearization Table';			{50712}
 BlackLevelRepeatDIM: Result:='Black Level Repeat DIM';			{50713}
 BlackLevel: Result:='Black Level';					{50714}
 BlackLevelDeltaH: Result:='Black Level	DeltaH';			{50715}
 BlackLevelDeltaV: Result:='Black Level	DeltaV';			{50716}
 WhiteLevel: Result:='White Level';					{50717}
 DefaultScale: Result:='Default	Scale';					{50718}
 DefaultCropOrigin: Result:='Default Crop Origin';			{50719}
 DefaultCropSize: Result:='Default Crop	Size';				{50720}
 ColorMatrix1: Result:='Color Matrix 1';				{50721}
 ColorMatrix2: Result:='Color Matrix 2';				{50722}
 CameraCalibration1: Result:='Camera Calibration 1';			{50723}
 CameraCalibration2: Result:='Camera Calibration 2';			{50724}
 ReductionMatrix1: Result:='Reduction Matrix 1';			{50725}
 ReductionMatrix2: Result:='Reduction Matrix 2';			{50726}
 AnalogBalance:	Result:='Analog	Balance';				{50727}
 ASShotNeutral:	Result:='ASShot	Neutral';				{50728}
 ASShotWhiteXY:	Result:='ASShot	White XY';				{50729}
 BaseLineExposure: Result:='Base Line Exposure';			{50730}
 BaseLineNoise:	Result:='Base Line Noise';				{50731}
 BaseLineSharpness: Result:='Base Line Sharpness';			{50732}
 BayerGreenSplit: Result:='Bayer Green Split';				{50733}
 LinearResponseLimit: Result:='Linear Response Limit';			{50734}
 CameraSerialNumber: Result:='Camera Serial Number';			{50735}
 LensInfo: Result:='Lens Info';						{50736}
 ChromaBlurRadius: Result:='Chroma Blur	Radius';			{50737}
 AntiAliasStrength: Result:='Anti Alias	Strength';			{50738}
 ShadowScale: Result:='Shadow Scale';					{50739}
 DNGPrivateData: Result:='DNG Private Data';				{50740}
 MakerNoteSafety: Result:='Maker Note Safety';				{50741}
 CalibrationIlluminant1: Result:='Calibration Illuminant 1';		{50778}
 CalibrationIlluminant2: Result:='Calibration Illuminant 2';		{50779}
 BestQualityScale: Result:='Best Quality Scale';			{50780}
 RawDataUniqueID: Result:='Raw Data Unique ID';				{50781}
 OriginalRawFilename: Result:='Original	Raw Filename';			{50827}
 OriginalRawFiledata: Result:='Original	Raw Filedata';			{50828}
 ActiveArea: Result:='Active Area';					{50829}
 MaskedAreas: Result:='Masked Areas';					{50830}
 ASShotICCProfile: Result:='ASShot ICC Profile';			{50831}
 AsShotPreProfileMatrix: Result:='As Shot Pre Profile Matrix';		{50832}
 CurrentICCProfile: Result:='Current ICC Profile';			{50833}
 CurrentPreProfileMatrix: Result:='Current Pre Profile Matrix';		{50834}
 DCSHueShiftValues: Result:='DCSHue Shift Values';			{65535}
 else Result:='Unknown';
 end;
end;



{------------------ TIFF Support ------------------------------------}

function SignedByteToInteger(B: byte): integer;
{Convert byte into signed integer }
begin
if B>$7F then Result:=B or $FFFFFF00
else Result:=B;
end;


function Pixels2Bytes(N: integer): integer;
{Tiff specified method of calculating the
number bytes to hold N Pixels}
begin
Result:=(N+7) div 8;
end;



function BytesPerScanline(PixelsPerScanline, BitsPerPixel, Alignment: Longint): Longint;
{Returns numbers of bytes per scanline based on alignment}
{Windoww bitmaps must be DWord aligned }
begin
Dec(Alignment);
Result := ((PixelsPerScanline * BitsPerPixel) + Alignment) and not Alignment;
Result := Result div 8;
end;




procedure WriteTIFTag(Stream: TStream; Tag,TagType: Word; Length,Offset: DWord);
{Write one TIFF tag to the IFD  }
begin
WriteStreamWord(Stream,Tag);
WriteStreamWord(Stream,TagType);
WriteStreamDWord(Stream,Length);
WriteStreamDWord(Stream,Offset);
inc(TagCount);
end;




procedure WriteStreamRational(Stream: TStream; N,D: DWord);
begin
Stream.Write(N,4);
Stream.Write(D,4);
end;


function RationalToFloat(N,D: DWord): double;
begin

end;


{------------------- Handle Progress Event --------------------------}


procedure AdvanceProgress(Stage: TProgressStage; Steps: integer);
{Increment progress value and calls Progress event }
{if a procedure is assigned to FOnProgress, it is
{called periodically during file loading process }
var DrawNow: boolean;
var Msg: string;
begin
DrawNow:=False;
case Stage of
 psStarting:
	begin
	ProgressValue:=0;
	ProgressInc:=100/Steps;
	Msg:='Starting';
	end;
 psRunning:
	begin
	ProgressValue:=ProgressValue+ProgressInc;
	Msg:='Running';
	end;
 psEnding:
	begin
	DrawNow:=True;
	Msg:='Ending';
	end;
 end;

if Assigned(OnTIFProgress) then
    OnTIFProgress(nil,Stage,trunc(ProgressValue),DrawNow,Rect(0,0,0,0),Msg);
end;


end.
