unit TifWrite;

interface

uses Windows,Classes,Graphics,SysUtils,DSupport,TifGlobal,GeoTiff,StreamLib, BitmapLib;

procedure SaveTIFFToFile(FileName: string; Bitmap: TBitmap; GeoTiff,Compress: boolean);
procedure SaveTIFFToStream(Stream: TStream; Bitmap:TBitmap; GeoTiff,Compress: boolean);


implementation


{Set to separate RGB bitmaps from palettized}

const TRGBBitmap: set of TPixelFormat=[pf15Bit,pf16Bit,pf24bit];

{Set of bitmaps that can't be converted - note TIF does seem to handle 32-bit correctly}

const TInvalidBitmap: set of TPixelFormat=[pfCustom,pfDevice,pf32bit];



type TBGRTripleByte = record
 bgrBlue,bgrGreen,bgrRed: Byte;
 end;

type TBGRArray = array [0..32767] of TBGRTripleByte;
type PBGRArray = ^TBGRArray;


type TRGBarray = array [0..32767] of TRGBTriple;
type pRGBarray = ^TRGBarray;

type TTifPalette = array [0..2,0..255] of Word;


type TPaletteEntries = array [Byte] of TPaletteEntry;

const SoftwareName= 'Fountain';



{---------------- Compression schemes ---------------------------}

procedure WritePackBitsLine(Stream: TStream; P: PByteArray; N: integer);
{Do packbits compression for one line of image data  }
var Literals: array [0..127] of byte;
var LitInx,I,OutCnt,InIndex: word;


	procedure WriteLiterals;
	{Write accumulated literals}
	begin
	WriteStreamByte(Stream,(LitInx-1) and $7F);
	Inc(OutCnt);
	Stream.Write(Literals,LitInx);
	Inc(OutCnt,LitInx);
	LitInx:=0;
	end;

	procedure WriteByteCode;
	{Write code count and value}
	begin
	WriteStreamByte(Stream,not I+1);
	WriteStreamByte(Stream,P[InIndex+I]);
	Inc(OutCnt,2);
	Inc(InIndex,I+1);
	end;


begin
LitInx:=0; I:=0;
OutCnt:=0; InIndex:=0;
repeat
	begin 
	I:=0;
	while (P[InIndex+I] = P[InIndex+I+1]) and (I < 127) and
	      (I < (N-1)) and ((InIndex+I+1) < N) do Inc(I);

	if (I > 1) or (LitInx >= 127) then
		begin 
		if LitInx<>0 then WriteLiterals;
		if I<>0 then WriteByteCode;
		end
	else
		begin
		Literals[LitInx]:=P[InIndex];
		Inc(LitInx); Inc(InIndex);
		end;
	end
until InIndex >= N;

if LitInx<>0 then WriteLiterals;
if (OutCnt and $01)<>0  then WriteStreamByte(Stream,$80);
end;



{------------------ Routines to handle Paletted BMPs --------------------}


procedure BMP2TifPalette(Columns: integer; Format: TPixelFormat; BMPPal: TPaletteEntries; var TifPal: TTifPalette);
{Convert BMP palette to TIFF Palette}
var I: integer;
begin
for I:=0 to Columns-1 do
	begin
	case Format of
	 pf1Bit:
		begin
		TifPal[0,I]:=BMPPal[I].peRed + BMPPal[I].peRed shl 8 ;
		TifPal[1,I]:=BMPPal[I].peGreen + BMPPal[I].peGreen shl 8;
		TifPal[2,I]:=BMPPal[I].peBlue + BMPPal[I].peBlue shl 8;
		end;
	 pf4Bit:
		begin
		TifPal[0,I]:=BMPPal[I].peRed + BMPPal[I].peRed shl 8;
		TifPal[0,I+16]:=BMPPal[I].peGreen + BMPPal[I].peGreen shl 8;
		TifPal[0,I+32]:=BMPPal[I].peBlue + BMPPal[I].peBlue shl 8;
		end;
	 pf8Bit:
		begin
		TifPal[0,I]:=BMPPal[I].peRed shl 8;
		TifPal[1,I]:=BMPPal[I].peGreen shl 8;
		TifPal[2,I]:=BMPPal[I].peBlue shl 8;
		end;
	 end;
	end;
end;


procedure GetPaletteParams(Bitmap: TBitmap; var ColorsPerPal,BitsPerPixel: integer);
{Get parameters based on palette format}
begin
case Bitmap.PixelFormat of
 pf1Bit: BitsPerPixel:=1;
 pf4Bit: BitsPerPixel:=4;
 pf8Bit: BitsPerPixel:=8;
 end;
ColorsPerPal:=1 shl BitsPerPixel;
end;



function Pixels2Bytes(N: integer): integer;
{Tiff specified method of calculating the
number bytes to hold N Pixels}
begin
Result:=(N+7) div 8;
end;


procedure OutputScanLine(Stream: TStream; P: PByteArray; N: integer; Compress: boolean);
{Output compressed or uncompressed scan line}
begin
if Compress then WritePackBitsLine(Stream, P, N)
else Stream.Write(P^,N);
end;




procedure SaveTIFFToStream(Stream: TStream; Bitmap: TBitmap; GeoTiff,Compress: boolean);
{Save Bitmap as TIFF file}
var ColorsPerPal,BitsPerPixel: Integer;
var ColorMapLoc,IFDLoc, XResLoc,YResLoc,BitPerSampleLoc,SoftNameLoc, StripLoc: Integer;
var BitmapPalette: TPaletteEntries;
var TifPalette: TTifPalette;
var RGBFormat: boolean;
var CompressTag: DWord;
var NX,DX,NY,DY: integer;


	procedure SendPaletteTags(Stream: TStream);
	var PM: DWord;
	begin
	if Bitmap.PixelFormat=pf1bit then
		begin
		if BitmapPalette[0].peRed <> 0 then PM:=Photometric_ZeroIsWhite
		else PM:=Photometric_ZeroIsBlack;
		end
	else PM:=PHOTOMETRIC_PALETTE;

	WriteTIFTag(Stream, NewSubFile, tifLong, 1, 0 );
	WriteTIFTag(Stream, ImageWidth, tifShort, 1, Bitmap.Width );
	WriteTIFTag(Stream, ImageLength, tifShort, 1, Bitmap.Height );
	WriteTIFTag(Stream, BitsPerSample, tifShort, 1, BitsPerPixel );
	WriteTIFTag(Stream, Compression, tifShort, 1, CompressTag );
	WriteTIFTag(Stream, PhotoMetricInterp, tifShort, 1, PM);
	WriteTIFTag(Stream, StripOffsets, tifLong, 1, StripLoc);
	WriteTIFTag(Stream, SamplesPerPixel, tifShort, 1, 1 );
	WriteTIFTag(Stream, RowsPerStrip, tifLong, 1, Bitmap.Height);
	WriteTIFTag(Stream, StripByteCounts, tifLong, 1, Bitmap.Width * Bitmap.Height );
	WriteTIFTag(Stream, XResolution, tifFractional, 1,  XResLoc);
	WriteTIFTag(Stream, YResolution, tifFractional, 1,  YResLoc );
	WriteTIFTag(Stream, ResolutionUnit, tifShort, 1, RESUNIT_INCH);
	WriteTIFTag(Stream, Software, tifASCII, Length(SoftwareName), SoftNameLoc );
	WriteTIFTag(Stream, ColorMap, tifShort, ColorsPerPal * 3, ColorMapLoc );
	end;



	procedure SendRGBTags(Stream: TStream);
	begin
	WriteTIFTag(Stream, NewSubFile, tifLong, 1, 0 );
	WriteTIFTag(Stream, ImageWidth, tifShort, 1, Bitmap.Width);
	WriteTIFTag(Stream, ImageLength, tifShort, 1,Bitmap.Height);
	WriteTIFTag(Stream, BitsPerSample, tifShort, 1, 8 );
	WriteTIFTag(Stream, Compression, tifShort, 1, CompressTag);
	WriteTIFTag(Stream, PhotoMetricInterp, tifShort, 1, PHOTOMETRIC_RGB);
	WriteTIFTag(Stream, StripOffSets, tifLong, 1, StripLoc );
	WriteTIFTag(Stream, SamplesPerPixel, tifShort, 1, 3 );
	WriteTIFTag(Stream, RowsPerStrip, tifLong, 1, Bitmap.Height );
	WriteTIFTag(Stream, StripByteCounts, tifLong, 1, 3 * Bitmap.Width * Bitmap.Height );
	WriteTIFTag(Stream, XResolution, tifFractional, 1, XResLoc);
	WriteTIFTag(Stream, YResolution, tifFractional, 1, YResLoc);
	WriteTIFTag(Stream, PlanarConfiguration, tifShort, 1, PLANARCONFIG_CONTIG );
	WriteTIFTag(Stream, ResolutionUnit, tifShort, 1, RESUNIT_INCH);
	WriteTIFTag(Stream, Software, tifASCII,Length(SoftwareName), SoftNameLoc);
	WriteTIFTag(Stream, ExtraSamples, tifASCII, 1, EXTRASAMPLE_ASSOCALPHA );
	end;



	procedure SendTags(RGBTags: boolean);
	var TempStream: TMemoryStream;
	begin
	TempStream:=TMemoryStream.Create;
	try
	IFDLoc := Stream.Position;
	TempStream.Position:=0;
	TagCount:=0;
	if RGBTags then SendRGBTags(TempStream)
	else SendPaletteTags(TempStream);
	if GeoTiff then WriteGeoTiffTags(TempStream);
	WriteStreamWord(Stream,TagCount);
	TempStream.Position:=0;
	Stream.CopyFrom(TempStream,TempStream.Size);
	finally TempStream.Free; end;
	end;


	procedure WritePalette;
	begin
	GetPaletteParams(Bitmap,ColorsPerPal,BitsPerPixel);
	GetPaletteEntries(Bitmap.Palette,0,ColorsPerPal,BitmapPalette);
	BMP2TifPalette(ColorsPerPal,Bitmap.PixelFormat,BitmapPalette,TifPalette);
	ColorMapLoc:=Stream.Position;
	Stream.Write(TifPalette, 2 * 3 * ColorsPerPal);
	end;


	procedure SendPaletizedImage;
	var I: integer;
	var SrcScanLine: PByteArray;
	var N: integer;
	var TempStream: TMemoryStream;
	begin
	TempStream:=TMemoryStream.Create;
	TempStream.SetSize(Bitmap.Width * Bitmap.Height);
	try
	StripLoc:=Stream.Position;
	for I:=0 to Bitmap.Height-1 do
		begin
		AdvanceProgress(psRunning,0);
		SrcScanLine:=Bitmap.ScanLine[I];
		N:=Bitmap.Width;
		OutputScanline(TempStream, SrcScanLine,N,Compress);
		end;
	TempStream.Position:=0;
	Stream.WriteBuffer(TempStream.Memory^,TempStream.Size);
	finally TempStream.Free; end;
	end;


	procedure SendRGBImage;
	var I,J: integer;
	var SrcRow: pRGBArray;
	var DestRow: TBGRArray;
	var RGB: TRGBTriple;
	begin
	StripLoc := Stream.Position;
	for I:= 0 to Bitmap.Height-1 do
		begin
		AdvanceProgress(psRunning,0);
		SrcRow:=Bitmap.ScanLine[I];
		for J:=0 to Bitmap.Width -1 do
			begin
			RGB:=SrcRow[J];
			DestRow[J].bgrBlue:=RGB.rgbtRed;
			DestRow[J].bgrGreen:=RGB.rgbtGreen;
			DestRow[J].bgrRed:=RGB.rgbtBlue;
			end;
		OutputScanline(Stream, PByteArray(@DestRow),SizeOf(TBGRTripleByte) * Bitmap.Width,Compress);
		end;
	end;


	procedure GetResolution(BM: TBitmap; var NX,DX,NY,DY: integer);
	var PPMX,PPMY: integer;
	begin
	GetBmpPPM(BM,PPMX,PPMY);
	NX:=round(PPMX / Meters2Inches);
	NY:=round(PPMY / Meters2Inches);
	DX:=1; DY:=1;
	end;



begin
GetResolution(Bitmap,NX,DX,NY,DY);
if Bitmap.PixelFormat in TInvalidBitmap then raise Exception.Create('Invalid Pixel Format');
RGBFormat:=Bitmap.PixelFormat in TRGBBitmap;
if Compress then CompressTag:=Compress_PACKBITS
else CompressTag:=Compress_NONE;
AdvanceProgress(psStarting,Bitmap.Height);


{  write the header  }
WriteStreamString(Stream,'II');
StreamEndian:=LittleEnd;
WriteStreamWord(Stream,42);
WriteStreamDWord(Stream,0);

{Write tag values and save the locations }
XResLoc := Stream.Position;
WriteStreamRational(Stream,NX,DX);
YResLoc := Stream.Position;
WriteStreamRational(Stream,NY,DY);

BitPerSampleLoc := Stream.Position;
Stream.Write(BitsPerPixel,sizeof(BitsPerPixel));
SoftNameLoc := Stream.Position;
WriteStreamString(Stream,SoftwareName);
if RGBFormat then SendRGBImage
else
 	begin
	WritePalette;
	SendPaletizedImage;
	end;
if GeoTiff then WriteGeoData(Stream);
SendTags(RGBFormat);
WriteStreamDWord(Stream,0);
Stream.Seek(4, soFromBeginning ) ;
Stream.Write(IFDLoc, sizeof(IFDLoc));
AdvanceProgress(psEnding,0);

OnTIFProgress:=nil;
end;



procedure SaveTIFFToFile(FileName: String; Bitmap: TBitmap; GeoTiff,Compress: boolean);
var Stream: TMemoryStream;
begin
Stream := TMemoryStream.Create;
SaveTIFFToStream(Stream,Bitmap,GeoTiff,Compress);
Stream.SaveToFile(filename);
Stream.Free;
end;



end.
