unit TreeLib;

interface

uses Windows,Messages,Controls,ComCtrls,ExtCtrls,ThreadLib,SysUtils,DSupport;


type TAutoScroller = class(TObject)
  private
    FWindow: TWinControl;
    FTimer: TTimer;
    procedure ScrollWindow(Up: boolean);
    procedure DoScroll(Sender: TObject);
    function GetCursorPosition: TPoint;
    function IsScrollPoint(Window: TWinControl; PT: TPoint): boolean;
  protected
  public
    Scrolling: boolean;
    Interval: integer;
    constructor Create;
    destructor Destroy; override;
    procedure HandleScroll(Window: TWinControl);
  end;



type TAutoExpander = class(TObject)
  private
    FTreeView: TCustomTreeView;
    FTimer: TTimer;
    procedure DoExpand(Sender: TObject);
  protected
  public
    Expanding: boolean;
    Delay: integer;
    constructor Create;
    destructor Destroy; override;
    procedure HandleExpand(Tree: TCustomTreeView);
  end;



implementation


{-----------------------------------------------------------------------------}


constructor TAutoExpander.Create;
begin
inherited Create;
FTimer:=TTimer.Create(nil);
FTimer.Enabled:=False;
FTimer.OnTimer:=DoExpand;
Delay:=700;
end;


destructor TAutoExpander.Destroy;
begin
FTimer.Free;
inherited Destroy;
end;



procedure TAutoExpander.DoExpand(Sender: TObject);
var PT: TPoint;
var Node: TTreeNode;
begin
FTimer.Enabled:=False;
GetCursorPos(PT);
PT:=FTreeView.ScreenToClient(PT);
Node:=FTreeView.GetNodeAt(PT.X,PT.Y);
if Node<>nil then Node.Expand(False);
FTreeView.Repaint;
FTreeView.ControlStyle := FTreeView.ControlStyle + [csDisplayDragImage];
Expanding:=False;
end;


procedure TAutoExpander.HandleExpand(Tree: TCustomTreeView);
var PT: TPoint;
var Node: TTreeNode;
begin
if Expanding then exit;
FTreeView:=Tree;
GetCursorPos(PT);
PT:=FTreeView.ScreenToClient(PT);
Node:=FTreeView.GetNodeAt(PT.X,PT.Y);
if (Node<>nil) and Node.HasChildren and not Node.Expanded then
	begin
	Expanding:=True;
	FTreeView.ControlStyle := FTreeView.ControlStyle - [csDisplayDragImage];
	FTimer.Interval:=Delay;
	FTimer.Enabled:=True;
	end;
end;


{-----------------------------------------------------------------------------}


function TAutoScroller.GetCursorPosition: TPoint;
begin
GetCursorPos(Result);
Result:=FWindow.ScreenToClient(Result);
end;



constructor TAutoScroller.Create;
begin
inherited Create;
FTimer:=TTimer.Create(nil);
FTimer.Enabled:=False;
FTimer.OnTimer:=DoScroll;
Interval:=300;
Scrolling:=False;
end;

destructor TAutoScroller.Destroy;
begin
FTimer.Free;
inherited Destroy;
end;


procedure TAutoScroller.ScrollWindow(Up: boolean);
var Direction: integer;
begin
if Up then Direction:=SB_LINEUP else Direction:=SB_LINEDOWN;
FWindow.Perform(WM_VSCROLL,Direction,0);
FWindow.Repaint;
end;


function  TAutoScroller.IsScrollPoint(Window: TWinControl; PT: TPoint): boolean;
const Margin = 12;
var Position,Min,Max: integer;
var NotMax,NotMin: boolean;
begin
GetScrollRange(Window.Handle,SB_VERT,Min,Max);
Position:=GetScrollPos(Window.Handle,SB_VERT);
NotMax:=Position <> Max;
NotMin:=Position <> Min;
Result:=((PT.X>=0) and (PT.X<=Window.ClientWidth)) and
(((PT.Y>=-Margin) and (PT.Y<Margin) and NotMin) or
((PT.Y>(Window.ClientHeight-Margin)) and (PT.Y<Window.ClientHeight+Margin) and NotMax));
end;



procedure DisplayCursor(Display: boolean);
begin
if Display then repeat until ShowCursor(True)>0
else repeat until ShowCursor(False)<1;
end;




procedure TAutoScroller.DoScroll(Sender: TObject);
var PT: TPoint;
begin
PT:=GetCursorPosition;
if not IsScrollPoint(FWindow,PT) then FTimer.Enabled:=False;
if PT.Y<10 then ScrollWindow(True)
else if PT.Y>(FWindow.ClientHeight-10) then ScrollWindow(False);
end;



procedure TAutoScroller.HandleScroll(Window: TWinControl);
var PT: TPoint;
begin
FWindow:=WIndow;
PT:=GetCursorPosition;
if IsScrollPoint(FWindow,PT) then
	begin
	FWindow.ControlStyle := FWindow.ControlStyle - [csDisplayDragImage];
	Scrolling:=True;
	FTimer.Interval:=Interval;
	FTimer.Enabled:=True;
	end
else
	begin
	FTimer.Enabled:=False;
	Scrolling:=False;
	FWindow.ControlStyle := FWindow.ControlStyle + [csDisplayDragImage];
	end;
end;





end.
