unit WebImageLoader;

interface

uses Classes,SysUtils,Graphics,GifImage,NMHttp, IdBaseComponent, IdComponent, IdTCPConnection, IdTCPClient, IdHTTP,
     JPEG,PNGImage,BitmapLib,NetLib,StreamLib,WinHTTPClient;

var RawBitmapData: string;	{If the data is a bitmap, the raw data will be put here}

function LoadBitmapFromURL(IDHTTP: TIdHTTP; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
function LoadBitmapFromURL(NMHTTP: TNMHTTP; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
function LoadBitmapFromURL(IDHTTP: TIdHTTPEx; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
function LoadBitmapFromURL(HttpClient: TWinHttpClient; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;

function StreamToBitmap(Stream: TStream; URL: string; var BM: TBitmap; var BMList: TBitmapList): boolean;


procedure CopyGIFToBMList(var GifImage: TGIFImage; var BMList: TBitmapList);

implementation


{----------------- Image handling support routines  ---------------------------}


type TGraphicType = (gtUnknown,gtBmp,gtGIF,gtJPG,gtPNG);



procedure CopyGIFToBMList(var GifImage: TGIFImage; var BMList: TBitmapList);
var I: integer;
var BM: TBitmap;
var SI,BackImage: TGIFSubImage;
begin
BM:=TBitmap.Create;
try
BMList.Clear;

for I:=0 to GIFImage.Images.Count-1 do
	begin
	SI:=GIFImage.Images[I];
	if not SI.Transparent then BackImage:=SI;
	if SI.Bitmap<>nil then
		begin
		{First image forms base, so just draw it}
		if (I=0) then BM.Assign(SI.Bitmap)
                else
			begin
			{Now draw on previous surface with transparent parts showing through}
			SI.StretchDraw(BM.Canvas,SI.BoundsRect,SI.Transparent,false);
			end;
		BMList.AddBitmap(BM);
		end;
	end;
finally BM.Free; end;
end;



procedure GIFStreamToBMP(M: TStream; var BM: TBitmap; var BMList: TBitmapList);
{Convert GIF in a memory stream to BMP}
{if BMLIst is not NIL then load all images}
var GifImage: TGIFImage;
begin
GIfImage := TGIfImage.Create;
try
GIfImage.LoadFromStream(M);
if BMList<>nil then CopyGIFToBMList(GIFImage,BMList);
BM.Assign(GIfImage.Bitmap);
BM.PixelFormat:=pf24Bit;
finally GIfImage.Free; end;
end;




procedure JPGStreamToBMP(M: TStream; var BM: TBitmap);
{Convert Jpeg in a memory stream to BMP}
var JPEGImage: TJPEGImage;
begin
JPEGImage:=TJPEGImage.Create;
try
JPEGImage.LoadFromStream(M);
JPEGImage.DIBNeeded;
BM.Assign(JPEGImage);
BM.PixelFormat:=pf24Bit;
finally JPEGImage.Free; end;
end;




procedure PNGStreamToBMP(M: TStream; var BM: TBitmap);
{Convert PNG in a memory stream to BMP}
var PNGObect: TPNGObject;
begin
PNGObect:=TPNGObject.Create;
try
PNGObect.LoadFromStream(M);
BM.Assign(PNGObect);
BM.PixelFormat:=pf24Bit;
finally PNGObect.Free; end;
end;




procedure BMPStreamToBMP(M: TStream; var BM: TBitmap);
{Convert BMP in a memory stream to BMP}
begin
BM.LoadFromStream(M);
BM.PixelFormat:=pf24Bit;
end;


function GetExtension(URL: string): string;
{Get normalized extention of URL string}
begin
Result:=ExtractFileExt(URL);
if (Length(Result)>0) and (Result[1]='.') then Delete(Result,1,1);
Result:=LowerCase(Result);
end;



function GetExtensionType(URL: string): TGraphicType;
{Get Graphic Extention type from file extension}
var Ext: string;
begin
Result:=gtUnknown;
Ext:=GetExtension(URL);
if Ext = 'bmp' then Result:=gtBmp
else if Ext = 'gif' then Result:=gtGIF
else if (Ext = 'jpg') or (Ext = 'jpeg') then Result:=gtJPG
else if Ext ='png' then Result:=gtPNG;
end;


{------------- Find file type based on signature bytes ------------------------}



const MaxSignature =10;

const BMPSignature: array [0..2] of byte = ($42,$4D,0);
const GIFSignature: array [0..4] of byte = ($47,$49,$46,$38,0);
const JPGSignature: array [0..3] of byte = ($FF,$D8,$FF,0);
const PNGSignature: array [0..8] of byte = ($89,$50,$4E,$47,$0D,$0A,$1A,$0A,0);




function TestSignature(Signature, Test: array of byte): boolean;
{Return true if Test matches signature}
var I: integer;
begin
Result:=False;
for I:=0 to MaxSignature-1 do
	begin
	if Signature[I]=0 then break;
	if Signature[I]<>Test[I] then exit;
	end;
Result:=True;
end;



function GetGraphicType(Memory: TStream): TGraphicType;
{Get Graphic type by file signature}
var B: array [0..MaxSignature-1] of byte;
begin
Result:=gtUnknown;
Memory.Position:=0;
Memory.ReadBuffer(B,MaxSignature);
if TestSignature(BMPSignature,B) then Result:=gtBMP
else if TestSignature(GIFSignature,B) then Result:=gtGIF
else if TestSignature(JPGSignature,B) then Result:=gtJPG
else if TestSignature(PNGSignature,B) then Result:=gtPNG;
end;



{------------- Actual Load routines ------------------------------------------}

function StreamToBitmap(Stream: TStream; URL: string; var BM: TBitmap; var BMList: TBitmapList): boolean;
{Analize stream and extract bitmap}
var GraphicType: TGraphicType;
begin
Result:=False;
GraphicType:=GetGraphicType(Stream);
if GraphicType = gtUnknown then GraphicType:=GetExtensionType(URL);

Stream.Position:=0;
case GraphicType of
 gtBMP: BMPStreamToBMP(Stream,BM);
 gtGIF: GIFStreamToBMP(Stream,BM,BMList);
 gtJPG: JPGStreamToBMP(Stream,BM);
 gtPNG: PNGStreamToBMP(Stream,BM);
 gtUnknown:
	begin
	RawBitmapData:=StreamToString(Stream);
	raise Exception.Create('Unknown Image Type: '+GetExtension(URL));
	end;
 end;
Result:=True;
end;

{------------- Actual Load routines ------------------------------------------}


function LoadBitmapFromURL(NMHTTP: TNMHTTP; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
{Load bitmap from URL based on Netmasters HTTP Component}
var Memory: TMemoryStream;
var S: string;
begin
Result:=False;
Memory:=TMemoryStream.Create;
try
NMHTTP.InputFileMode:=FALSE;
NMHTTP.Get(URL);
S:=NMHTTP.Body;
Memory.Position:=0;
Memory.WriteBuffer(Pointer(Integer(S))^,Length(S));
Result:=StreamToBitmap(Memory,URL,BM,BMList);
finally Memory.Free; end;
end;




function LoadBitmapFromURL(IDHTTP: TIdHTTP; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
{Load bitmap from URL based on Intelicom  HTTP Component}
var Memory: TMemoryStream;
begin
Result:=False;
Memory:=TMemoryStream.Create;
try
IDHTTP.Get(URL,Memory);
Result:=StreamToBitmap(Memory,URL,BM,BMList);
finally Memory.Free; end;
end;




function LoadBitmapFromURL(IDHTTP: TIdHTTPEx; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
{Load bitmap from URL based on Extended Intelicom  HTTP Component}
var Memory: TMemoryStream;
var S: string;
begin
Result:=False;
Memory:=TMemoryStream.Create;
try
IDHTTP.Get(URL);
S:=IDHTTP.Body;
Memory.Position:=0;
Memory.WriteBuffer(Pointer(Integer(S))^,Length(S));
Result:=StreamToBitmap(Memory,URL,BM,BMList);
finally Memory.Free; end;
end;


var X: integer;

function LoadBitmapFromURL(HttpClient: TWinHttpClient; URL: string; var BM: TBitmap; BMList: TBitmapList): boolean; overload;
{Load bitmap from URL based on My WinHTTP Component}
var Memory: TMemoryStream;
var S: string;
begin
Result:=False;
Memory:=TMemoryStream.Create;
try
if not HttpClient.Get(URL) then exit;
S:=HttpClient.Body;
Memory.Position:=0;
Memory.WriteBuffer(Pointer(Integer(S))^,Length(S));
Result:=StreamToBitmap(Memory,URL,BM,BMList);
finally Memory.Free; end;
end;


end.
