unit WinIPHelperLib;




{
Library based on the Windows IP-Help API. Contains headers for useful subroutines
in the Windows "iphlpapi.dll" as well as useful function based on the API. }

interface

uses Windows;

{Data for a Delph-style array of port information}

type TPortInfo = record
 AppName: ShortString;
 ProcessID: DWord;
 LocalAddress: ShortString;
 LocalPort: integer;
 RemoteAddress: ShortString;
 RemotePort: integer;
 State: DWord;
 UDP: boolean;
 end;
type PPortInfo = ^TPortInfo;

type TPortArray = array of TPortInfo;



const ANY_SIZE = 1;
const iphlpapi = 'iphlpapi.dll';

{Port State}

const MIB_TCP_STATE_CLOSED = 1;
const MIB_TCP_STATE_LISTEN = 2;
const MIB_TCP_STATE_SYN_SENT = 3;
const MIB_TCP_STATE_SYN_RCVD = 4;
const MIB_TCP_STATE_ESTAB = 5;
const MIB_TCP_STATE_FIN_WAIT1 = 6;
const MIB_TCP_STATE_FIN_WAIT2 = 7;
const MIB_TCP_STATE_CLOSE_WAIT = 8;
const MIB_TCP_STATE_CLOSING = 9;
const MIB_TCP_STATE_LAST_ACK = 10;
const MIB_TCP_STATE_TIME_WAIT = 11;
const MIB_TCP_STATE_DELETE_TCB = 12;

{Type of TCP port data requested (different types, return different arrays)}

type TCP_TABLE_CLASS = Integer;

const TCP_TABLE_BASIC_LISTENER = 0;
const TCP_TABLE_BASIC_CONNECTIONS = 1;
const TCP_TABLE_BASIC_ALL = 2;
const TCP_TABLE_OWNER_PID_LISTENER = 3;
const TCP_TABLE_OWNER_PID_CONNECTIONS = 4;
const TCP_TABLE_OWNER_PID_ALL = 5;
const TCP_TABLE_OWNER_MODULE_LISTENER = 6;
const TCP_TABLE_OWNER_MODULE_CONNECTIONS = 7;
const TCP_TABLE_OWNER_MODULE_ALL = 8;

{Data types returned by various commands}
{Only MIB_TCPTABLE_OWNER_PID has been implied currently.}

{ TCP_TABLE_BASIC_ALL			= MIB_TCPTABLE }
{ TCP_TABLE_BASIC_CONNECTIONS		= MIB_TCPTABLE }
{ TCP_TABLE_BASIC_LISTENER		= MIB_TCPTABLE }

{ TCP_TABLE_OWNER_MODULE_ALL		= MIB_TCPTABLE_OWNER_MODULE }
{ TCP_TABLE_OWNER_MODULE_CONNECTIONS	= MIB_TCPTABLE_OWNER_MODULE }
{ TCP_TABLE_OWNER_MODULE_LISTENER	= MIB_TCPTABLE_OWNER_MODULE }

{ TCP_TABLE_OWNER_PID_ALL		= MIB_TCPTABLE_OWNER_PID }
{ TCP_TABLE_OWNER_PID_CONNECTIONS	= MIB_TCPTABLE_OWNER_PID }
{ TCP_TABLE_OWNER_PID_LISTENER		= MIB_TCPTABLE_OWNER_PID }

{Data array returned by the TCP_TABLE_OWNER_PID_ALL command,
the TCP_TABLE_OWNER_PID_CONNECTIONS command and the
TCP_TABLE_OWNER_PID_LISTENER command}

type TMibTcpRowOwnerPid  = packed record
 dwState: DWORD;
 dwLocalAddr: DWORD;
 dwLocalPort: DWORD;
 dwRemoteAddr: DWORD;
 dwRemotePort: DWORD;
 dwOwningPid: DWORD;
 end;
type PMibTcpRowOwnerPid = ^TMibTcpRowOwnerPid;

type MIB_TCPTABLE_OWNER_PID = packed record
 dwNumEntries: DWORD;
 table: array [0..ANY_SIZE - 1] of TMibTcpRowOwnerPid;
 end;
type PMIB_TCPTABLE_OWNER_PID  = ^MIB_TCPTABLE_OWNER_PID;


{------------------------------------------------------------------------------}

{Types of UDP port data requested (different types, return different arrays)}

type UDP_TABLE_CLASS = Integer;

const UDP_TABLE_BASIC = 0;
const UDP_TABLE_OWNER_PID = 1;
const UDP_TABLE_OWNER_MODULE = 2;

{Data types returned by various commands}
{Only MIB_UDPTABLE_OWNER_PID has been implied currently.}

{ UDP_TABLE_BASIC		MIB_UDPTABLE }
{ UDP_TABLE_OWNER_PID		MIB_UDPTABLE_OWNER_PID }
{ UDP_TABLE_OWNER_MODULE	MIB_UDPTABLE_OWNER_MODULE  }


{Data returned by the UDP_TABLE_OWNER_PID command}

type TMibUDPRowOwnerPID = packed record
 dwLocalAddr: DWORD;
 dwLocalPort: DWORD;
 dwOwningPid: DWORD;
 end;
type PMibUDPRowOwnerPID = ^TMibUDPRowOwnerPID;


type MIB_UDPTABLE_OWNER_PID = packed record
 dwNumEntries: DWORD;
 table: array [0..ANY_SIZE-1] of TMibUDPRowOwnerPID;
 end;
type PMIB_UDPTABLE_OWNER_PID = ^MIB_UDPTABLE_OWNER_PID;



{------------------------------------------------------------------------------}

{prototypes for 'iphlpapi.dll' API calls}

{$EXTERNALSYM GetExtendedTcpTable}
function GetExtendedTcpTable(pTcpTable: Pointer; dwSize: PDWORD; bOrder: BOOL; lAf: ULONG; TableClass: TCP_TABLE_CLASS; Reserved: ULONG): DWord; stdcall;

{$EXTERNALSYM GetExtendedUdpTable}
function GetExtendedUdpTable(pUdpTable: Pointer; dwSize: PDWORD; bOrder: BOOL; lAf: ULONG; TableClass: UDP_TABLE_CLASS; Reserved: ULONG): DWord; stdcall;

{------------------------------------------------------------------------------}

procedure GetTCPPortList(var List: TPortArray);
procedure GetUDPPortList(var List: TPortArray);
function TCPPortStateToStr(PS: integer): string;
function FindFirstFreeUDPPort(IPAddress: string; Start: integer): integer;


implementation

uses WinSock,TLHelp32;

{External declarations linking procedure names to DLL names}

function GetExtendedTcpTable;            external    iphlpapi name 'GetExtendedTcpTable';
function GetExtendedUdpTable;            external    iphlpapi name 'GetExtendedUdpTable';



function TCPPortStateToStr(PS: integer): string;
{Translate TCP state values to strings}
begin
case PS of
 MIB_TCP_STATE_CLOSED: Result:='Closed';
 MIB_TCP_STATE_LISTEN: Result:='Listen';
 MIB_TCP_STATE_SYN_SENT: Result:='Sync Sent';
 MIB_TCP_STATE_SYN_RCVD: Result:='Sync Received';
 MIB_TCP_STATE_ESTAB: Result:='Established';
 MIB_TCP_STATE_FIN_WAIT1: Result:='Fin Wait-1';
 MIB_TCP_STATE_FIN_WAIT2: Result:='Fin Wait-2';
 MIB_TCP_STATE_CLOSE_WAIT: Result:='Close Wait';
 MIB_TCP_STATE_CLOSING: Result:='Closing';
 MIB_TCP_STATE_LAST_ACK: Result:='Last Ack';
 MIB_TCP_STATE_TIME_WAIT: Result:='Time Wait';
 MIB_TCP_STATE_DELETE_TCB: Result:='DeletE TCB';
 else Result:='*';
 end;
end;



function PIDToExeName(PID: DWord): string;
{Get Exe Filename based on Process ID (PID)}
var SnapShot: THandle;
var ProcEntry: TProcessEntry32;
var Ret: bool;
begin
SetLastError(0);
Result:='Unknown';
SnapShot:= CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
try
if (SnapShot <> INVALID_HANDLE_VALUE) then ProcEntry.dwSize := Sizeof(procentry);
ret := Process32FIRST(SnapShot, ProcEntry);
while Ret do
	begin
	if (ProcEntry.th32ProcessID = PID) then
		begin
		Result := ProcEntry.szExeFile;
		break;
		end
	else ret := Process32Next(SnapShot, ProcEntry);
	end;
finally CloseHandle(SnapShot); end;
end;

{------------------------------------------------------------------------------}

procedure GetTCPPortList(var List: TPortArray);
{Returns a list all active TCP ports}
var TableSize: DWORD;
var I: integer;
var pTcpTable: PMIB_TCPTABLE_OWNER_PID;
var AppName: string;
const TableClass = TCP_TABLE_OWNER_PID_ALL;
begin
SetLength(List,0);
TableSize := 0;
{Get the size o the tcp table}
if GetExtendedTcpTable(nil, @TableSize, False, AF_INET, TableClass, 0) <> ERROR_INSUFFICIENT_BUFFER then exit;
GetMem(pTcpTable, TableSize);
try
{ get the tcp table data }
if GetExtendedTcpTable(pTcpTable, @TableSize, TRUE, AF_INET, TableClass, 0) <> NO_ERROR then exit;
SetLength(List,pTcpTable.dwNumEntries);

{Copy TCP table data to a delphi-style list}
for I := 0 to pTcpTable.dwNumEntries - 1 do
	begin
	AppName:=PIDToExeName(pTcpTable.Table[i].dwOwningPid);
	List[I].AppName:=AppName;
	List[I].ProcessID:=pTcpTable.Table[i].dwOwningPid;
	List[I].LocalAddress:=inet_ntoa(TInAddr(pTcpTable.Table[i].dwLocalAddr));
	List[I].LocalPort:=ntohs(pTcpTable.Table[i].dwLocalPort);
	List[I].RemoteAddress:=inet_ntoa(TInAddr(pTcpTable.Table[i].dwRemoteAddr));
	List[I].RemotePort:=ntohs(pTcpTable.Table[i].dwRemotePort);
	List[I].State:=pTcpTable.Table[i].dwState;
	List[I].UDP:=False;
	end;

finally FreeMem(pTcpTable); end;
end;

{------------------------------------------------------------------------------}


procedure GetUDPPortList(var List: TPortArray);
var TableSize: DWORD;
var I: integer;
var pUDPTable: PMIB_UDPTABLE_OWNER_PID;
var AppName: string;
const TableClass = UDP_TABLE_OWNER_PID;
begin
SetLength(List,0);
TableSize := 0;
{Get the size o the tcp table}
if GetExtendedUdpTable(nil, @TableSize, False, AF_INET, TableClass, 0) <> ERROR_INSUFFICIENT_BUFFER then exit;
GetMem(pUDPTable, TableSize);
try
{ get the tcp table data }
if GetExtendedUdpTable(pUDPTable, @TableSize, TRUE, AF_INET, TableClass, 0) <> NO_ERROR then exit;
SetLength(List,pUDPTable.dwNumEntries);

{Copy UDP list to Delphi-style list}
for I := 0 to pUDPTable.dwNumEntries - 1 do
	begin
	AppName:=PIDToExeName(pUDPTable.Table[I].dwOwningPid);
	List[I].AppName:=AppName;
	List[I].ProcessID:=pUDPTable.Table[I].dwOwningPid;
	List[I].LocalAddress:=inet_ntoa(TInAddr(pUDPTable.Table[I].dwLocalAddr));
	List[I].LocalPort:=ntohs(pUDPTable.Table[I].dwLocalPort);
	List[I].RemoteAddress:='';
	List[I].RemotePort:=0;
	List[I].State:=0;
	List[I].UDP:=True;
	end;

finally FreeMem(pUDPTable); end;
end;


function FindFirstFreeUDPPort(IPAddress: string; Start: integer): integer;
{Find the first UDP port beginning at "Start"}
var TableSize: DWORD;
var I: integer;
var pUDPTable: PMIB_UDPTABLE_OWNER_PID;
var AppName: string;
const TableClass = UDP_TABLE_OWNER_PID;

	function PortInUse(Port: integer): boolean;
	{Test to see if port is in the UDPTable}
	var I,IPAdd: cardinal;
	begin
	Result:=True;
	IPAdd:=inet_addr(PAnsiChar(AnsiString(IPAddress)));
	for I:=0 to pUDPTable.dwNumEntries-1 do
	 if (ntohs(pUDPTable.Table[I].dwLocalPort)=Port) then
		begin
		if (pUDPTable.Table[I].dwLocalAddr=IPAdd) then exit;
		{IP 0.0.0.0 is a wild-card address. It listens on all local addresses}
		if (pUDPTable.Table[I].dwLocalAddr=0) then exit;
		end;
	Result:=False;
	end;

begin
Result:=-1;
TableSize := 0;
{Get the size o the tcp table}
if GetExtendedUdpTable(nil, @TableSize, False, AF_INET, TableClass, 0) <> ERROR_INSUFFICIENT_BUFFER then exit;
GetMem(pUDPTable, TableSize);
try
{ get the tcp table data }
if GetExtendedUdpTable(pUDPTable, @TableSize, TRUE, AF_INET, TableClass, 0) <> NO_ERROR then exit;
{Test ports to see they are in the list}
for Result:=Start to 65535 do
 if not PortInUse(Result) then exit;
Result:=-1;
finally FreeMem(pUDPTable); end;
end;


end.
