unit XEDExpressEval;

interface

uses DSupport,SysUtils,WinTypes,WinProcs,Messages,XEDGlobal,XEDScoopReg,ScrollEditor;



type TXEDExpression = class(TObject)
 private
  FOnStatusUpdate: TStatusEvent;
  function GetScoopString: string;
  procedure DisplayDecimal;
  procedure DisplayHex;
  procedure InsertDecimalString;
  procedure InsertHexString;
  procedure LoadStoreVariable(Negative: boolean; Inx: integer);
  procedure Scoop2Decimal;
  procedure Scoop2Hexadecimal;
    procedure ShowStatus(Status: string);
 protected
 public
  Global : integer;
  VarArray: array [0..9] of integer;
  ExpString: string;
  XRegister: TXRegister;
  Editor: TScrollEditor;
  constructor Create(ScrollEdit: TScrollEditor; XReg: TXRegister);
  destructor Destroy; override;
  function DoConditional(var S: string): boolean;
  function EvaluateExpression(ExpString: string): integer;
  function GlobalCommand( Negative: boolean; var S: string): boolean;
  property OnStatusUpdate: TStatusEvent read FOnStatusUpdate write FOnStatusUpdate;
 end;

implementation


procedure TXEDExpression.ShowStatus(Status: string);
begin
if Assigned(FOnStatusUpdate) then FOnStatusUpdate(Status,Self);
end;



function TXEDExpression.GetScoopString: string;
{Return first 80 chars of scoop as a string}
var P,Stop: longint;
const XMax=80;
begin
Result:='';
if XRegister.Inx>XMax then Stop:=XMax else Stop:=0;
for P:=XRegister.Inx-1 downto Stop do Result:=Result+XRegister.Buffer[P];
end;


procedure TXEDExpression.Scoop2Decimal;
{ Interpret Text In Scoop As A Decimal Number }
begin
Global:=StrToIntDef(GetScoopString,0);
end;



procedure TXEDExpression.Scoop2Hexadecimal;
{ Interpret Text In Scoop As A Hex Number }
begin
Global:=StrToIntDef(GetScoopString,0);
{Same routine works for Hex or Decimal}
end;



procedure TXEDExpression.DisplayHex;
{ Display Global As Hex On Command Line }
begin
ShowStatus(IntToHex(Global,8));
end;



procedure TXEDExpression.DisplayDecimal;
{ Display Global As Hex On Command Line }
begin
ShowStatus(IntToStr(Global));
end;


procedure TXEDExpression.InsertDecimalString;
{ Insert Global Into Buffer As A Decimal String }
begin
Editor.InsertString(IntToStr(Global));
end;


procedure TXEDExpression.InsertHexString;
{ Insert Global Into Buffer As A Hex String }
begin
Editor.InsertString(IntToHex(Global,8));
end;



procedure  TXEDExpression.LoadStoreVariable(Negative: boolean; Inx: integer);
{ Load Or Store Value In Variable }
begin
if Negative then Global:=VarArray[Inx]
else VarArray[Inx]:=Global;
end;


function TXEDExpression.GlobalCommand( Negative: boolean; var S: string): boolean;
{ Commands To Display Or Insert Global }
var C: char;
begin
Result:=False;
Delete(S,1,1);
if Length(S)<1 then
	begin
	ShowStatus('Missing B-Command Arg');
	exit;
	end;
C:=S[1];
Delete(S,1,1);
if IsDigit(C) then LoadStoreVariable(Negative,StrToInt(C))
else case C of
  'X','x': Scoop2Hexadecimal;
  'T','t': Scoop2Decimal;
  'N','n': DisplayDecimal;
  'S','s': DisplayHex;
  'A','a': Editor.InsertString(char(Global));
  'D','d': InsertDecimalString;
  'H','h': InsertHexString;
   end;
Result:=True;
end;


{ ======================= Expression Evaluator =========================== }


function TXEDExpression.EvaluateExpression( ExpString: string): integer;
{ Evaluate Expression starting at EvalPtr }
var Atom: char;
var EvalPtr: integer;


	procedure GetChar;
	begin
	if EvalPtr > Length(ExpString) then
		begin
		ShowStatus('expr?');
		Atom:= ')';
		end
	 else	begin
		Atom:= ExpString[EvalPtr];
		Inc(EvalPtr);
		end;
	end;



	procedure SkipWhiteSpace;
	{ Skip Tabs And Spaces In Linebuf }
	begin
	while (Atom=TAB_CHAR) or (Atom=SP_char) do GetChar;
	end;



	procedure SkipSpaces;
	{ Get Next Character, Ignoring Any Space Characters }
	begin
	repeat GetChar until Atom <> SP_CHAR;
	end;



	function AfterCursor: longint;
	{ Get Character After Cursor }
	begin
	Result:=byte(Editor.GetNextChar);
	if Result=0 then ShowStatus('End?');
	GetChar;
	end;



	function BeforeCursor: longint;
	{ Get Character before Cursor }
	begin
	Result:= 0;
	if Editor.IsCursorAtStart then ShowStatus('start?')
	else Result:=byte(Editor.GetLastChar);
	GetChar;
	end;

	function GetFreespace: longint;
	{ Get editor free space }
	begin
	Result:=1024 * 1024;
	end;


	function GetASCII: longint;
	begin
	GetChar;
	Result:= byte(Atom);
	GetChar;
	end;


	function GetDecimal: longint;
	{ Read In A Decimal String And Return Its Value }
	var S: string;
	begin
	Result:=0;
	S:='';
	while True do
		begin
		if not IsDigit(Atom) then break;
		S:=S+Atom;
		GetChar;
		end;
	if S='' then ShowStatus('number?')
	else Result:=StrToInt(S);
	end;



	function GetHexadecimal: longint;
	{ Return The Value Of A Hex String, E.g: $1ac4 }
	var S: string;
	begin
	Result:=0;
	S:='$';
	GetChar;
	while True do
		begin
		if not IsHexDigit(Atom) then break;
		S:=S+Atom;
		GetChar;
		end;
	if S='$' then ShowStatus('hex?')
	else Result:=StrToInt(S);
	end;


	function GetKeyStrokeX: longint;
	var Msg: TMsg;
	begin
	Result:=0;
	ShowStatus('Waiting');
	while true  do
		begin
		PeekMessage(Msg,0,0,0,PM_REMOVE);
		if Msg.Message = WM_QUIT then break;
		if (Msg.Message = WM_CHAR) then
			begin
			Result:=longint(Msg.wParam);
			break;
			end;
		TranslateMessage(Msg);
		DispatchMessage(Msg);
		end;
	ShowStatus('ok');
	end;


{ ---------------------------------------------------------------------- }

	function Expression: longint;
	{ Returns The Value Of An Expression }



		function Factor: longint;
		{ Returns The Value Of A Factor }
		var NEG: boolean;
		begin
		Result:=0;
		while Atom='+' do SkipSpaces;		{ Ignore Unary "+" }
		NEG:= False;
		while Atom ='-' do			{ Unary "-" }
			begin SkipSpaces; NEG:= not NEG; end;

		if (Atom>='0') and (Atom<='9') then Result:= GetDecimal	{ Unsigned Integer }
		 else case Atom of
		  '(':	begin				{ Subexpression }
			SkipSpaces;
			Result:= Expression;
			if Atom<>')' then ShowStatus('expr?');
			SkipSpaces;
			end;
		  'K','k': begin Result:= GetKeyStrokeX; GetChar; end;
		  'Z','z': begin Result:= GetFreeSpace; GetChar; end;
		  'C','c': begin Result:= Editor.GetCurrentColumn; GetChar; end;
		  '%':	begin Result:= Global; GetChar; end;
		  '$':	Result:= GetHexadecimal;
		  '^':	Result:= GetASCII;
		  '@':	Result:= AfterCursor;
		  '_':  Result:= BeforeCursor;
		  '.':	begin Result:=Editor.CursorToLinearPos(Editor.WorkingCursor.Position); SkipSpaces; end;
		  else	ShowStatus('syntax');
		  end;
		{ Identifiers, Numbers, Etc. May Terminate With A Space Or Tab }
		SkipWhiteSpace;
		if NEG then Result:=-Result;
		end;	{ Factor }



		function Term: Longint;
		{ Returns Factor * Factor, Etc. }
		begin
		Result:= Factor;
		while True do
			case Atom of
			  '*':	begin SkipSpaces; Result:= Result * Factor; end;
			  '/':	begin SkipSpaces; Result:= Result div Factor; end;
			  '\':	begin SkipSpaces; Result:= Result Mod Factor; end;
			  else	break;
			end;
		end;
		{ Term }



		function AlgebraicExpression: Longint;
		{ Returns Term + Term, Etc. }
		begin
		Result:= Term;
		while True do
			case Atom of
			  '+':	begin SkipSpaces; Result:= Result + Term; end;
			  '-':	begin SkipSpaces; Result:= Result - Term; end
			else	break;
			end;
		end; { Algexp }



		function LogicalExpression: Longint;
		{ Returns A Boolean, E.g: Algexp < Algexp }
		begin
		if Atom='~' then				{ Unary 'not' Operator }
			begin
			SkipSpaces;
			Result:= not LogicalExpression;
			end
		 else	begin
			Result:= AlgebraicExpression;
			case Atom of
			 '=':	begin SkipSpaces; Result:= longint(Result = AlgebraicExpression); end;
			 '#':	begin SkipSpaces; Result:= longint(Result <> AlgebraicExpression); end;
			 '>':	begin
				SkipSpaces;
				if Atom<>'=' then Result:= longint(Result > AlgebraicExpression)
				else begin SkipSpaces; Result:= longint(Result >= AlgebraicExpression); end;
				end;
			 '<':	begin
				SkipSpaces;
				if Atom<>'=' then Result:= longint(Result < AlgebraicExpression)
				else begin SkipSpaces; Result:= longint(Result <= AlgebraicExpression); end;
				end;
			 end;
			end;
		end; { Logexp }



		function BooleanTerm: Longint;
		{ Returns Logexp & Logexp }
		begin
		Result:= LogicalExpression;
		while True do	begin
			if Atom='&' then
				begin SkipSpaces; Result:= Result  and  LogicalExpression; end
			 else break;
			end;
		end;	{ Boolterm }



	begin	{ Expression }
	SkipWhiteSpace;		{ Must Not Have A Tab Or Space }
	Result:= BooleanTerm;
	while True do
		begin
		if Atom='!' then begin SkipSpaces; Result:= Result  or  BooleanTerm; end
		else break;
		end;
	end;	{ Expression }



begin	{ EvaluateExpression }
{Move past initial "(" }
EvalPtr:=2;
try
GetChar;
Global:= Expression;
except ShowStatus('div by 0'); end;
if Atom <> ')' then ShowStatus('expr?');
Result:= EvalPtr;
end;	{ EvaluateExpression }

{----------------------------------------------------------------------}


function TXEDExpression.DoConditional(var S: string): boolean;
{ Handle Conditionals }
var BC,I: Integer;
begin
Result:=True;
Delete(S,1,1);
{ If True Just Return And Execute Block }
if Global<>0 then exit;
{ If False delete up to Matching Bracket }
BC:= 1;
for I:=1 to Length(S) do
	begin
	case S[I] of
	 '[':	Inc(BC);
	 ']':
		begin
		Dec(BC);
		if BC<1 then begin Delete(S,1,I); exit; end;
		end;
	 end
	end;

ShowStatus('missing ]');
Result:= False
end;



{ TXEDExpression }

constructor TXEDExpression.Create(ScrollEdit: TScrollEditor; XReg: TXRegister);
begin
Editor:=ScrollEdit;
XRegister:=XReg;
end;

destructor TXEDExpression.Destroy;
begin

inherited;
end;

end.
