unit XEDScoopReg;

interface

uses Windows,Classes,Graphics,Clipbrd,HugeArry,ScrollEditor,HugeMem,DSupport;


type
 TXRegister = class(TObject)
 private
  FOnChange: TNotifyEvent;
  Screen: TScrollEditor;
    FScoopOn: boolean;
  function ScreenToScoop(Mark: TRect): boolean;
  function GetScoopSize: integer;
    procedure SetScoopOn(const Value: boolean);
 protected
  procedure Changed; virtual;
 public
  BlockFlag: boolean;
  Inx: integer;
  Buffer: THugeChar;
  constructor Create(Scrn: TScrollEditor);
  destructor Destroy; override;
  procedure Initialize;
  function InsertChar(C: char): boolean;
  function InsertString(S: string): boolean;
  function GetString: string;

  function StoreChanges(Editor: TScrollEditor; Command: TScoopCommand; Mark: TRect): boolean;
  function InsertScoop: boolean;
  function CopyToClipBoard: boolean;
  function CopyFromClipBoard: boolean;
    procedure DoScoopOff(Editor: TScrollEditor);
    procedure DoScoopOn(Editor: TScrollEditor);
  procedure BlockInsert;
  procedure BlockCopy(Block: TRect);
  procedure BlockFill(Block: TRect);
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property ScoopOn: boolean read FScoopOn write SetScoopOn;
 end;



implementation



procedure TXRegister.Changed;
begin
if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TXRegister.Initialize;
begin
Inx:=0;
ScoopOn:=False;
Changed;
end;


constructor TXRegister.Create(Scrn: TScrollEditor);
begin
inherited Create;
Buffer:=THugeChar.Create(100000);
Screen:=Scrn;
Initialize;
end;



destructor TXRegister.Destroy;
begin
Buffer.Free;
inherited Destroy;
end;



function TXRegister.InsertChar(C: char): boolean;
begin
if Inx>=Buffer.MaxItems then begin Result:=False; exit; end;
Buffer.Value[Inx]:=C;
Inc(Inx);
Changed;
Result:=True;
end;



function TXRegister.InsertString(S: string): boolean;
{Insert string in reverse order in keeping with scoop format}
var I: integer;
begin
Result:=False;
for I:=Length(S) downto 1 do
	begin
	if Inx>=Buffer.MaxItems then exit;
	Buffer[Inx]:=S[I];
	Inc(Inx);
	end;
Changed;
Result:=True;
end;




function TXRegister.GetString: string;
{Get string in reverse order in keeping with scoop format}
var I: integer;
begin
Result:='';
for I:=Inx-1 downto 0 do Result:=Result + Buffer.Value[I];
end;




function TXRegister.ScreenToScoop(Mark: TRect): boolean;
{Copy characters from screen into scoop}
var S: string;
begin
Result:=False;
S:=Screen.GetMarkedText(Mark);
InsertString(S);
Result:=True;
end;



function TXRegister.InsertScoop: boolean;
{Insert scoop into buffer}
var S: string;
begin
Result:=False;
S:=GetString;
Screen.InsertString(S);
Result:=True;
end;




function TXRegister.StoreChanges(Editor: TScrollEditor; Command: TScoopCommand; Mark: TRect): boolean;
var Delta: integer;
begin
Result:=True;
if not ScoopOn or BlockFlag then exit;
case Command of
 scScoop: ScreenToScoop(Mark);
 scUnScoop:
	begin
	Delta:=Screen.GetCountBetweenCursors(Mark);
	Inc(Inx,-Delta);
	if Inx<0 then
		begin
		Inx:=0;
		DoScoopOff(Editor);
		Result:=False;
		end;
	Changed;
	end;
 scAbort: DoScoopOff(Editor);
 end;
end;


{---------------- Clipboard Routines ----------------------------------}


function TXRegister.GetScoopSize: integer;
var I,LF: integer;
begin
LF:=0;
for I:=0 to Inx-1 do
 if Buffer.Value[I]=CR_CHAR then Inc(LF);
Result:=LF+Inx;
end;



function TXRegister.CopyToClipBoard: boolean;
begin
Result:=False;
ClipBoard.Open;
ClipBoard.AsText:=GetString;
ClipBoard.Close;
Result:=True;
end;




function TXRegister.CopyFromClipboard: boolean;
begin
Result:=False;
ClipBoard.Open;
InsertString(ClipBoard.AsText);
ClipBoard.Close;
Result:=True;
end;

{---------------- Block Mode Commands  ----------------------------------------}



procedure TXRegister.BlockInsert;
{ Insert X-register Block Mode }
begin
ScoopOn:= False;
Screen.BlockInsert(GetString);
end;




procedure TXRegister.BlockCopy(Block: TRect);
{ Copy A Block Of Text Along Row/column Lines }
var S: string;
var I: integer;
begin
ScoopOn:= False;
S:=Screen.GetTextBlock(Block);
for I:=Length(S) downto 1 do InsertChar(S[I]);
end;




procedure TXRegister.BlockFill(Block: TRect);
{ Fill A Block Of Text Along Row/column Lines With Spaces }
begin
Screen.DisposeOfBlock(Block,SP_Char,True);
end;


procedure TXRegister.DoScoopOn(Editor: TScrollEditor);
begin
Editor.ResetMarkers;
//Editor.DoChange(False);
Editor.MoveMarking:=True;
Editor.WorkingCursor.ColumnMarking:=False;
ScoopOn:=True;
end;



procedure TXRegister.DoScoopOff(Editor: TScrollEditor);
begin
Editor.MoveMarking:=False;
Editor.WorkingCursor.ColumnMarking:=False;
if ScoopOn then Editor.ResetMarkers;
ScoopOn:=False;
Editor.Invalidate;
end;


procedure TXRegister.SetScoopOn(const Value: boolean);
begin
if FScoopOn<>Value then
	begin
	FScoopOn:=Value;
	Changed;
	end;
end;



end.
