unit XMLDocLibrary;
{
Support routines for TXMLDocument
Note: routines for manual XML reading/writing are in XMLLibrary
}

interface

uses Graphics,Types,Classes,SysUtils,DSupport,Vector,HugeArry,GLQuatLib,XMLIntf,XMLDoc,Variants,DateUtils;

{Open array of XML nodes}

type TNodeArray = array of IXMLNode;


function GetXMLNestingLevel(Node: IXMLNode): Integer;
function GetXMLIndentStr(Indent: integer): string;
function GetChildNodeValue(Node: IXMLNode; Target: string; var V: OleVariant): boolean;
function GetAttributeStr(Node: IXMLNode; Att: string): string;

function XMLFindNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): IXMLNode; overload;
function XMLFindNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType; Wild: char): IXMLNode; overload;
function XMLFindNode(XMLDoc: TXMLDocument; Tag,AttName,Attvalue: string): IXMLNode; overload;
function XMLFindNodeByType(XMLDoc: TXMLDocument; NodeType: TNodeType; Name: string): IXMLNode;
function XMLFindSubNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): IXMLNode;

procedure GetNodeAncestors(var Nodes: TNodeArray; StartNode: IXMLNode);
function XMLGetChildIndex(Parent: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): integer;
function FindNodeByType(XMLNode: IXMLNode; NodeType: TNodeType; Name: string): IXMLNode;


procedure XMLDeleteAttribute(var Node: IXMLNode; Name: string);


function ReadXMLVector2D(Node: IXMLNode): T2DVector; overload;
function ReadXMLVector3D(Node: IXMLNode): T3DVector; overload;
function ReadXMLPoint(Node: IXMLNode): TPoint; overload;
function ReadXMLInteger(Node: IXMLNode): integer; overload;
function ReadXMLDouble(Node: IXMLNode): double; overload;
function ReadXMLString(Node: IXMLNode): string; overload;
function ReadXMLBoolean(Node: IXMLNode): boolean; overload;

function ReadXMLString(Node: IXMLNode; Def: string): string; overload;
function ReadXMLBoolean(Node: IXMLNode; Def: boolean): boolean; overload;
function ReadXMLDouble(Node: IXMLNode; Def: double): double; overload;
function ReadXMLInteger(Node: IXMLNode; Def: integer): integer; overload;
function ReadXMLPoint(Node: IXMLNode; Def: TPoint): TPoint; overload;
function ReadXMLVector2D(Node: IXMLNode; Def: T2DVector): T2DVector; overload;
function ReadXMLVector3D(Node: IXMLNode; Def: T3DVector): T3DVector; overload;

procedure ReadXMLDateTime(XMLNode: IXMLNode; var DT: TDateTime);
procedure ReadXMLFont(XMLNode: IXMLNode; Font: TFont);

procedure SaveXMLQuat(Node: IXMLNode; Q: TQuaternion);
procedure SaveTagXMLQuat(Node: IXMLNode; Tag: string; Q: TQuaternion);
procedure SaveXMLBoolean(Node: IXMLNode; B: Boolean);
procedure SaveTagXMLBoolean(Node: IXMLNode; Tag: string; B: Boolean);
procedure SaveXMLDouble(Node: IXMLNode;  D: double);
procedure SaveTagXMLDouble(Node: IXMLNode; Tag: string; D: double);
procedure SaveXMLInteger(Node: IXMLNode;I: integer);
procedure SaveTagXMLInteger(Node: IXMLNode; Tag: string; I: integer);
procedure SaveXMLString(Node: IXMLNode; S: string);
procedure SaveTagXMLString(Node: IXMLNode; Tag: string; S: string);
procedure SaveTagXMLPoint(Node: IXMLNode; Tag: string;  P: TPoint);
procedure SaveXMLPoint(Node: IXMLNode; P: TPoint);
procedure SaveTagXML2DVector(Node: IXMLNode; Tag: string; V: T2DVector);
procedure SaveXML2DVector(Node: IXMLNode; V: T2DVector);
procedure SaveTagXML3DVector(Node: IXMLNode; Tag: string; V: T3DVector);
procedure SaveXML3DVector(Node: IXMLNode; V: T3DVector);
procedure SaveXMLDateTime(Date: TDateTime; XMLNode: IXMLNode);
procedure SaveXMLFont(Node: IXMLNode; F: TFont);
procedure SaveXMLTagFont(Node: IXMLNode; Tag: string; F: TFont);

function RemoveExtendedChars(Source: TStream): Int64;


implementation


function GetAttributeStr(Node: IXMLNode; Att: string): string;
{Get attribute string for node assuring that the variant is not NULL}
var V: Variant;
begin
V:=Node.Attributes[Att];
if V<>NULL then Result:=V else Result:='';
end;



function GetXMLNestingLevel(Node: IXMLNode): Integer;
var PNode: IXMLNode;
begin
Result:= 0;
PNode:= Node.ParentNode;
while PNode <> nil do
	begin
	Inc(Result);
	PNode := PNode.ParentNode;
	end;
end;


function GetXMLIndentStr(Indent: integer): string;
var I: integer;
begin
Result:='';
for I:=1 to Indent do Result:=Result+' ';
end;



procedure XMLDeleteAttribute(var Node: IXMLNode; Name: string);
var AN: IXMLNode;
begin
AN:=Node.AttributeNodes.FindNode(Name);
if AN=nil then exit;
Node.AttributeNodes.Remove(AN);
end;




{---------------- Read Common Types -------------------------------------------}



function GetChildNodeValue(Node: IXMLNode; Target: string; var V: OleVariant): boolean;
var ChildNode: IXMLNode;
begin
ChildNode:= Node.ChildNodes.First;
if ChildNode.LocalName<>Target then raise Exception.Create('Expected '+Target+'. Found: '+ChildNode.LocalName);
Result:=TVarData(ChildNode.NodeValue).VPointer<>nil;
if Result then V:=ChildNode.NodeValue;
end;


{-------------- version with default options ---------------------------------}

function ReadXMLString(Node: IXMLNode; Def: string): string;
var V: OleVariant;
begin
if GetChildNodeValue(Node,'String',V) then Result:=V
else Result:=Def;
end;



function ReadXMLBoolean(Node: IXMLNode; Def: boolean): boolean;
var V: OleVariant;
begin
Result:=Def;
if GetChildNodeValue(Node,'Boolean',V) then Result:=V;
end;



function ReadXMLDouble(Node: IXMLNode; Def: double): double;
var V: OleVariant;
begin
Result:=Def;
if GetChildNodeValue(Node,'Float',V) then
	begin
	{Don't use OLE conversion - it won't internationalize}
	Result:=StrToFloat(V);
	end;
end;



function ReadXMLInteger(Node: IXMLNode; Def: integer): integer;
var V: OleVariant;
begin
Result:=Def;
if GetChildNodeValue(Node,'Integer',V) then Result:=V;
end;


function ReadXMLVector2D(Node: IXMLNode; Def: T2DVector): T2DVector;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
Result:=Def;
if GetChildNodeValue(Node,'Vector2D',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Y:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;



function ReadXMLPoint(Node: IXMLNode; Def: TPoint): TPoint;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
Result:=Def;
if GetChildNodeValue(Node,'Point',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToIntDef(ExtractToken(S,',',Inx),0);
	Result.Y:=StrToIntDef(ExtractToken(S,',',Inx),0);
	end;
end;


function ReadXMLVector3D(Node: IXMLNode; Def: T3DVector): T3DVector;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
Result:=Def;
if GetChildNodeValue(Node,'Vector3D',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Y:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Z:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;



function ReadXMLString(Node: IXMLNode): string;
var V: OleVariant;
begin
if GetChildNodeValue(Node,'String',V) then Result:=V
else Result:='';
end;



function ReadXMLBoolean(Node: IXMLNode): boolean;
var V: OleVariant;
begin
if GetChildNodeValue(Node,'Boolean',V) then Result:=V;
end;



function ReadXMLDouble(Node: IXMLNode): double;
var V: OleVariant;
begin
if GetChildNodeValue(Node,'Float',V) then Result:=V;
end;



function ReadXMLInteger(Node: IXMLNode): integer;
var V: OleVariant;
begin
if GetChildNodeValue(Node,'Integer',V) then Result:=V;
end;


function ReadXMLVector2D(Node: IXMLNode): T2DVector;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
if GetChildNodeValue(Node,'Vector2D',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Y:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;


function ReadXMLPoint(Node: IXMLNode): TPoint;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
if GetChildNodeValue(Node,'Point',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToIntDef(ExtractToken(S,',',Inx),0);
	Result.Y:=StrToIntDef(ExtractToken(S,',',Inx),0);
	end;
end;



function ReadXMLVector3D(Node: IXMLNode): T3DVector;
var S: string;
var V: OleVariant;
var Inx: integer;
begin
if GetChildNodeValue(Node,'Vector3D',V) then
	begin
	S:=V;
	Inx:=1;
	Result.X:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Y:=StrToFloat(ExtractToken(S,',',Inx));
	Result.Z:=StrToFloat(ExtractToken(S,',',Inx));
	end;
end;


procedure ReadXMLDateTime(XMLNode: IXMLNode; var DT: TDateTime);
var ChildNode: IXMLNode;
var Year, Month, Day, Hour, Min, Sec, MS: Word;
begin
ChildNode:= XMLNode.ChildNodes.First;
while ChildNode <> nil do
	begin
	if ChildNode.LocalName='Month' then Month:=ChildNode.NodeValue
	else if ChildNode.LocalName='Day' then Day:=ChildNode.NodeValue
	else if ChildNode.LocalName='Year' then Year:=ChildNode.NodeValue
	else if ChildNode.LocalName='Hours' then Hour:=ChildNode.NodeValue
	else if ChildNode.LocalName='Minutes' then Min:=ChildNode.NodeValue
	else if ChildNode.LocalName='Seconds' then Sec:=ChildNode.NodeValue
	else if ChildNode.LocalName='MilliSecs' then MS:=ChildNode.NodeValue;
	ChildNode:= ChildNode.NextSibling;
	end;
DT:=EncodeDateTime(Year, Month, Day, Hour, Min, Sec, MS);
end;

{------------------------------------------------------------------------------}

function GetStyles(Style: string): TFontStyles;
begin
Result:=[];
if Pos('fsBold',Style)>0 then Result:=Result + [fsBold];
if Pos('fsItalic',Style)>0 then Result:=Result + [fsItalic];
if Pos('fsUnderline',Style)>0 then Result:=Result + [fsUnderline];
if Pos('fsStrikeOut',Style)>0 then Result:=Result + [fsStrikeOut];
end;


procedure ReadXMLFont(XMLNode: IXMLNode; Font: TFont);
var ChildNode: IXMLNode;
begin
ChildNode:=XMLNode.ChildNodes.First;
while ChildNode<>nil do
	begin
	if ChildNode.LocalName='Charset' then Font.Charset:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Color' then Font.Color:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Name' then Font.Name:=ReadXMLString(ChildNode)
	else if ChildNode.LocalName='Size' then Font.Size:=ReadXMLInteger(ChildNode)
	else if ChildNode.LocalName='Style' then Font.Style:=GetStyles(ReadXMLString(ChildNode));
	ChildNode:=ChildNode.NextSibling;
	end;
end;


{-----------------Save Common XML types ---------------------------------------}


procedure SaveXMLQuat(Node: IXMLNode; Q: TQuaternion);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Quaternion');
ChildXMLNode.NodeValue:=GetQuatString(Q);
end;


procedure SaveTagXMLQuat(Node: IXMLNode; Tag: string; Q: TQuaternion);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXMLQuat(ChildXMLNode,Q);
end;


{-----------------------------------------------------------------------------}

procedure SaveXMLBoolean(Node: IXMLNode; B: Boolean);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Boolean');
ChildXMLNode.NodeValue:=B;
end;



procedure SaveTagXMLBoolean(Node: IXMLNode; Tag: string; B: Boolean);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXMLBoolean(ChildXMLNode,B);
end;

{-----------------------------------------------------------------------------}


procedure SaveXMLDouble(Node: IXMLNode;  D: double);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Float');
ChildXMLNode.NodeValue:=D;
end;



procedure SaveTagXMLDouble(Node: IXMLNode; Tag: string; D: double);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXMLDouble(ChildXMLNode,D);
end;

{-----------------------------------------------------------------------------}


procedure SaveXMLInteger(Node: IXMLNode;I: integer);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Integer');
ChildXMLNode.NodeValue:=I;
end;



procedure SaveTagXMLInteger(Node: IXMLNode; Tag: string; I: integer);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXMLInteger(ChildXMLNode,I);
end;

{-----------------------------------------------------------------------------}


procedure SaveXMLString(Node: IXMLNode; S: string);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('String');
ChildXMLNode.NodeValue:=S;
end;


procedure SaveTagXMLString(Node: IXMLNode; Tag: string; S: string);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:=Node.AddChild(Tag);
SaveXMLString(ChildXMLNode,S);
end;

{-----------------------------------------------------------------------------}

procedure SaveXML3DVector(Node: IXMLNode; V: T3DVector);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Vector3D');
ChildXMLNode.NodeValue:=Vector3DToString(V);
end;


procedure SaveTagXML3DVector(Node: IXMLNode; Tag: string; V: T3DVector);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXML3DVector(ChildXMLNode,V);
end;

{-----------------------------------------------------------------------------}

procedure SaveXML2DVector(Node: IXMLNode; V: T2DVector);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Vector2D');
ChildXMLNode.NodeValue:=Vector2DToString(V);
end;


procedure SaveTagXML2DVector(Node: IXMLNode; Tag: string; V: T2DVector);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXML2DVector(ChildXMLNode,V);
end;


{-----------------------------------------------------------------------------}

function PointToString(P: TPoint): string;
begin
Result:=IntToStr(P.X)+','+IntToStr(P.Y);
end;


procedure SaveXMLPoint(Node: IXMLNode; P: TPoint);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild('Point');
ChildXMLNode.NodeValue:=PointToString(P);
end;


procedure SaveTagXMLPoint(Node: IXMLNode; Tag: string;  P: TPoint);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:= Node.AddChild(Tag);
SaveXMLPoint(ChildXMLNode,P);
end;
{-----------------------------------------------------------------------------}


procedure SaveXMLDateTime(Date: TDateTime; XMLNode: IXMLNode);
var Child,Child2: IXMLNode;
var Year, Month, Day, Hour, Min, Sec, MS: Word;
begin
DecodeDateTime(Date, Year, Month, Day, Hour, Min, Sec, MS);
Child:= XMLNode.AddChild('Date');

Child2:=Child.AddChild('Month');
Child2.NodeValue:=Month;
Child2:=Child.AddChild('Day');
Child2.NodeValue:=Day;
Child2:=Child.AddChild('Year');
Child2.NodeValue:=Year;

Child2:=Child.AddChild('Hours');
Child2.NodeValue:=Hour;
Child2:=Child.AddChild('Minutes');
Child2.NodeValue:=Min;
Child2:=Child.AddChild('Seconds');
Child2.NodeValue:=Sec;
Child2:=Child.AddChild('MilliSecs');
Child2.NodeValue:=MS;
end;

{-----------------------------------------------------------------------------}


procedure AppendStringComma(var Destination: string; Source: string);
begin
if Destination<>'' then Destination:=Destination+',';
Destination:=Destination+Source;
end;


function GetStyleString(Style: TFontStyles): string;
begin
Result:='';
if fsBold in Style then AppendStringComma(Result,'fsBold');
if fsItalic in Style then AppendStringComma(Result,'fsItalic');
if fsUnderline in Style then AppendStringComma(Result,'fsUnderline');
if fsStrikeOut in Style then AppendStringComma(Result,'fsStrikeOut');
end;



procedure SaveXMLFont(Node: IXMLNode; F: TFont);
begin
SaveTagXMLInteger(Node,'Charset',F.Charset);
SaveTagXMLInteger(Node,'Color',integer(F.Color));
SaveTagXMLString(Node,'Name',F.Name);
SaveTagXMLInteger(Node,'Size',F.Size);
SaveTagXMLString(Node,'Style',GetStyleString(F.Style));
end;


procedure SaveXMLTagFont(Node: IXMLNode; Tag: string; F: TFont);
var ChildXMLNode: IXMLNode;
begin
ChildXMLNode:=Node.AddChild(Tag);
SaveXMLFont(ChildXMLNode,F);
end;



{-----------------------------------------------------------------------------}




function XMLFindNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): IXMLNode;
{Find subnodes and siblings at or below the level of "XMLNode"}
{Find nodes that match "Tag," "AttName," "AttValue," and "NodeType" }
{If no matching parameters found, recurse SubNodes and return first node that matches tag}
var ChildNode: IXMLNode;
var T,AV: string;
begin
while XMLNode <> nil do
	begin
	T:=XMLNode.LocalName;
	if AttName<>'' then AV:=GetAttributeStr(XMLNode,AttName);
	if (T=Tag) and
	   ((AttName='') or (AV=AttValue)) and
	   ((NodeType=ntReserved) or (XMLNode.NodeType=NodeType)) then
		begin
		Result:=XMLNode;
		Exit;
		end
	else
		begin
		ChildNode:=XMLNode.ChildNodes.First;
		if ChildNode<>nil then
			begin
			Result:=XMLFindNode(ChildNode,Tag,AttName,AttValue,NodeType);
			if Result<>nil then exit;
			end;
		end;
	XMLNode:= XMLNode.NextSibling;
	end;
Result:=nil;
end;



function XMLFindNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType; Wild: char): IXMLNode;
{Find subnodes and siblings at or below the level of "XMLNode"}
{Find nodes that match "Tag," "AttName," "AttValue," and "NodeType" }
{If no matching parameters found, recurse SubNodes and return first node that matches tag}
{Use "Wild" to get fuzzy matches to "AttName" }
var ChildNode: IXMLNode;
var AV: string;
begin
while XMLNode <> nil do
	begin
	if AttName<>'' then AV:=GetAttributeStr(XMLNode,AttName);
	if (Tag=XMLNode.LocalName) and
	   ((AttName='') or WildCompare(AttValue,AV,Wild)) and
	   ((NodeType=ntReserved) or (XMLNode.NodeType=NodeType)) then
		begin
		Result:=XMLNode;
		Exit;
		end
	else
		begin
		ChildNode:=XMLNode.ChildNodes.First;
		if ChildNode<>nil then
			begin
			Result:=XMLFindNode(ChildNode,Tag,AttName,AttValue,NodeType,Wild);
			if Result<>nil then exit;
			end;
		end;
	XMLNode:= XMLNode.NextSibling;
	end;
Result:=nil;
end;



function XMLFindNode(XMLDoc: TXMLDocument; Tag,AttName,Attvalue: string): IXMLNode;
{Find specifed XML Tag with matching attribute and attribute value}
{If AttName is empty returns first matching tag}
var XMLNode: IXMLNode;
begin
XMLNode:=XMLDoc.Node;
Result:=XMLFindNode(XMLNode,Tag,AttName,AttValue,ntReserved);
end;



function XMLFindSubNode(XMLNode: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): IXMLNode;
{Find subnodes but no siblings at or below the level of "XMLNode"}
{Find nodes that match "Tag," "AttName," "AttValue," and "NodeType" }
{If no matching parameters found, recurse SubNodes and return first node that matches tag}
var ChildNode: IXMLNode;
var T,AV: string;
begin
T:=XMLNode.LocalName;
if AttName<>'' then AV:=GetAttributeStr(XMLNode,AttName);
if (T=Tag) and
   ((AttName='') or (AV=AttValue)) and
   ((NodeType=ntReserved) or (XMLNode.NodeType=NodeType)) then
	begin
	Result:=XMLNode;
	Exit;
	end
else
	begin
	ChildNode:=XMLNode.ChildNodes.First;
	if ChildNode<>nil then
		begin
		Result:=XMLFindNode(ChildNode,Tag,AttName,AttValue,NodeType);
		if Result<>nil then exit;
		end;
	end;
Result:=nil;
end;



{-----------------------------------------------------------------------------}


function FindNodeByType(XMLNode: IXMLNode; NodeType: TNodeType; Name: string): IXMLNode;
{Recurse looking for XML first node with matching node type and Name}
{If name is empty returns first matching NodeType}
var ChildNode: IXMLNode;
var T: string;
begin
while XMLNode <> nil do
	begin
	T:=XMLNode.LocalName;
	if (XMLNode.NodeType=NodeType) and ((Name='') or (Name=T)) then
		begin
		Result:=XMLNode;
		Exit;
		end
	else
		begin
		ChildNode:=XMLNode.ChildNodes.First;
		if ChildNode<>nil then
			begin
			Result:=FindNodeByType(ChildNode,NodeType,Name);
			if Result<>nil then exit;
			end;
		end;
	XMLNode:= XMLNode.NextSibling;
	end;
Result:=nil;
end;



function XMLFindNodeByType(XMLDoc: TXMLDocument; NodeType: TNodeType; Name: string): IXMLNode;
{Recurse looking for XML first node with matching node type and Name}
{If name is empty returns first matching NodeType}
var XMLNode: IXMLNode;
begin
XMLNode:=XMLDoc.Node;
Result:=FindNodeByType(XMLNode,NodeType,Name);
end;




function XMLGetChildIndex(Parent: IXMLNode; Tag,AttName,AttValue: string; NodeType: TNodeType): integer;
{Find the index a the XML node with the specified properties}
var N: IXMLNode;
begin
N:=XMLFindNode(Parent,Tag,AttName,AttValue,ntElement);
Result:=Parent.ChildNodes.IndexOf(N);
end;




procedure GetNodeAncestors(var Nodes: TNodeArray; StartNode: IXMLNode);
{Get a list of all ancestors to the specified node, includes Start Node}
{Nodes are ordered from the lowest to the highest}
var OldNode,NewNode: IXMLNode;
begin
SetLength(Nodes,0);
OldNode:=nil;
NewNode:=StartNode;
while (NewNode<>nil) and (NewNode<>OldNode) do
	begin
	SetLength(Nodes,Length(Nodes)+1);
	Nodes[High(Nodes)]:=NewNode;
	OldNode:=NewNode;
	NewNode:=NewNode.ParentNode;
	end;

end;


{-----------------------------------------------------------------------------}



function RemoveExtendedChars(Source: TStream): Int64;
const MaxBufSize = $F000;
var Count,P: Int64;
var BufSize, N, I: Integer;
var Buffer: PChar;
begin
Count:=Source.Size;
if Count > MaxBufSize then BufSize := MaxBufSize else BufSize := Count;
GetMem(Buffer, BufSize);
try
while Count <> 0 do
	begin
	if Count > BufSize then N := BufSize else N := Count;
	P:=Source.Position;
	Source.ReadBuffer(Buffer^, N);
	for I:=0 to N-1 do
		begin
		Buffer[I]:=char(byte(Buffer[I]) and $7F);
		end;
	Source.Position:=P;
	Source.WriteBuffer(Buffer^, N);
	Dec(Count, N);
	end;
finally FreeMem(Buffer, BufSize); end;
end;



{-----------------------------------------------------------------------------}

end.
