unit Autobttn;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, ExtCtrls,ComCtrls;


{ TAutoButton}

type
  TAutoButton = class(TSpeedButton)
  private
    FInitialDelay: integer;
    FRepeatDelay: integer;
    FRepeatTimer: TTimer;
    FDoRepeat: boolean;
    FOnMouseUp: TNotifyEvent;
    FButtonPressed: boolean;
    FControlRepeat: boolean;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Pressed: boolean read FButtonPressed;
    procedure Click; override;
  published
    property Align;
    property ControlRepeat: boolean read FControlRepeat write FControlRepeat default False;
    property DoRepeat: boolean read FDoRepeat write FDoRepeat default false;
    property InitialDelay: integer read FInitialDelay write FInitialDelay default 400;
    property RepeatDelay: integer read FRepeatDelay write FRepeatDelay default 100;
    property OnMouseUp: TNotifyEvent read FOnMouseUp write FOnMouseUp;
  end;

{ TAutoToolButton}

type
  TAutoToolButton = class(TToolButton)
  private
    FInitialDelay: integer;
    FRepeatDelay: integer;
    FRepeatTimer: TTimer;
    FDoRepeat: boolean;
    procedure TimerExpired(Sender: TObject);
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property DoRepeat: boolean read FDoRepeat write FDoRepeat default false;
    property InitialDelay: integer read FInitialDelay write FInitialDelay default 400;
    property RepeatDelay: integer read FRepeatDelay write FRepeatDelay default 100;
  end;


procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TAutoButton]);
RegisterComponents('Fountain', [TAutoToolButton]);
end;

{TAutoButton}

procedure TAutoButton.Click;
begin
inherited;
end;


constructor TAutoButton.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FInitialDelay:=400;
FRepeatDelay:=100;
FDoRepeat:=false;
FControlRepeat:=False;
end;


destructor TAutoButton.Destroy;
begin
if FRepeatTimer <> nil then FRepeatTimer.Free;
inherited Destroy;
end;


procedure TAutoButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
FButtonPressed:=True;
inherited MouseDown (Button, Shift, X, Y);
if FDoRepeat or
   (((GetAsyncKeyState(VK_CONTROL) and $8000)<>0) and FControlRepeat)  then
	begin
	if FRepeatTimer = nil then FRepeatTimer := TTimer.Create(Self);
	FRepeatTimer.OnTimer := TimerExpired;
	FRepeatTimer.Interval := FInitialDelay;
	FRepeatTimer.Enabled  := True;
	end;
end;

procedure TAutoButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);
begin
FButtonPressed:=False;
inherited MouseUp(Button, Shift, X, Y);
if FRepeatTimer <> nil then FRepeatTimer.Enabled:= False;
if Assigned(FOnMouseUp) then FOnMouseUp(Self);
end;

procedure TAutoButton.TimerExpired(Sender: TObject);
begin
FRepeatTimer.Interval := FRepeatDelay;
if (FState = bsDown) and MouseCapture then
	begin
	try
	Click;
	except
	  FRepeatTimer.Enabled := False;
	  raise;
	  end;
	end;
end;


{TAutoToolButton}

constructor TAutoToolButton.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FInitialDelay:=400;
FRepeatDelay:=100;
FDoRepeat:=false;
end;


destructor TAutoToolButton.Destroy;
begin
if FRepeatTimer <> nil then FRepeatTimer.Free;
inherited Destroy;
end;

procedure TAutoToolButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
inherited MouseDown (Button, Shift, X, Y);
if FDoRepeat then
	begin
	if FRepeatTimer = nil then FRepeatTimer := TTimer.Create(Self);
	FRepeatTimer.OnTimer := TimerExpired;
	FRepeatTimer.Interval := FInitialDelay;
	FRepeatTimer.Enabled  := True;
	end;
end;

procedure TAutoToolButton.MouseUp(Button: TMouseButton; Shift: TShiftState;
                                  X, Y: Integer);
begin
inherited MouseUp (Button, Shift, X, Y);
if FRepeatTimer <> nil then FRepeatTimer.Enabled  := False;
end;

procedure TAutoToolButton.TimerExpired(Sender: TObject);
begin
FRepeatTimer.Interval := FRepeatDelay;
if MouseCapture then
	begin
	try
	Click;
	except
	  FRepeatTimer.Enabled := False;
	  raise;
	  end;
	end;
end;


end.
