unit ClipListen;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;


type TClipboardListener = class(TComponent)
 private
  NextViewer: HWND;
  FWindowHandle: HWND;
  FOnClipboardChange: TNotifyEvent;
  procedure WndProc(var Msg: TMessage);
  procedure WMChangeCBChain(var Msg: TMessage);
  procedure WMDrawClipBoard(var Msg: TMessage);
 protected
 public
 published
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property OnClipboardChange: TNotifyEvent read FOnClipboardChange write FOnClipboardChange;
 end;


procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TClipboardListener]);
end;

{ TClipboardListener }


constructor TClipboardListener.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FWindowHandle := Classes.AllocateHWnd(WndProc);
NextViewer:=SetClipboardViewer(FWindowHandle);
end;



destructor TClipboardListener.Destroy;
begin
ChangeClipboardChain(FWindowHandle,NextViewer);
PostQuitMessage(0);
Classes.DeallocateHWnd(FWindowHandle);
inherited Destroy;
end;



procedure TClipboardListener.WMChangeCBChain(var Msg: TMessage);
begin
{ If the next window is closing, repair the chain.}
{ Otherwise, pass the message to the next link.}
if Msg.wParam = NextViewer then NextViewer:=Msg.lParam
else if NextViewer<>0 then SendMessage(NextViewer, Msg.Msg, Msg.wParam, Msg.lParam);
end;



procedure TClipboardListener.WMDrawClipBoard(var Msg: TMessage);
begin
if Assigned(FOnClipboardChange) then FOnClipboardChange(Self);
end;


procedure TClipboardListener.WndProc(var Msg: TMessage);
begin
case Msg.Msg of
 WM_CHANGECBCHAIN: WMChangeCBChain(Msg);
 WM_DRAWCLIPBOARD: WMDrawClipBoard(Msg);
 else Msg.Result := DefWindowProc(FWindowHandle, Msg.Msg, Msg.wParam, Msg.lParam);
 end;
end;



end.
