unit CodeEditor;

interface

uses Windows, Graphics, Messages, Forms,SysUtils, Classes, Controls, ComCtrls, ExtCtrls,
     ScrollEditor, DSupport, Clipbrd, Dialogs,CmdLine,XEDMacroLib,Menus,IniFiles;

type TCodeEditor = class(TCustomPanel)
 private
  FCurrentFile: string;
  FOnFileChange: TNotifyEvent;
  MacroDecoder: TMacroDecoder;
  FIniFile: string;
  FOnEditKeyDown: TKeyEvent;
  FOnEditKeyPress: TKeyPressEvent;
    FOnSaveAs: TNotifyEvent;
  procedure SetupToolBar;
  procedure SetupStatusBar;
  function GetHexUnderCursor: string;
  procedure CopyHandler(Sender: TObject);
  procedure CutHandler(Sender: TObject);
  procedure PasteHandler(Sender: TObject);
  procedure SetupEditor;
  procedure SetCurrentFilename(Filename: string);
  procedure SetupButton(var Btn: TToolButton; Index: integer; Hint: string);
  procedure AddStatusPanel(var Pan: TPanel; Width: integer; AL: TAlignment; Hint: string);
  procedure KeyPress(Sender: TObject; var Key: Char);
  procedure HandleCmdKey(Sender: TObject;var Key: char);
  procedure HandleStatusEvent(Status: string; Sender: TObject);
  procedure HandleCmdBtnClick(Sender: TObject);
  procedure SetupCmdLine;
  procedure CmdCopyHandler(Sender: TObject);
  procedure CmdCutHandler(Sender: TObject);
  procedure CmdPasteHandler(Sender: TObject);
  procedure CmdDeleteHandler(Sender: TObject);
  procedure CmdSelectAllHandler(Sender: TObject);
  procedure DeleteHandler(Sender: TObject);
  procedure SelectAllHandler(Sender: TObject);
  procedure HandleDecCmdLine(Sender: TObject);
  procedure HandleIncCmdLine(Sender: TObject);
  procedure EnterCmdLine;
    procedure EditorScoopEvent(Command: TScoopCommand; Mark: TRect);
    procedure KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    function HandleCtrlCommands(Key: Word; Shift: TShiftState): boolean;
    function HandleNormalCommands(Key: Word; Shift: TShiftState): boolean;
    procedure DoWordBackSearch;
    procedure DoWordFwdSearch;
    function GetTarget: string;
    procedure WordBackward(WordStart: boolean);
    procedure WordForward(WordStart: boolean);
    function SaveChanges: boolean;
    procedure EditToString(var S: string);
    procedure RestoreCurrentMacros;
    procedure SaveCurrentMacros;
    procedure StringToEdit(S: string);
    procedure SetCmdLineInx(Inx: integer);
    procedure FindHandler(Sender: TObject);
    function BackupAndSave(Filename: string): boolean;
    procedure WordSearchBackHandler(Sender: TObject);
    procedure WordSearchForwardHandler(Sender: TObject);
    procedure ToggleSearchCase(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ToggleSearchDir(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure HandleMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure HandleMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
    procedure HandleExpressionEvent(Status: string; Sender: TObject);
 protected
  procedure Invalidate; override;
 public
  OpenDialog: TOpenDialog;
  SaveDialog: TSaveDialog;
  FindDialog: TFindDialog;
  ImageList: TImageList;
  Toolbar: TToolBar;
  OpenBtn,SaveBtn,SaveAsBtn,CutBtn,CopyBtn,PasteBtn,FindLeftBtn,FindRightBtn: TToolButton;
  StatusDis,CmdNumberDis,ScoopStatusDis,CmdSizeDis,SearchCaseDis,SearchDirDis,OverwriteDis,LineNumDis,ColumnDis,ModifiedDis,CharHexDis: TPanel;
  Label1,Label2,ExpressDis: TPanel;
  CmdLine: TCmdLine;
  StatusBar: TPanel;
  Editor: TScrollEditor;
  CmdPanel: TPanel;
  EditorMenu,CmdMenu: TPopupMenu;
  FileChange: boolean;
  constructor Create(AOwner: TComponent); override;

  destructor Destroy; override;
  procedure AddMenuItem(Menu: TMenu; Caption: string; OnClick: TNotifyEvent; ShortCut: TShortCut=0);
   procedure InsertMenuItem(Index: integer; Menu: TMenu; Caption: string; OnClick: TNotifyEvent; ShortCut: TShortCut=0);
  property CurrentFile: string read FCurrentFile;
  procedure ShowStatus(Sender: TObject);
  procedure LoadSourceFile(Filename: string);
  function OpenFile(Filename: string): boolean;
  procedure OpenHandler(Sender: TObject);
  procedure SaveAsHandler(Sender: TObject);
  procedure SaveHandler(Sender: TObject);
  procedure NewHandler;
  procedure FindExecuteHandler(Sender: TObject);
  procedure Initialize;
  procedure Finalize;
 published
  property IniFile: string read FIniFile write FIniFile;
  property OnFileChange: TNotifyEvent read FOnFileChange write FOnFileChange;
  property OnEditKeyPress: TKeyPressEvent read FOnEditKeyPress write FOnEditKeyPress;
  property OnEditKeyDown: TKeyEvent read FOnEditKeyDown write FOnEditKeyDown;
  property OnSaveAs: TNotifyEvent read FOnSaveAs write FOnSaveAs;
    property Align;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
 end;




procedure Register;


implementation



{$R CODEEDITOR.RES}


procedure Register;
begin
RegisterComponents('Fountain', [TCodeEditor]);
end;



{ TCodeEditor }


procedure AddToolbarButton(var Bar: TToolBar; var Button: TToolButton);
var LastInx: integer;
begin
Button:= TToolButton.Create(Bar);
Button.Caption:='';
LastInx := Bar.ButtonCount - 1;
if LastInx > -1 then Button.Left:=Bar.Buttons[LastInx].Left + Bar.Buttons[LastInx].Width
else Button.Left:=1;
Button.Parent:=Bar;
end;



procedure AddToolBarSeparator(var Bar: TToolBar);
var Newbtn: TToolButton;
var LastBtnIdx: integer;
begin
NewBtn := TToolButton.Create(bar);
NewBtn.Style:= tbsSeparator;
LastBtnIdx := bar.ButtonCount - 1;
if LastBtnIdx > -1 then Newbtn.Left:= Bar.Buttons[LastBtnIdx].Left + Bar.Buttons[LastBtnIdx].Width
else newbtn.Left := 0;
NewBtn.Parent:=Bar;
end;



procedure AddListBMP(IL: TImageList; Name: String);
var BMP: TBitmap;
begin
BMP:=TBitmap.Create;
try
BMP.Handle:=LoadBitmap(HInstance, PChar(Name));
IL.Add(BMP,nil);
finally BMP.Free; end;
end;







procedure TCodeEditor.InsertMenuItem(Index: integer; Menu: TMenu; Caption: string; OnClick: TNotifyEvent; ShortCut: TShortCut=0);
var NewItem: TMenuItem;
begin
NewItem:=TMenuItem.Create(Self);
NewItem.Caption:=Caption;
NewItem.OnClick:=OnClick;
NewItem.ShortCut:=ShortCut;
Menu.Items.Insert(Index, NewItem);
end;



procedure TCodeEditor.AddMenuItem(Menu: TMenu; Caption: string; OnClick: TNotifyEvent; ShortCut: TShortCut=0);
var NewItem: TMenuItem;
begin
NewItem:=TMenuItem.Create(Self);
NewItem.Caption:=Caption;
NewItem.OnClick:=OnClick;
NewItem.ShortCut:=ShortCut;
Menu.Items.Add(NewItem);
end;




procedure TCodeEditor.SetupButton(var Btn: TToolButton; Index: integer; Hint: string);
begin
AddToolbarButton(ToolBar,Btn);
Btn.ImageIndex:=Index;
Btn.Hint:=Hint;
Btn.ShowHint:=True;
end;




procedure TCodeEditor.SetupToolBar;
begin
ToolBar:=TToolBar.Create(Self);
ToolBar.Parent:=Self;
ToolBar.Visible:=True;
ToolBar.ButtonWidth:=24;
ToolBar.ButtonHeight:=24;
ToolBar.Name:='ToolBar';

SetupButton(OpenBtn,0,'Open');
SetupButton(SaveBtn,1,'Save');
SetupButton(SaveAsBtn,2,'Save As');
AddToolBarSeparator(ToolBar);
SetupButton(CutBtn,3,'Cut');
SetupButton(CopyBtn,4,'Copy');
SetupButton(PasteBtn,5,'Paste');
AddToolBarSeparator(ToolBar);
SetupButton(FindLeftBtn,6,'Search Left');
SetupButton(FindRightBtn,7,'Search Right');

ImageList.Height:=16;
ImageList.Width:=16;
AddListBMP(ImageList,'OPENBMP');
AddListBMP(ImageList,'SAVEBMP');
AddListBMP(ImageList,'SAVEASBMP');
AddListBMP(ImageList,'CUTBMP');
AddListBMP(ImageList,'COPYBMP');
AddListBMP(ImageList,'PASTEBMP');
AddListBMP(ImageList,'FINDLEFTBMP');
AddListBMP(ImageList,'FINDRIGHTBMP');

ToolBar.Images:=ImageList;
OpenBtn.OnClick:=OpenHandler;
SaveBtn.OnClick:=SaveHandler;
SaveAsBtn.OnClick:=SaveAsHandler;

CutBtn.OnClick:=CutHandler;
CopyBtn.OnClick:=CopyHandler;
PasteBtn.OnClick:=PasteHandler;
FindLeftBtn.OnClick:=WordSearchBackHandler;
FindRightBtn.OnClick:=WordSearchForwardHandler;
end;


procedure TCodeEditor.AddStatusPanel(var Pan: TPanel; Width: integer; AL: TAlignment; Hint: string);
var OldPPI,NewPPI: integer;
begin
{Handle variations in PPI video settings}
if Self.Owner is TForm then
	begin
	OldPPI:=TForm(Self.Owner).PixelsPerInch;
	NewPPI:=Screen.PixelsPerInch;
	if OldPPI<>NewPPI then Width:=MulDiv(Width,NewPPI,OldPPI)
	end;
Pan:=TPanel.Create(Self);
Pan.Parent:=StatusBar;
Pan.Width:=Width;
Pan.Align:=alLeft;
Pan.BevelOuter:=bvLowered;
Pan.OnMouseDown:=HandleMouseDown;
Pan.Hint:=Hint;
Pan.ShowHint:=Hint<>'';
end;


procedure TCodeEditor.SetupStatusBar;
begin
StatusBar:=TPanel.Create(Self);
StatusBar.Align:=alBottom;
StatusBar.Parent:=Self;
StatusBar.Height:=24;
StatusBar.BorderWidth:=2;
StatusBar.Visible:=true;
StatusBar.Name:='StatusBar';
StatusBar.Caption:='';
AddStatusPanel(StatusDis,140,taLeftJustify,'Status');
AddStatusPanel(ExpressDis,75,taCenter,'Expression Output');
AddStatusPanel(CmdNumberDis,47,taCenter,'Command #');

AddStatusPanel(ScoopStatusDis,65,taCenter,'Scoop Status');
AddStatusPanel(CmdSizeDis,44,taRightJustify,'Scoop Size');

AddStatusPanel(SearchCaseDis,25,taCenter,'Search Case');
AddStatusPanel(SearchDirDis,25,taCenter,'Search Direction');
AddStatusPanel(OverwriteDis,25,taCenter,'Overwrite');
AddStatusPanel(Label1,26,taRightJustify,'');
AddStatusPanel(LineNumDis,80,taRightJustify,'Line Number');
AddStatusPanel(Label2,30,taRightJustify,'');
AddStatusPanel(ColumnDis,59,taRightJustify,'Column');
AddStatusPanel(ModifiedDis,36,taCenter,'Modified');
AddStatusPanel(CharHexDis,26,taCenter,'Char Hex Value');

StatusBar.Font.Name:='Arial';
StatusBar.Font.Charset:=DEFAULT_CHARSET;
StatusBar.Font.Size:=8;
StatusBar.Font.Style:=[fsBold];
SearchCaseDis.OnMouseDown:=ToggleSearchCase;
SearchDirDis.OnMouseDown:=ToggleSearchDir;
StatusBar.OnMouseDown:=HandleMouseDown;
end;





procedure TCodeEditor.EditorScoopEvent(Command: TScoopCommand; Mark: TRect);
begin
MacroDecoder.XRegister.StoreChanges(Editor,Command,Mark);
if MacroDecoder.XRegister.ScoopOn then ShowStatus(Self);
end;


procedure TCodeEditor.HandleMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
inherited MouseDown(Button, Shift, X,Y);
end;


procedure TCodeEditor.HandleMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
if Sender is TScrollEditor then inherited MouseMove(Shift,X,Y);
end;


procedure TCodeEditor.SetupEditor;
var I: integer;
begin
Editor:=TScrollEditor.Create(Self);
Editor.Parent:=Self;
Editor.Align:=alClient;
Editor.OnChange:=ShowStatus;
Editor.BevelKind:=bkTile;
Editor.StandAlone:=True;
Editor.KeyMapping:=kmXED;
Editor.IsModified:=False;
Editor.OnKeyPress:=KeyPress;
Editor.OnKeyDown:=KeyDown;
Editor.Name:='Editor';
EditorMenu:=TPopupMenu.Create(Self);
AddMenuItem(EditorMenu,'Cu&t',CutHandler);
AddMenuItem(EditorMenu,'&Copy',CopyHandler);
AddMenuItem(EditorMenu,'&Paste',PasteHandler);
AddMenuItem(EditorMenu,'&Delete',DeleteHandler);
AddMenuItem(EditorMenu,'&Select All',SelectAllHandler);
AddMenuItem(EditorMenu,'-',nil);
AddMenuItem(EditorMenu,'&Find...',FindExecuteHandler);
Editor.PopupMenu:=EditorMenu;
Editor.OnScoopEvent:=EditorScoopEvent;
Editor.OnMouseDown:=HandleMouseDown;
Editor.OnMouseMove:=HandleMouseMove;
Toolbar.OnMouseDown:=HandleMouseDown;
for I:=0 to Toolbar.ComponentCount-1 do
 if Toolbar.Components[I] is TToolButton then TToolButton(Toolbar.Components[I]).OnMouseDown:=HandleMouseDown;
end;




procedure TCodeEditor.SetupCmdLine;
begin
CmdLine:=TCmdLine.Create(Self);
CmdLine.Parent:=Self;
CmdLine.Align:=alBottom;
CmdLine.OnKeyPress:=HandleCmdKey;
CmdLine.OnCmdBtnClick:=HandleCmdBtnClick;
CmdLine.OnUpClick:=HandleIncCmdLine;
CmdLine.OnDownClick:=HandleDecCmdLine;
CmdMenu:=TPopupMenu.Create(Self);
AddMenuItem(CmdMenu,'Cu&t',CmdCutHandler);
AddMenuItem(CmdMenu,'&Copy',CmdCopyHandler);
AddMenuItem(CmdMenu,'&Paste',CmdPasteHandler);
AddMenuItem(CmdMenu,'&Delete',CmdDeleteHandler);
AddMenuItem(CmdMenu,'-',nil);
AddMenuItem(CmdMenu,'&Select All',CmdSelectAllHandler);
CmdLine.CmdEditor.PopupMenu:=CmdMenu;
CmdLine.CmdEditor.OnMouseDown:=HandleMouseDown;
end;


{---------------- Save Command line Macros ------------------------------------}


procedure TCodeEditor.EditToString(var S: string);
begin
S:=CmdLine.CmdEditor.EditBuffer.Text;
end;


procedure TCodeEditor.StringToEdit(S: string);
begin
CmdLine.CmdEditor.ClearBuffer;
CmdLine.CmdEditor.InsertString(S);
CmdLine.CmdEditor.UndoControl.Initialize;
CmdLine.CmdEditor.RedoControl.Initialize;
CmdLine.CmdEditor.Invalidate;
end;



procedure TCodeEditor.SetCmdLineInx(Inx: integer);
begin
with MacroDecoder do
 begin
 if (Inx<0) or (Inx>CmdArraySize) then exit;
 EditToString(CmdArray[CmdArrayInx]);
 CmdArrayInx:=Inx;
 StringToEdit(CmdArray[CmdArrayInx]);
 end;
end;


procedure TCodeEditor.SaveCurrentMacros;
{Save current macro to INI}
var S: string;
var XeditIni: TIniFile;
var I: integer;
const Section='CurrentMacros';
begin
EditToString(S);
XeditIni:=TIniFile.Create(IniFile);
with XeditIni,MacroDecoder do
 begin
 EraseSection(Section);
 EditToString(S);
 CmdArray[CmdArrayInx]:=S;
 for I:=0 to CmdArraySize-1 do
 	EncodeINIString(XeditIni,Section,'Macro'+IntToStr(I),CmdArray[I]);
 WriteInteger(Section,'CurrentInx',CmdArrayInx);
 end;
XeditIni.Free;
end;


procedure TCodeEditor.RestoreCurrentMacros;
{Restore current macro }
var XeditIni: TIniFile;
var I,Inx: integer;
const Section='CurrentMacros';
begin
XeditIni:=TIniFile.Create(IniFile);
with XeditIni,MacroDecoder do
 begin
 for I:=0 to CmdArraySize-1 do
	CmdArray[I]:=DecodeINIString(XeditIni,Section,'Macro'+IntToStr(I),'');
 StringToEdit(CmdArray[CmdArrayInx]);
 Inx:=ReadInteger(Section,'CurrentInx',0);
 SetCmdLineInx(Inx);
 end;
XeditIni.Free;
end;


{------------------------------------------------------------------------------}



constructor TCodeEditor.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
{Hide the control until it is built, otherwise a gray box will appears}
Visible:=False;
try
{This is necessary because the toolbar
control recreates the window. If the
parent is not set at this time, you'll
get the "Control has no parent window" error.}
Parent:=TWinControl(AOwner);
Width:=200;
Height:=160;
ImageList:=TImageList.Create(Self);
SetupToolBar;
SetupStatusBar;
SetupEditor;
SetupCmdLine;
OpenDialog:=TOpenDialog.Create(Self);
SaveDialog:=TSaveDialog.Create(Self);
SaveDialog.Options := SaveDialog.Options + [ofOverwritePrompt];
FindDialog:=TFindDialog.Create(Self);
FindDialog.OnFind:=FindHandler;
MacroDecoder:=TMacroDecoder.Create(Editor,CmdLine.CmdEditor);
MacroDecoder.OnStatusUpdate:=HandleStatusEvent;
MacroDecoder.ExpressEval.OnStatusUpdate:=HandleExpressionEvent;
MacroDecoder.OnChange:=ShowStatus;
MacroDecoder.OnSave:=SaveHandler;
MacroDecoder.XRegister.OnChange:=ShowStatus;
FIniFile:='ConEdit';
Editor.Searcher.IgnoreCase:=True;
FileChange:=False;
ShowStatus(Self);
finally Visible:=True; end;
end;



destructor TCodeEditor.Destroy;
begin
MacroDecoder.Free;
FindDialog.Free;
SaveDialog.Free;
OpenDialog.Free;
CmdLine.Free;
ImageList.Free;
{Don't free Editor, StatusBar or ToolBar.
The panel as a container does it for us.}
inherited Destroy;
end;



{------------------------------------------------------------------------------}

function TCodeEditor.GetHexUnderCursor: string;
{return hex string for character under cursor}
var C: char;
begin
C:=Editor.GetLastChar;
if C=#0 then Result:='-'
else Result:='$'+IntToHex(Byte(C),2);
end;


procedure SetPanelDis(Pan: TPanel; B: boolean; S1,S2: string; C1,C2: TColor);
var C: TColor;
var S: string;
begin
if B then begin S:=S1; C:=C1; end
else begin S:=S2; C:=C2; end;
Pan.Caption:=S;
Pan.Color:=C;
end;


procedure TCodeEditor.ShowStatus(Sender: TObject);
{Update the status line }
begin
SetPanelDis(SearchCaseDis,Editor.Searcher.IgnoreCase,'S=s','S#s',clLime,clBtnFace);
SetPanelDis(SearchDirDis,MacroDecoder.FwdSearch,'=>','<=',clBtnFace,clLime);
CmdNumberDis.Caption:='Cmd: '+IntToStr(Self.MacroDecoder.CmdArrayInx);
ScoopStatusDis.Caption:='Scoop: '+Pickstring(MacroDecoder.XRegister.ScoopOn,'On','Off');
CmdSizeDis.Caption:=FloatToStrF(MacroDecoder.XRegister.Buffer.MaxItems-MacroDecoder.XRegister.Inx,ffNumber,18,0);

SetPanelDis(OverWriteDis,Editor.OverWrite,'Ovr','Ins',clYellow,clBtnFace);
Label1.Caption:='Ln:';
Label2.Caption:='Col:';
LineNumDis.Caption:=FloatToStrF(Editor.WorkingCursor.Position.Y,ffNumber,18,0);
ColumnDis.Caption:=FloatToStrF(Editor.GetCurrentColumn,ffNumber,18,0);
ModifiedDis.Caption:=PickString(Editor.IsModified,'Mod','');
SaveBtn.Enabled:=Editor.IsModified;
CharHexDis.Caption:=GetHexUnderCursor;
end;


{------------------------------------------------------------------------------}


procedure TCodeEditor.CmdCutHandler(Sender: TObject);
begin
CmdLine.CmdEditor.CutToClipboard;
PasteBtn.Enabled:=ClipBoard.HasFormat(CF_TEXT);
if CmdLine.CanFocus then CmdLine.SetFocus;
end;



procedure TCodeEditor.CmdCopyHandler(Sender: TObject);
begin
CmdLine.CmdEditor.CopyToClipboard;
PasteBtn.Enabled:=ClipBoard.HasFormat(CF_TEXT);
if CmdLine.CanFocus then CmdLine.SetFocus;
end;




procedure TCodeEditor.CmdPasteHandler(Sender: TObject);
begin
CmdLine.CmdEditor.PasteFromClipboard;
if CmdLine.CanFocus then CmdLine.SetFocus;
end;



procedure TCodeEditor.CmdDeleteHandler(Sender: TObject);
begin
CmdLine.CmdEditor.DeleteHighlighted;
if CmdLine.CanFocus then CmdLine.SetFocus;
end;




procedure TCodeEditor.CmdSelectAllHandler(Sender: TObject);
begin
CmdLine.CmdEditor.SelectAll;
if CmdLine.CanFocus then CmdLine.SetFocus;
end;



{------------------------------------------------------------------------------}


procedure TCodeEditor.CutHandler(Sender: TObject);
begin
Editor.CutToClipboard;
PasteBtn.Enabled:=ClipBoard.HasFormat(CF_TEXT);
if Editor.CanFocus then Editor.SetFocus;
end;



procedure TCodeEditor.CopyHandler(Sender: TObject);
begin
Editor.CopyToClipboard;
PasteBtn.Enabled:=ClipBoard.HasFormat(CF_TEXT);
if Editor.CanFocus then Editor.SetFocus;
end;




procedure TCodeEditor.PasteHandler(Sender: TObject);
begin
Editor.PasteFromClipboard;
if Editor.CanFocus then Editor.SetFocus;
end;


procedure TCodeEditor.WordSearchBackHandler(Sender: TObject);
begin
DoWordBackSearch;
end;


procedure TCodeEditor.WordSearchForwardHandler(Sender: TObject);
begin
DoWordFwdSearch;
end;




procedure TCodeEditor.DeleteHandler(Sender: TObject);
begin
Editor.DeleteHighlighted;
if Editor.CanFocus then Editor.SetFocus;
end;




procedure TCodeEditor.SelectAllHandler(Sender: TObject);
begin
Editor.SelectAll;
if Editor.CanFocus then Editor.SetFocus;
end;



procedure TCodeEditor.FindHandler(Sender: TObject);
var Target: string;
begin
Target:=FindDialog.FindText;
Editor.Searcher.WholeWordMode:=(FindDialog.Options*[frWholeWord])=[frWholeWord];
Editor.Searcher.IgnoreCase:=not ((FindDialog.Options * [frMatchCase]) = [frMatchCase]);
if (FindDialog.Options * [frDown]) = [frDown] then Editor.ForwardSearch(Target)
else Editor.BackwardSearch(Target);
Editor.Invalidate;
end;



procedure TCodeEditor.FindExecuteHandler(Sender: TObject);
var S: string;
begin
S:=Editor.GetHighlightedText;
if S='' then S:=Editor.GetNearestWord;
FindDialog.FindText:=S;
FindDialog.Execute;
end;


procedure TCodeEditor.ToggleSearchCase(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
Editor.Searcher.IgnoreCase:=not Editor.Searcher.IgnoreCase;
ShowStatus(Self);
end;


procedure TCodeEditor.ToggleSearchDir(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
MacroDecoder.FwdSearch:=not MacroDecoder.FwdSearch;
ShowStatus(Self);
end;


{------------------------------------------------------------------------------}


procedure TCodeEditor.HandleStatusEvent(Status: string; Sender: TObject);
begin
StatusDis.Caption:=' '+Status;
end;


procedure TCodeEditor.HandleExpressionEvent(Status: string; Sender: TObject);
begin
ExpressDis.Caption:=' '+Status;
end;



procedure TCodeEditor.HandleCmdBtnClick(Sender: TObject);
begin
MacroDecoder.DecodeMacro(CmdLine.CmdEditor.Text);
end;



procedure TCodeEditor.HandleIncCmdLine(Sender: TObject);
begin
with MacroDecoder do
 begin
 EditToString(CmdArray[CmdArrayInx]);
 MacroDecoder.IncCmdArrayInx;
 StringToEdit(CmdArray[CmdArrayInx]);
 ShowStatus(Self);
 end;
end;



procedure TCodeEditor.HandleDecCmdLine(Sender: TObject);
begin
with MacroDecoder do
 begin
 EditToString(CmdArray[CmdArrayInx]);
 MacroDecoder.DecCmdArrayInx;
 StringToEdit(CmdArray[CmdArrayInx]);
 ShowStatus(Self);
 end;
end;

{------------------------------------------------------------------------------}

procedure TCodeEditor.SetCurrentFilename(Filename: string);
{Set Current source file name}
begin
if Filename<>FCurrentFile then FileChange:=True;
FCurrentFile:=Filename;
ChangeDefaultDir(Filename);
if Assigned(FOnFileChange) then FOnFileChange(Self);
end;



procedure TCodeEditor.LoadSourceFile(Filename: string);
begin
Editor.LoadFromFile(Filename);
Editor.Invalidate;
SetCurrentFilename(Filename);
Editor.IsModified:=False;
ShowStatus(Self);
end;



function TCodeEditor.SaveChanges: boolean;
var R: integer;
begin
Result:=False;
if Editor.IsModified then
	begin
	R:=MessageDlg('Save Changes?', mtConfirmation, [mbYes, mbNo, mbCancel], 0);
	case R of
	 mrYes: SaveHandler(Self);
	 mrCancel: exit;
	 end
	end;
Result:=True;
end;



procedure TCodeEditor.NewHandler;
begin
if not SaveChanges then exit;
Editor.ClearBuffer;
Editor.IsModified:=False;
SetCurrentFilename('');
ShowStatus(Self);
end;


function TCodeEditor.OpenFile(Filename: string): boolean;
begin
Result:=False;
if not FileExists(FileName) then
	begin
	MessageDlg('File Does Not Exist.', mtWarning,[mbOk], 0);
	exit;
	end;
if not SaveChanges then exit;
LoadSourceFile(Filename);
Result:=True;
end;


procedure TCodeEditor.OpenHandler(Sender: TObject);
begin
if OpenDialog.Execute then
	begin
	OpenFile(OpenDialog.FileName);
	end;
end;




function TCodeEditor.BackupAndSave(Filename: string): boolean;
{ Backup and save to current WorkFilename}
var BakName,SafeTmp,SaveName: string;
begin
Result:=False;
SetCurrentFilename(Filename);
BakName:=ChangeFileExt(Filename,'.BAK');
SafeTmp:=ExtractFilePath(FileName)+'xedsave.xet';
SaveName:=FileName;
if FileExists(FileName) then
	begin
	{backup if file already exists}
	if FileExists(BakName) then DeleteFile(BakName);
	if not RenameFile(FileName,BakName) then
		begin
		{Save to special file if workfile is in use}
		DeleteFile(SafeTmp);
		SaveName:=SafeTmp;
		MessageDlg('File is open by another program.'+#13#10+
			'Results saved to: '+ExtractFilename(SafeTmp),
			 mtWarning,[mbOk], 0);
		end;
	end;
Editor.SaveToFile(SaveName);
Editor.IsModified:=False;
ShowStatus(Self);
if Editor.CanFocus then Editor.SetFocus;
if Assigned(FOnSaveAs) then FOnSaveAs(Self);
{True if no exceptions thrown}
Result:=True;
end;



procedure TCodeEditor.SaveAsHandler(Sender: TObject);
begin
SaveDialog.FileName:=CurrentFile;
if SaveDialog.Execute then BackupAndSave(SaveDialog.Filename);
end;



procedure TCodeEditor.SaveHandler(Sender: TObject);
begin
if CurrentFile='' then SaveAsHandler(Sender)
else BackupAndSave(CurrentFile);
end;


{------------------------------------------------------------------------------}



procedure TCodeEditor.EnterCmdLine;
begin
with MacroDecoder do
 begin
 CmdArray[CmdArrayInx]:=CmdLine.CmdEditor.Text;
 IncCmdArrayInx;
 CmdLine.CmdEditor.ClearBuffer;
 CmdLine.CmdEditor.InsertChar(ESC_CHAR);
 CmdLine.CmdEditor.SetFocus;
 ShowStatus(Self);
 end;
end;

{------------------------------------------------------------------------------}



function TCodeEditor.GetTarget: string;
begin
if not Editor.IsHighlighted then Editor.MarkWord;
Result:=Editor.GetHighlightedText;
end;


procedure TCodeEditor.DoWordBackSearch;
var Target: string;
begin
Target:=GetTarget;
Editor.UndoControl.SetupDeltaUndo(Editor.WorkingCursor.Position);
Editor.CusorToHighlightStart;
Editor.UndoControl.DoDeltaCursorStore(Editor.WorkingCursor.Position);
Editor.Searcher.WholeWordMode:=True;
Editor.BackwardSearch(Target);
end;



procedure TCodeEditor.DoWordFwdSearch;
var Target: string;
begin
Target:=GetTarget;
Editor.UndoControl.SetupDeltaUndo(Editor.WorkingCursor.Position);
Editor.CusorToHighlightStart;
Editor.CursorToHighlightEnd;
Editor.UndoControl.DoDeltaCursorStore(Editor.WorkingCursor.Position);
Editor.Searcher.WholeWordMode:=True;
Editor.ForwardSearch(Target);
end;



procedure TCodeEditor.WordBackward(WordStart: boolean);
{Move back a word using word movement settings}
{Alt = do the oposite of setting}
begin
Editor.ResetMarkers;
Editor.BackWord(WordStart);
end;


procedure TCodeEditor.WordForward(WordStart: boolean);
{Move forward a word using word movement settings}
{Alt = do the oposite of setting}
begin
Editor.ResetMarkers;
Editor.ForwardWord(WordStart);
end;


function TCodeEditor.HandleCtrlCommands(Key: Word; Shift: TShiftState): boolean;
{Handle command where Control key is down}
begin
Result:=True;
 case Key of
 VK_F1: if MacroDecoder.XRegister.CopyFromClipboard then HandleStatusEvent('Pasted',Self);
 VK_F2: if MacroDecoder.XRegister.CopyToClipboard then
	begin
	HandleStatusEvent('Copied',Self);
	PasteBtn.Enabled:=ClipBoard.HasFormat(CF_TEXT);
	end;
 VK_F9: DoWordBackSearch;
 VK_F10: DoWordFwdSearch;
 VK_LEFT: WordBackward(False);
 VK_RIGHT: WordForward(False);
 VK_UP: Editor.ScrollUpLine;
 VK_DOWN: Editor.ScrollDownLine;
 VK_PAUSE: MacroDecoder.BreakFlag:=True;
 else Result:=False;
 end;
end;



function TCodeEditor.HandleNormalCommands(Key: Word; Shift: TShiftState): boolean;
{Handle command where Control, Shift or Alt are not down}
begin
Result:=True;
case Key of
 VK_F1: HandleDecCmdLine(Self);
 VK_F2: HandleIncCmdLine(Self);
 VK_F5:	begin Editor.ResetMarkers; MacroDecoder.XRegister.InsertScoop; end;
 VK_F8:	MacroDecoder.XRegister.Initialize;
 VK_F9: MacroDecoder.DropLocMarker;
 VK_F10: MacroDecoder.GoToLocMarker;
 VK_PAUSE: MacroDecoder.BreakFlag:=True;
 else Result:=False;
 end;
end;



procedure TCodeEditor.KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{All virtual keys such as F1, Home, PageUp etc. are handled here }
var Handled: boolean;
begin
MacroDecoder.BreakFlag:=False;
{Ignore keyboard-state keys like Control, Shift etc.}
if (Key=VK_SHIFT) or (Key=VK_CONTROL) or (Key=VK_MENU) or
   (Key=VK_CAPITAL) then exit;
if Assigned(FOnEditKeyDown) then FOnEditKeyDown(Sender,Key,Shift);
Handled:=True;
if ssCtrl in Shift then Handled:=HandleCtrlCommands(Key,Shift)
else Handled:=HandleNormalCommands(Key,Shift);
if Handled then begin Key:=0; ShowStatus(Sender); end;
end;


procedure TCodeEditor.KeyPress(Sender: TObject; var Key: Char);
var Handled: boolean;
var Shift: boolean;
begin
Handled:=True;
if Assigned(FOnEditKeyPress) then FOnEditKeyPress(Sender,Key);
Shift:=(GetAsyncKeyState(VK_SHIFT) and $8000)<>0;

if Shift and (Key=CtrlV) then Editor.InsertString(CmdLine.CmdEditor.EditBuffer.Text)
else case Key of
  ESC_Char:
	begin
	EnterCmdLine;
	ShowStatus(Sender);
	end;
  CtrlG:
	begin
	MacroDecoder.DecodeMacro(CmdLine.CmdEditor.Text);
	end;
  else  Handled:=MacroDecoder.ExtendedImmediate(Key);
  end;

if Handled then Key:=#0;
end;




procedure TCodeEditor.HandleCmdKey(Sender: TObject; var Key: char);
var S: string;
var L: integer;
begin
S:=CmdLine.CmdEditor.Text;
L:=Length(S);
if (Key=Esc_Char) and (L>0) and (S[L]=Esc_Char) then
	begin
	Editor.SetFocus;
	MacroDecoder.DecodeMacro(S);
	end;
end;


procedure TCodeEditor.Invalidate;
begin
inherited;
if Editor<>nil then Editor.Invalidate;
if Toolbar<>nil then Toolbar.Invalidate;
if StatusBar<>nil then StatusBar.Invalidate;
end;


procedure TCodeEditor.Finalize;
begin
SaveCurrentMacros
end;



procedure TCodeEditor.Initialize;
begin
RestoreCurrentMacros
end;




end.
