unit Colorbar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls,BitmapLib,Vector,DSupport,DepthColors;

{ Structure used for the array of bands }

const
 ColorBase=100000;		{Base value for Offsets}
 MaxColors=48;			{Maximum number of colors}

type TColorBand=record		{Color band structure}
 Color: TColor;			{Color of the band on the color bar}
 Bottom: Longint;		{Bottom of individual color bad}
 Depth: double;			{Scaled bottom of band}
 end;

type TColorArray=record		{Structure for array of color bands}
 Count: integer;		{Number of bands currently used}
 Bands: array [0..MaxColors-1] of TColorBand;
 end;

TBarEvent = procedure(Sender: TObject; Index: integer;
                      Bottom:LongInt;Depth: double) of object;

type TColorCount=(dc12,dc20,dc24,dc30,dc32);

{Prototypes for TColorBar}

type
TColorBar = class(TPaintBox)
private
  RangeTop,RangeBottom,RangeDelta: Double;
  MoveInx: integer;
  StartY, EndY, CurY: integer;
  FBarMoving: boolean;
  FSelected: integer;
  FAdjustEnds: boolean;
  FAdjustable: boolean;
  FLabelBars: boolean;
  FOnChangeSelect: TNotifyEvent;
  FOnChangeSpread: TNotifyEvent;
  FOnChangingBar: TBarEvent;
  FColorCount: TColorCount;
  FScaleFactor: double;
  FDecimals: integer;

  procedure SetDecimals(V:integer);
  procedure SetLabelBars(V:Boolean);
  procedure SetScaleFactor(V: double);
  procedure SetColorCount(V:TColorCount);
  procedure SetColors12;
  procedure SetColors20;
  procedure SetColors24;
  procedure SetColors30;
  procedure SetColors32;
  procedure SetSelected(New: integer);

  procedure PaintOneBand(Canvas: TCanvas; Inx: Integer; DoLabel: boolean);
  procedure FocusSelected;
  procedure MoveBars(Inx,EY: integer; Stretch: boolean);
  function OverBar(Y: integer): integer;
  procedure DrawBar(Y: integer);
  procedure ResetAllDepths;
  procedure ResetDepthItem(I: integer);
  function BottomToDepth(V: Longint): double;
  procedure SetDepthCount(const Value: integer);
protected
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure Paint; override;
  procedure ChangeSelect; dynamic;
  procedure ChangeSpread; dynamic;
  procedure ChangingBar; dynamic;
public
  ColorDepth: TColorArray;
  procedure DrawColorbar(Canvas: TCanvas);
  procedure SetBarByDepth(Inx:integer;D: double);
  function DepthToBand(V: double): integer;
  function DepthToColor(V: double): TColor;
  procedure SetBarPosition(Inx: integer;Pos: Longint);
  procedure SetDefaultColors;
  procedure ResetSpacing;
  procedure SpaceEvenly(P1,P2: integer);
  procedure DoGradient(P1,P2: integer);
  procedure HueGradient(P1, P2: integer);
  procedure SetRange(T,B:double);
  procedure InvertColors;
  function MouseToBand(Y: integer): integer;
  function IndexOf(V: double): integer;
  function ColorOf(V: Double): TColor;
  constructor Create(AOwner: TComponent); override;
  property BarMoving: boolean read FBarMoving;
  property Count: integer read ColorDepth.Count write SetDepthCount;
published
 property Decimals: integer read FDecimals write SetDecimals default 0;
 property LabelBars: boolean read FLabelBars write SetLabelBars default False;
 property ScaleFactor: double read FScaleFactor write SetScaleFactor;
 property Selected: integer read FSelected write SetSelected default 1;
 property OnChangeSelect: TNotifyEvent read FOnChangeSelect write FOnChangeSelect;
 property OnChangeSpread: TNotifyEvent read FOnChangeSpread write FOnChangeSpread;
 property OnChangingBar: TBarEvent read FOnChangingBar write FOnChangingBar;
 property Adjustable: boolean read FAdjustable write FAdjustable default True;
 property AdjustEnds: boolean read FAdjustEnds write FAdjustEnds default True;
 property ColorCount: TColorCount read FColorCount write SetColorCount default dc12;
end;

procedure Register;

implementation


function MulDiv2(X,Y,Z: longint):longint;
{32 bit version of Windows MulDiv function}
begin
Result:=(X*Y) div Z;
end;


{------------------ Support Routines ------------------------------}


function TColorBar.BottomToDepth(V: Longint): double;
{Calculate the depth from the a bottom value}
begin
Result:=RangeTop-(RangeDelta*(V/ColorBase));
end;


procedure TColorBar.ResetDepthItem(I: integer);
{Set Depth to match a Bottom}
begin
with ColorDepth.Bands[I] do
Depth:=BottomToDepth(Bottom);
end;


procedure TColorBar.ResetAllDepths;
{Set Depths to match Bottoms}
var P: integer;
begin
for P:=0 to ColorDepth.Count-1 do
  with ColorDepth.Bands[P] do Depth:=BottomToDepth(Bottom);
end;


procedure TColorBar.SetRange(T,B:double);
{Set range of depth and adjust according to band}
begin
RangeTop:=T;
RangeBottom:=B;
RangeDelta:=T-B;
ResetAllDepths;
end;


procedure TColorBar.ResetSpacing;
{Set colors for an even spread}
var P: integer;
begin
with ColorDepth do
 begin
 Bands[0].Bottom:=0;
 for P:=1 to Count-1 do Bands[P].Bottom:=MulDiv2(P,ColorBase,Count-1);
 ResetAllDepths;
 end;
Invalidate;
end;


procedure TColorBar.SpaceEvenly(P1,P2: integer);
{ Set even spacing between P1 and P2 }
var P,B,D: integer;
begin
with ColorDepth do
 begin
 if P1<0 then P1:=0;
 if P2>Count then P2:=Count;
 B:=Bands[P2].Bottom-Bands[P1].Bottom;
 D:=P2 - P1;
 for P:=P1+1 to P2-1 do Bands[P].Bottom:=MulDiv2(P-P1,B,D)+Bands[P1].Bottom;
 end;
ResetAllDepths;
Invalidate;
end;


function TColorBar.DepthToBand(V: double): integer;
{Return the index of the band that the value falls into}
var I: integer;
var L,H,T: double;
begin
Result:=-1;
for I:=0 to ColorDepth.Count-1 do
	begin
	H:=ColorDepth.Bands[I].Depth;
        L:=ColorDepth.Bands[I+1].Depth;
	if L>H then begin T:=L; L:=H; H:=T; end;
	if (V>=L) and (V<=H) then begin Result:=I+1; break; end;
	end;
end;



function TColorBar.DepthToColor(V: double): TColor;
var Inx: integer;
begin
Inx:=DepthToBand(V);
Result:=ColorDepth.Bands[Inx].Color;
end;


procedure TColorBar.HueGradient(P1,P2: integer);
{Create color gradient between the colors in P1 and P2}
var P,T,DI:integer;
var CV1,CV2: THSBRecord;
var HD: THSBRecord;
begin
if P1<1 then P1:=1;
if P2>=ColorDepth.Count then P2:=ColorDepth.Count-1;
if P2<P1 then SwapIntegers(P1,P2);
DI:=P2-P1;
if DI<=2 then exit;
CV1:=RGBToHSB(ColorToTriple(ColorDepth.Bands[P1].Color));
CV2:=RGBToHSB(ColorToTriple(ColorDepth.Bands[P2].Color));
HD.Hue:=(CV2.Hue - CV1.Hue)/DI;
HD.Sat:=(CV2.Sat - CV1.Sat)/DI;
HD.Bright:=(CV2.Bright - CV1.Bright)/DI;
for P:=P1+1 to P2-1 do
	begin
	CV1.Hue:=CV1.Hue+HD.Hue;
	CV1.Sat:=CV1.Sat+HD.Sat;
	CV1.Bright:=CV1.Bright+HD.Bright;
	ColorDepth.Bands[P].Color:=TripleToColor(HSBtoRGB(CV1));
	end;
Invalidate;
end;



procedure TColorBar.DoGradient(P1,P2: integer);
{Create color gradient between the colors in P1 and P2}
var P,T,DI:integer;
var R1,G1,B1: byte;
var R2,G2,B2: byte;
var CV1,CV2,IV: T3DVector;
begin
if P1<1 then P1:=1;
if P2>=ColorDepth.Count then P2:=ColorDepth.Count-1;
if P2<P1 then begin T:=P1; P1:=P2; P2:=T; end;
DI:=P2-P1+1;
if DI<=2 then exit;
CV1:=ColorToVector(ColorDepth.Bands[P1].Color);
CV2:=ColorToVector(ColorDepth.Bands[P2].Color);
IV:=ScalarDivide3D(VectorSubtract3D(CV2,CV1),DI);

ColorToRGBBytes(ColorDepth.Bands[P1].Color,R1,G1,B1);
ColorToRGBBytes(ColorDepth.Bands[P2].Color,R2,G2,B2);
for P:=P1+1 to P2-1 do
	begin
	CV1:=VectorAdd3D(CV1,IV);
	ColorDepth.Bands[P].Color:=VectorToColor(CV1);
	end;
Invalidate;
end;



procedure TColorBar.SetColors12;
var I: integer;
begin
for I:=0 to High(DepthColors13) do ColorDepth.Bands[I].Color:=DepthColors13[I];
ColorDepth.Count:=High(DepthColors13)+1;
end;


procedure TColorBar.SetColors20;
var I: integer;
begin
for I:=0 to High(GeoColors20) do ColorDepth.Bands[I].Color:=GeoColors20[I];
ColorDepth.Count:=High(GeoColors20)+1;
end;



procedure TColorBar.SetColors24;
var I: integer;
begin
for I:=0 to High(DepthColors24) do ColorDepth.Bands[I].Color:=DepthColors24[I];
ColorDepth.Count:=High(DepthColors24)+1;
end;


procedure TColorBar.SetColors30;
var I: integer;
begin
for I:=0 to High(Geocolors30) do ColorDepth.Bands[I].Color:=Geocolors30[I];
ColorDepth.Count:=High(Geocolors30)+1;
end;



procedure TColorBar.SetColors32;
var I: integer;
begin
for I:=0 to High(ColorTemp32) do ColorDepth.Bands[I].Color:=ColorTemp32[I];
ColorDepth.Count:=High(ColorTemp32)+1;
end;


procedure TColorBar.SetDefaultColors;
{Set default colors }
begin
case FColorCount of
 dc12: SetColors12;
 dc20: SetColors20;
 dc24: SetColors24;
 dc30: SetColors30;
 dc32: SetColors32;
 end;
Invalidate;
end;


procedure TColorBar.SetColorCount(V:TColorCount);
{Set default colors }
begin
if FColorCount<>V then
	begin
	FColorCount:=V;
	SetDefaultColors;
	ResetSpacing;
	end;
end;


procedure TColorBar.SetLabelBars(V: boolean);
{Set Label Bars options }
begin
if FLabelBars<>V then
	begin
	FLabelBars:=V;
	Invalidate;
	end;
end;


procedure TColorBar.SetScaleFactor(V: double);
{Set Scale factor }
begin
if FScaleFactor<>V then
	begin
	FScaleFactor:=V;
	Invalidate;
	end;
end;


procedure TColorBar.SetDecimals(V:integer);
{Set digits to display after decimal point}
begin
if FDecimals<>V then
	begin
	FDecimals:=V;
	Invalidate;
	end;
end;


procedure TColorBar.FocusSelected;
{Draw focus rectangle inside color band}
var H,X1,Y1,X2,Y2: integer;
begin
X1:=0;
H:=Height; X2:=Width;
if (FSelected>0) and (FSelected<ColorDepth.Count) then
	begin
        if FSelected=0 then Y1:=0
	else Y1:=MulDiv2(H,ColorDepth.Bands[FSelected-1].Bottom,ColorBase);
	Y2:=MulDiv2(H,ColorDepth.Bands[FSelected].Bottom,ColorBase);
	Canvas.Brush.Style:=bsClear;
	Canvas.Pen.Mode:=pmNot;
	Canvas.Pen.Style:=psDot;
 	Canvas.Rectangle(X1+1,Y1+1,X2-1,Y2-1);
	end;
end;



procedure TColorBar.DrawBar(Y: integer);
{Draw moving bar}
var X1,X2: integer;
begin
with Canvas do
 begin
 X1:=ClipRect.Left;
 X2:=ClipRect.Right;
 Pen.Mode:=pmNot;
 Pen.Color:=clBlack;
 MoveTo(X1,Y);
 LineTo(X2,Y);
 CurY:=Y;
 end;
end;


function TColorBar.OverBar(Y: integer): integer;
{Return index if Y is over a bar}
var P,H,L,First,Last: integer;
begin
Result:=-1;
H:=Height;
if not FAdjustable then exit;
if FAdjustEnds then
	begin
	First:=0;
	Last:=ColorDepth.Count-1;
	end
else
	begin
	First:=1;
	Last:=ColorDepth.Count-2;
	end;
for P:=First to Last do
	begin
	L:=MulDiv2(H,ColorDepth.Bands[P].Bottom,ColorBase);
	if (Y>=L-1) and (Y<=L+1) then
		begin
		Result:=P;
		break;
		end;
	end;
end;



function TColorBar.MouseToBand(Y: integer): integer;
{Return index if Y is in range}
var P,H,Y1,Y2: integer;
begin
H:=Height;
Y2:=MulDiv2(H,ColorDepth.Bands[0].Bottom,ColorBase);
Result:=-1;
for P:=1 to ColorDepth.Count-1 do
	begin
	Y1:=Y2;
	Y2:=MulDiv2(H,ColorDepth.Bands[P].Bottom,ColorBase);
	if (Y>=Y1) and (Y<=Y2) then
		begin
		Result:=P;
		break;
		end;
	end;
end;



procedure TColorBar.SetBarPosition(Inx: integer; Pos: Longint);
{Set and normalize bar position}
var P: integer;
var T: longint;
var Pixel: integer;
begin
Pixel:=Trunc(ColorBase * (1/ Height)) * 2;
with ColorDepth do
 begin
 ColorDepth.Bands[Inx].Bottom:=Pos;
 ResetDepthItem(Inx);
 for P:=Inx to Count-1 do
 	if Bands[P].Bottom>=Bands[P+1].Bottom then
		begin
		T:=Bands[P].Bottom+Pixel;
		if T>ColorBase then T:=ColorBase;
		ColorDepth.Bands[P+1].Bottom:=T;
		ResetDepthItem(P+1);
		end;
 for P:=Inx downto 1 do
 	if Bands[P].Bottom<=Bands[P-1].Bottom then
		begin
		T:=Bands[P].Bottom-Pixel;
		if T<0 then T:=0;
		ColorDepth.Bands[P-1].Bottom:=T;
		ResetDepthItem(P-1);
		end;
 Invalidate;
 end;
end;



procedure TColorBar.SetBarByDepth(Inx: integer; D: double);
{Set bar based on depth}
begin
SetBarPosition(Inx,Trunc(((RangeTop-D)/RangeDelta)*ColorBase));
end;




procedure TColorBar.MoveBars(Inx,EY: integer; Stretch: boolean);
{Set bar at Inx to value corresponding to EY}
var Down: boolean;
var NewPos: integer;
begin
NewPos:=MulDiv2(ColorBase,EY,Height);
Down:=ColorDepth.Bands[Inx].Bottom < NewPos;
SetBarPosition(Inx,NewPos);
if Stretch then
	begin
	SpaceEvenly(0,Inx);
	SpaceEvenly(Inx,ColorDepth.Count-1);
	end;
Invalidate;
end;

        

function TColorBar.IndexOf(V: double): integer;
{Find the band that corresponds to value}
var P: integer;
begin
Result:=0;
with ColorDepth do
 begin
 if  V>ColorDepth.Bands[0].Depth then exit;
 for P:=1 to Count-1 do
  if  V>=ColorDepth.Bands[P].Depth then
	begin Result:=P; break; end;
 end;
end;



function TColorBar.ColorOf(V: Double): TColor;
{Find color that corresponds to Value}
begin
Result:=ColorDepth.Bands[IndexOf(V)].Color;
end;


{------------------ Object Handling Routines -------------------}


procedure Register;
begin
RegisterComponents('Fountain', [TColorBar]);
end;


procedure TColorBar.ChangingBar;
var T: LongInt;
begin
T:=MulDiv2(ColorBase,CurY,Height);
if Assigned(FOnChangingBar) then FOnChangingBar(Self,MoveInx,T,BottomToDepth(T));
end;


procedure TColorBar.ChangeSpread;
begin
if Assigned(FOnChangeSpread) then FOnChangeSpread(Self);
end;


procedure TColorBar.ChangeSelect;
begin
if Assigned(FOnChangeSelect) then FOnChangeSelect(Self);
end;


constructor TColorBar.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
SetRange(ColorBase,0);
SetDefaultColors;
ResetSpacing;
FSelected:=1;
FAdjustEnds:=True;
FAdjustable:=True;
FLabelBars:=False;
FScaleFactor:=1;
FDecimals:=0;
end;



function GetIntensity(C: TColor): Integer;
begin
C:=(C and $00FFFFFF);
Result:=(GetRValue(C)+GetGValue(C)+GetBValue(C)) div 3;
end;



function ContrastColor(C: TColor): TColor;
{Return a contrasting color }
begin
Result:=0;
if (C and $FF0000)<$800000 then Result:=$FF0000;
if (C and $FF00)<$8000 then Result:=Result or $FF00;
if (C and $FF)<$80 then Result:=Result or $FF;
end;


procedure OutputText(Canvas: TCanvas; Str: String; X1,Y1,X2,Y2: integer);
{Output string into a rectangular area}
var Temp: array [0..20] of char;
var R: TRect;
begin
if (Canvas.TextHeight(Str)+1)>=(Y2-Y1) then exit;
R.left:= X1; R.right:= X2; R.top:=Y1; R.bottom:=Y2;
InflateRect(R,-1,-1);
Canvas.Font.Color:=ContrastColor(Canvas.Brush.Color);
DrawText(Canvas.Handle, StrPCopy(@Temp, Str), -1, R, DT_NOCLIP or DT_SINGLELINE or DT_CENTER or DT_VCENTER);
end;



procedure TColorBar.PaintOneBand(Canvas: TCanvas; Inx: integer; DoLabel: Boolean);
var Y1,Y2: integer;
begin
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Style:=psSolid;
Canvas.Brush.Style:=bsSolid;
if Inx=0 then Y1:=0 else Y1:=MulDiv2(Height,ColorDepth.Bands[Inx-1].Bottom,ColorBase);
if Inx>=ColorDepth.Count then Y2:=Height else Y2:=MulDiv2(Height,ColorDepth.Bands[Inx].Bottom,ColorBase);
Canvas.Brush.Color:=ColorDepth.Bands[Inx].Color;
if GetIntensity(Canvas.Brush.Color)<50 then Canvas.Pen.Color:= clGray
else Canvas.Pen.Color:= clBlack;
Canvas.Rectangle(0,Y1-1,Width,Y2);
if DoLabel then OutputText(Canvas,FloatToStrF(FScaleFactor*ColorDepth.Bands[Inx].Depth,ffFixed,18,Decimals),0,Y1,Width,Y2);
if Enabled and (Inx=Selected) then FocusSelected;
end;



procedure TColorBar.DrawColorbar(Canvas: TCanvas);
var P: integer;
begin
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Style:=psSolid;
PaintOneBand(Canvas,0,False);
for P:=1 to ColorDepth.Count-1 do PaintOneBand(Canvas,P,FLabelBars);
PaintOneBand(Canvas,ColorDepth.Count,False);
end;



procedure TColorBar.Paint;
begin
inherited Paint;
DrawColorbar(Canvas);
end;



procedure TColorBar.SetSelected(New: integer);
begin
if (New=FSelected) or
   (New<0) or (New>=ColorDepth.Count) then exit;
FocusSelected;
FSelected:=New;
FocusSelected;
ChangeSelect;
end;



{---------------- Message Handlers -----------------}


procedure TColorBar.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
MoveInx:=OverBar(Y);
if MoveInx>-1 then
	begin
	FBarMoving:=True;
	StartY:=Y;
	DrawBar(Y);
	end
else SetSelected(MouseToBand(Y));
inherited MouseDown(Button, Shift, X, Y);
end;


procedure TColorBar.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
inherited MouseMove(Shift, X, Y);
if FBarMoving then
	begin
	DrawBar(CurY);
	DrawBar(Y);
	ChangingBar;
	end
else
	begin
	if OverBar(Y)>-1 then Cursor:=crVSplit
	else Cursor:=crDefault;
	end;
end;



procedure TColorBar.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited MouseUp(Button, Shift, X, Y);
if FBarMoving then
	begin
	FBarMoving:=False;
	DrawBar(CurY);
	EndY:=Y;
	MoveBars(MoveInx,EndY,ssCtrl in Shift);
	ChangeSpread;
	end;
end;


procedure TColorBar.InvertColors;
var I,J: integer;
begin
I:=1; J:=ColorDepth.Count-1;
while I<=J do
	begin
	SwapColors(ColorDepth.Bands[I].Color,ColorDepth.Bands[J].Color);
	Inc(I); Dec(J);
	end;
Invalidate;
end;


procedure TColorBar.SetDepthCount(const Value: integer);
begin
if (Value<0) or (Value>=MaxColors) then exit;
ColorDepth.Count:=Value;
ResetSpacing;
Invalidate;
end;


end.
