unit ContextMenu;

interface

uses Windows,Controls,Types,Classes,ComCtrls,Clipbrd,Menus;

type TContextMenu = class(TPopupMenu)
 private
  FRichEdit: TRichEdit;
  procedure SelectHandler(Sender: TObject);
  procedure AddMenuItem(Popup: TPopupMenu; Caption: string; ClickHandler: TNotifyEvent; Tag: integer);
  procedure HandlePopup(Sender: TObject);
  procedure SetRichEdit(Value: TRichEdit);
 protected
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
 published
  property RichEdit: TRichEdit read FRichEdit write SetRichEdit;
 end;

{Note: the word Register must have upper case R or it won't Register}

procedure Register;


implementation


procedure Register;
begin
  RegisterComponents('Fountain', [TContextMenu]);
end;



procedure TContextMenu.SetRichEdit(Value: TRichEdit);
begin
if Value<>nil then
	begin
	FRichEdit:=Value;
	FRichEdit.PopupMenu:=Self;
	end;
end;



procedure TContextMenu.HandlePopup(Sender: TObject);
var HoldsText: boolean;
begin
if FRichEdit=nil then exit;
HoldsText:=FRichEdit.SelLength>0;
Self.Items[0].Enabled:=FRichEdit.CanUndo;
Self.Items[2].Enabled:=HoldsText;
Self.Items[3].Enabled:=HoldsText;
Self.Items[4].Enabled:=ClipBoard.HasFormat(CF_TEXT);
Self.Items[5].Enabled:=HoldsText;
end;



procedure TContextMenu.AddMenuItem(Popup: TPopupMenu; Caption: string; ClickHandler: TNotifyEvent; Tag: integer);
var Item: TMenuItem;
begin
Item:=TMenuItem.Create(Popup);
Item.Caption:=Caption;
Item.OnClick:=ClickHandler;
Item.Tag:=Tag;
Popup.Items.Add(Item);
end;



constructor TContextMenu.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
Self.Items.Clear;
Self.OnPopup:=HandlePopup;
if csDesigning in ComponentState then exit;
AddMenuItem(Self,'Undo',SelectHandler,0);
AddMenuItem(Self,'-',SelectHandler,-1);
AddMenuItem(Self,'Cut',SelectHandler,1);
AddMenuItem(Self,'Copy',SelectHandler,2);
AddMenuItem(Self,'Paste',SelectHandler,3);
AddMenuItem(Self,'Delete',SelectHandler,4);
AddMenuItem(Self,'-',SelectHandler,-1);
AddMenuItem(Self,'Select All',SelectHandler,5);
end;



destructor TContextMenu.Destroy;
begin
inherited Destroy;
end;



procedure TContextMenu.SelectHandler(Sender: TObject);
begin
if FRichEdit=nil then exit;
case TMenuItem(Sender).Tag of
 0: FRichEdit.Undo;
 1: FRichEdit.CutToClipboard;
 2: FRichEdit.CopyToClipboard;
 3: FRichEdit.PasteFromClipboard;
 4: FRichEdit.ClearSelection;
 5: FRichEdit.SelectAll;
 end;
end;


end.
