unit Datespin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls,SpinButtons,Harmonizer;

type
  TDateSpin = class(TPanel)
private
  FSpin1,FSpin2,FSpin3: TUpDownButton;
  FDate: TDateTime;
  FDateOrder: String[3];
  FOnChanged: TNotifyEvent;
  procedure MakeSpinButton(var B: TUpDownButton);
  procedure IncrementDate(I:Integer; P: AnsiChar);
  procedure ShowDate;
  function GetDateOrder(const DateFormat: string): String;
  function GetMonthDays(Y,M: word): word;
  procedure SetYear(New: Word);
  function GetYear: Word;
  procedure SetMonth(New: Word);
  function GetMonth: Word;
  procedure SetDay(New: Word);
  function GetDay: Word;
  procedure SetDate(New: TDateTime);
  procedure FixDate(var M,D,Y: word);
protected
  procedure Changed; dynamic;
  procedure UpClick (Sender: TObject); virtual;
  procedure DownClick (Sender: TObject); virtual;
public
  constructor Create(AOwner: TComponent); override;
  property Date: TDateTime read FDate write SetDate;
published
  property Year: word read GetYear write SetYear;
  property Month: word read GetMonth write SetMonth;
  property Day: word read GetDay write SetDay;
  property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  end;

procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Fountain', [TDateSpin]);
end;


procedure TDateSpin.ShowDate;
begin
Caption:=' '+DateToStr(FDate);
end;


procedure TDateSpin.MakeSpinButton(var B: TUpDownButton);
begin
B:=TUpDownButton.Create(Self);
B.Visible := True;
B.Parent := Self;
B.Width := 15;
B.Align:=alRight;
B.OnUpClick:=UpClick;
B.OnDownClick:=DownClick;
FDate:=Now;
ShowDate;
end;


constructor TDateSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
Alignment:=taLeftJustify;
BevelOuter:=bvLowered;
Width := 131;
 Height := 26;
MakeSpinButton(FSpin1);
MakeSpinButton(FSpin2);
MakeSpinButton(FSpin3);
FDateOrder:=GetDateOrder(IndSystemData.ShortDateFormat);
end;


procedure TDateSpin.Changed;
begin
if Assigned(FOnChanged) then FOnChanged(Self);
end;

procedure TDateSpin.SetDate(New: TDateTime);
begin
FDate:=New;
ShowDate;
Changed;
end;



function TDateSpin.GetDateOrder(const DateFormat: string): String;
{Return string that specifies the date format}
var I: Integer;
begin
for I:=1 to Length(DateFormat) do
	case Chr(Ord(DateFormat[I]) and $DF) of
	 'Y': begin Result := 'YMD'; exit; end;
	 'M': begin Result := 'MDY'; exit; end;
	 'D': begin Result := 'DMY'; exit; end;
	 end;
Result := 'MDY';
end;


function TDateSpin.GetMonthDays(Y,M: word): word;
{ Get days in a month }
const DaysPerMonth: array[1..12] of Integer =
    (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
Result:=DaysPerMonth[M];
if (M=2) and (Y mod 4 = 0) and
   ((Y mod 100 <> 0) or (Y mod 400 = 0)) then Result:=Result+1;
end;


procedure TDateSpin.FixDate(var M,D,Y: word);
{Repair defective date}
var C: integer;
begin
if Y<=0 then Y:=1;
if M<=0 then M:=1;
if M>12 then M:=12;
C:=GetMonthDays(Y,M);
if D>C then D:=C;
if D<=0 then D:=1;
end;


procedure TDateSpin.SetYear(New: Word);
var M,D,Y: word;
begin
DecodeDate(FDate,Y,M,D);
Y:=New;
FixDate(M,D,Y);
SetDate(EncodeDate(Y,M,D));
end;


function TDateSpin.GetYear: Word;
var M,D: word;
begin
DecodeDate(FDate,Result,M,D);
end;


procedure TDateSpin.SetMonth(New: Word);
var M,D,Y: word;
begin
DecodeDate(FDate,Y,M,D);
M:=New;
FixDate(M,D,Y);
SetDate(EncodeDate(Y,M,D));
end;


function TDateSpin.GetMonth: Word;
var D,Y: word;
begin
DecodeDate(FDate,Y,Result,D);
end;


procedure TDateSpin.SetDay(New: Word);
var M,D,Y: word;
begin
DecodeDate(FDate,Y,M,D);
D:=New;
FixDate(M,D,Y);
SetDate(EncodeDate(Y,M,D));
end;


function TDateSpin.GetDay: Word;
var Y,M: word;
begin
DecodeDate(FDate,Y,M,Result);
end;


procedure TDateSpin.IncrementDate(I:Integer; P: AnsiChar);
var M,D,Y,C: word;

	procedure IncYear;
	begin
	Y:=Y+I; if Y<1 then Y:=1;
	C:=GetMonthDays(Y,M);
	if D>C then D:=C;
	end;

	procedure IncMonth;
	begin
	M:=M+I;
	if M<1 then begin M:=12; IncYear; end;
	if M>12 then begin M:=1; Y:=Y+1; end;
	C:=GetMonthDays(Y,M);
	if D>C then D:=C;
	end;

	procedure IncDay;
	begin
	D:=D+I;
	C:=GetMonthDays(Y,M);
	if D>C then begin D:=1; IncMonth; end;
	if D<1 then begin D:=31; IncMonth end;
	end;

begin
DecodeDate(FDate,Y,M,D);
case P of
 'Y': IncYear;
 'M': IncMonth;
 'D': IncDay;
 end;
FDate:=EncodeDate(Y,M,D);
end;

procedure TDateSpin.UpClick(Sender: TObject);
begin
if Sender=FSpin1 then IncrementDate(1,FDateOrder[1]) else
if Sender=FSpin2 then IncrementDate(1,FDateOrder[2]) else
if Sender=FSpin3 then IncrementDate(1,FDateOrder[3]);
ShowDate;
Changed;
end;

procedure TDateSpin.DownClick(Sender: TObject);
begin
if Sender=FSpin1 then IncrementDate(-1,FDateOrder[1]) else
if Sender=FSpin2 then IncrementDate(-1,FDateOrder[2]) else
if Sender=FSpin3 then IncrementDate(-1,FDateOrder[3]);
ShowDate;
Changed;
end;


end.
