unit DottedBox;

interface

uses Windows,Classes,Controls,Types,SysUtils,Graphics,ExtCtrls,DSupport,IntVector,
     GraphicsLib, GLPolygons;



type TDottedBox = class(TComponent)
  private
  BMP: TBitmap;
  FImage: TImage;
  Timer: TTimer;
  targetHandle:thandle;
  targetCanvas:TCanvas;
  CounterStart: integer;
  FAnimated: boolean;
  FDotWidth: integer;
  Looper: integer;
  DrawWhite: boolean;
  Visible: boolean;
  procedure SetImage(const Value: TImage);
  procedure SetAnimated(const Value: boolean);
  procedure SetDotWidth(const Value: integer);
  procedure RemoveCursorRestrictions;
  procedure RestrictCursorToDrawingArea(const Image: TImage);
  procedure HandleTimer(Sender: TObject);
  function AdjustRectangle(R: TRect): TRect;
  procedure AdjustPolygon(var Points: TPolygonArray);
  procedure DrawPolygon(var Poly: TPolygonArray);
 protected
  procedure Notification(AComponent: TComponent; Operation: TOperation); override;
 public
  Box: TRect;
  Polygon: TPolygonArray;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure ShowRectangle;
  procedure HideRectangle;
    procedure ShowPolygon;
    procedure HidePolygon;
  procedure HandleMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  procedure HandleMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  procedure HandleMouseUp;
 published
  property Image: TImage read FImage write SetImage;
  property Animated: boolean read FAnimated write SetAnimated;
  property DotWidth: integer read FDotWidth write SetDotWidth default 5;
 end;


procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TDottedBox]);
end;


type TDDAData = packed record
 DotWidth: integer;
 DrawWhite: Pboolean;
 Looper: PInteger;
 Canvas: TCanvas;
 end;

type PDDAData = ^TDDAData;

procedure MovingDots(X,Y: Integer; DDA: PDDAData); stdcall;
begin
Inc(DDA^.Looper^);
DDA^.DrawWhite^:=((DDA^.Looper^ div DDA^.DotWidth) and 1)=1;
if DDA^.DrawWhite^ then DDA^.Canvas.Pixels[X,Y]:=clWhite
else DDA^.Canvas.Pixels[X,Y]:=clBlack;
end;




procedure TDottedBox.RestrictCursorToDrawingArea(const Image: TImage);
var CursorClipArea: TRect;
begin
CursorClipArea := Bounds(Image.ClientOrigin.X, Image.ClientOrigin.Y, Image.Width, Image.Height);
Windows.ClipCursor(@CursorClipArea)
end;


procedure TDottedBox.RemoveCursorRestrictions;
begin
Windows.ClipCursor(nil)
end;



function TDottedBox.AdjustRectangle(R: TRect): TRect;
var Off: TPoint;
begin
R:=NormalizeRect(R);
Off:=Point(Image.Left,Image.Top);
Result.TopLeft:=IntVectorAdd(R.TopLeft,Off);
Result.BottomRight:=IntVectorAdd(R.BottomRight,Off);
end;



procedure TDottedBox.AdjustPolygon(var Points: TPolygonArray);
var Off: TPoint;
var I,J: integer;
begin
Off:=Point(Image.Left,Image.Top);
for I:=0 to High(Points) do
 for J:=0 to High(Points[I]) do
  Points[I,J]:=IntVectorAdd(Points[I,J],Off);
end;




procedure TDottedBox.DrawPolygon(var Poly: TPolygonArray);
var DDA: TDDAData;
var P1,P2: TPoint;
var I,J: integer;
begin
if FImage=nil then raise Exception.Create('No Image Selected');
DDA.DotWidth:=FDotWidth;
DDA.Canvas:=TargetCanvas;
DDA.Looper:=@Looper;
DDA.DrawWhite:=@DrawWhite;
AdjustPolygon(Poly);
Looper:=CounterStart;
for I:=0 to High(Poly) do
 for J:=0 to High(Poly[I]) do
	begin
	P1:=Poly[I,J];
	if J=High(Poly[i]) then P2:=Poly[I,0] else P2:=Poly[I,J+1];
	LineDDA(P1.X,P1.Y,P2.X,P2.Y,@MovingDots,integer(@DDA));
	end;
end;



procedure TDottedBox.ShowRectangle;
var DDA: TDDAData;
var R: TRect;
var Poly: TPolygonArray;
begin
SetLength(Poly,1);
SetLength(Poly[0],4);
Poly[0,0]:=Box.TopLeft;
Poly[0,1]:=Point(Box.Right,Box.Top);
Poly[0,2]:=Box.BottomRight;
Poly[0,3]:=Point(Box.Left,Box.Bottom);
DrawPolygon(Poly);
Visible:=True;
end;



procedure TDottedBox.HideRectangle;
begin
FImage.Parent.Repaint;
Visible:=False;
end;




procedure TDottedBox.ShowPolygon;
begin
DrawPolygon(Polygon);
Visible:=True;
end;


procedure TDottedBox.HidePolygon;
begin
FImage.Parent.Repaint;
Visible:=False;
end;




procedure TDottedBox.HandleTimer(Sender: TObject);
begin
if not Visible then exit;
Inc(CounterStart);
ShowRectangle;
end;



constructor TDottedBox.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
SetLength(Polygon,0);
Box:=Rect(0,0,0,0);
targetCanvas:=TControlCanvas.Create;

Timer:=TTimer.Create(nil);
Timer.Interval:=100;
Timer.Enabled:=False;
Timer.OnTimer:=HandleTimer;
Looper:=CounterStart;
DotWidth:=5;
Visible:=False;
end;



destructor TDottedBox.Destroy;
begin
Timer.Free;
inherited;
end;


procedure TDottedBox.SetImage(const Value: TImage);
begin
if FImage<>Value then
	begin
	FImage:=Value;
	if csDesigning in ComponentState then exit;
	TControlCanvas(targetCanvas).Control:=FImage.Parent;
	targethandle:=FImage.Parent.handle;
	end;
end;



procedure TDottedBox.HandleMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
if FImage=nil then raise Exception.Create('No Image Selected');
HideRectangle;
Box.TopLeft:=Point(X,Y);
Box.BottomRight:=Point(X,Y);
//RestrictCursorToDrawingArea(Image);
end;


procedure TDottedBox.HandleMouseMove(Sender: TObject; Shift: TShiftState; X,Y: Integer);
begin
if FImage=nil then raise Exception.Create('No Image Selected');
if ssLeft in Shift then
	begin
	HideRectangle;
	Box.Right:=X; Box.Bottom:=Y;
	ShowRectangle;
	end;
end;


procedure TDottedBox.HandleMouseUp;
begin
if FImage=nil then raise Exception.Create('No Image Selected');
//RemoveCursorRestrictions
end;


procedure TDottedBox.SetAnimated(const Value: boolean);
begin
if FAnimated<>Value then
	begin
	FAnimated := Value;
	Timer.Enabled:=Value;
	end;
end;




procedure TDottedBox.SetDotWidth(const Value: integer);
begin
if FDotWidth<>Value then
	begin
	if Value<1 then FDotWidth:=1
	else FDotWidth := Value;
	end;
end;




procedure TDottedBox.Notification(AComponent: TComponent;
  Operation: TOperation);
{Handle the live deletion of an image controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) and (AComponent = FImage) then FImage := nil;
end;



end.
 