unit DropController;

interface

uses Windows, Messages, SysUtils, Classes, Controls, ShellAPI;


type TDropController = class(TComponent)
  private
    FOnDrop: TNotifyEvent;
    FDropTarget: TWinControl;
    procedure SetDropTarget(const Value: TWinControl);
  protected
     Drop : THandle;
     OldLBWindowProc: TWndMethod;
     procedure WMDropFiles(var Msg: TMessage); message wm_DropFiles;
     procedure LBWindowProc(var Message: TMessage);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
  published
    FileList: TStringList;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property OnDrop: TNotifyEvent read FOnDrop write FOnDrop;
    property DropTarget: TWinControl read FDropTarget write SetDropTarget;
  end;

procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TDropController]);
end;


{ TDropController }

constructor TDropController.Create(AOwner: TComponent);
begin
inherited;
FileList:=TStringList.Create;
end;



destructor TDropController.Destroy;
begin
if not (csDesigning in ComponentState) and assigned(FDropTarget) then
	begin
	FDropTarget.WindowProc:=OldLBWindowProc;
	DragFinish(Drop);
	end;
FileList.Free;
inherited;
end;




procedure TDropController.WMDropFiles(var Msg: TMessage);
var i, NumFiles, NameLength: integer;
var Fname: String;
var TempFile: array[0..255] of Char;
begin
Drop:= Msg.wParam;
FileList.Clear;
NumFiles:=DragQueryFile(Msg.wParam, $FFFFFFFF, Nil, 0);
for I:= 0 to NumFiles-1 do
	begin
	NameLength:= DragQueryFile(Msg.wParam, I, Nil , 0);
	DragQueryFile(Msg.wParam, I, TempFile, NameLength+1);
	FName:=StrPas(TempFile);
	if DirectoryExists(FName) then
	 if FName[Length(FName)]<>'\' then FName:=FName+'\';
	FileList.Add(FName);
	end;
if Assigned(FOnDrop) then FOnDrop(Self);
end;



procedure TDropController.LBWindowProc(var Message: TMessage);
begin
if Message.Msg = WM_DROPFILES then WMDropFiles(Message);
OldLBWindowProc(Message);
end;


procedure TDropController.Notification(AComponent: TComponent;
  Operation: TOperation);
{Handle the live deletion of the drop target controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) and (AComponent = FDropTarget) then FDropTarget := nil;
end;



procedure TDropController.SetDropTarget(const Value: TWinControl);
begin
FDropTarget := Value;
if not (csDesigning in ComponentState) and assigned(FDropTarget) then
	begin
	FDropTarget.FreeNotification(Self);
	OldLBWindowProc := FDropTarget.WindowProc;
	FDropTarget.WindowProc := LBWindowProc;
	DragAcceptFiles(FDropTarget.Handle, True);
	end;
end;



end.
