unit EditSpin;

interface

uses Windows,Messages,Forms,Types,Classes,Controls,StdCtrls,ExtCtrls,Graphics,SysUtils,SpinButtons;


{================== TEditSpin -  Edit Spin  ==========================}

type TValidateEvent = function(Sender: TObject; C: Char): boolean of object;
type
  TEditSpin = class(TCustomSpin)
  private
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    FOnValidateChar: TValidateEvent;
    procedure SetValue (NewValue: string);
    procedure SetNoEventValue(NewValue: string);
    function CheckValue (NewValue: string): string;
    function GetValue: string;
    procedure CMExit(var Message: TCMExit);   message CM_EXIT;
    procedure UpClick (Sender: TObject); virtual;
  protected
    function IsValidChar(Key: Char): Boolean; virtual;
    procedure DownClick (Sender: TObject); virtual;
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    constructor Create(AOwner: TComponent); override;
    property NoEventValue: string read GetValue write SetNoEventValue;
  published
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property OnValidateChar: TValidateEvent read FOnValidateChar write FOnValidateChar;
    property Value: string read GetValue write SetValue;

    property AutoSelect;
    property AutoSize;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;


implementation

procedure Register;
begin
RegisterComponents('Fountain', [TEditSpin]);
end;

{================== TIntegerSpin ================================================}


constructor TEditSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FButton.OnUpClick := UpClick;
FButton.OnDownClick := DownClick;
end;


procedure TEditSpin.UpClick (Sender: TObject);
begin
if Assigned(FOnUpClick) then FOnUpClick(Sender);
end;


procedure TEditSpin.DownClick (Sender: TObject);
begin
if Assigned(FOnDownClick) then FOnDownClick(Sender);
end;



procedure TEditSpin.CMExit(var Message: TCMExit);
begin
inherited;
SetValue(Value);
end;


function TEditSpin.GetValue: string;
begin
Result:=Text;
Result:=CheckValue(Result);
end;



procedure TEditSpin.SetValue(NewValue: string);
var OldReadOnly: boolean;
begin
OldReadOnly:=ReadOnly;
ReadOnly:=False;
Text:=NewValue;
ReadOnly:=OldReadOnly;
end;



procedure TEditSpin.SetNoEventValue(NewValue: string);
var OldBlock: boolean;
begin
OldBlock:=BlockChangeEvent;
try
BlockChangeEvent:=True;
SetValue(NewValue);
finally BlockChangeEvent:=OldBlock; end;
end;



function TEditSpin.IsValidChar(Key: Char): Boolean;
begin
Result:=True;
if Assigned(FOnValidateChar) then Result:=FOnValidateChar(Self,Key);
end;



procedure TEditSpin.KeyPress(var Key: Char);
begin
if not IsValidChar(Key) then
	begin
	Key := #0;
	MessageBeep(0)
	end;
if Key <> #0 then inherited KeyPress(Key);
end;



function TEditSpin.CheckValue(NewValue: string): string;
begin
Result := NewValue;

end;


procedure TEditSpin.KeyDown(var Key: Word; Shift: TShiftState);
begin
if Key = VK_UP then UpClick(Self)
else if Key = VK_DOWN then DownClick(Self);
inherited KeyDown(Key, Shift);
end;



end.

