unit FancyGrids;

interface

uses Windows,Classes,Controls,Types,SysUtils, Graphics, Grids, DSupport;

type TColumnInfo = record
 Column: integer;
 end;

type TColumnClickEvent = procedure(Sender: TObject; ColInfo: TColumnInfo) of object;

type TGetCheckEvent = function(ACol, ARow: Integer): boolean of object;
type TToggleCheckEvent= procedure(ACol, ARow: Integer) of object;
type TSetCheckEvent = procedure(ACol, ARow: Integer; Checked,Update: boolean) of object;
type TGetItemEvent = function(ACol, ARow: Integer): string of object;


type TDrawGridExt = class(TDrawGrid)
 private
  MouseCell: TPoint;
  FOnColumnClick: TColumnClickEvent;
 protected
  ButtonDown: boolean;
  procedure Paint; override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure ColClick(Info: TColumnInfo);  dynamic;
 public
  constructor Create(AOwner: TComponent); override;
  property InplaceEditor;
 published
  property OnColumnClick: TColumnClickEvent read FOnColumnClick write FOnColumnClick;
 end;



type TCheckGrid = class(TDrawGridExt)
 private
  FCheck, FNoCheck: TBitmap;
  FCheckPattern: int64;
  FOnGetCheck: TGetCheckEvent;
  FOnSetCheck: TSetCheckEvent;
  FCheckColor: TColor;
  FUpdateCount: integer;
  FOnGetItem: TGetItemEvent;
  procedure GetCheckPattern(const Value: int64);
  procedure ToggleCheckBox(ACol,ARow: integer);
  function IsCheckedColumn(Col: integer): boolean;
  procedure SetCheckColor(const Value: TColor);
 protected
  procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure KeyPress(var Key: Char); override;
  procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  function SelectCell(ACol, ARow: Longint): Boolean; override;
  function CanEditShow: Boolean; override;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function GetCheck(ACol, ARow: Integer): boolean;
  procedure SetCheck(ACol, ARow: integer; Checked: boolean);
  procedure BeginUpdate;
  procedure EndUpdate;
  function GetItem(ACol,ARow: integer): string;
 published
  property CheckPattern: int64 read FCheckPattern write GetCheckPattern;
  property CheckColor: TColor read FCheckColor write SetCheckColor default clBtnFace;
  property OnGetCheck: TGetCheckEvent read FOnGetCheck write FOnGetCheck;
  property OnSetCheck: TSetCheckEvent read FOnSetCheck write FOnSetCheck;
  property OnGetItem: TGetItemEvent read FOnGetItem write FOnGetItem;
 end;

procedure Register;


implementation


procedure Register;
begin
RegisterComponents('Fountain', [TCheckGrid]);
end;


{ TDrawGridExt }


constructor TDrawGridExt.Create(AOwner: TComponent);
begin
inherited ;
ButtonDown:=False;
end;



procedure TDrawGridExt.Paint;
var R: TRect;
var Offset: integer;
begin
inherited;
if not ButtonDown then exit;
Offset:=LeftCol-1;
R:=CellRect(MouseCell.X,MouseCell.Y);
Canvas.Pen.Color:=RGB(160,160,160);
Canvas.Polyline([Point(R.Left,R.Bottom-1), Point(R.Left,R.Top), Point(R.Right,R.Top)]);
Canvas.Pen.Color:=FixedColor;
Canvas.Polyline([Point(R.Left+1,R.Bottom-1), Point(R.Right-1,R.Bottom-1), Point(R.Right-1,R.Top)]);

Canvas.Pen.Color:=RGB(160,160,160);
Canvas.Polyline([Point(R.Left,R.Bottom), Point(R.Right,R.Bottom), Point(R.Right,R.Top-1)]);
end;


procedure TDrawGridExt.ColClick(Info: TColumnInfo);
begin
  if Assigned(FOnColumnClick) then FOnColumnClick(Self, Info);
end;


procedure TDrawGridExt.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
inherited;
MouseToCell(x, y, MouseCell.X, MouseCell.Y);
if MouseCell.Y <=FixedRows-1 then
	begin
	ButtonDown:=True;
	Invalidate;
	end;
end;

procedure TDrawGridExt.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var ColInfo: TColumnInfo;
begin
inherited;
if ButtonDown then
	begin
	ButtonDown:=False;
	Invalidate;
	ColInfo.Column:=MouseCell.X;
	ColClick(ColInfo);
	end;
end;


{ TCheckGrid }


constructor TCheckGrid.Create(AOwner: TComponent);
var BMP: TBitmap;
begin
inherited;
FCheck:= TBitmap.Create;
FNoCheck:= TBitmap.Create;
BMP:= TBitmap.create;
try
{Get 4x3 array of bitmaps for CheckBoxes and RadioButtons}
BMP.Handle:=LoadBitmap( 0, PChar(OBM_CHECKBOXES));
{the first subimage is the unchecked box}
FNoCheck.Width := bmp.width div 4;
FNoCheck.Height := bmp.height div 3;
FNoCheck.Canvas.CopyRect( FNoCheck.Canvas.ClipRect, BMP.Canvas, FNoCheck.Canvas.ClipRect );
{the second subimage is the checked box}
FCheck.Width := bmp.Width div 4;
FCheck.Height := bmp.Height div 3;
FCheck.Canvas.CopyRect(FCheck.Canvas.ClipRect, BMP.Canvas, rect( FCheck.Width, 0, 2*FCheck.Width, FCheck.Height ));
finally bmp.free end;
FCheckColor:=clBtnFace;
FCheckPattern:=5;
end;



destructor TCheckGrid.Destroy;
begin
FNoCheck.Free;
FCheck.Free;
inherited;
end;


function TCheckGrid.IsCheckedColumn(Col: integer): boolean;
begin
if Col<FixedCols then Result:=False
else Result:=(CheckPattern and (1 shl (Col-FixedCols)))>0;
end;



procedure TCheckGrid.DrawCell(ACol, ARow: Integer; ARect: TRect;
  AState: TGridDrawState);
begin
inherited;
if (aRow >= fixedRows) and IsCheckedColumn(ACol) then
	begin
	Canvas.Brush.Color := CheckColor;
	Canvas.FillRect(ARect);
	if GetCheck(ACol,ARow) then  Canvas.Draw( (ARect.right + ARect.left - FCheck.width) div 2, (ARect.bottom + ARect.top - FCheck.height) div 2, FCheck )
	else Canvas.Draw( (ARect.right + ARect.left - FNoCheck.width) div 2, (ARect.bottom + ARect.top - FNoCheck.height) div 2, FNoCheck);
	EditorMode:=False;
	end;
end;



procedure TCheckGrid.GetCheckPattern(const Value: int64);
begin
if FCheckPattern<>Value then
	begin
	FCheckPattern := Value;
	Invalidate;
	end;
end;



procedure TCheckGrid.SetCheck(ACol,ARow: integer; Checked: boolean);
begin
if Assigned(OnSetCheck) then OnSetCheck(ACol,ARow,Checked,FUpdateCount=0);
if FUpdateCount=0 then Invalidate;
end;



function TCheckGrid.GetCheck(ACol, ARow: Integer): boolean;
begin
Result:=False;
if Assigned(OnGetCheck) then Result:=OnGetCheck(ACol,ARow);
end;



procedure TCheckGrid.ToggleCheckBox(ACol,ARow: integer);
begin
SetCheck(ACol,ARow,not GetCheck(ACol,ARow));
if FUpdateCount=0 then invalidate;
end;



procedure TCheckGrid.SetCheckColor(const Value: TColor);
begin
if FCheckColor<>Value then
	begin
	FCheckColor := Value;
	Invalidate;
	end;
end;


procedure TCheckGrid.BeginUpdate;
begin
Inc(FUpdateCount);
end;


procedure TCheckGrid.EndUpdate;
begin
if FUpdateCount>0 then Dec(FUpdateCount);
if FUpdateCount=0 then Invalidate;
end;



function TCheckGrid.GetItem(ACol, ARow: integer): string;
begin
Result:='';
if (ACol<0) or (ACol>=ColCount) or (ARow<0) or (ARow>=RowCount) then exit;
if Assigned(FOnGetItem) then Result:=FOnGetItem(ACol,ARow);
end;


{-----------------------------------------------------------------------------}


function TCheckGrid.SelectCell(ACol, ARow: Integer): Boolean;
begin
Result:=False;
{Block }
//if IsCheckedColumn(ACol) then exit;
Result:=inherited SelectCell(ACol, ARow);
end;




procedure TCheckGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var aCol, aRow: Integer;
var OldOptions: TGridOptions;
begin
{We need to turn off goEditing for mouse clicks on checkbox Cells}
{Otherwise editor will be enabled and will block updating a cell}
MouseToCell(X, Y, aCol, aRow );
if IsCheckedColumn(aCol) then
	begin
	OldOptions:=Options;
	Options:=Options - [goEditing];
	inherited;
	Options:=OldOptions;
	end
else inherited;
end;


procedure TCheckGrid.KeyPress(var Key: Char);
begin
{If not IsCheckedColumn(Col) then } inherited;
end;


procedure TCheckGrid.KeyDown(var Key: Word; Shift: TShiftState);
begin
{If not IsCheckedColumn(Col) then } inherited;
end;


function TCheckGrid.CanEditShow: Boolean;
begin
Result:=inherited CanEditShow;
if Result then Result:=not IsCheckedColumn(Col);
end;


procedure TCheckGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var aCol, aRow: Integer;
begin
inherited;
MouseToCell(X, Y, aCol, aRow );
If IsCheckedColumn(aCol) and (aRow >= fixedRows) then
	begin
	{ click landed in a checkbox cell}
	ToggleCheckBox(aCol, aRow );
	end;
end;

end.
