unit FlexGrid;


interface

uses Messages, Windows, SysUtils, Classes, Grids, Variants, Graphics,
  Menus, Controls, Forms, StdCtrls, ExtCtrls, Mask,DSupport,ScrollBarWindow,
  SpinButtons, IntVector, HugeArry, ScrollEditor;


type TCellSize = record
 Size,SplitPos,Offset: integer;
 TabStop: boolean;
 end;

type TCellSizeArray = array of TCellSize;

type TCellPos = (cpInside,cpOutLeft,cpOutRight,cpOutTop,cpOutBottom,cpHalfOutRight,cpHalfOutBottom);
type TCellStates = set of TCellPos;


type TGridInfo = record
 Sender: TObject;
 ClientRect: TRect;
 CurrentPos: TPoint;
 end;


type TMoveSelectedEvent = function (Info: TGridInfo; var X,Y: integer): boolean of object;
type TAdjustSizeEvent = procedure (Sender: TObject; Index, NewSize: integer) of object;

type TMouseDownState = (mdNone,mdColSize,mdRowSize,mdColMove,mdRowMove,mdCellSelect,mdRowSelect,mdOffBottom);
type THorizScrollType = (stNone,stLeft,stRight);
type TVertScrollType = (vtNone,vtUp,vtDown);


type TSelection = record
 StartPos, EndPos: Integer;
 end;


type TRowArray = class(TObject)
 private
  FArray: array of boolean;
  FClearNewItems: boolean;
  function GetCount: integer;
  procedure SetCount(const Value: integer);
  procedure SetRange(First,Last: integer; Value: boolean);
  function Get(Index: integer): boolean;
  procedure Put(Index: integer; const Value: boolean);
  function GetSelectCount: integer;
 protected
 public
  constructor Create;
  procedure Clear;
  property Count: integer read GetCount write SetCount;
  property Value[Index: integer]: boolean read Get write Put; default;
  property SelectCount: integer read GetSelectCount;
 end;

type TColMoveEvent = procedure(Sender: TObject; Col: integer; var Accept: Boolean) of object;


type TFlexGrid = class;

TInplaceEdit = class(TEditEx)
 private
  FGrid: TFlexGrid;
  procedure Deselect;
 protected
  procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  procedure KeyPress(var Key: Char); override;
 public
   constructor Create(AOwner: TComponent); override;
   procedure Hide;
   property  Grid: TFlexGrid read FGrid;
 end;


 TFlexGrid = class(TScrollBarWindow)
 private
  FCanEditModify: boolean;
  FOnColumnResize: TAdjustSizeEvent;
  FOnRowResize: TAdjustSizeEvent;
  FOnRowMoved: TMovedEvent;
  FRowCount: integer;
  FColCount: integer;
  FFixedRows: Integer;
  FFixedCols: Integer;
  FFixedColor: TColor;
  FColSizes: TCellSizeArray;
  FRowSIzes: TCellSizeArray;
  FDefaultColWidth: Integer;
  FDefaultRowHeight: Integer;
  FScrollOffset: TPoint;
  FSelected: TPoint;
  FOptions: TGridOptions;
  SizerCellCol: integer;
  SizerCellRow: integer;	{Row sizing not currently implemented}
  SizerLineStart: integer;
  SizerStartX: integer;
  FEditorMode: boolean;
  FOnDrawCell: TDrawCellEvent;
  EditorCol,EditorRow: integer;
  LastFullCol,LastFullRow: integer;
  FCursor: TCursor;
  FBackColor: TColor;
  FDefaultDrawing: Boolean;
  FCellBorder: TColor;
  FOnPositionChanged: TNotifyEvent;
    FOnColMoveStart: TColMoveEvent;
    FOnColMoveOver: TColMoveEvent;
    FOnColMoveDrop: TColMoveEvent;
    FFixedColSizing: boolean;
  procedure SetBorderStyle(const Value: TBorderStyle);
  procedure SetRowCount(Value: integer);
  procedure SetColCount(Value: integer);
  procedure SetFixedCols(const Value: Integer);
  procedure SetFixedRows(const Value: Integer);
  procedure SetFixedColor(const Value: TColor);
  procedure Draw3DRect(Canvas: TCanvas; R: TRect; Horiz,Vert: boolean);
  procedure SetDefaultColWidth(const Value: Integer);
  procedure SetDefaultRowHeight(const Value: Integer);
  function GetColWidths(Index: Integer): integer;
  function GetRowHeights(Index: Integer): integer;
  procedure SetColWidths(Index: Integer; Value: integer);
  procedure SetRowHeights(Index: Integer; Value: integer);
  function GetColSplits(Index: Integer): Integer;
  function GetRowSplits(Index: Integer): Integer;
  procedure SetColSplits(Index: Integer; const Value: Integer);
  procedure SetRowSplits(Index: Integer; const Value: Integer);
  function ColSizerHitTest(X,Y: integer; var Col: integer): boolean;
  function MoveSpits(Inx, NewPos: integer): integer;
  procedure DrawSizingLine(Start,PenWidth: integer; Vertical: boolean);
  procedure SetLeftCol(Value: Integer);
  procedure SetTopRow(Value: Integer);
  procedure SetScrollBarParams;
  procedure SetHScrollBarRange;
  procedure SetVScrollBarRange;
  function GetCellRect(Col, Row: integer): TRect; overload;
  procedure GetCellRect(Col, Row: integer; var TopLeft,BottomRight: TPoint); overload;
  function GetPixelScrollOffset(Col,Row: integer): TPoint;
  function GetCellTopLeft(Col, Row: integer): TPoint;
  function GetCellBottomRight(Col, Row: integer): TPoint;
  procedure DrawAllCells;
  function IsFixed(Col,Row: integer): boolean;
  procedure SetCol(const Value: integer);
  procedure SetRow(const Value: integer);
  function ColumnHitTest(X: integer): integer;
  function RowHitTest(Col,Y: integer): integer;
  procedure SetSplitsFromSizes(var A: TCellSizeArray);
  procedure SetSizesFromSplits(var A: TCellSizeArray);
  procedure SetCellSizeDefaults(var A: TCellSizeArray; Default: integer);
  procedure SetCellSizeValues(var A: TCellSizeArray; Count, Default: integer);
  procedure PlaceEditor(R: TRect; ACol,ARow: integer);
  procedure EditKeyDownHandler(Sender: TObject; var Key: Word; Shift: TShiftState);
  procedure EditKeyPressHandler(Sender: TObject; var Key: Char);
  function GetColOffset(Index: Integer): Integer;
  procedure SetColOffset(Index: Integer; const Value: Integer);
  function GetCellState(X,Y: integer; var R: TRect): TCellStates;
  procedure SetOptions(Value: TGridOptions);
  procedure InvalidateGrid;
  procedure UpdateText;
  procedure EditChangeHandler(Sender: TObject);
  procedure SetEditorMode(Value: Boolean);
  procedure WMGetDlgCode(var Msg: TWMGetDlgCode); message WM_GETDLGCODE;
  procedure OptimizeScrolling(Cell: TPoint); overload;
  procedure OptimizeScrolling; overload;
  function GetTabStops(Index: Integer): Boolean;
  procedure SetTabStops(Index: Integer; const Value: Boolean);
  function CanEditShow: Boolean;
  function IsActiveControl: Boolean;
  procedure ShowEditorChar(Ch: Char);
  procedure WMChar(var Msg: TWMChar);  message WM_CHAR;
  procedure WMNCHitTest(var Msg: TWMNCHitTest); message WM_NCHITTEST;
  procedure DrawDottedLine(Line: TLine; PenWidth: integer);
  procedure HandleMoveTimerEvent(Sender: TObject);
  function GetClipOffRect(P: TPoint): TRect;
  function GetHorzScrollType(X, Y: integer): THorizScrollType;
  function GetVertScrollType(X, Y: integer): TVertScrollType;
  procedure ReadColWidths(Reader: TReader);
  procedure ReadRowHeights(Reader: TReader);
  procedure WriteColWidths(Writer: TWriter);
  procedure WriteRowHeights(Writer: TWriter);
  procedure UpdateDesigner;
  function GetPixelOffset: TPoint;
  procedure SetBackColor(const Value: TColor);
  function OffBottom(Y: integer): boolean;
    function GetGridHeight: integer;
    function GetGridWidth: integer;
    procedure SelectRange(StartRow, EndRow: integer);
 protected
  FOnSelectCell: TSelectCellEvent;
  FOnColumnMoved: TMovedEvent;
  FOnMoveSelected: TMoveSelectedEvent;
  FOnGetEditText: TGetEditEvent;
  FOnSetEditText: TSetEditEvent;
  FBorderStyle: TBorderStyle;
  MoveTimer: TTimer;
  MouseDownState: TMouseDownState;
  StartMoveCell,EndMoveCell: TPoint;
  Editor: TInplaceEdit;
    function CanColMoveStart(Col: integer): boolean; dynamic;
    function CanColMoveOver(Col: integer): boolean; dynamic;
    function CanColMoveDrop(Col: integer): boolean; dynamic;
  procedure DoRowSelect(Row: integer; Value: boolean); dynamic;
  procedure ToggleRowSelect(Row: integer); dynamic;
  procedure HideEditor; dynamic;
  procedure ShowEditor; dynamic;
  procedure SetFieldText(ACol, ARow: integer; Value: string); dynamic;
  procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  procedure KeyPress(var Key: Char); override;
  procedure DefineProperties(Filer: TFiler); override;
  procedure CreateParams(var Params: TCreateParams); override;
  procedure CMShowingChanged(var Message: TMessage); message CM_SHOWINGCHANGED;
  procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
  procedure WMSetCursor(var Msg: TWMSetCursor); message WM_SETCURSOR;
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
  procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  procedure DoHorzChange(Sender: TObject); override;
  procedure DoVertChange(Sender: TObject); override;
  procedure Resize; override;
  procedure ColumnMoved(FromIndex, ToIndex: Longint); dynamic;
  procedure RowMoved(FromIndex, ToIndex: Longint); dynamic;
  function CanEditAcceptKey(Key: Char): Boolean; dynamic;
  function CanEditModify: boolean; dynamic;
  procedure DoExit; override;
  function GetOffsetRect(X, Y: integer): TRect; overload;
  function GetOffsetRect(P: TPoint): TRect; overload;
  procedure DrawCell(X,Y: integer); dynamic;
  procedure OwnerDraw(ACol, ARow: integer; Rect: TRect; DrawState: TGridDrawState); dynamic;
  function SelectCell(ACol, ARow: Integer): Boolean; dynamic;
  function TestMoveSelected(Info: TGridInfo; var X, Y: integer): boolean; dynamic;
  procedure ColumnResize(NewSize,Index: integer); dynamic;
  function GetEditText(ACol, ARow: Integer): string; dynamic;
  procedure SetEditText(ACol, ARow: Integer; const Value: string); dynamic;
  function DoMouseWheelDown(Shift: TShiftState; MousePos: TPoint): Boolean; override;
  function DoMouseWheelUp(Shift: TShiftState; MousePos: TPoint): Boolean; override;
 public
  LastEditText,CurrentEditText: string;
  RowSelect: TRowArray;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function CellHitTest(X, Y: integer; var Cell: TPoint): boolean;
  procedure Paint; override;
  property ColWidths[Index: Integer]: Integer read GetColWidths write SetColWidths;
  property ColOffset[Index: Integer]: Integer read GetColOffset write SetColOffset;
  property RowHeights[Index: Integer]: Integer read GetRowHeights write SetRowHeights;
  property ColSplits[Index: Integer]: Integer read GetColSplits write SetColSplits;
  property RowSplits[Index: Integer]: Integer read GetRowSplits write SetRowSplits;
  property TopRow: Integer read FScrollOffset.Y write SetTopRow;
  property LeftCol: Integer read FScrollOffset.X write SetLeftCol;
  property EditorMode: Boolean read FEditorMode write SetEditorMode;
  property TabStops[Index: Longint]: Boolean read GetTabStops write SetTabStops;
  property GridWidth: integer read GetGridWidth;
  property GridHeight: integer read GetGridHeight;
 published
  property OnMoveSelected: TMoveSelectedEvent read FOnMoveSelected write FOnMoveSelected;
  property OnDrawCell: TDrawCellEvent read FOnDrawCell write FOnDrawCell;
  property FixedCols: Integer read FFixedCols write SetFixedCols default 1;
  property FixedRows: Integer read FFixedRows write SetFixedRows default 1;
  property BorderStyle: TBorderStyle read FBorderStyle write SetBorderStyle default bsSingle;
  property RowCount: integer read FRowCount write SetRowCount default 5;
  property ColCount: integer read FColCount write SetColCount default 5;
  property FixedColor: TColor read FFixedColor write SetFixedColor default clBtnFace;
  property BackColor: TColor read FBackColor write SetBackColor default clBackground;
  property CellBorder: TColor read FCellBorder write FCellBorder default clSilver;
  property Color default clWindow;
  property DefaultColWidth: Integer read FDefaultColWidth write SetDefaultColWidth default 64;
  property DefaultRowHeight: Integer read FDefaultRowHeight write SetDefaultRowHeight default 24;
  property Row: integer read FSelected.Y write SetRow;
  property Col: integer read FSelected.X write SetCol;
  property Canvas;
  property Options: TGridOptions read FOptions write SetOptions default [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine, goRangeSelect];
  property OnGetEditText: TGetEditEvent read FOnGetEditText write FOnGetEditText;
  property OnSetEditText: TSetEditEvent read FOnSetEditText write FOnSetEditText;
  property OnSelectCell: TSelectCellEvent read FOnSelectCell write FOnSelectCell;
  property OnKeyPress;
  property OnPositionChanged: TNotifyEvent read FOnPositionChanged write FOnPositionChanged;
  property OnColumnMoved: TMovedEvent read FOnColumnMoved write FOnColumnMoved;
  property OnRowMoved: TMovedEvent read FOnRowMoved write FOnRowMoved;
  property OnColumnResize: TAdjustSizeEvent read FOnColumnResize write FOnColumnResize;
  property OnRowResize: TAdjustSizeEvent read FOnRowResize write FOnRowResize;
  property DefaultDrawing: Boolean read FDefaultDrawing write FDefaultDrawing default True;
  property OnColMoveStart: TColMoveEvent read FOnColMoveStart  write FOnColMoveStart;
  property OnColMoveOver: TColMoveEvent read FOnColMoveOver write FOnColMoveOver;
  property OnColMoveDrop: TColMoveEvent read FOnColMoveDrop write FOnColMoveDrop;
  property FixedColSizing: boolean read FFixedColSizing write FFixedColSizing;
 end;


function GetCellStatesStr(CS: TCellStates): string;


procedure Register;

implementation


uses Math, Consts, RTLConsts;

procedure Register;
begin
RegisterComponents('Fountain', [TFlexGrid]);
end;




{ TRowArray }


procedure TRowArray.SetRange(First,Last: integer; Value: boolean);
var I: integer;
begin
for I:=First to Last do
 FArray[I]:=Value;
end;


procedure TRowArray.Clear;
begin
SetRange(0,Count-1,False);
end;


function TRowArray.GetCount: integer;
begin
Result:=Length(FArray);
end;



procedure TRowArray.SetCount(const Value: integer);
var OldCount: integer;
var I: integer;
begin
OldCount:=Count;
if Value<>Count then
	begin
	SetLength(FArray,Value);
	if Count>OldCount then SetRange(OldCount,Count-1,False);
	end;
end;


constructor TRowArray.Create;
begin
end;


function TRowArray.Get(Index: integer): boolean;
begin
Result:=FArray[Index];
end;


procedure TRowArray.Put(Index: integer; const Value: boolean);
begin
FArray[Index]:=Value
end;



function TRowArray.GetSelectCount: integer;
var I: integer;
begin
Result:=0;
for I:=0 to Count-1 do
 if FArray[I] then Inc(Result);
end;



{ TInplaceEdit }


procedure TInplaceEdit.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
inherited;
if goTabs in Grid.Options then
    Message.Result := Message.Result or DLGC_WANTTAB;
end;


procedure TInplaceEdit.Deselect;
begin
SendMessage(Handle, EM_SETSEL, $7FFFFFFF, Longint($FFFFFFFF));
end;



procedure TInplaceEdit.KeyPress(var Key: Char);
var Selection: TSelection;
begin
Grid.KeyPress(Key);
if (Key in [#32..#255]) and not Grid.CanEditAcceptKey(Key) then
	begin
	Key := #0;
	MessageBeep(0);
	end;
case Key of
 #9, #27: Key := #0;
 #13:
	begin
	SendMessage(Handle, EM_GETSEL, Longint(@Selection.StartPos), Longint(@Selection.EndPos));
	if (Selection.StartPos = 0) and (Selection.EndPos = GetTextLen) then Deselect else SelectAll;
	Key := #0;
	end;
 ^H, ^V, ^X, #32..#255: if not Grid.CanEditModify then Key := #0;
 end;
if Key <> #0 then inherited KeyPress(Key);
end;



procedure TInplaceEdit.KeyDown(var Key: Word; Shift: TShiftState);

type
  TSelection = record
    StartPos, EndPos: Integer;
  end;


	procedure SendToParent;
	begin
	Grid.KeyDown(Key, Shift);
	Key := 0;
	end;


	procedure ParentEvent;
	var GridKeyDown: TKeyEvent;
	begin
	GridKeyDown := Grid.OnKeyDown;
	if Assigned(GridKeyDown) then GridKeyDown(Grid, Key, Shift);
	end;


	function ForwardMovement: Boolean;
	begin
	Result := goAlwaysShowEditor in Grid.Options;
	end;


	function Ctrl: Boolean;
	begin
	Result := ssCtrl in Shift;
	end;



	function Selection: TSelection;
	begin
	SendMessage(Handle, EM_GETSEL, Longint(@Result.StartPos), Longint(@Result.EndPos));
	end;



	function CaretPos: Integer;
	var P: TPoint;
	begin
	Windows.GetCaretPos(P);
	Result := SendMessage(Handle, EM_CHARFROMPOS, 0, MakeLong(P.X, P.Y));
	end;



	function RightSide: Boolean;
	begin
	with Selection do
	Result := (CaretPos = GetTextLen) and
		  ((StartPos = 0) or (EndPos = StartPos)) and (EndPos = GetTextLen);
	end;

	function LeftSide: Boolean;
	begin
	with Selection do
	Result := (CaretPos = 0) and (StartPos = 0) and
		  ((EndPos = 0) or (EndPos = GetTextLen));
	end;



begin
case Key of
 VK_UP, VK_DOWN, VK_PRIOR, VK_NEXT, VK_ESCAPE: SendToParent;
 VK_INSERT: if Shift = [] then SendToParent
	    else if (Shift = [ssShift]) and not Grid.CanEditModify then Key := 0;
 VK_LEFT: if ForwardMovement and (Ctrl or LeftSide) then SendToParent;
 VK_RIGHT: if ForwardMovement and (Ctrl or RightSide) then SendToParent;
 VK_HOME: if ForwardMovement and (Ctrl or LeftSide) then SendToParent;
 VK_END: if ForwardMovement and (Ctrl or RightSide) then SendToParent;
 VK_F2:
	begin
	ParentEvent;
	if Key = VK_F2 then
		begin
		Deselect;
		exit;
		end;
	end;
 VK_TAB: if not (ssAlt in Shift) then SendToParent;
 VK_DELETE: if Ctrl then SendToParent else if not Grid.CanEditModify then Key := 0;
 end;

if Key <> 0 then
	begin
	ParentEvent;
	inherited KeyDown(Key, Shift);
	end;
end;



constructor TInplaceEdit.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ParentCtl3D := False;
Ctl3D := False;
TabStop := False;
BorderStyle := bsNone;
DoubleBuffered := False;
end;



procedure TInplaceEdit.Hide;
begin
if HandleAllocated and IsWindowVisible(Handle) then
	begin
	Invalidate;
	SetWindowPos(Handle, 0, 0, 0, 0, 0, SWP_HIDEWINDOW or SWP_NOZORDER or SWP_NOREDRAW);
	if Focused then Windows.SetFocus(Grid.Handle);
	end;
end;




{ TFlexGrid}



function GetCellStatesStr(CS: TCellStates): string;
begin
if cpInside in CS then Result:=Result+','+'cpInside';
if cpOutLeft in CS then Result:=Result+','+'cpOutLeft';
if cpOutRight in CS then Result:=Result+','+'cpOutRight';
if cpOutTop in CS then Result:=Result+','+'cpOutTop';
if cpOutBottom in CS then Result:=Result+','+'cpOutBottom';
if cpHalfOutRight in CS then Result:=Result+','+'cpHalfOutRight';
if cpHalfOutBottom in CS then Result:=Result+','+'cpHalfOutBottom';
end;




constructor TFlexGrid.Create(AOwner: TComponent);
const GridStyle = [csCaptureMouse, csOpaque, csDoubleClicks];
begin
inherited Create(AOwner);
RowSelect:=TRowArray.Create;
if NewStyleControls then ControlStyle := GridStyle
else ControlStyle := GridStyle + [csFramed];
FCanEditModify := True;
FEditorMode:=False;
Editor:=TInplaceEdit.Create(Self);
Editor.FGrid:=Self;
Editor.Name:='InPlaceEditor';
Editor.ControlStyle := Editor.ControlStyle + [csNoDesignVisible];
Editor.Visible:=False;
Editor.Parent:=Self;
Editor.OnKeyDown:=EditKeyDownHandler;
Editor.OnKeyPress:=EditKeyPressHandler;
Editor.OnChange:=EditChangeHandler;
Editor.ShortCutMode:=smStandard;
MoveTimer:=TTimer.Create(Self);
MoveTimer.Enabled:=False;
MoveTimer.OnTimer:=HandleMoveTimerEvent;

BorderWidth:=0;
FBorderStyle := bsSingle;
ScrollBars := ssBoth;
ParentColor:=False;
SizerCellCol:=-1;
Options := [goFixedVertLine, goFixedHorzLine, goVertLine, goHorzLine, goRangeSelect];
FDefaultDrawing := True;

FDefaultColWidth := 64;
FDefaultRowHeight := 24;
FFixedCols := 1;
FFixedRows := 1;
ColCount:=5;
RowCount:=5;
TabStop:=True;
FSelected:=Point(1,1);
FFixedColor:=clBtnFace;
FBackColor:=clBackground;
FCellBorder:=clSilver;
Color:=clWindow;
DoubleBuffered:=True;
FFixedColSizing:=False;
end;




procedure TFlexGrid.CreateParams(var Params: TCreateParams);
begin
inherited CreateParams(Params);
with Params do
  begin
  Style := Style or WS_TABSTOP;
  WindowClass.style := CS_DBLCLKS;
  if FBorderStyle = bsSingle then
   if NewStyleControls and Ctl3D then
   	begin
   	Style := Style and not WS_BORDER;
   	ExStyle := ExStyle or WS_EX_CLIENTEDGE;
   	end
   else Style := Style or WS_BORDER;
  end;
end;



destructor TFlexGrid.Destroy;
begin
MoveTimer.Free;
Editor.Free;
RowSelect.Free;
inherited Destroy;
end;



procedure TFlexGrid.InvalidateGrid;
begin
Invalidate;
end;



procedure TFlexGrid.UpdateDesigner;
var ParentForm: TCustomForm;
begin
if (csDesigning in ComponentState) and
    HandleAllocated and
    not (csUpdating in ComponentState) then
	begin
	ParentForm := GetParentForm(Self);
	if Assigned(ParentForm) and Assigned(ParentForm.Designer) then
	ParentForm.Designer.Modified;
	end;
end;


procedure TFlexGrid.SetOptions(Value: TGridOptions);
begin
if FOptions <> Value then
	begin
	if goRowSelect in Value then Exclude(Value, goAlwaysShowEditor);
	FOptions := Value;
	if (not (goEditing in FOptions)) and  FEditorMode then EditorMode:=False;
	VertScrollBar.DoTrack:=goThumbTracking in Options;
	HorzScrollBar.DoTrack:=goThumbTracking in Options;
	if goAlwaysShowEditor in Value then ShowEditor else HideEditor;
	//if goRowSelect in Value then MoveCurrent(Col, Row,  True, False);
	InvalidateGrid;
	end;
end;

{------------------------------------------------------------------------------}


procedure TFlexGrid.WMGetDlgCode(var Msg: TWMGetDlgCode);
begin
  Msg.Result := DLGC_WANTARROWS;
  if goRowSelect in Options then Exit;
  if goTabs in Options then Msg.Result := Msg.Result or DLGC_WANTTAB;
  if goEditing in Options then Msg.Result := Msg.Result or DLGC_WANTCHARS;
end;


{---------------- Basic Cell Support Routines ---------------------------------}




function TFlexGrid.GetCellTopLeft(Col,Row: integer): TPoint;
{Get the unscrolled Top-Left corner of a cell in pixels}
begin
if Col=0 then Result.X:=0 else Result.X:=ColSplits[Col-1];
if Row=0 then Result.Y:=0 else Result.Y:=RowSplits[Row-1];
if Row>=FixedRows then
	begin
	if Row=0 then Result.Y:=FColSizes[Col].Offset
	else Result.Y:=Result.Y + FColSizes[Col].Offset;
	end;
end;



function TFlexGrid.GetCellBottomRight(Col,Row: integer): TPoint;
{Get the unscrolled Bottom/Right corner of a cell in pixels}
begin
Result.X:=ColSplits[Col];
Result.Y:=RowSplits[Row];
if Row>=FixedRows then Result.Y:=Result.Y + FColSizes[Col].Offset;
end;



procedure TFlexGrid.GetCellRect(Col, Row: integer; var TopLeft,BottomRight: TPoint);
{Get the unscrolled rectangle of a cell in pixels}
begin
TopLeft:=GetCellTopLeft(Col,Row);
BottomRight:=GetCellBottomRight(Col,Row);
end;



function TFlexGrid.GetCellRect(Col,Row: integer): TRect;
{Get the unscrolled rectangle of a cell in pixels}
begin
GetCellRect(Col,Row,Result.TopLeft,Result.BottomRight);
end;



function TFlexGrid.GetOffsetRect(X,Y: integer): TRect;
{Get cell retangle offset according to scroll position}
{This is the true drawing position of the of retangle}
var Off: integer;
begin
Result:=GetCellRect(X,Y);
if (FixedCols=0) or (X>=FixedCols) then
	begin
	Off:=ColSplits[FixedCols+FScrollOffset.X-1]-ColSplits[FixedCols-1];
	OffsetRect(Result,-Off,0);
	end;
if (FixedRows=0) or (Y>=FixedRows) then
	begin
	Off:=RowSplits[FixedRows+FScrollOffset.Y-1]-RowSplits[FixedRows-1];
	OffsetRect(Result,0,-Off);
	end;
if X>0 then Inc(Result.Left);
if Y>0 then Inc(Result.Top);
end;




function TFlexGrid.GetOffsetRect(P: TPoint): TRect;
begin
Result:=GetOffsetRect(P.X,P.Y);
end;



function TFlexGrid.GetClipOffRect(P: TPoint): TRect;
begin
Result:=GetOffsetRect(P);
if Result.Right>ClientRect.Right then Result.Right:=ClientRect.Right;
if Result.Bottom>ClientRect.Bottom then Result.Bottom:=ClientRect.Bottom;
end;



function TFlexGrid.GetPixelScrollOffset(Col,Row: integer): TPoint;
{Returns the current scroll offset in pixels}
{This is the difference between the fixed}
{ Rows/Columns and the first visible scrolled cell}
var R: TRect;
begin
R:=GetOffsetRect(Col,Row);
Result:=R.BottomRight;
end;



{---------------- Inplace Editor handling routines --------------------------------------}


function TFlexGrid.GetEditText(ACol, ARow: Longint): string;
begin
Result:='';
if Assigned(FOnGetEditText) then FOnGetEditText(Self, ACol, ARow, Result);
LastEditText:=Result;
end;


procedure TFlexGrid.SetEditText(ACol, ARow: Longint; const Value: string);
begin
if Assigned(FOnSetEditText) then FOnSetEditText(Self, ACol, ARow, Value);
end;



procedure TFlexGrid.UpdateText;
begin
if Editor.Visible then
	SetEditText(EditorCol, EditorRow, Editor.Text);
end;



procedure TFlexGrid.HideEditor;
begin
FEditorMode:=False;
Editor.Hide;
Invalidate;
end;



procedure TFlexGrid.ShowEditor;
begin
Editor.Undoer.Clear;
FEditorMode:=True;
Editor.Visible:=True;
Repaint;
Editor.SelectAll;
end;


procedure TFlexGrid.SetEditorMode(Value: Boolean);
begin
if not Value then HideEditor
else
	begin
	ShowEditor;
	end;
end;


function TFlexGrid.IsActiveControl: Boolean;
begin
Result:=(Screen.ActiveControl=Self) or (Screen.ActiveControl=Self.Editor);
end;



function TFlexGrid.CanEditShow: Boolean;
begin
Result := (goEditing in Options) and
	  not (csDesigning in ComponentState) and
	  HandleAllocated and
	  ((goAlwaysShowEditor in Options) or IsActiveControl);
end;


procedure TFlexGrid.PlaceEditor(R: TRect; ACol,ARow: integer);
begin
if CanEditShow and FEditorMode then
	begin
	EditorCol:=ACol; EditorRow:=ARow;
	Canvas.Brush.Color:=Color;
	Canvas.Rectangle(R);
	Editor.SetBounds(R.Left,R.Top+2,(R.Right-R.Left),(R.Bottom-R.Top)-4);
	Editor.Perform(EM_SETMARGINS, EC_LEFTMARGIN, 2);
	Editor.Visible:=True;
	if Editor.CanFocus then Editor.SetFocus;
	Editor.Text:=GetEditText(ACol,ARow);
	end
else Editor.Visible:=False;
end;


procedure TFlexGrid.EditChangeHandler(Sender: TObject);
begin
UpdateText;
end;



procedure TFlexGrid.EditKeyDownHandler(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
end;




procedure TFlexGrid.EditKeyPressHandler(Sender: TObject; var Key: Char);
var Handled: boolean;
begin
inherited KeyPress(Key);

Handled:=True;
case Key of
 CR_Char: begin FEditorMode:=False; Invalidate; end;
 else Handled:=False;
 end;
if Handled then Key:=#0;
end;


{---------------- Cell drawing routines --------------------------------------}


procedure TFlexGrid.Draw3DRect(Canvas: TCanvas; R: TRect; Horiz,Vert: boolean);
{Draw the 3D raised sides for fixed cells}
begin
Canvas.Pen.Color:=clBtnShadow;
if Vert then Canvas.Polyline([Point(R.Right-1,R.Bottom),Point(R.Right-1,R.Top-1)]);
if Horiz then Canvas.Polyline([Point(R.Left,R.Bottom-1),Point(R.Right+1,R.Bottom-1)]);
Canvas.Pen.Color:=clWhite;
if Vert then Canvas.Polyline([Point(R.Left,R.Bottom),Point(R.Left,R.Top-1)]);
if Horiz then Canvas.Polyline([Point(R.Left,R.Top),Point(R.Right+1,R.Top)]);
end;



function TFlexGrid.IsFixed(Col,Row: integer): boolean;
{Determines if a cell is in the fixed region}
begin
Result:=(Col<FixedCols) or (Row<FixedRows);
end;



function TFlexGrid.GetCellState(X,Y: integer; var R: TRect): TCellStates;
var X2,Y2: integer;
begin
Result:=[];
if X>=FixedCols then
	begin
	X2:=X-FScrollOffset.X;
	if X2<FixedCols then Result:=Result+[cpOutLeft];
	end;
if Y>=FixedRows then
	begin
	Y2:=Y-FScrollOffset.Y;
	if Y2<FixedRows then Result:=Result+[cpOutTop];
	end;
R:=GetOffsetRect(X,Y);
if R.Left>ClientRect.Right then Result:=Result+[cpOutRight]
else if R.Right>ClientRect.Right then Result:=Result+[cpHalfOutRight];

if R.Top>ClientRect.Bottom then Result:=Result+[cpOutBottom]
else if R.Bottom>ClientRect.Bottom then Result:=Result+[cpHalfOutBottom];
if Result=[] then Result:=[cpInside];
end;



procedure TFlexGrid.OwnerDraw(ACol,ARow: integer; Rect: TRect; DrawState: TGridDrawState);
{Child objects can override this subroutine instead of using OnCellDraw propert}
begin
end;




procedure TFlexGrid.DrawCell(X,Y: integer);
{Draw the specified cell}
var CellRect: TRect;
var DrawState: TGridDrawState;


	procedure SetColors(Fixed: boolean);
	var CC: TColor;                                                   
	begin
	if Fixed then CC:=FixedColor
	else if (goRowSelect in Options) and RowSelect[Y] then CC:=clHighlight
	else if (gdSelected in DrawState) and
		(goDrawFocusSelected in Options) then CC:=clSkyBlue
	else CC:=Self.Color;
	Canvas.Brush.Color:=CC;
	Canvas.Pen.Color:=CC;
	end;


	procedure ClearCell(R: TRect; Fixed: boolean);
	var C: TColor;
	begin
	{Include bottom-right border in clearing}
	R.Right:=R.Right+1;
	R.Bottom:=R.Bottom+1;
	SetColors(Fixed);
	Canvas.Rectangle(R);
	end;


	procedure DrawFixedCell(X,Y: integer; Horiz,Vert: boolean);
	{Draw the specified cell as a fixed cell}
	begin
	Draw3DRect(Canvas,CellRect,Horiz,Vert);
	Canvas.Pen.Color:=clBlack;
	if Horiz then Canvas.Polyline([Point(CellRect.Left,CellRect.Bottom),Point(CellRect.Right+1,CellRect.Bottom)]);
	if Vert then Canvas.Polyline([Point(CellRect.Right,CellRect.Bottom),Point(CellRect.Right,CellRect.Top-1)]);
	end;


	procedure DrawNormalCell(X,Y: integer);
	{Draw the specified cell as a normal cell}
	begin
	Canvas.Pen.Color:=CellBorder;
	{Horizontal line along bottom}
	if goHorzLine in Options then Canvas.Polyline([Point(CellRect.Left,CellRect.Bottom), Point(CellRect.Right,CellRect.Bottom)]);
	{Vertical line along right}
	if goVertLine in Options then Canvas.Polyline([Point(CellRect.Right,CellRect.Bottom), Point(CellRect.Right,CellRect.Top-1)]);
        {Horizontal along top}
	if (goHorzLine in Options) and  (Y>=FixedRows){ and (FColSizes[X].Offset<>0) } then Canvas.Polyline([Point(CellRect.Left,CellRect.Top-1),Point(CellRect.Right,CellRect.Top-1)]);
	{Vertical along left}
	if (goVertLine in Options) and (X>=FixedCols) then Canvas.Polyline([Point(CellRect.Left-1,CellRect.Top-1),Point(CellRect.Left-1,CellRect.Bottom)]);

	if gdSelected in DrawState  then
		begin
		DrawFocusRect(Canvas.Handle,CellRect);
		PlaceEditor(CellRect,X,Y);
		end;
	end;


begin
CellRect:=GetOffsetRect(X,Y);
DrawState:=[];
if IsFixed(X,Y) then Include(DrawState,gdFixed);
if (FSelected.X=X) and (FSelected.Y=Y) then Include(DrawState, gdSelected);
if Self.Focused then Include(DrawState, gdFocused);

ClearCell(CellRect,gdFixed in DrawState);
Canvas.Font:=Self.Font;
OwnerDraw(X, Y, CellRect, DrawState);
if Assigned(FOnDrawCell) then FOnDrawCell(Self, X, Y, CellRect, DrawState);
if gdFixed in DrawState then DrawFixedCell(X,Y,goFixedHorzLine in Options, goFixedVertLine in Options)
else DrawNormalCell(X,Y);
end;



procedure TFlexGrid.DrawAllCells;
var X,Y,Start: integer;
var CS: TCellStates;
var R: TRect;
begin
LastFullCol:=-1;
LastFullRow:=-1;
{Draw non-fixed cells}
for X:=FixedCols+FScrollOffset.X to ColCount-1 do
	begin
	CS:=GetCellState(X,0,R);
	if cpOutRight in CS then break;
	Start:=FixedRows+FScrollOffset.Y;
	if FColSizes[X].Offset<>0 then Start:=Start-(1+(FColSizes[X].Offset div (R.Bottom-R.Top)));
	for Y:=Start to RowCount-1 do
		begin
		CS:=GetCellState(X,Y,R);
		if cpOutBottom in CS then break
		else
			begin
			DrawCell(X,Y);
			end;
		end;
	end;

{Draw upper-left corner cells}
for Y:=0 to FixedRows-1 do
 for X:=0 to FixedCols-1 do
	DrawCell(X,Y);
{Draw top fixed rows}
for Y:=0 to FixedRows-1 do
 for X:=FixedCols+FScrollOffset.X to ColCount-1 do
	begin
	CS:=GetCellState(X,Y,R);
	if (cpInside in CS) or (cpHalfOutRight in CS) then
		begin
		if not (cpHalfOutRight in CS) then LastFullCol:=X;
		DrawCell(X,Y)
		end
	else break;
	end;

{Draw left fixed columns}
for X:=0 to FixedCols-1 do
 for Y:=FixedRows+FScrollOffset.Y to RowCount-1 do
	begin
	CS:=GetCellState(X,Y,R);
	if (cpInside in CS) or (cpHalfOutBottom in CS) then
		begin
		DrawCell(X,Y);
		if not (cpHalfOutBottom in CS) then LastFullRow:=Y;
		end
	else break;
	end;
end;



procedure TFlexGrid.Paint;
begin
inherited;
Canvas.Pen.Color:=BackColor;
Canvas.Brush.Color:=BackColor;
Canvas.Rectangle(Self.ClientRect);
DrawAllCells;
end;


{----------- Routines to Handle Column Sizing and Fixed Row Splits -------------}


procedure TFlexGrid.SetSplitsFromSizes(var A: TCellSizeArray);
var I,Sum: integer;
begin
Sum:=0;
for I:=0 to High(A) do
	begin
	Sum:=Sum+A[I].Size;
	A[I].SplitPos:=Sum;
	Inc(Sum);
	end;
end;



procedure TFlexGrid.SetSizesFromSplits(var A: TCellSizeArray);
var I: integer;
var Size: integer;
begin
for I:=0 to High(A) do
	begin
	if I=0 then Size:=A[I].SplitPos
	else Size:=A[I].SplitPos-A[I-1].SplitPos;
	A[I].Size:=Size;
	end;
end;



procedure TFlexGrid.SetCellSizeDefaults(var A: TCellSizeArray; Default: integer);
var I: integer;
begin
for I:=0 to High(A) do
	begin
	A[I].Size:=Default;
	A[I].TabStop:=True;
	end;
SetSplitsFromSizes(A);
end;



procedure TFlexGrid.SetCellSizeValues(var A: TCellSizeArray; Count,Default: integer);
begin
SetLength(A,Count);
SetCellSizeDefaults(A,Default);
end;



function TFlexGrid.GetPixelOffset: TPoint;
{Get the width of the cells scrolled off left or under fixed cells}
var I: integer;
begin
Result:=Point(0,0);
{Add up sizes of cells between last fixed cell and visible cell}
for I:=FixedCols to (FixedCols+LeftCol)-1 do Result.X:=Result.X+FColSizes[I].Size;
{Add up sizes of cells between last fixed cell and visible cell}
for I:=FixedRows to (FixedRows+TopRow)-1 do Result.Y:=Result.Y+FRowsizes[I].Size;
end;



function TFlexGrid.MoveSpits(Inx,NewPos: integer): integer;
{Move a split by the specified Delta - Returns -1 if splits not changed
{handles shifting subsequent cells}
var Old,Delta,I: integer;
var Off: TPoint;
const GuardBand = 5;
begin
Result:=-1;
Off:=GetPixelOffset;
NewPos:=NewPos + Off.X;
Old:=FColSizes[Inx].SplitPos;
Delta:=NewPos-Old;
if (Old=NewPos) or (NewPos<GuardBand) or
   ((Inx>0) and (NewPos<(FColSizes[Inx-1].SplitPos+GuardBand))) then exit;

FColSizes[Inx].SplitPos:=NewPos;
for I:=Inx+1 to High(FColSizes) do
  FColSizes[I].SplitPos:=FColSizes[I].SplitPos+Delta;
SetSizesFromSplits(FColSizes);
Invalidate;
Result:=FColSizes[Inx].Size;
end;




{-------------- Scroll Bar Handling Routines ----------------------------------}



procedure TFlexGrid.SetHScrollBarRange;
var Max,Sum,I,W: integer;
var SW,GW: integer;
begin
SW:=GetSystemMetrics(SM_CXVSCROLL);
Max:=FColCount-FixedCols;
W:=Width;
if VertScrollBar.Max<>0 then Dec(W,SW);
if BorderStyle=bsSingle then Dec(W,2);

Max:=0;
GW:=GridWidth;
for I:=1 to FColCount-1 do
	begin
	if GW<W then break;
	GW:=GW-(FColSizes[I].Size+1);
	Inc(Max);
	end;
if FColSizes[FColCount-1].Size>=GW then Dec(Max);
HorzScrollBar.Max:=Max;
DoHorzChange(Self);
end;



procedure TFlexGrid.SetVScrollBarRange;
var Max,Sum,I,H,Off: integer;
begin
{Get the number of scrolling rows minus one}
Max:=(FRowCount-FixedRows)-1;
{Get client pixel height minus scroll bar height}
H:=Height-GetSystemMetrics(SM_CYHSCROLL);
{Subtract fixed rows from pixel height}
for I:=0 to FixedRows-1 do Dec(H,FRowSizes[I].Size+2);
{Get the maximum column offset }
Off:=0;
for I:=0 to FColCount-1 do
 if FColSizes[I].Offset>Off then Off:=FColSizes[I].Offset;
{Subtract off offset}
H:=H-Off;
{Subtract off rows that will fit on last scroll position}
Sum:=FRowSizes[FRowCount-1].Size+2;
for I:=FRowCount-2 downto FixedRows do
	begin
	Sum:=Sum+FRowSizes[I].Size+2;
	if Sum>H then break;
	Dec(Max);
	end;
if Max<0 then Max:=0;
{Set scroll bar range}
VertScrollBar.Max:=Max;
DoVertChange(Self);
end;



procedure TFlexGrid.SetScrollBarParams;
begin
SetHScrollBarRange;
SetVScrollBarRange;
end;



{------------------- Cell Hit Testing ----------------------------------------}



function TFlexGrid.ColumnHitTest(X: integer): integer;
var Off: integer;
begin
for Result:=0 to FixedCols-1 do
 if X<FColSizes[Result].SplitPos then exit;
Off:=ColSplits[FixedCols+FScrollOffset.X-1]-ColSplits[FixedCols-1];
for Result:=FScrollOffset.X to ColCount-1 do
 if X<(FColSizes[Result].SplitPos-Off) then exit;
Result:=ColCount;
end;


function TFlexGrid.RowHitTest(Col,Y: integer): integer;
var Off,LastFixed: integer;
begin
{Is Y in the fixed rows? exit it if is}
LastFixed:=0;
for Result:=0 to FixedRows-1 do
	begin
	LastFixed:=FRowSizes[Result].SplitPos;
	if Y<LastFixed then exit;
	end;

{Look at gap in offset}
Result:=-1;
if Y<(LastFixed+FColSizes[Col].Offset) then exit;

{Look at scrolling rows}
Off:=RowSplits[FixedRows+FScrollOffset.Y-1]-RowSplits[FixedRows-1];
Off:=Off-FColSizes[Col].Offset;
for Result:=FScrollOffset.Y to RowCount-1 do
 if Y<(FRowSizes[Result].SplitPos-Off) then exit;
Result:=RowCount;
end;



function TFlexGrid.CellHitTest(X,Y: integer; var Cell: TPoint): boolean;
begin
Cell.X:=ColumnHitTest(X);
Cell.Y:=RowHitTest(Cell.X,Y);
if Cell.X>=ColCount then Cell.X:=-1;
if Cell.Y>=RowCount then Cell.Y:=-1;
Result:=(Cell.X>-1) and (Cell.Y>-1);
end;




function TFlexGrid.OffBottom(Y: integer): boolean;
var Cell: TPoint;
begin
Result:=RowHitTest(0,Y)>=RowCount;
end;

{------------------ Tabstop Routines ------------------------------------------}



function TFlexGrid.GetTabStops(Index: Integer): Boolean;
begin
if (Index>=0) and (Index<=High(FColSizes)) then Result:=FColSizes[Index].TabStop
else Result:=False;
end;



procedure TFlexGrid.SetTabStops(Index: Integer; const Value: Boolean);
begin
if (Index>=0) and (Index<=High(FColSizes)) then FColSizes[Index].TabStop:=Value;
end;


{----------------- Keyboard Event Handlers ------------------------------------}



function TFlexGrid.SelectCell(ACol, ARow: Longint): Boolean;
begin
Result := True;
if Assigned(FOnSelectCell) then FOnSelectCell(Self, ACol, ARow, Result);
end;





procedure TFlexGrid.OptimizeScrolling(Cell: TPoint);
var R: TRect;
var CS: TCellStates;
var S: string;
begin
S:='';
while true do
	begin
	CS:=GetCellState(Cell.X,Cell.Y,R);
	S:=S+','+GetCellStatesStr(CS);
	if [cpInside] = CS then break;
	if cpOutLeft in CS then SetLeftCol(LeftCol-1);
	if (cpOutRight in CS) or (cpHalfOutRight in CS) then SetLeftCol(LeftCol+1);;
	if cpOutTop in CS then SetTopRow(TopRow-1);
	if (cpOutBottom in CS) or (cpHalfOutBottom in CS)  then SetTopRow(TopRow+1);
	end;
end;


procedure TFlexGrid.OptimizeScrolling;
begin
OptimizeScrolling(FSelected);
end;



{ Default key behavior:

Up Arrow =    Move selected cell up one row
Down Arrow =  Move selected cell down one row
Left Arrow =  Move selected cell left one column
Right Arrow = Move selected cell right one column
Page Up = Move selected cell Up one page
Page Down = Move selected cell Down one page
Home = Move selected cell to start of current row
End = Move selected cell to End of Current Row

Control Up Arrow =    Scroll up one row - selected cell doesn't change
Control Down Arrow =  Scroll Down one row - selected cell doesn't change
Control Left Arrow =  Move selected cell left one Page
Control Right Arrow = Move selected cell right one Page
Control Page Up = Move selected cell to Top Row of current page
Control Page Down = Move selected cell to Bottom Row of current page
Control Home = Move selected cell to First Column and Row
Control End = Move selected cell to Last Column and Row

Tab = Move selected cell right, wrap left to right and bottom to top
Shift-Tab = Move selected cell Left, wrap left to right and bottom to top
Tabs also skip cells that are set to False in the "TabStops" property

}



function TFlexGrid.TestMoveSelected(Info: TGridInfo; var X,Y: integer): boolean;
{Test if you user wants to modify the movement}
begin
Result:=True;
if Assigned(FOnMoveSelected) then Result:=FOnMoveSelected(Info, X,Y);
end;



procedure TFlexGrid.KeyDown(var Key: Word; Shift: TShiftState);
var Changed: boolean;
var W,H: integer;
var Info: TGridInfo;



	procedure MoveSelected(X,Y: integer; Wrap: boolean);
	{Move the selection, handle wrapping}
	{Assumes that we are wrapping X first then Y}
	var NX,NY: integer;
	begin
	{Test if you user wants to modify the movement}
	if not TestMoveSelected(Info,X,Y) then exit;

	{Get new position}
	NX:=FSelected.X+X;
	NY:=FSelected.Y+Y;
	{Test if off grid to left }
	if NX<FixedCols then
		begin
		if Wrap then
			begin
			NX:=ColCount-1;
			Dec(NY);
			end
		else NX:=FixedCols;
		end;
	{Test if off grid to right }
	if NX>=ColCount then
		begin
		if Wrap then
			begin
                        NX:=FixedCols;
			Inc(NY);
			end
		else NX:=ColCount-1;
		end;
	{Test if off grid to the top }
	if NY<FixedRows then
		begin
		if Wrap then NY:=RowCount-1
		else NY:=FixedRows;
		end;
	{Test if off grid to the bottom }
	if NY>=RowCount then
		begin
		if Wrap then NY:=FixedRows
		else NY:=RowCount-1;
		end;

	if SelectCell(NX,NY) then
		begin
		FSelected.X:=NX;
		FSelected.Y:=NY;
		OptimizeScrolling;
		if not (goAlwaysShowEditor in Options) then HideEditor;
		Invalidate;
		end;
	end;


	procedure SetSelectedPos(X,Y: integer);
	var IncX,IncY: integer;
	begin
	{Find increment}
	IncX:=X-FSelected.X;
	IncY:=Y-FSelected.Y;
	{Test if you user wants to modify the increment}
	if not TestMoveSelected(Info,IncX,IncY) then exit;

	{Move set selected position}
	Inc(FSelected.X,IncX);
	Inc(FSelected.Y,IncY);
	OptimizeScrolling;
	Invalidate;
	end;



	procedure TopBottom(Top: boolean);
	var Y: integer;
	begin
	if Top then Y:=TopRow+FixedRows
	else Y:=LastFullRow;
	SetSelectedPos(0,Y);
	end;


	procedure LeftRight(Left: boolean);
	var X: integer;
	begin
	if Left then X:=FixedCols
	else X:=ColCount-1;
	SetSelectedPos(X,FSelected.Y);
	end;


	procedure StartEnd(Start: boolean);
	var X,Y: integer;
	begin
	if Start then
		begin
		X:=FixedCols;
		Y:=TopRow+FixedRows;
		end
	else
		begin
		X:=ColCount-1;
		Y:=RowCount-1;
		end;
	SetSelectedPos(X,Y);
	end;


	procedure HandleTab;
	{Handle tabs stopping only cells that are set as tabstops}
	var OldX: integer;
	begin
	OldX:=FSelected.X;
	repeat
		begin
		if ssShift in Shift then MoveSelected(-1,0,True)
		else MoveSelected(1,0,True);
		end
	until TabStops[FSelected.X] or (FSelected.X = OldX);
	end;




	procedure ScrollGrid(X,Y: integer);
	begin
	LeftCol:=LeftCol+X;
	TopRow:=TopRow+Y;
	Invalidate;
	end;

begin
inherited;
Info.CurrentPos:=FSelected;
Info.Sender:=Self;
Info.ClientRect.TopLeft:=Point(FixedCols,FixedRows);;
Info.ClientRect.BottomRight:=Point(ColCount,RowCount);

W:=LastFullCol-LeftCol-FixedCols;
H:=LastFullRow-TopRow-FixedRows;
Changed:=True;
if ssCtrl in Shift then
	begin
	case Key of
	 VK_UP:    ScrollGrid(0,-1);
	 VK_DOWN:  ScrollGrid(0, 1);
	 VK_LEFT:  MoveSelected(-W,0,False);
	 VK_RIGHT: MoveSelected( W,0,False);
	 VK_PRIOR: TopBottom(True);
	 VK_NEXT: TopBottom(False);
	 VK_HOME: StartEnd(True);
	 VK_END: StartEnd(False);
	 else Changed:=False;
	 end;
	end
  else
	begin
	case Key of
	 VK_ESCAPE: if Editor.Visible then
	 	begin
		SetFieldText(FSelected.X,FSelected.Y,LastEditText);
	 	HideEditor;
	 	invalidate;
	 	end;
	 VK_UP: MoveSelected(0,-1,False);
	 VK_DOWN: MoveSelected(0,1,False);
	 VK_LEFT: MoveSelected(-1,0,False);
	 VK_RIGHT: MoveSelected(1,0,False);
	 VK_PRIOR: MoveSelected(0,-H,False);
	 VK_NEXT: MoveSelected(0,H,False);
	 VK_HOME: LeftRight(True);
	 VK_END: LeftRight(False);
	 VK_TAB: HandleTab;
	 VK_F2: EditorMode:=True;
	 else Changed:=False;
	 end;
	end;
if Changed then
	begin
	if Assigned(FOnPositionChanged) then FOnPositionChanged(Self);
	end;
end;


procedure TFlexGrid.ShowEditorChar(Ch: Char);
begin
ShowEditor;
if Editor <> nil then
 PostMessage(Editor.Handle, WM_CHAR, Word(Ch), 0);
end;


procedure TFlexGrid.WMChar(var Msg: TWMChar);
begin
if (goEditing in Options) and (Char(Msg.CharCode) in [^H, #32..#255]) then ShowEditorChar(Char(Msg.CharCode))
else inherited;
end;




procedure TFlexGrid.WMNCHitTest(var Msg: TWMNCHitTest);
var XPoint: TPoint;
begin
{Make scroll bars work at design time}
XPoint := ClientToScreen(GetClientRect.BottomRight);
if XPoint.x < Msg.Pos.X then Msg.Result := HTVSCROLL
else if XPoint.y < Msg.Pos.Y then Msg.Result := HTHSCROLL
else inherited;
end;




procedure TFlexGrid.KeyPress(var Key: Char);
begin
inherited KeyPress(Key);
if not (goAlwaysShowEditor in Options) and (Key = CR_Char) then
	begin
	if FEditorMode then HideEditor
	else ShowEditor;
	Key:=#0;
	end;
end;


{------------------ Column Sizing Routines -------------------------------------}


function TFlexGrid.ColSizerHitTest(X,Y: integer; var Col: integer): boolean;
{Test to see if the X,Y pixel position is over a colum sizer}
{And returns the column if it is}
var I,LastCol,RightDist, ColStart: integer;
var Pos: integer;
begin
Result:=False;
if X<0 then exit;
if (FFixedRows<1) or (Y>FRowSizes[FFixedRows-1].SplitPos) then exit;
Result:=True;
LastCol:=High(FColSizes);
if FixedColSizing then ColStart:=0 else ColStart:=FixedCols;
for I:=ColStart to LastCol do
	begin
        Pos:=GetPixelScrollOffset(I,0).X;
	RightDist:=ClientRect.Right-X;
	{Is the split position past right edge}
	if Pos>ClientRect.Right then
		begin
		{Check if mouse is at right edge}
		if (RightDist>1) and (RightDist<8) then
			begin
			Col:=I;
			exit;
			end;
		end
	else if Abs(Pos-X)<5 then
		begin
		Col:=I;
		exit;
		end;
	end;
Result:=False;
end;




var LastLine: TLine;




procedure TFlexGrid.DrawDottedLine(Line: TLine; PenWidth: integer);
var OldPen: TPen;
begin
OldPen:=TPen.Create;
try
OldPen.Assign(Canvas.Pen);
Canvas.Pen.Style:=psDot;
Canvas.Pen.Mode:=pmNot;
Canvas.Pen.Width:=PenWidth;

Canvas.MoveTo(Line.P1.X,Line.P1.Y);
Canvas.LineTo(Line.P2.X,Line.P2.Y);

LastLine:=Line;
finally Canvas.Pen.Assign(OldPen); end;
end;




procedure TFlexGrid.DrawSizingLine(Start,PenWidth: integer; Vertical: boolean);
var L: TLine;
begin
if Vertical then L:=Line(Start,0,Start,Height)
else L:=Line(0,Start,Width,Start);
DrawDottedLine(L,PenWidth);
SizerLineStart:=Start;
end;




{------------------ Mouse Event handlers --------------------------------------}


procedure TFlexGrid.DoRowSelect(Row: integer; Value: boolean);
begin
RowSelect[Row]:=Value;
Invalidate;
end;



procedure TFlexGrid.ToggleRowSelect(Row: integer);
begin
RowSelect[Row]:=not RowSelect[Row];
Invalidate;
end;



procedure TFlexGrid.SelectRange(StartRow,EndRow: integer);
var I: integer;
begin
EnsureRange(StartRow,0,RowSelect.Count-1);
EnsureRange(EndRow,0,RowSelect.Count-1);
SwapIntegerArgs(StartRow>EndRow,StartRow,EndRow);
RowSelect.Clear;
for I:=StartRow to EndRow do RowSelect[I]:=True;
Invalidate;
end;






procedure TFlexGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Col: integer;
var Cell: TPoint;
var CR: TRect;
var SameCell: boolean;



	procedure HandleFixedCell;
	begin
	if (goColMoving in Options) and
	   (Cell.Y < FixedRows) and
	   (Cell.X >= FixedCols) and
	   CanColMoveStart(Cell.X) then
		begin
		{Handle click in fixed column}
		MouseDownState:=mdColMove;
		StartMoveCell:=Cell;
		EndMoveCell:=Cell;
		CR:=GetClipOffRect(Cell);
		DrawSizingLine(CR.Left,5,True);
		RowSelect.Clear;
		end
	else if (ssCtrl in Shift) and
		(Cell.X < FixedCols) and (Cell.Y >= FixedRows)  then
		begin
		if (ssShift in Shift) then
			begin
			DoRowSelect(Cell.Y,True);
			SelectRange(StartMoveCell.Y,Cell.Y);
			end
		else
			begin
			ToggleRowSelect(Cell.Y);
			if RowSelect[Cell.Y] then
				begin
				MouseDownState:=mdRowSelect;
				StartMoveCell:=Cell;
				EndMoveCell:=Cell;
				end;
			end;
		HideEditor;
		end
	else if (goRowMoving in Options) and
		(Cell.X < FixedCols) and (Cell.Y >= FixedRows) then
		begin
		{Handle click in fixed Row}
		MouseDownState:=mdRowMove;
		StartMoveCell:=Cell;
		EndMoveCell:=Cell;
		CR:=GetClipOffRect(Cell);
		RowSelect.Clear;
		Repaint;
		DrawSizingLine(CR.Top,5,False);
		end
	end;


begin
inherited MouseDown(Button, Shift, X, Y);
MouseDownState:=mdNone;
if ssLeft in Shift then
	begin
	if ((goColSizing in Options) or
	    (csDesigning in ComponentState)) and
	   ColSizerHitTest(X,Y,Col) then
		begin
		MouseDownState:=mdColSize;
		SizerCellCol:=Col;
		DrawSizingLine(X,1,True);
		SizerStartX:=X;
		end
	else if CellHitTest(X,Y,Cell) then
		begin
		if (Cell.Y < FixedRows) or (Cell.X < FixedCols) then HandleFixedCell
		else if SelectCell(Cell.X,Cell.Y) then
			begin
			if (csDesigning in ComponentState) then exit;
			MouseDownState:=mdCellSelect;
			SetFocus;
			SameCell:=(FSelected.X=Cell.X) and (FSelected.Y=Cell.Y);
			FSelected:=Cell;
			RowSelect.Clear;
			if SameCell or (goAlwaysShowEditor in Options) then ShowEditor else HideEditor;
			end;
		end
	else if (ssCtrl in Shift) and OffBottom(Y) then
		begin
		MouseDownState:=mdOffBottom;
		CR:=GetClipOffRect(Point(0,RowCount));
		RowSelect.Clear;
		Repaint;
		DrawSizingLine(CR.Top,5,False);
		end;
	end;
end;




function TFlexGrid.GetHorzScrollType(X,Y: integer): THorizScrollType;
var Cell: TPoint;
begin
CellHitTest(X,Y,Cell);
if Cell.X<0 then Result:=stNone
else if (Cell.X<FixedCols) or (X<0) then Result:=stLeft
else if (X>ClientRect.Right) then Result:=stRight
else Result:=stNone;
end;




function TFlexGrid.GetVertScrollType(X,Y: integer): TVertScrollType;
var Cell: TPoint;
begin
CellHitTest(X,Y,Cell);
if Cell.Y<0 then Result:=vtNone
else if (Cell.Y<FixedRows) or (Y<0) then Result:=vtUp
else if (Y>ClientRect.Bottom) then Result:=vtDown
else Result:=vtNone;
end;




procedure TFlexGrid.HandleMoveTimerEvent(Sender: TObject);
var Point: TPoint;
var CR: TRect;


	procedure DoHorizScroll;
	var ST: THorizScrollType;
	var LP: integer;
	begin
	ST:=GetHorzScrollType(Point.X,Point.Y);
	if ST=stNone then begin MoveTimer.Enabled:=False; exit; end;

	{Erase the old line}
	DrawSizingLine(LastLine.P1.X,5,True);
	case ST of
	 stLeft: if LeftCol>0 then
		begin
		SetLeftCol(LeftCol-1);
		Repaint;
		Dec(EndMoveCell.X);
		end;
	 stRight: if LeftCol<ColCount-2 then
		begin
		SetLeftCol(LeftCol+1);
		Repaint;
		if EndMoveCell.X<ColCount-1 then Inc(EndMoveCell.X);
		end;
	 end;
	CR:=GetClipOffRect(EndMoveCell);
	if (EndMoveCell.X<=StartMoveCell.X) then LP:=CR.Left else LP:=CR.Right;
	DrawSizingLine(LP,5,True);
	end;



	procedure DoVertScroll;
	var ST: TVertScrollType;
	var LP: integer;
	begin
	ST:=GetVertScrollType(Point.X,Point.Y);
	if ST=vtNone then begin MoveTimer.Enabled:=False; exit; end;

	{Erase the old line}
	DrawSizingLine(LastLine.P1.Y,5,False);
	case ST of
	 vtUp: if TopRow>0 then
		begin
		SetTopRow(TopRow-1);
		Repaint;
		Dec(EndMoveCell.Y);
		end;
	 vtDown: if TopRow<RowCount-2 then
		begin
		SetTopRow(TopRow+1);
		Repaint;
		if EndMoveCell.Y<RowCount-1 then Inc(EndMoveCell.Y);
		end;
	 end;
	CR:=GetClipOffRect(EndMoveCell);
	if (EndMoveCell.Y<=StartMoveCell.Y) then LP:=CR.Top else LP:=CR.Bottom;
	DrawSizingLine(LP,5,False);
	end;


begin
GetCursorPos(Point);
Point := ScreenToClient(Point);
case MouseDownState of
 mdColMove: DoHorizScroll;
 mdRowMove: DoVertScroll;
 end;
end;




procedure TFlexGrid.MouseMove(Shift: TShiftState; X, Y: Integer);
var Col: integer;
var CR: TRect;
var Cell: TPoint;



	function HandleHorzScrolling: boolean;
	var SC: THorizScrollType;
	begin
	Result:=False;
	SC:=GetHorzScrollType(X,Y);
	if SC=stNone then exit;
	MoveTimer.Interval:=150;
	MoveTimer.Enabled:=True;
	Result:=True;
	end;



	function HandleVertScrolling: boolean;
	var SC: TVertScrollType;
	begin
	Result:=False;
	SC:=GetVertScrollType(X,Y);
	if SC=vtNone then exit;
	MoveTimer.Interval:=150;
	MoveTimer.Enabled:=True;
	Result:=True;
	end;


begin
inherited ;
if ((goColSizing in Options) or
    (csDesigning in ComponentState)) and ColSizerHitTest(X,Y,Col) then FCursor:=crHSplit else FCursor:=crDefault;
if not (ssLeft in Shift) then exit;
case MouseDownState of
 mdColSize:
	begin
	{Erase and draw}
	DrawSizingLine(SizerLineStart,1,True);
	DrawSizingLine(X,1,True);
	end;
 mdRowSize:
	begin
	end;
 mdColMove:
	begin
	{Note: if left-mouse is down, X/Y can go beyond ClientRetangle}
	if CellHitTest(X,Y,Cell) and CanColMoveOver(Cell.X) then
		begin
		{If we are still in the same cell and not off left/right edge do nothing}
		if (Cell.X=EndMoveCell.X) and not ((X>ClientRect.Right) or (X<0)) then exit;
		if HandleHorzScrolling then exit;
		{Erase the old line}
		DrawSizingLine(LastLine.P1.X,5,True);
		{Draw new line}
		CR:=GetClipOffRect(Cell);
		if (Cell.X<=StartMoveCell.X) then DrawSizingLine(CR.Left,5,True)
		else DrawSizingLine(CR.Right,5,True);
		EndMoveCell:=Cell;
		end;
	end;
 mdRowMove:
	begin
	{Note: if left-mouse is down, X/Y can go beyond ClientRetangle}
	if CellHitTest(X,Y,Cell) then
		begin
		{If we are still in the same cell and not off left/right edge do nothing}
		if (Cell.Y=EndMoveCell.Y) and not ((Y>ClientRect.Bottom) or (Y<0)) then exit;
		if HandleVertScrolling then exit;
		{Erase the old line}
		DrawSizingLine(LastLine.P1.Y,5,False);
		{Draw new line}
		CR:=GetClipOffRect(Cell);
		if (Cell.Y<=StartMoveCell.Y) then DrawSizingLine(CR.Top,5,False)
		else DrawSizingLine(CR.Bottom,5,False);
		EndMoveCell:=Cell;
		end;
	end;
 mdRowSelect:
	begin
	if CellHitTest(X,Y,Cell) then
	 if EndMoveCell.Y<>Cell.Y then
		begin
		if abs(EndMoveCell.Y-StartMoveCell.Y)>abs(Cell.Y-StartMoveCell.Y) then
			begin
			DoRowSelect(EndMoveCell.Y,False)
			end
		else
			begin
			if Cell.Y<FixedRows then
				begin
				Cell.Y:=TopRow;
				SetTopRow(TopRow-1);
				end;
			DoRowSelect(Cell.Y,True);
			end;

		OptimizeScrolling(Cell);
		EndMoveCell:=Cell;
		end;
	end;
 end;
end;






procedure TFlexGrid.ColumnResize(NewSize,Index: integer);
begin
if (NewSize>=0) and Assigned(FOnColumnResize) then FOnColumnResize(Self,SizerCellCol,NewSize);
end;



procedure TFlexGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var NewSize: integer;
begin
inherited;
case MouseDownState of
 mdColSize:
	begin
	{Clear sizer line}
	DrawSizingLine(SizerLineStart,1,True);
	NewSize:=MoveSpits(SizerCellCol,X);
	UpdateDesigner;
	ColumnResize(NewSize,SizerCellCol);
	end;
 mdRowSize:
	begin
	UpdateDesigner;
	{Row sizing not currently implemented}
	NewSize:=0;
	if Assigned(FOnRowResize) then FOnRowResize(Self,SizerCellRow,NewSize);
	end;
 mdColMove: if CanColMoveDrop(EndMoveCell.X) then
	begin
	{Clear move line}
	DrawSizingLine(LastLine.P1.X,5,True);
	ColumnMoved(StartMoveCell.X,EndMoveCell.X);
	UpdateDesigner;
	end;
 mdRowMove:
	begin
	{Clear move line}
	DrawSizingLine(LastLine.P1.Y,5,False);
	if StartMoveCell.Y<>EndMoveCell.Y then RowMoved(StartMoveCell.Y,EndMoveCell.Y);
	UpdateDesigner;
	end;
 end;
MouseDownState:=mdNone;
MoveTimer.Enabled:=False;
end;





{------------ Property Field Access Routines ----------------------------------}



function TFlexGrid.GetColSplits(Index: Integer): Integer;
begin
Result:=0;
if Index<0 then exit;
if Index >High(FColSizes) then Index:=High(FColSizes);
Result:=FColSizes[Index].SplitPos;
end;



function TFlexGrid.GetRowSplits(Index: Integer): Integer;
begin
Result:=0;
if Index<0 then exit;
if Index >High(FRowSizes) then Index:=High(FRowSizes);
Result:=FRowSizes[Index].SplitPos;
end;



function TFlexGrid.GetGridWidth: integer;
var I: integer;
begin
Result:=0;
for I:=0 to FColCount-1 do
 Result:=Result+FColSizes[I].Size+1;
Inc(Result);
end;



function TFlexGrid.GetGridHeight: integer;
var I: integer;
begin
Result:=0;
for I:=0 to FRowCount-1 do
 Result:=Result+FRowSizes[I].Size+1;
Inc(Result);
end;



procedure TFlexGrid.SetColSplits(Index: Integer; const Value: Integer);
begin
if (Index<0) or (Index >High(FColSizes)) then raise Exception.Create('Index Out Of Range');
if Value <> FColSizes[Index].SplitPos then
	begin
	FColSizes[Index].SplitPos:=Value;
	SetSizesFromSplits(FColSizes);
	Invalidate;
	end;
end;



procedure TFlexGrid.SetRowSplits(Index: Integer; const Value: Integer);
begin
if (Index<0) or (Index >High(FRowSizes)) then raise Exception.Create('Index Out Of Range');
if Value <> FRowSizes[Index].SplitPos then
	begin
	FRowSizes[Index].SplitPos:=Value;
	SetSizesFromSplits(FRowSizes);
	Invalidate;
	end;
end;



procedure TFlexGrid.SetRowCount(Value: integer);
begin
if FRowCount<>Value then
	begin
	if Value < 1 then Value := 1;
	FRowCount:=Value;
	SetCellSizeValues(Self.FRowSizes,FRowCount,FDefaultRowHeight);
	SetVScrollBarRange;
	Invalidate;
	RowSelect.Count:=Value;
	end;
end;





procedure TFlexGrid.SetColCount(Value: integer);
begin
if FColCount<>Value then
	begin
	if Value < 1 then Value := 1;
	FColCount:=Value;
	SetCellSizeValues(FColSizes,FColCount,FDefaultColWidth);
	SetHScrollBarRange;
	Invalidate;
	end;
end;



procedure TFlexGrid.SetDefaultColWidth(const Value: Integer);
begin
{Don't test if value has changed because}
{ a n individual row could have changed}
FDefaultColWidth := Value;
SetCellSizeDefaults(FColSizes,Value);
Invalidate;
end;



procedure TFlexGrid.SetDefaultRowHeight(const Value: Integer);
begin
{Don't test if value has changed because}
{ a n individual row could have changed}
FDefaultRowHeight := Value;
SetCellSizeDefaults(FRowSizes,Value);
Invalidate;
end;



function TFlexGrid.GetColWidths(Index: Integer): integer;
begin
if (Index<0) or (Index >High(FColSizes)) then
	begin
	Result:=DefaultColWidth;
	end
else Result:=FColSizes[Index].Size;
end;




function TFlexGrid.GetColOffset(Index: Integer): Integer;
begin
if (Index<0) or (Index >High(FColSizes)) then
	begin
	Result:=0;
	end
else Result:=FColSizes[Index].Offset;
end;



procedure TFlexGrid.SetColOffset(Index: Integer; const Value: Integer);
begin
if (Index<0) or (Index >High(FColSizes)) then raise Exception.Create('Index Out Of Range');
if Value <> FColSizes[Index].Offset then
	begin
	FColSizes[Index].Offset:=Value;
	Invalidate;
	end;
end;


function TFlexGrid.GetRowHeights(Index: Integer): integer;
begin
if (Index<0) or (Index >High(FRowSizes)) then
	begin
	Result:=DefaultRowHeight;
	end
else Result:=FRowSizes[Index].Size;
end;



procedure TFlexGrid.SetColWidths(Index: Integer; Value: integer);
begin
if Value<5 then Value:=5;
if (Index<0) or (Index >High(FColSizes)) then raise Exception.Create('Index Out Of Range');
if Value <> FColSizes[Index].Size then
	begin
	FColSizes[Index].Size:=Value;
	SetSplitsFromSizes(FColSizes);
	Invalidate;
	end;

end;



procedure TFlexGrid.SetRowHeights(Index: Integer; Value: integer);
begin
if Value<5 then Value:=5;
if (Index<0) or (Index >High(FRowSizes)) then raise Exception.Create('Index Out Of Range');
if Value <> FRowSizes[Index].Size then
	begin
	FRowSizes[Index].Size:=Value;
	SetSplitsFromSizes(FRowSizes);
	Invalidate;
	end;
end;

{------------------------------------------------------------------------------}


procedure TFlexGrid.ReadColWidths(Reader: TReader);
var I: Integer;
begin
with Reader do
 begin
 ReadListBegin;
 for I := 0 to ColCount - 1 do ColWidths[I] := ReadInteger;
 ReadListEnd;
 end;
end;



procedure TFlexGrid.ReadRowHeights(Reader: TReader);
var I: Integer;
begin
with Reader do
 begin
 ReadListBegin;
 for I := 0 to RowCount - 1 do RowHeights[I] := ReadInteger;
 ReadListEnd;
 end;
end;



procedure TFlexGrid.WriteColWidths(Writer: TWriter);
var I: Integer;
begin
with Writer do
 begin
 WriteListBegin;
 for I := 0 to ColCount - 1 do WriteInteger(ColWidths[I]);
 WriteListEnd;
 end;
end;



procedure TFlexGrid.WriteRowHeights(Writer: TWriter);
var I: Integer;
begin
with Writer do
 begin
 WriteListBegin;
 for I := 0 to RowCount - 1 do WriteInteger(RowHeights[I]);
 WriteListEnd;
 end;
end;



procedure TFlexGrid.DefineProperties(Filer: TFiler);


	function DoColWidths(G1,G2: TFlexGrid): boolean;
	{Test if we should save column widths}
	var Col,Cnt: integer;
	begin
	Result:=True;
	{Yes if has no ancestor but has descendent /c at least one item to save}
	if (G1=nil) and (G2<>nil) and (G2.ColCount>0) then exit;
	{Test if there is both an ancestor and descendent }
	if (G1<>nil) and (G2<>nil) then
		begin
		{Yes if they have different counts}
	 	if G1.ColCount<>G2.ColCount then exit;
		{Yes if they have same count, but items are different}
		 for Col:=0 to G1.ColCount-1 do
		  if G1.ColWidths[Col]<>G2.ColWidths[Col] then exit;
		end;
	{No for everything else}
	Result:=False;
	end;



	function DoRowHeights(G1,G2: TFlexGrid): boolean;
	{Test if we should save Row Heights}
	var Row,Cnt: integer;
	begin
	Result:=True;
	{Yes if has no ancestor but has descendent /c at least one item to save}
	if (G1=nil) and (G2<>nil) and (G2.RowCount>0) then exit;
	{Test if there is both an ancestor and descendent }
	if (G1<>nil) and (G2<>nil) then
		begin
		{Yes if they have different counts}
	 	if G1.RowCount<>G2.RowCount then exit;
		{Yes if they have same count, but items are different}
		 for Row:=0 to G1.RowCount-1 do
		  if G1.RowHeights[Row]<>G2.RowHeights[Row] then exit;
		end;
	{No for everything else}
	Result:=False;
	end;

begin
inherited DefineProperties(Filer);
Filer.DefineProperty('ColWidths', ReadColWidths, WriteColWidths, DoColWidths(TFlexGrid(Filer.Ancestor),Self));
Filer.DefineProperty('RowHeights', ReadRowHeights, WriteRowHeights, DoRowHeights(TFlexGrid(Filer.Ancestor),Self));
end;


{------------------------------------------------------------------------------}



procedure TFlexGrid.SetBorderStyle(const Value: TBorderStyle);
begin
if FBorderStyle <> Value then
	begin
	FBorderStyle := Value;
	RecreateWnd;
	end;
end;



procedure TFlexGrid.SetCol(const Value: integer);
begin
if Value<>FSelected.X then
	begin
	FSelected.X:=Value;
	Invalidate;
	end;
end;




procedure TFlexGrid.SetRow(const Value: integer);
begin
if Value<>FSelected.Y then
	begin
	FSelected.Y:=Value;
	Invalidate;
	end;
end;

procedure TFlexGrid.SetFixedColor(const Value: TColor);
begin
if FFixedColor <> Value then
	begin
	FFixedColor := Value;
	Invalidate;
	end;
end;




procedure TFlexGrid.SetBackColor(const Value: TColor);
begin
if FBackColor <> Value then
	begin
	FBackColor := Value;
	Invalidate;
	end;
end;




procedure TFlexGrid.SetFixedCols(const Value: Integer);
begin
if FFixedCols <> Value then
	begin
	if Value < 0 then raise Exception.Create('Negative Fixed Column');
	if Value > ColCount then raise Exception.Create('Fixed Column Too Big');
	FFixedCols:=Value;
	Invalidate;
	end;
end;



procedure TFlexGrid.SetFixedRows(const Value: Integer);
begin
if FFixedRows <> Value then
	begin
	if Value < 0 then raise Exception.Create('Negative Fixed Row');
	if Value > RowCount then raise Exception.Create('Fixed Row Too Big');
	FFixedRows:=Value;
	Invalidate;
	end;
end;


procedure TFlexGrid.SetLeftCol(Value: Integer);
begin
if Value<0 then Value:=0;
if Value>(ColCount-FixedCols) then Value:=ColCount-FixedCols;
if  FScrollOffset.X<>Value then
	begin
	FScrollOffset.X:=Value;
	Editor.Visible:=False;
	HorzScrollBar.Position:=Value;
	Invalidate;
	end;
end;



procedure TFlexGrid.SetTopRow(Value: Integer);
begin
if Value<0 then Value:=0;
if Value>VertScrollBar.Max then Value:=VertScrollBar.Max;
if  FScrollOffset.Y<>Value then
	begin
	FScrollOffset.Y:=Value;
	Editor.Visible:=False;
	VertScrollBar.Position:=Value;
	Invalidate;
	end;
end;



procedure TFlexGrid.DoHorzChange(Sender: TObject);
begin
inherited;
LeftCol:=Self.HorzScrollBar.Position;
end;



procedure TFlexGrid.DoVertChange(Sender: TObject);
begin
inherited;
TopRow:=VertScrollBar.Position;
end;



procedure TFlexGrid.CMShowingChanged(var Message: TMessage);
begin
inherited;
if Showing then SetScrollBarParams;
end;


procedure TFlexGrid.CMDesignHitTest(var Message: TCMDesignHitTest);
{This message sends mouse information and allows the program
to control when mouse messages get sent to the program at
design time. Returning 1 indicates that the mouse event messages
should be sent to the normal message handlers, 0 indicates on messages
should be sent. }
var Col: integer;
var Sizing: boolean;
begin
{Allow design interaction under three conditions:
1) Mouse over split - to allow sizing
2) Mouse down and moving split - to allow sizing
3) cursor not default - to cursor to return to normal}
Sizing:=ColSizerHitTest(Message.Pos.X, Message.Pos.Y,Col) or
        (MouseDownState=mdColSize) or (FCursor<>crDefault);
Message.Result := Longint(BOOL(Sizing));
end;


procedure TFlexGrid.WMSetCursor(var Msg: TWMSetCursor);
begin
SetCursor(Screen.Cursors[FCursor]);
end;




procedure TFlexGrid.Resize;
begin
inherited;
SetScrollBarParams;
Invalidate;
end;



procedure TFlexGrid.ColumnMoved(FromIndex, ToIndex: Integer);
begin
if Assigned(FOnColumnMoved) then FOnColumnMoved(Self, FromIndex, ToIndex);
end;



procedure TFlexGrid.RowMoved(FromIndex, ToIndex: Integer);
begin
if Assigned(FOnRowMoved) then FOnRowMoved(Self, FromIndex, ToIndex);
end;


function TFlexGrid.CanEditAcceptKey(Key: Char): Boolean;
begin
Result := True;
end;


function TFlexGrid.CanEditModify: Boolean;
begin
Result := FCanEditModify;
end;



procedure TFlexGrid.DoExit;
begin
inherited DoExit;
if not (goAlwaysShowEditor in Options) then HideEditor;
end;



procedure TFlexGrid.SetFieldText(ACol, ARow: integer; Value: string);
{Dynamic method to filled in by descendent Objects}
begin
end;



function TFlexGrid.DoMouseWheelDown(Shift: TShiftState;
  MousePos: TPoint): Boolean;
begin
inherited DoMouseWheelDown(Shift,MousePos);
TopRow:=TopRow+1;
Invalidate;
Result:=True;
end;



function TFlexGrid.DoMouseWheelUp(Shift: TShiftState;
  MousePos: TPoint): Boolean;
begin
inherited DoMouseWheelUp(Shift,MousePos);
TopRow:=TopRow-1;
Invalidate;
Result:=True;
end;



function TFlexGrid.CanColMoveStart(Col: integer): boolean;
begin
Result:=True;
if Assigned(FOnColMoveStart) then FOnColMoveStart(Self,Col,Result);
end;



function TFlexGrid.CanColMoveOver(Col: integer): boolean;
begin
Result:=True;
if Assigned(FOnColMoveOver) then FOnColMoveOver(Self,Col,Result);
end;



function TFlexGrid.CanColMoveDrop(Col: integer): boolean;
begin
Result:=True;
if Assigned(FOnColMoveDrop) then FOnColMoveDrop(Self,Col,Result);
end;





end.
