unit Fxcombo;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  TFXCustomCombo = class(TCustomComboBox)
  private
  procedure BackErase(var Msg: TWMEraseBKGnd); message WM_ERASEBKGND;
  protected
    { Protected declarations }
  public
    { Public declarations }
  published
   property ItemIndex;
   property Align;
  end;



type TFXCombo = class(TFXCustomCombo)
  published
    property AutoComplete default True;
    property AutoDropDown default False;
    property BevelEdges;
    property BevelInner;
    property BevelKind default bkNone;
    property BevelOuter;
    property Style; {Must be published before Items}
    property Anchors;
    property BiDiMode;
    property CharCase;
    property Color;
    property Constraints;
    property Ctl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property ItemIndex default -1;
    property MaxLength;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnCloseUp;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnSelect;
    property OnStartDock;
    property OnStartDrag;
    property Items; { Must be published after OnMeasureItem }
end;



type TCheckedCombo = class(TFXCombo)
 private
   FShowCheckBoxes: boolean;
    FComboListInstance: Pointer;
    FOldComboListProc: Pointer;
    FLeaveListDown: boolean;
    FOnCheckBoxChange: TNotifyEvent;
    FFOnCheckBoxChange: TNotifyEvent;
   function GetChecked(Index: integer): boolean;
   procedure SetChecked(Index: integer; const Value: boolean);
   procedure SetShowCheckBoxes(const Value: boolean);
    procedure SetDropDownWidth;
    procedure SubClassCombo;
    procedure ComboListProc(var Message: TMessage);
    function ComboListHandle(Combo: TComboBox): HWND;
    procedure UnSubClassCombo;
 protected
   procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
   procedure Select; override;
   procedure SetStyle(Value: TComboBoxStyle); override;
   procedure DropDown; override;
   procedure DoEnter; override;
   procedure DoExit; override;
 public
  constructor Create(AOwner: TComponent); override;
  property Checked[Index: integer]: boolean read GetChecked write SetChecked;
 published
  property ShowCheckBoxes: boolean read FShowCheckBoxes write SetShowCheckBoxes default True;
  property LeaveListDown: boolean read FLeaveListDown write FLeaveListDown default False;
  property OnCheckBoxChange: TNotifyEvent read FFOnCheckBoxChange write FFOnCheckBoxChange;
 end;



procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Fountain', [TFXCombo]);
RegisterComponents('Fountain', [TCheckedCombo]);
end;



procedure TFXCustomCombo.BackErase(var Msg: TWMEraseBKGnd);
begin
inherited;
FillRect(Msg.DC, ClientRect, Parent.Brush.Handle);
end;


{ TCheckedCombo }


procedure TCheckedCombo.SetStyle(Value: TComboBoxStyle);
begin
if ShowCheckBoxes and (Value<>csOwnerDrawFixed) then exit;
inherited;
end;




procedure TCheckedCombo.SetShowCheckBoxes(const Value: boolean);
begin
if FShowCheckBoxes<>Value then
	begin
	FShowCheckBoxes := Value;
	if Value then Style:=csOwnerDrawFixed;
	end;
end;



function TCheckedCombo.GetChecked(Index: integer): boolean;
begin
Result:=Boolean(Items.Objects[Index]);
end;



procedure TCheckedCombo.SetChecked(Index: integer; const Value: boolean);
begin
Items.Objects[Index]:=Pointer(Value);
if assigned(FFOnCheckBoxChange) then FFOnCheckBoxChange(Self);
end;



procedure TCheckedCombo.Select;
begin
inherited;
Checked[ItemIndex] := not Checked[ItemIndex];
end;





procedure TCheckedCombo.SetDropDownWidth;
{Set drop-down width so it handles widest list item}
const Padding = 3;
var DropWidth, ItemWidth: integer;
var I: integer;
begin
DropWidth := 0;
{ get the width of the widest item in the list}
for I := 0 to Items.Count-1 do
	begin
	ItemWidth := Self.Canvas.TextWidth(Items[I]);
	Inc(itemWidth, 2 * Padding);
	if (itemWidth > DropWidth) then DropWidth := itemWidth;
	end;
{ set the width of drop down if needed }
if (DropWidth > Self.Width) then
	begin
	{ Add the scrollbar width if there is a scroll bar}
	if Self.DropDownCount < Items.Count then DropWidth := DropWidth + GetSystemMetrics(SM_CXVSCROLL);
	if ShowCheckBoxes then Inc(DropWidth, GetSystemMetrics(SM_CXMENUCHECK)+ Padding);
	SendMessage(Self.Handle, CB_SETDROPPEDWIDTH, DropWidth, 0);
	end;
end;


procedure TCheckedCombo.DropDown;
begin
inherited;
SetDropDownWidth;
end;



constructor TCheckedCombo.Create(AOwner: TComponent);
begin
inherited;
LeaveListDown:=False;
ShowCheckBoxes:=True;
end;



procedure TCheckedCombo.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
if not ShowCheckBoxes or (odComboBoxEdit in State) then
	begin
	inherited;
	exit;
	end;
TControlCanvas(Self.Canvas).UpdateTextFlags;
if Assigned(OnDrawItem) then OnDrawItem(Self, Index, Rect, State)
else
	begin
	Canvas.FillRect(Rect);
	Rect.Left := Rect.Left + 1;
	Rect.Right := Rect.Left + 13;
	Rect.Bottom := Rect.Bottom;
	Rect.Top := Rect.Top;

	if not (odSelected in State) and (Checked[Index]) then DrawFrameControl(Canvas.Handle, Rect, DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_CHECKED or DFCS_FLAT)
	else if (odFocused in State) and (Checked[Index]) then DrawFrameControl(Canvas.Handle, Rect, DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_CHECKED or DFCS_FLAT)
	else if (Checked[Index]) then DrawFrameControl(Canvas.Handle, Rect, DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_CHECKED or DFCS_FLAT)
	else if not (Checked[Index]) then DrawFrameControl(Canvas.Handle, Rect, DFC_BUTTON, DFCS_BUTTONCHECK or DFCS_FLAT);
	Canvas.TextOut(Rect.Left + GetSystemMetrics(SM_CXMENUCHECK), Rect.Top, Self.Items[Index]);
	end;
end;


{------------------------------------------------------------------------------}


function TCheckedCombo.ComboListHandle(Combo: TComboBox): HWND;
var Info: TComboBoxInfo;
begin
Assert(Assigned(Combo));
Info.cbSize := SizeOf(Info);
GetComboBoxInfo(Combo.Handle, Info);
Result := Info.hwndList;
end;



procedure TCheckedCombo.ComboListProc(var Message: TMessage);
var Res,Index: Integer;
var  Inside: Boolean;
begin
if LeaveListDown then
case Message.Msg of
 WM_LBUTTONDOWN, WM_LBUTTONUP, WM_LBUTTONDBLCLK:
	begin
	Res := SendMessage(ComboListHandle(TComboBox(Self)), LB_ITEMFROMPOINT, 0, Message.LParam);
	Inside := Res and $FFFF0000 = 0;
	Index := Res and $FFFF;
	if Inside then
		begin
		case Message.Msg of
		 WM_LBUTTONDOWN:
			begin
			Checked[Index] := not Checked[Index];
                 	Text:=Items[Index];
			InvalidateRect(ComboListHandle(TComboBox(Self)), nil, False);
			UpdateWindow(ComboListHandle(TComboBox(Self)));
			end;
                 WM_LBUTTONUP:
                 	begin
                 	Text:=Items[Index];
                 	Repaint;
			if CanFocus then SetFocus; 
                 	end;
                 end;
		exit;
		end;
	end;
 end;
with Message do
 Result := CallWindowProc(FOldComboListProc, ComboListHandle(TComboBox(Self)), Msg, WParam, LParam);
end;




procedure TCheckedCombo.SubClassCombo;
begin
FComboListInstance := MakeObjectInstance(ComboListProc);
FOldComboListProc := Pointer(SetWindowLong(ComboListHandle(TComboBox(Self)), GWL_WNDPROC, Integer(FComboListInstance)));
end;




procedure TCheckedCombo.UnSubClassCombo;
begin
SetWindowLong(ComboListHandle(TComboBox(Self)), GWL_WNDPROC, Integer(FOldComboListProc));
FreeObjectInstance(FComboListInstance);
end;




procedure TCheckedCombo.DoEnter;
begin
inherited;
SubClassCombo;
end;

procedure TCheckedCombo.DoExit;
begin
inherited;
UnSubClassCombo;
end;

end.
