unit GPSReader2;

interface

uses Windows,Forms, Messages, SysUtils, Classes,SyncObjs,MMSystem,DSupport, CommPort;

type TArgArray = array of string;

type TTimeTickEvent = procedure(DateTime: TDateTime; Valid: boolean) of object;

type TGeoReference = record
 Longitude,Latitude: double;
 Altitude: double;
 SatelliteCount: integer;
 Quality: integer;
 end;

var BusyFlag: TCriticalSection;


type TGPSReader2 = class(TCommPort)
private
  FCommandString: string;
  FDateTime: TDateTime;
  FValidPos: boolean;
  FLastValidTime: DWord;
  FOnTimeTick: TTimeTickEvent;
  FGeoReference: TGeoReference;
  FReading: boolean;
  procedure DecodeCommands(S: string);
  procedure DecodeTime(ArgArray: TArgArray; CharCnt: integer; var DateTime: TDateTime; var Valid: boolean);
  procedure DecodeGeoReference(ArgArray: TArgArray; var Geo: TGeoReference; var Valid: boolean);
  procedure ExtractArguments(Com: string; var ArgArray: TArgArray);
  procedure StoreCommandString(Str: string);
  function GetValidTime: boolean;
  procedure DoTimeTick;
  function CharsToTime(C: integer): double;
protected
  procedure ReceiveData(Sender: TObject; Count: Word);
public
 constructor Create(AOwner: TComponent); override;
 function GetDateTime(var DateTime: TDateTime): boolean;
published
 property ValidTime: boolean read GetValidTime;
 property OnTimeTick: TTimeTickEvent read FOnTimeTick write FOnTimeTick;
 property GeoReference: TGeoReference read FGeoReference;
 property Reading: boolean read FReading;
end;

procedure Register;

implementation

procedure Register;
begin
RegisterComponents('Fountain', [TGPSReader2]);
end;


constructor TGPSReader2.Create(AOwner: TComponent);
begin
inherited;
OnReceive:=ReceiveData;
Events:=[tceRxChar];
FReading:=False;
FGeoReference.SatelliteCount:=0;
end;



procedure TGPSReader2.ExtractArguments(Com: string; var ArgArray: TArgArray);
var I: integer;
var S: string;
begin
I:=1;
while true do
	begin
	S:=ExtractToken(Com,',',I);
	if S='' then break;
	SetLength(ArgArray,Length(ArgArray)+1);
	ArgArray[High(ArgArray)]:=S;
	end;
end;



function TGPSReader2.CharsToTime(C: integer): double;
{Calculate the time delay for specified number of character}
{Used to find the time delay for the GPS data}
var CT: double;
begin
CT:=1/(GetBaudRateValue(BaudRate)/10);
Result:=C * CT;
end;


// $GPRMC,032141,A,3941.4837,N,10459.1376,W,000.0,313.9,100108,010.0,E*6D

procedure TGPSReader2.DecodeTime(ArgArray: TArgArray; CharCnt: integer; var DateTime: TDateTime; var Valid: boolean);
var S: string;
var Year,Month,Day: integer;
var Hours,Min,Sec: integer;
var Delays: double;
const TimeSlice = 0.018;
const SecondsPerDay = 24 * 60 * 60;
begin
Valid:=False;
if High(ArgArray)<9 then exit;
Valid:=ArgArray[2]='A';
S:=ArgArray[1,1]; S:=S+ArgArray[1,2];
Hours:=StrToIntDef(S,0);
S:=ArgArray[1,3]; S:=S+ArgArray[1,4];
Min:=StrToIntDef(S,0);
S:=ArgArray[1,5]; S:=S+ArgArray[1,6];
Sec:=StrToIntDef(S,0);

S:=ArgArray[9,1]; S:=S+ArgArray[9,2];
Day:=StrToIntDef(S,0);
S:=ArgArray[9,3]; S:=S+ArgArray[9,4];
Month:=StrToIntDef(S,0);
S:=ArgArray[9,5]; S:=S+ArgArray[9,6];
Year:=StrToIntDef(S,0)+2000;
DateTime:=EncodeDate(Year,Month,Day);

{Milliseconds is alway zero because time always sent on the second}
DateTime:=DateTime + EncodeTime(Hours,Min,Sec,0);

{Add in all delays}
Delays:=CharsToTime(CharCnt)+TimeSlice*4;
DateTime:=DateTime + Delays/SecondsPerDay;
end;

//$GPGGA,032141,3941.4837,N,10459.1376,W,2,06,2.0,1615.9,M,-21.2,M,,*49

procedure TGPSReader2.DecodeGeoReference(ArgArray: TArgArray; var Geo: TGeoReference; var Valid: boolean);
var S: string;
var Deg,Min: double;
begin
Valid:=False;
Geo.SatelliteCount:=0;
if High(ArgArray)<9 then exit;
S:=Copy(ArgArray[2],1,2);
Deg:=StrToFloatDef(S,0);
S:=Copy(ArgArray[2],3,7);
Min:=StrToFloatDef(S,0);
Geo.Latitude:=Deg + (Min/60);
if ArgArray[3]='S' then Geo.Latitude:=-Geo.Latitude;

S:=Copy(ArgArray[4],1,3);
Deg:=StrToFloatDef(S,0);
S:=Copy(ArgArray[4],4,7);
Min:=StrToFloatDef(S,0);
Geo.Longitude:=Deg + (Min/60);
if ArgArray[5]='W' then Geo.Longitude:=-Geo.Longitude;

Geo.Quality:=StrToIntDef(ArgArray[6],0);
Geo.SatelliteCount:=StrToIntDef(ArgArray[7],0);
Geo.Altitude:=StrToFloatDef(ArgArray[9],0);
Valid:=True;
end;



function CalculateCheckSum(S: string): byte;
var I: integer;
begin
Result:=0;
for I := 2 to Length(S) do
  begin
  if S[I]='*' then break;
  Result:=Result xor byte(S[I]);
  end;
end;


function ValidChecksum(S: string): boolean;
var CS1,CS2: byte;
var S1: string;
var Inx: integer;
begin
Result:=False;
CS1:=CalculateChecksum(S);
Inx:=Pos('*',S);
if Inx<1 then exit;
S1:='$'+Copy(S,Inx+1,2);
CS2:=StrToIntDef(S1,0);
Result:=CS1=CS2;
end;



procedure TGPSReader2.DecodeCommands(S: string);
var ArgArray: TArgArray;
begin
FValidPos:=ValidChecksum(S);
if not FValidPos  then exit;
SetLength(ArgArray,0);
ExtractArguments(S,ArgArray);
if ArgArray[0]='$GPRMC' then
	begin
	DecodeTime(ArgArray,Length(S)+1,FDateTime,FValidPos);
	FLastValidTime:=timeGetTime;
	DoTimeTick;
	end
else if ArgArray[0]='$GPGGA' then
	begin
	DecodeGeoReference(ArgArray, FGeoReference,FValidPos);
	end;
end;



procedure TGPSReader2.StoreCommandString(Str: string);
{Store Command string data and decode if a complete }
{ command packet has been received.}
var S,E: integer;
var Com: string;
begin
FCommandString:=FCommandString+Str;
while true do
	begin
	S:=Pos('$',FCommandString);
	if S>0 then Delete(FCommandString,1,S-1)
	else begin FCommandString:=''; break; end;

	E:=Pos(CR_Char,FCommandString);
	if E=0 then break;
	Com:=Copy(FCommandString,1,E);
	Delete(FCommandString,1,E);
	DecodeCommands(Com);
	end;
end;



procedure TGPSReader2.ReceiveData(Sender: TObject; Count: Word);
var SS: string;
begin
FReading:=True;
BusyFlag.Acquire;
try
ReadString(SS,Count);
StoreCommandString(SS);
finally
 FReading:=False;
 BusyFlag.Release;
 end;
end;



function TGPSReader2.GetDateTime(var DateTime: TDateTime): boolean;
const MSPerDay = 24 * 60 * 60 * 1000;
begin
BusyFlag.Acquire;
try
DateTime:=FDateTime+(timeGetTime-FLastValidTime)/MSPerDay;
Result:=ValidTime;
finally BusyFlag.Release; end;
end;



function TGPSReader2.GetValidTime: boolean;
var DT: DWord;
begin
DT:=timeGetTime-FLastValidTime;
Result:=(DT<60000) and FValidPos and (FGeoReference.SatelliteCount>0);
end;



procedure TGPSReader2.DoTimeTick;
var DT: TDateTime;
var Valid: boolean;
begin
if Assigned(FOnTimeTick) then
	begin
	Valid:=GetDateTime(DT);
	FOnTimeTick(DT,Valid);
	end;
end;


initialization
BusyFlag:=TCriticalSection.Create;
finalization
BusyFlag.Free;
end.
