unit Graphbox;

interface

uses DSupport,Math,Vector,ExtCtrls,Classes,Windows,Graphics,
     SysUtils,Controls,Forms,Menus,Clipbrd,Printers,Buttons,
     ImageLib, PaintPan, GraphicsLib, HugeArry, Autobttn,Dialogs;




{------------------- Declarations For Custom Graph Component --------------}

type
 TDataPoint=record
 X,Y,Width: double;
 Color: TColor;
 Group: integer;
 Text: string;
 end;

const MaxGroups = 1024;

type TGroupArray = array [-MaxGroups..MaxGroups] of integer;


type
  TCustomGraph = class(TCustomPanel)
 private
  BackBitmap: TBitmap;
  FDrawLastPixel: boolean;
  FBlockRefresh: boolean;
  procedure SetGridColor(V: TColor);
  procedure SetDecimals1(V: integer);
  procedure SetDecimals2(V: integer);
  procedure SetBorder(V: integer);
  procedure GetImageSize(var ImageWidth,ImageHeight: integer;var ImageRect: TRect);
  function GetCount: integer;
  function GetGraphData(Index: Cardinal): TDataPoint;
  procedure SetGraphData(Index: Cardinal; const Value: TDataPoint);
 protected
  FDefaultData: boolean;
  FGridColor: TColor;
  FDecimals1: integer;
  FDecimals2: integer;
  FGraphData: TList;
  GroupCount: TGroupArray;
  FGraphBorder,LeftSide,RightSide,TopSide,BottomSide: integer;
  function GetBorderWidth(TextSize: integer): integer;
  procedure SetData(X,Y,Width: double; Group: integer; Text: string; Color: TColor);
  procedure GetModuloRange(Low,High,Modulo: double;var S,E:double; var N: integer);
  procedure DoGraph(Canvas: TCanvas; AWidth,AHeight: integer); overload; virtual;
  procedure DoGraph(Canvas: TCanvas); overload; virtual;
  procedure Paint; override;
  procedure SetDefaultData(V: boolean); virtual; abstract;
  procedure Resize; override;
  property DrawLastPixel: boolean read FDrawLastPixel write FDrawLastPixel default False;
  property DefaultData: boolean read FDefaultData write SetDefaultData;
 public
  MinValue,MaxValue: T2DVector;
  WrkCanvas: TCanvas;
  property Canvas;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure ClearData;
  procedure CopyToClipboard;
  procedure SaveToFile(Name: string; AWidth,AHeight: integer); overload;
  procedure SaveToFile(Name: string); overload;
  procedure SaveToMetaFile(Name: string; Enhanced: boolean); overload;
  procedure SaveToMetaFile(Name: string; Enhanced: boolean; AWidth,AHeight: integer); overload;
  procedure MetafileToClipboard(Enhanced: boolean);
  procedure PrintByMetaFile;
  procedure BeginUpdate;
  procedure EndUpdate;
  procedure Restore;
  property Count: integer read GetCount;
  property GraphData[Index: Cardinal]: TDataPoint read GetGraphData write SetGraphData;
  property BlockRefresh: boolean read FBlockRefresh write FBlockRefresh default False;
 published
  property GridColor: TColor read FGridColor write SetGridColor default clBlack;
  property GraphBorder: integer read FGraphBorder write SetBorder default 20;
  property Decimals1: integer read FDecimals1 write SetDecimals1 default 0;
  property Decimals2: integer read FDecimals2 write SetDecimals2 default 0;
 end;




{------------------- Declarations For Axis Parameters Object --------------}

type TModulo = 1..High(integer);

type TLabelType = (ltPrimary,ltSecondary,ltBoth);

type
 TAxisParams = class(TPersistent)
 private
  FOnChanged: TNotifyEvent;
  FMultiplier: double;
  FCycle: double;
  FOffset: double;
  FLines: integer;
  FShowLabels: boolean;
  FMinorLabels: boolean;
  FLog: boolean;
  FUnits: string;
  FLabelModulo: TModulo;
  FExponentLabels: boolean;
  FMinorLines: boolean;
  FLabelType: TLabelType;
  FUnsigned: boolean;
  FWrap: boolean;
  procedure ReadMax(Reader: TReader);
  procedure WriteMax(Writer:TWriter) ;
  procedure ReadMin(Reader: TReader);
  procedure WriteMin(Writer:TWriter) ;
  procedure RangeCheckMinMax;
  procedure SetLog(V: boolean);
  procedure SetMinorLabels(V: boolean);
  procedure SetMin(V: double);
  procedure SetMax(V: double);
  procedure SetMultiplier(V: double);
  procedure SetLines(V: integer);
  procedure SetShowLabels(V: boolean);
  procedure SetUnits(S: string);
  procedure SetLabelModulo(const Value: TModulo);
  procedure SetExponentLabels(const Value: boolean);
  procedure SetMinorLines(const Value: boolean);
  procedure SetOffset(const Value: double);
  function ApplyScaleOffset(V: double): double;
  procedure SetCycle(const Value: double);
  procedure SetLabelType(const Value: TLabelType);
  procedure SetUnsigned(const Value: boolean);
  procedure Change;
  procedure SetWrap(const Value: boolean);
 protected
  FMin,FMax: double;
  procedure DefineProperties(Filer: TFiler); override;
 public
  constructor Create;
  procedure SetMinMax(Min,Max: double);
 published
  property ExponentLabels: boolean read FExponentLabels write SetExponentLabels;
  property Log: boolean read FLog write SetLog;
  property MinorLabels: boolean read FMinorLabels write SetMinorLabels;
  property MinorLines: boolean read FMinorLines write SetMinorLines;
  property Min: double read FMin write SetMin stored false;
  property Max: double read FMax write SetMax stored false;
  property Multiplier: double read FMultiplier write SetMultiplier;
  property Offset: double read FOffset write SetOffset;
  property Lines: integer read FLines write SetLines;
  property ShowLabels: boolean read FShowLabels write SetShowLabels;
  property LabelType: TLabelType read FLabelType write SetLabelType;

  property Units: string read FUnits write SetUnits;
  property LabelModulo: TModulo read FLabelModulo write SetLabelModulo default 1;
  property Cycle: double read FCycle write SetCycle;
  property Unsigned: boolean read FUnsigned write SetUnsigned;
  property Wrap: boolean read FWrap write SetWrap;
 end;

{------------------- Declarations for graph title Object --------------}

type TTitlePosition = (tpCenter,tpLeft,tpRight);


type
 TGraphTitle = class(TPersistent)
 private
  FOnChanged: TNotifyEvent;
  FCaption: string;
  FShowCaption: boolean;
  FBoxed: boolean;
  FPosition: TTitlePosition;
  procedure SetShowCaption(V: boolean);
  procedure SetCaption(S: string);
  procedure SetBoxed(V: boolean);
  procedure SetPosition(V: TTitlePosition);
 protected
 public
 published
  property Caption: string read FCaption write SetCaption;
  property ShowCaption: boolean read FShowCaption write SetShowCaption;
  property Boxed: boolean read FBoxed write SetBoxed;
  property Position: TTitlePosition read FPosition write SetPosition;
 end;



{------------------- Declarations For GraphBox Component --------------}


type
 TGraphType = (gtLine,gtHorzBar,gtVertBar);

type TExponential = record
 Base,Exp: integer;
 end;

type TLogProcedure = procedure(E: TExponential; Major: boolean) of object;


type TGraphRange = record
 Min,Max: double;
 end;

type TGraphExtents = record
 Vertical,Horizontal: TGraphRange;
 end;

type TLabelEvent = function(Value: double): string of object;

type TCustomGraphBox = class(TCustomGraph)
 private
  FGraphType: TGraphType;
  FXAxis,FYAxis: TAxisParams;
  FTitle: TGraphTitle;

  Extents: TGraphExtents;
  MaxX,MaxY,XOrigin,YOrigin : Integer;
  XScale,YScale,XOffset,YOffset : double;
  FOnPaint: TNotifyEvent;
    FOnHorizLabel: TLabelEvent;
    FOnVertLabel: TLabelEvent;
  procedure SetGraphType(V: TGraphType);

  procedure DrawVertBarGraph;
  procedure DrawHorzBarGraph;
  procedure DrawLineGraph;
  procedure DrawGraph;
  procedure SetGraphSize(AWidth,AHeight: integer); virtual;
  procedure UpdateGraph(AWidth,AHeight: integer);
  procedure ForceInvalidate(Sender: TObject);
  function GetMaxTextWidth: integer;
  procedure DisplayTitle(AWidth: integer);
  function DrawGroup(Group: integer): integer;
  procedure VertGridCallback(E: TExponential; Major: boolean);
  procedure DoLogSteps(Start,Stop: double; LogProc: TLogProcedure);
  procedure VertLabelsCallback(E: TExponential; Major: boolean);
  procedure HorizGridCallback(E: TExponential; Major: boolean);
  procedure HorizLabelsCallback(E: TExponential; Major: boolean);
  procedure HorizLogGrid(Start,Stop: double);
  procedure GetLogsValues(var X,Y: double);
  function GetWidthStr(Axis: TAxisParams; V: double): string;
  procedure LabelPoint(Data: TDataPoint);
  function GetHorizBarRect(X, Y, Width: double): T2DRect; dynamic;
  procedure SetScales(StartX, StartY, StopX, StopY: double);
    procedure DrawGraphBorder;
    procedure SetLeftRightBorders(AWidth: integer);
    procedure SetTopBottomBorders(AHeight: integer);
    procedure DrawGraphLine(Line: T2DLine; Color: TColor);
    procedure MarkDataPoint(P: T2DVector; Size: double; BorderColor,CenterColor: TColor);
    procedure DoResetAxis(A: TAxisParams; Range: TGraphRange);
 protected
  FFixedScaleX,FFixedScaleY: boolean;
  procedure DoGraph(Canvas: TCanvas; AWidth,AHeight: integer); overload; override;
  procedure DoGraph(Canvas: TCanvas); overload; override;
  procedure Paint; override;
  procedure HandleAxisChange(Sender: TObject); dynamic;
  procedure SetDefaultData(V: boolean); override;
  property GraphType: TGraphType read FGraphType write SetGraphType default gtHorzBar;
  property XAxis: TAxisParams read FXAxis write FXAxis;
  property YAxis: TAxisParams read FYAxis write FYAxis;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  procedure DoMinMax;
  procedure ScaledRectangle(X1,Y1,X2,Y2: double; C: TColor); overload;
  procedure ScaledRectangle(Rect: T2DRect; C: TColor); overload;
  procedure ScaledFrameRectangle(X1, Y1, X2, Y2: double; C: TColor);
  function GetScaledPoint(X,Y: double): TPoint;
  procedure OrgLine(X,Y: integer;C: TColor);
  procedure OrgMove(X,Y: integer);
  procedure ScaledLine(X,Y: double; C: TColor);
  procedure ScaledMove(X,Y: double);
  procedure DrawPoint(X,Y: double; Radius: integer; BorderColor,CenterColor: TColor);
  procedure VerticalGrid(LineCount: integer; Start,Stop: double);
  procedure VerticalLogGrid(Start,Stop: double);
  procedure HorizontalGrid(Y: integer; Start,Stop: double);
  procedure XYToScaled(X,Y: integer; var X2,Y2: double);
  procedure PanGraph(Mult: double; Horizontal: boolean);
  procedure ZoomGraph(Zoom: double; Horizontal: boolean);
  procedure SetBaseZoomPan;
  procedure ResetPanZoom(Horizontal: boolean);

  procedure GetHorizontalRange(var Start,Finish: double; var LineCount: integer);
  procedure GetVerticalRange(var Start,Finish: double; var LineCount: integer);
  procedure SetHorizontalRange(Start,Finish: double; LineCount: integer);
  procedure SetVerticalRange(Start,Finish: double; LineCount: integer);
  procedure SetModHorizontalRange(Start,Finish,Width: double);
  procedure SetModVerticalRange(Start,Finish,Width: double);
  procedure AdjustHorizontalRange(Modulo: boolean; Spacing: double);
  procedure AdjustVerticalRange(Modulo: boolean; Spacing: double);
  procedure AdjustRange(Modulo: boolean; Spacing: double);
  procedure AutoScaleVertical(MaxLines: integer);
  procedure AutoScaleHorizontal(MaxLines: integer);
  procedure SetDataPoint(X,Y,Width: double; Group: integer; Color: TColor); overload;
  procedure SetDataPoint(X,Y,Width: double; Group: integer; Text: string; Color: TColor); overload;
 published
  property Title: TGraphTitle read FTitle write FTitle;
  property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
  property OnVertLabel: TLabelEvent read FOnVertLabel write FOnVertLabel;
  property OnHorizLabel: TLabelEvent read FOnHorizLabel write FOnHorizLabel;
end;

type TGraphBox = class(TCustomGraphBox)
 private
 protected
 public
 published
  property GraphType;
  property XAxis;
  property YAxis;
  property DrawLastPixel;
  property DefaultData;

  property Align;
  property Anchors;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BorderWidth;
  property BorderStyle;
  property Constraints;
  property DragCursor;
  property DragMode;
  property Enabled;
  property Color;
  property Ctl3D;
  property Font;
  property Locked;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
 end;


{------------------- Declarations For Bar-Meter Component --------------}

type TBarMeter = class(TCustomGraphBox)
 private
  FPosition: double;
  FBarColor: TColor;
  FBanded: boolean;
  FCentered: boolean;
  FCenter: double;
  FBandSize: integer;
  FRightLabel: string;
  FLeftLabel: string;
  function GetHorizBarRect(X, Y, Width: double): T2DRect; override;
  procedure SetPosition(const Value: double);
  procedure SetBarColor(const Value: TColor);
  procedure SetGraphSize(AWidth,AHeigh: integer); override;
  procedure SetLeftLabel(const Value: string);
  procedure SetRightLabel(const Value: string);
  procedure SetBanded(const Value: boolean);
  procedure SetCenter(const Value: double);
  procedure SetCentered(const Value: boolean);
  procedure SetBandSize(const Value: integer);
 protected
  property YAxis;
  property GraphType;
  procedure Paint; override;
 public
  constructor Create(AOwner: TComponent); override;
 published
  property BarColor: TColor read FBarColor write SetBarColor;
  property Position: double read FPosition write SetPosition;
  property BandSize: integer read FBandSize write SetBandSize default 10;
  property Banded: boolean read FBanded write SetBanded default True;
  property Centered: boolean read FCentered write SetCentered default False;
  property CenterPos: double read FCenter write SetCenter;
  property LeftLabel: string read FLeftLabel write SetLeftLabel;
  property RightLabel: string read FRightLabel write SetRightLabel;
  property XAxis;
  property Align;
  property Anchors;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BorderWidth;
  property BorderStyle;
  property Constraints;
  property DragCursor;
  property DragMode;
  property Enabled;
  property Color;
  property Ctl3D;
  property Font;
  property Locked;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
 end;


{------------------- Declarations For RoseDiagram Component --------------}

const RC=57.2957795131;


type
 TRoseType = (rtLine,rtRose);
 TQuadrants = (qdNW,qdNE,qdSE,qdSW);
 TQuadSet = set of TQuadrants;


type
  TRoseDiagram = class(TCustomGraph)
private
  FAngleUnits,FRadiusUnits: string;
  FQuadrants: TQuadset;
  FRoseType: TRoseType;
  FRadiusLabels: boolean;

  FRadiusCount: integer;
  FRadiusMin,FRadiusMax: double;

  FAngleCount: integer;
  FAngleMin,FAngleMax,FAngleOffset: double;

  AngleBase,AngleOffset,AngleScale: double;
  RadiusBase,RadiusScale: double;

  GraphRect: TRect;
  Radius,XCenter,YCenter : Integer;
  procedure SetQuadrants(V: TQuadSet);
  procedure SetRoseType(V: TRoseType);
  procedure SetAngleUnits(V: String);
  procedure SetRadiusUnits(V: string);
  procedure SetRadiusLabels(V: boolean);

  procedure DrawCircle(CX,CY,Radius: integer;C: TColor);
  procedure DrawSlice(A1,A2: double; R: integer;C: TColor);
  procedure DrawScaledSlice(R,A,W: double;C: TColor);
  procedure PolarToCartesean(R,A:double; var X,Y: integer);
  function ScaleAngle(A: double): double;
  function ScaleRadius(R: double): double;

  procedure DrawRoseGraph;
  procedure DrawLineGraph;
  procedure DrawGraph;
  procedure SetGraphSize(AWidth,AHeight: integer);
  procedure UpdateGraph;
protected
  procedure Paint; override;
  procedure DoGraph(Canvas: TCanvas; AWidth,AHeight: integer); overload; override;
  procedure DoGraph(Canvas: TCanvas); overload; override;
  procedure SetDefaultData(V: boolean); override;
public
  constructor Create(AOwner: TComponent); override;

  function GetScaledPoint(R,A:double): TPoint;
  procedure PolarLine(R,A:double; C: TColor);
  procedure PolarMove(R,A:double);
  procedure ScaledLine(R,A:double;C: TColor);
  procedure ScaledMove(R,A:double);
  procedure RadiusGrid(Count: integer; R1,R2: double);
  procedure AngleGrid(Count: integer; R1,R2,Off: double);

  procedure SetAngleRange(Start,Finish,Offset: double; LineCount: integer);
  procedure SetRadiusRange(Start,Finish: double; LineCount: integer);
  procedure SetModRadiusRange(Start,Finish,Width: double);
  procedure SetDataPoint(Angle,Radius,Width: double;Color: TColor);
  procedure CarteseanToPolar(X,Y: integer;var R,A:double);

published
  property AngleUnits: string read FAngleUnits write SetAngleUnits;
  property RadiusUnits: string read FRadiusUnits write SetRadiusUnits;
  property Quadrants: TQuadSet read FQuadrants write SetQuadrants default [qdNW,qdNE,qdSE,qdSW];
  property RoseType: TRoseType read FRoseType write SetRoseType default rtRose;
  property RadiusLabels: boolean read FRadiusLabels write SetRadiusLabels default true;
  property DrawLastPixel;
  property DefaultData;
  property Align;
  property Anchors;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BorderWidth;
  property BorderStyle;
  property Constraints;
  property DragCursor;
  property DragMode;
  property Enabled;
  property Color;
  property Ctl3D;
  property Font;
  property Locked;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
end;


{-------------------- Strip Chart routines ---------------------------}


type TMajorAxis = (maXAxis,maYAxis);


type
 TMarker=record
 Position: double;
 Color: TColor;
 Width: integer;
 end;


type
  TStripChart = class(TCustomGraphBox)
 private
  procedure DrawMarkers;
 protected
  FMajorAxis: TMajorAxis;
  FStripInc: double;
  FFixedScale: boolean;
  FMarkers: TList;
  procedure SetMajorAxis(V: TMajorAxis);
  procedure RemoveOldValues(Start: double);
  procedure SetFixedScale(V: boolean);
  procedure Paint; override;
  procedure DoGraph(Canvas: TCanvas); override;
  procedure HandleAxisChange(Sender: TObject); override;
public
  DeleteOldValues: boolean;
  StripPosition: double;
  procedure ReReferenceData;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function AddValue(Value: double; Group: integer; IncPos: boolean; Color: TColor): integer; overload;
  function AddValue(Value: double; Group: integer;Color: TColor): integer; overload;
  procedure IncrementPosition;
  procedure ClearData;
  procedure SetMarker(Width: integer; Color: TColor);
  procedure ClearMarkers;
published
  property MajorAxis: TMajorAxis read FMajorAxis write SetMajorAxis default maXAxis;
  property FixedScale: boolean read FFixedScale write SetFixedScale default false;
  property Increment: double read FStripInc write FStripInc;
  property DrawLastPixel;
  property DefaultData;
  property GraphType;
  property XAxis;
  property YAxis;

  property Align;
  property Anchors;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BorderWidth;
  property BorderStyle;
  property Constraints;
  property DragCursor;
  property DragMode;
  property Enabled;
  property Color;
  property Ctl3D;
  property Font;
  property Locked;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnClick;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
end;


{Tool bars for pan/zoom of a graph box}

type TGraphToolbar = class(TCustomPanel)
 private
  FHorizontal: boolean;
    FGraphBox: TCustomGraphBox;
  procedure SetupButton(var Btn: TAutoButton; Size: TPoint; Name, GlyphName: string);
  procedure SetHorizontal(const Value: boolean);
  procedure ConfigureControl;
  function GetDoRepeat: boolean;
  procedure SetDoRepeat(const Value: boolean);
  function GetEnabled: boolean;
  procedure SetEnabled(const Value: boolean);
    procedure ButtonHandler(Sender: TObject);
 protected
  UpBtn, DownBtn, LeftBtn, RightBtn, ZoomInBtn, ZoomOutBtn, ResetBtn: TAutoButton;
  procedure Resize; override;
  procedure Notification(AComponent: TComponent; Operation: TOperation); override;
 public
  constructor Create(AOwner: TComponent); override;
 published
  property Horizontal: boolean read FHorizontal write SetHorizontal;
  property DoRepeat: boolean read GetDoRepeat write SetDoRepeat default false;
  property Enabled: boolean read GetEnabled write SetEnabled default True;
  property GraphBox: TCustomGraphBox read FGraphBox write FGraphBox;
    property Align;
    property Anchors;
    property AutoSize;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Constraints;
    property Ctl3D;
    property UseDockManager default True;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property FullRepaint;
    property Font;
    property Locked;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
 end;





procedure Register;


implementation

{$R GraphBoxButtons.res}

{-------------------- Support Routines --------------------------------}



function NormalizeRange(range: double; round: boolean): double;
{Returns the closest 1-2-5 range to the specified range}
{if round =true then Rounds the number}
{if round=false then takes the ceiling}
var exponent: double;		{ exponent of range }
var fraction: double;		{ fractional part of range }
var NormalizeFraction: double;	{ Normalize, rounded fraction }
begin
exponent:=floor(log10(range));
fraction:=range / power(10, exponent);
if Round then
	begin
	if fraction < 1.5 then NormalizeFraction:=1
	else if fraction < 3 then NormalizeFraction:=2
	else if fraction < 7 then NormalizeFraction:=5
	else NormalizeFraction:=10;
	end
else
	begin
	if fraction <= 1 then NormalizeFraction:=1
	else if fraction <= 2 then NormalizeFraction:=2
	else if fraction <= 5 then NormalizeFraction:=5
	else NormalizeFraction:=10;
	end;
Result:=NormalizeFraction * power(10, exponent);
end;




procedure CalculateGraphScale(MaxValue,MinValue: double; MaxLines: integer; var NewMax,NewMin,NewSpacing: double; var NewLineCount: integer);
{Calculate normalized scale and line count given a range of values and a maximum number of lines}
{Normalized means the 1-2-5 series times a power of ten that matches the range}
{Note: it may return more than the maxium number of lines specified. I haven't analysed why}
var Range: double;
begin
SwapDoubleArgs(MinValue>MaxValue,MinValue,MaxValue);
if MaxLines<2 then MaxLines:=2;

Range:=NormalizeRange(MaxValue - MinValue, false);
NewSpacing:=NormalizeRange(range / (MaxLines - 1), true);
NewMin:=floor(MinValue / NewSpacing) * NewSpacing;
NewMax:=ceil(MaxValue / NewSpacing) * NewSpacing;
NewLineCount:=trunc(abs(NewMax-NewMin)/NewSpacing);
end;



{------------------- Routines For Custom Graph Component --------------}


function TCustomGraph.GetBorderWidth(TextSize: integer): integer;

	function GetTextBandSize(TextSize: integer): integer;
	const TextMargin = 1;
	var MinTextSize: integer;
	begin
	MinTextSize:=TextSize + (2 * TextMargin);
	if MinTextSize>FGraphBorder then Result:=MinTextSize
	else Result:=FGraphBorder;
	end;

begin
Result:=GetTextBandSize(TextSize)+BevelWidth+BorderWidth;
end;


procedure TCustomGraph.Resize;
begin
inherited;
BackBitmap.Width:=Width;
BackBitmap.Height:=Height;
end;


procedure TCustomGraph.Restore;
begin
Canvas.Draw(0,0,BackBitmap);
end;


procedure TCustomGraph.ClearData;
var I: integer;
begin
for I:=Low(GroupCount) to High(GroupCount) do GroupCount[I]:=0;
for I:=0 to FGraphData.Count-1 do Dispose(FGraphData.Items[I]);
FGraphData.Clear;
MaxValue:=MakeVector2D(-MaxDouble,-MaxDouble);
MinValue:=MakeVector2D(MaxDouble,MaxDouble);
Invalidate;
end;



procedure TCustomGraph.SetData(X,Y,Width: double; Group: integer; Text: string; Color: TColor);
var DP: ^TDataPoint;
begin
Inc(GroupCount[Group]);
New(DP);
DP^.X:=X;
DP^.Y:=Y;
DP^.Width:=Width;
DP^.Group:=Group;
DP^.Color:=Color;
DP^.Text:=Text;
FGraphData.Add(DP);
MinValue:=GetMinimumVector2D(MinValue,MakeVector2D(X,Y));
MaxValue:=GetMaximumVector2D(MaxValue,MakeVector2D(X,Y));
if not FBlockRefresh then Invalidate;
end;




procedure TCustomGraph.GetModuloRange(Low,High,Modulo: double;var S,E:double; var N: integer);
begin
S:=NearestMultiple(Low,Modulo,True);
E:=NearestMultiple(High,Modulo,False);
N:=Trunc((E-S)/Modulo);
end;



procedure TCustomGraph.DoGraph(Canvas: TCanvas; AWidth,AHeight: integer);
begin
end;



procedure TCustomGraph.DoGraph(Canvas: TCanvas);
begin
end;



constructor TCustomGraph.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
{$IF CompilerVersion >= 17}
ParentBackground:=False;
{$IFEND}
BackBitmap:=TBitmap.Create;
FGraphData:=TList.Create;
ControlStyle := ControlStyle - [csSetCaption];
Color:=clWhite;
BevelOuter:=bvLowered;
FGridColor:=clBlack;
Width:=90;Height:=90;
FGraphBorder:=20;
FBlockRefresh:=False;
FDrawLastPixel:=False;
end;



destructor TCustomGraph.Destroy;
begin
ClearData;
FGraphData.Free;
BackBitmap.Free;
inherited Destroy;
end;


procedure TCustomGraph.BeginUpdate;
begin
FBlockRefresh:=True;
end;

procedure TCustomGraph.EndUpdate;
begin
FBlockRefresh:=False;
Invalidate;
end;



procedure TCustomGraph.SetDecimals1(V: integer);
begin
if V<>FDecimals1 then
	begin
	FDecimals1:=V;
	Invalidate;
	end;
end;


procedure TCustomGraph.SetDecimals2(V: integer);
begin
if V<>FDecimals2 then
	begin
	FDecimals2:=V;
	Invalidate;
	end;
end;


procedure TCustomGraph.SetGridColor(V: TColor);
begin
if V<>FGridColor then
	begin
	FGridColor:=V;
	Invalidate;
	end;
end;


procedure TCustomGraph.SetBorder(V: integer);
begin
if V<>FGraphBorder then
	begin
	FGraphBorder:=V;
	Invalidate;
	end;
end;


procedure TCustomGraph.GetImageSize(var ImageWidth,ImageHeight: integer;var ImageRect: TRect);
var BevelSize: Integer;
begin
ImageRect:=ClientRect;
BevelSize := BorderWidth;
if BevelOuter <> bvNone then Inc(BevelSize, BevelWidth);
if BevelInner <> bvNone then Inc(BevelSize, BevelWidth);
InflateRect(ImageRect, -BevelSize, -BevelSize);
ImageWidth:=ImageRect.Right-ImageRect.Left;
ImageHeight:=ImageRect.Bottom-ImageRect.Top;
end;



procedure TCustomGraph.Paint;
var W,H: integer;
var R: TRect;
begin
inherited Paint;
GetImageSize(W,H,R);
Canvas.Brush.Color:=clBlack;
if Color=clWhite then Canvas.FrameRect(R);
Canvas.Brush.Color:=Color;
Canvas.Brush.Style:=bsClear;
end;



procedure TCustomGraph.CopyToClipboard;
var B: TBitmap;
var R: TRect;
var W,H: integer;
begin
GetImageSize(W,H,R);
B:=TBitmap.Create;
B.Width:=W; B.Height:=H;
DoGraph(B.Canvas);
ClipBoard.Assign(B);
B.Free;
end;




procedure TCustomGraph.MetafileToClipboard(Enhanced: boolean);
var MyFormat: Word;
var AData: THandle;
var APalette: hPalette;
var W,H: integer;
var R: TRect;
var MFile: TMetaFile;
var MCanvas: TMetafileCanvas;
begin
MFile:=TMetafile.Create;
try
GetImageSize(W,H,R);
MFile.Enhanced:=Enhanced;
MFile.Width:=W;
MFile.Height:=H;
MCanvas:=TMetafileCanvas.Create(MFile, 0);
try
MCanvas.Brush.Color:=Color;
MCanvas.FillRect(Rect(0,0,W,H));
DoGraph(MCanvas);
finally MCanvas.Free; end;
MFile.SaveToClipBoardFormat(MyFormat,AData,APalette);
ClipBoard.SetAsHandle(MyFormat,AData);
finally MFile.Free; end;
end;



procedure TCustomGraph.SaveToFile(Name: string; AWidth,AHeight: integer);
var B: TBitmap;
var R: TRect;
var W,H: integer;
begin
Repaint;
B:=TBitmap.Create;
try
B.Width:=AWidth; B.Height:=AHeight;
DoGraph(B.Canvas,AWidth,AHeight);
SaveBitmapByExtension(B,Name,False,False,False);
finally B.Free; end;
end;



procedure TCustomGraph.SaveToFile(Name: string);
var B: TBitmap;
var R: TRect;
var W,H: integer;
begin
GetImageSize(W,H,R);
SaveToFile(Name,W,H);
end;




procedure TCustomGraph.SaveToMetaFile(Name: string; Enhanced: boolean; AWidth,AHeight: integer);
var W,H: integer;
var R: TRect;
var MFile: TMetaFile;
var MCanvas: TMetafileCanvas;
begin
MFile:=TMetafile.Create;
try
MFile.Enhanced:=Enhanced;
MFile.Width:=AWidth;
MFile.Height:=AHeight;
MCanvas:=TMetafileCanvas.Create(MFile, 0);
try
MCanvas.Brush.Color:=Color;
MCanvas.FillRect(Rect(0,0,AWidth,AWidth));
DoGraph(MCanvas,AWidth,AWidth);
finally MCanvas.Free; end;
MFile.SaveToFile(Name);
finally MFile.Free; end;
end;




procedure TCustomGraph.SaveToMetaFile(Name: string; Enhanced: boolean);
var W,H: integer;
var R: TRect;
begin
GetImageSize(W,H,R);
SaveToMetaFile(Name, Enhanced, W,H);
end;



procedure TCustomGraph.PrintByMetaFile;
var W,H: integer;
var R: TRect;
var MFile: TMetaFile;
var MCanvas: TMetafileCanvas;
begin
MFile:=TMetafile.Create;
try
GetImageSize(W,H,R);
MFile.Enhanced:=True;
MFile.Width:=W;
MFile.Height:=H;
MCanvas:=TMetafileCanvas.Create(MFile, 0);
try
MCanvas.Brush.Color:=Color;
MCanvas.FillRect(Rect(0,0,W,H));
DoGraph(MCanvas);
finally MCanvas.Free; end;
if W>H then R:=Rect(0,0,Printer.PageWidth,MulDiv(Printer.PageWidth,H,W))
else R:=Rect(0,0,MulDiv(Printer.PageHeight,W,H),Printer.PageHeight);
Printer.BeginDoc;
Printer.Canvas.StretchDraw(R,MFile);
Printer.EndDoc;
finally MFile.Free; end;
end;



function TCustomGraph.GetCount: integer;
begin
Result:=FGraphData.Count;
end;



function TCustomGraph.GetGraphData(Index: Cardinal): TDataPoint;
begin
Result:=TDataPoint(FGraphData.Items[Index]^);
end;



procedure TCustomGraph.SetGraphData(Index: Cardinal; const Value: TDataPoint);
begin
TDataPoint(FGraphData.Items[Index]^):=Value;
end;



{----------------------------------------------------------------------------}
{---------------------- Graph Title Routines --------------------------------}
{----------------------------------------------------------------------------}



procedure TGraphTitle.SetShowCaption(V: boolean);
begin
if V<>FShowCaption then
	begin
	FShowCaption:=V;
	if Assigned(FOnChanged) then FOnChanged(Self);
	end;
end;


procedure TGraphTitle.SetPosition(V: TTitlePosition);
begin
if V<>FPosition then
	begin
	FPosition:=V;
	if Assigned(FOnChanged) then FOnChanged(Self);
	end;
end;


procedure TGraphTitle.SetBoxed(V: boolean);
begin
if V<>FBoxed then
	begin
	FBoxed:=V;
	if Assigned(FOnChanged) then FOnChanged(Self);
	end;
end;



procedure TGraphTitle.SetCaption(S: string);
begin
if S<>FCaption then
	begin
	FCaption:=S;
	if Assigned(FOnChanged) then FOnChanged(Self);
	end;
end;



{----------------------------------------------------------------------------}
{-------------------- Axis Parameter Routines -------------------------------}
{----------------------------------------------------------------------------}


procedure TAxisParams.DefineProperties(Filer: TFiler);
begin
inherited DefineProperties(Filer);
with Filer do
 begin
 DefineProperty('Min',ReadMin,WriteMin,true);
 DefineProperty('Max',ReadMax,WriteMax,true);
 end;
end;


function RangeCheckLog(var X: double): boolean;
begin
Result:=X>0;
if not Result then X:=1;
end;


procedure TAxisParams.RangeCheckMinMax;
begin
if not FLog then exit;
RangeCheckLog(FMax);
RangeCheckLog(FMin);
end;


procedure TAxisParams.ReadMax(Reader: TReader);
begin
FMax:=Reader.ReadFloat;
end;

procedure TAxisParams.WriteMax(Writer:TWriter) ;
begin
Writer.WriteFloat(FMax);
end;

procedure TAxisParams.ReadMin(Reader: TReader);
begin
FMin:=Reader.ReadFloat;
end;

procedure TAxisParams.WriteMin(Writer:TWriter) ;
begin
Writer.WriteFloat(FMin);
end;


procedure TAxisParams.Change;
begin
if Assigned(FOnChanged) then FOnChanged(Self);
end;



procedure TAxisParams.SetMinMax(Min,Max: double);
{Sets values but does not trigger events}
{Important so Base values are changed with Pan and Zoom}
begin
FMin:=Min; FMax:=Max;
RangeCheckMinMax;
end;




procedure TAxisParams.SetMin(V: double);
begin
if V<>FMin then
	begin
	FMin:=V;
	RangeCheckMinMax;
	Change;
	end;
end;


procedure TAxisParams.SetMax(V: double);
begin
if V<>FMax then
	begin
	FMax:=V;
	RangeCheckMinMax;
	Change;
	end;
end;


procedure TAxisParams.SetMultiplier(V: double);
begin
if V<>FMultiplier then
	begin
	FMultiplier:=V;
	Change;
	end;
end;


procedure TAxisParams.SetOffset(const Value: double);
begin
if FOffset<>Value then
	begin
	FOffset := Value;
	Change;
	end;
end;



procedure TAxisParams.SetCycle(const Value: double);
begin
if FCycle<>Value then
	begin
	FCycle := Value;
	Change;
	end;
end;



procedure TAxisParams.SetLabelType(const Value: TLabelType);
begin
if Value<>FLabelType then
	begin
	FLabelType:=Value;
	Change;
	end;
end;




function TAxisParams.ApplyScaleOffset(V: double): double;
begin
Result:=(V + Offset)*Multiplier;
if Cycle<>0 then Result:=FloatMod(Result,Cycle);
if Unsigned then Result:=Abs(Result);
end;



procedure TAxisParams.SetLines(V: integer);
begin
if V<>FLines then
	begin
	FLines:=V;
	Change;
	end;
end;


procedure TAxisParams.SetShowLabels(V: boolean);
begin
if V<>FShowLabels then
	begin
	FShowLabels:=V;
	Change;
	end;
end;



procedure TAxisParams.SetLabelModulo(const Value: TModulo);
begin
if FLabelModulo<>Value then
	begin
	FLabelModulo:=Value;
	Change;
	end;
end;



procedure TAxisParams.SetMinorLabels(V: boolean);
begin
if V<>FMinorLabels then
	begin
	FMinorLabels:=V;
	Change;
	end;
end;

procedure TAxisParams.SetLog(V: boolean);
begin
if V<>FLog then
	begin
	Flog:=V;
	RangeCheckMinMax;
	Change;
	end;
end;


procedure TAxisParams.SetUnits(S: string);
begin
if S<>FUnits then
	begin
	FUnits:=S;
	Change;
	end;
end;


constructor TAxisParams.Create;
begin
FLabelModulo:=1;
FMultiplier:=1;
FOffset:=0;
FCycle:=0;
FLabelType:=ltPrimary;
end;




procedure TAxisParams.SetExponentLabels(const Value: boolean);
begin
if FExponentLabels <> Value then
	begin
	FExponentLabels:=Value;
	Change;
	end;
end;




procedure TAxisParams.SetMinorLines(const Value: boolean);
begin
if FMinorLines<>Value then
	begin
	FMinorLines:=Value;
	Change;
	end;
end;


procedure TAxisParams.SetUnsigned(const Value: boolean);
begin
if FUnsigned<>Value then
	begin
	FUnsigned := Value;
	Change;
	end;
end;


procedure TAxisParams.SetWrap(const Value: boolean);
begin
if FWrap<>Value then
	begin
	FWrap := Value;
	Change;
	end;
end;



{----------------------------------------------------------------------------}
{-------------------- Graph Box Routines ------------------------------------}
{----------------------------------------------------------------------------}



procedure Register;
begin
RegisterComponents('Fountain', [TGraphBox]);
RegisterComponents('Fountain', [TRoseDiagram]);
RegisterComponents('Fountain', [TStripChart]);
RegisterComponents('Fountain', [TBarMeter]);
RegisterComponents('Fountain', [TGraphToolbar]);
end;


{------------------ Orgin Referenced Graph Drawing Routines ---------------}

procedure TCustomGraphBox.OrgMove(X,Y: integer);
{ Move, Referenced To Origin }
begin
WrkCanvas.MoveTo(XOrigin+X,YOrigin-Y);
end;








procedure TCustomGraphBox.OrgLine(X,Y: integer;C: TColor);
{ Draw Line, Referrence To Origin }
var X1,Y1: integer;
var P: TPoint;
begin
WrkCanvas.Pen.Color:=C;
X1:=XOrigin+X; Y1:=YOrigin-Y;
if DrawLastPixel then
	begin
	P:=WrkCanvas.PenPos;
	GetLastPixelPos(P.X,P.Y,X1,Y1);
	end;
WrkCanvas.LineTo(X1,Y1);
end;

{------------------ Scaled Graph Drawing  Routines ------------------------}


procedure TCustomGraphBox.XYToScaled(X,Y: integer; var X2,Y2: double);
begin
X2:=(X-XOrigin)/XScale+XOffset;
Y2:=(YOrigin-Y)/YScale+YOffset;
if FXAxis.Log then X2:=AntiLog(X2);
if FYAxis.Log then Y2:=AntiLog(Y2);
end;


procedure TCustomGraphBox.GetLogsValues(var X,Y: double);
begin
if FXAxis.Log then
	begin
	if X<=0 then X:=1;
	X:=Log10(X);
	end;
if FYAxis.Log then
	begin
	if Y<=0 then Y:=1;
	Y:=Log10(Y);
	end;
end;



function TCustomGraphBox.GetScaledPoint(X,Y: double): TPoint;
{ Get point scaled to grid }
begin
GetLogsValues(X,Y);
Result.X:=XOrigin+round((X-XOffset)*XScale);
Result.Y:=YOrigin-round((Y-YOffset)*YScale);
end;


procedure TCustomGraphBox.ScaledLine(X,Y: double;C: TColor);
{ Draw Scaled And Offset Line }
begin
GetLogsValues(X,Y);
OrgLine(Trunc((X-XOffset)*XScale),Trunc((Y-YOffset)*YScale),C);
end;


procedure TCustomGraphBox.ScaledMove(X,Y: double);
{ Move Scaled And Offset }
begin
GetLogsValues(X,Y);
OrgMove(Trunc((X-XOffset)*XScale),Trunc((Y-YOffset)*YScale));
end;



procedure TCustomGraphBox.DrawPoint(X,Y: double; Radius: integer; BorderColor,CenterColor: TColor);
var R: TRect;
var P: TPoint;
begin
P:=GetScaledPoint(X,Y);
R.Left:=P.X - Radius; R.Top:=P.Y - Radius;
R.Right:=P.X + Radius; R.Bottom:=P.Y + Radius;
WrkCanvas.Pen.Color:=BorderColor;
WrkCanvas.Brush.Color:=CenterColor;
WrkCanvas.Brush.Style:=bsSolid;
WrkCanvas.Ellipse(R);
end;




{------------------ Exponential Routines ----------------------------------------}


procedure IncExponential(var E: TExponential);
begin
Inc(E.Base);
if E.Base>=10 then begin E.Base:=1; Inc(E.Exp); end;
end;


function ExpToReal(E: TExponential): double;
begin
Result:=E.Base * IntPower(10,E.Exp);
end;


function GetNexExponential(V: double): TExponential;
begin
Result.Exp:=Floor(Log10(V));
Result.Base:=Trunc(V / AntiLog(Result.Exp));
IncExponential(Result);
end;


procedure TCustomGraphBox.DoLogSteps(Start,Stop: double; LogProc: TLogProcedure);
var E: TExponential;
begin
E:=GetNexExponential(Start);
repeat
	begin
	LogProc(E,E.Base=1);
	IncExponential(E);
	end
until ExpToReal(E)>=Stop;
end;


{----------- Vertical Log Grid Routines ---------------------------------------}


procedure TCustomGraphBox.VertGridCallback(E: TExponential; Major: boolean);
var X: integer;
var V: double;
begin
if not Major and not FXAxis.MinorLines then exit;
if Major then Canvas.Pen.Style:=psSolid else Canvas.Pen.Style:=psDot;
V:=ExpToReal(E);
X:=Trunc((Log10(V)-XOffset)*XScale);
OrgMove(X,0);
OrgLine(X,MaxY,FGridColor);
end;




procedure TCustomGraphBox.VertLabelsCallback(E: TExponential; Major: boolean);
var X,Y,W: integer;
var S: string;
var V: double;
begin
if not FXAxis.MinorLabels and not Major then exit;
V:=ExpToReal(E);
X:=Trunc((Log10(V)-XOffset)*XScale)+XOrigin;
Y:=YOrigin+1;
if Major then
	begin
        if FXAxis.ExponentLabels then
		begin
		S:='10^'+IntToStr(E.Exp);
		Y:=Y + MulDiv(WrkCanvas.TextHeight(S),40,100);
		end
	else S:=FloatToStrF(FXAxis.ApplyScaleOffset(V),ffFixed,18,FDecimals2)+FXAxis.Units
	end
else S:=IntToStr(E.Base);
W:=WrkCanvas.TextWidth(S);
if not Major then X:=X-MulDiv(W,65,100);
OutputSuperScriptText(WrkCanvas,S,X,Y);
end;





procedure TCustomGraphBox.VerticalLogGrid(Start,Stop: double);
var L1,L2,DX: double;
begin
if Start<=0 then Start:=1;
L1:=Log10(Start); L2:=Log10(Stop);
DX:=L2-L1;
if DX=0 then XScale:=1e-99 else XScale:=MaxX/DX;
XOffset:=L1;

DoLogSteps(Start,Stop,VertGridCallback);
if FXAxis.ShowLabels then DoLogSteps(Start,Stop,VertLabelsCallBack);
Canvas.Pen.Style:=psSolid;
end;


{------------------ Horizontal Log Grids --------------------------------------}


procedure TCustomGraphBox.HorizGridCallback(E: TExponential; Major: boolean);
var Y: integer;
var V: double;
begin
if not Major and not FYAxis.MinorLines then exit;
if Major then Canvas.Pen.Style:=psSolid
else Canvas.Pen.Style:=psDot;
V:=ExpToReal(E);
Y:=Trunc((Log10(V)-YOffset)*YScale);
OrgMove(0,Y);
OrgLine(MaxX,Y,FGridColor);
end;




procedure TCustomGraphBox.HorizLabelsCallback(E: TExponential; Major: boolean);
var Y,W: integer;
var S: string;
var V: double;
begin
if not FYAxis.MinorLabels and not Major then exit;
V:=ExpToReal(E);
Y:=Trunc((Log10(V)-YOffset)*YScale);
if Major then
	begin
        if FYAxis.ExponentLabels then
		begin
		S:='10^'+IntToStr(E.Exp);
		end
	else S:=FloatToStrF(FYAxis.ApplyScaleOffset(V),ffFixed,18,FDecimals2)+FYAxis.Units;
	end
else
	begin
	S:=IntToStr(E.Base);
	Y:=Y - WrkCanvas.TextHeight('H') div 2;
	end;
W:=WrkCanvas.TextWidth(S);
if YAxis.LabelType in [ltPrimary,ltBoth] then OutputSuperScriptText(WrkCanvas,S,XOrigin-W-2,round(YOrigin-Y));
if YAxis.LabelType in [ltSecondary,ltBoth] then OutputSuperScriptText(WrkCanvas,S,RightSide+2,round(YOrigin-Y));

	

end;



procedure TCustomGraphBox.HorizLogGrid(Start,Stop: double);
var L1,L2,DY: double;
begin
if Start<=0 then Start:=1;
L1:=Log10(Start); L2:=Log10(Stop);
DY:=L2-L1;
if DY=0 then YScale:=1e-99 else YScale:=MaxY/DY;
YOffset:=L1;

SetTextAlign(WrkCanvas.Handle,TA_BASELINE);
SetBkMode(WrkCanvas.Handle, TRANSPARENT);

DoLogSteps(Start,Stop,HorizGridCallback);
if FYAxis.ShowLabels then DoLogSteps(Start,Stop,HorizLabelsCallBack);
Canvas.Pen.Style:=psSolid;
end;


{------------------ Draw No-Log Grids ----------------------------------------}


procedure TCustomGraphBox.VerticalGrid(LineCount: integer; Start,Stop: double);
{ Draw Vertical Grid - marking X-axis lines}
{ The number of lines are passed in X }
{ The Start And End Of Scale are passed In R1 And R2 }
var O,DL,DX,LV: double;
var I,W : Integer;
var S: string;
begin
DX:=Stop-Start;
if DX=0 then XScale:=1e-99 else XScale:=MaxX/DX;
XOffset:=Start;

O:=MaxX / LineCount;
for I:=1 to LineCount do
	begin
	OrgMove(round(I*O),0);
	OrgLine(round(I*O),MaxY,FGridColor);
	end;
SetBkMode(WrkCanvas.Handle, TRANSPARENT);

if not FXAxis.ShowLabels then exit;
DL:=(Stop-Start)/LineCount;
if FFixedScaleX then
 if Stop>Start then  Start:=0 else Start:=Start - Stop;

for I:=0 to LineCount do
	begin
	LV:=FXAxis.ApplyScaleOffset(Start);
	if Assigned(FOnHorizLabel) then S:=FOnHorizLabel(LV)
	else S:=FloatToStrF(LV,ffFixed,18,FDecimals2)+FXAxis.Units;
	W:=WrkCanvas.TextWidth(S);
	if (I mod FXAxis.FLabelModulo) = 0 then
		begin
		WrkCanvas.TextOut(round(I*O+XOrigin)-MulDiv(W,65,100),YOrigin+1,S);
		end;
	Start:=Start+DL;
	end;
end;





procedure TCustomGraphBox.HorizontalGrid(Y: integer; Start,Stop: double);
{ Draw Horizontal Grid }
{ Number Of Lines In X }
{ Start And End Of Scale In R1 And R2 }
var DL,O,DY,LV: double;
var I: Integer;


	procedure DrawHorzLine(V,Y: double);
	var S: string;
	var W : Integer;
	begin
	OrgMove(0,round(Y));
	OrgLine(MaxX,round(Y),FGridColor);
	if not FYAxis.ShowLabels or ((I mod FYAXis.FLabelModulo)<>0) then exit;
	LV:=FYAxis.ApplyScaleOffset(V);
	if Assigned(FOnVertLabel) then S:=FOnVertLabel(LV)
	else S:=FloatToStrF(LV,ffFixed,18,FDecimals1)+FYAxis.Units;
	W:=WrkCanvas.TextWidth(S);
	if YAxis.LabelType in [ltPrimary,ltBoth] then WrkCanvas.TextOut(XOrigin-W-2,round(YOrigin-Y),S);
	if YAxis.LabelType in [ltSecondary,ltBoth] then WrkCanvas.TextOut(RightSide+2,round(YOrigin-Y),S);
	end;

begin
DY:=Stop-Start;
if DY =0 then YScale:=1e-99 else YScale:=MaxY/DY;
YOffset:=Start;

SetTextAlign(WrkCanvas.Handle,TA_BASELINE);
SetBkMode(WrkCanvas.Handle, TRANSPARENT);

DL:=(Stop-Start)/Y;
O:=MaxY/Y;
if FFixedScaleY then
 if Stop>Start then Start:=0 else  Start:=Start - Stop;
for I:=0 to Y do
	begin
	DrawHorzLine(Start,O*I);
	Start:=Start+DL;
	end;
end;


{-------------------------------------------------------------------------------}



procedure TCustomGraphBox.SetVerticalRange(Start,Finish: double; LineCount: integer);
begin
FYAxis.Lines:=LineCount;
FYAxis.Min:=Start; FYAxis.Max:=Finish;
Invalidate;
end;


procedure TCustomGraphBox.SetModVerticalRange(Start,Finish,Width: double);
var N: integer;
begin
GetModuloRange(Start,Finish,Width,Start,Finish,N);
if (FYAxis.Lines<>N) or
   (FYAxis.Min<>Start) or
   (FYAxis.Max<>Finish) then
	begin
	FYAxis.Lines:=N;
	FYAxis.Min:=Start; FYAxis.Max:=Finish;
	Invalidate;
	end
end;


procedure TCustomGraphBox.SetHorizontalRange(Start,Finish: double; LineCount: integer);
begin
FXAxis.Lines:=LineCount;
FXAxis.Min:=Start; FXAxis.Max:=Finish;
Invalidate;
end;



procedure TCustomGraphBox.SetModHorizontalRange(Start,Finish,Width: double);
var N: integer;
begin
GetModuloRange(Start,Finish,Width,Start,Finish,N);
if (FXAxis.Lines<>N) or
   (FXAxis.Min<>Start) or
   (FXAxis.Max<>Finish) then
	begin
	FXAxis.Lines:=N;
	FXAxis.Min:=Start; FXAxis.Max:=Finish;
	Invalidate;
	end
end;

{-------------------------------------------------------------------------------}


function GetDecimalPoint(Spacing: double): integer;
var L: double;
begin
L:=Log(abs(Spacing)/10);
if L>=0 then Result:=0
else Result:=Round(Abs(L));
end;



procedure TCustomGraphBox.AutoScaleVertical(MaxLines: integer);
{Scale the vertical axis and set number of lines for maximum readability}
{Note: MaxLines may only limit the maximum to 1.5 times the value}
var NewLineCount: integer;
var NewMin, NewMax,NewSpacing: double;
var V: double;
begin
if FGraphData.Count>0 then V:=GraphData[FGraphData.Count-1].Y else V:=0;
CalculateGraphScale(MaxValue.Y,MinValue.Y,MaxLines,NewMax,NewMin,NewSpacing,NewLineCount) ;
if abs(MaxValue.Y-MinValue.Y)=0 then
	begin
	NewMin:=V - NewSpacing;
	NewMax:=V + NewSpacing;
	NewLineCount:=2;
	end;
SetVerticalRange(NewMin,NewMax,NewLineCount);
Decimals1:=GetDecimalPoint(NewSpacing);
end;




procedure TCustomGraphBox.AutoScaleHorizontal(MaxLines: integer);
{Scale the Horizonal axis and set number of lines for maximum readability}
{Note: MaxLines may only limit the maximum to 1.5 times the value}
var NewLineCount: integer;
var NewMin, NewMax,NewSpacing: double;
var V: double;
begin
if FGraphData.Count>0 then V:=GraphData[FGraphData.Count-1].X else V:=0;
CalculateGraphScale(MaxValue.X,MinValue.X,MaxLines,NewMax,NewMin,NewSpacing,NewLineCount) ;
if abs(MaxValue.X-MinValue.X)=0 then
	begin
	NewMin:=V - NewSpacing;
	NewMax:=V + NewSpacing;
	NewLineCount:=2;
	end;
SetHorizontalRange(NewMin,NewMax,NewLineCount);
Decimals2:=GetDecimalPoint(NewSpacing);
end;




{-------------------------------------------------------------------------------}


procedure TCustomGraphBox.GetVerticalRange(var Start,Finish: double; var LineCount: integer);
begin
LineCount:=FYAxis.Lines;
Start:=FYAxis.Min; Finish:=FYAxis.Max;
end;


procedure TCustomGraphBox.GetHorizontalRange(var Start,Finish: double; var LineCount: integer);
begin
LineCount:=FXAxis.Lines;
Start:=FXAxis.Min; Finish:=FXAxis.Max;
end;




{------------------ Routines To Handle Stored Data Points --------------}



procedure TCustomGraphBox.DoMinMax;
var I: integer;
var P: TDataPoint;
var V: T2DVector;
begin
MaxValue:=MakeVector2D(-MaxDouble,-MaxDouble);
MinValue:=MakeVector2D(MaxDouble,MaxDouble);
for I:=0 to FGraphData.Count-1 do
	begin
	P:=TDataPoint(FGraphData.Items[I]^);
	V:=MakeVector2D(P.X,P.Y);
	MinValue:=GetMinimumVector2D(MinValue,V);
	MaxValue:=GetMaximumVector2D(MaxValue,V);
	end;
end;



procedure TCustomGraphBox.AdjustVerticalRange(Modulo: boolean; Spacing: double);
begin
if (MaxValue.Y>YAxis.Max) or (MinValue.Y<YAxis.Min) then
	begin
	if Modulo then SetModVerticalRange(MinValue.Y,MaxValue.Y,Spacing)
	else SetVerticalRange(MinValue.Y,MaxValue.Y,YAxis.Lines);
 	end;
end;




procedure TCustomGraphBox.AdjustHorizontalRange(Modulo: boolean; Spacing: double);
begin
if (MaxValue.X>XAxis.Max) or (MinValue.X<XAxis.Min) then
	begin
	if Modulo then SetModHorizontalRange(MinValue.X,MaxValue.X,Spacing)
	else SetHorizontalRange(MinValue.X,MaxValue.X,XAxis.Lines);
 	end;
end;



procedure TCustomGraphBox.AdjustRange(Modulo: boolean; Spacing: double);
begin
AdjustVerticalRange(Modulo,Spacing);
AdjustHorizontalRange(Modulo,Spacing);
end;



procedure TCustomGraphBox.SetDataPoint(X,Y,Width: double; Group: integer; Text: string; Color: TColor);
begin
SetData(X,Y,Width,Group,Text,Color);
end;



procedure TCustomGraphBox.SetDataPoint(X,Y,Width: double; Group: integer;Color: TColor);
begin
SetData(X,Y,Width,Group,'',Color);
end;


procedure TCustomGraphBox.LabelPoint(Data: TDataPoint);
var P: TPoint;
var Offset: integer;
begin
if Data.Text='' then exit;
P:=GetScaledPoint(Data.X,Data.Y);
Offset:=round(Data.Width/2)+1;
WrkCanvas.TextOut(P.X+Offset,P.Y-Offset,Data.Text);
end;



procedure TCustomGraphBox.DrawGraphLine(Line: T2DLine; Color: TColor);
var ClipWin: T2DRect;
var WrappedLines: T2DLineArray;
var P1,P2: TPoint;
var I: integer;
begin
if XAxis.Wrap or YAXis.Wrap then
	begin
	ClipWin:=MakeRect2D(XAxis.Min,YAxis.Max,XAxis.Max,YAxis.Min);
	WrapLine2D(ClipWin,Line,WrappedLines,XAxis.Wrap,YAXis.Wrap);
	for I:=0 to High(WrappedLines) do
		begin
		ScaledMove(WrappedLines[I].P1.X,WrappedLines[I].P1.Y);
		ScaledLine(WrappedLines[I].P2.X,WrappedLines[I].P2.Y,Color);
		end;
	end
else
	begin
	ScaledMove(Line.P1.X,Line.P1.Y);
	ScaledLine(Line.P2.X,Line.P2.Y,Color);
	end;
end;



procedure TCustomGraphBox.MarkDataPoint(P: T2DVector; Size: double; BorderColor,CenterColor: TColor);
var ClipWin: T2DRect;
var WrappedLines: T2DLineArray;
var P1,P2: TPoint;
var I: integer;
begin
if XAxis.Wrap or YAXis.Wrap then
	begin
	ClipWin:=MakeRect2D(XAxis.Min,YAxis.Max,XAxis.Max,YAxis.Min);
	P:=WrapPoint2D(ClipWin, P,XAxis.Wrap,YAXis.Wrap);
	end;
DrawPoint(P.X,P.Y,trunc(Size),BorderColor,CenterColor);
end;




function TCustomGraphBox.DrawGroup(Group: integer): integer;
var First: boolean;
var I,Count,ModCnt: integer;
var Data: TDataPoint;
var Line: T2DLine;
begin
First:=True;
Result:=0;
Count:=GroupCount[Group];
ModCnt:= (Count div Width div 50) + 1 ;

for I:=0 to FGraphData.Count-1 do
	begin
	Data:=TDataPoint(FGraphData.Items[I]^);
	if Group=Data.Group then
		begin
		Inc(Result);
		if (Result mod ModCnt)<>0 then continue;
		if Group<0 then MarkDataPoint(MakeVector2D(Data.X,Data.Y),trunc(Data.Width),clBlack,Data.Color)
		else
			begin
			WrkCanvas.Pen.Width:=trunc(Data.Width);
			if First then Line.P1:=MakeVector2D(Data.X,Data.Y)
			else
				begin
				Line.P2:=MakeVector2D(Data.X,Data.Y);
				DrawGraphLine(Line,Data.Color);
				Line.P1:=Line.P2;
				end;
			end;
		First:=False;
 		end;
	end;
WrkCanvas.Brush.Style:=bsClear;
for I:=0 to FGraphData.Count-1 do
	begin
	Data:=TDataPoint(FGraphData.Items[I]^);
	if Group=Data.Group then LabelPoint(Data);
	end;
WrkCanvas.Pen.Width:=1;
end;



procedure TCustomGraphBox.DrawLineGraph;
var Group,Total: integer;
begin
WrkCanvas.Pen.Color:=clRed;
Total:=0;
for Group:=-MaxGroups to MaxGroups-1 do
 if GroupCount[Group]<>0 then
	begin
	if Total>=FGraphData.Count then break;
	Total:=Total + DrawGroup(Group);
	end;
end;



procedure TCustomGraphBox.ScaledRectangle(X1,Y1,X2,Y2: double; C: TColor);
var P1,P2: TPoint;
begin
P1:=GetScaledPoint(X1,Y1);
P2:=GetScaledPoint(X2,Y2);
WrkCanvas.Brush.Color:=C;
WrkCanvas.Pen.Color:=FGridColor;
WrkCanvas.Rectangle(P1.X,P1.Y,P2.X,P2.Y);
end;





procedure TCustomGraphBox.ScaledRectangle(Rect: T2DRect; C: TColor);
begin
ScaledRectangle(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom,C);
end;


function NormalizeRectangle(R: TRect): TRect;
begin
Result:=R;
SwapIntegerArgs(Result.Left>Result.Right,Result.Left,Result.Right);
SwapIntegerArgs(Result.Top>Result.Bottom,Result.Top,Result.Bottom);
end;


procedure TCustomGraphBox.ScaledFrameRectangle(X1,Y1,X2,Y2: double; C: TColor);
var P1,P2: TPoint;
var R: TRect;
begin
P1:=GetScaledPoint(X1,Y1);
P2:=GetScaledPoint(X2,Y2);
WrkCanvas.Brush.Color:=C;
R:=NormalizeRectangle(Rect(P1,P2));
WrkCanvas.FrameRect(R);
end;



function TCustomGraphBox.GetHorizBarRect(X,Y,Width: double): T2DRect;
begin
Result.Left:=0;
Result.Right:=X;
Result.Top:=Y-Width/2;
Result.Bottom:=Y+Width/2;
end;



procedure TCustomGraphBox.DrawHorzBarGraph;
var I: integer;
begin
for I:=0 to FGraphData.Count-1 do
 with TDataPoint(FGraphData.Items[I]^) do
	begin
	ScaledRectangle(GetHorizBarRect(X,Y,Width),Color);
	end;
end;



procedure TCustomGraphBox.DrawVertBarGraph;
var I: integer;
begin
for I:=0 to FGraphData.Count-1 do
 with TDataPoint(FGraphData.Items[I]^) do
	begin
	ScaledRectangle(X-Width/2,Y,X+Width/2,0,Color);
	end;
end;



procedure TCustomGraphBox.DrawGraph;
begin
case FGraphType of
 gtVertBar: DrawVertBarGraph;
 gtHorzBar: DrawHorzBarGraph;
 gtLine: DrawLineGraph;
 end;
end;


{-------------------- Basic Event Handlers ----------------------------}


function TCustomGraphBox.GetWidthStr(Axis: TAxisParams; V: double): string;
begin
if Axis.ExponentLabels then
	begin
	Result:='10^'+IntToStr(trunc(Log(V)));
	end
else Result:=FloatToStrF(Axis.ApplyScaleOffset(V),ffFixed,18,FDecimals1)+Axis.Units;
end;




function TCustomGraphBox.GetMaxTextWidth: integer;
var S1,S2: string;
var W1,W2: integer;
var M1,M2: double;
begin
Result:=0;
if not FYAxis.FShowLabels then exit;
if FFixedScaleY then begin M1:=0; M2:=FYAxis.Max - FYAxis.Min; end
else begin M1:=FYAxis.Min; M2:=FYAxis.Max; end;
S1:=GetWidthStr(FYAxis,M1);
S2:=GetWidthStr(FYAxis,M2);
W1:=WrkCanvas.TextWidth(S1);
W2:=WrkCanvas.TextWidth(S2);
Result:=Max(W1,W2)+2;
end;


procedure TCustomGraphBox.DrawGraphBorder;
begin
WrkCanvas.Pen.Style:=psSolid;
WrkCanvas.MoveTo(LeftSide,BottomSide);
WrkCanvas.LineTo(RightSide,BottomSide); WrkCanvas.LineTo(RightSide,TopSide);
WrkCanvas.LineTo(LeftSide,TopSide); WrkCanvas.LineTo(LeftSide,BottomSide);
end;







procedure TCustomGraphBox.SetLeftRightBorders(AWidth: integer);
var O,T: integer;
var TextBorder,NoTextBorder: integer;
begin
T:=GetMaxTextWidth;
TextBorder:=GetBorderWidth(T);
NoTextBorder:=FGraphBorder+BevelWidth+BorderWidth;
if YAxis.ShowLabels and (T<>0) then
	begin
	if YAxis.LabelType in [ltPrimary,ltBoth] then LeftSide:=TextBorder
	else LeftSide:=NoTextBorder;
	if YAxis.LabelType in [ltSecondary,ltBoth] then RightSide:=AWidth - TextBorder
	else RightSide:=AWidth - NoTextBorder;
	end
else
	begin
	LeftSide:=NoTextBorder;
	RightSide:=AWidth - NoTextBorder;
	end;
end;



procedure TCustomGraphBox.SetTopBottomBorders(AHeight: integer);
var O,T: integer;
var TextBorder,NoTextBorder: integer;
begin
T:=WrkCanvas.TextHeight(Title.Caption);
TextBorder:=GetBorderWidth(T);
NoTextBorder:=FGraphBorder+BevelWidth+BorderWidth;
if Title.ShowCaption and (T<>0) then TopSide:=TextBorder
else TopSide:=NoTextBorder;

T:=WrkCanvas.TextHeight('X');
TextBorder:=GetBorderWidth(T);

if XAxis.ShowLabels then BottomSide:=AHeight - TextBorder
else BottomSide:=AHeight - NoTextBorder;
end;



procedure TCustomGraphBox.SetGraphSize(AWidth,AHeight: integer);
begin
SetLeftRightBorders(AWidth);
SetTopBottomBorders(AHeight);

MaxX:=Abs(RightSide-LeftSide); MaxY:=Abs(BottomSide-TopSide);

XOrigin:=LeftSide; YOrigin:=BottomSide;
DrawGraphBorder;
end;



procedure TCustomGraphBox.DisplayTitle(AWidth: integer);
var TitleLeft,TitleTop,TitleWidth,TitleHeight: integer;
var R: TRect;
var TextArea, Gap: integer;
const BoxBorder = 2;
begin
if not FTitle.FShowCaption then exit;
TitleLeft:=XOrigin + 2;
TitleWidth:=WrkCanvas.TextWidth(FTitle.FCaption);
TitleHeight:=WrkCanvas.TextHeight(FTitle.FCaption);

TextArea:=TopSide - (BevelWidth+BorderWidth);
Gap:=(TextArea - TitleHeight) div 2;
TitleTop:=TopSide - (Gap + TitleHeight);
case FTitle.FPosition of
 tpLeft: TitleLeft:= LeftSide + BoxBorder;
 tpCenter: TitleLeft:=(AWidth - TitleWidth) div 2;
 tpRight: TitleLeft:=(RightSide - TitleWidth) - BoxBorder;
 end;

R:=Rect(0,0,TitleWidth+1,TitleHeight);
OffsetRect(R,TitleLeft,TitleTop);
WrkCanvas.TextRect(R,TitleLeft,TitleTop,FTitle.FCaption);

if not FTitle.Boxed then exit;
InflateRect(R,BoxBorder,BoxBorder);
WrkCanvas.Brush.Color := clBlack;
WrkCanvas.FrameRect(R);
end;


{------------------------------------------------------------------------------}

procedure GetLogScaleOffset(Start,Stop,Max: double; var Scale,Offset: double);
var L1,L2,DX: double;
begin
if Start<=0 then Start:=1;
L1:=Log10(Start); L2:=Log10(Stop);
DX:=L2-L1;
if DX=0 then Scale:=1e-99 else Scale:=Max/DX;
Offset:=L1;
end;



procedure GetScaleOffset(Start,Stop,Max: double; var Scale,Offset: double);
var DX: double;
begin
DX:=Stop-Start;
if DX=0 then Scale:=1e-99 else Scale:=Max/DX;
Offset:=Start;
end;




procedure TCustomGraphBox.SetScales(StartX,StartY,StopX,StopY: double);
begin
if FXAxis.Log then GetLogScaleOffset(StartX,StopX,MaxX, XScale, XOffset)
else GetScaleOffset(StartX,StopX,MaxX, XScale,XOffset);
if FXAxis.Log then GetLogScaleOffset(StartY,StopY,MaxY, YScale, YOffset)
else GetScaleOffset(StartY,StopY,MaxY, YScale,YOffset);
end;


{------------------------------------------------------------------------------}


procedure TCustomGraphBox.UpdateGraph(AWidth,AHeight: integer);
var hRG: HRGN;
begin
SetScales(FXAxis.Min,FYAxis.Min,FXAxis.Max,FYAxis.Max);
{Draw Grid}
WrkCanvas.Brush.Style:=bsClear;
WrkCanvas.Pen.Color:=FGridColor;
DisplayTitle(AWidth);
if FXAxis.Log then VerticalLogGrid(FXAxis.Min,FXAxis.Max)
else VerticalGrid(FXAxis.Lines,FXAxis.Min,FXAxis.Max);

if FYAxis.Log then HorizLogGrid(FYAxis.Min,FYAxis.Max)
else HorizontalGrid(FYAxis.Lines,FYAxis.Min,FYAxis.Max);

{Draw graph on top of grid}
hRG := CreateRectRgn(LeftSide,TopSide,RightSide,BottomSide+1);
SelectClipRgn(WrkCanvas.Handle,hRG);
DeleteObject(hRG);
DrawGraph;
SelectClipRgn(WrkCanvas.Handle, 0);
end;



procedure TCustomGraphBox.DoGraph(Canvas: TCanvas; AWidth,AHeight: integer);
begin
WrkCanvas:=Canvas;
WrkCanvas.Pen.Color:=FGridColor;
{Scale the font based on the original size of graph vs new size }
WrkCanvas.Font.Height := MulDiv(Canvas.Font.Height, AHeight, Height);
WrkCanvas.Font.Color:=Self.Font.Color;

SetGraphSize(AWidth,AHeight);
UpdateGraph(AWidth,AHeight);
BackBitmap.Canvas.CopyRect(Rect(0,0,BackBitmap.Width,BackBitmap.Height),Canvas,Rect(0,0,BackBitmap.Width,BackBitmap.Height));
if Assigned(FOnPaint) then FOnPaint(Self);
end;


procedure TCustomGraphBox.DoGraph(Canvas: TCanvas);
begin
DoGraph(Canvas,Width,Height);
end;



procedure TCustomGraphBox.Paint;
begin
inherited Paint;
DoGraph(Canvas);
end;



procedure TCustomGraphBox.ForceInvalidate(Sender: TObject);
begin
Invalidate;
end;


procedure TCustomGraphBox.HandleAxisChange(Sender: TObject);
begin
SetBaseZoomPan;
Invalidate;
end;



procedure TCustomGraphBox.SetDefaultData(V: boolean);
begin
if V=FDefaultData then exit;
FDefaultData:=V;
if FDefaultData then
	begin
	SetDataPoint(10,10,5,0,clRed);
	SetDataPoint(15,25,5,0,clGreen);
	SetDataPoint(50,50,5,0,clBlue);
	SetDataPoint(128,100,5,0,clSilver);
	end
else Self.ClearData;
end;



constructor TCustomGraphBox.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FXAxis:=TAxisParams.Create;
FYAxis:=TAxisParams.Create;
FTitle:=TGraphTitle.Create;
FXAxis.FOnChanged:=HandleAxisChange;
FYAxis.FOnChanged:=HandleAxisChange;
FTitle.FOnChanged:=ForceInvalidate;

FFixedScaleX:=False;
FFixedScaleY:=False;

SetHorizontalRange(0,75,3);
SetVerticalRange(0,75,3);

FDecimals1:=0;
FDecimals2:=0;

DefaultData:=True;
FXAxis.ShowLabels:=True;
FYAxis.ShowLabels:=True;
FGraphType:=gtHorzBar;
FXAxis.Units:='';
FYAxis.Units:='';
FTitle.FShowCaption:=False;
FXAxis.FLog:=False;
FYAxis.FLog:=False;
FXAxis.FMinorLabels:=False;
FYAxis.FMinorLabels:=False;
SetBaseZoomPan;
end;



destructor TCustomGraphBox.Destroy;
begin
FXAxis.Free;
FYAxis.Free;
FTitle.Free;
inherited Destroy;
end;



{------------------- Property Setting Procedures ----------------------}



procedure TCustomGraphBox.SetGraphType(V: TGraphType);
begin
if V<>FGraphType then
	begin
	FGraphType:=V;
	Invalidate;
	end;
end;



{------------------- Pan/Zoom Routines ------------------------------}


procedure TCustomGraphBox.SetBaseZoomPan;
begin
Extents.Horizontal.Min:=Self.XAxis.Min;
Extents.Horizontal.Max:=Self.XAxis.Max;
Extents.Vertical.Min:=Self.YAxis.Min;
Extents.Vertical.Max:=Self.YAxis.Max;
end;




procedure TCustomGraphBox.DoResetAxis(A: TAxisParams; Range: TGraphRange);
begin
A.SetMinMax(Range.Min, Range.Max);
Invalidate;
end;





procedure TCustomGraphBox.ResetPanZoom(Horizontal: boolean);
begin
if Horizontal then DoResetAxis(Self.XAxis,Extents.Horizontal)
else DoResetAxis(Self.YAxis,Extents.Vertical);
end;





procedure TCustomGraphBox.ZoomGraph(Zoom: double; Horizontal: boolean);
var D,C: double;
var A: TAxisParams;
begin                                
if Horizontal then A:=Self.XAxis
else A:=Self.YAxis;
D:=((A.Max - A.Min)/2) * Zoom;
C:=(A.Max + A.Min)/2;
A.SetMinMax(C - D,C + D);
Invalidate;
end;



procedure TCustomGraphBox.PanGraph(Mult: double; Horizontal: boolean);
var Inc: double;
var A: TAxisParams;
begin
if Horizontal then A:=Self.XAxis
else A:=Self.YAxis;
Inc:=(A.Max-A.Min) * Mult;
A.SetMinMax(A.Min + Inc,A.Max + Inc);
Invalidate;
end;



{---------------------------------------------------------------------}
{-------------------- TGraphToolbar routines -------------------------}
{---------------------------------------------------------------------}



const WideDim = 23;
const NarrowDim = 15;


procedure PutBitmapOnButton(Btn: TSpeedButton; Name: String);
var BMP: TBitmap;
begin
BMP:=TBitmap.Create;
try
BMP.Handle:=LoadBitmap(HInstance, PChar(Name));
Btn.Glyph.Assign(Bmp);
finally BMP.Free; end;
end;



procedure TGraphToolbar.SetupButton(var Btn: TAutoButton; Size: TPoint; Name, GlyphName: string);
begin
Btn := TAutoButton.Create (Self);
{This allow the control to be hidden at design time}
Btn.ControlStyle:=Btn.ControlStyle+[csNoDesignVisible];
Btn.Align:=alNone;
Btn.Name:=Name;
Btn.Width := Size.X;
Btn.Height := Size.Y;
Btn.Visible := True;
Btn.Parent := Self;
Btn.OnClick:=ButtonHandler;
PutBitmapOnButton(Btn, GlyphName);
end;



procedure TGraphToolbar.ConfigureControl;
var I,T: integer;

	procedure SetHorizModeDims;
	var T: integer;
	begin
	T:=2 * WideDim + 3 * NarrowDim  + (Self.BevelWidth+Self.BorderWidth) * 2;
	Self.Width:=T;
	Self.Height:=NarrowDim + (Self.BevelWidth+Self.BorderWidth) * 2;
	end;


	procedure SetVertModeDims;
	var T: integer;
	begin
	T:=2 * WideDim + 3 * NarrowDim + (Self.BevelWidth+Self.BorderWidth) * 2;
	Self.Height:=T;
	Self.Width:=NarrowDim + (Self.BevelWidth+Self.BorderWidth) * 2;
	end;


begin
UpBtn.Visible:=not Horizontal;
DownBtn.Visible:=not Horizontal;
LeftBtn.Visible:=Horizontal;
RightBtn.Visible:=Horizontal;
ZoomInBtn.Visible:=True;
ZoomOutBtn.Visible:=True;
ResetBtn.Visible:=True;

if Horizontal then
	begin
	SetHorizModeDims;
	LeftBtn.Top:=0;
	ZoomInBtn.Top:=0;
	ResetBtn.Top:=0;
	ZoomOutBtn.Top:=0;
	RightBtn.Top:=0;

	LeftBtn.Left:=0;
	ZoomInBtn.Left:=LeftBtn.Left+LeftBtn.Width;
	ResetBtn.Left:=ZoomInBtn.Left+ZoomInBtn.Width;
	ZoomOutBtn.Left:=ResetBtn.Left+ResetBtn.Width;
	RightBtn.Left:=ZoomOutBtn.Left+ZoomOutBtn.Width;

	end
else
	begin
	SetVertModeDims;
	UpBtn.Left:=0;
	ZoomInBtn.Left:=0;
	ResetBtn.Left:=0;
	ZoomOutBtn.Left:=0;
	DownBtn.Left:=0;

	UpBtn.Top:=0;
	ZoomInBtn.Top:=UpBtn.Top+UpBtn.Height;
	ResetBtn.Top:=ZoomInBtn.Top+ZoomInBtn.Height;
	ZoomOutBtn.Top:=ResetBtn.Top+ResetBtn.Height;
	DownBtn.Top:=ZoomOutBtn.Top+ZoomOutBtn.Height;
	end;
end;



procedure TGraphToolbar.ButtonHandler(Sender: TObject);
var Btn: TAutoButton;
const ZoomStep = 0.1;
const PanStep = 0.01;
var Multiplier: double;
begin
if not Assigned(GraphBox) then exit;
if not (Sender is TAutoButton) then exit;
Btn:=TAutoButton(Sender);
if ((GetAsyncKeyState(VK_Control) and $8000)<>0) then Multiplier:=10
else Multiplier:=1;

if Btn.Name='LeftButton' then GraphBox.PanGraph(-PanStep * Multiplier,True)
else if Btn.Name='RightButton' then GraphBox.PanGraph(PanStep * Multiplier,True)
else if Btn.Name='UpButton' then GraphBox.PanGraph(PanStep * Multiplier,False)
else if Btn.Name='DownButton' then GraphBox.PanGraph(-PanStep * Multiplier,False)
else if Btn.Name='PlusButton' then GraphBox.ZoomGraph(1-ZoomStep * Multiplier,Horizontal)
else if Btn.Name='MinuButton' then GraphBox.ZoomGraph(1+ZoomStep * Multiplier,Horizontal)
else if Btn.Name='ResetButton' then GraphBox.ResetPanZoom(Horizontal);
end;



constructor TGraphToolbar.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
BevelOuter:=bvLowered;
BorderWidth:=1;
Width:=200;
Height:=12;
SetupButton(UpBtn, Point(NarrowDim,WideDim),'UpButton',  'GBUPARROW');
SetupButton(DownBtn, Point(NarrowDim,WideDim),'DownButton',  'GBDOWNARROW');

SetupButton(LeftBtn, Point(WideDim,NarrowDim),'LeftButton', 'GBLEFTARROW');
SetupButton(ZoomInBtn, Point(NarrowDim,NarrowDim),'PlusButton',  'GBPLUS');
SetupButton(ResetBtn, Point(NarrowDim,NarrowDim),'ResetButton',  'GBRESET');
SetupButton(ZoomOutBtn, Point(NarrowDim,NarrowDim),'MinuButton',  'GBMINUS');
SetupButton(RightBtn, Point(WideDim,NarrowDim),'RightButton', 'GBRIGHTARROW');

Self.FHorizontal:=True;
ConfigureControl;
DoRepeat:=True;
end;



procedure TGraphToolbar.SetHorizontal(const Value: boolean);
begin
if FHorizontal<>Value then
	begin
	FHorizontal := Value;
	ConfigureControl;
	end;
end;




function TGraphToolbar.GetDoRepeat: boolean;
begin
Result:=LeftBtn.DoRepeat;
end;



procedure TGraphToolbar.SetDoRepeat(const Value: boolean);
begin

ResetBtn.DoRepeat:=False;

UpBtn.DoRepeat:=Value;
ZoomInBtn.DoRepeat:=Value;
ZoomOutBtn.DoRepeat:=Value;
DownBtn.DoRepeat:=Value;
LeftBtn.DoRepeat:=Value;
RightBtn.DoRepeat:=Value;
end;



function TGraphToolbar.GetEnabled: boolean;
begin
Result:=inherited Enabled;
end;



procedure TGraphToolbar.SetEnabled(const Value: boolean);
var I: integer;
begin
for I:=0 to Self.ControlCount-1 do
  Self.Controls[I].Enabled:=Value;
end;




procedure TGraphToolbar.Resize;
begin
inherited;
//ConfigureControl;
end;



procedure TGraphToolbar.Notification(AComponent: TComponent;
  Operation: TOperation);
{Handle the live deletion of a Scroll Box object controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) then
	begin
	if (AComponent = FGraphBox) then FGraphBox := nil;
	end;
end;


{---------------------------------------------------------------------}
{-------------------- TBarMeter routines -----------------------------}
{---------------------------------------------------------------------}



constructor TBarMeter.Create(AOwner: TComponent);
begin
inherited ;
Width := 150;
Height := 57;
GraphBorder:=17;
DefaultData:=False;
GraphType:=gtHorzBar;
BandSize:=10;
Banded:=True;
Centered:=False;
CenterPos:=0;
Title.ShowCaption:=True;
XAxis.Lines:=10;
XAxis.Min:=-5;
XAxis.Max:=5;
XAxis.Offset:=0;

YAxis.Lines:=1;
YAxis.Max:=20;
YAxis.Min:=0;
YAxis.ShowLabels:=False;
FBarColor:=clRed;
Position:=2;
DoubleBuffered:=True;
end;



procedure TBarMeter.SetPosition(const Value: double);
begin
if FPosition<>Value then
	begin
	FPosition := Value;
	ClearData;
	SetDataPoint(Value,10,22,0,BarColor);
	Invalidate;
	end;
end;



procedure TBarMeter.SetBarColor(const Value: TColor);
begin
if FBarColor<>Value then
	begin
	FBarColor := Value;
	ClearData;
	SetDataPoint(FPosition,10,22,0,BarColor);
	Invalidate;
	end;
end;



procedure TBarMeter.SetGraphSize;
var O,T1,T2: integer;
begin
T1:=WrkCanvas.TextWidth(LeftLabel);
T2:=WrkCanvas.TextWidth(RightLabel);

LeftSide:=GetBorderWidth(T1);
RightSide:=Width - GetBorderWidth(T2);

SetTopBottomBorders(Height);

MaxX:=Abs(RightSide-LeftSide); MaxY:=Abs(BottomSide-TopSide);

XOrigin:=LeftSide; YOrigin:=BottomSide;
DrawGraphBorder;
end;



function TBarMeter.GetHorizBarRect(X,Y,Width: double): T2DRect;
var BandWidth: double;
begin
BandWidth:=(BandSize/XScale);
if Banded then
	begin
	Result.Left:=X-BandWidth;
	Result.Right:=X+BandWidth+(1/XScale);
	end
else if Centered then
	begin
	Result.Left:=CenterPos;
	Result.Right:=X;
	end
else
	begin
	Result.Left:=XAxis.Min;
	Result.Right:=X;
	end;

Result.Top:=(Y-Width/2)+1;
Result.Bottom:=(Y+Width/2)-1;
end;




procedure TBarMeter.SetLeftLabel(const Value: string);
begin
if FLeftLabel<>Value then
	begin
	FLeftLabel := Value;
	Repaint;
	end;
end;

procedure TBarMeter.SetRightLabel(const Value: string);
begin
if FRightLabel<>Value then
	begin
	FRightLabel := Value;
	Repaint;
	end;
end;



procedure TBarMeter.SetBanded(const Value: boolean);
begin
if FBanded<>Value then
	begin
	FBanded := Value;
	Repaint;
	end;
end;



procedure TBarMeter.SetCenter(const Value: double);
begin
if  FCenter<>Value then
	begin
	FCenter := Value;
	Repaint;
	end;
end;


procedure TBarMeter.SetCentered(const Value: boolean);
begin
if FCentered<>Value then
	begin
	FCentered := Value;
	Repaint;
	end;
end;


procedure TBarMeter.SetBandSize(const Value: integer);
begin
if FBandSize<>Value then
	begin
	FBandSize := Value;
	Repaint;
	end;
end;



procedure TBarMeter.Paint;
var W,CY: integer;
begin
inherited;
Canvas.Brush.Color:=Color;
CY:=((TopSide+BottomSide) div 2) + (WrkCanvas.TextHeight(LeftLabel) div 2);
W:=WrkCanvas.TextWidth(LeftLabel);
WrkCanvas.TextOut(XOrigin-W-2,CY,LeftLabel);


CY:=((TopSide+BottomSide) div 2) + (WrkCanvas.TextHeight(RightLabel) div 2);
W:=WrkCanvas.TextWidth(RightLabel);
WrkCanvas.TextOut(RightSide+2,CY,RightLabel);
end;



{---------------------------------------------------------------------}
{-------------------- Rose diagram routines --------------------------}
{---------------------------------------------------------------------}


{------------ Basic Scaled Graph Drawing Routines --------------------}


procedure TRoseDiagram.CarteseanToPolar(X,Y: integer;var R,A:double);
begin
R:=VectorAbs2D(MakeVector2D(X-XCenter,Y-YCenter));
if R=0 then A:=0
else A:=ArcSin((Y-YCenter)/R)*RC+90;
if X<XCenter then A:=360-A;
A:=((A/AngleScale)-AngleOffset)+AngleBase;
R:=(R/RadiusScale)+RadiusBase;
end;



procedure TRoseDiagram.PolarToCartesean(R,A:double; var X,Y: integer);
begin
A:=(A-90)/RC;
X:=trunc(R*cos(A)+XCenter); Y:=trunc(R*sin(A)+YCenter);
end;



procedure TRoseDiagram.DrawSlice(A1,A2: double; R: integer;C: TColor);
var X1,Y1,X2,Y2: integer;
var Left,Right,Top,Bottom: integer;
begin
Left:=XCenter-R;Right:=XCenter+R;
Top:=YCenter-R; Bottom:=YCenter+R;
WrkCanvas.Brush.Color:=C;
PolarToCartesean(R,A2,X1,Y1);
PolarToCartesean(R,A1,X2,Y2);
if (X1=X2) and (Y1=Y2) then
	begin
	WrkCanvas.MoveTo(XCenter,XCenter);
	WrkCanvas.LineTo(X2,Y2);
	end
else WrkCanvas.Pie(Left,Top,Right,Bottom, X1, Y1, X2, Y2);
end;


function TRoseDiagram.ScaleAngle(A: double): double;
begin
Result:=(A-AngleBase+AngleOffset)*AngleScale;
end;


function TRoseDiagram.ScaleRadius(R: double): double;
begin
Result:=(R-RadiusBase)*RadiusScale;
end;


procedure TRoseDiagram.DrawScaledSlice(R,A,W: double;C: TColor);
var A1,A2: double;
begin
A1:=A-W/2; A2:=A+W/2;
DrawSlice(ScaleAngle(A1),ScaleAngle(A2),Trunc(ScaleRadius(R)),C);
end;


procedure TRoseDiagram.DrawCircle(CX,CY,Radius: integer;C: TColor);
begin
WrkCanvas.Pen.Color:=C;
WrkCanvas.Ellipse(CX-Radius,CY-Radius,CX+Radius,CY+Radius);
end;


procedure TRoseDiagram.PolarMove(R,A:double);
{ Move to Polar coordinates }
var X,Y: integer;
begin
PolarToCartesean(R,A,X,Y);
WrkCanvas.MoveTo(X,Y);
end;


procedure TRoseDiagram.PolarLine(R,A:double;C: TColor);
{ Draw Line to Move to Polar coordinate }
var X,Y: integer;
begin
PolarToCartesean(R,A,X,Y);
WrkCanvas.Pen.Color:=C;
WrkCanvas.LineTo(X,Y);
end;


function TRoseDiagram.GetScaledPoint(R,A:double): TPoint;
{ Get point scaled to grid }
begin
PolarToCartesean(ScaleRadius(R),ScaleAngle(A),Result.X,Result.Y);
end;

procedure TRoseDiagram.ScaledLine(R,A:double;C: TColor);
{ Draw Scaled And Offset Line }
begin
PolarLine(ScaleRadius(R),ScaleAngle(A),C);
end;



procedure TRoseDiagram.ScaledMove(R,A:double);
{ Move Scaled And Offset }
begin
PolarMove(ScaleRadius(R),ScaleAngle(A));
end;


{------------------ Draw Grids ----------------------------------------}

procedure TRoseDiagram.RadiusGrid(Count: integer; R1,R2: double);
{ Draw Radius Grid }
{ Number Of Lines In X }
{ Start And End Of Scale In R1 And R2 }
var O,DL : double;
var I,X : Integer;
var S: string;
begin
RadiusScale:=Radius/(R2-R1);
RadiusBase:=R1;

O:=Radius / Count;
for I:=1 to Count do DrawCircle(XCenter,YCenter,round(I*O),FGridColor);

SetTextAlign(WrkCanvas.Handle,TA_BASELINE or TA_LEFT);
SetBkMode(WrkCanvas.Handle, TRANSPARENT);

if not FRadiusLabels then exit;
WrkCanvas.Pen.Style:=psDot;
DL:=(R2-R1)/Count;
for I:=0 to Count do
	begin
	X:=XCenter+Radius+FGraphBorder;
	WrkCanvas.MoveTo(XCenter+5,YCenter+round(I*O));
	WrkCanvas.LineTo(X,YCenter+round(I*O));
	S:=FloatToStrF(R1,ffFixed,18,FDecimals2)+FRadiusUnits;
	WrkCanvas.TextOut(X,YCenter+round(I*O),S);
	R1:=R1+DL;
	end;
WrkCanvas.Pen.Style:=psSolid;
end;



procedure TRoseDiagram.AngleGrid(Count: integer; R1,R2,Off: double);
{ Draw Angle Grid }
{ Number Of Lines In X }
{ Start And End Of Scale In R1 And R2 }
var DL,O : double;
var I: Integer;
var S: string;
var X,Y: integer;

	procedure SetTextPosition(A: double);
	var F: word;
	begin
	if (A>270) or (A<90) then F:=TA_BOTTOM else F:=TA_TOP;
	if (A>350) or (A<10) or ((A>170) and (A<190)) then F:=F or TA_CENTER
	else if (A<180) then F:=F or TA_LEFT else F:=F or TA_RIGHT;
	SetTextAlign(WrkCanvas.Handle,F);
	end;

function IsActiveQuad(A: double): boolean;
begin
while A>360 do A:=A-360;
while A<0 do A:=A+360;
if (A>=0) and (A<=90) and (qdNE in FQuadrants) then Result:=True
else if (A>=90) and (A<=180) and (qdSE in FQuadrants) then Result:=True
else if (A>=180) and (A<=270) and (qdSW in FQuadrants) then Result:=True
else if ((A>=270) or (A=0)) and (qdNW in FQuadrants) then Result:=True
else Result:=False;
end;

begin
AngleScale:=360/(R2-R1);
AngleBase:=R1;
AngleOffset:=Off;
O:=360/Count;
for I:=0 to Count-1 do
 if IsActiveQuad(O*I) then
	begin
	PolarMove(0,0);
	PolarLine(Radius,round(O*I),FGridColor);
	end;
SetTextAlign(WrkCanvas.Handle,TA_BASELINE);
SetBkMode(WrkCanvas.Handle, TRANSPARENT);
DL:=(R2-R1)/Count;
for I:=0 to Count-1 do
	begin
	if IsActiveQuad(O*I+AngleOffset) then
		begin
		SetTextPosition(round(O*I+AngleOffset));
		PolarToCartesean(Radius+2,round(O*I+AngleOffset),X,Y);
		S:=FloatToStrF(R1,ffFixed,18,FDecimals1)+FAngleUnits;
		WrkCanvas.TextOut(X,Y,S);
		end;
	R1:=R1+DL;
	end;
end;



procedure TRoseDiagram.SetAngleRange(Start,Finish,Offset: double; LineCount: integer);
begin
FAngleCount:=LineCount;
FAngleMin:=Start;
FAngleMax:=Finish;
FAngleOffset:=Offset;
Invalidate;
end;


procedure TRoseDiagram.SetRadiusRange(Start,Finish: double; LineCount: integer);
begin
FRadiusCount:=LineCount;
FRadiusMin:=Start; FRadiusMax:=Finish;
Invalidate;
end;

procedure TRoseDiagram.SetModRadiusRange(Start,Finish,Width: double);
var N: integer;
begin
GetModuloRange(Start,Finish,Width,Start,Finish,N);
FRadiusCount:=N;
FRadiusMin:=Start; FRadiusMax:=Finish;
Invalidate;
end;



{------------------ Routines To Handle Stored Data Points --------------}



procedure TRoseDiagram.SetDataPoint(Angle,Radius,Width: double;Color: TColor);
begin
SetData(Radius,Angle,Width,0,'',Color);
end;


procedure TRoseDiagram.DrawLineGraph;
var I: integer;
begin
for I:=0 to FGraphData.Count-1 do
 with TDataPoint(FGraphData.Items[I]^) do
	begin
	if I=0 then ScaledMove(X,Y)
	else ScaledLine(X,Y,Color);
	end;
end;


procedure TRoseDiagram.DrawRoseGraph;
var I: integer;
begin
for I:=0 to FGraphData.Count-1 do
 with TDataPoint(FGraphData.Items[I]^) do
	begin
	DrawScaledSlice(X,Y,Width,Color);
	end;
end;


procedure TRoseDiagram.DrawGraph;
begin
case FRoseType of
 rtRose: DrawRoseGraph;
 else DrawLineGraph;
 end;
end;


{-------------------- Basic Event Handlers ----------------------------}




procedure TRoseDiagram.SetGraphSize(AWidth,AHeight: integer);
var T: integer;
var Border: integer;
begin
Border:=GetBorderWidth(WrkCanvas.TextWidth('270'));
if Border<FGraphBorder then Border:=FGraphBorder;

GraphRect.Left:=Border;
GraphRect.Top:=Border;
if FRadiusLabels then
	begin
	if (AWidth-Border)>AHeight then T:=AHeight
	else T:=AWidth-Border;
	end
else
	begin
	if AWidth>AHeight then T:=AHeight
	else T:=AWidth;
	end;


GraphRect.Right:=T-Border;
GraphRect.Bottom:=T-Border;

Radius:=(GraphRect.Right-GraphRect.Left) div 2;

XCenter:=Radius+Border;
YCenter:=Radius+Border;
DrawCircle(XCenter,YCenter,Radius,FGridColor);
end;


procedure TRoseDiagram.UpdateGraph;
var hRG: HRGN;
var R: TRect;
begin
WrkCanvas.Pen.Color:=FGridColor;
RadiusGrid(FRadiusCount,FRadiusMin,FRadiusMax);
AngleGrid(FAngleCount,FAngleMin,FAngleMax,FAngleOffset);
R:=GraphRect;
InflateRect(R,-1,-1);
hRG := CreateEllipticRgnIndirect(R);
SelectClipRgn(WrkCanvas.Handle,hRG);
DeleteObject(hRG);

DrawGraph;
SelectClipRgn(WrkCanvas.Handle, 0);
end;


procedure TRoseDiagram.DoGraph(Canvas: TCanvas; AWidth,AHeight: integer);
begin
WrkCanvas:=Canvas;
WrkCanvas.Pen.Color:=FGridColor;
WrkCanvas.Brush.Style:=bsClear;
{Scale the font based on the original size of graph vs new size }
WrkCanvas.Font.Height := MulDiv(Canvas.Font.Height, AHeight, Height);

SetGraphSize(AWidth,AHeight);
UpdateGraph;
end;


procedure TRoseDiagram.DoGraph(Canvas: TCanvas);
begin
DoGraph(Canvas,Width,Height);
end;


procedure TRoseDiagram.Paint;
begin
inherited Paint;
DoGraph(Canvas);
end;



procedure TRoseDiagram.SetDefaultData(V: boolean);
begin
if V=FDefaultData then exit;
FDefaultData:=V;
if FDefaultData then
	begin
	SetDataPoint(10,10,5,clRed);
	SetDataPoint(25,15,10,clGreen);
	SetDataPoint(50,50,15,clBlue);
	SetDataPoint(110,128,20,clYellow);
	end
else Self.ClearData;
end;



constructor TRoseDiagram.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FQuadrants:=[qdNW,qdNE,qdSE,qdSW];

SetAngleRange(0,360,0,8);
SetRadiusRange(0,75,3);
FDecimals1:=0;
FDecimals2:=0;

DefaultData:=True;
FRoseType:=rtRose;
FAngleUnits:='';
FRadiusUnits:='';
FRadiusLabels:=True;
end;


{------------------- Property Setting Procedures ----------------------}

procedure TRoseDiagram.SetQuadrants(V: TQuadSet);
begin
if V<>FQuadrants then
	begin
	FQuadrants:=V;
	Invalidate;
	end;
end;



procedure TRoseDiagram.SetRoseType(V: TRoseType);
begin
if V<>FRoseType then
	begin
	FRoseType:=V;
	Invalidate;
	end;
end;



procedure TRoseDiagram.SetAngleUnits(V: string);
begin
if V<>FAngleUnits then
	begin
	FAngleUnits:=V;
	Invalidate;
	end;
end;



procedure TRoseDiagram.SetRadiusUnits(V: string);
begin
if V<>FRadiusUnits then
	begin
	FRadiusUnits:=V;
	Invalidate;
	end;
end;



procedure TRoseDiagram.SetRadiusLabels(V: boolean);
begin
if V<>FRadiusLabels then
	begin
	FRadiusLabels:=V;
	Invalidate;
	end;
end;


{---------------------------------------------------------------------}
{-------------------- Strip Chart routines ---------------------------}
{---------------------------------------------------------------------}



constructor TStripChart.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FMarkers:=TList.Create;
FFixedScale:=False;
DoubleBuffered:=True;
ClearData;
GraphType:=gtLine;
FStripInc:=1;
StripPosition:=0;
DeleteOldValues:=True;
end;


procedure TStripChart.ClearMarkers;
var I: integer;
begin
for I:=0 to FMarkers.Count-1 do Dispose(FMarkers.Items[I]);
FMarkers.Clear;
Invalidate;
end;


destructor TStripChart.Destroy;
begin
ClearMarkers;
FMarkers.Free;
inherited Destroy;
end;



procedure TStripChart.DrawMarkers;
var I: integer;
var Data: TMarker;
var hRG: HRGN;
begin
hRG := CreateRectRgn(LeftSide,TopSide,RightSide,BottomSide);
SelectClipRgn(WrkCanvas.Handle,hRG);
DeleteObject(hRG);
for I:=0 to FMarkers.Count-1 do
	begin
	Data:=TMarker(FMarkers.Items[I]^);
	WrkCanvas.Pen.Width:=Data.Width;
	if FMajorAxis=maXAxis then
		begin
		ScaledMove(Data.Position,FYAxis.Min);
		ScaledLine(Data.Position,FYAxis.Max,Data.Color);
		end
	else
		begin
		ScaledMove(FXAxis.Min,Data.Position);
		ScaledLine(FXAxis.Max,Data.Position,Data.Color);
		end;

	end;
SelectClipRgn(WrkCanvas.Handle, 0);
WrkCanvas.Pen.Width:=1;
end;



procedure TStripChart.DoGraph(Canvas: TCanvas);
begin
inherited DoGraph(Canvas);
DrawMarkers;
end;



procedure TStripChart.Paint;
begin
inherited Paint;
end;


procedure TStripChart.ClearData;
var D: double;
begin
inherited ClearData;
ClearMarkers;
StripPosition:=0;
if FMajorAxis=maXAxis then
	begin
	D:= FXAxis.FMax - FXAxis.FMin;
	FXAxis.FMin:=0;
	FXAxis.FMax:=D;
	end
else
	begin
	D:= FYAxis.FMax - FYAxis.FMin;
	FYAxis.FMin:=0;
	FYAxis.FMax:=D;
	end;
end;


procedure TStripChart.SetFixedScale(V: boolean);
begin
if V<>FFixedScale then
	begin
	FFixedScale:=V;
	FFixedScaleX:=FFixedScale and (FMajorAxis = maXAxis);
	FFixedScaleY:=FFixedScale and (FMajorAxis = maYAxis);
	Invalidate;
	end;
end;


procedure TStripChart.SetMajorAxis(V: TMajorAxis);
begin
if V<>FMajorAxis then
	begin
	FMajorAxis:=V;
	FFixedScaleX:=FFixedScale and (FMajorAxis = maXAxis);
	FFixedScaleY:=FFixedScale and (FMajorAxis = maYAxis);
	Invalidate;
	end;
end;



procedure TStripChart.RemoveOldValues(Start: double);
var I,OldCount: integer;
var Point: TDataPoint;
var Marker: TMarker;
var Pos: double;
begin
OldCount:=FGraphData.Count;
while FGraphData.Count>0 do
	begin
	Point:=TDataPoint(FGraphData.Items[0]^);
	if (FMajorAxis=maXAxis) then Pos:=Point.X else Pos:=Point.Y;
	if Pos<Start then
		begin
		Dispose(FGraphData.Items[0]);
		FGraphData.Delete(0);
		if GroupCount[Point.Group]>0 then Dec(GroupCount[Point.Group]);
		end
	else break;
	end;
if OldCount<>FGraphData.Count then DoMinMax;
for I:=FMarkers.Count-1 downto 0 do
	begin
	Marker:=TMarker(FMarkers.Items[I]^);
	if Marker.Position<Start then
		begin
		Dispose(FMarkers.Items[I]);
		FMarkers.Delete(I);
		end;
	end;
end;


procedure TStripChart.ReReferenceData;
var I: integer;
begin
StripPosition:=FGraphData.Count*FStripInc;
for I:=0 to FGraphData.Count-1 do
	begin
	if (FMajorAxis=maXAxis) then TDataPoint(FGraphData.Items[I]^).X:=I * FStripInc
	else  TDataPoint(FGraphData.Items[I]^).Y:=I * FStripInc;
	end;
end;



procedure TStripChart.HandleAxisChange(Sender: TObject);
begin
inherited HandleAxisChange(Sender);
if (Sender = XAxis) and (FMajorAxis=maXAxis) then ReReferenceData;
if (Sender = YAxis) and (FMajorAxis=maYAxis) then ReReferenceData;
end;



procedure TStripChart.IncrementPosition;
begin
StripPosition:=StripPosition+FStripInc;
end;



function TStripChart.AddValue(Value: double; Group: integer; IncPos: boolean; Color: TColor): integer;
var Start,Finish,D: double;
var Biggest,Smallest: ^double;


	procedure GetRange(var Start,Finish: double);
	begin
	if FMajorAxis=maXAxis then begin Start:=FXAxis.Min; Finish:=FXAxis.Max end
	else begin Start:=FYAxis.Min; Finish:=FYAxis.Max end;
	end;


	procedure SetRange(var Start,Finish: double);
	begin
	{Must use FMin and FMax here so the data is not Re-referenced}
	if FMajorAxis=maXAxis then begin FXAxis.FMin:=Start; FXAxis.FMax:=Finish; end
	else begin FYAxis.FMin:=Start; FYAxis.FMax:=Finish; end ;
	end;

begin
GetRange(Start,Finish);
if Start>Finish then begin Biggest:=@Start; Smallest:=@Finish; end
else begin Smallest:=@Start; Biggest:=@Finish; end;
if DeleteOldValues then RemoveOldValues(Smallest^);
if StripPosition>Biggest^ then
	begin
	D:=Biggest^-Smallest^;
	Biggest^:=StripPosition;
	Smallest^:=Biggest^-D;
	SetRange(Start,Finish);
	end;
if FMajorAxis=maXAxis then SetDataPoint(StripPosition,Value,1,Group,Color)
else SetDataPoint(Value,StripPosition,1,Group,Color);
if IncPos then IncrementPosition;
Result:=FGraphData.Count;
end;




function TStripChart.AddValue(Value: double; Group: integer; Color: TColor): integer;
begin
Result:=AddValue(Value,Group,True,Color)
end;



procedure TStripChart.SetMarker(Width: integer; Color: TColor);
var DP: ^TMarker;
begin
New(DP);
DP^.Position:=StripPosition-FStripInc;
DP^.Width:=Width;
DP^.Color:=Color;
FMarkers.Add(DP);
Invalidate;
end;

end.
