unit ImageControls;

interface

uses Forms, Windows, Graphics,Messages, SysUtils, Classes, Controls, StdCtrls,
     ComCtrls,ExtCtrls,BitmapLib,ImageLib,Autobttn,Math,IntVector;

type TScaleChangeEvent = procedure(Sender: TObject; OldScale,NewScale: double) of object;

type TScrollImage = class(TCustomPanel)
 private
 x: TNotifyEvent;
  ScalePanel: TPanel;
  ScaleLabel: TLabel;
  ResetBtn: TAutoButton;
  FScale: double;
  FOnPaint: TNotifyEvent;
    FOnScaleChange: TScaleChangeEvent;
  procedure HandlePaint(Sender: TObject);
  function GetScaleBarScale: double;
  procedure HandleTrackChange(Sender: TObject);
  procedure SetScale(const Value: double);
  function GetCanvas: TCanvas;
  procedure HandleResetBtn(Sender: TObject);
  procedure ScaleCentered;
    function GetAutoScroll: boolean;
    procedure SetAutoScroll(const Value: boolean);
    function GetOnClick: TNotifyEvent;
    procedure SetOnClick(const Value: TNotifyEvent);
    function GetOnDblClick: TNotifyEvent;
    procedure SetOnDblClick(const Value: TNotifyEvent);
    function GetOnMouseDown: TMouseEvent;
    function GetOnMouseMove: TMouseMoveEvent;
    function GetOnMouseUp: TMouseEvent;
    procedure SetOnMouseDown(const Value: TMouseEvent);
    procedure SetOnMouseMove(const Value: TMouseMoveEvent);
    procedure SetOnMouseUp(const Value: TMouseEvent);
 protected
 public
  ScaleBar: TTrackBar;
  ScrollBox: TScrollBox;
  PaintBox: TPaintBox;
  Bitmap: TBitmap;
  Filename: string;
  procedure Invalidate; override;
  procedure Reset;
    procedure ResetScroll;
  procedure LoadFromFile(Filename: string);
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property Canvas: TCanvas read GetCanvas;
  property DockManager;
 published
  property Scale: double read FScale write SetScale;
  property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
  property Align;
  property Alignment;
  property Anchors;
  property AutoScroll: boolean read GetAutoScroll write SetAutoScroll;
  property AutoSize;
  property BevelEdges;
  property BevelInner;
  property BevelKind;
  property BevelOuter;
  property BevelWidth;
  property BiDiMode;
  property BorderStyle;
  property BorderWidth;
  property Color;
  property Constraints;
  property Ctl3D;
  property DockSite;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property Font;
  property FullRepaint;
  property OnCanResize;
  property OnClick: TNotifyEvent read GetOnClick write SetOnClick;
  property OnConstrainedResize;
  property OnContextPopup;
  property OnDblClick: TNotifyEvent read GetOnDblClick write SetOnDblClick;
  property OnScaleChange: TScaleChangeEvent read FOnScaleChange write FOnScaleChange;
  property OnDockDrop;
  property OnDockOver;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnGetSiteInfo;
  property OnMouseDown: TMouseEvent read GetOnMouseDown write SetOnMouseDown;
  property OnMouseMove: TMouseMoveEvent read GetOnMouseMove write SetOnMouseMove;
  property OnMouseUp: TMouseEvent read GetOnMouseUp write SetOnMouseUp;
  property OnMouseWheel;
  property OnMouseWheelDown;
  property OnMouseWheelUp;
  property OnResize;
  property OnStartDock;
  property OnStartDrag;
  property OnUnDock;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property UseDockManager default True;
  property Visible;
 end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TScrollImage]);
end;



{ TScrollImage }


function TScrollImage.GetScaleBarScale: double;
begin
Result:=ScaleBar.Position/100;
if Result>1 then Result:=Power(Result,2);
end;



procedure TScrollImage.HandlePaint(Sender: TObject);
var SR,DR: TRect;
begin
if (Bitmap.Width>0) and (Bitmap.Height>0) then
	begin
	SR:=Rect(0,0,Bitmap.Width,Bitmap.Height);
	DR:=Rect(0,0,PaintBox.Width,PaintBox.Height);
	OversampleBitmap(PaintBox.Canvas,DR,Bitmap);
	if Assigned(FOnPaint) then FOnPaint(Sender);
	end;
end;



procedure TScrollImage.HandleTrackChange(Sender: TObject);
var OldScale: double;
begin
OldScale:=Scale;
Scale:=GetScaleBarScale;
if Assigned(FOnScaleChange) then FOnScaleChange(Self,OldScale,Scale);
end;



procedure TScrollImage.HandleResetBtn(Sender: TObject);
begin
Reset;
end;




constructor TScrollImage.Create(AOwner: TComponent);
begin
inherited ;
Caption:='';
BevelInner:=bvNone;
BevelOuter:=bvNone;
BorderStyle:=bsNone;

ScalePanel:=TPanel.Create(Self);
ScalePanel.Name:='ScalePanel';
ScalePanel.Parent:=Self;
ScalePanel.Align:=alRight;
ScalePanel.Width:=22;
ScalePanel.BevelInner:=bvNone;
ScalePanel.BevelOuter:=bvNone;
ScalePanel.Name:='ScalePanel';

ScaleLabel:=TLabel.Create(Self);
ScaleLabel.Name:='ScaleLabel';
ScaleLabel.Parent:=ScalePanel;
ScaleLabel.Align:=alTop;
ScaleLabel.Caption:='Mag';
ScaleLabel.Name:='ScaleLabel';

ResetBtn:=TAutoButton.Create(Self);
ResetBtn.Name:='ResetBtn';
ResetBtn.Parent:=ScalePanel;
ResetBtn.Align:=alBottom;
ResetBtn.Caption:='O';
ResetBtn.Height:=21;
ResetBtn.OnClick:=HandleResetBtn;
ResetBtn.Name:='ResetBtn';

ScaleBar:=TTrackBar.Create(Self);
ScaleBar.Name:='ScaleBar';
ScaleBar.Parent:=ScalePanel;
ScaleBar.Align:=alClient;
ScaleBar.Orientation:=trVertical;
ScaleBar.Min:=2;
ScaleBar.Max:=200;
ScaleBar.Position:=100;
ScaleBar.OnChange:=HandleTrackChange;
ScaleBar.Name:='ScaleBar';
FScale:=1;

ScrollBox:=TScrollBox.Create(Self);
ScrollBox.Name:='ScrollBox';
ScrollBox.Parent:=Self;
ScrollBox.Align:=alClient;
ScrollBox.Visible:=True;
ScrollBox.HorzScrollBar.Tracking:=True;
ScrollBox.VertScrollBar.Tracking:=True;
ScrollBox.DoubleBuffered:=True;
ScrollBox.Name:='ScrollBox';

PaintBox:=TPaintBox.Create(Self);
PaintBox.Name:='PaintBox';
PaintBox.Parent:=ScrollBox;
PaintBox.Left:=0;
PaintBox.Top:=0;
PaintBox.Width:=1550;
PaintBox.Height:=1550;
PaintBox.OnPaint:=HandlePaint;
PaintBox.ParentFont:=False;
PaintBox.Visible:=True;
PaintBox.Name:='PaintBox';
Bitmap:=TBitmap.Create;
end;



destructor TScrollImage.Destroy;
begin
Bitmap.Free;
{Don't free the PaintBox or ScrollBox }
{ They are freed automatically by the Panel}
inherited;
end;

procedure TScrollImage.ResetScroll;
begin
ScrollBox.HorzScrollBar.Position:=0;
ScrollBox.VertScrollBar.Position:=0;
end;


procedure TScrollImage.Reset;
begin
ScaleBar.Position:=101;
ScaleBar.Position:=100;
HandleTrackChange(Self);
PaintBox.Invalidate;
end;


procedure TScrollImage.LoadFromFile(Filename: string);
begin
Self.Filename:=Filename;
LoadBitmapByExtension(Bitmap,FileName,False);
Reset;
end;



procedure TScrollImage.ScaleCentered;
{Scale Image and Adjust scrollbars to maintain}
{center on the same point in image}
var BaseSize: TPoint;
var PH,PV: double;
var Size: TPoint;
begin
Size:=IntVectorSubtract(ScrollBox.ClientRect.BottomRight,ScrollBox.ClientRect.TopLeft);
BaseSize:=Point(Bitmap.Width,Bitmap.Height);
PH:=ScrollBox.HorzScrollBar.Position + (Size.X/2);
PH:=PH/ScrollBox.HorzScrollBar.Range;
PV:=ScrollBox.VertScrollBar.Position + (Size.Y/2);
PV:=PV/ScrollBox.VertScrollBar.Range;

PaintBox.Width:=trunc(Bitmap.Width*Scale);
PaintBox.Height:=trunc(Bitmap.Height*Scale);
ScrollBox.HorzScrollBar.Position:=round((BaseSize.X * PH * Scale)-(Size.X/2));
ScrollBox.VertScrollBar.Position:=round((BaseSize.Y * PV * Scale)-(Size.Y/2));
Invalidate;
end;


procedure TScrollImage.SetScale(const Value: double);
begin
if FScale<>Value then
	begin
	FScale:=Value;
	ScaleCentered;
	Invalidate;
	end
end;

function TScrollImage.GetCanvas: TCanvas;
begin
Result:=PaintBox.Canvas;
end;

function TScrollImage.GetAutoScroll: boolean;
begin
Result:=ScrollBox.AutoScroll;
end;

procedure TScrollImage.SetAutoScroll(const Value: boolean);
begin
ScrollBox.AutoScroll:=Value;
end;

function TScrollImage.GetOnClick: TNotifyEvent;
begin
Result:=ScrollBox.OnClick;
end;

procedure TScrollImage.SetOnClick(const Value: TNotifyEvent);
begin
ScrollBox.OnClick:=Value;
PaintBox.OnClick:=Value;
end;

function TScrollImage.GetOnDblClick: TNotifyEvent;
begin
Result:=ScrollBox.OnDblClick;
end;

procedure TScrollImage.SetOnDblClick(const Value: TNotifyEvent);
begin
ScrollBox.OnDblClick:=Value;
PaintBox.OnDblClick:=Value;
end;

function TScrollImage.GetOnMouseDown: TMouseEvent;
begin
Result:=PaintBox.OnMouseDown;
end;

function TScrollImage.GetOnMouseMove: TMouseMoveEvent;
begin
Result:=PaintBox.OnMouseMove;
end;

function TScrollImage.GetOnMouseUp: TMouseEvent;
begin
Result:=PaintBox.OnMouseUp;
end;


procedure TScrollImage.SetOnMouseDown(const Value: TMouseEvent);
begin
PaintBox.OnMouseDown:=Value;
end;

procedure TScrollImage.SetOnMouseMove(const Value: TMouseMoveEvent);
begin
PaintBox.OnMouseMove:=Value;
end;

procedure TScrollImage.SetOnMouseUp(const Value: TMouseEvent);
begin
PaintBox.OnMouseUp:=Value;
end;

procedure TScrollImage.Invalidate;
begin
inherited Invalidate;
if ScrollBox<>nil then ScrollBox.Invalidate;
if PaintBox<>nil then PaintBox.Invalidate;
end;

end.
