unit PlayerBar;

interface

uses Windows,Classes,SysUtils,ExtCtrls,Controls,Graphics,
     FancyButton,Autobttn;

type TPlayerState = (psNoFile,psStopped,psPaused,psPlaying);
type TPlayerCommand = (pcRewind,pcStepBack,pcPlay,pcPause,pcResume,pcStop,pcStepForward,pcEnd);
type TButtonTags = (btBegin=0,btStepBack=1,btPlay=2,btStop=3,btStepForward=4,btEnd=5);

type TCommandEvent = function(Command: TPlayerCommand): boolean of Object;


type TPlayerBar = class(TCustomPanel)
 private
  FOnCommand: TCommandEvent;
  FCenterButtons: boolean;
  FPlayerState: TPlayerState;
  FStandardFrame: boolean;
  FGradient: boolean;
  FDownInvert: boolean;
  FFrameLightest: TColor;
  FFrameLight: TColor;
  FButtonColor: TColor;
  FFrameDarkest: TColor;
  FGradColor2: TColor;
  FGradColor1: TColor;
  FFrameDark: TColor;
 procedure AddButton(var Button: TFancySpeedBtn; GlyphCount,Tag: integer; GlyphName, Hint: string);
  procedure ResizeButtons;
  procedure UpdateButtonState;
  procedure HandleButtonClick(Sender: TObject);
  function HandleCommand(Com: TPlayerCommand): boolean;
  procedure SetCenterButtons(const Value: boolean);
  procedure DoCenterButtons;
  procedure UpdateButtons;
  procedure SetPlayerState(const Value: TPlayerState);
  procedure SetAllButtons;
  procedure SetButon(Button: TFancySpeedBtn);
  procedure SetButtonColor(const Value: TColor);
  procedure SetDownInvert(const Value: boolean);
  procedure SetFrameDark(const Value: TColor);
  procedure SetFrameDarkest(const Value: TColor);
  procedure SetFrameLight(const Value: TColor);
  procedure SetFrameLightest(const Value: TColor);
  procedure SetGradColor1(const Value: TColor);
  procedure SetGradColor2(const Value: TColor);
  procedure SetGradient(const Value: boolean);
  procedure SetStandardFrame(const Value: boolean);
protected
   procedure Resize; override;
   procedure Loaded; override;
 public
  BeginBtn, StepBackBtn, PlayBtn,
  StopBtn, StepForwardBtn, EndBtn: TFancySpeedBtn;
  ImageList: TImageList;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property PlayerState: TPlayerState read FPlayerState write SetPlayerState;
 published
  property OnCommand: TCommandEvent read FOnCommand write FOnCommand;
  property Align;
  property Alignment;
  property Anchors;
  property AutoSize;
  property BevelInner;
  property BevelOuter;
  property BevelWidth;
  property BiDiMode;
  property BorderWidth;
  property BorderStyle;
  property CenterButtons: boolean read FCenterButtons write SetCenterButtons default False;
  property Caption;
  property Color;
  property Constraints;
  property Ctl3D;
  property UseDockManager default True;
  property DockSite;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property FullRepaint;
  property Font;
  property Locked;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PopupMenu;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Visible;
  property OnCanResize;
  property OnClick;
  property OnConstrainedResize;
  property OnContextPopup;
  property OnDockDrop;
  property OnDockOver;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnGetSiteInfo;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnResize;
  property OnStartDock;
  property OnStartDrag;
  property OnUnDock;
  property FrameDarkest: TColor read FFrameDarkest write SetFrameDarkest;
  property FrameDark: TColor read FFrameDark write SetFrameDark;
  property FrameLightest: TColor read FFrameLightest write SetFrameLightest;
  property FrameLight: TColor read FFrameLight write SetFrameLight;
  property GradColor1: TColor read FGradColor1 write SetGradColor1 default clWhite;
  property GradColor2: TColor read FGradColor2 write SetGradColor2 default clBlack;
  property Gradient: boolean read FGradient write SetGradient default False;
  property DownInvert: boolean read FDownInvert write SetDownInvert default false;
  property ButtonColor: TColor read FButtonColor write SetButtonColor default clBtnFace;
  property StandardFrame: boolean read FStandardFrame write SetStandardFrame default False;
end;

procedure Register;

implementation


{$R MPlayer.res}

procedure Register;
begin
RegisterComponents('Fountain', [TPlayerBar]);
end;

{ TPlayerBar }



procedure TPlayerBar.ResizeButtons;
var I,BtnWidth,AW,RM: integer;
begin
AW:=Width - ((BorderWidth + BevelWidth) * 2);
BtnWidth:=AW div ComponentCount;
{Calculate the amount left over}
RM:=(AW-(BtnWidth * ComponentCount))+BtnWidth;
for I:=0 to ComponentCount-1 do
 if Components[I] is TFancySpeedBtn then
	begin
	TFancySpeedBtn(Components[I]).Align:=alLeft;
 	TFancySpeedBtn(Components[I]).Width:=BtnWidth;
 	end;
{Add in interization remained for esthetics}
TFancySpeedBtn(Components[ComponentCount-1]).Width:=RM+1;
end;


procedure TPlayerBar.DoCenterButtons;
var I,Width1,Margin: integer;
var FullBorder,WorkWidth: integer;
begin
FullBorder:=((BorderWidth + BevelWidth) * 2);
WorkWidth:=Width - FullBorder;
Width1:=Height - FullBorder+1;
Margin:=(WorkWidth - (Width1*ComponentCount)) div 2;
for I:=0 to ComponentCount-1 do
 if Components[I] is TFancySpeedBtn then
	begin
	TFancySpeedBtn(Components[I]).Align:=alNone;
 	TFancySpeedBtn(Components[I]).Width:=Width1;
 	TFancySpeedBtn(Components[I]).Height:=Width1;
        TFancySpeedBtn(Components[I]).Top:=BorderWidth + BevelWidth;
        TFancySpeedBtn(Components[I]).Left:=Margin;
        Inc(Margin,Width1);
 	end;

end;



procedure TPlayerBar.UpdateButtons;
begin
if CenterButtons then DoCenterButtons
else ResizeButtons;
end;



function TPlayerBar.HandleCommand(Com: TPlayerCommand): boolean;
begin
Result:=False;
if assigned(FOnCommand) then Result:=FOnCommand(Com);
end;


procedure TPlayerBar.UpdateButtonState;
var BS: string[6];
begin
case PlayerState of
 psNoFile: BS:='000000';
 psStopped: BS:='001000';
 psPaused: BS:='111111';
 psPlaying: BS:='111111';
 end;
BeginBtn.Enabled:=BS[1]='1';
StepBackBtn.Enabled:=BS[2]='1';
PlayBtn.Enabled:=BS[3]='1';
StopBtn.Enabled:=BS[4]='1';
StepForwardBtn.Enabled:=BS[5]='1';
EndBtn.Enabled:=BS[6]='1';
end;



procedure TPlayerBar.HandleButtonClick(Sender: TObject);
var Btn: TFancySpeedBtn;
begin
Btn:=TFancySpeedBtn(Sender);
case TButtonTags(Btn.Tag) of
 btBegin:
	begin
	if PlayBtn.Down then HandleCommand(pcRewind);
	end;
 btStepBack:
 	begin
	HandleCommand(pcStepBack);
 	end;
 btPlay:
	begin
	if PlayBtn.Down then
		begin
                if PlayerState=psPaused then HandleCommand(pcResume)
                else if HandleCommand(pcPlay) then PlayerState:=psPlaying;
		end
	else
		begin
		PlayerState:=psPaused;
		HandleCommand(pcPause);
		end;
	end;
 btStop:
	begin
	if PlayBtn.Down or
	   (PlayerState=psPaused) then
		begin
		PlayBtn.Down:=False;
		PlayerState:=psStopped;
		HandleCommand(pcStop);
		end;
	end;
 btStepForward:
 	begin
	HandleCommand(pcStepForward);
 	end;
 btEnd:
 	begin
 	if PlayBtn.Down then HandleCommand(pcEnd);
 	end;
 end;
end;


procedure TPlayerBar.AddButton(var Button: TFancySpeedBtn; GlyphCount,Tag: integer; GlyphName, Hint: string);
begin
Button:=TFancySpeedBtn.Create(Self);
Button.Caption:='';
Button.Parent:=Self;
Button.Align:=alLeft;
Button.Glyph.Handle:=LoadBitmap(HInstance, PChar(GlyphName));
Button.NumGlyphs:=GlyphCount;
Button.Tag:=Tag;
Button.Hint:=Hint;
Button.ShowHint:=True;
Button.AllowAllUp:=True;
Button.OnClick:=HandleButtonClick;
end;



constructor TPlayerBar.Create(AOwner: TComponent);
begin
inherited;
FCenterButtons:=False;
ButtonColor:=clBtnFace;
GradColor1:=clWhite;
GradColor2:=clBlack;
FrameDarkest:=cl3DDKShadow;
FrameDark:=clBtnShadow;
FrameLight:=cl3DLight;
FrameLightest:=clBtnHighlight;
Gradient:=False;

AddButton(BeginBtn,2,0,'Beginning','Rewind');
AddButton(StepBackBtn,2,1,'StepBack','Step Backwards');
AddButton(PlayBtn,4,2,'Play','Play/Pause');
PlayBtn.GroupIndex:=1;
AddButton(StopBtn,2,3,'Stop','Stop');
AddButton(StepForwardBtn,2,4,'StepForward','Step Forward');
AddButton(EndBtn,2,5,'Ending','To End');
if csDesigning in ComponentState then PlayerState:=psPlaying
else PlayerState:=psStopped;
SetAllButtons;
end;



destructor TPlayerBar.Destroy;
var I: integer;
begin
{Don't free Buttons - The panel as a container does it for us.}
inherited;
end;



procedure TPlayerBar.Resize;
begin
inherited;
UpdateButtons;
end;



procedure TPlayerBar.Loaded;
begin
inherited;
UpdateButtons;
Caption:='';
end;

procedure TPlayerBar.SetCenterButtons(const Value: boolean);
begin
if FCenterButtons<>Value then
	begin
	FCenterButtons := Value;
	UpdateButtons;
	end;
end;

procedure TPlayerBar.SetPlayerState(const Value: TPlayerState);
begin
if FPlayerState=Value then exit;
FPlayerState := Value;
case FPlayerState of
 psStopped, psNoFile: PlayBtn.Down:=False;
 psPaused: PlayBtn.Down:=False;
 psPlaying: PlayBtn.Down:=True;
 end;
UpdateButtonState;
end;



procedure TPlayerBar.SetButon(Button: TFancySpeedBtn);
begin
if Button=nil then exit;
Button.FrameDarkest:=FFrameDarkest;
Button.FrameDark:=FFrameDark;
Button.FrameLightest:=FFrameLightest;
Button.FrameLight:=FFrameLight;
Button.GradColor1:=FGradColor1;
Button.GradColor2:=FGradColor2;
Button.Gradient:=FGradient;
Button.DownInvert:=FDownInvert;
Button.Color:=FButtonColor;
Button.StandardFrame:=FStandardFrame;
end;


procedure TPlayerBar.SetAllButtons;
begin
SetButon(BeginBtn);
SetButon(StepBackBtn);
SetButon(PlayBtn);
SetButon(StopBtn);
SetButon(StepForwardBtn);
SetButon(EndBtn);
end;


procedure TPlayerBar.SetButtonColor(const Value: TColor);
begin
if FButtonColor<>Value then
	begin
	FButtonColor := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetDownInvert(const Value: boolean);
begin
if FDownInvert<>Value then
	begin
	FDownInvert := Value;
	SetAllButtons;
	end;
end;



procedure TPlayerBar.SetFrameDark(const Value: TColor);
begin
if FFrameDark<>Value then
	begin
	FFrameDark := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetFrameDarkest(const Value: TColor);
begin
if FFrameDarkest <> Value then
	begin
	FFrameDarkest := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetFrameLight(const Value: TColor);
begin
if FFrameLight <> Value then
	begin
	FFrameLight := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetFrameLightest(const Value: TColor);
begin
if FFrameLightest <> Value then
	begin
	FFrameLightest := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetGradColor1(const Value: TColor);
begin
if FGradColor1 <> Value then
	begin
	FGradColor1 := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetGradColor2(const Value: TColor);
begin
if FGradColor2 <> Value then
	begin
	FGradColor2 := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetGradient(const Value: boolean);
begin
if FGradient <> Value then
	begin
	FGradient := Value;
	SetAllButtons;
	end;
end;


procedure TPlayerBar.SetStandardFrame(const Value: boolean);
begin
if FStandardFrame <> Value then
	begin
	FStandardFrame := Value;
	SetAllButtons;
	end;
end;

end.
