unit Plotwind;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls,Vector,PaintPan,IntVector,GraphicsLib;


type TPlotWindow = class(TPaintPanel)
private
  FBorder: Integer;
  PScale: double;
  ScreenOffset: T2DVector;
  PlanVIew: boolean;
  FOnPaint: TNotifyEvent;
  FDrawFrame: boolean;
  FFrameColor: TColor;
  procedure DrawFrameLines;
  procedure SetDrawFrame(const Value: boolean);
  procedure SetFrameColor(const Value: TColor);
 protected
  procedure Paint; override;
 public
  MinExtents,MaxExtents: T3DVector;
  constructor Create(AOwner: TComponent); override;
  procedure ClearBackground;
  procedure Plotline(Com: Char; VX,VY,VZ: double); overload;
  procedure Plotline(Com: Char; V: T3DVector); overload;
  procedure PlotPolygon(A: array of T3DVector);
  procedure PlotTriangle(P1, P2, P3: T3DVector);
  procedure PlotQuad(P1,P2,P3,P4: T3DVector);
  procedure PlotRect(TopLeft, BottomRight: T3DVector);
  procedure PlotPoint(Pos: T3DVector); overload;
  procedure PlotPoint(Pos: T3DVector; Size: integer; C: TColor); overload;
  procedure PlotArrowLine(V1, V2: T3DVector; StartArrow, EndArrow: boolean; Length, Width: integer);
  procedure PlotText(Text: string; Pos: T3DVector); overload;
  procedure PlotText(Text: string; Pos: T3DVector; Off: TPoint); overload;
  procedure PlotText(Text: string; Height: double; Pos: T3DVector); overload;
  procedure PlotText(Text: string; Pos: T3DVector; Off: TPoint; Flags: Cardinal); overload;
  procedure BoxText(Text: string; Pos: T3DVector; HorizAlign: THorizAlign; VertAlign: TVertAlign);
  procedure RealToPixel(V: T3DVector; var P: TPoint); overload;
  procedure RealToPixel(RX,RY,RZ: double;var X,Y: integer); overload;
  procedure RealToPixel(V: T2DVector; var P: TPoint); overload;
  procedure PixelToReal(X,Y: integer; var RX,RY,RZ: double); overload;
  procedure PixelToReal(X,Y: integer; var V: T2DVector); overload;
  procedure PixelToReal(X,Y: integer; var V: T3DVector); overload;
  procedure SetScale(Plan: boolean; MinX,MaxX,MinY,MaxY,MinZ,MaxZ: double); overload;
  procedure SetScale(Plan: boolean; Min,Max: T3DVector); overload;
  procedure SetClipWindow(R: TRect);
  procedure ClearClipWindow;
  procedure SetRealClipWindow(TopLeft,BottomRight: T2DVector);
 published
  property BorderPercent: Integer read FBorder write FBorder default 5;
  property Scale: double read PScale;
  property Offset: T2DVector read ScreenOffset;
  property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
  property DrawFrame: boolean read FDrawFrame write SetDrawFrame default False;
  property FrameColor: TColor read FFrameColor write SetFrameColor;
 end;

procedure Register;


implementation



procedure TPlotWindow.SetClipWindow(R: TRect);
var hRG: HRGN;
begin
hRG := CreateRectRgn(R.Left,R.Top,R.Right,R.Bottom);
SelectClipRgn(Canvas.Handle,hRG);
DeleteObject(hRG);
end;



procedure TPlotWindow.SetRealClipWindow(TopLeft,BottomRight: T2DVector);
var TL,BR: TPoint;
begin
RealToPixel(TopLeft,TL);
RealToPixel(BottomRight,BR);
SetClipWindow(Rect(TL.X,TL.Y,BR.X,BR.Y));
end;



procedure TPlotWindow.ClearClipWindow;
begin
SelectClipRgn(Canvas.Handle, 0);
end;



constructor TPlotWindow.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FBorder:=5;
DrawFrame:=False;
FrameColor:=clBlack;
end;



procedure TPlotWindow.Paint;
begin
{We override the FOnPaint in parent}
inherited Paint;
if DrawFrame then DrawFrameLines;
if Assigned(FOnPaint) then FOnPaint(Self);
end;




procedure TPlotWindow.ClearBackground;
begin
Canvas.Brush.Color:=Color;
Canvas.FillRect(Canvas.ClipRect);
end;


procedure TPlotWindow.DrawFrameLines;
var BP: TPoint;
var R: TRect;
begin
BP.X:=MulDiv(Width,FBorder,100);
BP.Y:=MulDiv(Height,FBorder,100);
R:=Rect(BP,IntVectorSubtract(Point(Width,Height),BP));
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=FrameColor;
Canvas.Pen.Mode:=pmCopy;
Canvas.Rectangle(R);
end;



procedure TPlotWindow.PixelToReal(X,Y: integer; var RX,RY,RZ: double);
begin
if PScale =0 then exit;
RX:=(X+ScreenOffset.X)/PScale;
if PlanView then
	begin
	RY:=(ScreenOffset.Y-Y)/PScale;
	RZ:=0;
	end
else
	begin
	RZ:=(ScreenOffset.Y-Y)/PScale;
	RY:=0;
	end;
end;



procedure TPlotWindow.PixelToReal(X,Y: integer; var V: T3DVector);
begin
PixelToReal(X,Y,V.X,V.Y,V.Z);
end;


procedure TPlotWindow.PixelToReal(X,Y: integer; var V: T2DVector);
var Pos: T3DVector;
begin
PixelToReal(X,Y,Pos);
V:=Vector3Dto2DZ(Pos,PlanView);
end;



procedure TPlotWindow.RealToPixel(RX,RY,RZ: double; var X,Y: integer);
begin
X:=Trunc((PScale*RX)-ScreenOffset.X);
if PlanView then Y:=Trunc(ScreenOffset.Y-(PScale*RY))
else Y:=Trunc(ScreenOffset.Y-(PScale*RZ));
end;




procedure TPlotWindow.RealToPixel(V: T2DVector; var P: TPoint);
begin
RealToPixel(V.X,V.Y,V.Y,P.X,P.Y);
end;




procedure TPlotWindow.RealToPixel(V: T3DVector; var P: TPoint);
begin
RealToPixel(V.X,V.Y,V.Z,P.X,P.Y);
end;




procedure  TPlotWindow.Plotline(Com: Char; VX,VY,VZ: double);
var X,Y: integer;
begin
RealToPixel(VX,VY,VZ,X,Y);
case Com of
 'M': Canvas.MoveTo(X,Y);
 'D': Canvas.LineTo(X,Y);
 end;
end;



procedure TPlotWindow.Plotline(Com: Char; V: T3DVector);
begin
PlotLine(Com,V.X,V.Y,V.Z);
end;




procedure TPlotWindow.PlotArrowLine(V1,V2: T3DVector; StartArrow,EndArrow: boolean; Length,Width: integer);
var P1,P2: TPoint ;
begin
RealToPixel(V1,P1);
RealToPixel(V2,P2);
DrawArrowLine(Canvas,P1,P2,StartArrow,EndArrow,Length,Width);
end;



procedure  TPlotWindow.PlotPolygon(A: array of T3DVector);
var P: array of TPoint ;
var I: integer;
begin
SetLength(P,Length(A));
for I:=0 to High(A) do RealToPixel(A[I],P[I]);
Canvas.Polygon(P);
end;




procedure  TPlotWindow.PlotQuad(P1,P2,P3,P4: T3DVector);
var P: array of TPoint ;
begin
SetLength(P,4);
RealToPixel(P1,P[0]);
RealToPixel(P2,P[1]);
RealToPixel(P3,P[2]);
RealToPixel(P4,P[3]);
Canvas.Polygon(P);
end;



procedure  TPlotWindow.PlotTriangle(P1,P2,P3: T3DVector);
var P: array of TPoint ;
begin
SetLength(P,3);
RealToPixel(P1,P[0]);
RealToPixel(P2,P[1]);
RealToPixel(P3,P[2]);
Canvas.Polygon(P);
end;



procedure  TPlotWindow.PlotPoint(Pos: T3DVector);
var P: TPoint ;
begin
RealToPixel(Pos,P);
DrawRoundHandle(Canvas,P,clRed);
end;





procedure  TPlotWindow.PlotPoint(Pos: T3DVector; Size: integer; C: TColor);
var R: TRect;
var P: TPoint;
begin
RealToPixel(Pos,P);
R:=Rect(P.X-Size,P.Y-Size,P.X+Size,P.Y+Size);
Canvas.Brush.Style:=bsSolid;
Canvas.Brush.Color:=C;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=C;
Canvas.Pen.Mode:=pmCopy;
Canvas.Ellipse(R);
end;



procedure  TPlotWindow.PlotText(Text: string; Pos: T3DVector; Off: TPoint; Flags: Cardinal);
var P: TPoint;
begin
RealToPixel(Pos,P);
SetTextAlign(Canvas.Handle,Flags);
Canvas.TextOut(P.X+Off.X,P.Y+Off.Y,Text);
end;


procedure  TPlotWindow.BoxText(Text: string; Pos: T3DVector; HorizAlign: THorizAlign; VertAlign: TVertAlign);
var P: TPoint;
begin
RealToPixel(Pos,P);
DrawTextBox(Canvas, Text, P, HorizAlign,VertAlign);
end;



procedure  TPlotWindow.PlotText(Text: string; Pos: T3DVector; Off: TPoint);
begin
PlotText(Text,Pos,Off,TA_BASELINE);
end;



procedure  TPlotWindow.PlotText(Text: string; Pos: T3DVector);
var P: TPoint ;
begin
PlotText(Text,Pos,Point(0,0))
end;


procedure  TPlotWindow.PlotText(Text: string; Height: double; Pos: T3DVector);
var P: TPoint ;
begin
Canvas.Font.Height:=Trunc(PScale*Height);
RealToPixel(Pos,P);
SetTextAlign(Canvas.Handle,TA_BASELINE);
Canvas.TextOut(P.X,P.Y,Text);
end;



procedure  TPlotWindow.PlotRect(TopLeft,BottomRight: T3DVector);
var P: array of TPoint ;
var TopRight,BottomLeft: T3DVector;
begin
SetLength(P,4);
TopRight:=MakeVector3D(BottomRight.X,TopLeft.Y,TopLeft.Z);
BottomLeft:=MakeVector3D(TopLeft.X,BottomRight.Y,BottomRight.Z);
RealToPixel(TopLeft,P[0]);
RealToPixel(TopRight,P[1]);
RealToPixel(BottomRight,P[2]);
RealToPixel(BottomLeft,P[3]);
Canvas.Polygon(P);
end;



procedure  TPlotWindow.SetScale(Plan: boolean; MinX,MaxX,MinY,MaxY,MinZ,MaxZ: double);
var Aspect: double;
var Min,Max,Delta: T2DVector;
var B: double;
begin
MinExtents:=MakeVector3D(MinX,MinY,MinZ);
MaxExtents:=MakeVector3D(MaxX,MaxY,MaxZ);
PlanView:=Plan;
Min:=Vector.Vector3Dto2DZ(MinExtents,PlanView);
Max:=Vector.Vector3Dto2DZ(MaxExtents,PlanView);
Delta:=VectorSubtract2D(Max,Min);

Aspect:=Height/Width;
B:=1 - (FBorder/100);
if (Aspect*Delta.X)>=Delta.Y then
	begin
	PScale:=(Width*B)/Delta.X;
	ScreenOffset.X:=Trunc(PScale*Min.X) - ((Width-Trunc(Delta.X*PScale)) div 2);
	ScreenOffset.Y:=Trunc(Height+PScale*Min.Y) - (((Height-Trunc(Delta.Y*PScale)) div 2)+1);
	end
else
	begin
	PScale:=(Height*B)/Delta.Y;
	ScreenOffset.X:=Trunc(PScale*Min.X) - ((Width-Trunc(Delta.X*PScale)) div 2);
	ScreenOffset.Y:=Trunc(Height+PScale*Min.Y) - (((Height-Trunc(Delta.Y*PScale)) div 2)+1);
	end;
end;



procedure  TPlotWindow.SetScale(Plan: boolean; Min,Max: T3DVector);
begin
SetScale(Plan,Min.X,Max.X,Min.Y,Max.Y,Min.Z,Max.Z);
end;



procedure Register;
begin
RegisterComponents('Fountain', [TPlotWindow]);
end;



procedure TPlotWindow.SetDrawFrame(const Value: boolean);
begin
if FDrawFrame<>Value then
	begin
	FDrawFrame:=Value;
	Invalidate;
	end;
end;



procedure TPlotWindow.SetFrameColor(const Value: TColor);
begin
if FFrameColor<>Value then
	begin
	FFrameColor := Value;
	Invalidate;
	end;
end;



end.
 