unit Realspin;

interface

uses Windows,Messages,Forms,Types,Classes,Controls,StdCtrls,ExtCtrls,Graphics,
     SysUtils,SpinButtons,DSupport,Harmonizer;

{================== TRealSpin - Real number spin edit ==========================}

type TNumberFormat = (nfStandard,nfScientific,nfFixed,nfThousands);

type TRealIncEvent = procedure(Sender: TObject; Inc: boolean; var Value: double) of object;

type
  TRealSpin = class(TCustomSpin)
  private
    FFormat: TNumberFormat;
    FValue: double;
    FDigits: integer;
    FMinValue: double;
    FMaxValue: double;
    FIncrement: double;
    FLastValid: double;
    FOwnerIncrement: boolean;
    FOnIncrement: TRealIncEvent;
    procedure HandleUserIncrement(Inc: boolean);
    procedure SetValue (NewValue: double);
    procedure SetNoEventValue(NewValue: double);
    function CheckValue (NewValue: double): double;
    function TextToDouble: double;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMWinIniChange(var Message: TWMWinIniChange); message CM_WININICHANGE;
    procedure UpClick (Sender: TObject); virtual;
    procedure DisplayValue(V: double);
    procedure SetMaxValue(const Value: double);
    procedure SetMinValue(const Value: double);
    procedure SetDigits(const Value: integer);
  protected
    function IsValidChar(Key: Char): Boolean; virtual;
    procedure DownClick (Sender: TObject); virtual;
    procedure KeyPress(var Key: Char); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Change; override;
  public
    procedure ChainedIncrement(ParentSpin: TRealSpin; Inc: boolean; var Value: double);
    constructor Create(AOwner: TComponent); override;
    property NoEventValue: double read FValue write SetNoEventValue;
  published
    property Increment: double read FIncrement write FIncrement ;
    property MaxValue: double read FMaxValue write SetMaxValue;
    property MinValue: double read FMinValue write SetMinValue;
    property Digits: integer read FDigits write SetDigits default 5;
    property NumFormat: TNumberFormat read FFormat write FFormat default nfStandard;
    property Value: double read FValue write SetValue;
    property OwnerIncrement: boolean read FOwnerIncrement write FOwnerIncrement default False;
    property OnIncrement: TRealIncEvent read FOnIncrement write FOnIncrement;
    property AutoSelect;
    property AutoSize;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text stored False;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation



procedure Register;
begin
  RegisterComponents('Fountain', [TRealSpin]);
end;



{================== TRealSpin ================================================}



procedure TRealSpin.ChainedIncrement(ParentSpin: TRealSpin; Inc: boolean; var Value: double);
{Recursive incrementor for multi-stage real spin display i.e. Minutes, seconds and milliseconds}
{The routine will increment each spinner in the chain and wrap around when needed}
{Each spinner must be a TIntegerSpin, and OwnerIncrement must be enabled }
{The Max and Min value must be set to the limit values you want - they are used to wrap or limit}
{If Wrap is enabled, the value wrap and the next spinner up the chain is advanced}
{If Wrap is not enabled, the value is clamped to the upper and lower limit values}
{This routine should be called from you increment event of each spinner including the last one  }
var T: double;

	procedure HandleOverflow;
	begin
	if ParentSpin<>nil then
		begin
		T:=ParentSpin.Value;
		if assigned(ParentSpin.OnIncrement) then ParentSpin.OnIncrement(Self,Inc,T);
		ParentSpin.Value:=T;
		end;
	end;

begin
if Inc then
	begin
	Value:=Value+1;
	if Value>=Self.MaxValue then
		begin
		if Self.Wrap then
			begin
			Value:=Self.MinValue;
			HandleOverflow;
			end
		else Value:=Self.MaxValue;
		end;
	end
else
	begin
	Value:=Value-1;
	if Value<Self.MinValue then
		begin
		if Self.Wrap then
			begin
			Value:=Self.MaxValue-1;
			HandleOverflow;
			end
		else Value:=Self.MinValue;
		end;
	end;
end;


constructor TRealSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FButton.OnUpClick := UpClick;
FButton.OnDownClick := DownClick;
FIncrement := 1;
FFormat:=nfStandard;
FDigits:=5;
FOwnerIncrement:=False;
FLastValid:=Value;
end;



procedure TRealSpin.HandleUserIncrement(Inc: boolean);
var VT: double;
begin
VT:=Value;
if Assigned(FOnIncrement) then FOnIncrement(Self,Inc,VT);
Value:=VT;
end;


procedure TRealSpin.UpClick(Sender: TObject);
var M: integer;
begin
if ((GetAsyncKeyState(VK_Control) and $8000)<>0) then M:=10 else M:=1;;
if ReadOnly then MessageBeep(0)
else if FOwnerIncrement then HandleUserIncrement(True)
else Value := Value + FIncrement * M;
end;


procedure TRealSpin.DownClick(Sender: TObject);
var M: integer;
begin
if ((GetAsyncKeyState(VK_Control) and $8000)<>0) then M:=10 else M:=1;;
if ReadOnly then MessageBeep(0)
else if FOwnerIncrement then HandleUserIncrement(False)
else Value := Value - FIncrement * M;
end;




function RemoveThousandSeparators(S: string): string;
var I: integer;
begin
for I:=1 to Length(S) do
 if S[I]<>IndSystemData.ThousandSeparator then Result:=Result + S[I];
end;



procedure TRealSpin.CMExit(var Message: TCMExit);
begin
inherited;
if Text='' then Value:=FLastValid;
SetValue(Value);
end;



function TRealSpin.TextToDouble: double;
var S: string;
begin
S:=RemoveThousandSeparators(Text);
if TryStrToFloat(S,Result) then
	begin
	Result:=CheckValue(Result);
	FLastValid:=Result;
	end
else Result:=FLastValid;
end;


procedure TRealSpin.DisplayValue(V: double);
begin
case FFormat of
 nfScientific: Text:=FloatToStrF(CheckValue(V),ffExponent,FDigits,4);
 nfFixed: Text:=FloatToStrF(CheckValue (V),ffFixed,18,FDigits);
 nfStandard: Text:=FloatToStr(CheckValue (V));
 nfThousands: Text:=FloatToStrF(CheckValue (V),ffNumber,18,FDigits);
 end;
end;



procedure TRealSpin.SetValue(NewValue: double);
begin
NewValue:=CheckValue(NewValue);
FLastValid:=NewValue;
FValue:=NewValue;
DisplayValue(FValue);
end;



procedure TRealSpin.SetNoEventValue (NewValue: double);
var OldBlock: boolean;
begin
OldBlock:=BlockChangeEvent;
try
BlockChangeEvent:=True;
SetValue(NewValue);
finally BlockChangeEvent:=OldBlock; end;
end;



function TRealSpin.IsValidChar(Key: Char): Boolean;
begin
Result := (Key in [IndSystemData.DecimalSeparator,IndSystemData.ThousandSeparator,'+','-','e','E','0'..'9']) or ((Key < #32) and (Key <> Chr(VK_RETURN)));
if not FEditorEnabled and Result and ((Key >= #32) or (Key = Char(VK_BACK)) or (Key = Char(VK_DELETE))) then Result := False;
end;



procedure TRealSpin.KeyPress(var Key: Char);
begin
if IsValidChar(Key) then inherited KeyPress(Key)
else
	begin
	Key := #0;
	MessageBeep(0)
	end;
end;




procedure TRealSpin.Change;
begin
{when you edit numerical text in the edit box,
the editing takes place on the formated text.
IF the format is set to display just few digits
the underlying value will lose precision.}
FValue:=TextToDouble;
inherited Change;
end;



function TRealSpin.CheckValue (NewValue: double): double;
begin
Result := NewValue;
if FUseLimit then
	begin
	if NewValue >= FMaxValue then
		begin
		if Wrap and (FMinValue<>FMaxValue) then
			begin
			while Result>=FMaxValue do Result:=Result - FMaxValue;
			Result:=FMinValue + Result;
			Wrapped(wmExceededMax);
			end
		else Result := FMaxValue;
		end;
	if NewValue < FMinValue then
		begin
		if Wrap and (FMinValue<>FMaxValue)  then
			begin
			while Result<FMinValue do Result:=FMinValue-Result;
			Result:=FMaxValue-Result;
			Wrapped(wmExceededMin);
			end
		else Result:= FMinValue;
		end;
	end;
end;



procedure TRealSpin.KeyDown(var Key: Word; Shift: TShiftState);
begin
if Key = VK_UP then UpClick(Self)
else if Key = VK_DOWN then DownClick(Self);
inherited KeyDown(Key, Shift);
end;




procedure TRealSpin.CMWinIniChange(var Message: TWMWinIniChange);
var OldFlag: boolean;
begin
OldFlag:=InternationalFlag;
Try
SetThreadLocale(LOCALE_USER_DEFAULT);
SetInternational(True);
DisplayValue(FValue);
finally SetInternational(OldFlag); end;
end;



procedure TRealSpin.SetMaxValue(const Value: double);
begin
if FMaxValue<>Value then
	begin
	{Value overflow must be handled first or else "Value" will come }
	{back already trimmed to MaxValue and the "if" will always fail }
	if Self.Value>Value then Self.Value:=Value;
	FMaxValue := Value;
	end;
end;



procedure TRealSpin.SetMinValue(const Value: double);
begin
if FMinValue<>Value then
	begin
	{Value overflow must be handled first or else "Value" will come }
	{back already trimmed to MaxValue and the "if" will always fail }
	if Self.Value>FMinValue then Self.Value:=FMinValue;
	FMinValue := Value;
	end;
end;


procedure TRealSpin.SetDigits(const Value: integer);
begin
if (FDigits<>Value) and (Value>=0) then
	begin
	FDigits := Value;
	DisplayValue(FValue);
	end;
end;

end.

