unit Registrar;

interface

uses Windows,Forms,Classes,Controls,StdCtrls,ExtCtrls,Registry,SysUtils,Graphics,
     DSupport, LinkLabel,WinSupport;


{Registery Key Identifiers}

const PasswordKey = 'PW';		{Key to retrieve user entered password}
const MajorVersionKey = 'MajorVersion';	{Key to retrieve Major Version Number}
const MinorVersionKey = 'MinorVersion';	{Key to retrieve Minor Version Number}
const DateKey = 'D';			{Key to retreive Installation date}


const DefaultPassword: shortstring = (char(byte('f') xor $55)+ char(byte('r') xor $55)+
				      char(byte('o') xor $55)+ char(byte('g') xor $55));


{Data structure to hold registration data}

type TRegisterData = record
 Date: TDate;
 Password: string;
 MajorVersion: integer;
 MinorVersion: integer;
 Tampered: boolean;
 end;

type TPassword = shortstring;

type TRegistryKey = (HKClasses_Root, HK_Current_User, HK_Local_Machine , HK_Users, HK_Performance_Data, HK_Current_Config, HK_Dyn_Data);


type TRegistDlg = class;	{Forward declaration}

 TRegistrar = class(TComponent)
 private
  FMajorVersion, FMinorVersion, FTrialPeriod: integer;
  FTargetPassword: TPassword;
  FCompanyName: string;
  FProgramName: string;
  FWebURL,FWebMessage: string;
  FRootKey: TRegistryKey;
  FRootPath: string;
  function GetRootPath: string;
  function GetWebMessage: string;
  procedure SetWebMessage(V: string);
  function GetWebURL: string;
  procedure SetWebURL(V: string);
 protected
 public
  RegistDlg: TRegistDlg;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function GetRootKeyHandle: HKey;
  procedure RemoveRegistrationInfo;
  procedure SetPassword(S: string);
  procedure DoFirstInstallation;
  procedure SetDaysToExperation(Days: integer);
  procedure GetRegistrationStatus(var Installed,Expired,Registered,Tampered: boolean; var Days: integer);
  function GetRegistrationData(var Data: TRegisterData): boolean;
  function IsInstalled: boolean;
  procedure GuaranteeInstalled;
  procedure DisplayRegistrationDialog;
  procedure DisplayRegistrationStatus;
  function HandleProgramStartup: boolean;
 published
  property MajorVersion: integer read FMajorVersion write FMajorVersion default 1;
  property MinorVersion: integer read FMinorVersion write FMinorVersion default 0;
  property TargetPassword: TPassword read FTargetPassword write FTargetPassword;
  property TrialPeriod: integer read FTrialPeriod write FTrialPeriod default 30;
  property RootKey:TRegistryKey  read FRootKey write FRootKey default HK_Local_Machine;
  property RootPath: string read FRootPath write FRootPath;
  property CompanyName: string read FCompanyName write FCompanyName;
  property ProgramName: string read FProgramName write FProgramName;
  property WebMessage: string read GetWebMessage write SetWebMessage;
  property WebURL: string read GetWebURL write SetWebURL;
end;

{Declaration for the Registration dialog}

TRegistDlg = class(TForm)
    CloseBtn: TButton;
    StatusGroup: TGroupBox;
    RegStatusDis: TPanel;
    PasswordGroup: TGroupBox;
    PasswordEdit: TEdit;
    ApplyPassBtn: TButton;
    PasswordStatusDis: TPanel;
    LinkLabel1: TWebLinkLabel;
    Label1: TLabel;
    procedure ApplyPassBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
  public
    Regist: TRegistrar;
  end;

{$R RegistDlg.DFM}

function EncryptString(S: string): string;

implementation

{------------------ Support Routines -----------------------------------------}


function EncryptString(S: string): string;
{Simple encription/de-encryption routine}
var I: integer;
begin
Result:='';
for I:=1 to Length(S) do Result:=Result+char($55 xor Byte(S[I]));
end;



type TMergedNumber = array [0..7] of Byte;
const MergeMask: array [0..7] of byte = ($55,$95,$59,$99,$65,$A5,$69,$AA);


function CreateTamperProof(Number,Signature: cardinal): TMergedNumber;
var I: integer;
var PN,PS: PByteArray;
begin
PN:=@Number; PS:=@Signature;
for I:=0 to 7 do
 if (I and 1)=0 then Result[I]:=PN[I div 2]
 else Result[I]:=PS[I div 2];
for I:=0 to 7 do Result[I]:=Result[I] xor MergeMask[I];
end;



procedure DecodeTamperProof(M: TMergedNumber; var Number,Signature: cardinal);
var I: integer;
var PN,PS: PByteArray;
begin
PN:=@Number; PS:=@Signature;
for I:=0 to 7 do M[I]:=M[I] xor MergeMask[I];
for I:=0 to 7 do
 if (I and 1)=0 then PN[I div 2]:= M[I]
 else PS[I div 2]:=M[I];
end;

{-----------------------------------------------------------------------------}

const TargetSignature = 12345678;


procedure WriteDate(Reg: TRegistry; Key: string; Date: TDate);
var M: TMergedNumber;
begin
M:=CreateTamperProof(trunc(Date),TargetSignature);
Reg.WriteBinaryData(Key,M,8);
end;



function ReadDate(Reg: TRegistry; Key: string; var Date: TDate; var Tampered: boolean): boolean;
var M: TMergedNumber;
var Number,Signature: cardinal;
begin
Tampered:=False;
Result:=False;
if Reg.ReadBinaryData(Key,M,8) = 8 then
	begin
	DecodeTamperProof(M,Number,Signature);
	Date:=Number;
	Tampered:=Signature <> TargetSignature;
	Result:=True;
	end;
end;


{------------------ Registrar Object ------------------------------------------}



function TRegistrar.GetRootKeyHandle: HKey;
begin
Result:=HKEY_LOCAL_MACHINE;
case RootKey of
 HKClasses_Root: Result:=HKEY_CLASSES_ROOT;
 HK_Current_User : Result:= HKEY_CURRENT_USER;
 HK_Local_Machine : Result:= HKEY_LOCAL_MACHINE;
 HK_Users : Result:= HKEY_USERS;
 HK_Performance_Data : Result:= HKEY_PERFORMANCE_DATA;
 HK_Current_Config : Result:= HKEY_CURRENT_CONFIG;
 HK_Dyn_Data : Result:= HKEY_DYN_DATA;
 end;
end;



function TRegistrar.GetRootPath: string;
var S1,S2,S3: string;
begin
S1:=RootPath;
S2:=CompanyName;
S3:=ProgramName;
EatLeadTrailChars(S1,'\');
EatLeadTrailChars(S2,'\');
EatLeadTrailChars(S3,'\');
Result:='\'+S1+'\'+S2+'\'+S3;
end;




constructor TRegistrar.Create(AOwner:TComponent);
begin
inherited Create(AOwner);
RegistDlg:=TRegistDlg.Create(Self);
RegistDlg.Regist:=Self;

MajorVersion:= 1;
MinorVersion:= 0;
TargetPassword:= DefaultPassword;
TrialPeriod:= 30;

RootKey:= HK_LOCAL_MACHINE;
RootPath:= 'Software';
CompanyName:='Fountain';
ProgramName:='Software';
WebMessage:='Click here to purchase password';
WebURL:='http://www.fountainware.com';
end;



destructor TRegistrar.Destroy;
begin
RegistDlg.Free;
inherited Destroy;
end;


function TRegistrar.GetWebMessage: string;
begin
Result:=FWebMessage;
end;


procedure TRegistrar.SetWebMessage(V: string);
begin
if FWebMessage<>V then
	begin
	FWebMessage:=V;
	if not (csDesigning in ComponentState) then
		begin
		RegistDlg.LinkLabel1.Caption:=FWebMessage;
		end;
	end;
end;


function TRegistrar.GetWebURL: string;
begin
Result:=FWebURL;
end;


procedure TRegistrar.SetWebURL(V: string);
begin
if FWebURl<>V then
	begin
	FWebURl:=V;
	if not (csDesigning in ComponentState) then
		begin
		RegistDlg.LinkLabel1.Hint:=FWebURl;
		end;
	end;
end;




function TRegistrar.GetRegistrationData(var Data: TRegisterData): boolean;
{Get raw Registration data}
var Reg: TRegistry;
begin
Data.Password:='';
Data.MajorVersion:=0;
Data.MinorVersion:=0;
Data.Tampered:=False;
Data.Date:=0;

Reg := TRegistry.Create;
try
Reg.RootKey := GetRootKeyHandle;
Result:=Reg.OpenKey(GetRootPath, False);
if Result then
	begin
	try
	ReadDate(Reg,DateKey,Data.Date,Data.Tampered);
	Data.Password:=Reg.ReadString(PasswordKey);
	Data.MajorVersion:=Reg.ReadInteger(MajorVersionKey);
	Data.MinorVersion:=Reg.ReadInteger(MinorVersionKey);
	except Result:=False; end;
	end;
Reg.CloseKey;
finally Reg.Free; end;
end;


procedure TRegistrar.GetRegistrationStatus(var Installed,Expired,Registered,Tampered: boolean;var Days: integer);
{Get registration status}
var Data: TRegisterData;
begin
Expired:=False;
Registered:=False;
Days:=0;
Installed:=GetRegistrationData(Data);
Tampered:=Data.Tampered;
if Tampered then begin Expired:=True; exit; end;

if Installed then
	begin
	Registered:=(MajorVersion = Data.MajorVersion) and (EncryptString(TargetPassword) = Data.Password);
	if not Registered then
		begin
		Days:=Trunc(TrialPeriod - (Date - Data.Date));
		Expired:=Days<=0;
		end;
	end;
end;


procedure TRegistrar.SetDaysToExperation(Days: integer);
var Reg: TRegistry;
begin
Reg := TRegistry.Create;
try
Reg.RootKey := GetRootKeyHandle;
Reg.OpenKey(GetRootPath, True);
WriteDate(Reg,DateKey,(Date+Days)-TrialPeriod);
Reg.CloseKey;
finally Reg.Free; end;
end;



procedure TRegistrar.DoFirstInstallation;
var Reg: TRegistry;
begin
Reg := TRegistry.Create;
try
Reg.RootKey := GetRootKeyHandle;
Reg.OpenKey(GetRootPath, True);
WriteDate(Reg,DateKey,Date);
Reg.WriteString(PasswordKey,'');
Reg.WriteInteger(MajorVersionKey,MajorVersion);
Reg.WriteInteger(MinorVersionKey,MinorVersion);
Reg.CloseKey;
finally Reg.Free; end;
end;



procedure TRegistrar.SetPassword(S: string);
var Reg: TRegistry;
begin
Reg := TRegistry.Create;
try
Reg.RootKey := GetRootKeyHandle;
Reg.OpenKey(GetRootPath, True);
Reg.WriteString(PasswordKey,S);
Reg.CloseKey;
finally Reg.Free; end;
end;





procedure TRegistrar.RemoveRegistrationInfo;
var Reg: TRegistry;
begin
Reg := TRegistry.Create;
try
Reg.RootKey := GetRootKeyHandle;
Reg.DeleteKey(GetRootPath);
Reg.CloseKey;
finally Reg.Free; end;
end;



function TRegistrar.IsInstalled: boolean;
var Expired,Registered,Tampered: boolean;
var Days: integer;
begin
GetRegistrationStatus(Result,Expired,Registered,Tampered,Days);
if Tampered then Result:=False;
end;



procedure TRegistrar.GuaranteeInstalled;
begin
if not IsInstalled then DoFirstInstallation;
end;



procedure TRegistrar.DisplayRegistrationStatus;
var Installed,Expired,Registered,Tampered: boolean;
var Days: integer;
var S: string;
begin
GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
if Tampered then  S:='Tampered'
else if not Installed then S:='Not Installed'
else if Expired then S:='30 Day Trial has expired.'
else if Registered then S:='Registered and Authorized'
else S:=IntToStr(Days)+' Day left on Free Trial';
RegistDlg.RegStatusDis.Caption:=' '+S;
end;




function TRegistrar.HandleProgramStartup: boolean;
var Installed,Expired,Registered,Tampered: boolean;
var Days: integer;
begin
GuaranteeInstalled;
GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
if Expired then
	begin
	DisplayRegistrationDialog;
	GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
        end
else if not Registered then
	begin
	SplashMessageDlg(IntToStr(Days)+' Day Left On Your Free Trial',2);
	end;
GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
Result:=(not Expired) or Registered;
end;



procedure TRegistrar.DisplayRegistrationDialog;
begin
GuaranteeInstalled;
DisplayRegistrationStatus;
RegistDlg.ShowModal;
end;


{-------------------- Register Dialog ----------------------------------------}


procedure TRegistDlg.ApplyPassBtnClick(Sender: TObject);
var Installed,Expired,Registered,Tampered: boolean;
var Days: integer;
begin
Regist.GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
if Registered then
	begin
	PasswordStatusDis.Font.Color:=clRed;
	PasswordStatusDis.Caption:=' Already Registered';
	exit;
	end;

Regist.SetPassword(PasswordEdit.Text);
Regist.DisplayRegistrationStatus;
Regist.GetRegistrationStatus(Installed,Expired,Registered,Tampered,Days);
if Registered then
	begin
	PasswordStatusDis.Font.Color:=clNavy;
	PasswordStatusDis.Caption:=' Password accepted'
	end
else
	begin
	PasswordStatusDis.Font.Color:=clRed;
	PasswordStatusDis.Caption:=' Invalid Password';
	end;
end;


procedure TRegistDlg.FormCreate(Sender: TObject);
begin
end;


procedure TRegistDlg.FormDestroy(Sender: TObject);
begin
end;


end.
