unit SelectBox;

interface

uses Windows,Forms,ExtCtrls,SysUtils,Classes,Controls,Types,Graphics,Dialogs,
     GraphicsLib,IntVector,DSupport,Vector,Math;

type TGraphicControlEx = class(TGraphicControl)
 end;


type TBoxCorners=(bcNone,bcTopLeft,bcTopRight,bcBottomLeft,bcBottomRight,bcCenter,bcLeft,bcRight,bcTop,bcBottom);

type TCornerSwap = (csNone,csHorizontal,csVertical,csBoth);


type THandleRects = record
 Left,Right,Top,Bottom: TRect;
 TopLeft,TopRight,BottomLeft,BottomRight: TRect;
 end;


type TBackgroundInfo = record
 Left,Top,Right,Bottom: TRect;
 Saved: boolean;
 end;

const HandleSize = 3;

type TSelectionBox = class(TComponent)
 private
   LeftBM,TopBM,RightBM,BottomBM: TBitmap;
   FLeftDown: boolean;
   FUserSizeable,FUserMoveable: boolean;
   FBoxVisible: boolean;
   FBoxDragMode: TBoxCorners;
   FStartX,FStartY: integer;
   FCenterPos: TPoint;
   FBackInfo: TBackgroundInfo;
   FSelectBox: TRect;
   FDrawingSurface: TGraphicControl;
   FCanvasRect: TRect;
   FScrollBox: TScrollBox;
   FCenterMode: boolean;
    FScale2D: T2DVector;
    FLimitBox: boolean;
   procedure GetHandleRects(var Rects: THandleRects);
   function CornerHitTest(P: TPoint): TBoxCorners;
   procedure DrawRectangle(Canvas: TCanvas; R: TRect);
   procedure DrawHandle(Canvas: TCanvas; P: TPoint);
   procedure DrawBox(Canvas: TCanvas; R: TRect; ClearClipRegion,RestoreBackground: boolean);
   procedure SetBitmapSizes(var LeftBM,TopBM,RightBM,BottomBM: TBitmap; R: TRect);
   procedure GetBitmapSizes(var Size1,Size2: TPoint; R: TRect);
   procedure GetBitmapRects(var LeftRect,TopRect,RightRect,BottomRect: TRect; R: TRect);
   function NormalizeBox(var Box: TRect): TCornerSwap;
   procedure NormalizeBoxCorners(var Box: TRect);
   procedure SaveBoxBackground(Canvas: TCanvas; R: TRect);
   procedure RestoreBoxBackground(Canvas: TCanvas; ClearClipRegion: boolean);
   procedure SetSelectionBox(V: TRect);
   procedure SetDrawingSurface(V: TGraphicControl);
   procedure DoEdgeMode(X,Y: integer);
   procedure DoCenterMode(X,Y: integer);
   procedure OffsetSelectionBox(X,Y: integer);
   function GetAdjustedSelectRect: TRect;
   function GetBackgroundBox: TRect;
    procedure SetCornerCursor(BC: TBoxCorners);
    procedure SetScale(const Value: T2DVector);
    procedure LimitSelectionBox;
    procedure LimitedOffset(X, Y: integer);
    procedure SetLimitBox(const Value: boolean);
 protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
 public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure HideBox;
   procedure ShowBox;
   procedure DisplayBox(Restore: boolean);
   procedure SetAndShowBox(X1,Y1,X2,Y2: integer); overload;
   procedure SetAndShowBox(R: TRect); overload;
   procedure BoxMouseDown(X,Y: integer);
   procedure BoxMouseMove(X,Y: integer);
   procedure BoxMouseUp;
   property BoxVisible: boolean read FBoxVisible;
   property SelectionBox: TRect read FSelectBox write SetSelectionBox;
 published
   property UserMoveable: boolean read FUserMoveable write FUserMoveable default true;
   property UserSizeable: boolean read FUserSizeable write FUserSizeable default true;
   property CenterMode: boolean read FCenterMode write FCenterMode default false;
   property ScrollBox: TScrollBox read FScrollBox write FScrollBox;
   property DrawingSurface: TGraphicControl read FDrawingSurface write SetDrawingSurface;
   property Scale2D: T2DVector read FScale2D write SetScale;
   property LimitBox: boolean read FLimitBox write SetLimitBox default false;
 end;

procedure Register;

implementation



procedure Register;
begin
RegisterComponents('Fountain', [TSelectionBox]);
end;


{---------------- Routines handle underlying control -------------------------}

type TControlType = (ctImage,ctPaintBox,ctNotDrawable);


function GetControlType(C: TControl): TControlType;
begin
{TForm not used because of problems with its bounding box and clipping in onpaint}
if C is TImage then Result:=ctImage
else if C is TPaintBox then Result:=ctPaintBox
else Result:=ctNotDrawable;
end;



function GetDrawingSurfaceSize(C: TControl): TRect;
begin
case GetControlType(C) of
 ctImage: Result:=TImage(C).ClientRect;
 ctPaintBox: Result:=TPaintBox(C).ClientRect;
 end;
end;




function GetDrawingSurfaceCanvas(C: TControl): TCanvas;
begin
case GetControlType(C) of
 ctImage: Result:=TImage(C).Canvas;
 ctPaintBox: Result:=TPaintBox(C).Canvas;
 end;
end;



constructor TSelectionBox.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FLimitBox:=False;
FDrawingSurface:=nil;
FBoxVisible:=False;
FUserSizeable:=True;
FUserMoveable:=True;
FCenterMode:=False;
FBackInfo.Saved:=False;
LeftBM:=TBitmap.Create; TopBM:=TBitmap.Create;
RightBM:=TBitmap.Create; BottomBM:=TBitmap.Create;
Scale2D:=MakeVector2D(1,1);
end;



destructor TSelectionBox.Destroy;
begin
LeftBM.Free; TopBM.Free;
RightBM.Free; BottomBM.Free;
inherited;
end;




{-------------------Routines saving background  --------------------}



procedure TSelectionBox.GetBitmapSizes(var Size1, Size2: TPoint; R: TRect);
begin
Size1.X:=HandleSize * 2;
Size1.Y:=abs(R.Bottom - R.Top) + HandleSize * 2;
Size2.X:=abs(R.Right - R.Left) + HandleSize * 2;
Size2.Y:=HandleSize * 2;
end;


procedure TSelectionBox.SetBitmapSizes(var LeftBM,TopBM,RightBM,BottomBM: TBitmap; R: TRect);
var Size1,Size2: TPoint;
begin
GetBitmapSizes(Size1,Size2,R);
LeftBM.Width:=Size1.X; LeftBM.Height:=Size1.Y; RightBM.Width:=Size1.X; RightBM.Height:=Size1.Y;
TopBM.Width:=Size2.X; TopBM.Height:=Size2.Y; BottomBM.Width:=Size2.X; BottomBM.Height:=Size2.Y;
end;



procedure TSelectionBox.GetBitmapRects(var LeftRect,TopRect,RightRect,BottomRect: TRect; R: TRect);
begin
LeftRect.Top:=R.Top - HandleSize;
LeftRect.Left:=R.Left - HandleSize;
LeftRect.Bottom:=R.Bottom + HandleSize;
LeftRect.Right:=R.Left + HandleSize;

RightRect.Top:=R.Top - HandleSize;
RightRect.Left:=R.Right - HandleSize;
RightRect.Bottom:=R.Bottom + HandleSize;
RightRect.Right:=R.Right + HandleSize;

TopRect.Top:=R.Top - HandleSize;
TopRect.Left:=R.Left - HandleSize;
TopRect.Bottom:=R.Top + HandleSize;
TopRect.Right:=R.Right + HandleSize;

BottomRect.Top:=R.Bottom - HandleSize;
BottomRect.Left:=R.Left - HandleSize;
BottomRect.Bottom:=R.Bottom + HandleSize;
BottomRect.Right:=R.Right + HandleSize;
end;




procedure TSelectionBox.SaveBoxBackground(Canvas: TCanvas; R: TRect);
var V,H: TRect;
begin
SetBitmapSizes(LeftBM,TopBM,RightBM,BottomBM,R);
GetBitmapRects(FBackInfo.Left,FBackInfo.Top,FBackInfo.Right,FBackInfo.Bottom,R);
V:=Rect(0,0,LeftBM.Width,LeftBM.Height);
H:=Rect(0,0,TopBM.Width,TopBM.Height);
LeftBM.Canvas.CopyRect(V,Canvas,FBackInfo.Left);
RightBM.Canvas.CopyRect(V,Canvas,FBackInfo.Right);
TopBM.Canvas.CopyRect(H,Canvas,FBackInfo.Top);
BottomBM.Canvas.CopyRect(H,Canvas,FBackInfo.Bottom);
FBackInfo.Saved:=True;
end; 


var PS: TPaintStruct;
var OldDC: HDC;

procedure ResetPaintCanvasClip(Canvas: TCanvas; Rect: TRect);
{Routine used to reset the Clipping Regions inside an OnPaint call}
var DC: HDC;
var MyRgn: HRGN;
begin
OldDC:=Canvas.Handle;
DC:=BeginPaint(0,PS);
Canvas.Handle:=DC;
MyRgn := CreateRectRgn(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
SelectClipRgn(Canvas.Handle,MyRgn);
DeleteObject(MyRgn);
end;



procedure RestorPaintCanvasClip(Canvas: TCanvas);
{Routine used to restore the Clipping Regions inside an OnPaint call}
begin
EndPaint(0,PS);
Canvas.Handle:=OldDC;
end;




function TSelectionBox.GetAdjustedSelectRect: TRect;
var BR: TRect;
begin
Result:=Self.FSelectBox;
BR:=Self.FDrawingSurface.BoundsRect;
OffsetRect(Result,BR.Left,BR.Top);
InflateRect(Result,HandleSize,HandleSize);
end;



function TSelectionBox.GetBackgroundBox: TRect;
var BR: TRect;
begin
Result:=Rect(FBackInfo.Left.Left,FBackInfo.Top.Top,FBackInfo.Right.Right,FBackInfo.Bottom.Bottom);
BR:=Self.FDrawingSurface.BoundsRect;
OffsetRect(Result,BR.Left,BR.Top);
end;



procedure TSelectionBox.RestoreBoxBackground(Canvas: TCanvas; ClearClipRegion: boolean);
{Note: with paintboxs, it may not be able to restore the background (hide)}
{if called with the paintbox's OnPaint event. This is because the cliprect}
{is too small. I haven't figured out how to clear it.}
var V,H: TRect;
begin
if not FBackInfo.Saved then exit;
if ClearClipRegion then ResetPaintCanvasClip(Canvas,GetBackgroundBox);

V:=Rect(0,0,LeftBM.Width,LeftBM.Height);
H:=Rect(0,0,TopBM.Width,TopBM.Height);
Canvas.CopyRect(FBackInfo.Left,LeftBM.Canvas,V);
Canvas.CopyRect(FBackInfo.Right,RightBM.Canvas,V);
Canvas.CopyRect(FBackInfo.Top,TopBM.Canvas,H);
Canvas.CopyRect(FBackInfo.Bottom,BottomBM.Canvas,H);
FBackInfo.Saved:=False;
if ClearClipRegion then RestorPaintCanvasClip(Canvas);
end;



{-------------------Handle Drawing Clipping Box --------------------}


procedure TSelectionBox.GetHandleRects(var Rects: THandleRects);
begin
with Rects do
 begin
 TopLeft:=Rect(FSelectBox.Left-HandleSize,FSelectBox.Top-HandleSize,FSelectBox.Left+HandleSize,FSelectBox.Top+HandleSize);
 TopRight:=Rect(FSelectBox.Right-HandleSize,FSelectBox.Top-HandleSize,FSelectBox.Right+HandleSize,FSelectBox.Top+HandleSize);
 BottomLeft:=Rect(FSelectBox.Left-HandleSize,FSelectBox.Bottom-HandleSize,FSelectBox.Left+HandleSize,FSelectBox.Bottom+HandleSize);
 BottomRight:=Rect(FSelectBox.Right-HandleSize,FSelectBox.Bottom-HandleSize,FSelectBox.Right+HandleSize,FSelectBox.Bottom+HandleSize);

 Left.Left:=TopLeft.Left; Left.Top:=TopLeft.Bottom; Left.Right:=TopLeft.Right; Left.Bottom:=BottomLeft.Top;
 Right.Left:=TopRight.Left; Right.Top:=TopRight.Bottom; Right.Right:=TopRight.Right; Right.Bottom:=BottomRight.Top;
 Top.Left:=TopLeft.Right; Top.Top:=TopLeft.Top; Top.Right:=TopRight.Left; Top.Bottom:=TopLeft.Bottom;
 Bottom.Left:=BottomLeft.Right; Bottom.Top:=BottomLeft.Top; Bottom.Right:=BottomRight.Left; Bottom.Bottom:=BottomLeft.Bottom;
 end;
end;



function TSelectionBox.CornerHitTest(P: TPoint): TBoxCorners;
{Test if point falls in one of the corners}
var Rects: THandleRects;
begin
Result:=bcNone;

GetHandleRects(Rects);
if PtInRect(Rects.TopLeft,P) then Result:=bcTopLeft
else if PtInRect(Rects.TopRight,P) then Result:=bcTopRight
else if PtInRect(Rects.BottomLeft,P) then Result:=bcBottomLeft
else if PtInRect(Rects.BottomRight,P) then Result:=bcBottomRight
else if PtInRect(Rects.Left,P) then Result:=bcLeft
else if PtInRect(Rects.Right,P) then Result:=bcRight
else if PtInRect(Rects.Top,P) then Result:=bcTop
else if PtInRect(Rects.Bottom,P) then Result:=bcBottom
else if PtInRect(FSelectBox,P) then Result:=bcCenter;
end;



procedure TSelectionBox.DrawRectangle(Canvas: TCanvas; R: TRect);
{Draw a rectangle on specified canvas.}
begin
Canvas.MoveTo(R.Left,R.Top);
Canvas.LineTo(R.Right,R.Top);
Canvas.LineTo(R.Right,R.Bottom);
Canvas.LineTo(R.Left,R.Bottom);
Canvas.LineTo(R.Left,R.Top);
end;



procedure TSelectionBox.DrawHandle(Canvas: TCanvas; P: TPoint);
{Draw one of the corner handles}
var R: TRect;
begin
R:=Rect(P.X-HandleSize,P.Y-HandleSize,P.X+HandleSize,P.Y+HandleSize);
Canvas.Brush.Style:=bsSolid;
Canvas.Brush.Color:=clBlack;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Color:=clWhite;
Canvas.Pen.Mode:=pmCopy;
Canvas.Rectangle(R.Left,R.Top,R.Right,R.Bottom);
end;


{------------------------------------------------------------------------------}


function TSelectionBox.NormalizeBox(var Box: TRect): TCornerSwap;
var H,V: boolean;
begin
H:=Box.Left>Box.Right;
V:=Box.Top>Box.Bottom;
SwapIntegerArgs(H,Box.Left,Box.Right);
SwapIntegerArgs(V,Box.Top,Box.Bottom);
if H and V then Result:=csBoth
else if H then Result:=csHorizontal
else if V then Result:=csVertical
else Result:=csNone;
end;



procedure TSelectionBox.SetCornerCursor(BC: TBoxCorners);
begin
if DrawingSurface=nil then exit;
case BC of
 bcBottomRight: DrawingSurface.Cursor:=crSizeNWSE;
 bcTopLeft: DrawingSurface.Cursor:=crSizeNWSE;
 bcTopRight: DrawingSurface.Cursor:=crSizeNESW;
 bcBottomLeft: DrawingSurface.Cursor:=crSizeNESW;
 bcLeft:  DrawingSurface.Cursor:=crSizeWE;
 bcRight:  DrawingSurface.Cursor:=crSizeWE;
 bcTop:   DrawingSurface.Cursor:=crSizeNS;
 bcBottom: DrawingSurface.Cursor:=crSizeNS;
 bcCenter: DrawingSurface.Cursor:=crSizeAll;
 else DrawingSurface.Cursor:=crDefault;
 end;
end;



function GetDragModeStr(BC: TBoxCorners): string;
begin
case BC of
 bcNone: Result:='None';
 bcTopLeft: Result:='TopLeft';
 bcTopRight: Result:='TopRight';
 bcBottomLeft: Result:='BottomLeft';
 bcBottomRight: Result:='BottomRight';
 bcCenter: Result:='Center';
 end;
end;


procedure TSelectionBox.NormalizeBoxCorners(var Box: TRect);
begin
case NormalizeBox(Box) of
 csNone: ;
 csHorizontal:
	begin
	case FBoxDragMode of
	 bcTopRight: FBoxDragMode:=bcTopLeft;
	 bcBottomRight: FBoxDragMode:=bcBottomLeft;
	 bcTopLeft: FBoxDragMode:=bcTopRight;
	 bcBottomLeft: FBoxDragMode:=bcBottomRight;
	 bcLeft:  FBoxDragMode:=bcRight;
	 bcRight: FBoxDragMode:=bcLeft;
	 end;
	end;
 csVertical:
	begin
	case FBoxDragMode of
	 bcTopRight: FBoxDragMode:=bcBottomRight;
	 bcBottomRight: FBoxDragMode:=bcTopRight;
	 bcTopLeft: FBoxDragMode:=bcBottomLeft;
	 bcBottomLeft: FBoxDragMode:=bcTopLeft;
	 bcTop: FBoxDragMode:=bcBottom;
	 bcBottom: FBoxDragMode:=bcTop;
	 end;
	end;
 csBoth:
	begin
	case FBoxDragMode of
	 bcTopRight: FBoxDragMode:=bcBottomLeft;
	 bcBottomRight: FBoxDragMode:=bcTopLeft;
	 bcTopLeft: FBoxDragMode:=bcBottomRight;
	 bcBottomLeft: FBoxDragMode:=bcTopRight;
	 end;
	end;
 end;
SetCornerCursor(FBoxDragMode);
end;




procedure TSelectionBox.DrawBox(Canvas: TCanvas; R: TRect; ClearClipRegion,RestoreBackground: boolean);
{Draw current clipping box}
var R2: TRect;
var CS: TCanvasSaver;
begin
CS:=TCanvasSaver.Create;
try
CS.Save(Canvas);
if RestoreBackground then RestoreBoxBackground(Canvas,ClearClipRegion);
if ClearClipRegion then ResetPaintCanvasClip(Canvas,GetAdjustedSelectRect);
SaveBoxBackground(Canvas,R);
Canvas.Pen.Mode:=pmCopy;
Canvas.Pen.Style:=psSolid;
Canvas.Pen.Width:=1;
Canvas.Pen.Color:=clRed;
DrawRectangle(Canvas,R);

R2:=R;
InflateRect(R2,1,1);
Canvas.Pen.Color:=clBlack;
DrawRectangle(Canvas,R2);
if (R.Left=0) and (R.Top=0) and (R.Right=0) and (R.Bottom=0) then exit;
if UserSizeable then
	begin
	DrawHandle(Canvas,R.TopLeft);
	DrawHandle(Canvas,R.BottomRight);
	DrawHandle(Canvas,Point(R.Right,R.Top));
	DrawHandle(Canvas,Point(R.Left,R.Bottom));
	end;
if ClearClipRegion then RestorPaintCanvasClip(Canvas);
CS.Restore(Canvas);
finally CS.Free; end;
end;




procedure TSelectionBox.SetAndShowBox(X1,Y1,X2,Y2: integer);
{Erase old box and draw new box}
begin
FSelectBox:=Rect(X1,Y1,X2,Y2);
FCenterPos:=FindRectCenter(FSelectBox);
if FDrawingSurface=nil then exit;
if FBoxVisible then DrawBox(GetDrawingSurfaceCanvas(FDrawingSurface),FSelectBox,False,True);
end;



procedure TSelectionBox.SetAndShowBox(R: TRect);
{Erase old box and draw new box}
begin
SetAndShowBox(R.Left,R.Top,R.Right,R.Bottom);
end;



procedure TSelectionBox.HideBox;
{Hide current box}
{Note: with paintboxs, it may not be able to restore the background (hide)}
{if called with the paintbox's OnPaint event. This is because the cliprect}
{is too small. I haven't figured out how to clear it.}
var ClearClipRegion: boolean;
begin
if FDrawingSurface=nil then exit;
ClearClipRegion:=GetControlType(FDrawingSurface)=ctPaintBox;
RestoreBoxBackground(GetDrawingSurfaceCanvas(FDrawingSurface),ClearClipRegion);
FBoxVisible:=False;
end;


procedure TSelectionBox.DisplayBox(Restore: boolean);
{Display box and control restoring old background}
var ClearClipRegion: boolean;
begin
FBoxVisible:=True;
if FDrawingSurface=nil then exit;
ClearClipRegion:=GetControlType(FDrawingSurface)=ctPaintBox;
DrawBox(GetDrawingSurfaceCanvas(FDrawingSurface),FSelectBox,ClearClipRegion,Restore);
end;




procedure TSelectionBox.ShowBox;
{Restore hidden box}
var ClearClipRegion: boolean;
begin
FBoxVisible:=True;
DisplayBox(True);
end;


{---------------------Clipping Box Event Handlers -------------------}



procedure CursorScroll(ScrollBox: TScrollBox; Offset: TPoint; X,Y: integer);
{Scroll box based on position X,Y}
var RX,RY: integer;
var MovX,MovY: integer;
begin
RX:=X+Offset.X;
RY:=Y+Offset.Y;
if RX<0 then MovX:=-RX
else if RX>ScrollBox.ClientWidth then MovX:=RX-ScrollBox.ClientWidth
else MovX:=0;

if RY<0 then MovY:=-RY
else if RY>ScrollBox.ClientHeight then MovY:=RY-ScrollBox.ClientHeight
else MovY:=0;

if RX<0 then ScrollBox.HorzScrollBar.Position:=ScrollBox.HorzScrollBar.Position-MovX;
if RX>ScrollBox.ClientWidth then ScrollBox.HorzScrollBar.Position:=ScrollBox.HorzScrollBar.Position+MovX;
if RY<0 then ScrollBox.VertScrollBar.Position:=ScrollBox.VertScrollBar.Position-MovY;
if RY>ScrollBox.ClientHeight then ScrollBox.VertScrollBar.Position:=ScrollBox.VertScrollBar.Position+MovY;
end;




procedure TSelectionBox.BoxMouseDown(X,Y: integer);
begin
if not FBoxVisible then exit;
X:=round(X / Scale2D.X); Y:=round(Y / Scale2D.Y);

FBoxDragMode:=CornerHitTest(Point(X,Y));
if UserSizeable then SetCornerCursor(FBoxDragMode);
if FBoxDragMode=bcNone then
	begin
	if UserSizeable then SetAndShowBox(X,Y,X,Y);
	FStartX:=X; FStartY:=Y;
	FCenterPos:=Point(X,Y);
	end;
FLeftDown:=True;
end;



procedure TSelectionBox.OffsetSelectionBox(X,Y: integer);
var TmpRect: TRect;
begin
if UserMoveable then
	begin
	TmpRect:=FSelectBox;
	OffsetRect(TmpRect,X-FStartX,Y-FStartY);
	SetAndShowBox(TmpRect);
	FCenterPos:=FindRectCenter(FSelectBox);
	end;
end;


procedure TSelectionBox.LimitSelectionBox;
{Limit rectange to the drawing surface client rectangel}
begin
FSelectBox.Left:=EnsureRange(FSelectBox.Left,DrawingSurface.ClientRect.Left,DrawingSurface.ClientRect.Right-1);
FSelectBox.Right:=EnsureRange(FSelectBox.Right,DrawingSurface.ClientRect.Left,DrawingSurface.ClientRect.Right-1);
FSelectBox.Top:=EnsureRange(FSelectBox.Top,DrawingSurface.ClientRect.Top,DrawingSurface.ClientRect.Bottom-1);
FSelectBox.Bottom:=EnsureRange(FSelectBox.Bottom,DrawingSurface.ClientRect.Top,DrawingSurface.ClientRect.Bottom-1);
end;


procedure TSelectionBox.LimitedOffset(X,Y: integer);
{Offset the box if "LimitBox" property is true }
{limit the offset so no edge exceeds the client boundary}
var X1,Y1: integer;
begin
if LimitBox then
	begin
	X1:=X-FStartX;
	Y1:=Y-FStartY;
	if FSelectBox.Right+X1>=DrawingSurface.ClientRect.Right then X:=FStartX;
	if FSelectBox.Left+X1<DrawingSurface.ClientRect.Left then X:=FStartX;
	if FSelectBox.Top+Y1<DrawingSurface.ClientRect.Top then Y:=FStartY;
	if FSelectBox.Bottom+Y1>=DrawingSurface.ClientRect.Bottom then Y:=FStartY;
	end;
OffsetSelectionBox(X,Y);
end;


procedure TSelectionBox.DoEdgeMode(X,Y: integer);
begin
case FBoxDragMode of
 bcNone, bcBottomRight:
	begin
	{Force mode in case it is bcNone}
	FBoxDragMode:=bcBottomRight;
	FSelectBox:=Rect(FSelectBox.Left,FSelectBox.Top,X,Y);
	end;
 bcTopLeft: FSelectBox:=Rect(X,Y,FSelectBox.Right,FSelectBox.Bottom);
 bcTopRight: FSelectBox:=Rect(FSelectBox.Left,Y,X,FSelectBox.Bottom);
 bcBottomLeft: FSelectBox:=Rect(X,FSelectBox.Top,FSelectBox.Right,Y);
 bcLeft: FSelectBox:=Rect(X,FSelectBox.Top,FSelectBox.Right,FSelectBox.Bottom);
 bcRight: FSelectBox:=Rect(FSelectBox.Left,FSelectBox.Top,X,FSelectBox.Bottom);
 bcTop: FSelectBox:=Rect(FSelectBox.Left,Y,FSelectBox.Right,FSelectBox.Bottom);
 bcBottom: FSelectBox:=Rect(FSelectBox.Left,FSelectBox.Top,FSelectBox.Right,Y);
 bcCenter: begin LimitedOffset(X,Y); exit; end;
 end;
if LimitBox then LimitSelectionBox;
end;




procedure TSelectionBox.DoCenterMode(X,Y: integer);
{Create selection box centered around point C. Offset in Point M}
var D: TPoint;
var BoxAspect,WinAspect : double;
var CanvasSize,MousePos: TPoint;
begin
if FBoxDragMode=bcCenter then begin LimitedOffset(X,Y); exit; end;
MousePos:=Point(X,Y);
CanvasSize:=IntVectorSubtract(FCanvasRect.BottomRight,FCanvasRect.TopLeft);
WinAspect:=FDrawingSurface.Width/FDrawingSurface.Height;
D:=IntVectorSubtract(MousePos,FCenterPos);
if D.Y=0 then BoxAspect:=1 else BoxAspect :=abs(D.X / D.Y );
if BoxAspect <= WinAspect then D.X := trunc(D.Y * WinAspect)
else D.Y := trunc(D.X / WinAspect);

FSelectBox.TopLeft:=IntVectorSubtract(FCenterPos,D);
FSelectBox.BottomRight:=IntVectorAdd(FCenterPos,D);
if LimitBox then LimitSelectionBox;
end;




procedure TSelectionBox.BoxMouseMove(X,Y: integer);
var SX,SY: integer;
begin
if not FBoxVisible then exit;
SX:=round(X / Scale2D.X); SY:=round(Y / Scale2D.Y);
if FLeftDown then
	begin
	if (FScrollBox<>nil) and (FDrawingSurface<>nil) then CursorScroll(FScrollBox,Point(FDrawingSurface.Left,FDrawingSurface.Top),X,Y);
	if UserSizeable then
		begin
		if FCenterMode then DoCenterMode(SX,SY)
		else DoEdgeMode(SX,SY);
		if FBoxDragMode<>bcCenter then
			begin
			NormalizeBoxCorners(FSelectBox);
			SetAndShowBox(FSelectBox);
			end;
		end;
	end
else
	begin
	if UserSizeable then
		begin
		FBoxDragMode:=CornerHitTest(Point(SX,SY));
		SetCornerCursor(FBoxDragMode);
		end;

	if UserMoveable then
		begin
		if Self.DrawingSurface<>nil then
		case FBoxDragMode of
		 bcCenter: DrawingSurface.Cursor:=crSizeAll;
		 end;
		end;
	end;

FStartX:=SX; FStartY:=SY;
end;



procedure TSelectionBox.BoxMouseUp;
begin
if not FBoxVisible then exit; 
FLeftDown:=False;
end;




procedure TSelectionBox.SetSelectionBox(V: TRect);
begin
if not EqualRect(V,FSelectBox) then Self.SetAndShowBox(V);
end;



procedure TSelectionBox.SetDrawingSurface(V: TGraphicControl);
var OldVisible: boolean;
begin
if V=FDrawingSurface then exit;
if GetControlType(V)=ctNotDrawable then
	begin
	MessageDlg('Must be TImage, TPaintBox or TForm',mtWarning,[mbOK],0);
	exit;
	end;
OldVisible:=FBoxVisible;
HideBox;
FDrawingSurface:=V;
FCanvasRect:=GetDrawingSurfaceSize(FDrawingSurface);
if OldVisible then ShowBox;
end;


procedure TSelectionBox.SetScale(const Value: T2DVector);
begin
if (Value.X>0) and (Value.Y>0) and
   not AreEqual2D(Value,FScale2D) then FScale2D := Value;
end;



procedure TSelectionBox.Notification(AComponent: TComponent;
  Operation: TOperation);
{Handle the live deletion of a Scroll Box object controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) then
	begin
	if (AComponent = FScrollBox) then FScrollBox := nil;
	if (AComponent = FDrawingSurface) then FDrawingSurface := nil;
	end;
end;



procedure TSelectionBox.SetLimitBox(const Value: boolean);
begin
if FLimitBox<>Value then
	begin
	FLimitBox := Value;
	end;
end;

end.