unit SpinButtons;

interface

uses Windows,Messages,Forms,Types,Classes,Controls,StdCtrls,ExtCtrls,Graphics,
     DSupport, Autobttn;


type TUndoItem = record
 Text: string;
 Pos,Len: integer;
 end;

type TUndoArray = array of TUndoItem;

type TShortcutMode = (smNone,smStandard);


{Control to undo changes in an TEdit box}


type TEditUndoer = class(TObject)
 private
 protected
 public
  UndoArray: TUndoArray;
  RedoArray: TUndoArray;
  procedure AddUndoItem(var UA: TUndoArray; Item: TUndoItem); overload;
  procedure AddUndoItem(var UA: TUndoArray; Text: string; Pos, Len: integer); overload;
  function GetLastUndoItem(var UA: TUndoArray; var Item: TUndoItem): boolean; overload;
  function GetLastUndoItem(var UA: TUndoArray; var Text: string; var Pos,Len: integer): boolean; overload;
  procedure GetReDo(Edit: TEdit);
  procedure GetUndo(Edit: TEdit);
  procedure StoreReDo(Edit: TEdit);
  procedure StoreUndo(Edit: TEdit);
  procedure Clear;
 end;



{Extended Edit Control}

type TCustomEditEx = class(TCustomEdit)
 private
  FAlignment: TAlignment;
  FOnKillFocus: TNotifyEvent;
    FOnReDo: TNotifyEvent;
    FOnUndo: TNotifyEvent;
    FShortCutMode: TShortcutMode;
  procedure SetAlignment(const Value: TAlignment);
 protected
   procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
   procedure KeyPress(var Key: Char); override;
 public
  Undoer: TEditUndoer;
  constructor Create(AOwner:TComponent); override;
  destructor Destroy; override;
  procedure CreateParams(var Params: TCreateParams); override;
  procedure DoUndo;
  procedure DoRedo;
 published
  property Align;
  property Anchors;
  property Alignment : TAlignment read FAlignment write SetAlignment;
  property Constraints;
  property OnKillFocus: TNotifyEvent read FOnKillFocus write FOnKillFocus;
  property OnUndo: TNotifyEvent read FOnUndo write FOnUndo;
  property OnReDo: TNotifyEvent read FOnReDo write FOnReDo;
  property ShortCutMode: TShortcutMode read FShortCutMode write FShortCutMode default smNone;
 end;


{User accessible Extended Edit Control}

type TEditEx = class(TCustomEditEx)
 private
 public
 published
  property Anchors;
  property AutoSelect;
  property AutoSize;
  property BevelEdges;
  property BevelInner;
  property BevelKind default bkNone;
  property BevelOuter;
  property BiDiMode;
  property BorderStyle;
  property CharCase;
  property Color;
  property Constraints;
  property Ctl3D;
  property DragCursor;
  property DragKind;
  property DragMode;
  property Enabled;
  property Font;
  property HideSelection;
  property ImeMode;
  property ImeName;
  property MaxLength;
  property OEMConvert;
  property ParentBiDiMode;
  property ParentColor;
  property ParentCtl3D;
  property ParentFont;
  property ParentShowHint;
  property PasswordChar;
  property PopupMenu;
  property ReadOnly;
  property ShowHint;
  property TabOrder;
  property TabStop;
  property Text;
  property Visible;
  property OnChange;
  property OnClick;
  property OnContextPopup;
  property OnDblClick;
  property OnDragDrop;
  property OnDragOver;
  property OnEndDock;
  property OnEndDrag;
  property OnEnter;
  property OnExit;
  property OnKeyDown;
  property OnKeyPress;
  property OnKeyUp;
  property OnMouseDown;
  property OnMouseMove;
  property OnMouseUp;
  property OnStartDock;
  property OnStartDrag;
  end;


{Spinner button that does not use a bitmap }
{to reduce the number handles in use}

const ArrowWidth = 7;
const ArrowHeight = 3;


type  TArrowButton = class(TAutoButton)
  private
    FUpArrow: boolean;
    FDrawFocus: boolean;
    procedure DrawDownArrow;
    procedure DrawUpArrow;
    procedure SetUpArrow(V: boolean);
  protected
    procedure Paint; override;
  public
  published
    property UpArrow: boolean read FUpArrow write SetupArrow default false;
  end;

{A combined up and down arrow button using bitmap handle free arrows}

type  TUpDownButton = class(TWinControl)
  private
    FUpButton: TArrowButton;
    FDownButton: TArrowButton;
    FOnUpClick: TNotifyEvent;
    FOnDownClick: TNotifyEvent;
    FFocusedButton: TArrowButton;
    FFocusControl: TWinControl;
    function CreateButton: TArrowButton;
    procedure AdjustSpinSize(var W,H: integer);
    procedure SetFocusBtn (Btn: TArrowButton);
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure BtnClick(Sender: TObject);
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure BtnMouseDown (Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    function GetButtonPressed: boolean;
  protected
    procedure SetEnabled(V: boolean); override;
    procedure Loaded; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    property UpButton: TArrowButton read FUpButton;
    property DownButton: TArrowButton read FDownButton;
  published
    property FocusControl: TWinControl read FFocusControl write FFocusControl;
    property OnUpClick: TNotifyEvent read FOnUpClick write FOnUpClick;
    property OnDownClick: TNotifyEvent read FOnDownClick write FOnDownClick;
    property Pressed: boolean read GetButtonPressed;
    property Align;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDock;
    property OnStartDrag;
 end;

{ ========== Declaration for the TCustomSpin ==================================}
{ ========== Used as the basis of Integer and Real spins ======================}

type TWrapMode = (wmExceededMax,wmExceededMin);

type  TWrapEvent = procedure(Sender: TObject; WrapMode: TWrapMode) of object;

type

  TDisPanel = class(TCustomPanel);

  TCustomSpin = class(TCustomEditEx)
  private
    FBlockChangeEvent: boolean;
    FPanel: TDisPanel;
    FPanelWidth: integer;
    FWrap: boolean;
    FOnWrap: TWrapEvent;
    function GetMinHeight: Integer;
    procedure SetUnitsString(V: string);
    function GetUnitsString: string;
    procedure UpdatePanelWidth;
    procedure SetPanelWidth(NewValue: integer);
    procedure SetEditRect;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure WMPaste(var Message: TWMPaste);   message WM_PASTE;
    procedure WMCut(var Message: TWMCut);   message WM_CUT;
  protected
    FUseLimit: boolean;
    FEditorEnabled: Boolean;
    FButton: TUpDownButton;
    procedure SetEnabled(Value: Boolean); override;
    procedure CreateWnd; override;
    procedure Change; override;
    procedure Wrapped(WrapMode: TWrapMode);
  public
    procedure CreateParams(var Params: TCreateParams); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Button: TUpDownButton read FButton;
  published
    property EditorEnabled: Boolean read FEditorEnabled write FEditorEnabled default True;
    property UseLimit: boolean read FUselimit write FUselimit default False;
    property UnitsString: string read GetUnitsString write SetUnitsString;
    property PanelWidth: integer read FPanelWidth write SetPanelWidth;
    property BlockChangeEvent: boolean read FBlockChangeEvent write FBlockChangeEvent default false;
    property Wrap: boolean read FWrap write FWrap default False;
    property OnWrap: TWrapEvent read FOnWrap write FOnWrap;
  end;

procedure Register;


implementation

procedure Register;
begin
RegisterComponents('Fountain', [TArrowButton]);
RegisterComponents('Fountain', [TUpDownButton]);
RegisterComponents('Fountain',[TEditEx]);
end;

procedure TEditUndoer.AddUndoItem(var UA: TUndoArray; Item: TUndoItem);
begin
SetLength(UA, Length(UA)+1);
UA[High(UA)]:=Item;
end;


procedure TEditUndoer.AddUndoItem(var UA: TUndoArray; Text: string; Pos, Len: integer);
var Item: TUndoItem;
begin
Item.Text:=Text;
Item.Pos:=Pos;
Item.Len:=Len;
AddUndoItem(UA,Item);
end;


function TEditUndoer.GetLastUndoItem(var UA: TUndoArray; var Item: TUndoItem): boolean;
begin
Result:=False;
if Length(UA)<1 then exit;
Item:=UA[High(UA)];
SetLength(UA,Length(UA)-1);
Result:=True;
end;



function TEditUndoer.GetLastUndoItem(var UA: TUndoArray; var Text: string; var Pos, Len: integer): boolean;
var Item: TUndoItem;
begin
Result:=GetLastUndoItem(UA,Item);
if not Result then exit;
Text:=Item.Text;
Pos:=Item.Pos;
Len:=Item.Len;
end;

{------------------------------------------------------------------------------}

procedure TEditUndoer.StoreUndo(Edit: TEdit);
begin
AddUndoItem(UndoArray,Edit.Text,Edit.SelStart,Edit.SelLength);
SetLength(RedoArray,0);
end;


procedure TEditUndoer.Clear;
begin
SetLength(UndoArray,0);
SetLength(RedoArray,0);
end;


procedure TEditUndoer.StoreReDo(Edit: TEdit);
begin
AddUndoItem(RedoArray,Edit.Text,Edit.SelStart,Edit.SelLength);
end;


procedure TEditUndoer.GetUndo(Edit: TEdit);
var Item: TUndoItem;
begin
if Length(UndoArray)<1 then exit;
StoreReDo(Edit);
GetLastUndoItem(UndoArray,Item);
Edit.Text:=Item.Text;
If Edit.CanFocus then Edit.SetFocus;
Edit.Invalidate;
Edit.SelStart:=Item.Pos;
Edit.SelLength:=Item.Len;
end;



procedure TEditUndoer.GetReDo(Edit: TEdit);
var Item: TUndoItem;
begin
if Length(RedoArray)<1 then exit;
{Add without clearing redo array}
AddUndoItem(UndoArray,Edit.Text,Edit.SelStart,Edit.SelLength);
if not GetLastUndoItem(RedoArray,Item) then exit;
Edit.Text:=Item.Text;
If Edit.CanFocus then Edit.SetFocus;
Edit.Invalidate;
Edit.SelStart:=Item.Pos;
Edit.SelLength:=Item.Len;
end;



{------------------------------------------------------------------------------}




procedure TCustomEditEx.SetAlignment(const Value: TAlignment);
begin
if FAlignment <> Value then
	begin
	FAlignment := Value;
	RecreateWnd;
	end;
end;



procedure TCustomEditEx.CreateParams(var Params: TCreateParams);
const Alignments: array[TAlignment] of DWORD = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
inherited CreateParams(Params);
with Params do Style:= Style or Alignments[FAlignment];
end;



constructor TCustomEditEx.Create(AOwner: TComponent);
begin
inherited;
Undoer:=TEditUndoer.Create;
FAlignment := taLeftJustify;
FShortCutMode:=smNone;
end;



destructor TCustomEditEx.Destroy;
begin
Undoer.Free;
inherited;
end;


procedure TCustomEditEx.WMKillFocus(var Message: TWMKillFocus);
begin
inherited;
if Assigned(FOnKillFocus) then FOnKillFocus(Self);
end;




procedure TCustomEditEx.DoUndo;
begin
Undoer.GetUndo(TEdit(Self));
if Assigned(FOnUndo) then FOnUndo(Self);
end;



procedure TCustomEditEx.DoRedo;
begin
Undoer.GetReDo(TEdit(Self));
if Assigned(FOnRedo) then FOnRedo(Self);
end;



procedure TCustomEditEx.KeyDown(var Key: Word; Shift: TShiftState);

	function CanForwardDel: boolean;
	begin
	Result:=(SelStart<Length(Text)) or (SelLength>0);
	end;



begin
if Key=VK_Delete then
	begin
        if CanForwardDel then Undoer.StoreUndo(TEdit(Self));
        end;
inherited;
end;




procedure TCustomEditEx.KeyPress(var Key: Char);
var Handled: boolean;


	function CanBackspace: boolean;
	begin
	Result:=(SelStart>0) or (SelLength>0);
	end;


	function ExtendedShortCuts: boolean;
	begin
	Result:=True;
	case Key of
	 CtrlA: SelectAll;
	 CtrlZ: DoUndo;
	 CtrlY: DoRedo;
	 CtrlH: if CanBackspace then Result:=False;
	 else Result:=False;
	 end;
	end;


begin
Handled:=False;
case FShortCutMode of
 smStandard: Handled:=ExtendedShortCuts;
 end;
if Handled then Key:=#0
else Undoer.StoreUndo(TEdit(Self));
inherited;
end;



{--------------------- Routines for Arrow Button ----------------------}

procedure TArrowButton.SetUpArrow(V: boolean);
begin
if V <> FUpArrow then
	begin
	FUpArrow:=V;
	Invalidate;
	end;
end;


procedure TArrowButton.DrawDownArrow;
var X1,X2,Y1,Y2,X3: integer;
var Color: TColor;
begin
X1:=(Width - ArrowWidth) div 2;
X2:=X1 + ArrowWidth-1;
X3:=X1 + (ArrowWidth div 2);
Y1:=(Height-ArrowHeight) div 2;
Y2:=Y1 + ArrowHeight;
if Enabled then Color:=clBlack else Color:=clGray;
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Color;
Canvas.Polygon([Point(X1, Y1), Point(X2, Y1), Point(X3, Y2)]);
end;



procedure TArrowButton.DrawUpArrow;
var X1,X2,Y1,Y2,X3: integer;
var Color: TColor;
begin
X1:=(Width - ArrowWidth) div 2;
X2:=X1 + ArrowWidth-1;
X3:=X1 + (ArrowWidth div 2);
Y2:=((Height-ArrowHeight) div 2)-1;
Y1:=Y2 + ArrowHeight;
if Enabled then Color:=clBlack else Color:=clGray;
Canvas.Pen.Color:=Color;
Canvas.Brush.Color:=Color;
Canvas.Polygon([Point(X1, Y1), Point(X2, Y1), Point(X3, Y2)]);
end;


procedure TArrowButton.Paint;
var R: TRect;
begin
inherited Paint;
if UpArrow then DrawUpArrow else DrawDownArrow;
if FDrawFocus then
	begin
	R:=Bounds(0, 0, Width, Height);
	InflateRect(R, -3, -3);
	if Down then OffsetRect(R, 1, 1);
	Canvas.Brush.Color:=clWhite;
	Canvas.DrawFocusRect(R);
	end;
end;



{----------------- Routines for Up/Down Arrow button ----------------------}


function TUpDownButton.CreateButton: TArrowButton;
begin
Result := TArrowButton.Create(Self);
Result.OnClick := BtnClick;
Result.OnMouseDown := BtnMouseDown;
Result.Visible := True;
Result.Enabled := True;
Result.DoRepeat:=true;;
Result.NumGlyphs := 1;
Result.Glyph:=nil;
Result.Parent := Self;
end;


constructor TUpDownButton.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
ControlStyle := ControlStyle - [csAcceptsControls, csSetCaption] + [csFramed, csOpaque];

FUpButton := CreateButton;
FDownButton := CreateButton;
FUpButton.UpArrow:=True;
FDownButton.UpArrow:=False;

Width := 17; Height := 23;
FFocusedButton := FUpButton;
end;



procedure TUpDownButton.SetEnabled(V: boolean);
begin
inherited SetEnabled(V);
FUpButton.Enabled:=V;
FDownButton.Enabled:=V;
end;



procedure TUpDownButton.AdjustSpinSize(var W,H: integer);
begin
if (FUpButton = nil) or (csLoading in ComponentState) then Exit;
if W < 15 then W := 15;
FUpButton.SetBounds(0, 0, W, H div 2);
FDownButton.SetBounds(0, FUpButton.Height - 1, W, H - FUpButton.Height + 1);
end;



procedure TUpDownButton.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var W, H: Integer;
begin
W := AWidth;
H := AHeight;
AdjustSpinSize (W, H);
inherited SetBounds (ALeft, ATop, W, H);
end;



procedure TUpDownButton.WMSize(var Message: TWMSize);
var W,H: Integer;
begin
inherited;
W := Width; H := Height;
AdjustSpinSize(W, H);
if (W <> Width) or (H <> Height) then inherited SetBounds(Left, Top, W, H);
Message.Result := 0;
end;


procedure TUpDownButton.BtnClick(Sender: TObject);
begin
if Sender = FUpButton then
	begin
	if Assigned(FOnUpClick) then FOnUpClick(Self);
	end
else if Assigned(FOnDownClick) then FOnDownClick(Self);
end;



procedure TUpDownButton.Loaded;
var W, H: Integer;
begin
inherited Loaded;
W := Width;
H := Height;
AdjustSpinSize (W, H);
if (W <> Width) or (H <> Height) then inherited SetBounds (Left, Top, W, H);
end;



procedure TUpDownButton.WMSetFocus(var Message: TWMSetFocus);
begin
FFocusedButton.FDrawFocus:=True;
FFocusedButton.Invalidate;
end;



procedure TUpDownButton.WMKillFocus(var Message: TWMKillFocus);
begin
FFocusedButton.FDrawFocus:=False;
FFocusedButton.Invalidate;
end;



procedure TUpDownButton.SetFocusBtn(Btn: TArrowButton);
begin
if TabStop and CanFocus and  (Btn <> FFocusedButton) then
	begin
	FFocusedButton.FDrawFocus:=False;
	FFocusedButton := Btn;
	if (GetFocus = Handle) then
		begin
		FFocusedButton.FDrawFocus:=True;
		Invalidate;
		end;
	end;
end;



procedure TUpDownButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
case Key of
 VK_UP:
	begin
	SetFocusBtn (FUpButton);
	FUpButton.Click;
	end;
 VK_DOWN:
	begin
	SetFocusBtn (FDownButton);
	FDownButton.Click;
	end;
 VK_SPACE: FFocusedButton.Click;
 end;
end;



procedure TUpDownButton.BtnMouseDown (Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
if Button = mbLeft then
	begin
	SetFocusBtn(TArrowButton(Sender));
	if (FFocusControl <> nil) and FFocusControl.TabStop and 
	    FFocusControl.CanFocus and (GetFocus <> FFocusControl.Handle) then FFocusControl.SetFocus
	else if TabStop and (GetFocus <> Handle) and CanFocus then SetFocus;
	end;
end;



procedure TUpDownButton.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
Message.Result := DLGC_WANTARROWS;
end;




function TUpDownButton.GetButtonPressed: boolean;
begin
Result:=Self.UpButton.Pressed or Self.DownButton.Pressed;
end;



procedure TUpDownButton.Notification(AComponent: TComponent;
  Operation: TOperation);
{Handle the live deletion of a Focus Control controlled by the object at designtime }
begin
inherited Notification(AComponent, Operation);
if (Operation = opRemove) and (AComponent = FFocusControl) then FFocusControl := nil;
end;

{================== TCustomSpin ===============================================}


constructor TCustomSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FButton := TUpDownButton.Create (Self);
FButton.Width := 15;
FButton.Height := 17;
FButton.Visible := True;
FButton.Parent := Self;
FButton.FocusControl := Self;

FPanel := TDisPanel.Create (Self);
FPanel.Parent := Self;
FPanel.Width := 0;
FPanel.Height := 17;
FPanel.Visible := False;
FPanel.BevelInner:=bvNone;
FPanel.BevelOuter:=bvRaised;
FPanel.BorderStyle:=bsNone;
FPanel.ParentCtl3D:=False;
FPanel.SetDesigning(False);

Text := '0';
ControlStyle := ControlStyle - [csSetCaption];
FEditorEnabled := True;
FUseLimit:=False;
FBlockChangeEvent:=False;
end;

destructor TCustomSpin.Destroy;
begin
FPanel:=Nil;
FButton := nil;
inherited Destroy;
end;



procedure TCustomSpin.SetEnabled(Value: Boolean);
begin
inherited SetEnabled(Value);     
if Value then FPanel.Font.Color:=clBlack
else FPanel.Font.Color:=clGray;
FButton.Enabled:=Value;
FPanel.Enabled:=Value;
end;



procedure TCustomSpin.CreateParams(var Params: TCreateParams);
begin
inherited CreateParams(Params);
{  Params.Style := Params.Style and not WS_BORDER;  }
Params.Style := Params.Style or ES_MULTILINE or WS_CLIPCHILDREN;
end;

procedure TCustomSpin.Change;
begin
if FBlockChangeEvent then exit;
inherited Change;
end;

procedure TCustomSpin.CreateWnd;
begin
inherited CreateWnd;
SetEditRect;
end;


procedure TCustomSpin.SetEditRect;
var Loc: TRect;
begin
SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
Loc.Bottom := ClientHeight;
Loc.Right := ClientWidth - FButton.Width - 2;
if FPanel.Visible then Dec(Loc.Right,FPanel.Width);
Loc.Top := 0;
Loc.Left := 0;
SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));  {debug}
end;



procedure TCustomSpin.WMSize(var Message: TWMSize);
var  MinHeight: Integer;
begin
inherited;
if (csLoading in ComponentState) then MinHeight:=0
else MinHeight := GetMinHeight;
{ text edit bug: if size to less than minheight, then edit ctrl does not display the text }
if Height < MinHeight then Height := MinHeight
else
	begin
	if FButton<>nil then
		begin
		if (Lo(GetVersion) >= 4) and Ctl3D then FButton.SetBounds(Width - FButton.Width-4, 0, FButton.Width, Height - 4)
		else FButton.SetBounds(Width - FButton.Width-1, 1, FButton.Width, Height - 2);
		end;
	if FPanel<>nil then FPanel.SetBounds(FButton.Left-FPanel.Width,FButton.Top,FPanel.Width,FButton.Height);
	SetEditRect;
	end;
end;


function TCustomSpin.GetMinHeight: Integer;
var DC: HDC;
var SaveFont: HFont;
var Metrics: TTextMetric;
begin
DC:=GetDC(0);
try
SaveFont:=SelectObject(DC, Font.Handle);
GetTextMetrics(DC, Metrics);
SelectObject(DC, SaveFont);
finally ReleaseDC(0, DC); end;
Result := Metrics.tmHeight + GetSystemMetrics(SM_CYBORDER) * 2 + 2;
end;



procedure TCustomSpin.WMPaste(var Message: TWMPaste);
begin
if not FEditorEnabled or ReadOnly then Exit;
inherited;
end;


procedure TCustomSpin.WMCut(var Message: TWMPaste);
begin
if not FEditorEnabled or ReadOnly then Exit;
inherited;
end;




procedure TCustomSpin.UpdatePanelWidth;
var   Size: Longint;
begin
if FPanel.Caption='' then FPanel.Width:=0
else if FPanelWidth<>0 then FPanel.Width:=FPanelWidth
else FPanel.Width:=FPanel.Canvas.TextWidth(FPanel.Caption)+6;
Size := ClientWidth + (Longint(ClientHeight) shl 16);
SendMessage(Handle, WM_SIZE, SIZE_RESTORED, Size);
end;



procedure TCustomSpin.SetUnitsString(V: string);
begin
FPanel.Caption:=V;
FPanel.Visible:=FPanel.Caption<>'';
UpdatePanelWidth;
end;


function TCustomSpin.GetUnitsString: string;
begin
Result:=FPanel.Caption;
end;


procedure TCustomSpin.SetPanelWidth(NewValue: integer);
begin
if FPanelWidth <> NewValue then
	begin
	FPanelWidth:=NewValue;
	UpdatePanelWidth;
	end;
end;


procedure TCustomSpin.CMEnter(var Message: TCMGotFocus);
begin
if AutoSelect and not (csLButtonDown in ControlState) then SelectAll;
inherited;
end;



procedure TCustomSpin.Wrapped(WrapMode: TWrapMode);
begin
if Assigned(FOnWrap) then FOnWrap(Self,WrapMode);
end;


end.
