unit VectorSpin;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, Menus, StrUtils, DSupport,
  Realspin, SpinButtons, Vector;


type
 TVectorSpin = class(TCustomPanel)
 private
  FAlignment: TAlignment;
  FBlockUpdate: boolean;
  FXSpin,FYSpin,FZSpin: TRealSpin;

  FXLabel: TLabel;
  FYLabel: TLabel;
  FZLabel: TLabel;
  FSpinFont: TFont;
  FLabelFont: TFont;
  FOnChange: TNotifyEvent;
  FBlockChangeEvent: boolean;
  FWrap: boolean;
  FSyncValues: boolean;
  FUse2DVector: boolean;
  procedure SetUnitsString(V: string);
  function GetUnitsString: string;
  procedure SetAlignment(Value: TAlignment);
  procedure Arrange;
  procedure Change(Sender: TObject);
  procedure SetSpinFont(Value: TFont);
  procedure SetLabelFont(Value: TFont);
  procedure SetDigits(Value: integer);
  procedure FontChanged(Sender: TObject);
  procedure SetNoEventValue(NewValue: T3DVector);
  procedure WMSize(var Message: TWMSize);  message WM_SIZE;
  function GetVector: T3DVector;
  procedure SetVector(const Value: T3DVector);
  function GetDigits: integer;
  function GetFormat: TNumberFormat;
  procedure SetFormat(const Value: TNumberFormat);
  function GetMaxValue: double;
  function GetMinValue: double;
  function GetUselimit: boolean;
  procedure SetMaxValue(const Value: double);
  procedure SetMinValue(const Value: double);
  procedure SetUselimit(const Value: boolean);
  function GetWrap: boolean;
  procedure SetWrap(const Value: boolean);
  function GetIncrement: double;
  procedure SetIncrement(const Value: double);
  procedure SetXValue(const Value: double);
  procedure SetYValue(const Value: double);
  procedure SetZValue(const Value: double);
  function GetXValue: double;
  function GetYValue: double;
  function GetZValue: double;
  function GetButtonPressed: boolean;
  procedure HandleXChange(Sender: TObject);
  procedure HandleYChange(Sender: TObject);
  procedure HandleZChange(Sender: TObject);
  function GetXCaption: string;
  function GetYCaption: string;
  function GetZCaption: string;
  procedure SetXCaption(const Value: string);
  procedure SetYCaption(const Value: string);
  procedure SetZCaption(const Value: string);
  procedure SetUse2DVector(const Value: boolean);
  procedure Set2DVector(const Value: T2DVector);
  function Get2DVector: T2DVector;
 protected
  procedure SetEnabled(Value: Boolean); override;
  procedure Paint; override;
 public
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  property NoEventValue: T3DVector read GetVector write SetNoEventValue;
  property Pressed: boolean read GetButtonPressed;
 published
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property SpinFont: TFont read FSpinFont write SetSpinFont;
  property LabelFont: TFont read FLabelFont write SetLabelFont;
  property Digits: integer read GetDigits write SetDigits default 1;
  property BlockChangeEvent: boolean read FBlockChangeEvent write FBlockChangeEvent default false;
  property UnitsString: string read GetUnitsString write SetUnitsString;
  property Alignment: TAlignment read FAlignment write SetAlignment;
  property Value: T3dVector read GetVector write SetVector;
  property Vector2D: T2DVector read Get2DVector write Set2DVector;
  property NumFormat: TNumberFormat read GetFormat write SetFormat default nfStandard;
  property SyncValues: boolean read FSyncValues write FSyncValues default False;
  property MaxValue: double read GetMaxValue write SetMaxValue;
  property MinValue: double read GetMinValue write SetMinValue;
  property UseLimit: boolean read GetUselimit write SetUselimit default False;
  property Wrap: boolean read GetWrap write SetWrap default False;
  property Increment: double read GetIncrement write SetIncrement;
  property XValue: double read GetXValue write SetXValue;
  property YValue: double read GetYValue write SetYValue;
  property ZValue: double read GetZValue write SetZValue;
  property XCaption: string read GetXCaption write SetXCaption;
  property YCaption: string read GetYCaption write SetYCaption;
  property ZCaption: string read GetZCaption write SetZCaption;
  property Use2DVector: boolean read FUse2DVector write SetUse2DVector default False;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Color;
    property Ctl3D;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
 end;



procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Fountain', [TVectorSpin]);
end;



procedure TVectorSpin.SetAlignment(Value: TAlignment);
begin
if FAlignment<>Value then
	begin
	FAlignment:=Value;
	FXSpin.Alignment:=Value;
	FYSpin.Alignment:=Value;
	FZSpin.Alignment:=Value;
	end;
end;



procedure TVectorSpin.SetEnabled(Value: Boolean);
begin
inherited SetEnabled(Value);
FXSpin.Enabled:=Value;
FYSpin.Enabled:=Value;
FZSpin.Enabled:=Value;
FXLabel.Enabled:=Value;
FYLabel.Enabled:=Value;
FZLabel.Enabled:=Value;
end;



procedure TVectorSpin.SetUnitsString(V: string);
begin
FXSpin.UnitsString:=V;
end;


function TVectorSpin.GetUnitsString: string;
begin
Result:=FXSpin.UnitsString;
end;




{-----------------------------------------------------------------------------}





procedure TVectorSpin.Change(Sender: TObject);
begin
if BlockChangeEvent then exit;
if csLoading in ComponentState then exit;
if Assigned(FOnChange) then FOnChange(Self);
end;




procedure TVectorSpin.Arrange;
var ItemWidth,RelWidth,Padding: integer;
var SpinTop, ItemCount: integer;

	procedure SetupFonts;
	begin
	FXSpin.Font:=FSpinFont;
	FYSpin.Font:=FSpinFont;
	FZSpin.Font:=FSpinFont;
	FXLabel.Font:=FLabelFont;
	FYLabel.Font:=FLabelFont;
	FZLabel.Font:=FLabelFont;
	end;


	procedure SetWidths(Width: integer);
	begin
	FXSpin.Width:=Width;
	FYSpin.Width:=Width;
	FZSpin.Width:=Width;
	end;


begin
Caption:='';
SetupFonts;
{Setup for two or three spinner display}
if Use2DVector then
	begin
	RelWidth:=45;
        ItemCount:=2;
        FZSpin.Visible:=False;
        FZLabel.Visible:=False;
	end
else
	begin
	RelWidth:=30;
	ItemCount:=3;
	FZSpin.Visible:=True;
	FZLabel.Visible:=True;
	end;
{Work out the spinner width}
ItemWidth:=MulDiv(Width,RelWidth,100);
SetWidths(ItemWidth);

{Work out padding between spinners}
Padding:=(Width-(ItemCount*ItemWidth)) div (ItemCount+1);

{Set position of spinners}
FXSpin.Left:=Padding+1;
FYSpin.Left:=(2*Padding)+ItemWidth+1;
FZSpin.Left:=(3*Padding)+(2*ItemWidth)+1;

{Set lable position}
FXLabel.Left:=FXSpin.Left;
FYLabel.Left:=FYSpin.Left;
FZLabel.Left:=FZSpin.Left;


{Work out spinner top position}
SpinTop:=Height-(FXSpin.Height+2);
SpinTop:=MulDiv(SpinTop,75,100);

{Set spinner tops}
FXSpin.Top:=SpinTop; FYSpin.Top:=SpinTop; FZSpin.Top:=SpinTop;

{Set lable tops}
FXLabel.Top:=FXSpin.Top-FXLabel.Height;
FYLabel.Top:=FYSpin.Top-FXLabel.Height;
FZLabel.Top:=FZSpin.Top-FXLabel.Height;
end;



procedure TVectorSpin.WMSize(var Message: TWMSize);
begin
inherited;
Arrange;
end;



procedure TVectorSpin.Paint;
begin
inherited Paint;
end;


procedure TVectorSpin.FontChanged(Sender: TObject);
begin
Arrange;
end;


procedure TVectorSpin.SetSpinFont(Value: TFont);
begin
FSpinFont.Assign(Value);
Arrange;
end;


procedure TVectorSpin.SetLabelFont(Value: TFont);
begin
FLabelFont.Assign(Value);
Arrange;
end;


procedure TVectorSpin.SetDigits(Value: integer);
begin
FXSpin.Digits:=Value;
FYSpin.Digits:=Value;
FZSpin.Digits:=Value;
end;



function TVectorSpin.GetDigits: integer;
begin
Result:=FXSpin.Digits;
end;



procedure TVectorSpin.SetNoEventValue(NewValue: T3DVector);
begin
FXSpin.NoEventValue:=NewValue.X;
FYSpin.NoEventValue:=NewValue.Y;
FZSpin.NoEventValue:=NewValue.Z;
end;



procedure TVectorSpin.HandleXChange(Sender: TObject);
begin
if FSyncValues then
	begin
	FYSpin.NoEventValue:=FXSpin.Value;
	FZSpin.NoEventValue:=FXSpin.Value;
	end;
Change(Sender);
end;



procedure TVectorSpin.HandleYChange(Sender: TObject);
begin
if FSyncValues then
	begin
	FXSpin.NoEventValue:=FYSpin.Value;
	FZSpin.NoEventValue:=FYSpin.Value;
	end;
Change(Sender);
end;



procedure TVectorSpin.HandleZChange(Sender: TObject);
begin
if FSyncValues then
	begin
	FXSpin.NoEventValue:=FZSpin.Value;
	FYSpin.NoEventValue:=FZSpin.Value;
	end;
Change(Sender);
end;



constructor TVectorSpin.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FXSpin:=TRealSpin.Create(Self);
FXSpin:=TRealSpin.Create(Self);
FYSpin:=TRealSpin.Create(Self);
FZSpin:=TRealSpin.Create(Self);
{This allow the control to be hidden at design time}
FZSpin.ControlStyle:=FZSpin.ControlStyle+[csNoDesignVisible];

FXSpin.Parent:=Self;
FYSpin.Parent:=Self;
FZSpin.Parent:=Self;

FXSpin.OnChange:=HandleXChange;
FYSpin.OnChange:=HandleYChange;
FZSpin.OnChange:=HandleZChange;

FLabelFont:=TFont.Create;
FLabelFont.Size:=8;
FLabelFont.Name:='Arial';
FLabelFont.Style:=[];
FLabelFont.OnChange:=FontChanged;

FSpinFont:=TFont.Create;
FSpinFont.OnChange:=FontChanged;

Height:=50;

FXLabel:=TLabel.Create(Self);
FYLabel:=TLabel.Create(Self);
FZLabel:=TLabel.Create(Self);
{This allow the control to be hidden at design time}
FZLabel.ControlStyle:=FZLabel.ControlStyle+[csNoDesignVisible];

FYLabel.Parent:=Self;
FXLabel.Parent:=Self;
FZLabel.Parent:=Self;

FXLabel.Caption:='X';
FYLabel.Caption:='Y';
FZLabel.Caption:='Z';
FZLabel.Visible:=False;

FBlockUpdate:=False;
Digits:=1;
FUse2DVector:=False;

MaxValue:=0;
MinValue:=0;
UseLimit:=False;
FBlockChangeEvent:=False;
FWrap:=False;
NumFormat:=nfStandard;
Uselimit:=False;
Wrap:=False;
FZSpin.Visible:=False;
Arrange;
end;



destructor TVectorSpin.Destroy;
begin
FXSpin.Free; FYSpin.Free; FZSpin.Free;
FXLabel.Free; FYLabel.Free; FZLabel.Free;
FLabelFont.Free;
FSpinFont.Free;

inherited Destroy;
end;



function TVectorSpin.GetVector: T3DVector;
begin
Result:=MakeVector3D(FXSpin.Value,FYSpin.Value,FZSpin.Value);
end;


procedure TVectorSpin.SetVector(const Value: T3DVector);
begin
FXSpin.Value:=Value.X;
FYSpin.Value:=Value.Y;
FZSpin.Value:=Value.Z;
end;


function TVectorSpin.GetFormat: TNumberFormat;
begin
Result:=FXSpin.NumFormat;
end;



procedure TVectorSpin.SetFormat(const Value: TNumberFormat);
begin
FXSpin.NumFormat:=Value;
FYSpin.NumFormat:=Value;
FZSpin.NumFormat:=Value;
end;



function TVectorSpin.GetMaxValue: double;
begin
Result:=FXSpin.MaxValue;
end;



function TVectorSpin.GetMinValue: double;
begin
Result:=FXSpin.MinValue;
end;


function TVectorSpin.GetUselimit: boolean;
begin
Result:=FXSpin.UseLimit;
end;



procedure TVectorSpin.SetMaxValue(const Value: double);
begin
FXSpin.MaxValue:=Value;
FYSpin.MaxValue:=Value;
FZSpin.MaxValue:=Value;
end;



procedure TVectorSpin.SetMinValue(const Value: double);
begin
FXSpin.MinValue:=Value;
FYSpin.MinValue:=Value;
FZSpin.MinValue:=Value;
end;



procedure TVectorSpin.SetUselimit(const Value: boolean);
begin
FXSpin.UseLimit:=Value;
FYSpin.UseLimit:=Value;
FZSpin.UseLimit:=Value;
end;



function TVectorSpin.GetWrap: boolean;
begin
Result:=FXSpin.Wrap;
end;

procedure TVectorSpin.SetWrap(const Value: boolean);
begin
FXSpin.Wrap:=Value;
FYSpin.Wrap:=Value;
FZSpin.Wrap:=Value;
end;



function TVectorSpin.GetIncrement: double;
begin
Result:=FXSpin.Increment;
end;



procedure TVectorSpin.SetIncrement(const Value: double);
begin
FXSpin.Increment:=Value;
FYSpin.Increment:=Value;
FZSpin.Increment:=Value;
end;



procedure TVectorSpin.SetXValue(const Value: double);
begin
if FXSpin.Value<>Value then
	begin
	FXSpin.Value:=Value;
	end;
end;



procedure TVectorSpin.SetYValue(const Value: double);
begin
if FYSpin.Value<>Value then
	begin
	FYSpin.Value:=Value;
	end;
end;



procedure TVectorSpin.SetZValue(const Value: double);
begin
if FZSpin.Value<>Value then
	begin
	FZSpin.Value:=Value;
	end;
end;



function TVectorSpin.GetXValue: double;
begin
Result:=FXSpin.Value;
end;



function TVectorSpin.GetYValue: double;
begin
Result:=FYSpin.Value;
end;



function TVectorSpin.GetZValue: double;
begin
Result:=FZSpin.Value;
end;


function TVectorSpin.GetButtonPressed: boolean;
begin
Result:=FXSpin.Button.Pressed or FYSpin.Button.Pressed or FZSpin.Button.Pressed;
end;


function TVectorSpin.GetXCaption: string;
begin
Result:=FXLabel.Caption;
end;

function TVectorSpin.GetYCaption: string;
begin
Result:=FYLabel.Caption;
end;

function TVectorSpin.GetZCaption: string;
begin
Result:=FZLabel.Caption;
end;

procedure TVectorSpin.SetXCaption(const Value: string);
begin
FXLabel.Caption:=Value;
end;

procedure TVectorSpin.SetYCaption(const Value: string);
begin
FYLabel.Caption:=Value;
end;

procedure TVectorSpin.SetZCaption(const Value: string);
begin
FZLabel.Caption:=Value;
end;

procedure TVectorSpin.SetUse2DVector(const Value: boolean);
begin
if FUse2DVector<>Value then
	begin
	FUse2DVector := Value;
	Arrange;
	end;
end;

procedure TVectorSpin.Set2DVector(const Value: T2DVector);
begin
FXSpin.Value:=Value.X;
FYSpin.Value:=Value.Y;
end;



function TVectorSpin.Get2DVector: T2DVector;
begin
Result:=MakeVector2D(FXSpin.Value,FYSpin.Value);
end;




end.
