unit ComSetup;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls,CommPort,Dialogs,IniFiles,BitmapProg;

type
  TCommSetupDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Bevel1: TBevel;
    PortCombo: TComboBox;
    Label1: TLabel;
    BaudCombo: TComboBox;
    Label2: TLabel;
    ParityCombo: TComboBox;
    Label3: TLabel;
    DataBitsCombo: TComboBox;
    Label4: TLabel;
    StopBitsCombo: TComboBox;
    Label5: TLabel;
    WarningImage: TImage;
    WarningDis: TLabel;
    procedure PortComboDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure PortComboChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  CommSetupDlg: TCommSetupDlg;

procedure DoCommSettingsDlg(Title: string; ComPort: TCommPort); overload;
procedure DoCommSettingsDlg(ComPort: TCommPort); overload;

procedure SetDefaultPortSettings(var ComPort: TCommPort);

procedure GetPortSettings(Ini: TIniFile; Section: string; var ComPort: TCommPort); overload;
procedure GetPortSettings(Filename: string; Section: string; var ComPort: TCommPort); overload;

procedure SavePortSettings(Ini: TIniFile; Section: string; ComPort: TCommPort); overload;
procedure SavePortSettings(Filename: string; Section: string; ComPort: TCommPort); overload;



implementation


{$R *.DFM}



{----------------- Deal with serial Poert settings ---------------------------}


procedure SetDefaultPortSettings(var ComPort: TCommPort);
begin
ComPort.Port:=tptNone;
ComPort.BaudRate:=tbr9600;
ComPort.DataBits:=tdbEight;
ComPort.Parity:=tpNone;
ComPort.StopBits:=tsbOne;
end;



procedure GetPortSettings(Ini: TIniFile; Section: string; var ComPort: TCommPort);
begin
ComPort.Port:=TSerialPort(Ini.ReadInteger(Section,'Port',Integer(tptNone)));
ComPort.BaudRate:=TBaudRate(Ini.ReadInteger(Section,'BaudRate', Integer(tbr9600)));
ComPort.DataBits:=TDataBits(Ini.ReadInteger(Section,'DataBits', Integer(tdbEight)));
ComPort.Parity:=TParity(Ini.ReadInteger(Section,'Parity', Integer(tpNone)));
ComPort.StopBits:=TStopBits(Ini.ReadInteger(Section,'StopBits', Integer(tsbOne)));
end;



procedure SavePortSettings(Ini: TIniFile; Section: string; ComPort: TCommPort);
begin
Ini.WriteInteger(Section,'Port', Integer(ComPort.Port));
Ini.WriteInteger(Section,'BaudRate', Integer(ComPort.BaudRate));
Ini.WriteInteger(Section,'DataBits', Integer(ComPort.DataBits));
Ini.WriteInteger(Section,'Parity', Integer(ComPort.Parity));
Ini.WriteInteger(Section,'StopBits', Integer(ComPort.StopBits));
end;



procedure GetPortSettings(Filename: string; Section: string; var ComPort: TCommPort);
var SerialIni: TIniFile;
begin
SerialIni:=TIniFile.Create(Filename);
try
GetPortSettings(SerialIni,Section,ComPort);
finally SerialIni.Free; end;
end;



procedure SavePortSettings(Filename: string; Section: string; ComPort: TCommPort);
var SerialIni: TIniFile;
begin
SerialIni:=TIniFile.Create(Filename);
try
SavePortSettings(SerialIni,Section,ComPort);
finally SerialIni.Free; end;
end;



{------------------------------------------------------------------------------}


function ListAvailablePorts: boolean;
{List all available ports. }
{Return false if no ports available.}
var I: integer;
var S: string;
var A: boolean;
begin
Result:=False;
CommSetupDlg.PortCombo.Clear;
DisplayProgress('Scanning Ports');
for I:=0 to MaxCommPorts-2 do
	begin
	Application.ProcessMessages;
	A:=IsPortAvailable(TSerialPort(I));
	S:='Com'+IntToStr(I+1);
	CommSetupDlg.PortCombo.Items.AddObject(S,TObject(A));
	if A then Result:=True;
	DoProgress(MulDiv(100,I,MaxCommPorts-2));
	end;
HideProgress;
CommSetupDlg.PortCombo.Items.AddObject('None',TObject(True));
end;



procedure DisplayAvailableWarning;
var A: boolean;
var I: integer;
var S: string;
var C: TColor;
begin
I:=CommSetupDlg.PortCombo.ItemIndex;
if (I=-1) or (I=PortToIndex(tptNone)) then
	begin
	S:='Warning! No Port Selected!';
	C:=clMaroon;
	end
else
	begin
	S:='Warning! Selected Port Unavailable !';
	C:=clRed;
	end;
CommSetupDlg.WarningDis.Caption:=S;
CommSetupDlg.WarningDis.Font.Color:=C;
A:=IsPortAvailable(TSerialPort(I));
CommSetupDlg.WarningDis.Visible:=not A;
CommSetupDlg.WarningImage.Visible:=not A;
end;






procedure GetSettings(ComPort: TCommPort);
begin
with ComPort,CommSetupDlg do
 begin
 PortCombo.ItemIndex:= PortToIndex(ComPort.Port);
 case BaudRate of
  tbr110: BaudCombo.ItemIndex:=0;
  tbr300: BaudCombo.ItemIndex:=1;
  tbr600: BaudCombo.ItemIndex:=2;
  tbr1200: BaudCombo.ItemIndex:=3;
  tbr2400: BaudCombo.ItemIndex:=4;
  tbr4800: BaudCombo.ItemIndex:=5;
  tbr9600: BaudCombo.ItemIndex:=6;
  tbr14400: BaudCombo.ItemIndex:=7;
  tbr19200: BaudCombo.ItemIndex:=8;
  tbr38400: BaudCombo.ItemIndex:=9;
  tbr56000: BaudCombo.ItemIndex:=10;
  tbr57600: BaudCombo.ItemIndex:=11;
  tbr115200: BaudCombo.ItemIndex:=12;
  tbr128000: BaudCombo.ItemIndex:=13;
  tbr230400: BaudCombo.ItemIndex:=14;
  tbr256000: BaudCombo.ItemIndex:=15;
  tbr460800: BaudCombo.ItemIndex:=16;
  tbr500000: BaudCombo.ItemIndex:=17;
  tbr512000: BaudCombo.ItemIndex:=18;
  tbr921600: BaudCombo.ItemIndex:=19;
  tbr1000000: BaudCombo.ItemIndex:=20;
  tbr1024000: BaudCombo.ItemIndex:=21;
  tbr1500000: BaudCombo.ItemIndex:=22;
  tbr2000000: BaudCombo.ItemIndex:=23;
  tbr2500000: BaudCombo.ItemIndex:=24;
  end;

 case Parity of
  tpNone: ParityCombo.ItemIndex:=0;
  tpOdd: ParityCombo.ItemIndex:=1;
  tpEven: ParityCombo.ItemIndex:=2;
  tpMark: ParityCombo.ItemIndex:=3;
  tpSpace: ParityCombo.ItemIndex:=4;
  end;

 case DataBits of 
  tdbFour: DataBitsCombo.ItemIndex:=0;
  tdbFive: DataBitsCombo.ItemIndex:=1;
  tdbSix: DataBitsCombo.ItemIndex:=2;
  tdbSeven: DataBitsCombo.ItemIndex:=3;
  tdbEight: DataBitsCombo.ItemIndex:=4;
  end;

 case StopBits of
  tsbOne: StopBitsCombo.ItemIndex:=0;
  tsbOnePointFive: StopBitsCombo.ItemIndex:=1;
  tsbTwo: StopBitsCombo.ItemIndex:=2;
  end;
 end;
end;


procedure SetSettings(ComPort: TCommPort);
begin
ComPort.Port:=CommPort.IndexToPort(CommSetupDlg.PortCombo.ItemIndex);

case CommSetupDlg.BaudCombo.ItemIndex of
 0: ComPort.BaudRate:=tbr110;
 1: ComPort.BaudRate:=tbr300;
 2: ComPort.BaudRate:=tbr600;
 3: ComPort.BaudRate:=tbr1200;
 4: ComPort.BaudRate:=tbr2400;
 5: ComPort.BaudRate:=tbr4800;
 6: ComPort.BaudRate:=tbr9600;
 7: ComPort.BaudRate:=tbr14400;
 8: ComPort.BaudRate:=tbr19200;
 9: ComPort.BaudRate:=tbr38400;
 10: ComPort.BaudRate:=tbr56000;
 11: ComPort.BaudRate:=tbr57600;
 12: ComPort.BaudRate:=tbr115200;
 13: ComPort.BaudRate:=tbr128000;
 14: ComPort.BaudRate:=tbr230400;
 15: ComPort.BaudRate:=tbr256000;
 16: ComPort.BaudRate:=tbr460800;
 17: ComPort.BaudRate:=tbr500000;
 18: ComPort.BaudRate:=tbr512000;
 19: ComPort.BaudRate:=tbr921600;
 20: ComPort.BaudRate:=tbr1000000;
 21: ComPort.BaudRate:=tbr1024000;
 22: ComPort.BaudRate:=tbr1500000;
 23: ComPort.BaudRate:=tbr2000000;
 24: ComPort.BaudRate:=tbr2500000;
 end;

case CommSetupDlg.ParityCombo.ItemIndex of
 0: ComPort.Parity:=tpNone;
 1: ComPort.Parity:=tpOdd;
 2: ComPort.Parity:=tpEven;
 3: ComPort.Parity:=tpMark;
 4: ComPort.Parity:=tpSpace;
 end;

case CommSetupDlg.DataBitsCombo.ItemIndex of
 0: ComPort.DataBits:=tdbFour;
 1: ComPort.DataBits:=tdbFive;
 2: ComPort.DataBits:=tdbSix;
 3: ComPort.DataBits:=tdbSeven;
 4: ComPort.DataBits:=tdbEight;
 end;

case CommSetupDlg.StopBitsCombo.ItemIndex of
 0: ComPort.StopBits:=tsbOne;
 1: ComPort.StopBits:=tsbOnePointFive;
 2: ComPort.StopBits:=tsbTwo;
 end;
end;



procedure DoCommSettingsDlg(Title: string; ComPort: TCommPort);
var OldPort: TSerialPort;
var OldActive: boolean;
begin
CommSetupDlg.Caption:='Serial Settings '+Title;
OldPort:=ComPort.Port;
ListAvailablePorts;
SetBaudStrings(CommSetupDlg.BaudCombo.Items);
GetSettings(ComPort);
DisplayAvailableWarning;
CommSetupDlg.ShowModal;
if CommSetupDlg.ModalResult = mrOK then
	begin
	OldActive:=ComPort.PortActive;
	ComPort.PortActive:=False;
	{Give some time for port to settle}
	Sleep(100);
	SetSettings(ComPort);
	{Give some time for port to settle}
	Sleep(100);
	try
	ComPort.PortActive:=True;
	except
	 on EOpenComError do
		begin
		MessageDlg('Unable To Open '+ComPort.GetComDescription+'.'+#10#13+
			   'It is either already in use or'+#10#13+
			   'does not exist on this computer.', mtInformation,[mbOk], 0);
		Comport.Port:=OldPort;
		end;
	 end;
	{Give some time for port to settle}
	Sleep(100);
	ComPort.PortActive:=OldActive;
	end;
end;




procedure DoCommSettingsDlg(ComPort: TCommPort);
begin
DoCommSettingsDlg('',Comport);
end;


procedure TCommSetupDlg.PortComboDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
PortCombo.Canvas.FillRect(Rect);
if boolean(PortCombo.Items.Objects[Index]) then
	begin
	PortCombo.Canvas.Font.Color:=clRed;
	PortCombo.Canvas.Font.Style:=[fsBold];
	end;
PortCombo.Canvas.TextOut(Rect.Left,Rect.Top,PortCombo.Items.Strings[Index]);
end;



procedure TCommSetupDlg.PortComboChange(Sender: TObject);
begin
DisplayAvailableWarning;
end;


end.