unit FrameCount;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Autobttn;

type
  TFrameCounter = class(TPanel)
private
  FFrameIndex: integer;
  FMaxFrames: integer;
  FWrap: boolean;
  FStartBtn,FLeftBtn,FRightBtn,FEndBtn: TAutoButton;
  FDoRepeat: boolean;
  FShowCount: boolean;
  FOnChange,FOnMaxChange: TNotifyEvent;
  procedure MakeSpinButton(var B: TAutoButton; A:TAlign);
  procedure ShowFrame;
  procedure SetMaxFrames(V: integer);
  function SetIndex(V: integer): boolean;
  procedure SetIndexChange(V: integer);
  procedure SetFrameIndex(V: integer);
  procedure SetRepeatEnable(V: boolean);
  procedure SetShowCount(V: boolean);
protected
  procedure DoChange; dynamic;
  procedure DoMaxChange; dynamic;
  procedure StartClick (Sender: TObject); virtual;
  procedure LeftClick (Sender: TObject); virtual;
  procedure RightClick (Sender: TObject); virtual;
  procedure EndClick (Sender: TObject); virtual;
  procedure SetEnabled(Value: Boolean); override;
public
  constructor Create(AOwner: TComponent); override;
published
  property FrameIndex: integer read FFrameIndex write SetFrameIndex default 0;
  property OnChange: TNotifyEvent read FOnChange write FOnChange;
  property OnMaxChange: TNotifyEvent read FOnMaxChange write FOnMaxChange;
  property MaxFrames: integer read FMaxFrames write SetMaxFrames default 5;
  property Wrap: boolean read FWrap write FWrap default false;
  property DoRepeat: boolean read FDoRepeat write SetRepeatEnable;
  property ShowCount: boolean read FShowCount write SetShowCount;
  end;

procedure Register;

implementation

{$R FrameCount}

procedure Register;
begin
RegisterComponents('Fountain', [TFrameCounter]);
end;


procedure TFrameCounter.ShowFrame;
begin
if ShowCount then Caption:=IntToStr(FFrameIndex+1)+'/'+IntToStr(FMaxFrames)
else
	begin
	if FMaxFrames>0 then Caption:=IntToStr(FFrameIndex+1)
	else Caption:='';
	end;
end;


procedure TFrameCounter.MakeSpinButton(var B: TAutoButton; A:TAlign);
begin
B:=TAutoButton.Create(Self);
B.Visible := True;
B.Parent := Self;
B.Width := Self.Height - (Self.BevelWidth + Self.BorderWidth);
B.Align:=A;
end;


procedure SetGlyph(B: TAutoButton; Name: String);
begin
B.Glyph.Handle := LoadBitmap(HInstance, PChar(Name));
B.NumGlyphs := 1;
B.Invalidate;
end;


procedure TFrameCounter.SetRepeatEnable(V: boolean);
begin
if V<>FDoRepeat then
	begin
	FLeftBtn.DoRepeat:=V;
	FRightBtn.DoRepeat:=V;
	FDoRepeat:=V;
	end;
end;


constructor TFrameCounter.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
Alignment:=taCenter;
BevelOuter:=bvLowered;
FMaxFrames:=5;
Width := 131;
Height := 20;
MakeSpinButton(FStartBtn,alLeft);
MakeSpinButton(FLeftBtn,alLeft);
MakeSpinButton(FRightBtn,alRight);
MakeSpinButton(FEndBtn,alRight);
SetGlyph(FStartBtn,'START');
SetGlyph(FLeftBtn,'LEFT');
SetGlyph(FRightBtn,'RIGHT');
SetGlyph(FEndBtn,'END');
FStartBtn.OnClick:=StartClick;
FLeftBtn.OnClick:=LeftClick;
FRightBtn.OnClick:=RightClick;
FEndBtn.OnClick:=EndClick;
SetRepeatEnable(True);


Self.Color:=clWhite;
ShowFrame;
end;


procedure TFrameCounter.DoChange;
begin
if Assigned(FOnChange) then FOnChange(Self);
end;


procedure TFrameCounter.DoMaxChange;
begin
if Assigned(FOnMaxChange) then FOnMaxChange(Self);
end;


function TFrameCounter.SetIndex(V: integer): boolean;
begin
Result:=(V<>FFrameIndex) and (V>=0) and (V<FMaxFrames);
if Result then
	begin
	FFrameIndex:=V;
	ShowFrame;
	end;
end;


procedure TFrameCounter.SetIndexChange(V: integer);
begin
if SetIndex(V) then DoChange;
end;


procedure TFrameCounter.StartClick(Sender: TObject);
begin
SetIndexChange(0);
end;


procedure TFrameCounter.LeftClick(Sender: TObject);
begin
if FFrameIndex>0 then SetIndexChange(FFrameIndex-1)
else if FWrap then SetIndexChange(FMaxFrames-1);
end;


procedure TFrameCounter.RightClick(Sender: TObject);
begin
if FFrameIndex<FMaxFrames-1 then SetIndexChange(FFrameIndex+1)
else if FWrap then SetIndexChange(0);
end;



procedure TFrameCounter.EndClick(Sender: TObject);
begin
SetIndexChange(FMaxFrames-1);
end;



procedure TFrameCounter.SetMaxFrames(V: integer);
begin
if V<>FMaxFrames then
	begin
	FMaxFrames:=V;
	if FFrameIndex>FMaxFrames-1 then
		begin
		FFrameIndex:=FMaxFrames-1;
		if FFrameIndex<0 then FFrameIndex:=0;
		end;
	ShowFrame;
	end;
DoMaxChange;
end;



procedure TFrameCounter.SetFrameIndex(V: integer);
begin
SetIndex(V);
end;



procedure TFrameCounter.SetShowCount(V: boolean);
begin
if FShowCount<>V then
	begin
	FShowCount:=V;
	ShowFrame;
	end;
end;



procedure TFrameCounter.SetEnabled(Value: Boolean);
begin
inherited;
FStartBtn.Enabled:=Value;
FLeftBtn.Enabled:=Value;
FRightBtn.Enabled:=Value;
FEndBtn.Enabled:=Value;
end;



end.
