unit Onoffbox;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls,Menus;

type
  TOnOffBox = class(TCustomPanel)
private
  FUserChangeable: boolean;
  FBoxOn: boolean;
  FOnColor,FOffColor: TColor;
  FOnString,FOffString: String;
  procedure SetOnOff(State: boolean);
  procedure SetOnColor(V: TColor);
  procedure SetOffColor(V: TColor);
  procedure SetOnString(New: String);
  procedure SetOffString(New: String);
  procedure SetUserChangeable(Value: Boolean);
  procedure ShowBoxState;
protected
  procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
public
  constructor Create(AOwner: TComponent); override;
published
  property OnString: String read FOnString write SetOnString;
  property OffString: String read FOffString write SetOffString;
  property OnColor: TColor read FOnColor write SetOnColor default clGreen;
  property OffColor: TColor read FOffColor write SetOffColor default clRed;
  property BoxOn: boolean read FBoxOn write SetOnOff default true;
  property UserChangeable: Boolean read FUserChangeable write SetUserChangeable default True;
    property Align;
    property BevelInner;
    property BevelOuter;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property Ctl3D;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
 end;

procedure Register;

implementation




procedure TOnOffBox.SetUserChangeable(Value: Boolean);
begin
if Value <> FUserChangeable then
	begin
	FUserChangeable := Value;
	ShowBoxState;
	end;
end;


procedure TOnOffBox.SetOnColor(V: TColor);
begin
if FOnColor <> V then
	begin
	FOnColor:=V;
	ShowBoxState;
	end;
end;


procedure TOnOffBox.SetOffColor(V: TColor);
begin
if FOffColor <> V then
	begin
	FOffColor:=V;
	ShowBoxState;
	end;
end;


procedure TOnOffBox.SetOnString(New: String);
begin
FOnString:=New;
if FBoxOn then Caption:=New;
end;


procedure TOnOffBox.SetOffString(New: String);
begin
FOffString:=New;
if not FBoxOn then Caption:=New;
end;


procedure TOnOffBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
inherited MouseDown(Button, Shift, X, Y);
if (Button = mbLeft) and FUserChangeable then BoxOn:=not BoxOn;
end;



procedure TOnOffBox.ShowBoxState;
begin
if not Enabled then begin Color:=clLtGray; exit; end;
if FBoxOn then
	begin
	Color:=FOnColor;
	Caption:=FOnString;
	end
else
	begin
	Color:=FOffColor;
	Caption:=FOffString;
	end;
end;



procedure TOnOffBox.SetOnOff(State: boolean);
begin
FBoxOn:=State;
ShowBoxState;
end;



constructor TOnOffBox.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
{$IF CompilerVersion >= 17}
ParentBackground:=False;
{$IFEND}
Width := 26;
Height := 26;
BevelInner:=bvNone;
BevelOuter:=bvLowered;
FUserChangeable:=True;
Alignment:=taCenter;
OffString:='Off'; OnString:='On';
FOnColor:=clLime; FOffColor:=clRed;
BoxOn:=True;
ShowBoxState;
end;



procedure Register;
begin
RegisterComponents('Fountain', [TOnOffBox]);
end;


end.
