unit subtimer;

interface

uses windows,Controls,extctrls,Classes,SysUtils,DSupport;


type
 TResolution=(rsMinutes,rsSeconds,rsMilliSeconds,rsAutoScale);

type
 TSubTimer=class(TPanel)
 private
  FResolution: TResolution;
  FDigits: integer;
  procedure SetDigits(const Value: integer);
  function GetMinutesStr(V: int64): string;
  function ReadElaspedTime: double;
  function GetHoursStr(V: int64): string;
    procedure SetElaspedTime(T: double);
    function GetCurrentCount: int64;
 public
  Running: boolean;
  StartCount,TotCount: int64;
  constructor Create(AOwner: TComponent); override;
  procedure Reset;
  procedure Start;
  procedure Stop;
  procedure Display;
  procedure ResetStart;
  procedure StopDisplay;
  function AutoScaleStr(V: int64): string;
 published
  property CurrentCount: int64 read GetCurrentCount;
  property Resolution: TResolution read FResolution write FResolution default rsMilliSeconds;
  property Digits: integer read FDigits write SetDigits default 3;
  property Time: double read ReadElaspedTime write SetElaspedTime;
 end;

procedure Register;

implementation


procedure Register;
begin
RegisterComponents('Fountain', [TSubTimer]);
end;



constructor TSubTimer.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
FResolution:=rsMilliSeconds;
Running:=False;
FDigits:=3;
end;



procedure TSubTimer.Reset;
begin
StartCount:=0;
TotCount:=0;
Running:=False;
end;



procedure TSubTimer.Start;
begin
{Don't restart if already running - required to handle timing in recursive routines}
if not Running then StartCount:=GetHiResTick64;
Running:=True;
end;


function TSubTimer.GetCurrentCount: int64;
begin
{Allows the time to be displayed while running}
if Running then Result:=TotCount+(GetHiResTick64-StartCount)
else Result:=TotCount;
end;


procedure TSubTimer.Stop;
begin
TotCount:=GetCurrentCount;
Running:=False;
end;




function TSubTimer.ReadElaspedTime: double;
{Read elapsed time in seconds}
var Count: int64;
begin
Count:=GetCurrentCount;
Result:=Count/1000000;
end;



procedure TSubTimer.SetElaspedTime(T: double);
var Count: int64;
begin
Count:=Trunc(T * 1000000);
if Running then
	begin
	StartCount:=GetHiResTick64-Count;
	TotCount:=0;
	end
else TotCount:=Count;
end;



function TSubTimer.GetMinutesStr(V: int64): string;
var T: TTime;
var Hour, Min, Sec, MSec: Word;
var FS: double;
var S: string;
begin
T:=V/(1000000);
T:=T/(24 * 60 * 60);
DecodeTime(T,Hour,Min,Sec,MSec);
FS:=Sec+(MSec/1000);
S:=FloatToStrF(FS,ffFixed,18,FDigits);
if FS<10 then S:='0'+S;
Result:=PadInteger(Min,2)+':'+S;
end;





function TSubTimer.GetHoursStr(V: int64): string;
var T: TTime;
var Hour, Min, Sec, MSec: Word;
begin
T:=V/(1000000);
T:=T/(24 * 60 * 60);
DecodeTime(T,Hour,Min,Sec,MSec);
Result:=IntToStr(Hour)+':'+PadInteger(Min,2)+':'+PadInteger(Sec,2);
end;



function TSubTimer.AutoScaleStr(V: int64): string;
var Sec: double;
begin
Sec:=V/1000000;
if Sec<1 then Result:=FloatToStrF(V/1000,ffFixed,18,FDigits)+' ms.'
else if Sec>3600 then Result:=GetHoursStr(V)+' hrs'
else if Sec>60 then Result:=GetMinutesStr(V)+' min'
else Result:=FloatToStrF(V/1000000,ffFixed,18,FDigits)+' Sec.';
end;



procedure TSubTimer.Display;
var Count: int64;
begin
Count:=GetCurrentCount;
case FResolution of
 rsSeconds: Caption:=FloatToStrF(Count/1000000,ffFixed,18,FDigits)+' Sec.';
 rsMilliseconds: Caption:=FloatToStrF(Count/1000,ffFixed,18,FDigits)+' ms.';
 rsMinutes: Caption:=GetMinutesStr(Count)+' min';
 rsAutoScale: Caption:=AutoScaleStr(Count);
 end;
end;



procedure TSubTimer.SetDigits(const Value: integer);
begin
if FDigits=Value then exit;
if Value<0 then exit;
FDigits:= Value;
end;



procedure TSubTimer.ResetStart;
begin
Reset;
Start;
end;



procedure TSubTimer.StopDisplay;
begin
Stop;
Display;
end;



end.
