unit MagIGRFData;

interface

{IGRF Model information. Available from: http://www.ngdc.noaa.gov/IAGA/vmod/igrf.html}
{You can 1) use the IGRFXX.COF available with the GEOMAG program and manually edit to the right format}
{Or 2) download the spreadsheet and use my "IGRFConverter" program to put it into the right format}

const MaxCoof = 104;	{Maximum cooeficients per Epoch}
const MaxModels =25;	{Maximum Epoch models}

{Coefficient information structure}

type
 TIGRFCoef = record
 M,N: integer;
 EpochG,EpochH: double;		{Epoch model coeficients}
 ChangeG,ChangeH: double;	{Change model coeficients}
 end;


{Epoch header structure}

type
 TIGRFEpoc = record
 ModelName: string;
 Epoch,Max1,Max2,Max3: integer;
 YearMin,YearMax,AltMin,AltMax: double;
 Coeff: array [0..MaxCoof-1] of TIGRFCoef;
 end;


{Table of IGRF values. Current Epochs for 1900 to 2015}
{With a secular variation model for 20015 to 2020}

const IGRFData : array [0..MaxModels-1] of TIGRFEpoc  =(

(ModelName:'IGRF00';Epoch:1900;Max1:10;Max2:0;Max3:0;
YearMin:1900;YearMax:1905;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-31543.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2298.00; EpochH:5922.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-677.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2905.00; EpochH:-1061.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:924.00; EpochH:1121.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1022.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1469.00; EpochH:-330.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1256.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:572.00; EpochH:523.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:876.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:628.00; EpochH:195.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:660.00; EpochH:-69.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-361.00; EpochH:-210.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:134.00; EpochH:-75.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-184.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:328.00; EpochH:-210.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:264.00; EpochH:53.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:5.00; EpochH:-33.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-86.00; EpochH:-124.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-16.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:63.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:61.00; EpochH:-9.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-11.00; EpochH:83.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-217.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-58.00; EpochH:-35.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:59.00; EpochH:36.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-90.00; EpochH:-69.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:70.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-55.00; EpochH:-45.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:0.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:34.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-41.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-21.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:18.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:8.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:1.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:2.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-9.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:5.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF05';Epoch:1905;Max1:10;Max2:0;Max3:0;
YearMin:1905;YearMax:1910;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-31464.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2298.00; EpochH:5909.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-728.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2928.00; EpochH:-1086.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1041.00; EpochH:1065.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1037.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1494.00; EpochH:-357.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1239.00; EpochH:34.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:635.00; EpochH:480.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:880.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:643.00; EpochH:203.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:653.00; EpochH:-77.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-380.00; EpochH:-201.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:146.00; EpochH:-65.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-192.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:328.00; EpochH:-193.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:259.00; EpochH:56.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-1.00; EpochH:-32.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-93.00; EpochH:-125.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-26.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:62.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:60.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-11.00; EpochH:86.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-221.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-57.00; EpochH:-32.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:57.00; EpochH:32.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-92.00; EpochH:-67.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:70.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-46.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:0.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:33.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-41.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-20.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:18.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:8.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:1.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:2.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-8.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:5.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF10';Epoch:1910;Max1:10;Max2:0;Max3:0;
YearMin:1910;YearMax:1915;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-31354.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2297.00; EpochH:5898.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-769.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2948.00; EpochH:-1128.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1176.00; EpochH:1000.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1058.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1524.00; EpochH:-389.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1223.00; EpochH:62.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:705.00; EpochH:425.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:884.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:660.00; EpochH:211.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:644.00; EpochH:-90.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-400.00; EpochH:-189.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:160.00; EpochH:-55.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-201.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:327.00; EpochH:-172.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:253.00; EpochH:57.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-9.00; EpochH:-33.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-102.00; EpochH:-126.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-38.00; EpochH:21.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:62.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:58.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-11.00; EpochH:89.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-224.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-54.00; EpochH:-29.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:54.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-95.00; EpochH:-65.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:71.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-47.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:1.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:32.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-40.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-19.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:18.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:8.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:1.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:2.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-8.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:5.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF15';Epoch:1915;Max1:10;Max2:0;Max3:0;
YearMin:1915;YearMax:1920;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-31212.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2306.00; EpochH:5875.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-802.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2956.00; EpochH:-1191.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1309.00; EpochH:917.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1084.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1559.00; EpochH:-421.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1212.00; EpochH:84.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:778.00; EpochH:360.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:887.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:678.00; EpochH:218.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:631.00; EpochH:-109.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-416.00; EpochH:-173.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:178.00; EpochH:-51.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-211.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:327.00; EpochH:-148.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:245.00; EpochH:58.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-16.00; EpochH:-34.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-111.00; EpochH:-126.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-51.00; EpochH:32.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:61.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:57.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-10.00; EpochH:93.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-228.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-51.00; EpochH:-26.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:49.00; EpochH:23.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-98.00; EpochH:-62.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:72.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-48.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:31.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-38.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-18.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:19.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:8.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:2.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:3.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-8.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:6.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:1.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF20';Epoch:1920;Max1:10;Max2:0;Max3:0;
YearMin:1920;YearMax:1925;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-31060.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2317.00; EpochH:5845.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-839.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2959.00; EpochH:-1259.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1407.00; EpochH:823.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1111.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1600.00; EpochH:-445.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1205.00; EpochH:103.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:839.00; EpochH:293.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:889.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:695.00; EpochH:220.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:616.00; EpochH:-134.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-424.00; EpochH:-153.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:199.00; EpochH:-57.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-221.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:326.00; EpochH:-122.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:236.00; EpochH:58.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-23.00; EpochH:-38.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-119.00; EpochH:-125.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-62.00; EpochH:43.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:61.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:55.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-10.00; EpochH:96.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-233.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-46.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:44.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-101.00; EpochH:-57.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:73.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-49.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:29.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-37.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-16.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:19.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-3.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:2.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:4.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-7.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:6.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:1.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF25';Epoch:1925;Max1:10;Max2:0;Max3:0;
YearMin:1925;YearMax:1930;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30926.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2318.00; EpochH:5817.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-893.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2969.00; EpochH:-1334.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1471.00; EpochH:728.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1140.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1645.00; EpochH:-462.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1202.00; EpochH:119.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:881.00; EpochH:229.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:891.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:711.00; EpochH:216.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:601.00; EpochH:-163.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-426.00; EpochH:-130.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:217.00; EpochH:-70.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-230.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:326.00; EpochH:-96.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:226.00; EpochH:58.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-28.00; EpochH:-44.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-125.00; EpochH:-122.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-69.00; EpochH:51.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:61.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:54.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-9.00; EpochH:99.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-238.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-40.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:39.00; EpochH:13.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-103.00; EpochH:-52.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:73.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-50.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:3.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:27.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-35.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-14.00; EpochH:29.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:19.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-3.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:2.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:4.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-7.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:7.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:1.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF30';Epoch:1930;Max1:10;Max2:0;Max3:0;
YearMin:1930;YearMax:1935;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30805.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2316.00; EpochH:5808.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-951.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2980.00; EpochH:-1424.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1517.00; EpochH:644.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1172.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1692.00; EpochH:-480.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1205.00; EpochH:133.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:907.00; EpochH:166.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:896.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:727.00; EpochH:205.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:584.00; EpochH:-195.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-422.00; EpochH:-109.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:234.00; EpochH:-90.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-237.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:327.00; EpochH:-72.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:218.00; EpochH:60.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-32.00; EpochH:-53.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-131.00; EpochH:-118.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-74.00; EpochH:58.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:60.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:53.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-9.00; EpochH:102.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-242.00; EpochH:19.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-32.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:32.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-104.00; EpochH:-46.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:74.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-54.00; EpochH:-51.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:4.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:25.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-34.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-12.00; EpochH:29.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:18.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-3.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:2.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:5.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-6.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:8.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:12.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:3.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-2.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:1.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF35';Epoch:1935;Max1:10;Max2:0;Max3:0;
YearMin:1935;YearMax:1940;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30715.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2306.00; EpochH:5812.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1018.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2984.00; EpochH:-1520.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1550.00; EpochH:586.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1206.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1740.00; EpochH:-494.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1215.00; EpochH:146.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:918.00; EpochH:101.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:903.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:744.00; EpochH:188.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:565.00; EpochH:-226.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-415.00; EpochH:-90.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:249.00; EpochH:-114.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-241.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:329.00; EpochH:-51.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:211.00; EpochH:64.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-33.00; EpochH:-64.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-136.00; EpochH:-115.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-76.00; EpochH:64.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:59.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:53.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-8.00; EpochH:104.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-246.00; EpochH:25.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-25.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:25.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-106.00; EpochH:-40.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:74.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-53.00; EpochH:-52.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:4.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:23.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-33.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-11.00; EpochH:29.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:18.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:6.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-3.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:1.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:6.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-6.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:8.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:7.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:11.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:3.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-2.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF40';Epoch:1940;Max1:10;Max2:0;Max3:0;
YearMin:1940;YearMax:1945;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30654.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2292.00; EpochH:5821.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1106.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2981.00; EpochH:-1614.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1566.00; EpochH:528.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1240.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1790.00; EpochH:-499.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1232.00; EpochH:163.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:916.00; EpochH:43.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:914.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:762.00; EpochH:169.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:550.00; EpochH:-252.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-405.00; EpochH:-72.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:265.00; EpochH:-141.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-241.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:334.00; EpochH:-33.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:208.00; EpochH:71.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-33.00; EpochH:-75.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-141.00; EpochH:-113.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-76.00; EpochH:69.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:57.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:54.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:-7.00; EpochH:105.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-249.00; EpochH:33.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-18.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:18.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-107.00; EpochH:-33.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:74.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-53.00; EpochH:-52.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:4.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:20.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-31.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-9.00; EpochH:29.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:17.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:5.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-3.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-10.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:1.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:6.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-5.00; EpochH:19.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:9.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:7.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:11.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:1.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:3.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-2.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF45';Epoch:1945;Max1:10;Max2:0;Max3:0;
YearMin:1945;YearMax:1950;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30594.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2285.00; EpochH:5810.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1244.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2990.00; EpochH:-1702.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1578.00; EpochH:477.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1282.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1834.00; EpochH:-499.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1255.00; EpochH:186.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:913.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:944.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:776.00; EpochH:144.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:544.00; EpochH:-276.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-421.00; EpochH:-55.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:304.00; EpochH:-178.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-253.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:346.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:194.00; EpochH:95.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-20.00; EpochH:-67.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-142.00; EpochH:-119.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-82.00; EpochH:82.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:59.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:57.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:6.00; EpochH:100.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-246.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-25.00; EpochH:-9.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:21.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-104.00; EpochH:-39.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:70.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-40.00; EpochH:-45.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:0.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:0.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-29.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-10.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:15.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:29.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:13.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-8.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-5.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:9.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:7.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-10.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:7.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:2.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:-21.00; EpochH:-27.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-11.00; EpochH:29.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:3.00; EpochH:-9.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:16.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-3.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:-4.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-3.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-4.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:1.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:2.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-5.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:-1.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:8.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:-1.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:-3.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-2.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF50';Epoch:1950;Max1:10;Max2:0;Max3:0;
YearMin:1950;YearMax:1955;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30554.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2250.00; EpochH:5815.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1341.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2998.00; EpochH:-1810.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1576.00; EpochH:381.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1297.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1889.00; EpochH:-476.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1274.00; EpochH:206.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:896.00; EpochH:-46.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:954.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:792.00; EpochH:136.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:528.00; EpochH:-278.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-408.00; EpochH:-37.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:303.00; EpochH:-210.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-240.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:349.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:211.00; EpochH:103.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-20.00; EpochH:-87.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-147.00; EpochH:-122.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-76.00; EpochH:80.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:54.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:57.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:4.00; EpochH:99.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-247.00; EpochH:33.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-16.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:12.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-105.00; EpochH:-30.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:65.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-55.00; EpochH:-35.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:1.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-40.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-7.00; EpochH:36.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:5.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:19.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:22.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:15.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-1.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:11.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:15.00; EpochH:-8.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-13.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:5.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-1.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:-7.00; EpochH:-24.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:-1.00; EpochH:19.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-25.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:10.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-5.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:3.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:8.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:4.00; EpochH:13.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:-1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:13.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:4.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:12.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:3.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:10.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:3.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF55';Epoch:1955;Max1:10;Max2:0;Max3:0;
YearMin:1955;YearMax:1960;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30500.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2215.00; EpochH:5820.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1440.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3003.00; EpochH:-1898.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1581.00; EpochH:291.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1302.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1944.00; EpochH:-462.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1288.00; EpochH:216.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:882.00; EpochH:-83.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:958.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:796.00; EpochH:133.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:510.00; EpochH:-274.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-397.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:290.00; EpochH:-230.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-229.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:360.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:230.00; EpochH:110.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-23.00; EpochH:-98.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-152.00; EpochH:-121.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-69.00; EpochH:78.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:47.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:57.00; EpochH:-9.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:3.00; EpochH:96.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-247.00; EpochH:48.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-8.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:7.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-107.00; EpochH:-24.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:65.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-56.00; EpochH:-50.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-24.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:10.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-32.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-11.00; EpochH:28.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:9.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:18.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:11.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:9.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-6.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-14.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:6.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:10.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-7.00; EpochH:23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:9.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:-4.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-5.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:4.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:1.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:2.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:2.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:5.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-5.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:-1.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:2.00; EpochH:-8.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-3.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:7.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:-2.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:6.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:-2.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF60';Epoch:1960;Max1:10;Max2:0;Max3:0;
YearMin:1960;YearMax:1965;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30421.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2169.00; EpochH:5791.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1555.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3002.00; EpochH:-1967.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1590.00; EpochH:206.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1302.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-1992.00; EpochH:-414.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1289.00; EpochH:224.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:878.00; EpochH:-130.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:957.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:800.00; EpochH:135.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:504.00; EpochH:-278.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-394.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:269.00; EpochH:-255.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-222.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:362.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:242.00; EpochH:125.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-26.00; EpochH:-117.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-156.00; EpochH:-114.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-63.00; EpochH:81.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:46.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:58.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:1.00; EpochH:99.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-237.00; EpochH:60.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-1.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:-2.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-113.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:67.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-56.00; EpochH:-55.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:5.00; EpochH:-28.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:15.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-32.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-7.00; EpochH:23.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:17.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:8.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:15.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-14.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-11.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:2.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:10.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-5.00; EpochH:23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:10.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:8.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:6.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:0.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-9.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:1.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:4.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:-2.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:1.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-3.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:4.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-1.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:4.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:6.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:-1.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF65';Epoch:1965;Max1:10;Max2:0;Max3:0;
YearMin:1965;YearMax:1970;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30334.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2119.00; EpochH:5776.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1662.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:2997.00; EpochH:-2016.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1594.00; EpochH:114.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1297.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2038.00; EpochH:-404.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1292.00; EpochH:240.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:856.00; EpochH:-165.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:957.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:804.00; EpochH:148.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:479.00; EpochH:-269.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-390.00; EpochH:13.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:252.00; EpochH:-269.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-219.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:358.00; EpochH:19.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:254.00; EpochH:128.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-31.00; EpochH:-126.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-157.00; EpochH:-97.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-62.00; EpochH:81.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:45.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:61.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:8.00; EpochH:100.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-228.00; EpochH:68.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:4.00; EpochH:-32.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:1.00; EpochH:-8.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-111.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:75.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-57.00; EpochH:-61.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:4.00; EpochH:-27.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:13.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-26.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-6.00; EpochH:26.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:13.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:1.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:13.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:5.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-4.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-14.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:0.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:8.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:-1.00; EpochH:24.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:11.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:4.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:2.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-13.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:10.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-1.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:5.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-3.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:4.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:0.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF70';Epoch:1970;Max1:10;Max2:0;Max3:0;
YearMin:1970;YearMax:1975;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30220.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2068.00; EpochH:5737.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1781.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3000.00; EpochH:-2047.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1611.00; EpochH:25.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1287.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2091.00; EpochH:-366.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1278.00; EpochH:251.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:838.00; EpochH:-196.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:952.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:800.00; EpochH:167.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:461.00; EpochH:-266.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-395.00; EpochH:26.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:234.00; EpochH:-279.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-216.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:359.00; EpochH:26.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:262.00; EpochH:139.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-42.00; EpochH:-139.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-160.00; EpochH:-91.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-56.00; EpochH:83.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:43.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:64.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:15.00; EpochH:100.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-212.00; EpochH:72.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:2.00; EpochH:-37.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:3.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-112.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:72.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-57.00; EpochH:-70.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:1.00; EpochH:-27.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:14.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-22.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:-2.00; EpochH:23.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:13.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:-2.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:14.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-2.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-13.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-3.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:5.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:0.00; EpochH:21.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:11.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:3.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:8.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:2.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:10.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-1.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:0.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:3.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-1.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-3.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-1.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:0.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-1.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF75';Epoch:1975;Max1:10;Max2:0;Max3:0;
YearMin:1975;YearMax:1980;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-30100.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-2013.00; EpochH:5675.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1902.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3010.00; EpochH:-2067.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1632.00; EpochH:-68.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1276.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2144.00; EpochH:-333.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1260.00; EpochH:262.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:830.00; EpochH:-223.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:946.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:791.00; EpochH:191.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:438.00; EpochH:-265.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-405.00; EpochH:39.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:216.00; EpochH:-288.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-218.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:356.00; EpochH:31.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:264.00; EpochH:148.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-59.00; EpochH:-152.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-159.00; EpochH:-83.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-49.00; EpochH:88.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:45.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:66.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:28.00; EpochH:99.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-198.00; EpochH:75.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:1.00; EpochH:-41.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:6.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-111.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:71.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-56.00; EpochH:-77.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:1.00; EpochH:-26.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:16.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-14.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:0.00; EpochH:22.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:12.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:-5.00; EpochH:-12.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:14.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-1.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-12.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-8.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:4.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:0.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:10.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:1.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:7.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:2.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:10.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-1.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:4.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-3.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:5.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:4.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:0.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-1.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF80';Epoch:1980;Max1:10;Max2:0;Max3:0;
YearMin:1980;YearMax:1985;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29992.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1956.00; EpochH:5604.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-1997.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3027.00; EpochH:-2129.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1663.00; EpochH:-200.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1281.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2180.00; EpochH:-336.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1251.00; EpochH:271.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:833.00; EpochH:-252.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:938.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:782.00; EpochH:212.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:398.00; EpochH:-257.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-419.00; EpochH:53.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:199.00; EpochH:-297.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-218.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:357.00; EpochH:46.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:261.00; EpochH:150.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-74.00; EpochH:-151.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-162.00; EpochH:-78.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-48.00; EpochH:92.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:48.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:66.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:42.00; EpochH:93.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-192.00; EpochH:71.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:4.00; EpochH:-43.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:14.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-108.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:72.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-59.00; EpochH:-82.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-27.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:21.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-12.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:1.00; EpochH:18.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:11.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:-2.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:18.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:7.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:0.00; EpochH:-18.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-11.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-7.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:4.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:3.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:6.00; EpochH:-13.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-1.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:16.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:9.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-3.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:7.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:2.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:5.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF85';Epoch:1985;Max1:10;Max2:0;Max3:0;
YearMin:1985;YearMax:1990;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29873.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1905.00; EpochH:5500.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2072.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3044.00; EpochH:-2197.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1687.00; EpochH:-306.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1296.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2208.00; EpochH:-310.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1247.00; EpochH:284.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:829.00; EpochH:-297.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:936.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:780.00; EpochH:232.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:361.00; EpochH:-249.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-424.00; EpochH:69.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:170.00; EpochH:-297.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-214.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:355.00; EpochH:47.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:253.00; EpochH:150.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-93.00; EpochH:-154.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-164.00; EpochH:-75.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-46.00; EpochH:95.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:53.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:65.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:51.00; EpochH:88.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-185.00; EpochH:69.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:4.00; EpochH:-48.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:16.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-102.00; EpochH:21.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:74.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-62.00; EpochH:-83.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:3.00; EpochH:-27.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:24.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-6.00; EpochH:20.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:4.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:10.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:0.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:21.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:0.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-11.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-9.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:4.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:4.00; EpochH:14.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:4.00; EpochH:-15.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-4.00; EpochH:-11.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:10.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:9.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-3.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:7.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-5.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:5.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF90';Epoch:1990;Max1:10;Max2:0;Max3:0;
YearMin:1990;YearMax:1995;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29775.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1848.00; EpochH:5406.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2131.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3059.00; EpochH:-2279.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1686.00; EpochH:-373.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1314.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2239.00; EpochH:-284.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1248.00; EpochH:293.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:802.00; EpochH:-352.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:939.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:780.00; EpochH:247.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:325.00; EpochH:-240.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-423.00; EpochH:84.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:141.00; EpochH:-299.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-214.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:353.00; EpochH:46.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:245.00; EpochH:154.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-109.00; EpochH:-153.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-165.00; EpochH:-69.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-36.00; EpochH:97.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:61.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:65.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:59.00; EpochH:82.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-178.00; EpochH:69.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:3.00; EpochH:-52.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:18.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-96.00; EpochH:24.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:77.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-64.00; EpochH:-80.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:2.00; EpochH:-26.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:26.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:-1.00; EpochH:21.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:5.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:9.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:0.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:23.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:5.00; EpochH:10.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-1.00; EpochH:-19.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-10.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-12.00; EpochH:-22.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:3.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:4.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:2.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-6.00; EpochH:-10.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:1.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-12.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:9.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-4.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-2.00; EpochH:9.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:7.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:1.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-6.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-4.00; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-5.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-2.00; EpochH:6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:4.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:3.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:3.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF95';Epoch:1995;Max1:10;Max2:0;Max3:0;
YearMin:1995;YearMax:2000;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29692.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1784.00; EpochH:5306.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2200.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3070.00; EpochH:-2366.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1681.00; EpochH:-413.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1335.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2267.00; EpochH:-262.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1249.00; EpochH:302.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:759.00; EpochH:-427.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:940.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:780.00; EpochH:262.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:290.00; EpochH:-236.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-418.00; EpochH:97.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:122.00; EpochH:-306.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-214.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:352.00; EpochH:46.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:235.00; EpochH:165.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-118.00; EpochH:-143.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-166.00; EpochH:-55.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-17.00; EpochH:107.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:68.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:67.00; EpochH:-17.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:68.00; EpochH:72.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-170.00; EpochH:67.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-1.00; EpochH:-58.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:19.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-93.00; EpochH:36.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:77.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-72.00; EpochH:-69.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:1.00; EpochH:-25.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:28.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:5.00; EpochH:24.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:4.00; EpochH:17.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:8.00; EpochH:-24.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:-2.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:25.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-6.00; EpochH:-21.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-9.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-14.00; EpochH:-23.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:9.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:6.00; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:-5.00; EpochH:-16.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-7.00; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:4.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.00; EpochH:-20.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:3.00; EpochH:15.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-10.00; EpochH:12.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:8.00; EpochH:-6.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-8.00; EpochH:-8.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.00; EpochH:8.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:10.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-2.00; EpochH:-8.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-8.00; EpochH:3.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-4.00; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-1.00; EpochH:5.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:4.00; EpochH:-5.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:2.00; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:2.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:5.00; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:1.00; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:0.00; EpochH:-7.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF2000';Epoch:2000;Max1:13;Max2:0;Max3:0;
YearMin:2000;YearMax:2005;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29619.40; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1728.20; EpochH:5186.10; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2267.70; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3068.40; EpochH:-2481.60; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1670.90; EpochH:-458.00; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1339.60; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2288.00; EpochH:-227.60; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1252.10; EpochH:293.40; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:714.50; EpochH:-491.10; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:932.30; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:786.80; EpochH:272.60; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:250.00; EpochH:-231.90; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-403.00; EpochH:119.80; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:111.30; EpochH:-303.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-218.80; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:351.40; EpochH:43.80; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:222.30; EpochH:171.90; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-130.40; EpochH:-133.10; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-168.60; EpochH:-39.30; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-12.90; EpochH:106.30; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:72.30; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:68.20; EpochH:-17.40; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:74.20; EpochH:63.70; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-160.90; EpochH:65.10; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-5.90; EpochH:-61.20; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:16.90; EpochH:0.70; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-90.40; EpochH:43.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:79.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-74.00; EpochH:-64.60; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:0.00; EpochH:-24.20; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:33.30; EpochH:6.20; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:9.10; EpochH:24.00; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:6.90; EpochH:14.80; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:7.30; EpochH:-25.40; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:-1.20; EpochH:-5.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:24.40; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:6.60; EpochH:11.90; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-9.20; EpochH:-21.50; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-7.90; EpochH:8.50; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-16.60; EpochH:-21.50; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:9.10; EpochH:15.50; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:7.00; EpochH:8.90; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:-7.90; EpochH:-14.90; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-7.00; EpochH:-2.10; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.40; EpochH:-19.70; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:3.00; EpochH:13.40; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-8.40; EpochH:12.50; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:6.30; EpochH:-6.20; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-8.90; EpochH:-8.40; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.50; EpochH:8.40; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:9.30; EpochH:3.80; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-4.30; EpochH:-8.20; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-8.20; EpochH:4.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-2.60; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.00; EpochH:1.70; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:1.70; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-3.10; EpochH:4.00; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-0.50; EpochH:4.90; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:3.70; EpochH:-5.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:1.00; EpochH:-1.20; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:2.00; EpochH:-2.90; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:4.20; EpochH:0.20; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:0.30; EpochH:-2.20; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-1.10; EpochH:-7.40; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:2.70; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:-1.70; EpochH:0.10; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:-1.90; EpochH:1.30; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:1.50; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:-0.10; EpochH:-2.60; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.10; EpochH:0.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:-0.70; EpochH:-0.70; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.70; EpochH:-2.80; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:1.70; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.10; EpochH:-1.20; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:1.20; EpochH:-1.90; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:4.00; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:-2.20; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:-0.30; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.20; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.90; EpochH:2.50; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:-0.20; EpochH:-2.60; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.90; EpochH:0.70; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:-0.50; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.30; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:-0.30; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:-0.40; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:-0.10; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:-0.20; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:-0.40; EpochH:0.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:-0.20; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:-0.90; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.30; EpochH:0.20; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.10; EpochH:1.80; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:-0.40; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:1.30; EpochH:-1.00; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:-0.40; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.70; EpochH:0.70; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:-0.40; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.30; EpochH:0.60; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:-0.10; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.40; EpochH:-0.20; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:0.00; EpochH:-0.50; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:0.10; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF2005';Epoch:2005;Max1:13;Max2:0;Max3:0;
YearMin:2005;YearMax:2010;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29554.63; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1669.05; EpochH:5077.99; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2337.24; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3047.69; EpochH:-2594.50; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1657.76; EpochH:-515.43; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1336.30; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2305.83; EpochH:-198.86; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1246.39; EpochH:269.72; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:672.51; EpochH:-524.72; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:920.55; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:797.96; EpochH:282.07; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:210.65; EpochH:-225.23; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-379.86; EpochH:145.15; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:100.00; EpochH:-305.36; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-227.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:354.41; EpochH:42.72; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:208.95; EpochH:180.25; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-136.54; EpochH:-123.45; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-168.05; EpochH:-19.57; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-13.55; EpochH:103.85; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:73.60; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:69.56; EpochH:-20.33; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:76.74; EpochH:54.75; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-151.34; EpochH:63.63; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-14.58; EpochH:-63.53; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:14.58; EpochH:0.24; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-86.36; EpochH:50.94; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:79.88; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-74.46; EpochH:-61.14; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:-1.65; EpochH:-22.57; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:38.73; EpochH:6.82; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:12.30; EpochH:25.35; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:9.37; EpochH:10.93; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:5.42; EpochH:-26.32; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:1.94; EpochH:-4.64; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:24.80; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:7.62; EpochH:11.20; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-11.73; EpochH:-20.88; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-6.88; EpochH:9.83; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-18.11; EpochH:-19.71; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:10.17; EpochH:16.22; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:9.36; EpochH:7.61; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:-11.25; EpochH:-12.76; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-4.87; EpochH:-0.06; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.58; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.76; EpochH:-20.11; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:3.58; EpochH:12.69; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-6.94; EpochH:12.67; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:5.01; EpochH:-6.72; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-10.76; EpochH:-8.16; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-1.25; EpochH:8.10; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:8.76; EpochH:2.92; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-6.66; EpochH:-7.73; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-9.22; EpochH:6.01; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-2.17; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.12; EpochH:2.19; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:1.42; EpochH:0.10; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-2.35; EpochH:4.46; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-0.15; EpochH:4.76; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:3.06; EpochH:-6.58; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:0.29; EpochH:-1.01; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:2.06; EpochH:-3.47; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:3.77; EpochH:-0.86; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:-0.21; EpochH:-2.31; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-2.09; EpochH:-7.93; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:2.95; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:-1.60; EpochH:0.26; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:-1.88; EpochH:1.44; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:1.44; EpochH:-0.77; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:-0.31; EpochH:-2.27; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.29; EpochH:0.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:-0.79; EpochH:-0.58; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.53; EpochH:-2.69; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:1.80; EpochH:-1.08; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.16; EpochH:-1.58; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.96; EpochH:-1.90; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:3.99; EpochH:-1.39; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:-2.15; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:-0.29; EpochH:-0.55; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.21; EpochH:0.23; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:0.89; EpochH:2.38; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:-0.38; EpochH:-2.63; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.96; EpochH:0.61; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:-0.30; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.46; EpochH:0.01; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:-0.35; EpochH:0.02; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:-0.36; EpochH:0.28; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.08; EpochH:-0.87; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:-0.49; EpochH:-0.34; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:-0.08; EpochH:0.88; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:-0.16; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:-0.88; EpochH:-0.76; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.30; EpochH:0.33; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.28; EpochH:1.72; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:-0.43; EpochH:-0.54; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:1.18; EpochH:-1.07; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:-0.37; EpochH:-0.04; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.75; EpochH:0.63; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:-0.26; EpochH:0.21; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.35; EpochH:0.53; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:-0.05; EpochH:0.38; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.41; EpochH:-0.22; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:-0.10; EpochH:-0.57; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:-0.18; EpochH:-0.82; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'DGRF2010';Epoch:2010;Max1:13;Max2:0;Max3:0;
YearMin:2010;YearMax:2015;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29496.57; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:1; EpochG:-1586.42; EpochH:4944.26; ChangeG:0.00; ChangeH:0.00),
(M:0; N:2; EpochG:-2396.06; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:2; EpochG:3026.34; EpochH:-2708.54; ChangeG:0.00; ChangeH:0.00),
(M:2; N:2; EpochG:1668.17; EpochH:-575.73; ChangeG:0.00; ChangeH:0.00),
(M:0; N:3; EpochG:1339.85; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:3; EpochG:-2326.54; EpochH:-160.40; ChangeG:0.00; ChangeH:0.00),
(M:2; N:3; EpochG:1232.10; EpochH:251.75; ChangeG:0.00; ChangeH:0.00),
(M:3; N:3; EpochG:633.73; EpochH:-537.03; ChangeG:0.00; ChangeH:0.00),
(M:0; N:4; EpochG:912.66; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:4; EpochG:808.97; EpochH:286.48; ChangeG:0.00; ChangeH:0.00),
(M:2; N:4; EpochG:166.58; EpochH:-211.03; ChangeG:0.00; ChangeH:0.00),
(M:3; N:4; EpochG:-356.83; EpochH:164.46; ChangeG:0.00; ChangeH:0.00),
(M:4; N:4; EpochG:89.40; EpochH:-309.72; ChangeG:0.00; ChangeH:0.00),
(M:0; N:5; EpochG:-230.87; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:5; EpochG:357.29; EpochH:44.58; ChangeG:0.00; ChangeH:0.00),
(M:2; N:5; EpochG:200.26; EpochH:189.01; ChangeG:0.00; ChangeH:0.00),
(M:3; N:5; EpochG:-141.05; EpochH:-118.06; ChangeG:0.00; ChangeH:0.00),
(M:4; N:5; EpochG:-163.17; EpochH:-0.01; ChangeG:0.00; ChangeH:0.00),
(M:5; N:5; EpochG:-8.03; EpochH:101.04; ChangeG:0.00; ChangeH:0.00),
(M:0; N:6; EpochG:72.78; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:6; EpochG:68.69; EpochH:-20.90; ChangeG:0.00; ChangeH:0.00),
(M:2; N:6; EpochG:75.92; EpochH:44.18; ChangeG:0.00; ChangeH:0.00),
(M:3; N:6; EpochG:-141.40; EpochH:61.54; ChangeG:0.00; ChangeH:0.00),
(M:4; N:6; EpochG:-22.83; EpochH:-66.26; ChangeG:0.00; ChangeH:0.00),
(M:5; N:6; EpochG:13.10; EpochH:3.02; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-78.09; EpochH:55.40; ChangeG:0.00; ChangeH:0.00),
(M:0; N:7; EpochG:80.44; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:7; EpochG:-75.00; EpochH:-57.80; ChangeG:0.00; ChangeH:0.00),
(M:2; N:7; EpochG:-4.55; EpochH:-21.20; ChangeG:0.00; ChangeH:0.00),
(M:3; N:7; EpochG:45.24; EpochH:6.54; ChangeG:0.00; ChangeH:0.00),
(M:4; N:7; EpochG:14.00; EpochH:24.96; ChangeG:0.00; ChangeH:0.00),
(M:5; N:7; EpochG:10.46; EpochH:7.03; ChangeG:0.00; ChangeH:0.00),
(M:6; N:7; EpochG:1.64; EpochH:-27.61; ChangeG:0.00; ChangeH:0.00),
(M:7; N:7; EpochG:4.92; EpochH:-3.28; ChangeG:0.00; ChangeH:0.00),
(M:0; N:8; EpochG:24.41; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:8.21; EpochH:10.84; ChangeG:0.00; ChangeH:0.00),
(M:2; N:8; EpochG:-14.50; EpochH:-20.03; ChangeG:0.00; ChangeH:0.00),
(M:3; N:8; EpochG:-5.59; EpochH:11.83; ChangeG:0.00; ChangeH:0.00),
(M:4; N:8; EpochG:-19.34; EpochH:-17.41; ChangeG:0.00; ChangeH:0.00),
(M:5; N:8; EpochG:11.61; EpochH:16.71; ChangeG:0.00; ChangeH:0.00),
(M:6; N:8; EpochG:10.85; EpochH:6.96; ChangeG:0.00; ChangeH:0.00),
(M:7; N:8; EpochG:-14.05; EpochH:-10.74; ChangeG:0.00; ChangeH:0.00),
(M:8; N:8; EpochG:-3.54; EpochH:1.64; ChangeG:0.00; ChangeH:0.00),
(M:0; N:9; EpochG:5.50; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:9.45; EpochH:-20.54; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:3.45; EpochH:11.51; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-5.27; EpochH:12.75; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:3.13; EpochH:-7.14; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-12.38; EpochH:-7.42; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-0.76; EpochH:7.97; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:8.43; EpochH:2.14; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-8.42; EpochH:-6.08; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-10.08; EpochH:7.01; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-1.94; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.24; EpochH:2.73; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:0.89; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:-1.07; EpochH:4.71; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-0.16; EpochH:4.44; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:2.45; EpochH:-7.22; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:-0.33; EpochH:-0.96; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:2.13; EpochH:-3.95; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:3.09; EpochH:-1.99; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:-1.03; EpochH:-1.97; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-2.80; EpochH:-8.31; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:3.05; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:-1.48; EpochH:0.13; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:-2.03; EpochH:1.67; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:1.65; EpochH:-0.66; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:-0.51; EpochH:-1.76; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.54; EpochH:0.85; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:-0.79; EpochH:-0.39; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.37; EpochH:-2.51; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:1.79; EpochH:-1.27; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:0.12; EpochH:-2.11; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.75; EpochH:-1.94; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:3.75; EpochH:-1.86; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:-2.12; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:-0.21; EpochH:-0.87; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.30; EpochH:0.27; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:1.04; EpochH:2.13; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:-0.63; EpochH:-2.49; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.95; EpochH:0.49; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:-0.11; EpochH:0.59; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.52; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:-0.39; EpochH:0.13; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:-0.37; EpochH:0.27; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.21; EpochH:-0.86; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:-0.77; EpochH:-0.23; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.04; EpochH:0.87; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:-0.09; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:-0.89; EpochH:-0.87; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.31; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.42; EpochH:1.66; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:-0.45; EpochH:-0.59; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:1.08; EpochH:-1.14; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:-0.31; EpochH:-0.07; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.78; EpochH:0.54; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:-0.18; EpochH:0.10; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.38; EpochH:0.49; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.02; EpochH:0.44; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.42; EpochH:-0.25; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:-0.26; EpochH:-0.53; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:-0.26; EpochH:-0.79; ChangeG:0.00; ChangeH:0.00 ))),

(ModelName:'IGRF2015';Epoch:2015;Max1:13;Max2:8;Max3:0;
YearMin:2015;YearMax:2020;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29442.00; EpochH:0.00; ChangeG:10.30; ChangeH:0.00),
(M:1; N:1; EpochG:-1501.00; EpochH:4797.10; ChangeG:18.10; ChangeH:-26.60),
(M:0; N:2; EpochG:-2445.10; EpochH:0.00; ChangeG:-8.70; ChangeH:0.00),
(M:1; N:2; EpochG:3012.90; EpochH:-2845.60; ChangeG:-3.30; ChangeH:-27.40),
(M:2; N:2; EpochG:1676.70; EpochH:-641.90; ChangeG:2.10; ChangeH:-14.10),
(M:0; N:3; EpochG:1350.70; EpochH:0.00; ChangeG:3.40; ChangeH:0.00),
(M:1; N:3; EpochG:-2352.30; EpochH:-115.30; ChangeG:-5.50; ChangeH:8.20),
(M:2; N:3; EpochG:1225.60; EpochH:244.90; ChangeG:-0.70; ChangeH:-0.40),
(M:3; N:3; EpochG:582.00; EpochH:-538.40; ChangeG:-10.10; ChangeH:1.80),
(M:0; N:4; EpochG:907.60; EpochH:0.00; ChangeG:-0.70; ChangeH:0.00),
(M:1; N:4; EpochG:813.70; EpochH:283.30; ChangeG:0.20; ChangeH:-1.30),
(M:2; N:4; EpochG:120.40; EpochH:-188.70; ChangeG:-9.10; ChangeH:5.30),
(M:3; N:4; EpochG:-334.90; EpochH:180.90; ChangeG:4.10; ChangeH:2.90),
(M:4; N:4; EpochG:70.40; EpochH:-329.50; ChangeG:-4.30; ChangeH:-5.20),
(M:0; N:5; EpochG:-232.60; EpochH:0.00; ChangeG:-0.20; ChangeH:0.00),
(M:1; N:5; EpochG:360.10; EpochH:47.30; ChangeG:0.50; ChangeH:0.60),
(M:2; N:5; EpochG:192.40; EpochH:197.00; ChangeG:-1.30; ChangeH:1.70),
(M:3; N:5; EpochG:-140.90; EpochH:-119.30; ChangeG:-0.10; ChangeH:-1.20),
(M:4; N:5; EpochG:-157.50; EpochH:16.00; ChangeG:1.40; ChangeH:3.40),
(M:5; N:5; EpochG:4.10; EpochH:100.20; ChangeG:3.90; ChangeH:0.00),
(M:0; N:6; EpochG:70.00; EpochH:0.00; ChangeG:-0.30; ChangeH:0.00),
(M:1; N:6; EpochG:67.70; EpochH:-20.80; ChangeG:-0.10; ChangeH:0.00),
(M:2; N:6; EpochG:72.70; EpochH:33.20; ChangeG:-0.70; ChangeH:-2.10),
(M:3; N:6; EpochG:-129.90; EpochH:58.90; ChangeG:2.10; ChangeH:-0.70),
(M:4; N:6; EpochG:-28.90; EpochH:-66.70; ChangeG:-1.20; ChangeH:0.20),
(M:5; N:6; EpochG:13.20; EpochH:7.30; ChangeG:0.30; ChangeH:0.90),
(M:6; N:6; EpochG:-70.90; EpochH:62.60; ChangeG:1.60; ChangeH:1.00),
(M:0; N:7; EpochG:81.60; EpochH:0.00; ChangeG:0.30; ChangeH:0.00),
(M:1; N:7; EpochG:-76.10; EpochH:-54.10; ChangeG:-0.20; ChangeH:0.80),
(M:2; N:7; EpochG:-6.80; EpochH:-19.50; ChangeG:-0.50; ChangeH:0.40),
(M:3; N:7; EpochG:51.80; EpochH:5.70; ChangeG:1.30; ChangeH:-0.20),
(M:4; N:7; EpochG:15.00; EpochH:24.40; ChangeG:0.10; ChangeH:-0.30),
(M:5; N:7; EpochG:9.40; EpochH:3.40; ChangeG:-0.60; ChangeH:-0.60),
(M:6; N:7; EpochG:-2.80; EpochH:-27.40; ChangeG:-0.80; ChangeH:0.10),
(M:7; N:7; EpochG:6.80; EpochH:-2.20; ChangeG:0.20; ChangeH:-0.20),
(M:0; N:8; EpochG:24.20; EpochH:0.00; ChangeG:0.20; ChangeH:0.00),
(M:1; N:8; EpochG:8.80; EpochH:10.10; ChangeG:0.00; ChangeH:-0.30),
(M:2; N:8; EpochG:-16.90; EpochH:-18.30; ChangeG:-0.60; ChangeH:0.30),
(M:3; N:8; EpochG:-3.20; EpochH:13.30; ChangeG:0.50; ChangeH:0.10),
(M:4; N:8; EpochG:-20.60; EpochH:-14.60; ChangeG:-0.20; ChangeH:0.50),
(M:5; N:8; EpochG:13.40; EpochH:16.20; ChangeG:0.40; ChangeH:-0.20),
(M:6; N:8; EpochG:11.70; EpochH:5.70; ChangeG:0.10; ChangeH:-0.30),
(M:7; N:8; EpochG:-15.90; EpochH:-9.10; ChangeG:-0.40; ChangeH:0.30),
(M:8; N:8; EpochG:-2.00; EpochH:2.10; ChangeG:0.30; ChangeH:0.00),
(M:0; N:9; EpochG:5.40; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:8.80; EpochH:-21.60; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:3.10; EpochH:10.80; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-3.30; EpochH:11.80; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:0.70; EpochH:-6.80; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-13.30; EpochH:-6.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:-0.10; EpochH:7.80; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:8.70; EpochH:1.00; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-9.10; EpochH:-4.00; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-10.50; EpochH:8.40; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-1.90; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.30; EpochH:3.20; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:0.10; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:0.50; EpochH:4.60; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-0.50; EpochH:4.40; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:1.80; EpochH:-7.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:-0.70; EpochH:-0.60; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:2.10; EpochH:-4.20; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:2.40; EpochH:-2.80; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:-1.80; EpochH:-1.20; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-3.60; EpochH:-8.70; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:3.10; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:-1.50; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:-2.30; EpochH:2.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:2.00; EpochH:-0.70; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:-0.80; EpochH:-1.10; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.60; EpochH:0.80; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:-0.70; EpochH:-0.20; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:0.20; EpochH:-2.20; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:1.70; EpochH:-1.40; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:-0.20; EpochH:-2.50; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.40; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:3.50; EpochH:-2.40; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:-1.90; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:-0.20; EpochH:-1.10; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.40; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:1.20; EpochH:1.90; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:-0.80; EpochH:-2.20; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.90; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.10; EpochH:0.70; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.50; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:-0.30; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:-0.40; EpochH:0.20; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.20; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:-0.90; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:0.00; EpochH:0.70; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:-0.90; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.40; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.50; EpochH:1.60; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:-0.50; EpochH:-0.50; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:1.00; EpochH:-1.20; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:-0.20; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.80; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:-0.10; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.30; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.10; EpochH:0.50; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.50; EpochH:-0.30; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:-0.40; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:-0.30; EpochH:-0.80; ChangeG:0.00; ChangeH:0.00))),


(ModelName:'IGRF2020';Epoch:2020;Max1:13;Max2:8;Max3:0;
YearMin:2020;YearMax:2025;AltMin:-1;AltMax:600; Coeff: (
(M:0; N:1; EpochG:-29404.80; EpochH:0.00; ChangeG:5.70; ChangeH:0.00),
(M:1; N:1; EpochG:-1450.90; EpochH:4652.50; ChangeG:7.40; ChangeH:-25.90),
(M:0; N:2; EpochG:-2499.60; EpochH:0.00; ChangeG:-11.00; ChangeH:0.00),
(M:1; N:2; EpochG:2982.00; EpochH:-2991.60; ChangeG:-7.00; ChangeH:-30.20),
(M:2; N:2; EpochG:1677.00; EpochH:-734.60; ChangeG:-2.10; ChangeH:-22.40),
(M:0; N:3; EpochG:1363.20; EpochH:0.00; ChangeG:2.20; ChangeH:0.00),
(M:1; N:3; EpochG:-2381.20; EpochH:-82.10; ChangeG:-5.90; ChangeH:6.00),
(M:2; N:3; EpochG:1236.20; EpochH:241.90; ChangeG:3.10; ChangeH:-1.10),
(M:3; N:3; EpochG:525.70; EpochH:-543.40; ChangeG:-12.00; ChangeH:0.50),
(M:0; N:4; EpochG:903.00; EpochH:0.00; ChangeG:-1.20; ChangeH:0.00),
(M:1; N:4; EpochG:809.50; EpochH:281.90; ChangeG:-1.60; ChangeH:-0.10),
(M:2; N:4; EpochG:86.30; EpochH:-158.40; ChangeG:-5.90; ChangeH:6.50),
(M:3; N:4; EpochG:-309.40; EpochH:199.70; ChangeG:5.20; ChangeH:3.60),
(M:4; N:4; EpochG:48.00; EpochH:-349.70; ChangeG:-5.10; ChangeH:-5.00),
(M:0; N:5; EpochG:-234.30; EpochH:0.00; ChangeG:-0.30; ChangeH:0.00),
(M:1; N:5; EpochG:363.20; EpochH:47.70; ChangeG:0.50; ChangeH:0.00),
(M:2; N:5; EpochG:187.80; EpochH:208.30; ChangeG:-0.60; ChangeH:2.50),
(M:3; N:5; EpochG:-140.70; EpochH:-121.20; ChangeG:0.20; ChangeH:-0.60),
(M:4; N:5; EpochG:-151.20; EpochH:32.30; ChangeG:1.30; ChangeH:3.00),
(M:5; N:5; EpochG:13.50; EpochH:98.90; ChangeG:0.90; ChangeH:0.30),
(M:0; N:6; EpochG:66.00; EpochH:0.00; ChangeG:-0.50; ChangeH:0.00),
(M:1; N:6; EpochG:65.50; EpochH:-19.10; ChangeG:-0.30; ChangeH:0.00),
(M:2; N:6; EpochG:72.90; EpochH:25.10; ChangeG:0.40; ChangeH:-1.60),
(M:3; N:6; EpochG:-121.50; EpochH:52.80; ChangeG:1.30; ChangeH:-1.30),
(M:4; N:6; EpochG:-36.20; EpochH:-64.50; ChangeG:-1.40; ChangeH:0.80),
(M:5; N:6; EpochG:13.50; EpochH:8.90; ChangeG:0.00; ChangeH:0.00),
(M:6; N:6; EpochG:-64.70; EpochH:68.10; ChangeG:0.90; ChangeH:1.00),
(M:0; N:7; EpochG:80.60; EpochH:0.00; ChangeG:-0.10; ChangeH:0.00),
(M:1; N:7; EpochG:-76.70; EpochH:-51.50; ChangeG:-0.20; ChangeH:0.60),
(M:2; N:7; EpochG:-8.20; EpochH:-16.90; ChangeG:0.00; ChangeH:0.60),
(M:3; N:7; EpochG:56.50; EpochH:2.20; ChangeG:0.70; ChangeH:-0.80),
(M:4; N:7; EpochG:15.80; EpochH:23.50; ChangeG:0.10; ChangeH:-0.20),
(M:5; N:7; EpochG:6.40; EpochH:-2.20; ChangeG:-0.50; ChangeH:-1.10),
(M:6; N:7; EpochG:-7.20; EpochH:-27.20; ChangeG:-0.80; ChangeH:0.10),
(M:7; N:7; EpochG:9.80; EpochH:-1.80; ChangeG:0.80; ChangeH:0.30),
(M:0; N:8; EpochG:23.70; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:8; EpochG:9.70; EpochH:8.40; ChangeG:0.10; ChangeH:-0.20),
(M:2; N:8; EpochG:-17.60; EpochH:-15.30; ChangeG:-0.10; ChangeH:0.60),
(M:3; N:8; EpochG:-0.50; EpochH:12.80; ChangeG:0.40; ChangeH:-0.20),
(M:4; N:8; EpochG:-21.10; EpochH:-11.70; ChangeG:-0.10; ChangeH:0.50),
(M:5; N:8; EpochG:15.30; EpochH:14.90; ChangeG:0.40; ChangeH:-0.30),
(M:6; N:8; EpochG:13.70; EpochH:3.60; ChangeG:0.30; ChangeH:-0.40),
(M:7; N:8; EpochG:-16.50; EpochH:-6.90; ChangeG:-0.10; ChangeH:0.50),
(M:8; N:8; EpochG:-0.30; EpochH:2.80; ChangeG:0.40; ChangeH:0.00),
(M:0; N:9; EpochG:5.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:9; EpochG:8.40; EpochH:-23.40; ChangeG:0.00; ChangeH:0.00),
(M:2; N:9; EpochG:2.90; EpochH:11.00; ChangeG:0.00; ChangeH:0.00),
(M:3; N:9; EpochG:-1.50; EpochH:9.80; ChangeG:0.00; ChangeH:0.00),
(M:4; N:9; EpochG:-1.10; EpochH:-5.10; ChangeG:0.00; ChangeH:0.00),
(M:5; N:9; EpochG:-13.20; EpochH:-6.30; ChangeG:0.00; ChangeH:0.00),
(M:6; N:9; EpochG:1.10; EpochH:7.80; ChangeG:0.00; ChangeH:0.00),
(M:7; N:9; EpochG:8.80; EpochH:0.40; ChangeG:0.00; ChangeH:0.00),
(M:8; N:9; EpochG:-9.30; EpochH:-1.40; ChangeG:0.00; ChangeH:0.00),
(M:9; N:9; EpochG:-11.90; EpochH:9.60; ChangeG:0.00; ChangeH:0.00),
(M:0; N:10; EpochG:-1.90; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:10; EpochG:-6.20; EpochH:3.40; ChangeG:0.00; ChangeH:0.00),
(M:2; N:10; EpochG:-0.10; EpochH:-0.20; ChangeG:0.00; ChangeH:0.00),
(M:3; N:10; EpochG:1.70; EpochH:3.60; ChangeG:0.00; ChangeH:0.00),
(M:4; N:10; EpochG:-0.90; EpochH:4.80; ChangeG:0.00; ChangeH:0.00),
(M:5; N:10; EpochG:0.70; EpochH:-8.60; ChangeG:0.00; ChangeH:0.00),
(M:6; N:10; EpochG:-0.90; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:7; N:10; EpochG:1.90; EpochH:-4.30; ChangeG:0.00; ChangeH:0.00),
(M:8; N:10; EpochG:1.40; EpochH:-3.40; ChangeG:0.00; ChangeH:0.00),
(M:9; N:10; EpochG:-2.40; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:10; N:10; EpochG:-3.80; EpochH:-8.80; ChangeG:0.00; ChangeH:0.00),
(M:0; N:11; EpochG:3.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:11; EpochG:-1.40; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:2; N:11; EpochG:-2.50; EpochH:2.50; ChangeG:0.00; ChangeH:0.00),
(M:3; N:11; EpochG:2.30; EpochH:-0.60; ChangeG:0.00; ChangeH:0.00),
(M:4; N:11; EpochG:-0.90; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:5; N:11; EpochG:0.30; EpochH:0.60; ChangeG:0.00; ChangeH:0.00),
(M:6; N:11; EpochG:-0.70; EpochH:-0.20; ChangeG:0.00; ChangeH:0.00),
(M:7; N:11; EpochG:-0.10; EpochH:-1.70; ChangeG:0.00; ChangeH:0.00),
(M:8; N:11; EpochG:1.40; EpochH:-1.60; ChangeG:0.00; ChangeH:0.00),
(M:9; N:11; EpochG:-0.60; EpochH:-3.00; ChangeG:0.00; ChangeH:0.00),
(M:10; N:11; EpochG:0.20; EpochH:-2.00; ChangeG:0.00; ChangeH:0.00),
(M:11; N:11; EpochG:3.10; EpochH:-2.60; ChangeG:0.00; ChangeH:0.00),
(M:0; N:12; EpochG:-2.00; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:12; EpochG:-0.10; EpochH:-1.20; ChangeG:0.00; ChangeH:0.00),
(M:2; N:12; EpochG:0.50; EpochH:0.50; ChangeG:0.00; ChangeH:0.00),
(M:3; N:12; EpochG:1.30; EpochH:1.40; ChangeG:0.00; ChangeH:0.00),
(M:4; N:12; EpochG:-1.20; EpochH:-1.80; ChangeG:0.00; ChangeH:0.00),
(M:5; N:12; EpochG:0.70; EpochH:0.10; ChangeG:0.00; ChangeH:0.00),
(M:6; N:12; EpochG:0.30; EpochH:0.80; ChangeG:0.00; ChangeH:0.00),
(M:7; N:12; EpochG:0.50; EpochH:-0.20; ChangeG:0.00; ChangeH:0.00),
(M:8; N:12; EpochG:-0.30; EpochH:0.60; ChangeG:0.00; ChangeH:0.00),
(M:9; N:12; EpochG:-0.50; EpochH:0.20; ChangeG:0.00; ChangeH:0.00),
(M:10; N:12; EpochG:0.10; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:11; N:12; EpochG:-1.10; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:12; N:12; EpochG:-0.30; EpochH:0.50; ChangeG:0.00; ChangeH:0.00),
(M:0; N:13; EpochG:0.10; EpochH:0.00; ChangeG:0.00; ChangeH:0.00),
(M:1; N:13; EpochG:-0.90; EpochH:-0.90; ChangeG:0.00; ChangeH:0.00),
(M:2; N:13; EpochG:0.50; EpochH:0.60; ChangeG:0.00; ChangeH:0.00),
(M:3; N:13; EpochG:0.70; EpochH:1.40; ChangeG:0.00; ChangeH:0.00),
(M:4; N:13; EpochG:-0.30; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:5; N:13; EpochG:0.80; EpochH:-1.30; ChangeG:0.00; ChangeH:0.00),
(M:6; N:13; EpochG:0.00; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:7; N:13; EpochG:0.80; EpochH:0.30; ChangeG:0.00; ChangeH:0.00),
(M:8; N:13; EpochG:0.00; EpochH:-0.10; ChangeG:0.00; ChangeH:0.00),
(M:9; N:13; EpochG:0.40; EpochH:0.50; ChangeG:0.00; ChangeH:0.00),
(M:10; N:13; EpochG:0.10; EpochH:0.50; ChangeG:0.00; ChangeH:0.00),
(M:11; N:13; EpochG:0.50; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:12; N:13; EpochG:-0.50; EpochH:-0.40; ChangeG:0.00; ChangeH:0.00),
(M:13; N:13; EpochG:-0.40; EpochH:-0.60; ChangeG:0.00; ChangeH:0.00)))

);




implementation

end.
